/*****************************************************************************/
/*                                                                           */
/* 		                        EDF()                                        */
/*                                                                           */
/* 	Function to determine the estimated # of chi-squared degrees of freedom  */
/*  for the modified Allan variance (MVAR) of a power law noise process.     */
/*                                                                           */
/*  Based on edf() of STAB_F38.C from Stable/DOS                             */
/*                                                                           */
/*	Original:                                                                */
/*	Parameters:		int beta  = exponent of Sx(f) of power law noise         */
/*                              where: 0 = W                                */
/*                                    -1 = F                                */
/*                                    -2 = W  f                              */
/*                                    -3 = F  f	                             */
/*                                    -4 = RW f                              */
/*					int m     = averaging factor                             */
/*					float p   = M/m                                          */
/*							    where: M = # phase analysis points           */
/*										 = N-3m+1                            */
/*										   where: N = # phase data points    */
/*  New Params:		int nBeta = beta                                         */
/*                  int nAvgFactor = m                                       */
/*                  int nNum = M                                             */
/*                                                                           */
/*                  nNum and nAvgFactor are used to find p                   */
/*                                                                           */
/*	Return:         float edf = estimated degrees of freedom                 */
/*                                                                           */
/*	Note:			This function calculates the MVAR edf for the usual      */
/*					case of fully overlapping samples (m1=1 & r=m)           */
/*                                                                           */
/*	Reference:		C. H. Greenhall, "Estimating the Modified Allan          */
/*                  Variance", 1995 IEEE International Frequency Control     */
/*                  Symposium (to be published)                              */
/*                                                                           */
/*                                                                           */
/*	(c) 1996		Hamilton Technical Services    All Rights Reserved       */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

float FAR PASCAL _export EDF(int nBeta, int nAvgFactor, int nNum)
{
	// Local variables
	// the a0 and a1 coefficients for estimating MVAR edf
	// are stored in the following static arrays
	//  varies from 0 to -4 ( - varies from 0 to 4)
	// m varies from 1 to >2 (m-1 varies from 0 to 2)

	// a0 term: a0[-][m-1]
	static float a0[5][3] =
	{
		{ 0.51429, 0.93506, 1.22450 },
		{ 0.57640, 0.97339, 1.00300 },
		{ 0.66667, 1.01010, 0.96774 },
		{ 0.81057, 1.02660, 0.94663 },
		{ 1.00000, 0.86580, 0.76791 }
	};

	// a1 term: a1[-][m-1]
	static float a1[5][3] =
	{
		{ 0.00000, 0.00000, 0.58929 },
		{ 0.00000, 0.00000, 0.60163 },
		{ 0.00000, 0.00000, 0.57124 },
		{ 0.00000, 0.00000, 0.41643 },
		{ 0.00000, 0.00000, 0.41115 }
	};

	// Convert args to those of original edf() function
	int beta=nBeta;
	int m=nAvgFactor;
	float p=(float)(nNum)/(float)nAvgFactor;

	// truncate m
	if(m>3)
	{
		m=3;
	}

	// calc and return edf
	return (a0[-beta][m-1]*p)/(1-(a1[-beta][m-1]/p));
}

/*****************************************************************************/
