/***************************************************************************/
/*                                                                         */
/*  					   CalcFastModHadSigma()                           */
/*                                                                         */
/*		Function to calculate modified Hadamard variance for phase data    */
/*		Uses faster algorithm that avoids recalculation of inner sums      */
/*                                                                         */
/*		Reference:      S. Bregni, "Improved Estimation of the Hurst       */
/*						Parameter of Long-Range Dependent Traffic          */
/*						Using the Modified Hadamard Variance",             */
/*						Unpublished paper, received via e-mail from        */
/*						the author, 01/17/06.                              */
/*                                                                         */
/*		Parameters:		F_TYPE x[]    = phase data                         */
/*                                      x[0] = # data phase points         */
/*			                            x[1]  = analysis start             */
/*                                      x[2] = analysis end                */
/*                      F_TYPE *p_sig = pointer to mod Hadamard deviation  */
/*			            F_TYPE tau    = analysis averaging time = m*tau0   */
/*			            int m         = averaging factor = tau/tau0        */
/*                                                                         */
/*		Return:         int           = # analysis points,                 */
/*										or -1 if error                     */
/*                                                                         */
/*		Notes:  		No gaps allowed in data; check for embedded zeros  */
/*                      before using this function. If gaps, use           */
/*                      FillGaps() first.                                  */
/*                                                                         */
/*			            Phase data is not averaged before calling this     */
/*                      function.  The data consists of num phase samples  */
/*                      at interval tau0.  Denominator term (num-4m+1)     */
/*                      must be >0.  This sets max m=(int)(num/4),         */
/*			            where num=# phase points before averaging.         */
/*                                                                         */
/*		Revision record:                                                   */
/*			01/19/06	Created from CFMS2.C of FrequenC library           */
/*						NOT CHECKED - DO NOT USE                           */
/*                                                                         */
/* (c) Copyright 2006   Hamilton Technical Services   All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

int __declspec(dllexport) FAR PASCAL CalcFastHadSigma(F_TYPE x[],
	F_TYPE *p_sig, F_TYPE tau, int m)
{
	double sum_i, sum_j;   /* summing variables to calc mod Hadamard sigma */
	float fm;                                        /* float version of m */
	int i;                                 /* index for inner summing loop */
	int j;                                 /* index for outer summing loop */
	int s = (int)*(x+1);                                 /* analysis start */
	int e = (int)*(x+2);                                   /* analysis end */
	int n_sum=0;                         /* # points in outer summing loop */
	int num = e-s+1;                   /* # phase data points for analysis */

	fm=(float)m;
	sum_j=0;
	sum_i=0;

	// Check arguments for validity
	if( (num<4*m) || (m<1)  || (tau<=0) )
	{
		return(-1);                                               /* error */
	}

	// For strightford (slower) algorithm w/o consideration of array header:
	// The outer summation index j goes from 1 to N-4m+1
	// where N=# phase data points and m=avg factor
	// The inner summation index i goes from j to j+m-1, and the inner sum
	// terms (before squaring) are x[i]-3x[i+m]+3x[i+2m]-x[i+3m]
	// The faster algorithm avoids full inner summation by simply dropping
	// the old term and adding the new one

	// First time find the average of all m phase points by summing them
	for(i=s+2; i<m+s+2; i++)           /* i is the actual phase data index */
	  /* it includes the term s to account for the analysis starting point */
			 /* the data array offset +2 is included in the indexing above */
	{
		sum_i += *(x+i) - (3*(*(x+i+m))) + (3*(*(x+i+(2*m)))) - *(x+i+(3*m));
	}

	sum_j += SQR(sum_i);
	n_sum++;

	// For rest of points need only drop 1st sum and add next sum
	for(j=2+2; j<=num-(4*m)+1+2; j++)  	      /* j goes from 2 to num-3m+1 */
								/* where num=# phase points & m=avg factor */
										  /* regardless of analysis limits */
			 /* the data array offset +2 is included in the indexing above */
	{
		i=j+s-4;                       /* index of old sum_i being dropped */
		sum_i -= *(x+i+(2*m)) - (2*(*(x+i+m))) + *(x+i);

		i=m+j-1+s-3;                    /* index of next sum_i being added */
		sum_i += *(x+i+(2*m)) - (2*(*(x+i+m))) + *(x+i);

		sum_j += SQR(sum_i);
		n_sum++;
	}                                                            /* end if */

	if(n_sum>0)                                          /* avoid div by 0 */
	{
		// The denominator term is 6*m*m*tau*tau*(N-4m+1), where the last
		// portion is the upper j index, the # of outer loop sums, n_sum		
		*p_sig = sqrt((sum_j)/(6.0*tau*tau*fm*fm*n_sum));
		return(n_sum);                                /* # analysis points */
	}
	else
	{
		return(-1);                                               /* error */
	}
}                                                /* end CalcFastHadSigma() */

/***************************************************************************/
