/*****************************************************************************/
/*                                                                           */
/*                                ModTotvarCalc()                            */
/*                                                                           */
/*      Filename: mtc.c                                                      */
/*      Function to calculate the Modified Total Variance from phase data    */
/*      Uses uninverted even reflection of offset-removed phase data         */
/*                                                                           */
/*      Parameters:     F_TYPE x[]      = phase data                         */
/*                                        x[0] = # data points               */
/*                                        x[1] = analysis start              */
/*                                        x[2] = analysis end                */
/*                      F_TYPE *fMTDev  = pointer to Mod-Totdev              */
/*                      F_TYPE fTau     = interval between phase data points */
/*                      int nAF         = averaging factor                   */
/*                      void *lpfnProgress = ptr to progress display funct   */
/*                                                                           */
/*      Return:         int           = # Mod-Totvar analysis points,        */
/*                                      or -1 if memory alloc error          */
/*                                      or -2 if no result error             */
/*                                                                           */
/*      Notes:          1. Calculation is done entirely with a new array.    */
/*                         that is deleted after this function closes.       */
/*                      2. Progress indicator and abort not necessary.       */
/*                      3. The phase data need not be endmatched before      */
/*                         calling this function.                            */
/*                                                                           */
/*      Description:    Given the phase data array x[], Mod-Totvar is        */
/*                      calculated using z[], which is computed from a       */
/*                      subsequence of x[] consisting of 3m points, where    */
/*                      m is the averaging factor nAF.  This subsequence is  */
/*                      given by x[i] where i=n thru n+3m-1, where n is an   */
/*                      overall index that goes from 1 to the # of phase     */
/*                      data points nNum.  The subsequence z[i] is found     */
/*                      from x[i] by removing the frequency offset by        */
/*                      subtracting the slope found by averaging the 1st     */
/*                      and last halves of the subsequence and dividing by   */
/*                      half the interval.                                   */
/*                                                                           */
/*                      This subsequence is then extended at both ends by    */
/*                      an uninverted even reflection according to z(n-k)=   */
/*                      z(n+k-1) and z(n+3m+k-1)=z(n+3m-k) for k=1 thru 3m,  */
/*                      thus creating a new subsequence having a tripled     */
/*                      range from i=n-3m to i=n+6m-1.                       */
/*                                                                           */
/*                      Mod-Totvar is then calculated as the average of its  */
/*                      nNum-3m Totvar subestimates.                         */
/*                                                                           */
/*      Reference:      D.A. Howe and F. Vernotte, "Generalization of the    */
/*                      Total Variance Approach to the Modified Allan        */
/*                      Variance", Proc. 31th PTTI Meeting, Dec. 9, 1999,    */
/*                      (to be published).                                   */
/*                                                                           */
/*      Revision record:                                                     */
/*          12/12/99    Adapted from TotvarCalc() and CalcPhaseModSigma()    */
/*						of FrequenC Library.                                 */
/*                                                                           */
/* (c) Copyright 1999   Hamilton Technical Services   All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

// Headers
#include "frequenc.h"
#include <windowsx.h>   // For GlobalAllocPtr() & GlobalAllocFree()

// For Win 3.1 and Win32 Compatibility
#ifndef WIN32
#define HUGE huge
#else
#define HUGE
#endif

// Defines
#define ARRAY_OFFSET 3  // Offset to 1st data point

int EXPORT FAR PASCAL ModTotvarCalc(F_TYPE fX[], F_TYPE *fMTDev, F_TYPE fTau,
    int nAF, void *lpfnProgress)
{
    // Local variables
    int i;              // Index
    int j;              // Auxilary index
    int nNum;           // # points
    int nStart;         // Analysis start
    int nDummy;         // Dummy variable to avoid complier warning
    int nCount=0;       // # analysis points

    double sum_i, sum_j;              /* summing variables to calc mod sigma */
    float fm;                                          /* float version of m */
    int n_sum=0;                           /* # points in outer summing loop */
    int num = (int)(*(x+2)-*(x+1)+1);    /* # phase data points for analysis */

    #if(0) // Not needed if no progress indicator
    int nTic;           // Increment for progress indicator
    int nLast_i=0;      // Previous value of index
    #endif

    double HUGE *fP;    // TOTVAR phase array pointer
    F_TYPE fSum=0;      // TOTVAR summation
    F_TYPE fEdge;       // Phase data value at edge of reflection

    // Avoid compiler warning if no progress indicator
    lpfnProgress=lpfnProgress;

    // Initializations
    nStart=(int)fX[1];          // Analysis start point
    nNum=(int)(fX[2]-fX[1]+1);  // # analysis phase data points = N

    #if(0) // Not needed if no progress indicator
    nTic=(int)ceil(nNum/100);   // Increment for progress report

    if(lpfnProgress!=NULL)
    {
        lpfnProgress(0, "Calculating Mod Total Sigma"); // Label progress bar
    }
    #endif

    #if(0) // For testing - check input params
    TRACE_EXP(fTau);
    TRACE_INT(nStart);
    TRACE_INT(nNum);
    #endif

    // Allocate a new "virtual" phase data array to size 3N-4, where N is
    // the # of phase data points to be analyzed.  This virtual array is the
    // result of extension by reflection about both endpoints.  N-2 reflected
    // points are added at both ends of the original phase data to be analyzed.
    // Do NOT need the array header (#, Start, End)
    // Note that fP is cast to a huge double ptr and that array size is long
    if((fP=(double HUGE *)GlobalAllocPtr(GHND,
        (long)(3*nNum-4)*sizeof(double)))==NULL)
    {
        #if(0) // Eliminated for FrequenC DLL usage
        AllocErrorMessage();
        #endif
        return(-1);  // Error - memory reallocation failed
    }

    // Note that in the referenced paper the index of the virtual phase data
    // array goes from 3-N (a negative number) to 2N-2 (a positive number),
    // with the original data having indices from 1 to N.  Our indices start at
    // 0 and go to 3N-5.  The lower reflected data has indices from 0 to N-3.
    // The original data in the middle of the virtual array has indices from
    // N-2 to 2N-3. The upper reflected data has indices from 2N-2 to 3N-5.

    // Copy original phase data array fX[] to (headerless) working array fP[]
    // Analysis limits are handled by copying only the points between the
    // start and end.  i is the index into the virtual array fP[].  j is the
    // variable part of the index into the original phase data array fX[].
    j=0;
    for(i=nNum-2; i<2*nNum-2; i++)
    {
        fP[i]=fX[j+nStart+ARRAY_OFFSET-1];
        j++;
    }

    // Fill the lower reflected phase data
    // These values are twice the first phase data point minus the particular
    // data point value to be reflected
    j=0;
    fEdge=2*fP[nNum-2];

    for(i=0; i<nNum-2; i++)
    {
        fP[i]=fEdge-fP[2*nNum-4-j];
        j++;
    }

    // Fill the upper reflected phase data
    // These values are twice the last phase data point minus the particular
    // data point value to be reflected
    j=0;
    fEdge=2*fP[2*nNum-3];

    for(i=2*nNum-2; i<3*nNum-4; i++)
    {
        fP[i]=fEdge-fP[2*nNum-4-j];
        j++;
    }

    #if(0) // For testing - examine virtual array
    for(i=0; i<3*nNum-4; i++)
    {
        TRACE_EXP(fP[i]);
    }
    #endif

    // Calc MVar using heritage code
	// Gap testing removed for initial clarity
	// Pointer notation changed to array subscripting
    fm=(float)m;
    sum_j=0;
    for(j=1; j<=num-(3*m)+1; j++)                              /* outer loop */
            /* j goes from 1 to N-3m+1 where N=# phase points & m=avg factor */
                                            /* regardless of analysis limits */
    {
        sum_i=0;
        for(i=j+(int)(*(x+1))-1; i<=m+j-1+(int)(*(x+1))-1; i++)/* inner loop */
                    /* i is the actual phase data index;  it includes a term */
                      /* *(x+1)-1 to account for the analysis starting point */
                   /* the data array offset +2 is included in the code below */
        {
            sum_i += x[i+2+(2*m)] - 2*x[i+2+m] + x[i+2];
        }                                                         
        if(sum_i)
        {
            sum_j += SQR(sum_i);
            n_sum++;
        }                                                          
    }                                                             
    if(n_sum>0)                                     /* make sure no div by 0 */
    {
        *p_sig = sqrt((sum_j)/(2.0*tau*tau*fm*fm*n_sum));
    }
    else
    {
        *p_sig = 0;                                                 
    }
    // End of heritage MVar code

        #if(0) // Not needed if no progress indicator or abort
        // Display progress
        // Note:  This code provides updates only if index has changed
        // by at least tic amount
        // Uses pointer to progress display function PFVi argument
        // Pass NULL for no progress display
        if(((i-nLast_i)>=nTic) && (lpfnProgress!=NULL))
        {
            lpfnProgress(0.5+100.0*i/nNum, "Calculating Total Sigma");
            nLast_i=i;
        }

        // Check for abort
        if(bAbort)
        {
            nCount=0;                                         /* force error */
            break;
        }
        #endif
    }

    // Scale result - See Eq (3) of Reference
    fSum/=(fTau*fTau*nAF*nAF*2*nCount);

    // Find TOTDEV
    if(nCount && fSum>0)
    {
        *fTotDev=sqrt(fSum);
    }
    else // No result error
    {
        *fTotDev=0.0;
        nCount=-2;
    }

    // Free memory
    nDummy=GlobalFreePtr(fP);
    nDummy=nDummy;

    return(nCount);  // Return # phase analysis points
}

/*****************************************************************************/
