/*****************************************************************************/
/*                                                                           */
/*                            STABLE_P.C                                     */
/*                                                                           */
/*		GraphiC Plotting Functions for WINDOWS 3.1 version of STABLE         */
/*                                                                           */
/*		Functions:		FreqPlot()                                           */
/*                      SigmaPlot()                                          */
/*                      PowerSpectrumPlot()                                  */
/*                      PhasePlot()                                          */
/*                      MsgBox()                                             */
/*                      TableBox()                                           */
/*                      HistoPlot()                                          */
/*						Dateline()                                           */
/*						PlotDlgProc()                                        */
/*						TVARLabel()                                          */
/*                                                                           */
/*      (c) 1996-7 W. Riley Hamilton Technical Services All Rights Reserved  */
/*                                                                           */
/*****************************************************************************/

#define STRICT 					// For Win32 compatibility

// Include the stable_w header file
#include "stable_w.h"           // STABLE_W header file

/*****************************************************************************/
/*                                                                           */
/*								FreqPlot()                                   */
/*                                                                           */
/*		Function to plot frequency data                                      */
/*      Parameters:                                                          */
/*			f_data[] = array of frequency data; f_data[0] = # data points    */
/*			f_data[1] = plot start; f_data[2] = plot end                     */
/*      	zero treated as gap in data                                      */
/*			aux_data[] = array of auxilary data; aux_data[0] = # points      */
/*			aux_data[1] = start; aux_data[2] = end                           */
/*			p plotdata structure elements:                                   */
/*				title = plot title                                           */
/*      		subtitle = plot subtitle                                     */
/*				x_label = x-axis label                                       */
/*				left_y_label = left-hand y-axis label                        */
/*				x_offset = x-axis scale offset                               */
/*				x_mult = x-axis scale multiplier                             */
/*				y_offset = y-axis scale offset                               */
/*				y_mult = y-axis scale multiplier                             */
/*          	y_min = y-axis scale minimun                                 */
/*          	y_max = y-axis scale maximum                                 */
/*          	y_div = # of y-axis major divisions                          */
/*          	y_tic = # of y-axis tics between major divisions             */
/*				do_box = draw box flag                                       */
/*      		do_line = draw regression line code                          */
/*			              0=None, 1=Line, 2=Log, 3=Menu, 4=Dual              */
/*						  The Menu code then leads to the following types:   */
/*						  0=None, 1=Endpoints, 2=Linear Fit, 3=Average,      */
/*						  4=Marker, 5=Null                                   */
/*				do_date = write date flag                                    */
/*     			do_mouse = activate mouse/cursor flag                        */
/*				new_file = new basic data file flag                          */
/*				new_name = new exact data file flag                          */
/*				position = position of message box                           */
/*          	plot_filename = .tkf filename for plot                       */
/*		Returns nothing (void)                                               */
/*                                                                           */
/*		Freq Plot do_line Codes:	 Freq Plot Line Menu line_type Types:    */
/*			FPLOT_NONE	0				FLINE_NONE	0                        */
/*			FPLOT_LINE	1				FLINE_END   1                        */
/*			FPLOT_LOG	2				FLINE_LIN   2                        */
/*			FPLOT_MENU	3				FLINE_AVG   3                        */
/*			FPLOT_DUAL	4				FLINE_MARK  4                        */
/*                                                                           */
/*		Revision record:                                                     */
/*			06/30/96	Adapted from plot_freq() of Stable/DOS               */
/*			07/12/96	Added code for drawing annotations                   */
/*			07/13/96	More adaptation of DOS version code                  */
/*			11/09/96	Ditto - Now all is basically functional              */
/*			11/12/96	Added clipping to line drawing code                  */
/*			11/18/96	Revised plot sizes, etc.                             */
/*                                                                           */
/*****************************************************************************/

void FAR PASCAL _export FreqPlot(F_TYPE f_data[], F_TYPE aux_data[],
	struct plotdata *p)
{
	// Local variables
	float *x,*y;
	float width;
	float x_max, x_min;                                    /* x scale limits */
	float y_max, y_min;                                    /* y scale limits */
	float xste;                                        /* x-axis tic spacing */
	float yste;                                        /* y-axis tic spacing */
	float save_start;
	float save_end;
	int i;
	int j;
	int k;
	int npts;
	int len;				                    	        /* string length */
	int lgth;                                    /* message length in pixels */
	int nDummy;
	int line_flag=FALSE;
	char label_str[100];
	char dual_label_str[50];
	char yformat[8];
	char xformat[8];
	char *szTemp;

	// Colors
	int background_color=BRT_WHITE;
	int grid_color=BLACK;
	int box_color=BLACK;
	int label_color=BLUE;
	int data_color=RED;
	int annotation_color=BLACK;
	int line_color=GREEN;
	int aux_color=GREEN;

	// Fit parameters
	F_TYPE avg;
	F_TYPE slope;
	F_TYPE intercept;
	static F_TYPE a, b, c, v;
	static BOOL bHaveLogFit=FALSE;	// Flag that valid log fit already done

	// Cursor variables
	float *fx;                  // Pointer to x-axis point array
	float *fy;                  // Pointer to y-axis point array
	int find;					// Point array dimensions

	#if(1) // New code 01/12/97
	// Reset have log fit flag if new file
	if(p->new_file)
	{
		bHaveLogFit=FALSE;
	}

	// Reset new file flag - NOTE: MAKE SURE THIS ISN'T NEEDED LATER
	p->new_file=FALSE;
	#endif

	#if(0) // Removed 11/09/96
	// Test for no data - Shouldn't be necessary
	// Note: NULL plot filename test eliminated
	if((f_data[2]-f_data[1])<=0)
	{
		MessageBox(hwndMain, "No Frequency Data!",
			"Stable/Win", MB_ICONEXCLAMATION);
		return;
	}                                                              /* end if */
	#endif

	// Allocate memory for plot arrays
	if((x=(float*)GlobalAllocPtr(GHND, (f_data[2]+2)*sizeof(float)))==NULL)
	{
		AllocErrorMessage();
		return;
	}
	if((y=(float*)GlobalAllocPtr(GHND, (f_data[2]+2)*sizeof(float)))==NULL)
	{
		AllocErrorMessage();
		return;
	}

	CalcMean(f_data, &avg, FREQ_DATA);

/*****************************************************************************/
#if(0) // Temporarily, eliminate some secondary code
/*****************************************************************************/
	for(i=0; i<FLINE_NUMLINES; i++) /* correct line parameters for averaging */
	{
		sFLine[i].fXStart /= avg_fact[1];
		sFLine[i].fXEnd   /= avg_fact[1];
	}                                                             /* end for */
	avg_fact[1]=1;                              /* only make correction once */

	if(p->new_file)
	{
		for(i=0; i<FLINE_NUMLINES; i++)
		{
			sFLine[i].nLineType=FLINE_NONE;
			sFLine[i].fXStart=f_data[1];
			sFLine[i].fXEnd=f_data[2];
		}                                                         /* end for */
	}                                                              /* end if */

	if(p->new_name)
	{
		for(i=0; i<FLINE_NUMLINES; i++)
		{
			sFLine[i].fYStart=p->y_min;
			sFLine[i].fYEnd=p->y_max;
		}                                                         /* end for */
	}                                                              /* end if */
/*****************************************************************************/
#endif
/*****************************************************************************/
/* This is the begining of the actual GraphiC plotting code                  */
/* Sequence of basic calls is per page I-34 of manual:                       */
/*	   bgnplot()            	Initializations                              */
/*	   startplot()          	Starts graphics                              */
/*	   font()               	Loads fonts                                  */
/*	   page()               	Sets overall picture size                    */
/*	   heading()				Draws title                                  */
/*	   xname(), yname()		 	Draws axis labels                            */
/*	   area2d()					Defines axis lengths                         */
/*	                                                                         */
/*****************************************************************************/

	bgnplot(1,'g',p->plot_filename);                      /* initialize plot */
	startplot(background_color);         /* start plot with background color */
	font(4,"news.fnt",'\310',"swiss.fnt",'\311',"newsgrm.fnt",'\312',
		 "simplex.fnt",'\313');                                /* load fonts */
	page(FULLPAGE);                  /* Sets the maximum allowable page size */
	heading("");    /* Blank heading allows space for title & subtitle later */
	fillfont(1);                                         /* Use filled fonts */

	// Kill the beep
	GPCSetBeepDefaults(0, 0);

	// Axis labeling code
	fntchg('\310');                                              /* news.fnt */
	if(p->do_line==FPLOT_DUAL)                                  /* dual plot */
	{
		rightax(1);
		color(data_color);                /* left y-axis label for dual plot */
	}                                                              /* end if */
	else
	{
		color(label_color);             /* left y-axis label for single plot */
	}                                                            /* end else */
	yname(p->left_y_label);                                 /* labels y-axis */
	color(label_color);                                  /* all other labels */
	if(p->do_line!=FPLOT_LOG)                                /* log fit line */
	{
		xname(p->x_label);                                  /* labels x-axis */
	}

	#if(0) // Old code
	area2d(7.6f, 5.0f);                         /* Sets the area of the plot */
	#endif

	#if(1) // New code 11/18/96
	area2d(7.8, 5.3);
	#endif

	// Draw box
	color(box_color);                                  /* set box line color */
	if(p->do_box)
	{
		box();                                    /* draw box around picture */
	}                                                              /* end if */

	// Draw grid
	// The grid() function draws the major grid lines (at tics)
	// The fgrid() function draws the fine grid lines (between tics)
	// The 1st fgrid() arg is: 0=no ticks, -1=tick marks
	// The 2nd fgrid() arg is # ticks=# intervals-1
	color(grid_color);                          /* set box & grid line color */
	grid(9);                         /* draws dotted grid through tick marks */
	if(p->do_line==FPLOT_DUAL)                   /* for all except dual plot */
	{
		fgrid(0,p->y_tic-1);                       /* no fine grid tic marks */
	}                                                              /* end if */
	else
	{
		fgrid(-1,p->y_tic-1);    /* fine grid tick marks between major ticks */
	}

	frame(1,1);                          /* add frame with ticks around plot */
	upright(1);                        /* causes y-axis labels to be upright */
	p->x_div=5;                    /* sets the desired # of x-axis divisions */
	fntchg('\310');                                              /* news.fnt */

	#if(0) // original code
	/* find plot scales */
	x_max=p->x_mult*(f_data[2]+p->x_offset);
	x_min=p->x_mult*(f_data[1]+p->x_offset);
	roundq(p->x_div, &x_min, &xste, &x_max, xformat);

	y_max=p->y_mult*(p->y_max+p->y_offset);
	y_min=p->y_mult*(p->y_min+p->y_offset);
	roundq(p->y_div, &y_min, &yste, &y_max, yformat);
	#endif

	#if(1) // New code 11/09/96
	// Set plot scales
	x_max=p->x_mult*(p->x_max+p->x_offset);
	x_min=p->x_mult*(p->x_min+p->x_offset);
	roundq(p->x_div, &x_min, &xste, &x_max, xformat);

	y_max=p->y_mult*(p->y_max+p->y_offset);
	y_min=p->y_mult*(p->y_min+p->y_offset);
	roundq(p->y_div, &y_min, &yste, &y_max, yformat);
	#endif

	#if(0) // For testing
	TRACE_EXP(x_min);
	TRACE_EXP(x_max);
	TRACE_EXP(y_min);
	TRACE_EXP(y_max);
	#endif

	// Set plot scales
	graf(xformat, x_min, xste, x_max, yformat, y_min, yste, y_max);

	// Draw annotations
	// The putlabel() function is used for user-defined annotations.
	// This must come before any plotting - including grid lines
	// Then it is supposed to stay clean.
	// But, for user units, it should come after plot scales are set
	// It provides user messages via the plot annotations dialog box
	// The 2nd & 3rd args of putlabel() are in plot user units.
	// The 4th arg of putlabel() is char hgth in inches (for full-page).
	// The sFNote[i].nSize value is point size.  There are 72 points/inch.
	// The 5th arg of putlabel is border code: 1=border, 2=no border (strange)
	// So we get it as 2-sFNote[i].bBorder, where bBorder is ordinary BOOL flag
	// Multiline labels are OK with \n (newline, ASCII LF, dec int 10)
	// But they are NG with CR & LF as produced by an edit control.
	// So we must elim CRs from the note text string
	fntchg('\313');                                           /* simplex.fnt */
	for(i=0; i<FNOTE_NUMNOTES; i++)
	{
		// Check if this is an active note (non-zero length)
		len=strlen(sFNote[i].szText);
		if(len)
		{
			// Allocate a temporary working buffer
			if((szTemp=(LPSTR)GlobalAllocPtr(GHND,
				(len+1)*sizeof(char)))==NULL)
			{
				AllocErrorMessage();
			}
			// Remove CRs from annotation text string
			// Note that we are also copying the EOS
			k=0;
			for(j=0; j<=len; j++)
			{
				if(sFNote[i].szText[j]!=13) // 13 = CR
				{
					szTemp[k]=sFNote[i].szText[j];
					k++;
				}
			}
			// Draw annotation - use temp string w/o CRs
			putlabel(szTemp, sFNote[i].fX, sFNote[i].fY,
				sFNote[i].nSize/72.0, 2-sFNote[i].bBorder, USER_UNITS);

			// Free temporary working buffer
			// nDummy is to avoid compiler warning
			nDummy=GlobalFreePtr(szTemp);
			nDummy=nDummy;
		}
	}
	fntchg('\310');                                              /* news.fnt */

	npts=0;
	for(i=f_data[1]+2; i<=f_data[2]+2; i++)          /* load data into array */
	{
		if(f_data[i])                           /* don't add offset to a gap */
		{
			y[npts]=p->y_mult*(f_data[i]+p->y_offset);
			if(y[npts]<y_min)                        /* clip if below bottom */
			{
				y[npts]=y_min;
			}
			if(y[npts]>y_max)                           /* clip if above top */
			{
				y[npts]=y_max;
			}
		}                                                          /* end if */
		else
		{
			y[npts]=0;
		}                                                        /* end else */
		x[npts]=p->x_mult*((i-2)+p->x_offset);
		npts++;
	}                                                             /* end for */

	fntchg('\311');                                             /* swiss.fnt */

	width=strwidth(p->title,0.3);                 /* get half-width of title */

	#if(0) // Old code
	prtfnt((4.75-width),6.35,p->title,0.3,0);                  /* draw title */
	#endif

	#if(1) // New code
	prtfnt((4.75-width),6.40,p->title,0.3,0);                  /* draw title */
	#endif

	width=strwidth(p->subtitle,0.15);          /* get half-width of subtitle */

	#if(0) // Old code
	prtfnt((4.75-width),6.05,p->subtitle,0.15,0);           /* draw subtitle */
	#endif

	#if(1) // New code
	prtfnt((4.75-width),6.175,p->subtitle,0.15,0);          /* draw subtitle */
	#endif

	color(data_color);                                         /* data color */
	for(i=0; !*(y+i); i++)
	{
		;                                   /* just increment i to skip gaps */
	}
	uline((*(x+i)-p->x_mult),*(y+i),*(x+i),*(y+i),1);         /* first point */
	if(*(y+i+1))                              /* check if 2nd point is a gap */
	{
		uline(*(x+i),*(y+i),*(x+i),*(y+i+1),0);
		uline(*(x+i),*(y+i+1),*(x+i+1),*(y+i+1),2);
	}                                                              /* end if */
	for(++i; i<npts-2; i++)                             /* continue plotting */
	{
		if(*(y+i) && *(y+i+1))                                  /* skip gaps */
		{
			uline((*(x+i)-p->x_mult),*(y+i),*(x+i),*(y+i),1);
			uline(*(x+i),*(y+i),*(x+i),*(y+i+1),0);        /* plot freq data */
			uline(*(x+i),*(y+i+1),*(x+i+1),*(y+i+1),2);
		}                                                          /* end if */
	}                                                             /* end for */
	if(*(y+i+1))                                               /* last point */
	{
		if(*(y+i))
		{
			uline((*(x+i)-p->x_mult),*(y+i),*(x+i),*(y+i),1);
			uline(*(x+i),*(y+i),*(x+i),*(y+i+1),0);
		}                                                          /* end if */
		else
		{
			uline(*(x+i),*(y+i+1),*(x+i+1),*(y+i+1),1);
		}                                                        /* end else */
		uline(*(x+i),*(y+i+1),*(x+i+1),*(y+i+1),2);
	}                                                              /* end if */

	// Draw lines
	// Note: Need error trapping for entry of NG line parameters
	if(p->do_line==FPLOT_LINE)                            /* linear fit line */
	{
		CalcLinFreqDrift(f_data, &intercept, &slope, &v);      /* calc drift */

		color(line_color);                                     /* line color */
		/* draw linear regression line */
		x[0]=p->x_mult*(f_data[1]+p->x_offset);
		y[0]=p->y_mult*((intercept+(f_data[1] * slope))+p->y_offset);
		x[1]=p->x_mult*(f_data[2]+p->x_offset);
		y[1]=p->y_mult*((intercept+(f_data[2] * slope))+p->y_offset);
		curve(x, y, 2, 0);

		sprintf(label_str,
				"Linear Fit: y(t)=a+bt, Intercept=a=%-10.7e, "
				"Slope=b=%-10.7e", intercept,slope);
		fntchg('\313');                                       /* simplex.fnt */
		color(annotation_color);                     /* fit annotation color */
		tmargin(6.655);
		ctline(label_str,0.075);
		#if(1) // New code 01/04/97
		line_flag=TRUE;
		#endif
	}                                                              /* end if */
	if(p->do_line==FPLOT_LOG)                                /* log fit line */
	{
		color(line_color);                             /* line message color */
		tmargin(6.4);
		if(!bHaveLogFit)
		{
			ctline("Calculating Log Fit - Please Wait",0.175);
			CalcLogFreqDrift(f_data, &a, &b, &c, &v);      /* log calc drift */
			color(background_color);    /* erase message to background color */
			tmargin(6.4);
			ctline("Calculating Log Fit - Please Wait",0.175);      /* erase */
			bHaveLogFit=TRUE;
		}
		fntchg('\310');                                          /* news.fnt */
		color(label_color);                                   /* label color */
		xname(p->x_label);                                  /* labels x-axis */
		color(line_color);                                     /* line color */
												 /* draw log regression line */
		npts=0;
		for(i=f_data[1]+2; i<=f_data[2]+2; i++)   /* put fit data into array */
		{
			y[npts]=p->y_mult*(c+(a*log(((i-2)*b)+1))+p->y_offset);
			x[npts]=p->x_mult*((i-2)+p->x_offset);
			npts++;
		}                                                         /* end for */
		curve(x, y, npts, 0);
		sprintf(label_str,
				"Log Fit: y(t)=a*ln(bt+1)+c, "
				"a=%-10.7e, b=%-10.7e, c=%-10.7e, slope=%-10.7e at end",
				 a, b, c, (a*b)/((b*npts)+1));
		fntchg('\313');                                       /* simplex.fnt */
		color(annotation_color);                     /* fit annotation color */
		tmargin(6.655);
		ctline(label_str,0.075);
		#if(1) // New code 01/04/97
		line_flag=TRUE;
		#endif

		#if(0) // Eliminate for now
		// Show log fit info
		if(logage_flag)
		{
			sprintf(label_str, "Drift/Day=%-5.2e, Days More=%-1.1f",
					(a*b*96)/((b*npts)+1),
					(((1/(b*96))*((a*b/-5.208e-16)-1))-(npts/96.0)));
			message_box(600,2550,label_str);
		}                                                          /* end if */
		#endif

	}                                                              /* end if */
	#if(0) // Eliminated 11/09/96
	// Want to do this independently
	if(p->do_line==FPLOT_MENU)                                  /* line menu */
	#endif
	{
		color(line_color);                                     /* line color */
		save_start=f_data[1];                         /* save starting point */
		save_end=f_data[2];                             /* save ending point */
		for(i=0; i<FLINE_NUMLINES; i++)
		{
			switch(sFLine[i].nLineType)
			{
				case FLINE_NONE:                                  /* no line */
				{
					break;
				}
				case FLINE_END:                    /* line between endpoints */
				{
					f_data[1]=sFLine[i].fXStart;
					f_data[2]=sFLine[i].fXEnd;

					#if(0) // Old code
					uline(p->x_mult*(f_data[1]+p->x_offset),
						p->y_mult*(sFLine[i].fYStart+p->y_offset),
						p->x_mult*(f_data[2]+p->x_offset),
						p->y_mult*(sFLine[i].fYEnd+p->y_offset), 3);
					#endif

					#if(1) // New code with clipping
					uline(max(p->x_mult*(f_data[1]+p->x_offset), x_min),
						max(p->y_mult*(sFLine[i].fYStart+p->y_offset), y_min),
						min(p->x_mult*(f_data[2]+p->x_offset), x_max),
						min(p->y_mult*(sFLine[i].fYEnd+p->y_offset), y_max),
						3);
					#endif

					break;
				}
				case FLINE_LIN:                           /* regression line */
				{
						f_data[1]=sFLine[i].fXStart;
						f_data[2]=sFLine[i].fXEnd;

						CalcLinFreqDrift(f_data, &intercept, &slope, &v);
															   /* calc drift */

						#if(0) // Old code
						uline(p->x_mult*(f_data[1]+p->x_offset),
							p->y_mult*((intercept+(f_data[1] *
							slope))+p->y_offset),
							p->x_mult*(f_data[2]+p->x_offset),
							p->y_mult*((intercept+(f_data[2] *
							slope))+p->y_offset), 3);
						#endif

						#if(1) // New code with clipping
						uline(max(p->x_mult*(f_data[1]+p->x_offset), x_min),
							max(p->y_mult*((intercept+(f_data[1] *
							slope))+p->y_offset), y_min),
							min(p->x_mult*(f_data[2]+p->x_offset), x_max),
							min(p->y_mult*((intercept+(f_data[2] *
							slope))+p->y_offset), y_max), 3);
						#endif

						if(!line_flag)           /* can list only 1 line fit */
						{
							sprintf(label_str,
							"1st Linear Fit: y(t)=a+bt, "
							"Intercept=a=%-10.7e, Slope=b=%-10.7e",
							intercept,slope);
							fntchg('\313');                   /* simplex.fnt */
							color(annotation_color);     /* annotation color */                                                /* black */
							tmargin(6.655);
							ctline(label_str,0.075);
							line_flag=TRUE;
							color(line_color);                 /* line color */
						}
						break;
				}
				case FLINE_AVG:                    /* horizontal line at avg */
				{
						f_data[1]=sFLine[i].fXStart;
						f_data[2]=sFLine[i].fXEnd;

						CalcMean(f_data, &avg, FREQ_DATA);   /* calc average */

						#if(0) // Old code
						uline(p->x_mult*(f_data[1]+p->x_offset),
							p->y_mult*(avg+p->y_offset),
							p->x_mult*(f_data[2]+p->x_offset),
							p->y_mult*(avg+p->y_offset),3);
						#endif

						#if(1) // New code with clipping
						uline(max(p->x_mult*(f_data[1]+p->x_offset), x_min),
							max(p->y_mult*(avg+p->y_offset), y_min),
							min(p->x_mult*(f_data[2]+p->x_offset), x_max),
							min(p->y_mult*(avg+p->y_offset), y_max), 3);
						#endif

						break;
				}
				case FLINE_MARK:                     /* vertical marker line */
				{
						f_data[1]=sFLine[i].fXStart;
						f_data[2]=sFLine[i].fXStart;

						#if(0) // Old code
						uline(p->x_mult*(f_data[1]+p->x_offset),
							p->y_mult*(p->y_min+p->y_offset),
							p->x_mult*(f_data[2]+p->x_offset),
							p->y_mult*(p->y_max+p->y_offset),3);
						#endif

						#if(1) // New code with clipping
						uline(max(p->x_mult*(f_data[1]+p->x_offset), x_min),
							max(p->y_mult*(p->y_min+p->y_offset), y_min),
							min(p->x_mult*(f_data[2]+p->x_offset), x_max),
							min(p->y_mult*(p->y_max+p->y_offset), y_max), 3);
						#endif

						break;
				}
			}                                                  /* end switch */
		}                                                         /* end for */
		f_data[1]=save_start;                      /* restore starting point */
		f_data[2]=save_end;                          /* restore ending point */
	}                                                              /* end if */

	if(p->do_line==FPLOT_DUAL)                                  /* dual plot */

	/* Note:  The x-scale of the dual plot is the same for both data sets. */
	/*        This x-scale is determined by the x-start and x-end limits   */
	/* 		  of f_data[].  The aux_data[] must exist over this x region.  */
	/* 		  The x-start and x-end limits of aux_data[] determine what    */
	/* 		  portion of that data is plotted (within the plot range).     */

	{
		npts=0;
		for(i=f_data[1]; i<=f_data[2]; i++)      /* load aux data into array */
				 /* using x-start and x-end of f_data[] to determine x scale */
		{
			y[npts]=aux_data[i+2];
			x[npts]=i;
			npts++;
		}                                                         /* end for */
		npts--;
		color(aux_color);                                  /* aux plot color */
		fntchg('\310');                                          /* news.fnt */
		scales(p->x_div,p->y_div,x,y,npts);     /* draws & scales x & y axes */
		npts=0;
		for(i=max(f_data[1],aux_data[1]); i<=min(f_data[2],aux_data[2]);
			i++)                           /* load aux data into array again */
			 /* using actual x-start and x-end limits of aux_data[] for plot */
		{
			y[npts]=aux_data[i+2];
			x[npts]=i;
			npts++;
		}                                                         /* end for */
		npts--;
		curve(x,y,npts,0);
		yname(p->right_y_label);
							   /* put right plot file info at bottom of plot */
		strcpy(label_str,p->right_y_label);
		strcat(label_str," Data File: ");

		strcat(label_str, szFileTitle);          /* global filename variable */

		sprintf(dual_label_str,
				" Data Points %-1.0f thru %-1.0f of %-1.0f",
				aux_data[1],aux_data[2],aux_data[0]);
		strcat(label_str,dual_label_str);
		fntchg('\313');                                       /* simplex.fnt */
		color(annotation_color);                    /* file annotation color */
		tmargin(6.655);
		ctline(label_str,0.075);
	}                                                              /* end if */

	fntchg('\313');                                           /* simplex.fnt */
	color(annotation_color);                             /* annotation color */                                                    /* black */

	// Write date
	if(p->do_date)
	{
		Dateline(1);                         /* put date & file info on plot */
	}                                                              /* end if */

	// Display message box
	// Locations are x, y in pixels, origin at bottom left, 8192 x 6266 space
	// Find half length of message box in pixels
	lgth=(H_MARGIN+strwidtq(p->message_str, CHAR_HGTH));

	// Write message
	if(p->message_str[0]!=NULL)         /* don't display a blank message box */
	{
		switch(p->position)
		{
			case MSG_TOP_LEFT:            /* message box at top left of plot */
			{
				MsgBox(300,4400,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_MID_LEFT:         /* message box at middle left of plot */
			{
				MsgBox(300,2270,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_BOT_LEFT:         /* message box at bottom left of plot */
			{
				MsgBox(300,200,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_TOP_CENTER:        /* message box at top center of plot */
			{
				MsgBox(3470-lgth,4400, p->message_str);
				break;
			}                                                    /* end case */
			case MSG_MID_CENTER:     /* message box at middle center of plot */
			{
				MsgBox(3470-lgth,2270,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_BOT_CENTER:     /* message box at bottom center of plot */
			{
				MsgBox(3470-lgth,200,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_TOP_RIGHT:          /* message box at top right of plot */
			{
				MsgBox(6770-2*lgth,4400,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_MID_RIGHT:       /* message box at middle right of plot */
			{
				MsgBox(6770-2*lgth,2270,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_BOT_RIGHT:       /* message box at bottom right of plot */
			{
				MsgBox(6770-2*lgth,200,p->message_str);
				break;
			}                                                    /* end case */
		}                                                      /* end switch */
	}

	#if(1) // New code 10/27/96
	// Write label
	if(bPlotLabel)
	{
		width=intopix(strwidth(szPlotLabel, 0.05));
		putlabel(szPlotLabel, 8192-2*(width+20), 20, 0.05, 0, PIXEL_UNITS);
	}
    #endif

	// Activate mouse cursor
	if(p->do_mouse)
	{
		// Allocate memory for x and y point arrays
		fx=(float*)gpcalloc(64, sizeof(float));
		fy=(float*)gpcalloc(64, sizeof(float));

		curson(0, 0, &fx, &fy, &find);                /* turn on mouse cursor*/

		// Free memory - this assumes that point info isn't needed
		gpcfree((void**)&fx);
		gpcfree((void**)&fy);
	}                                                              /* end if */

	rightax(0);
	endplot();
									/* finish plot and wait for instructions */
												/* to exit or draw next plot */
										/* resets file pointers and defaults */

	// Free plot arrays - i assignments are to avoid compiler warnings
	i=GlobalFreePtr(x);
	i=GlobalFreePtr(y);
	i=i;

	stopplot();            /* close files, return to text mode, exit program */
}                                                          /* end FreqPlot() */

/*****************************************************************************/
/*                                                                           */
/*								SigmaPlot()                                  */
/*                                                                           */
/*		Function to plot Allan variance data                                 */
/*      Parameters:                                                          */
/*			p plotdata structure elements:                                   */
/*      		title = plot title                                           */
/*      		subtitle = plot subtitle                                     */
/*				do_box = draw box flag                                       */
/*				do_errbar = draw error bars flag                             */
/*				do_date = write date flag                                    */
/*          	do_line = line menu flag                                     */
/*				do_table = write sigma-tau table on plot flag                */
/*				plotype = 0 normal sigma y                                   */
/*						  1 modified sigma y                                 */
/*						  2 sigma x                                          */
/*					  	  3 overlapping normal sigma y                       */
/*						  4 both overlapping & modified sigma y              */
/*						  5 all tau (not used)                               */
/*						  6 total sigma y                                    */
/*						  7 Hadamard deviation                               */
/*				new_file = new sigma data file flag                          */
/*				position = position of message box                           */
/*				plot_filename = .tkf filename for plot                       */
/*				x_min = tau scale minimum                                    */
/*				x_max = tau scale maximum                                    */
/*				y_min = sigma scale minimum                                  */
/*				y_max = sigma scale maximum                                  */
/*				message_str = message box text                               */
/*                                                                           */
/*				The following plotdata structure elements are not used:      */
/*					x_offset                                                 */
/*					x_mult                                                   */
/*					y_offset                                                 */
/*					y_mult                                                   */
/*					x_label                                                  */
/*					left_x_label                                             */
/*					right_y_label                                            */
/*					x_div                                                    */
/*					x_tic                                                    */
/*					y_div                                                    */
/*					y_tic                                                    */
/*              	config                                                   */
/*                                                                           */
/*          BOOL autoscale_s = flag for autoscaling of sigma axis            */
/*          BOOL autoscale_t = flag for autoscaling of tau axis              */
/*          BOOL read_flag   = flag indicating plot of stability data file   */
/*                                                                           */
/*																			 */
/* 			sigmadata structure elements for sigma data:                     */
/*				int		points                   number of sigma data points */
/*				float	*tau            pointer to averaging time data array */
/*				float	*num          pointer to number of data points array */
/*				float	*sigma                   pointer to sigma data array */
/*				float	*ci                   pointer to confidence interval */
/*				float	*sig_min      pointer to lower sigma error bar array */
/*				float	*sig_max      pointer to upper sigma error bar array */
/*				float	*std_dev         pointer to standard deviation array */
/*				int		*alpha          pointer to noise type exponent array */
/*                                                                           */
/*          The sigmadata structures are:                                    */
/*				sNormSigData;  // Normal & overlapping sigma data struct     */
/*				sModSigData;   // Modified & time sigma data struct          */
/*                                                                           */
/*			 SigmaPlotTypes:                                                 */
/*				NORMAL_SIGMA                                                 */
/*				OVERLAPPING_SIGMA                                            */
/*				MODIFIED_SIGMA                                               */
/*				TIME_SIGMA                                                   */
/*				TOTAL_SIGMA                                                  */
/*				HADAMARD_SIGMA                                               */
/*				BOTH_SIGMA                                                   */
/*				ALLTAU_SIGMA                                                 */
/*                                                                           */
/*      Returns nothing (void)                                               */
/*                                                                           */
/*		Revision record:                                                     */
/*			08/17/96	Adapted from plot_sigma() of Stable/DOS              */
/*			10/26/96	Got noise lines, sigma table, notes & messages going */
/*			10/28/96	Added bias factors to plot scales                    */
/*			11/18/96	Revised plot sizes, etc.                             */
/*			12/17/96	Adjusted x-axis label & table box positions          */
/*			01/10/97	Added bias factors to mod & time sigma plot scales   */
/*			01/28/97	Added some code for all tau sigma type               */
/*			01/29/97	Completed code for all tau sigma type                */
/*			01/30/97	Increased # allowable points for Read case           */
/*						Added provisions for multiple sets of stability data */
/*						and plotting of curve w/o symbols
/*                                                                           */
/*****************************************************************************/

void FAR PASCAL _export SigmaPlot(struct plotdata *p, BOOL autoscale_s,
	BOOL autoscale_t, BOOL read_flag)
{
	// Local macros
	#define LABELSIZE 100

	// Local variables
	// Chars
	char label_str[LABELSIZE+1];
	char *szTemp;
	// Ints
	int i;							// Index
	int j;                          // Ditto
	int k;                          // Ditto
	int nDummy;                     // Dummy used to avoid complier warning
	int noise_flag=FALSE;           //
	int xdec;                       // # x decades in plot
	int ydec;                       // # y decades in plot
	int len;
	int lgth;
	// Colors
	int background_color=BRT_WHITE;
	int grid_color=BLACK;
	int box_color=BLACK;
	int label_color=BLUE;
	int data_color=RED;
	int annotation_color=BLACK;
	int line_color=GREEN;
	int aux_color=GREEN;
	// BOOLS
	BOOL bAllocError=FALSE;         // Memory allocation error flag
	BOOL tdev_flag=FALSE;           // Flag for TDEV line annotation
	// Floats
	float *x, *y;     				// Pointers to plot data
	float x1, y1, x2, y2;           // Endpoints of TDEV line
	float width;                    // Half-width of title string
	float temp1, temp2;
	float line_value;               // Value of TDEV line
	static float sigma_1[8];

	// Start of code
	#if(0) // For testing
	TRACE_INT(sNormSigData.points);
	#endif

	#if(1) // To avoid complier warning
	read_flag=read_flag;
	#endif

	#if(0) // For testing
	TRACE_EXP(p->y_min);
	TRACE_EXP(p->y_max);
	#endif

	#if(0) // For testing
	autoscale_s=FALSE;
	autoscale_t=FALSE;
	#endif

	// Allocate memory for plot data arrays
	if( (nSigmaPlotType==ALLTAU_SIGMA) || (nPlotType==READ_PLOT) )
	{
		if((x=(float*)GlobalAllocPtr(GHND, (ALLSIGMA_SIZE)*sizeof(float)))==
			NULL)
		{
			bAllocError=TRUE;
		}

		if((y=(float*)GlobalAllocPtr(GHND, (ALLSIGMA_SIZE)*sizeof(float)))==
			NULL)
		{
			bAllocError=TRUE;
		}
	}
	else // Other sigma types
	{
		if((x=(float*)GlobalAllocPtr(GHND, (SIGMA_SIZE)*sizeof(float)))==NULL)
		{
			bAllocError=TRUE;
		}

		if((y=(float*)GlobalAllocPtr(GHND, (SIGMA_SIZE)*sizeof(float)))==NULL)
		{
			bAllocError=TRUE;
		}
	}

	// Announce memory allocation error
	if(bAllocError)
	{
		AllocErrorMessage();
		return;
	}

	// Sequence of basic GraphiC plotting calls is per page I-34 of manual:
	//	   bgnplot()           	Initializations
	//	   startplot()         	Starts graphics
	//	   font()              	Loads fonts
	//	   page()              	Sets overall picture size
	//	   heading()			Draws title
	//	   xname(), yname()	 	Draws axis labels
	//	   area2d()				Defines axis lengths

	bgnplot(1,'g',p->plot_filename);                      /* initialize plot */
	startplot(background_color);         /* start plot with background color */

	// Load fonts
	font(4,"news.fnt",'\310',"swiss.fnt",'\311',"newsgrm.fnt",'\312',
		 "simplex.fnt",'\313');

	page(FULLPAGE);                  /* Sets the maximum allowable page size */

	heading("");    /* Blank heading allows space for title & subtitle later */
	fillfont(1);                                         /* Use filled fonts */

	// Kill the beep
	GPCSetBeepDefaults(0, 0);

	// Axis labeling code

	#if(1) // New code 12/17/96
	SetLabelSpace(X_AXIS, 2);
	#endif

	fntchg('\310');                                              /* news.fnt */
	color(label_color);                 /* left y-axis label for single plot */
	LoadString(hInst, IDS_SPLOT_XAXIS_0, label_str,	LABELSIZE);
	xname(label_str);                                        /* x-axis label */

	// Compose y-axis label
	switch(nSigmaPlotType)
	{
		case OVERLAPPING_SIGMA:
		{
			LoadString(hInst, IDS_SPLOT_YAXIS_0, label_str,	LABELSIZE);
			yname(label_str);
			break;
		}
		case TOTAL_SIGMA:
		{
			LoadString(hInst, IDS_SPLOT_YAXIS_1, label_str,	LABELSIZE);
			yname(label_str);
			break;
		}
		case HADAMARD_SIGMA:
		{
			LoadString(hInst, IDS_SPLOT_YAXIS_2, label_str,	LABELSIZE);
			yname(label_str);
			break;
		}
		case TIME_SIGMA:
		{
			LoadString(hInst, IDS_SPLOT_YAXIS_3, label_str,	LABELSIZE);
			yname(label_str);
			break;
		}
		case MODIFIED_SIGMA:
		{
			LoadString(hInst, IDS_SPLOT_YAXIS_4, label_str,	LABELSIZE);
			yname(label_str);
			break;
		}
		case NORMAL_SIGMA:
		case BOTH_SIGMA:
		#if(1) // New code 01/28/97
		case ALLTAU_SIGMA:
		#endif
		{
			LoadString(hInst, IDS_SPLOT_YAXIS_5, label_str,	LABELSIZE);
			yname(label_str);
			break;
		}
	}


	#if(0) // Eliminated - This would allow user choice of axis labels
	// Want standarized labels depending on the variance type
	yname(p->left_y_label);                                 /* labels y-axis */
	xname(p->x_label);                                      /* labels x-axis */
	#endif

	area2d(7.8, 5.3);                         /* Sets the area of the plot */

	#if(1) // Trial code 11/28/96
	// Position plot on page
	// Can do this in absolute terms for the sigma plot
	// because the y-axis scale labels are always the same size
	physor(0.75, 0.80);
	#endif

	color(grid_color);                                         /* grid color */
	grid(4);                         /* draws dotted grid through tick marks */
	fgrid(9,0);                            /* dotted lines on subdivisions */
			/* log scales ignore requested # of subdiv and use 10 per decade */

	frame(1,1);                          /* add frame with ticks around plot */

	// Draw box
	color(box_color);                                  /* set box line color */
	if(p->do_box)
	{
		box();                                    /* draw box around picture */
	}                                                              /* end if */

	// Load data points
	#if(0) // Old code
	if(nSigmaPlotType==NORMAL_SIGMA || nSigmaPlotType==OVERLAPPING_SIGMA ||
		nSigmaPlotType==TOTAL_SIGMA  ||  nSigmaPlotType==BOTH_SIGMA ||
		nSigmaPlotType==HADAMARD_SIGMA)
	#endif

	#if(1) // New code 01/28/97
	if(nSigmaPlotType==NORMAL_SIGMA || nSigmaPlotType==OVERLAPPING_SIGMA ||
		nSigmaPlotType==TOTAL_SIGMA  ||  nSigmaPlotType==BOTH_SIGMA ||
		nSigmaPlotType==HADAMARD_SIGMA || nSigmaPlotType==ALLTAU_SIGMA)
	#endif
	{
		for(i=0; i<sNormSigData.points; i++)
		{        		                                 /* load data points */
			x[i+2]=sNormSigData.tau[i];
			y[i+2]=sNormSigData.sigma[i];
		}                                                         /* end for */

		if(autoscale_t)
		{
			// Put x scale limits into 1st 2 points
			x[0]=x[2];
			x[1]=x[2];
		}
		else
		{
			// Apply bias factors to assure correct scales
			x[0]=p->x_min*1.001;
			x[1]=p->x_max*0.999;
		}

		if(autoscale_s)
		{
			// Put y scale limits into 1st 2 points
			y[0]=y[2];
			y[1]=y[2];
		}
		else
		{
			// Apply bias factors to assure correct scales
			y[0]=p->y_min*1.001;
			y[1]=p->y_max*0.999;
		}

		// Set plot scales
		i=i;

		#if(1) // New code 11/06/96
		if( (autoscale_t) || (autoscale_s) )
		{
			lglgscle(x,y,sNormSigData.points+2); /* set scale using all data */
		}
		else
		{
			lglgscle(x,y,2);                  /* set scale using limits only */
		}
		#endif
	}
	else
	{
		for(i=0; i<sModSigData.points; i++)
		{        		                                 /* load data points */
			x[i+1]=sModSigData.tau[i];
			y[i+1]=sModSigData.sigma[i];
		}                                                         /* end for */

		if(autoscale_t)
		{
			x[0]=x[1];
			x[sModSigData.points+1]=x[sModSigData.points];
		}
		else
		{
			#if(0) // Old code
			x[0]=p->x_min;
			x[sModSigData.points+1]=p->x_max;
			#endif

			#if(1) // New code 01/10/97
			// Apply bias factors to assure correct scales
			x[0]=p->x_min*1.001;
			x[sModSigData.points+1]=p->x_max*0.999;
			#endif
		}

		if(autoscale_s)
		{
			y[0]=y[1];
			y[sModSigData.points+1]=y[sModSigData.points];
		}
		else
		{
			#if(0) // Old code
			y[0]=p->y_min;
			y[sModSigData.points+1]=p->y_max;
			#endif

			#if(1) // New code 01/10/97
			// Apply bias factors to assure correct scales
			y[0]=p->y_min*1.001;
			y[sModSigData.points+1]=p->y_max*0.999;
			#endif
		}

		// Set plot scales
		lglgscle(x,y,sModSigData.points+2);     /* self scaling log-log axes */
	}

	// Draw annotations
	// The putlabel() function is used for user-defined annotations.
	// This must come before any plotting - including grid lines
	// Then it is supposed to stay clean.
	// But, for user units, it should come after plot scales are set
	// It provides user messages via the plot annotations dialog box
	// The 2nd & 3rd args of putlabel() are in plot user units.
	// The 4th arg of putlabel() is char hgth in inches (for full-page).
	// The sFNote[i].nSize value is point size.  There are 72 points/inch.
	// The 5th arg of putlabel is border code: 1=border, 2=no border (strange)
	// So we get it as 2-sFNote[i].bBorder, where bBorder is ordinary BOOL flag
	// Multiline labels are OK with \n (newline, ASCII LF, dec int 10)
	// But they are NG with CR & LF as produced by an edit control.
	// So we must elim CRs from the note text string

	fntchg('\313');                                           /* simplex.fnt */
	for(i=0; i<SNOTE_NUMNOTES; i++)
	{
		// Check if this is an active note (non-zero length)
		len=strlen(sSNote[i].szText);
		if(len)
		{
			// Allocate a temporary working buffer
			if((szTemp=(LPSTR)GlobalAllocPtr(GHND,
				(len+1)*sizeof(char)))==NULL)
			{
				AllocErrorMessage();
			}
			// Remove CRs from annotation text string
			// Note that we are also copying the EOS
			k=0;
			for(j=0; j<=len; j++)
			{
				if(sSNote[i].szText[j]!=13) // 13 = CR
				{
					szTemp[k]=sSNote[i].szText[j];
					k++;
				}
			}
			// Draw annotation - use temp string w/o CRs
			putlabel(szTemp, sSNote[i].fX, sSNote[i].fY,
				sSNote[i].nSize/72.0, 2-sSNote[i].bBorder, USER_UNITS);

			// Free temporary working buffer
			// nDummy is to avoid compiler warning
			nDummy=GlobalFreePtr(szTemp);
			nDummy=nDummy;
		}
	}

	fntchg('\310');                                              /* news.fnt */

	// Plotting code starts here
	fntchg('\311');                                             /* swiss.fnt */
	width=strwidth(p->title,0.3);                 /* get half-width of title */
	prtfnt((4.75-width),6.40,p->title,0.3,0);                  /* draw title */
	width=strwidth(p->subtitle,0.15);          /* get half-width of subtitle */
	prtfnt((4.75-width),6.175,p->subtitle,0.15,0);           /* draw subtitle */
	color(data_color);                                         /* data color */

	#if(0) // Old code
	if(nSigmaPlotType==NORMAL_SIGMA || nSigmaPlotType==OVERLAPPING_SIGMA ||
		nSigmaPlotType==TOTAL_SIGMA  ||  nSigmaPlotType==BOTH_SIGMA ||
		nSigmaPlotType==HADAMARD_SIGMA)
	#endif

	#if(1) // New code 01/28/97
	if(nSigmaPlotType==NORMAL_SIGMA || nSigmaPlotType==OVERLAPPING_SIGMA ||
		nSigmaPlotType==TOTAL_SIGMA  ||  nSigmaPlotType==BOTH_SIGMA ||
		nSigmaPlotType==HADAMARD_SIGMA || nSigmaPlotType==ALLTAU_SIGMA)
	#endif
	{
		symht(0.075);                                     /* set symbol size */
		sympick(12);                                              /* solid o */


		#if(0) // Old code
		for(i=0; i<sNormSigData.points; i++)
		{        	                                     /* load data points */
			x[i]=sNormSigData.tau[i];
			y[i]=sNormSigData.sigma[i];
		}                                                         /* end for */

		curve(x,y,sNormSigData.points,1);         /* draw curve with symbols */

		if(p->do_errbar)
		{
			for(i=0; i<sNormSigData.points; i++)
			{
				{
					errorbar(sNormSigData.tau[i], sNormSigData.tau[i],
						sNormSigData.tau[i], sNormSigData.sigma[i],
						sNormSigData.sig_min[i], sNormSigData.sig_max[i]);
				}
			}                                                     /* end for */
		}										   	               /* end if */
		#endif

		#if(1) // New code 01/30/97
		i=0;
		// Loop thru data points, ploting them in sets
		while(i<sNormSigData.points)
		{        	                                     /* load data points */
			// Store 1st point of a set
			x[0]=sNormSigData.tau[i];
			y[0]=sNormSigData.sigma[i];

			// Store remaining points of a set
			j=1;
			// End set when all points stored, or if next tau is smaller
			while( (i+j<sNormSigData.points) &&
				(sNormSigData.tau[i+j]>sNormSigData.tau[i+j-1]))
			{
				x[j]=sNormSigData.tau[i+j];
				y[j]=sNormSigData.sigma[i+j];
				j++;
			}

			// Draw curve
			if(sNormSigData.num[i])
			{
				// Draw curve with symbols
				// Normal way
				curve(x,y,j,1);
			}
			else
			{
				// Draw curve without symbols
				// This is nice for a spec curve
				// Select by setting # analysis pts of 1st row of set = 0
				for(k=0; k<j-1; k++)
				{
					uline(sNormSigData.tau[i+k], sNormSigData.sigma[i+k],
						sNormSigData.tau[i+k+1], sNormSigData.sigma[i+k+1], 1);
				}
			}

			// Increment outer index
			i+=j;
		}                                                         /* end for */

		// Draw error bars - none drawn if no max & min sigma data
		if(p->do_errbar)
		{
			for(i=0; i<sNormSigData.points; i++)
			{
				{
					errorbar(sNormSigData.tau[i], sNormSigData.tau[i],
						sNormSigData.tau[i], sNormSigData.sigma[i],
						sNormSigData.sig_min[i], sNormSigData.sig_max[i]);
				}
			}                                                     /* end for */
		}										   	               /* end if */
		#endif
	}                                                              /* end if */

	if(nSigmaPlotType==MODIFIED_SIGMA || nSigmaPlotType==TIME_SIGMA ||
		nSigmaPlotType==BOTH_SIGMA)
	{
		sympick(1);                                                     /* o */
		if(nSigmaPlotType==4)
		{
			color(aux_color);                              /* aux data color */
		}
		for(i=0; i<sModSigData.points; i++)
		{        	                                     /* load data points */
			x[i]=sModSigData.tau[i];
			y[i]=sModSigData.sigma[i];
		}                                                         /* end for */
		curve(x,y,sModSigData.points,1);          /* draw curve with symbols */
		if(p->do_errbar)
		{
			for(i=0; i<sModSigData.points; i++)
			{
				errorbar(sModSigData.tau[i], sModSigData.tau[i],
					sModSigData.tau[i], sModSigData.sigma[i],
					sModSigData.sig_min[i], sModSigData.sig_max[i]);
			}                                                     /* end for */
		}											               /* end if */
	}                                                              /* end if */

	// Start of line ploting code
	color(line_color);                                         /* line color */
	for(i=0; i<8; i++)
	{
		switch(sSLine[i].nLineType)
		{
			case SLINE_NONE:
			{
				break;                                            /* no line */
			}
			case SLINE_END:
			{	                                   /* line between endpoints */
				uline(sSLine[i].fXStart, sSLine[i].fYStart, sSLine[i].fXEnd,
					sSLine[i].fYEnd, 3);
				break;
			}
			case SLINE_NOISE:
			{                                                  /* noise line */
				sigma_1[i]=sSLine[i].fSigma1;

				if( (nSigmaPlotType==MODIFIED_RUN) ||
					(nSigmaPlotType==TIME_RUN) )
				{
					if(sSLine[i].fSlope==-1.5)
					{
						temp1=sModSigData.tau[sSLine[i].nXStart];
						temp2=sModSigData.tau[sSLine[i].nXEnd];
						uline(temp1, sigma_1[i]/(temp1*sqrt(temp1)),
							temp2, sigma_1[i]/(temp2*sqrt(temp2)),3);
					}
					else if(sSLine[i].fSlope==-1.0)
					{
						uline(sModSigData.tau[sSLine[i].nXStart],
							sigma_1[i]/sModSigData.tau[sSLine[i].nXStart],
							sModSigData.tau[sSLine[i].nXEnd],
							sigma_1[i]/sModSigData.tau[sSLine[i].nXEnd],3);
					}
					else if(sSLine[i].fSlope==-0.5)
					{
						uline(sModSigData.tau[sSLine[i].nXStart], sigma_1[i]/
							sqrt(sModSigData.tau[sSLine[i].nXStart]),
							sModSigData.tau[sSLine[i].nXEnd], sigma_1[i]/
							sqrt(sModSigData.tau[sSLine[i].nXEnd]),3);
					}
					else if(sSLine[i].fSlope==0.0)
					{
						uline(sModSigData.tau[sSLine[i].nXStart],
						sigma_1[i],	sModSigData.tau[sSLine[i].nXEnd],
						sigma_1[i],3);
					}
					else if(sSLine[i].fSlope==0.5)
					{
						uline(sModSigData.tau[sSLine[i].nXStart], sigma_1[i]*
							sqrt(sModSigData.tau[sSLine[i].nXStart]),
							sModSigData.tau[sSLine[i].nXEnd], sigma_1[i]*
							sqrt(sModSigData.tau[sSLine[i].nXEnd]),3);
					}
					else if(sSLine[i].fSlope==1.0)
					{
						uline(sModSigData.tau[sSLine[i].nXStart],
						sigma_1[i]*sModSigData.tau[sSLine[i].nXStart],
						sModSigData.tau[sSLine[i].nXEnd],
						sigma_1[i]*sModSigData.tau[sSLine[i].nXEnd],3);
					}
					else if(sSLine[i].fSlope==1.5)
					{
						uline(sModSigData.tau[sSLine[i].nXStart],
						sigma_1[i]*sModSigData.tau[sSLine[i].nXStart]
						*sqrt(sModSigData.tau[sSLine[i].nXStart]),
						sModSigData.tau[sSLine[i].nXEnd],
						sigma_1[i]*sModSigData.tau[sSLine[i].nXEnd]
						*sqrt(sModSigData.tau[sSLine[i].nXEnd]),3);
					}
					else if(sSLine[i].fSlope==2.0)
					{
						uline(sModSigData.tau[sSLine[i].nXStart],
						sigma_1[i]*sModSigData.tau[sSLine[i].nXStart]
						*sModSigData.tau[sSLine[i].nXStart],
						sModSigData.tau[sSLine[i].nXEnd],
						sigma_1[i]*sModSigData.tau[sSLine[i].nXEnd]
						*sModSigData.tau[sSLine[i].nXEnd],3);
					}
				}
				else // ADEV
				{
					if(sSLine[i].fSlope==-1.5)
					{
						temp1=sNormSigData.tau[sSLine[i].nXStart];
						temp2=sNormSigData.tau[sSLine[i].nXEnd];
						uline(temp1, sigma_1[i]/(temp1*sqrt(temp1)),
							temp2, sigma_1[i]/(temp2*sqrt(temp2)),3);
					}
					else if(sSLine[i].fSlope==-1.0)
					{
						uline(sNormSigData.tau[sSLine[i].nXStart],
							sigma_1[i]/sNormSigData.tau[sSLine[i].nXStart],
							sNormSigData.tau[sSLine[i].nXEnd],
							sigma_1[i]/sNormSigData.tau[sSLine[i].nXEnd],3);
					}
					else if(sSLine[i].fSlope==-0.5)
					{
						uline(sNormSigData.tau[sSLine[i].nXStart], sigma_1[i]/
							sqrt(sNormSigData.tau[sSLine[i].nXStart]),
							sNormSigData.tau[sSLine[i].nXEnd], sigma_1[i]/
							sqrt(sNormSigData.tau[sSLine[i].nXEnd]),3);
					}
					else if(sSLine[i].fSlope==0.0)
					{
						uline(sNormSigData.tau[sSLine[i].nXStart],
						sigma_1[i],	sNormSigData.tau[sSLine[i].nXEnd],
						sigma_1[i],3);
					}
					else if(sSLine[i].fSlope==0.5)
					{
						uline(sNormSigData.tau[sSLine[i].nXStart], sigma_1[i]*
							sqrt(sNormSigData.tau[sSLine[i].nXStart]),
							sNormSigData.tau[sSLine[i].nXEnd], sigma_1[i]*
							sqrt(sNormSigData.tau[sSLine[i].nXEnd]),3);
					}
					else if(sSLine[i].fSlope==1.0)
					{
						uline(sNormSigData.tau[sSLine[i].nXStart],
						sigma_1[i]*sNormSigData.tau[sSLine[i].nXStart],
						sNormSigData.tau[sSLine[i].nXEnd],
						sigma_1[i]*sNormSigData.tau[sSLine[i].nXEnd],3);
					}
					else if(sSLine[i].fSlope==1.5)
					{
						uline(sNormSigData.tau[sSLine[i].nXStart],
						sigma_1[i]*sNormSigData.tau[sSLine[i].nXStart]
						*sqrt(sNormSigData.tau[sSLine[i].nXStart]),
						sNormSigData.tau[sSLine[i].nXEnd],
						sigma_1[i]*sNormSigData.tau[sSLine[i].nXEnd]
						*sqrt(sNormSigData.tau[sSLine[i].nXEnd]),3);
					}
					else if(sSLine[i].fSlope==2.0)
					{
						uline(sNormSigData.tau[sSLine[i].nXStart],
						sigma_1[i]*sNormSigData.tau[sSLine[i].nXStart]
						*sNormSigData.tau[sSLine[i].nXStart],
						sNormSigData.tau[sSLine[i].nXEnd],
						sigma_1[i]*sNormSigData.tau[sSLine[i].nXEnd]
						*sNormSigData.tau[sSLine[i].nXEnd],3);
					}
				}
				if( (!noise_flag) && (!tdev_flag) )
				 /* can show only 1 noise fit, and only if no TDEV shown */
				{
					sprintf(label_str,
					"1st Noise Fit: Sigma(1)=%-10.7e, Slope=%+4.1f",
					sigma_1[i], sSLine[i].fSlope);
					fntchg('\313');                           /* simplex.fnt */
					color(annotation_color);         /* fit annotation color */                                                /* black */
					#if(0) // Old code
					tmargin(6.655);
					#endif
					#if(1) // New code 12/17/96 for SetLabelSpace() call
					tmargin(6.7);
					#endif
					ctline(label_str,0.075);
					noise_flag=TRUE;
					color(line_color);                         /* line color */
				}
				break;
			}
			case SLINE_HORIZ:
			{	                                      /* horizontal line */
					uline(sSLine[i].fXStart, sSLine[i].fYStart,
						sSLine[i].fXEnd, sSLine[i].fYStart, 3);
					break;
			}
			case SLINE_VERT:
			{                                         /* vertical line */
					uline(sSLine[i].fXStart, sSLine[i].fYStart,
						sSLine[i].fXStart, sSLine[i].fYEnd, 3);
					break;
			}
			case SLINE_TDEV:
			{                                            /* TDEV lines */
					// calc & round # decades in x and y plot scales

					// Use plotdata struct limits for lines so they can
					// cover full size of manually emtered plot scales.
					xdec=floor((log10(p->x_max/p->x_min)+0.5));
					ydec=floor((log10(p->y_max/p->y_min)+0.5));

					// line endpoints are clipped to within plot limits
					for(j=0; j<xdec+ydec; j++)
					{
						x1=max(p->x_min,1.73*p->x_min*pow10(j-ydec));
						y1=min(1.73*p->y_min*pow10(j),p->y_max);
						x2=min(p->x_max,1.73*p->x_min*pow10(j));
						y2=max(1.73*p->y_min*pow10(j-xdec),p->y_min);
						uline(x1,y1,x2,y2,3);
						fntchg('\313');                       /* simplex.fnt */

						// round TDEV line value
						line_value=floor((log10(p->x_min*
								   p->y_min*pow10(j))+0.5));

						// get label in engineering notation
						TVARLabel((int)line_value,label_str);

						// write label by TDEV line
						pltfnt(0.85*sqrt(x1*x2),1.25*sqrt(y1*y2),
							   label_str,0.1,
							   -(180/3.14)*atan2(5.25/ydec,7.65/xdec));
					}

					if(!noise_flag)
					/* don't annotate TDEV lines if annotated noise line */
					/* with both noise and TDEV lines, will get note for */
					/* whichever one comes first */
					{
						sprintf(label_str,"Lines = Loci of Constant TDEV");
						fntchg('\313');                       /* simplex.fnt */
						color(annotation_color);     /* fit annotation color */                                                /* black */
						tmargin(6.655);
						ctline(label_str,0.075);
						color(line_color);                     /* line color */
						tdev_flag=TRUE;
						break;
					}                                              /* end if */
			}
		}                                                      /* end switch */
	}                                                             /* end for */
	// End of line plotting code

	// Select annotation font and color
	fntchg('\313');                                           /* simplex.fnt */
	color(annotation_color);                             /* annotation color */                                                    /* black */

	// Draw sigma-tau table
	// Note: y position was 2673 for Stable/DOS GraphiC 5.01
	// Last arg was 2642 - changed everywhere to 2715 on 12/17/96
	// # pixels doubled in GraphiC 7.1
	// x position values are changed in TableBox(), y values are used x2
	if(p->do_table)                                             /* do table? */
	{
		if(nSigmaPlotType==BOTH_SIGMA)                          /* dual plot */
		{
			TableBox(2, sNormSigData.points,2150,2715); /* do dual - table */
		}                                                          /* end if */
		else if(nSigmaPlotType==TIME_SIGMA)   /* time sig table at left side */
		{
					 /* adjust x location depending on int or exp tau format */
								/* to undo adjustment that table_box() makes */
											/* for right side table location */
			if(sModSigData.tau[0]>=1.0 &&
				sModSigData.tau[sModSigData.points-1]<=9999)
			{
				// Int format (was 250)
				TableBox(1, sModSigData.points,350,2715); /* write - table */
			}                                                      /* end if */
			else
			{
				// Exp format (was 458)
				TableBox(1, sModSigData.points,810,2715); /* write - table */
			}                                                    /* end else */
		}                                                          /* end if */

		else if(nSigmaPlotType==MODIFIED_SIGMA)
		{
			TableBox(1, sModSigData.points,2730,2715);    /* write - table */
		}                                                          /* end if */
		else											 /* other plot types */
		{
			TableBox(1, sNormSigData.points,2730,2715);   /* write - table */
		}                                                          /* end if */
	}                                                              /* end if */

	// Write date
	if(p->do_date)
	{
		Dateline(1);                         /* put date & file info on plot */
	}                                                              /* end if */

	// Display message box
	// Locations are x, y in pixels, origin at bottom left, 8192 x 6266 space
	// Find half length of message box in pixels
	lgth=(H_MARGIN+strwidtq(p->message_str, CHAR_HGTH));

	// Write message
	if(p->message_str[0]!=NULL)         /* don't display a blank message box */
	{
		switch(p->position)
		{
			case MSG_TOP_LEFT:            /* message box at top left of plot */
			{
				MsgBox(300,4440,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_MID_LEFT:         /* message box at middle left of plot */
			{
				MsgBox(300,2270,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_BOT_LEFT:         /* message box at bottom left of plot */
			{
				MsgBox(300,200,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_TOP_CENTER:        /* message box at top center of plot */
			{
				MsgBox(3470-lgth, 4440, p->message_str);
				break;
			}                                                    /* end case */
			case MSG_MID_CENTER:     /* message box at middle center of plot */
			{
				MsgBox(3470-lgth,2270,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_BOT_CENTER:     /* message box at bottom center of plot */
			{
				MsgBox(3470-lgth,200,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_TOP_RIGHT:          /* message box at top right of plot */
			{
				MsgBox(6770-2*lgth,4440,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_MID_RIGHT:       /* message box at middle right of plot */
			{
				MsgBox(6770-2*lgth,2270,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_BOT_RIGHT:       /* message box at bottom right of plot */
			{
				MsgBox(6770-2*lgth,200,p->message_str);
				break;
			}                                                    /* end case */
		}                                                      /* end switch */
	}

	// Write label
	if(bPlotLabel)
	{
		width=intopix(strwidth(szPlotLabel, 0.05));
		putlabel(szPlotLabel, 8192-2*(width+20), 20, 0.05, 0, PIXEL_UNITS);
	}

	#if(0) // Inactivated - Not wanted for sigma plot
	// Activate mouse cursor
	if(p->do_mouse)
	{
		// Allocate memory for x and y point arrays
		fx=(float*)gpcalloc(64, sizeof(float));
		fy=(float*)gpcalloc(64, sizeof(float));

		curson(0, 0, &fx, &fy, &find);                /* turn on mouse cursor*/

	}                                                              /* end if */
	#endif

	// Free plot data arrays
	// nDummy is to avoid compiler warning
	nDummy=GlobalFreePtr(x);
	nDummy=GlobalFreePtr(y);
	nDummy=nDummy;

	endplot();
									/* finish plot and wait for instructions */
												/* to exit or draw next plot */
										/* resets file pointers and defaults */

	stopplot();            /* close files, return to text mode, exit program */
}                                                         /* end SigmaPlot() */

/*****************************************************************************/
/*                                                                           */
/*			   		       PowerSpectrumPlot()                               */
/*                                                                           */
/*		Function to plot power spectrum data                                 */
/*      Parameters:                                                          */
/*      	f_data[] = array of Fourier freq data                            */
/*          s_data[] = array of power spectral data                          */
/*			int points = # power spectrum points                             */
/*			int type = phase noise type: 0=Sx(f), 1=S(f), 2=(f), 3=Sy(f)   */
/*		p plotdata structure elements:                                       */
/*      	spectrum_title = plot title                                      */
/*      	spectrum_subtitle = plot subtitle                                */
/*			do_box  = draw box flag                                          */
/*			do_date = write date flag                                        */
/*			do_line = add noise line to plot                                 */
/*			spectrum_plot_filename = .tkf filename for plot                  */
/*      Returns nothing (void)                                               */
/*                                                                           */
/*      Revision record:                                                     */
/*      	06/30/96	Adapted from plot_spectrum() of Stable/DOS           */
/*			11/18/96	Revised plot sizes, etc.                             */
/*			12/27/96	Up & running                                         */
/*                                                                           */
/*****************************************************************************/

void FAR PASCAL _export PowerSpectrumPlot(float f_data[], float s_data[],
	int points, int type, struct plotdata *p)
{
	int i;                                               /* outer loop index */
	int j;                                               /* inner loop index */
	int start;
	int af;                                              /* averaging factor */
	int line_num;                                       /* # line fit points */
	int background_color;
	int grid_color;
	int label_color;
	int data_color;
	int line_color;
	int annotation_color;
	int lgth;
	float width;
	double sum=0;                               /* sum variables for average */
	float line_s[12];                            /* line fit spectral points */
	float line_f[12];                           /* line fit frequency points */
	float num;                                  /* float version of line_num */
	double sum_f=0;                          /* line fit summation variables */
	double sum_s=0;
	double sum_ff=0;
	double sum_fs=0;
	double slope;                        /* power-law slope of S(f) line fit */
	double intercept;                      /* intercept of log S(f) line fit */
	float k;                                 /* numerical value of intercept */
	double temp;                     /* temp variable to hold log(line_f[i]) */
	char label_str[100];

	// Cursor variables
	float *fx;                  // Pointer to x-axis point array
	float *fy;                  // Pointer to y-axis point array
	int find;					// Point array dimensions

	// assign colors
	background_color=BRT_WHITE;
	grid_color=BLACK;
	label_color=BLUE;
	data_color=RED;
	line_color=GREEN;
	annotation_color=BLACK;

	// Sequence of basic GraphiC plotting calls S/B per page I-34 of manual:
	//	   bgnplot()           	Initializations
	//	   startplot()         	Starts graphics
	//	   font()              	Loads fonts
	//	   page()              	Sets overall picture size
	//	   heading()			Draws title
	//	   xname(), yname()	 	Draws axis labels
	//	   area2d()				Defines axis lengths

	bgnplot(1, 'g', p->plot_filename);                    /* initialize plot */
	startplot(background_color);         /* start plot with background color */

	// Load fonts
	font(4,"news.fnt", '\310', "swiss.fnt", '\311', "newsgrm.fnt", '\312',
		 "simplex.fnt", '\313');

	page(FULLPAGE);                  /* Sets the maximum allowable page size */

	heading("");    /* Blank heading allows space for title & subtitle later */
	fillfont(1);                                         /* Use filled fonts */

	// Kill the beep
	GPCSetBeepDefaults(0, 0);

	color(label_color);                                       /* label color */
	xname("\310Fourier Frequency, f, Hz");                   /* x-axis label */

	if(bDataType) // Sy(f)
	{
		yname("\310Log PSD, S]y[(f), Hz[-1");         /* label for freq data */
	}
	else                                                       /* phase data */
	{
		if(type==0) // Sx(f)
		{
			yname("\310Log PSD, S]x[(f), sec[2]/Hz");
		}
		if(type==1) // S(f)
		{
			yname("\310Phase Noise, Log S]\312f[\310(f), rad[2]/Hz");
		}
		if(type==2) // (f)
		{
			yname("\310Phase Noise, L(f), dBc/Hz");
		}
	}

	fntchg('\310');                                              /* news.fnt */
	area2d(7.8, 5.3);                                    /* plot dimenisions */
	upright(1);                                  /* horizontal y-axis labels */
	color(grid_color);                                         /* grid color */
	grid(2);                                          /* fine dot grid lines */
	fgrid(9, 0);                             /* dotted lines on subdivisions */
			/* log scales ignore requested # of subdiv and use 10 per decade */
	frame(1, 1);                         /* add frame with ticks around plot */

	if(p->do_box)
	{
		box();                              /* draw a box around the picture */
	}

	xlgscle(5, f_data, s_data, points);         /* self scaling semilog axes */

	// Note: to calc S(f), rad/Hz, must enter nominal frequency, v, and
	// multiply Sx(f) by (2v).  Sy(f), Sx(f) and S(f) are expressed as
	// log10 of their values.  (f) is S(f); it is expressed as 10log10 of
	// its numerical value. Those terms are conveyed by correction_factor to
	// SpectrumCalc()

	fntchg('\311');                                             /* swiss.fnt */
	width=strwidth(p->title, 0.3);                /* get half-width of title */
	prtfnt((4.75-width),6.40,p->title,0.3,0);                  /* draw title */
	width=strwidth(p->subtitle, 0.15);         /* get half-width of subtitle */
	prtfnt((4.75-width), 6.175, p->subtitle, 0.15, 0);      /* draw subtitle */
	color(data_color);                                         /* data color */
	sympick(1);
	curve(f_data, s_data, points, 0);          /* draw curve without symbols */

	if( (p->do_line) && (points>8) )
	{
		af=1;
		start=0;
		color(line_color);                                     /* line color */

		for(i=0; i<(log(points)/log(2))-1; i++)     /* calc log-weighted avg */
		{
			af*=2;
			sum=0;
			for(j=0; j<af; j++)
			{
				sum+=s_data[start+j];
			}
			line_s[i]=sum/af;
			line_f[i]=(f_data[start]+f_data[start+af-1])/2;
			start+=af;
			line_num=i+1;
		}

		curve(line_f, line_s, line_num, -1);  /* draw points as symbols only */

		// Find linear fit to all but 1st point
		num=line_num-1;
		for(i = 1; i < line_num; i++)
		{
			temp=log10(line_f[i]);
			sum_f  += temp;
			sum_s  += line_s[i];
			sum_ff += SQR (temp);
			sum_fs += temp * line_s[i];
		}

		slope =  sum_fs - (sum_f * sum_s / num);
		slope /= sum_ff - (sum_f * sum_f / num);
		intercept = (sum_s  - (slope * sum_f)) / num;

		// Draw the power law fit line
		if(type==0 || type==1 || type==3) // Sx(f) or S(f) or Sy(f)
		{
			k=pow(10.,intercept);
			uline(f_data[0],
				  log10(k*pow(f_data[0], slope)),
				  f_data[points],
				  log10(k*pow(f_data[points], slope)), 3);
		}
		if(type==2) // (f)
		{
			uline(f_data[0],
				  intercept+slope*log10(f_data[0]),
				  f_data[points],
				  intercept+slope*log10(f_data[points]), 3);
		}

		// Label the power law fit
		if(type==0) // Sx(f)
		{
			// Make exponent  (B used)
			// The newsgrm font  is too dark
			sprintf(label_str,
				"Power Law Fit: Sx(f)=kf[B], k=%10.7e,"
				" B=%10.7e", k, slope);
		}
		if(type==1) // S(f)
		{
			// Make exponent  (B used)
			sprintf(label_str,
				"Power Law Fit: So(f)=kf[B], k=%10.7e,"
				" B=%10.7e", k, slope);
		}
		if(type==2) // (f)
		{
			sprintf(label_str,
				"Power Law Fit: L(1)=%5.2f dBc/Hz, Slope=%5.2f dB/decade",
				intercept, slope);
		}
		if(type==3) // Sy(f)
		{
			sprintf(label_str,
				"Power Law Fit: Sy(f)=kf[a], k=%10.7e, a=%10.7e",
				k, slope);
		}

		fntchg('\313');                                       /* simplex.fnt */
		color(annotation_color);                         /* annotation color */
		tmargin(6.655);
		ctline(label_str, 0.075);
	}

	#if(1) // Display windowing info
	if(sPSPlot.do_table)
	{
		fntchg('\313');                                       /* simplex.fnt */
		color(annotation_color);                         /* annotation color */
			  /* right_y_label string used for annotation of FFT window type */
		if(p->position==MSG_BOT_LEFT)  /* win annotation at top left of plot */
		{
			MsgBox(300, 4440, p->right_y_label);
		}
		else                        /* win annotation at bottom left of plot */
		{
			MsgBox(300, 200, p->right_y_label);
		}
	}
	#endif

	if(p->do_date)
	{
		Dateline(1);                         /* put date & file info on plot */
	}

	// Display message box
	// Locations are x, y in pixels, origin at bottom left, 8192 x 6266 space
	// Find half length of message box in pixels
	lgth=(H_MARGIN+strwidtq(p->message_str, CHAR_HGTH));

	// Write message
	if(p->message_str[0]!=NULL)         /* don't display a blank message box */
	{
		switch(p->position)
		{
			case MSG_TOP_LEFT:            /* message box at top left of plot */
			{
				MsgBox(300, 4440, p->message_str);
				break;
			}
			case MSG_MID_LEFT:         /* message box at middle left of plot */
			{
				MsgBox(300, 2270, p->message_str);
				break;
			}
			case MSG_BOT_LEFT:         /* message box at bottom left of plot */
			{
				MsgBox(300, 200, p->message_str);
				break;
			}
			case MSG_TOP_CENTER:        /* message box at top center of plot */
			{
				MsgBox(3470-lgth, 4440, p->message_str);
				break;
			}
			case MSG_MID_CENTER:     /* message box at middle center of plot */
			{
				MsgBox(3470-lgth, 2270, p->message_str);
				break;
			}
			case MSG_BOT_CENTER:     /* message box at bottom center of plot */
			{
				MsgBox(3470-lgth, 200, p->message_str);
				break;
			}
			case MSG_TOP_RIGHT:          /* message box at top right of plot */
			{
				MsgBox(6770-2*lgth, 4440, p->message_str);
				break;
			}
			case MSG_MID_RIGHT:       /* message box at middle right of plot */
			{
				MsgBox(6770-2*lgth, 2270, p->message_str);
				break;
			}
			case MSG_BOT_RIGHT:       /* message box at bottom right of plot */
			{
				MsgBox(6770-2*lgth, 200, p->message_str);
				break;
			}
		}
	}

	// Write label
	if(bPlotLabel)
	{
		width=intopix(strwidth(szPlotLabel, 0.05));
		putlabel(szPlotLabel, 8192-2*(width+20), 20, 0.05, 0, PIXEL_UNITS);
	}

	// Activate mouse cursor
	if(p->do_mouse)
	{
		// Allocate memory for x and y point arrays
		fx=(float*)gpcalloc(64, sizeof(float));
		fy=(float*)gpcalloc(64, sizeof(float));

		curson(0, 0, &fx, &fy, &find);                /* turn on mouse cursor*/
	}

	endplot();                                             /* terminate plot */
	stopplot();                                      /* close files and quit */
}

/*****************************************************************************/
/*                                                                           */
/*								PhasePlot()                                  */
/*                                                                           */
/*		Function to plot phase data                                          */
/*      Parameters:                                                          */
/*			p_data[] = array of phase data; p_data[0] = # data points        */
/*			p_data[1] = plot start; p_data[2] = plot end                     */
/*      	zero treated as gap in data                                      */
/*		p plotdata struct elements:                                          */
/*          data_title = plot title                                          */
/*          data_subtitle = plot subtitle                                    */
/*			x_label = x-axis label                                           */
/*			y_label = y-axis label                                           */
/*			x_offset = x-axis scale offset                                   */
/*			x_mult = x-axis scale multiplier                                 */
/*			y_offset = y-axis scale offset                                   */
/*			y_mult = y-axis scale multiplier                                 */
/*          scale_min = y-axis scale minimun                                 */
/*          scale_max = y-axis scale maximum                                 */
/*          y_div = # of y-axis major divisions                              */
/*          y_tic = # of y-axis tics between major divisions                 */
/*			do_box = draw box flag                                           */
/*      	do_phase_line = draw regression line flag                        */
/*			do_date = write date flag                                        */
/*     		do_mouse = activate mouse/cursor flag                            */
/*			new_file = new basic phase data file flag                        */
/*			new_name = new exact phase data flag flag                        */
/*			position = position of message box                               */
/*          phase_plot_filename = .tkf filename for plot                     */
/*		Returns nothing (void)                                               */
/*                                                                           */
/*		Revision record:                                                     */
/*			06/30/96	Adapted from phase_plot() of Stable/DOS              */
/*			07/17/96	Adapted from FreqPlot() of Stable/Win                */
/*			11/12/96	Added clipping to line drawing code                  */
/*			11/18/96	Revised plot sizes, etc.                             */
/*			01/10/97	Quadratic drift made slope per data point interval   */
/*                                                                           */
/*****************************************************************************/

void FAR PASCAL _export PhasePlot(F_TYPE p_data[], struct plotdata *p)
{
	// Local variables
	float *x,*y;
	float width;
	float x_max, x_min;                                    /* x scale limits */
	float y_max, y_min;                                    /* y scale limits */
	float xste;                                        /* x-axis tic spacing */
	float yste;                                        /* y-axis tic spacing */
	float save_start;
	float save_end;
	int i;
	int j;
	#if(1) // Now used
	int k;
	int len;				                    	        /* string length */
	int nDummy;
	#endif
	int npts;
	int lgth;                                    /* message length in pixels */
	int line_flag=FALSE;
	char label_str[100];
	char yformat[8];
	char xformat[8];
	#if(1) // Now used
	char *szTemp;
	#endif

	// Colors
	int background_color=BRT_WHITE;
	int grid_color=BLACK;
	int box_color=BLACK;
	int label_color=BLUE;
	int data_color=RED;
	int annotation_color=BLACK;
	int line_color=GREEN;

	// Fit parameters
	F_TYPE avg;
	F_TYPE slope;
	F_TYPE intercept;
	F_TYPE a, b, c, v;
	F_TYPE tau;

	// Cursor variables
	float *fx;                  // Pointer to x-axis point array
	float *fy;                  // Pointer to y-axis point array
	int find;					// Point array dimensions

	// Test for no data
	// Note: NULL plot filename test eliminated
	if((p_data[2]-p_data[1])<=0)
	{
		MessageBox(hwndMain, "No Phase Data!",
			"Stable/Win", MB_ICONEXCLAMATION);
		return;
	}                                                              /* end if */

	// Allocate memory for plot arrays
	if((x=(float*)GlobalAllocPtr(GHND, (p_data[2]+2)*sizeof(float)))==NULL)
	{
		AllocErrorMessage();
		return;
	}
	if((y=(float*)GlobalAllocPtr(GHND, (p_data[2]+2)*sizeof(float)))==NULL)
	{
		AllocErrorMessage();
		return;
	}

	CalcMean(p_data, &avg, FREQ_DATA);

/*****************************************************************************/
#if(0) // Temporarily, eliminate some secondary code
/*****************************************************************************/
	for(i=0; i<PLINE_NUMLINES; i++) /* correct line parameters for averaging */
	{
		sPLine[i].fXStart /= avg_fact[1];
		sPLine[i].fXEnd   /= avg_fact[1];
	}                                                             /* end for */
	avg_fact[1]=1;                              /* only make correction once */

	if(p->new_file)
	{
		for(i=0; i<PLINE_NUMLINES; i++)
		{
			sPLine[i].nLineType=PLINE_NONE;
			sPLine[i].fXStart=p_data[1];
			sPLine[i].fXEnd=p_data[2];
		}                                                         /* end for */
	}                                                              /* end if */

	if(p->new_name)
	{
		for(i=0; i<PLINE_NUMLINES; i++)
		{
			sPLine[i].fYStart=p->y_min;
			sPLine[i].fYEnd=p->y_max;
		}                                                         /* end for */
	}                                                              /* end if */
/*****************************************************************************/
#endif
/*****************************************************************************/

/*****************************************************************************/
/* This is the begining of the actual GraphiC plotting code                  */
/* Sequence of basic calls is per page I-34 of manual:                       */
/*	   bgnplot()            	Initializations                              */
/*	   startplot()          	Starts graphics                              */
/*	   font()               	Loads fonts                                  */
/*	   page()               	Sets overall picture size                    */
/*	   heading()				Draws title                                  */
/*	   xname(), yname()		 	Draws axis labels                            */
/*	   area2d()					Defines axis lengths                         */
/*	                                                                         */
/*****************************************************************************/

	bgnplot(1,'g',p->plot_filename);                      /* initialize plot */
	bdos(0x0C,0,0);                                 /* clear keyboard buffer */
	startplot(background_color);         /* start plot with background color */
	font(4,"news.fnt",'\310',"swiss.fnt",'\311',"newsgrm.fnt",'\312',
		 "simplex.fnt",'\313');                                /* load fonts */

	page(FULLPAGE);                  /* Sets the maximum allowable page size */
	heading("");    /* Blank heading allows space for title & subtitle later */
	fillfont(1);                                         /* Use filled fonts */

	// Kill the beep
	GPCSetBeepDefaults(0, 0);

	// Axis labeling code
	fntchg('\310');                                              /* news.fnt */
	color(label_color);                                        /* all labels */
	yname(p->left_y_label);                                 /* labels y-axis */
	xname(p->x_label);                                  /* labels x-axis */

	area2d(7.8, 5.3);                         /* Sets the area of the plot */

	// Draw box
	color(box_color);                                  /* set box line color */
	if(p->do_box)
	{
		box();                                    /* draw box around picture */
	}                                                              /* end if */

	// Draw grid
	// The grid() function draws the major grid lines (at tics)
	// The fgrid() function draws the fine grid lines (between tics)
	// The 1st fgrid() arg is: 0=no ticks, -1=tick marks
	// The 2nd fgrid() arg is # ticks=# intervals-1
	color(grid_color);                          /* set box & grid line color */
	grid(9);                         /* draws dotted grid through tick marks */
	fgrid(-1,p->y_tic-1);        /* fine grid tick marks between major ticks */

	frame(1,1);                          /* add frame with ticks around plot */
	upright(1);                        /* causes y-axis labels to be upright */
	p->x_div=5;                    /* sets the desired # of x-axis divisions */
	fntchg('\310');                                              /* news.fnt */

	/* find plot scales */
	x_max=p->x_mult*(p_data[2]+p->x_offset);
	x_min=p->x_mult*(p_data[1]+p->x_offset);
	roundq(p->x_div, &x_min, &xste, &x_max, xformat);

	y_max=p->y_mult*(p->y_max+p->y_offset);
	y_min=p->y_mult*(p->y_min+p->y_offset);
	roundq(p->y_div, &y_min, &yste, &y_max, yformat);

	#if(0) // For testing
	TRACE_EXP(x_min);
	TRACE_EXP(x_max);
	TRACE_EXP(y_min);
	TRACE_EXP(y_max);
	#endif

	// Set plot scales
	graf(xformat, x_min, xste, x_max, yformat, y_min, yste, y_max);

	#if(1) // Newly implemented
	// Draw annotations
	// The putlabel() function is used for user-defined annotations.
	// This must come before any plotting - including grid lines
	// Then it is supposed to stay clean.
	// But, for user units, it should come after plot scales are set
	// It provides user messages via the plot annotations dialog box
	// The 2nd & 3rd args of putlabel() are in plot user units.
	// The 4th arg of putlabel() is char hgth in inches (for full-page).
	// The sPNote[i].nSize value is point size.  There are 72 points/inch.
	// The 5th arg of putlabel is border code: 1=border, 2=no border (strange)
	// So we get it as 2-sPNote[i].bBorder, where bBorder is ordinary BOOL flag
	// Multiline labels are OK with \n (newline, ASCII LF, dec int 10)
	// But they are NG with CR & LF as produced by an edit control.
	// So we must elim CRs from the note text string
	fntchg('\313');                                           /* simplex.fnt */
	for(i=0; i<PNOTE_NUMNOTES; i++)
	{
		// Check if this is an active note (non-zero length)
		len=strlen(sPNote[i].szText);
		if(len)
		{
			// Allocate a temporary working buffer
			if((szTemp=(LPSTR)GlobalAllocPtr(GHND,
				(len+1)*sizeof(char)))==NULL)
			{
				AllocErrorMessage();
			}
			// Remove CRs from annotation text string
			// Note that we are also copying the EOS
			k=0;
			for(j=0; j<=len; j++)
			{
				if(sPNote[i].szText[j]!=13) // 13 = CR
				{
					szTemp[k]=sPNote[i].szText[j];
					k++;
				}
			}
			// Draw annotation - use temp string w/o CRs
			putlabel(szTemp, sPNote[i].fX, sPNote[i].fY,
				sPNote[i].nSize/72.0, 2-sPNote[i].bBorder, USER_UNITS);

			// Free temporary working buffer
			// nDummy is to avoid compiler warning
			nDummy=GlobalFreePtr(szTemp);
			nDummy=nDummy;
		}
	}
	#endif

	fntchg('\310');                                              /* news.fnt */

	npts=0;
	for(i=p_data[1]+2; i<=p_data[2]+2; i++)        /* load data into array */
	{
		if(i==p_data[1]+2 || p_data[i] || i==p_data[2]+2)          /* gap? */
		{
			y[npts]=p->y_mult*(p_data[i]+p->y_offset);
			if(y[npts]<y_min)                      /* clip if below bottom */
			{
				y[npts]=y_min;
			}
			if(y[npts]>y_max)                         /* clip if above top */
			{
				y[npts]=y_max;
			}
		}                                                        /* end if */
		else
		{
			y[npts]=0;                                    /* keep gap zero */
		}                                                      /* end else */
		x[npts]=p->x_mult*((i-2)+p->x_offset);
		npts++;
	}                                                           /* end for */

	fntchg('\311');                                             /* swiss.fnt */
	width=strwidth(p->title,0.3);                 /* get half-width of title */
	prtfnt((4.75-width),6.40,p->title,0.3,0);                  /* draw title */
	width=strwidth(p->subtitle,0.15);          /* get half-width of subtitle */
	prtfnt((4.75-width),6.175,p->subtitle,0.15,0);           /* draw subtitle */
	color(data_color);                                         /* data color */

	for(i=0; i<npts-1; i++)                               /* plot phase data */
	{
		if((i==0) || (*(y+i) && *(y+i+1)) || (i==npts-2))            /* gap? */
		{
			uline(*(x+i),*(y+i),*(x+i+1),*(y+i+1),1);
		}                                                          /* end if */
	}                                                             /* end for */

	// Draw lines
	// Note: Need error trapping for entry of NG line parameters

	#if(0) // No such option
	if(p->do_line==PPLOT_LINE)                            /* linear fit line */
	{
		CalcLinFreqDrift(p_data, &intercept, &slope, &v);      /* calc drift */

		color(line_color);                                     /* line color */
		/* draw linear regression line */
		x[0]=p->x_mult*(p_data[1]+p->x_offset);
		y[0]=p->y_mult*((intercept+(p_data[1] * slope))+p->y_offset);
		x[1]=p->x_mult*(p_data[2]+p->x_offset);
		y[1]=p->y_mult*((intercept+(p_data[2] * slope))+p->y_offset);
		curve(x, y, 2, 0);

		sprintf(label_str,
				"Linear Fit: y(t)=a+bt, Intercept=a=%-10.7e, "
				"Slope=b=%-10.7e", intercept,slope);
		fntchg('\313');                                       /* simplex.fnt */
		color(annotation_color);                     /* fit annotation color */
		tmargin(6.655);
		ctline(label_str,0.075);
	}                                                              /* end if */
	#endif

	if(p->do_line==PPLOT_QUAD)                              /* quadratic fit */

///////////////////////////////////////////////////////////////////////////////
///////////////////////////// QUADRATIC DRIFT CALC NG /////////////////////////
////////////// Trouble is with CalcQuadraticDrift(), not plotting /////////////
////////////// quadratic() substituted - works OK /////////////////////////////
///////////////////////////////////////////////////////////////////////////////

  								///////////////////////////////////////////////
								/// Also- need proper start & end point     ///
								/// initialization - esp not start=0.       ///
								///////////////////////////////////////////////


	{
		color(line_color);                                     /* line color */
								/* calc and plot quadratic fit to phase data */
		tau=1.0;     /* set tau=1 since basic x-axis spacing is 1 data point */

		#if(1) // New code 07/18/96
		quadratic(p_data, tau, &a, &b, &c, &v);    /* quadratic fit */
		#endif

		#if(0)
		CalcQuadraticDrift(p_data, tau, &a, &b, &c, &v);    /* quadratic fit */
		#endif

		#if(0) // For testing
		TRACE_EXP(a);
		#endif

		npts=0;
		for(i=p_data[1]+2; i<=p_data[2]+2; i++)   /* put fit data into array */
		{
			y[npts]=p->y_mult*((a+b*(i-1-p_data[1])
					+c*(i-1-p_data[1])*(i-1-p_data[1]))+p->y_offset);
			x[npts]=p->x_mult*((i-2)+p->x_offset);
			npts++;
		}                                                         /* end for */
		for(i=0; i<npts-1; i++)                             /* plot fit data */
		{
			uline(*(x+i),*(y+i),*(x+i+1),*(y+i+1),1);
		}                                                         /* end for */
		sprintf(label_str,
			"Quadratic Fit: x(t)=a+bt+ct[2], "
			"a=%-10.7e, b=%-10.7e, c=%-10.7e", a, b, c);
		fntchg('\313');                                       /* simplex.fnt */
		color(annotation_color);                     /* fit annotation color */
		tmargin(6.655);
		ctline(label_str,0.075);
		#if(1) // New code 01/04/96
		line_flag=TRUE;
        #endif
	}                                                              /* end if */

	#if(0) // Eliminated 11/09/96
	// Want to do this independently
	if(p->do_line==PPLOT_MENU)                                  /* line menu */
	#endif
	{
		color(line_color);                                     /* line color */
		save_start=p_data[1];                         /* save starting point */
		save_end=p_data[2];                             /* save ending point */
		for(i=0; i<PLINE_NUMLINES; i++)
		{
			switch(sPLine[i].nLineType)
			{
				case PLINE_NONE:                                  /* no line */
				{
					break;
				}
				case PLINE_END:                    /* line between endpoints */
				{
					p_data[1]=sPLine[i].fXStart;
					p_data[2]=sPLine[i].fXEnd;

					#if(0) // Old code
					uline(p->x_mult*(p_data[1]+p->x_offset),
						p->y_mult*(sPLine[i].fYStart+p->y_offset),
						p->x_mult*(p_data[2]+p->x_offset),
						p->y_mult*(sPLine[i].fYEnd+p->y_offset),3);
					#endif

					#if(1) // New code with clipping
					uline(max(p->x_mult*(p_data[1]+p->x_offset), x_min),
						max(p->y_mult*(sPLine[i].fYStart+p->y_offset), y_min),
						min(p->x_mult*(p_data[2]+p->x_offset), x_max),
						min(p->y_mult*(sPLine[i].fYEnd+p->y_offset), y_max),
						3);
					#endif

					break;
				}
				case PLINE_LIN:                           /* regression line */
				{
					p_data[1]=sPLine[i].fXStart;
					p_data[2]=sPLine[i].fXEnd;

					CalcLinFreqDrift(p_data, &intercept, &slope, &v);
															   /* calc drift */
					#if(0) // Old code
					uline(p->x_mult*(p_data[1]+p->x_offset),
						p->y_mult*((intercept+(p_data[1] * slope))+p->y_offset),
						p->x_mult*(p_data[2]+p->x_offset),
						p->y_mult*((intercept+(p_data[2] * slope))+p->y_offset),
						3);
					#endif

					#if(1) // New code with clipping
					uline(max(p->x_mult*(p_data[1]+p->x_offset), x_min),
						max(p->y_mult*((intercept+(p_data[1] *
						slope))+p->y_offset), y_min),
						min(p->x_mult*(p_data[2]+p->x_offset), x_max),
						min(p->y_mult*((intercept+(p_data[2] *
						slope))+p->y_offset), y_max),
						3);
					#endif

					if(!line_flag)           /* can list only 1 line fit */
					{
						sprintf(label_str,
						"1st Linear Fit: y(t)=a+bt, "
						"Intercept=a=%-10.7e, Slope=b=%-10.7e",
						intercept,slope);
						fntchg('\313');                   /* simplex.fnt */
						color(annotation_color);     /* annotation color */                                                /* black */
						tmargin(6.655);
						ctline(label_str,0.075);
						line_flag=TRUE;
						color(line_color);                 /* line color */
					}
					break;
				}
				case PLINE_QUAD:                            /* quadratic fit */
				{
					p_data[1]=sPLine[i].fXStart;
					p_data[2]=sPLine[i].fXEnd;

					/* set tau=1 since x-axis spacing is 1 data point */
					tau=1;

					#if(1) // New code 7/18/96
					quadratic(p_data, tau, &a, &b, &c, &v);
					#endif

					#if(0)
					CalcQuadraticDrift(p_data, tau, &a, &b, &c, &v);
					#endif

					npts=p_data[2]-p_data[1]+1;
					for(j=0; j<=npts; j++)
					{
						y[j]=((a+b*(j+1)+c*(j+1)*(j+1))+p->y_offset);
						y[j]*=p->y_mult;
						x[j]=p->x_mult*((p_data[1]+j)+p->x_offset);
					}                                             /* end for */
					for(j=0; j<npts-1; j++)                 /* plot fit data */
					{
						#if(0) // Old code
						uline(*(x+j),*(y+j),*(x+j+1),*(y+j+1),1);
						#endif

						#if(1) // New code with clipping
						uline(max(*(x+j), x_min), max(*(y+j), y_min),
							min(*(x+j+1), x_max), min(*(y+j+1), y_max), 1);
						#endif

					}                                             /* end for */
					if(!line_flag)               /* can list only 1 line fit */
					{
						sprintf(label_str,
						"1st Curve Fit: x(t)=a+bt+ct[2], "
						"a=%-10.7e, b=%-10.7e, c=%-10.7e", a, b, c);
						fntchg('\313');                       /* simplex.fnt */
						color(annotation_color);         /* annotation color */                                                /* black */
						tmargin(6.655);
						ctline(label_str,0.075);
						line_flag=TRUE;
						color(line_color);                     /* line color */
					}
					break;
				}
				case PLINE_AVG:                    /* horizontal line at avg */
				{
						p_data[1]=sPLine[i].fXStart;
						p_data[2]=sPLine[i].fXEnd;

						CalcMean(p_data, &avg, PHASE_DATA);  /* calc average */

						#if(0) // Old code
						uline(p->x_mult*(p_data[1]+p->x_offset),
							p->y_mult*(avg+p->y_offset),
							p->x_mult*(p_data[2]+p->x_offset),
							p->y_mult*(avg+p->y_offset),3);
						#endif

						#if(1) // New code with clipping
						uline(max(p->x_mult*(p_data[1]+p->x_offset), x_min),
							max(p->y_mult*(avg+p->y_offset), y_min),
							min(p->x_mult*(p_data[2]+p->x_offset), x_max),
							min(p->y_mult*(avg+p->y_offset), y_max), 3);
						#endif

						break;
				}
				case PLINE_MARK:                     /* vertical marker line */
				{
						p_data[1]=sPLine[i].fXStart;
						p_data[2]=sPLine[i].fXStart;

						#if(0) // Old code
						uline(p->x_mult*(p_data[1]+p->x_offset),
							p->y_mult*(p->y_min+p->y_offset),
							p->x_mult*(p_data[2]+p->x_offset),
							p->y_mult*(p->y_max+p->y_offset),3);
						#endif

						#if(1) // New code with clipping
						uline(max(p->x_mult*(p_data[1]+p->x_offset), x_min),
							max(p->y_mult*(p->y_min+p->y_offset), y_min),
							min(p->x_mult*(p_data[2]+p->x_offset), x_max),
							min(p->y_mult*(p->y_max+p->y_offset), y_max), 3);
						#endif

						break;
				}
			}                                                  /* end switch */
		}                                                         /* end for */
		p_data[1]=save_start;                      /* restore starting point */
		p_data[2]=save_end;                          /* restore ending point */
	}                                                              /* end if */

	fntchg('\313');                                           /* simplex.fnt */
	color(annotation_color);                             /* annotation color */                                                    /* black */

	// Write date
	if(p->do_date)
	{
		Dateline(1);                         /* put date & file info on plot */
	}                                                              /* end if */

	// Display message box
	// Locations are x, y in pixels, origin at bottom left, 8192 x 6266 space
	// Find half length of message box in pixels
	lgth=(H_MARGIN+strwidtq(p->message_str, CHAR_HGTH));

	// Write message
	if(p->message_str[0]!=NULL)         /* don't display a blank message box */
	{
		switch(p->position)
		{
			case MSG_TOP_LEFT:            /* message box at top left of plot */
			{
				MsgBox(300,4400,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_MID_LEFT:         /* message box at middle left of plot */
			{
				MsgBox(300,2270,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_BOT_LEFT:         /* message box at bottom left of plot */
			{
				MsgBox(300,200,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_TOP_CENTER:        /* message box at top center of plot */
			{
				MsgBox(3470-lgth,4400, p->message_str);
				break;
			}                                                    /* end case */
			case MSG_MID_CENTER:     /* message box at middle center of plot */
			{
				MsgBox(3470-lgth,2270,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_BOT_CENTER:     /* message box at bottom center of plot */
			{
				MsgBox(3470-lgth,200,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_TOP_RIGHT:          /* message box at top right of plot */
			{
				MsgBox(6400-2*lgth,4400,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_MID_RIGHT:       /* message box at middle right of plot */
			{
				MsgBox(6400-2*lgth,2270,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_BOT_RIGHT:       /* message box at bottom right of plot */
			{
				MsgBox(6400-2*lgth,200,p->message_str);
				break;
			}                                                    /* end case */
		}                                                      /* end switch */
	}

	#if(1) // New code 10/27/96
	// Write label
	if(bPlotLabel)
	{
		width=intopix(strwidth(szPlotLabel, 0.05));
		putlabel(szPlotLabel, 8192-2*(width+20), 20, 0.05, 0, PIXEL_UNITS);
	}
	#endif

	// Activate mouse cursor
	if(p->do_mouse)
	{
		// Allocate memory for x and y point arrays
		fx=(float*)gpcalloc(64, sizeof(float));
		fy=(float*)gpcalloc(64, sizeof(float));

		curson(0, 0, &fx, &fy, &find);                /* turn on mouse cursor*/

		// Free memory - this assumes that point info isn't needed
		gpcfree((void**)&fx);
		gpcfree((void**)&fy);
	}                                                              /* end if */

	rightax(0);
	endplot();
									/* finish plot and wait for instructions */
												/* to exit or draw next plot */
										/* resets file pointers and defaults */

	// Free plot arrays - i assignments are to avoid compiler warnings
	i=GlobalFreePtr(x);
	i=GlobalFreePtr(y);
	i=i;

	stopplot();            /* close files, return to text mode, exit program */
}

/*****************************************************************************/
/*                                                                           */
/*								MsgBox()                                     */
/*                                                                           */
/*		Function to write message box on plot                                */
/*      Parameters:                                                          */
/*			x = lower left box x location in pixels                          */
/*			y = lower left box y location in pixels                          */
/*      	message_str = message string to be written on plot               */
/*		Screen has 4096 x 3120 pixels                                        */
/*      Bottom left corner is at (x,y) = (0,0)                               */
/*      Top right corner is at (x,y) = (4095,3119)                           */
/*		Character height is 64 pixels                                        */
/*		Vertical margin is 30 pixels                                         */
/*		Horizontal margin is 40 pixels                                       */
/*		Panel color 447 (white) prints OK                                    */
/*		Returns nothing (void)                                               */
/*                                                                           */
/*		Revision record:                                                     */
/*			06/30/96	Adapted from message_box() of Stable/DOS             */
/*			07/08/96	Rewritten for new panel filling functions            */
/*			10/26/96	Debugged - added pixtouser() usage - now seems AOK   */
/*                                                                           */
/*****************************************************************************/

void FAR PASCAL _export MsgBox( int x, int y, char *message_str)
{
	// Local variables
	int lgth;                                     /* length of box in pixels */
	int hgth=CHAR_HGTH+2*V_MARGIN;        /* height of message box in pixels */
	int dummy;                                             /* dummy variable */
	int i;                                                          /* index */
	float *xpan;                                /* pointer to x vertex array */
	float *ypan;                                /* pointer to y vertex array */

	// Start of code
	// Allocate memory for x and y vertex arrays which are in user plot units
	if((xpan=(float*)GlobalAllocPtr(GHND, (NUM_SIDES+1)*sizeof(float)))==NULL)
	{
		AllocErrorMessage();
	}

	if((ypan=(float*)GlobalAllocPtr(GHND, (NUM_SIDES+1)*sizeof(float)))==NULL)
	{
		AllocErrorMessage();
	}

	// Shift pixel location wrt physical origin
	x+=xphysorq;
	y+=yphysorq;

	// Find length of message box in pixels
	lgth=2*(H_MARGIN+strwidtq(message_str, CHAR_HGTH));
	// Note: Last arg now pixels, not mult factor

	// Load box vertices into array with user units
	// pixtouser() converts pixel units to user units
	// Vertices are as follows: [3  ]    [2  ]
	//                          [0,4]    [1  ]
	pixtouser(x, y, xpan, ypan);
	pixtouser(x+lgth, y, xpan+1, ypan+1);
	pixtouser(x+lgth, y+hgth, xpan+2, ypan+2);
	xpan[3]=xpan[0];
	ypan[3]=ypan[2];
	xpan[4]=xpan[0];             /* include these points to complete the box */
	ypan[4]=ypan[0];

	// Define panel area as bright white panel fill w/o border
	// Panel uses user units
	panel(xpan, ypan, NUM_SIDES+1, PAN_COLOR, FALSE);

	// Process panel
	panproc();

	// Draw border - do it this way so it will print OK
	// uline() uses user units
	for(i=0; i<4; i++)
	{
		uline(xpan[i], ypan[i], xpan[i+1], ypan[i+1], 3);
	}

	// Write message (xphysorq & yphysorq already added to x & y)
	// Notes: 3rd fntpltq() arg now pixels, not mult factor
	// 		  fntpltq() coord are wrt paper origin not plot scale origin
	//        xphysorq & yphysorq shift x & y coords from plot to paper origin
	fntpltq(x+H_MARGIN, y+V_MARGIN,message_str, CHAR_HGTH, 0);

	// Free x & y vertex arrays
	// dummy is to avoid compiler warning
	dummy=GlobalFreePtr(xpan);
	dummy=GlobalFreePtr(ypan);
	dummy=dummy;
}

/*****************************************************************************/
/*                                                                           */
/*								TableBox()                                   */
/*                                                                           */
/*		Function to write sigma-tau table box on plot                        */
/*      Parameters:                                                          */
/*			type= type of sigma                                              */
/*				  0 = normal or overlapping sigma                            */
/*				  1 = modified or time sigma                                 */
/*				  2 = dual, overlapping and modified sigma                   */
/*			points = number of data points                                   */
/*			x      = x location in pixels                                    */
/*					 (normal value = 2730, dual value = TBD)                 */
/*			y      = y location in pixels                                    */
/*					 (normal value = 2673, dual value = TBD)                 */
/*			The sigma and tau values are in the global arrays                */
/*          sNormSigData.sigma[i] and sNormSigData.tau[i] or                 */
/*          sModSigData.sigma[i] and sModSigData.tau[i],                     */
/*		Screen has 8192 x 6266 pixels                                        */
/*      Bottom left corner is at (x,y) = (0,0)                               */
/*      Top right corner is at (x,y) = (8191,6265)                           */
/*		Nominal character height multiplier (chm) is 1.5                     */
/*		Returns nothing (void)                                               */
/*                                                                           */
/*		Revision record:                                                     */
/*			06/30/96	Adapted from table_box() of Stable/DOS               */
/*			10/26/96	Rewritten for new panel filling functions            */
/*						Pixel #s scaled x2 for 8192 vs 4096 FS GraphiC 7.1   */
/*                                                                           */
/*****************************************************************************/

void FAR PASCAL _export TableBox(int type, int points, int x, int y)
{
	// Local macros
	#define CHR_HGT	2*42    // Base character height

	// Local variables
	// Chars
	char table_t_str[10];                                      /* tau string */
	char table_s1_str[20];                               /* 1st sigma string */
	char table_s2_str[20];                               /* 2nd sigma string */
	// Ints
	int i;                                                          /* index */
	int height;                                       /* height of table box */
	int dummy;                                             /* dummy variable */
	int dx;													     /* x offset */
	// BOOLs
	BOOL dual=FALSE;                                       /* dual plot flag */
	BOOL bIntFormat=FALSE;	      /* flag to use integer tau format in table */
	// Floats
	float chm;                                /* character height multiplier */
	float *xpan;                                /* pointer to x vertex array */
	float *ypan;                                /* pointer to y vertex array */

	// Start of code
	// Allocate memory for x and y vertex arrays which are in user plot units
	if((xpan=(float*)GlobalAllocPtr(GHND, (NUM_SIDES+1)*sizeof(float)))==NULL)
	{
		AllocErrorMessage();
	}

	if((ypan=(float*)GlobalAllocPtr(GHND, (NUM_SIDES+1)*sizeof(float)))==NULL)
	{
		AllocErrorMessage();
	}

	// Scale y for GraphiC Ver 7.1 # pixels change
	// Note: x is not used directly, so no need to scale it
	y*=2;

	/* calculate chm for # data points */
	chm=min(1.5, 1.5-(points-10)*0.045);

	/* adjust x for chm scaling */
	if(type==0)                                     /* normal or overlapping */
	{
		x=2*3830-2*1100*(chm/1.5);
	}
	else if( (type==1) && (x>1000) )                  /* modified - not time */
	{
		x=2*3830-2*1100*(chm/1.5);
	}
	else if(type==2)                                                 /* dual */
	{
		x=2*3830-2*1700*(chm/1.5);
		dual=TRUE;
		type=0;
	}

	#if(0) // Not used
	// Shift pixel location wrt physical origin
	x+=xphysorq;
	y+=yphysorq;
	#endif

	height=CHR_HGT*chm*1.6*(points+1.5);         /* calc height of table box */

	// Determine table format
	if( (nSigmaPlotType==MODIFIED_RUN) ||
		(nSigmaPlotType==TIME_RUN) )
	{
		if(sModSigData.tau[0]>=1.0 && sModSigData.tau[points-1]<=9999)
		{
			bIntFormat=TRUE;
		}
		else
		{
			bIntFormat=FALSE;
		}
	}
	else // ADEV
	{
		if(sNormSigData.tau[0]>=1.0 && sNormSigData.tau[points-1]<=9999)
		{
			bIntFormat=TRUE;
		}
		else
		{
			bIntFormat=FALSE;
		}
	}

	if(bIntFormat)
	{
		// 1 <= tau <= 9999, so use integer format in table
		dx=2*230;                                            /* add x offset */
	}
	else
	{
		// Use floating format for tau
		dx=0;                                                 /* no x offset */
	}

	// Load box vertices into array with user units
	// Vertices are as follows: [3  ]    [2  ]
	//                          [0,4]    [1  ]
	pixtouser(x+dx, y, xpan, ypan);
	pixtouser(x+2*40+(2*1060+dual*2*600)*chm/1.5, y, xpan+1, ypan+1);
	pixtouser(x+2*40+(2*1060+dual*2*600)*chm/1.5, y-height, xpan+2, ypan+2);
	xpan[3]=xpan[0];
	ypan[3]=ypan[2];
	xpan[4]=xpan[0];           /* include these points to complete the curve */
	ypan[4]=ypan[0];

	// Define panel area as bright white panel fill w/o border
	panel(xpan, ypan, NUM_SIDES+1, PAN_COLOR, FALSE);

	panproc();                                    /* do the panel processing */

	for(i=0; i<4; i++)           /* draw border this way so it will print ok */
	{
		uline(xpan[i], ypan[i], xpan[i+1], ypan[i+1], 3);
	}

	// Reload vertices with pixel units
	xpan[0]=x+dx;
	ypan[0]=y;
	xpan[1]=x+2*40+(2*1060+dual*2*600)*chm/1.5;
	ypan[1]=ypan[0];
	xpan[2]=xpan[1];
	ypan[2]=ypan[1]-height;
	xpan[3]=xpan[0];
	ypan[3]=ypan[2];
	xpan[4]=xpan[0];
	ypan[4]=ypan[0];

	// Write the sigma-tau values in a table on the plot.  The sigma & tau
	// values are written separately to line up the proportional font.

	for(i=0; i<points; i++)
	{
		if( (nSigmaPlotType==MODIFIED_RUN) ||
			(nSigmaPlotType==TIME_RUN) )
		{
			sprintf(table_s1_str,"%4.2e",sModSigData.sigma[i]);
			sprintf(table_s2_str,"%4.2e",sNormSigData.sigma[i]);
		}
		else // ADEV
		{
			sprintf(table_s1_str,"%4.2e",sNormSigData.sigma[i]);
			sprintf(table_s2_str,"%4.2e",sModSigData.sigma[i]);
		}

		if(bIntFormat)                                       /* int tau format */
		{
			if(dual)
			{
				fntpltq(xpan[0]+2*30, ypan[0]-2*90*chm/1.5,
					" Tau   Sigma   Mod Sigma", chm*CHR_HGT, 0);
			}
			else
			{
				fntpltq(xpan[0]+2*30, ypan[0]-2*90*chm/1.5,
					" Tau   Sigma", chm*CHR_HGT, 0);
			}

			if( (nSigmaPlotType==MODIFIED_RUN) ||
				(nSigmaPlotType==TIME_RUN) )
			{
				sprintf(table_t_str,"%4.0f",sModSigData.tau[i]);
			}
			else // ADEV
			{
				sprintf(table_t_str,"%4.0f",sNormSigData.tau[i]);
			}

			fntpltq(xpan[0]+2*40,
				ypan[0]-2*50*chm/1.5-CHR_HGT*chm-CHR_HGT*chm*1.6*(i+1),
				table_t_str, chm*CHR_HGT, 0);
			fntpltq(xpan[0]+2*40+2*300*chm/1.5,
				ypan[0]-2*50*chm/1.5-CHR_HGT*chm-CHR_HGT*chm*1.6*(i+1),
				table_s1_str, chm*CHR_HGT, 0);

			if(dual)
			{
				fntpltq(xpan[0]+2*40+2*840*chm/1.5,
					ypan[0]-2*50*chm/1.5-CHR_HGT*chm-CHR_HGT*chm*1.6*(i+1),
					table_s2_str, chm*CHR_HGT, 0);
			}
		}                                                          /* end if */
		else                                             /* float tau format */
		{
			if(dual)
			{
				fntpltq(xpan[0]+2*30, ypan[0]-2*90*chm/1.5,
					"   Tau      Sigma   Mod Sigma", chm*CHR_HGT,0);
			}
			else
			{
				fntpltq(xpan[0]+2*30,	ypan[0]-2*90*chm/1.5,
					"   Tau      Sigma", chm*CHR_HGT,0);
			}

			if( (nSigmaPlotType==MODIFIED_RUN) ||
				(nSigmaPlotType==TIME_RUN) )
			{
				sprintf(table_t_str,"%4.2e",sModSigData.tau[i]);
			}
			else // ADEV
			{
				sprintf(table_t_str,"%4.2e",sNormSigData.tau[i]);
			}

			fntpltq(xpan[0]+2*40,
				ypan[0]-2*50*chm/1.5-CHR_HGT*chm-CHR_HGT*chm*1.6*(i+1),
				table_t_str, chm*CHR_HGT, 0);
			fntpltq(xpan[0]+2*40+2*540*chm/1.5,
				ypan[0]-2*50*chm/1.5-CHR_HGT*chm-CHR_HGT*chm*1.6*(i+1),
				table_s1_str, chm*CHR_HGT, 0);

			if(dual)
			{
				fntpltq(xpan[0]+2*40+2*1080*chm/1.5,
					ypan[0]-2*50*chm/1.5-CHR_HGT*chm-CHR_HGT*chm*1.6*(i+1),
					table_s2_str, chm*CHR_HGT, 0);
			}
		}                                                        /* end else */
	}                                                             /* end for */

	// Free x & y vertex arrays
	// dummy is to avoid compiler warning
	dummy=GlobalFreePtr(xpan);
	dummy=GlobalFreePtr(ypan);
	dummy=dummy;
}

/*****************************************************************************/
/*                                                                           */
/*								HistoPlot()                                  */
/*                                                                           */
/*		Function to plot histogram data                                      */
/*      Parameters:                                                          */
/*      	f_data[] = array of freq bin data                                */
/*          h_data[] = array of histogram count data                         */
/*			int bins = # bins                                                */
/*		p plotdata structure elements:                                       */
/*      	p.title = plot title                                             */
/*      	p.subtitle = plot subtitle                                       */
/*			p.do_box  = draw box flag                                        */
/*			p.do_date = write date flag                                      */
/*			p.plot_filename = .tkf filename for plot                         */
/*			p.message_str = text of message to be put in box on plot         */
/*      Returns nothing (void)                                               */
/*                                                                           */
/*      Revision record:                                                     */
/*      	06/30/96	Adapted from plot_histo() of Stable/DOS              */
/*			11/18/96	Revised plot sizes, etc.                             */
/*			12/29/96	Up and running                                       */
/*                                                                           */
/*****************************************************************************/

void FAR PASCAL _export HistoPlot(float f_data[], float h_data[],
   int bins,  struct plotdata *p)
{
	int i;                                                  /* general index */
	int background_color;
	int grid_color;
	int label_color;
	int bar_color;
	int separator_color;
	int line_color;
	int annotation_color;
	int lgth;                                    /* message length in pixels */
	float x[5];                                         /* x vertices of bar */
	float y[5];                                         /* y vertices of bar */
	float temp;                     /* temporary storage for 1st bin's value */
	float strwidth(char *string, float height);    /* GraphiC float function */
	float width;                             /* title or subtitle half width */
	float range;                                            /* x-scale range */
	float sd;                                          /* standard deviation */
	float m;                                                         /* mean */
	float x_start;
	float x_end;
	float p_start;                    /* normal probability for (x-m) and sd */
	float p_end;
	char label_str[100];

	// Cursor variables
	float *fx;                  // Pointer to x-axis point array
	float *fy;                  // Pointer to y-axis point array
	int find;					// Point array dimensions

	// assign colors
	background_color=BRT_WHITE;
	grid_color=BLACK;
	label_color=BLUE;
	bar_color=RED;
	separator_color=BLUE;
	line_color=GREEN;
	annotation_color=BLACK;

/*****************************************************************************/
/* This is the begining of the actual GraphiC plotting code                  */
/* Sequence of basic calls is per page I-34 of manual:                       */
/*	   bgnplot()            	Initializations                              */
/*	   startplot()          	Starts graphics                              */
/*	   font()               	Loads fonts                                  */
/*	   page()               	Sets overall picture size                    */
/*	   heading()				Draws title                                  */
/*	   xname(), yname()		 	Draws axis labels                            */
/*	   area2d()					Defines axis lengths                         */
/*	                                                                         */
/*****************************************************************************/

	bgnplot(1,'g',p->plot_filename);                      /* initialize plot */
	startplot(background_color);        /* start plot & set background color */
	font(4,"news.fnt",'\310',"swiss.fnt",'\311',"newsgrm.fnt",'\312',
		 "simplex.fnt",'\313');                                /* load fonts */
	page(FULLPAGE);                  /* Sets the maximum allowable page size */
	heading("");    /* Blank heading allows space for title & subtitle later */
	fillfont(1);                                         /* Use filled fonts */

	// Kill the beep
	GPCSetBeepDefaults(0, 0);

	color(label_color);                                       /* label color */
	fntchg('\310');                                              /* news.fnt */
	xname(p->x_label);                                       /* x-axis label */
	yname(p->left_y_label);                                  /* y-axis label */
	area2d(7.8, 5.3);                                    /* plot dimenisions */

	upright(1);                                  /* horizontal y-axis labels */
	color(grid_color);                                         /* grid color */
	grid(2);                                          /* fine dot grid lines */
	fgrid(9, 0);                             /* dotted lines on subdivisions */
	frame(1,1);                          /* add frame with ticks around plot */

	if(p->do_box)
	{
		box();                              /* draw a box around the picture */
	}                                                              /* end if */

	temp=h_data[0];                                  /* save 1st bin's value */
	h_data[0]=0;             /* set it = 0 to force y-scale to start at zero */
	scales(5,5,f_data,h_data,bins);              /* self scaling linear axes */
	h_data[0]=temp;                                         /* restore value */

	fntchg('\311');                                             /* swiss.fnt */
	width=strwidth(p->title,0.3);                 /* get half-width of title */
	prtfnt((4.75-width),6.40,p->title,0.3,0);                  /* draw title */
	width=strwidth(p->subtitle,0.15);          /* get half-width of subtitle */
	prtfnt((4.75-width),6.175,p->subtitle,0.15,0);           /* draw subtitle */

	for(i=0; i<bins; i++)                                      /* draw bars */
	{                                                           /*   1   2   */
		if(h_data[i])                                           /*           */
		{                                                       /*  0,4  3   */
			x[0]=f_data[i];
			x[1]=x[0];
			x[2]=f_data[i+1];
			x[3]=x[2];
			x[4]=x[0];
			y[0]=0;
			y[1]=h_data[i];
			y[2]=y[1];
			y[3]=0;
			y[4]=y[0];
			color(separator_color);                   /* bar separator color */
			panel(x,y,4,-bar_color,1);                      /* histogram bar */
			panproc();
		}                                                          /* end if */
	}                                                             /* end for */

	if(p->do_line)                         /* plot normal distribution curve */
	{
		color(line_color);                                     /* line color */
		x_start=p->x_min;
		range=p->x_max - p->x_min;
		m=p->y_offset;                                               /* mean */
		sd=p->y_mult;                                             /* std dev */
		p_start=p->y_max*exp(((m-x_start)*(x_start-m))/(2*sd*sd));
		for(i=1; i<=100; i++)
		{
			x_end=p->x_min+i*(range/100);
			p_end=p->y_max*exp(((m-x_end)*(x_end-m))/(2*sd*sd));
			uline(x_start, p_start, x_end, p_end, 3);
			x_start=x_end;
			p_start=p_end;
		}                                                         /* end for */
		sprintf(label_str,
			"Normal Curve Fit: "
			"Mean=%-10.7e, Std Dev=%-10.7e, Peak=%-3.0f",
			m, sd, p->y_max);
		fntchg('\313');                                       /* simplex.fnt */
		color(annotation_color);                     /* fit annotation color */
		tmargin(6.655);
		ctline(label_str,0.075);
}                                                      /* end if(p->do_line) */
	color(annotation_color);                 /* date & file annotation color */                                                  /* black */
	if(p->do_date)
	{
		Dateline(1);                         /* put date & file info on plot */
	}                                                              /* end if */

	// Display message box
	// Locations are x, y in pixels, origin at bottom left, 8192 x 6266 space
	// Find half length of message box in pixels
	lgth=(H_MARGIN+strwidtq(p->message_str, CHAR_HGTH));

	// Write message
	if(p->message_str[0]!=NULL)         /* don't display a blank message box */
	{
		switch(p->position)
		{
			case MSG_TOP_LEFT:            /* message box at top left of plot */
			{
				MsgBox(300, 4400, p->message_str);
				break;
			}                                                    /* end case */
			case MSG_MID_LEFT:         /* message box at middle left of plot */
			{
				MsgBox(300, 2270, p->message_str);
				break;
			}                                                    /* end case */
			case MSG_BOT_LEFT:         /* message box at bottom left of plot */
			{
				MsgBox(300, 200, p->message_str);
				break;
			}                                                    /* end case */
			case MSG_TOP_CENTER:        /* message box at top center of plot */
			{
				MsgBox(3470-lgth, 4400, p->message_str);
				break;
			}                                                    /* end case */
			case MSG_MID_CENTER:     /* message box at middle center of plot */
			{
				MsgBox(3470-lgth, 2270, p->message_str);
				break;
			}                                                    /* end case */
			case MSG_BOT_CENTER:     /* message box at bottom center of plot */
			{
				MsgBox(3470-lgth, 200, p->message_str);
				break;
			}                                                    /* end case */
			case MSG_TOP_RIGHT:          /* message box at top right of plot */
			{
				MsgBox(6400-2*lgth, 4400, p->message_str);
				break;
			}                                                    /* end case */
			case MSG_MID_RIGHT:       /* message box at middle right of plot */
			{
				MsgBox(6400-2*lgth, 2270, p->message_str);
				break;
			}                                                    /* end case */
			case MSG_BOT_RIGHT:       /* message box at bottom right of plot */
			{
				MsgBox(6400-2*lgth, 200, p->message_str);
				break;
			}                                                    /* end case */
		}                                                      /* end switch */
	}

	// Write label
	if(bPlotLabel)
	{
		width=intopix(strwidth(szPlotLabel, 0.05));
		putlabel(szPlotLabel, 8192-2*(width+20), 20, 0.05, 0, PIXEL_UNITS);
	}

	// Activate mouse cursor
	if(p->do_mouse)
	{
		// Allocate memory for x and y point arrays
		fx=(float*)gpcalloc(64, sizeof(float));
		fy=(float*)gpcalloc(64, sizeof(float));

		curson(0, 0, &fx, &fy, &find);                /* turn on mouse cursor*/

		// Free memory - this assumes that point info isn't needed
		gpcfree((void**)&fx);
		gpcfree((void**)&fy);
	}                                                              /* end if */

	endplot();                                             /* terminate plot */
	stopplot();                                      /* close files and quit */
}                                                          /* end plot_histo */

/*****************************************************************************/
/*                                                                           */
/*							      Dateline()                                 */
/*                                                                           */
/*		Function to display date & file information at top of plot           */
/*		Parameters: BOOL ct_flag = flag to control Points & Tau display      */
/*		Returns nothing (void)                                               */
/*                                                                           */
/*		Revision record:                                                     */
/*			07/13/96	Adapted from dateline() of Stable/DOS                */
/*			01/30/97	Added provisions for displaying stability file title */
/*                                                                           */
/*****************************************************************************/

void FAR PASCAL _export Dateline(BOOL ct_flag)
{
	#include <time.h>

	// Local macros
	#define LABEL_STR_SIZE 100

	// Local variables
	char label_str[LABEL_STR_SIZE];
	char date_str[9];
	char time_str[9];
	F_TYPE fTau;

	// Select font
	fntchg('\313');                                         /* simplex.fnt */

	// Write date and time at upper left
	_strdate(date_str);
	_strtime(time_str);
	strcpy(label_str, "  Date: ");
	strcat(label_str, date_str);
	strcat(label_str, "  Time: ");
	strcat(label_str, time_str);
	tmargin(0);
	ltline(label_str, 0.075) ;

	// Write data point and tau info at upper center
	if(ct_flag)
	{
		if(bDataType==FREQ_DATA)
		{
			// Get tau
			GetWindowText(hwndFreqTau, (LPSTR)label_str,
				LABEL_STR_SIZE-1);
			fTau=(F_TYPE)atof(label_str);

			// Compose string
			sprintf(label_str, "Data Points %-1.0f thru %-1.0f of %-1.0f\
			Tau=%-10.7e", fFreqData[1], fFreqData[2], fFreqData[0],	fTau);
		}
		else
		{
			// Get tau
			GetWindowText(hwndPhaseTau, (LPSTR)label_str,
				LABEL_STR_SIZE-1);
				fTau=(F_TYPE)atof(label_str);

			// Compose string
			sprintf(label_str,"Data Points %-1.0f thru %-1.0f of %-1.0f\
			Tau=%-10.7e", fPhaseData[1], fPhaseData[2], fPhaseData[0], fTau);
		}
		tmargin(0);
		ctline(label_str, 0.075);
	}

	// Write filename at upper right
	strcpy(label_str, "File: ");
	#if(1) // New code 01/30/97
	if(nPlotType==READ_PLOT)
	{
		strcat(label_str, szStabFileTitle);
	}
	else
	#endif
	{
		strcat(label_str, szFileTitle);
	}
	strcat(label_str, "   ");
	tmargin(0);
	rtline(label_str, 0.075) ;
}                                                        /* end Dateline() */

/*****************************************************************************/
/*                                                                           */
/*		Plot dialog box procedure                                            */
/*                                                                           */
/*		Revision record:                                                     */
/*			07/13/96	Written                                              */
/*			11/16/96	Added trap for plot with too few analysis points     */
/*                                                                           */
/*****************************************************************************/

BOOL CALLBACK PlotDlgProc(HWND hDlg, UINT message, WPARAM wParam,
	LPARAM lParam)
{
	// Local variables
	HWND hChildWnd=NULL;
	HANDLE hChildAccel=NULL;

	// To avoid complier warning
	lParam=lParam;

	switch(message)
	{
		case WM_INITDIALOG:
		{
			// Assign global dialog box handle
			hDialog=hDlg;
			return TRUE;
		}

		case WM_COMMAND:
		{
			switch(wParam)
			{
				// Respond to help button
				case IDHELP:
				{
					WinHelp(hDlg, szHelpFile, HELP_CONTEXT,
						PLOT_FUNCTION);
				}
				break;

				// Respond to plot button
				case IDC_PLOT_PLOT:
				{
					#if(1) // New code 11/16/96
					// Check for enough analysis points - need at least 4
					if(fData[2]-fData[1]<3)
					{
						TooFewMessage();
						break;
					}
					#endif

					// Disable buttons
					EnableWindow(GetDlgItem(hDlg, IDCANCEL), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_PLOT), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_OPTIONS), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_LINES), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_NOTES), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDHELP), FALSE);

					// Create child window for plot
					GPCWINCreate(hChildWnd, hInst, hDlg, SW_SHOWNORMAL,
						&hChildAccel, NULL);
				}
				break;

				// Respond to options button
				case IDC_PLOT_OPTIONS:
				{
					// Disable buttons
					EnableWindow(GetDlgItem(hDlg, IDCANCEL), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_PLOT), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_OPTIONS), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_LINES), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_NOTES), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDHELP), FALSE);

					// Open options box
					if(nPlotType==FREQ_PLOT)
					{
						// Open freq plot options dialog box
						InvokeDialogBox(hInst, "DataPlotBox", hwndMain,
							(DLGPROC)FreqPlotDlgProc);
					}
					else // Phase plot
					{
						// Open phase plot options dialog box
						InvokeDialogBox(hInst, "DataPlotBox", hwndMain,
							(DLGPROC)PhasePlotDlgProc);
					}

					// Enable buttons
					EnableWindow(GetDlgItem(hDlg, IDCANCEL), TRUE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_PLOT), TRUE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_OPTIONS), TRUE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_LINES), TRUE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_NOTES), TRUE);
					EnableWindow(GetDlgItem(hDlg, IDHELP), TRUE);

					// Give plot control panel the focus
					SetFocus(hDlg);
				}
				break;

				// Respond to lines button
				case IDC_PLOT_LINES:
				{
					// Disable buttons
					EnableWindow(GetDlgItem(hDlg, IDCANCEL), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_PLOT), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_OPTIONS), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_LINES), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_NOTES), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDHELP), FALSE);

					// Open lines box
					if(nPlotType==FREQ_PLOT)
					{
						// Open freq line options dialog box
						InvokeDialogBox(hInst, "PlotLineBox", hwndMain,
							(DLGPROC)FreqLineDlgProc);
					}
					else // Phase plot
					{
						// Open phase plot options dialog box
						InvokeDialogBox(hInst, "PlotLineBox", hwndMain,
							(DLGPROC)PhaseLineDlgProc);
					}

					// Enable buttons
					EnableWindow(GetDlgItem(hDlg, IDCANCEL), TRUE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_PLOT), TRUE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_OPTIONS), TRUE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_LINES), TRUE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_NOTES), TRUE);
					EnableWindow(GetDlgItem(hDlg, IDHELP), TRUE);

					// Give plot control panel the focus
					SetFocus(hDlg);
				}
				break;

				// Respond to options button
				case IDC_PLOT_NOTES:
				{
					// Disable buttons
					EnableWindow(GetDlgItem(hDlg, IDCANCEL), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_PLOT), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_OPTIONS), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_LINES), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_NOTES), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDHELP), FALSE);

					// Open notes box
					if(nPlotType==FREQ_PLOT)
					{
						// Open frequency annotations dialog box
						InvokeDialogBox(hInst, "PlotNoteBox", hwndMain,
							(DLGPROC)FreqNoteDlgProc);
					}
					else // Phase plot
					{
						// Open phase plot options dialog box
						InvokeDialogBox(hInst, "PlotNoteBox", hwndMain,
							(DLGPROC)PhaseNoteDlgProc);
					}

					// Enable buttons
					EnableWindow(GetDlgItem(hDlg, IDCANCEL), TRUE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_PLOT), TRUE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_OPTIONS), TRUE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_LINES), TRUE);
					EnableWindow(GetDlgItem(hDlg, IDC_PLOT_NOTES), TRUE);
					EnableWindow(GetDlgItem(hDlg, IDHELP), TRUE);

					// Give plot control panel the focus
					SetFocus(hDlg);
				}
				break;

				// Respond to cancel button
				case IDCANCEL:
				{
					// Close dialog box
					EndDialog (hDlg, 0);
				}
				return TRUE;
			}
		}
		break;
	 }
	 return FALSE;
}

/*****************************************************************************/
/*                                                                           */
/*						        TVARLabel()                                  */
/*                                                                           */
/*		TVAR line labeling function                                          */
/*		Parameters:                                                          */
/*			int val     = exponent of tvar line, seconds                     */
/*			char *label = label string for tvar line                         */
/*							                                                 */
/*      Returns nothing (void)                                               */
/*                                                                           */
/*		Revision record:                                                     */
/*			10/26/96	Created from Stable/DOS function of same name        */
/*                                                                           */
/*****************************************************************************/
void FAR PASCAL _export TVARLabel(int val, char *label)
{
	switch(val)
	{
		case  6:	strcpy(label,"1Ms");
					break;
		case  5:	strcpy(label,"100ks");
					break;
		case  4:	strcpy(label,"10ks");
					break;
		case  3:	strcpy(label,"1ks");
					break;
		case  2:	strcpy(label,"100s");
					break;
		case  1:	strcpy(label,"10s");
					break;
		case  0:	strcpy(label,"1s");
					break;
		case -1:	strcpy(label,"100ms");
					break;
		case -2:	strcpy(label,"10ms");
					break;
		case -3:	strcpy(label,"1ms");
					break;
		case -4:	strcpy(label,"100us");
					break;
		case -5:	strcpy(label,"10us");
					break;
		case -6:	strcpy(label,"1us");
					break;
		case -7:	strcpy(label,"100ns");
					break;
		case -8:	strcpy(label,"10ns");
					break;
		case -9:	strcpy(label,"1ns");
					break;
		case -10:	strcpy(label,"100ps");
					break;
		case -11:	strcpy(label,"10ps");
					break;
		case -12:	strcpy(label,"1ps");
					break;
		case -13:	strcpy(label,"100fs");
					break;
		case -14:	strcpy(label,"10fs");
					break;
		case -15:	strcpy(label,"1fs");
					break;
		default:    sprintf(label,"%-4.0e",pow10(val));
					break;
	}                                                        /* end switch */
}                                                      /* end tvar_label() */

/***************************************************************************/

