/*****************************************************************************/
/*                                                                           */
/*								PhasePlot()                                  */
/*                                                                           */
/*		Function to plot phase data                                          */
/*      Parameters:                                                          */
/*			p_data[] = array of phase data; p_data[0] = # data points        */
/*			p_data[1] = plot start; p_data[2] = plot end                     */
/*      	zero treated as gap in data                                      */
/*		p plotdata struct elements:                                          */
/*          data_title = plot title                                          */
/*          data_subtitle = plot subtitle                                    */
/*			x_label = x-axis label                                           */
/*			y_label = y-axis label                                           */
/*			x_offset = x-axis scale offset                                   */
/*			x_mult = x-axis scale multiplier                                 */
/*			y_offset = y-axis scale offset                                   */
/*			y_mult = y-axis scale multiplier                                 */
/*          scale_min = y-axis scale minimun                                 */
/*          scale_max = y-axis scale maximum                                 */
/*          y_div = # of y-axis major divisions                              */
/*          y_tic = # of y-axis tics between major divisions                 */
/*			do_box = draw box flag                                           */
/*      	do_phase_line = draw regression line flag                        */
/*			do_date = write date flag                                        */
/*     		do_mouse = activate mouse/cursor flag                            */
/*			new_file = new basic phase data file flag                        */
/*			new_name = new exact phase data flag flag                        */
/*			position = position of message box                               */
/*          phase_plot_filename = .tkf filename for plot                     */
/*		Returns nothing (void)                                               */
/*                                                                           */
/*		Revision record:                                                     */
/*			06/30/96	Adapted from phase_plot() of Stable/DOS              */
/*			07/17/96	Adapted from FreqPlot() of Stable/Win                */
/*			11/12/96	Added clipping to line drawing code                  */
/*			11/18/96	Revised plot sizes, etc.                             */
/*			01/10/97	Quadratic drift made slope per data point interval   */
/*			03/15/97	Changed clipping to use ClipPlot()                   */
/*			12/12/98	Added ability to insert mini plot                    */
/*			03/02/00	Added display of average fit with lowest priority    */
/*			03/11/01	Added provisions for thicker lines                   */
/*			03/10/02	Made changes in plot scale code                      */
/*			03/21/02	Added use of x_start plotdata struct item            */
/*                                                                           */
/*****************************************************************************/

void WINAPI PhasePlot(F_TYPE p_data[], struct plotdata *p)
{
	// Local variables
	float *x,*y;
	float width;
	float x_max, x_min;                                    /* x scale limits */
	float x_start;                                  /* start value of x-axis */
	float y_max, y_min;                                    /* y scale limits */
	float xste;                                        /* x-axis tic spacing */
	float yste;                                        /* y-axis tic spacing */
	float save_start;
	float save_end;
	int i;
	int j;
	int k;
	int len;				                    	        /* string length */
	int npts;
	int lgth;                                    /* message length in pixels */
	int line_flag=FALSE;
	int x_div;						           /* # x-scale div for roundq() */
	char label_str[LABELSIZE+1];
	char title_str[BUFFER_SIZE+1];         /* error message box title string */
	char yformat[8];
	char xformat[8];
	char *szTemp;


	// Colors
	int background_color=BRT_WHITE;
	int grid_color=BLACK;
	int box_color=BLACK;
	int label_color=BLUE;
	int data_color=RED;
	int annotation_color=BLACK;
	int line_color=GREEN;

	// Fit parameters
	F_TYPE avg;
	F_TYPE slope;
	F_TYPE intercept;
	F_TYPE a, b, c, v;
	F_TYPE tau;

	// Cursor variables
	float *fx;                  // Pointer to x-axis point array
	float *fy;                  // Pointer to y-axis point array
	int find;					// Point array dimensions
	FILE *file;									             /* file pointer */
	char szInsertFile[FILE_NAME_SIZE+1];			  /* insert tkf filename */
	
	// Test for no data
	// Note: NULL plot filename test eliminated
	if((p_data[2]-p_data[1])<=0)
	{
		LoadString(hInst, IDS_MESSAGE_NOPHASEDATA, label_str, LABELSIZE);
		MessageBox(hwndMain, label_str,	"Stable/Win", MB_ICONEXCLAMATION);
		return;
	}                                                              /* end if */

	// Allocate memory for plot arrays
	if((x=(float*)malloc(((int)p_data[2]+2)*sizeof(float)))==NULL)
	{
		AllocErrorMessage();
		return;
	}
	if((y=(float*)malloc(((int)p_data[2]+2)*sizeof(float)))==NULL)
	{
		AllocErrorMessage();
		free(x);
		return;
	}

	CalcMean(p_data, &avg, FREQ_DATA);

	// This is the begining of the actual GraphiC plotting code
	// Sequence of basic calls is per page I-34 of manual:
	//	   bgnplot()            	Initializations
	//	   startplot()          	Starts graphics
	//	   font()               	Loads fonts
	//	   page()               	Sets overall picture size
	//	   heading()				Draws title
	//	   xname(), yname()		 	Draws axis labels
	//	   area2d()					Defines axis lengths
	//

	// Write log info
	WriteLog(37, p->plot_filename);

	bgnplot(1,'g',p->plot_filename);                      /* initialize plot */
	startplot(background_color);         /* start plot with background color */
	font(4,"news.fnt",'\310',"swiss.fnt",'\311',"newsgrm.fnt",'\312',
		 "simplex.fnt",'\313');                                /* load fonts */

	page(FULLPAGE);                  /* Sets the maximum allowable page size */
	heading("");    /* Blank heading allows space for title & subtitle later */
	fillfont(1);                                         /* Use filled fonts */

	// Kill the beep
	GPCSetBeepDefaults(0, 0);

	// Avoid unwanted clipping 
	UnclipPlot();
	
	// Axis labeling code
	fntchg('\310');                                              /* news.fnt */
	color(label_color);                                        /* all labels */
	yname(p->left_y_label);                                 /* labels y-axis */
	xname(p->x_label);                                      /* labels x-axis */

	area2d((float)7.8, (float)5.3);             /* Sets the area of the plot */

	// Draw box
	color(box_color);                                  /* set box line color */
	if(p->do_box)
	{
		box();                                    /* draw box around picture */
	}                                                              /* end if */

	// Draw grid
	// The grid() function draws the major grid lines (at tics)
	// The fgrid() function draws the fine grid lines (between tics)
	// The 1st fgrid() arg is: 0=no ticks, -1=tick marks
	// The 2nd fgrid() arg is # ticks=# intervals-1
	color(grid_color);                          /* set box & grid line color */
	grid(9);                         /* draws dotted grid through tick marks */
	fgrid(-1,p->y_tic-1);        /* fine grid tick marks between major ticks */

	frame(1,1);                          /* add frame with ticks around plot */
	upright(1);                        /* causes y-axis labels to be upright */
	fntchg('\310');                                              /* news.fnt */

	// X-scale
	// Set x-axis plot scale for roundq()
	x_max=p->x_max;
	x_min=p->x_min; // Was x_min-p->x_min; until 07/19/16 

	// Set the # x divisions for roundq()
	x_div=p->x_div;
	
	// Use roundq() to set xformat
	roundq(x_div, &x_min, &xste, &x_max, xformat);

	// Reset the scale limits to those actually wanted
	x_min=p->x_min;
	x_max=p->x_max;

	// Set the x-scale div size
	xste=((x_max-x_min)/p->x_div);

	// Set the x-axis starting value
	x_start=p->x_start;

	// Y-scale
	// Set the y-scale max & min
	y_max=p->y_mult*(p->y_max+p->y_offset);
	y_min=p->y_mult*(p->y_min+p->y_offset);

	// Set the y-scale tic separation
	yste=((y_max-y_min)/p->y_div);
	
	// Round tic separation to avoid the dreaded 0.000000
	sprintf(yformat, "%-1.3f", yste);
	yste=(float)atof(yformat);

	// Make sure yste is >0
	if(yste==0)
	{
		while(yste==0) // Adjust phase scale to avoid O/F
		{
			// Adjust phase scale multiplier
			p->y_mult*=10.0;
			y_max=p->y_mult*(p->y_max+p->y_offset);
			y_min=p->y_mult*(p->y_min+p->y_offset);

			// Set the y-scale tic separation
			yste=((y_max-y_min)/p->y_div);

			// Round tic separation
			sprintf(yformat, "%-1.3f", yste);
			yste=(float)atof(yformat);

			// Change the y-axis label
			yname("NG Phase Scale");
		}
	}

	// Adjust the y-scale format to avoid equal adjacent tic labels
	if(yste>=10.0)
	{
		strcpy(yformat, "%-1.0f");
	}
	else if(yste>=1.0)
	{
		strcpy(yformat, "%-1.1f");
	}
	else if(yste>=0.1)
	{
		strcpy(yformat, "%-1.2f");
	}
	else
	{
		strcpy(yformat, "%-1.3f");
	}

	// Don't use the Graphic scale setting routine
	// This can cause the scale to ignore the user inputs
	// roundq(p->y_div, &y_min, &yste, &y_max, yformat);

	// Set plot scales
	graf(xformat, x_min, xste, x_max, yformat, y_min, yste, y_max);

	// Draw annotations
	// The putlabel() function is used for user-defined annotations.
	// This must come before any plotting - including grid lines
	// Then it is supposed to stay clean.
	// But, for user units, it should come after plot scales are set
	// It provides user messages via the plot annotations dialog box
	// The 2nd & 3rd args of putlabel() are in plot user units.
	// The 4th arg of putlabel() is char hgth in inches (for full-page).
	// The sPNote[i].nSize value is point size.  There are 72 points/inch.
	// The 5th arg of putlabel is border code: 1=border, 2=no border (strange)
	// So we get it as 2-sPNote[i].bBorder, where bBorder is ordinary BOOL flag
	// Multiline labels are OK with \n (newline, ASCII LF, dec int 10)
	// But they are NG with CR & LF as produced by an edit control.
	// So we must elim CRs from the note text string
	fntchg('\313');                                           /* simplex.fnt */
	for(i=0; i<PNOTE_NUMNOTES; i++)
	{
		// Check if this is an active note (non-zero length)
		len=strlen(sPNote[i].szText);
		if(len)
		{
			// Allocate a temporary working buffer
			if((szTemp=(LPSTR)malloc((len+1)*sizeof(char)))==NULL)
			{
				AllocErrorMessage();
				free(x);
				free(y);
				return;
			}
			// Remove CRs from annotation text string
			// Note that we are also copying the EOS
			k=0;
			for(j=0; j<=len; j++)
			{
				if(sPNote[i].szText[j]!=13) // 13 = CR
				{
					szTemp[k]=sPNote[i].szText[j];
					k++;
				}
			}
			// Draw annotation - use temp string w/o CRs
			putlabel(szTemp, (float)sPNote[i].fX, (float)sPNote[i].fY,
				(float)(sPNote[i].nSize/72.0), (int)(2-sPNote[i].bBorder),
				USER_UNITS);

			// Free temporary working buffer
			free(szTemp);
		}
	}

	fntchg('\310');                                            /* news.fnt */

	npts=0;
	for(i=(int)(p_data[1]+2); i<=(int)(p_data[2]+2); i++)     /* load data */
	{
		if(i==p_data[1]+2 || p_data[i] || i==p_data[2]+2)          /* gap? */
		{
			y[npts]=(float)(p->y_mult*(p_data[i]+p->y_offset));
			if(y[npts]<y_min)                      /* clip if below bottom */
			{
				y[npts]=y_min;
			}
			if(y[npts]>y_max)                         /* clip if above top */
			{
				y[npts]=y_max;
			}
		}                                                        /* end if */
		else
		{
			y[npts]=0;                                    /* keep gap zero */
		}                                                      /* end else */
		x[npts]=p->x_mult*((i-2)+p->x_offset);
		npts++;
	}                                                           /* end for */

	fntchg('\311');                                             /* swiss.fnt */

	color(label_color);                                   /* set label color */

	width=strwidth(p->title,(float)0.3);                 /* get half-width of title */
	prtfnt((float)(4.75-width),(float)6.40,p->title,(float)0.3,0);                  /* draw title */
	width=strwidth(p->subtitle,(float)0.15);          /* get half-width of subtitle */
	prtfnt((float)(4.75-width),(float)6.175,p->subtitle,(float)0.15,0);           /* draw subtitle */

	color(data_color);                                         /* data color */

	// Set plot line thickness
	// 0 is default, 20 is wider
	tline(p->thick);

	// Plot phase data
	// Note that data is plotted on x-axis versus point number
	// plus an offset called x_start
	for(i=0; i<npts-1; i++)                               /* plot phase data */
	{
		if((i==0) || (*(y+i) && *(y+i+1)) || (i==npts-2))            /* gap? */
		{
			uline(*(x+i)+x_start,*(y+i),*(x+i+1)+x_start,*(y+i+1),1);
		}                                                          /* end if */
	}                                                             /* end for */

	// Restore plot line thickness
	tline(0);

	// Draw lines
	// Note: Need error trapping for entry of NG line parameters

	if(p->do_line==PPLOT_QUAD)                              /* quadratic fit */
	{
		color(line_color);                                     /* line color */
								/* calc and plot quadratic fit to phase data */
		tau=1.0;     /* set tau=1 since basic x-axis spacing is 1 data point */

		CallQuadratic(p_data, tau, &a, &b, &c, &v);         /* quadratic fit */
		npts=0;
		for(i=(int)(p_data[1]+2); i<=(int)(p_data[2]+2); i++)   /* put fit data into array */
		{
			y[npts]=(float)(p->y_mult*((a+b*(i-1-p_data[1])
					+c*(i-1-p_data[1])*(i-1-p_data[1]))+p->y_offset));
			x[npts]=p->x_mult*((i-2)+p->x_offset);
			npts++;
		}                                                         /* end for */

		ClipPlot();
		for(i=0; i<npts-1; i++)                             /* plot fit data */
		{
			uline(*(x+i)+x_start,*(y+i),*(x+i+1)+x_start,*(y+i+1),1);
		}                                                         /* end for */
		UnclipPlot();

		sprintf(label_str,
			"Quadratic Fit: x(t)=a+bt+ct[2], "
			"a=%-10.7e, b=%-10.7e, c=%-10.7e", a, b, c);
		#ifdef TWO_DIGIT_EXP
		ShortenExp(label_str);
		#endif
		fntchg('\313');                                       /* simplex.fnt */
		color(annotation_color);                     /* fit annotation color */
		tmargin((float)6.655);
		if(sPPlot.do_fit)
		{
			ctline(label_str,(float)0.075);
		}
		line_flag=TRUE;
	}                                                              /* end if */

	{
		color(line_color);                                     /* line color */
		save_start=(float)p_data[1];                  /* save starting point */
		save_end=(float)p_data[2];                      /* save ending point */

		// Clip lines
		ClipPlot();

		for(i=0; i<PLINE_NUMLINES; i++)
		{
			switch(sPLine[i].nLineType)
			{
				case PLINE_NONE:                                  /* no line */
				{
					break;
				}

				case PLINE_END:                    /* line between endpoints */
				{
					if( (sPLine[i].fXStart>0)                &&
						(sPLine[i].fXStart<=npts)            &&
						(sPLine[i].fXEnd>=sPLine[i].fXStart) &&
						(sPLine[i].fXEnd<=npts)              )
					{

					p_data[1]=sPLine[i].fXStart;
					p_data[2]=sPLine[i].fXEnd;

					// Draw line
					uline((float)(p->x_mult*(p_data[1]+p->x_offset)),
						(float)(p->y_mult*(sPLine[i].fYStart+p->y_offset)),
						(float)(p->x_mult*(p_data[2]+p->x_offset)),
						(float)(p->y_mult*(sPLine[i].fYEnd+p->y_offset)), 3);
					}
					else
					{
						MessageBeep(0);
						LoadString(hInst, IDS_FORMAT_20, title_str,
							LABELSIZE);
						sprintf(label_str, title_str, i+1);
						LoadString(hInst, IDS_MESSAGE_OMITLINE, title_str,
							LABELSIZE);
						strcat(label_str, title_str);
						LoadString(hInst, IDS_TITLE_PHASEPLOT, title_str,
							BUFFER_SIZE);
						MessageBox(hDialog, label_str, title_str,
							MB_ICONEXCLAMATION);
					}
					break;
				}

				// PLINE_LIN not activated
				case PLINE_LIN:                           /* regression line */
				{
					if( (sPLine[i].fXStart>0)               &&
						(sPLine[i].fXStart<npts)            &&
						(sPLine[i].fXEnd>sPLine[i].fXStart) &&
						(sPLine[i].fXEnd<=npts)              )
					{
						p_data[1]=sPLine[i].fXStart;
						p_data[2]=sPLine[i].fXEnd;

						CalcLinFreqDrift(p_data, &intercept, &slope, &v);
															   /* calc drift */
						// Draw line
						uline((float)(p->x_mult*(p_data[1]+p->x_offset)),
							(float)(p->y_mult*((intercept+(p_data[1] *
							slope))+p->y_offset)),
							(float)(p->x_mult*(p_data[2]+p->x_offset)),
							(float)(p->y_mult*((intercept+(p_data[2] *
							slope))+p->y_offset)), 3);

						if(!line_flag)           /* can list only 1 line fit */
						{
							UnclipPlot();
							sprintf(label_str,
							"1st Linear Fit: y(t)=a+bt, "
							"Intercept=a=%-10.7e, Slope=b=%-10.7e",
							intercept,slope);
							#ifdef TWO_DIGIT_EXP
							ShortenExp(label_str);
							#endif
							fntchg('\313');                   /* simplex.fnt */
							color(annotation_color);     /* annotation color */                                                /* black */
							tmargin((float)6.655);
							if(sPPlot.do_fit)
							{
								ctline(label_str,(float)0.075);
							}
							line_flag=TRUE;
							color(line_color);                 /* line color */
							ClipPlot();
						}
					}
					else
					{
						MessageBeep(0);
						LoadString(hInst, IDS_FORMAT_20, title_str,
							LABELSIZE);
						sprintf(label_str, title_str, i+1);
						LoadString(hInst, IDS_MESSAGE_OMITLINE, title_str,
							LABELSIZE);
						strcat(label_str, title_str);
						LoadString(hInst, IDS_TITLE_PHASEPLOT, title_str,
							BUFFER_SIZE);
						MessageBox(hDialog, label_str, title_str,
							MB_ICONEXCLAMATION);
					}
					break;
				}

				case PLINE_QUAD:                            /* quadratic fit */
				{
					if( (sPLine[i].fXStart>0)                 &&
						(sPLine[i].fXStart<npts-2)            &&
						(sPLine[i].fXEnd>sPLine[i].fXStart)+2 &&
						(sPLine[i].fXEnd<=npts)               )
					{
						p_data[1]=sPLine[i].fXStart;
						p_data[2]=sPLine[i].fXEnd;

						/* set tau=1 since x-axis spacing is 1 data point */
						tau=1;

						CallQuadratic(p_data, tau, &a, &b, &c, &v);

						npts=(int)(p_data[2]-p_data[1]+1);
						for(j=0; j<=npts; j++)
						{
							y[j]=(float)((a+b*(j+1)+c*(j+1)*(j+1))+p->y_offset);
							y[j]*=p->y_mult;
							x[j]=(float)(p->x_mult*((p_data[1]+j)+p->x_offset));
						}                                         /* end for */

						for(j=0; j<npts-1; j++)             /* plot fit data */
						{
							// Draw line
							uline(*(x+j), *(y+j), *(x+j+1), *(y+j+1), 1);
						}                                         /* end for */

						if(!line_flag)           /* can list only 1 line fit */
						{
							UnclipPlot();
							sprintf(label_str,
							"1st Curve Fit: x(t)=a+bt+ct[2], "
							"a=%-10.7e, b=%-10.7e, c=%-10.7e", a, b, c);
							#ifdef TWO_DIGIT_EXP
							ShortenExp(label_str);
							#endif
							fntchg('\313');                   /* simplex.fnt */
							color(annotation_color);     /* annotation color */                                                /* black */
							tmargin((float)6.655);
							if(sPPlot.do_fit)
							{
								ctline(label_str,(float)0.075);
							}
							line_flag=TRUE;
							color(line_color);                 /* line color */
							ClipPlot();
						}
					}
					else
					{
						MessageBeep(0);
						LoadString(hInst, IDS_FORMAT_20, title_str,
							LABELSIZE);
						sprintf(label_str, title_str, i+1);
						LoadString(hInst, IDS_MESSAGE_OMITLINE, title_str,
							LABELSIZE);
						strcat(label_str, title_str);
						LoadString(hInst, IDS_TITLE_PHASEPLOT, title_str,
							BUFFER_SIZE);
						MessageBox(hDialog, label_str, title_str,
							MB_ICONEXCLAMATION);
					}
					break;
				}
				case PLINE_AVG:                    /* horizontal line at avg */
				{
					if( (sPLine[i].fXStart>0)               &&
						(sPLine[i].fXStart<npts)            &&
						(sPLine[i].fXEnd>sPLine[i].fXStart) &&
						(sPLine[i].fXEnd<=npts)              )
					{
						p_data[1]=sPLine[i].fXStart;
						p_data[2]=sPLine[i].fXEnd;

						CalcMean(p_data, &avg, PHASE_DATA);  /* calc average */

						// Draw line
						uline((float)(p->x_mult*(p_data[1]+p->x_offset)),
							(float)(p->y_mult*(avg+p->y_offset)),
							(float)(p->x_mult*(p_data[2]+p->x_offset)),
							(float)(p->y_mult*(avg+p->y_offset)), 3);

						// List average fit
						if(!line_flag)			 /* can list only 1 line fit */
						{
							UnclipPlot();
							sprintf(label_str,
							"1st Average Fit: x(avg)=%-10.7e", avg);
							#ifdef TWO_DIGIT_EXP
							ShortenExp(label_str);
							#endif
							fntchg('\313');					  /* simplex.fnt */
							color(annotation_color);		   /* note color */                                                /* black */
							tmargin((float)6.655);
							if(sPPlot.do_fit)
							{
								ctline(label_str,(float)0.075);
							}
							line_flag=TRUE;
							color(line_color);                 /* line color */
							ClipPlot();
						}
					}
					else
					{
						MessageBeep(0);
						LoadString(hInst, IDS_FORMAT_20, title_str,
							LABELSIZE);
						sprintf(label_str, title_str, i+1);
						LoadString(hInst, IDS_MESSAGE_OMITLINE, title_str,
							LABELSIZE);
						strcat(label_str, title_str);
						LoadString(hInst, IDS_TITLE_PHASEPLOT, title_str,
							BUFFER_SIZE);
						MessageBox(hDialog, label_str, title_str,
							MB_ICONEXCLAMATION);
					}
					break;
				}
				case PLINE_MARK:                     /* vertical marker line */
				{
					if( (sPLine[i].fXStart>0)     &&
						(sPLine[i].fXStart<=npts) )
					{
						p_data[1]=sPLine[i].fXStart;
						p_data[2]=sPLine[i].fXStart;

						// Draw line
						uline((float)(p->x_mult*(p_data[1]+p->x_offset)),
							(float)(p->y_mult*(p->y_min+p->y_offset)),
							(float)(p->x_mult*(p_data[2]+p->x_offset)),
							(float)(p->y_mult*(p->y_max+p->y_offset)), 3);
					}
					else
					{
						MessageBeep(0);
						LoadString(hInst, IDS_FORMAT_20, title_str,
							LABELSIZE);
						sprintf(label_str, title_str, i+1);
						LoadString(hInst, IDS_MESSAGE_OMITLINE, title_str,
							LABELSIZE);
						strcat(label_str, title_str);
						LoadString(hInst, IDS_TITLE_PHASEPLOT, title_str,
							BUFFER_SIZE);
						MessageBox(hDialog, label_str, title_str,
							MB_ICONEXCLAMATION);
					}
					break;
				}
			}                                                  /* end switch */
			// Unclip lines - probably unnecessary
			UnclipPlot();
		}                                                         /* end for */
		p_data[1]=save_start;                      /* restore starting point */
		p_data[2]=save_end;                          /* restore ending point */
	}                                                              /* end if */

	fntchg('\313');                                           /* simplex.fnt */
	color(annotation_color);                             /* annotation color */                                                    /* black */

	// Write date
	if(p->do_date)
	{
		Dateline(1);                         /* put date & file info on plot */
	}                                                              /* end if */

	// Display message box
	// Locations are x, y in pixels, origin at bottom left, 8192 x 6266 space
	// Find half length of message box in pixels
	lgth=(H_MARGIN+strwidtq(p->message_str, CHAR_HGTH));

	// Write message
	if(p->message_str[0]!='\0')         /* don't display a blank message box */
	{
		switch(p->position)
		{
			case MSG_TOP_LEFT:            /* message box at top left of plot */
			{
				MsgBox(300,4400,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_MID_LEFT:         /* message box at middle left of plot */
			{
				MsgBox(300,2270,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_BOT_LEFT:         /* message box at bottom left of plot */
			{
				MsgBox(300,200,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_TOP_CENTER:        /* message box at top center of plot */
			{
				MsgBox(3470-lgth,4400, p->message_str);
				break;
			}                                                    /* end case */
			case MSG_MID_CENTER:     /* message box at middle center of plot */
			{
				MsgBox(3470-lgth,2270,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_BOT_CENTER:     /* message box at bottom center of plot */
			{
				MsgBox(3470-lgth,200,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_TOP_RIGHT:          /* message box at top right of plot */
			{
				MsgBox(6400-2*lgth,4400,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_MID_RIGHT:       /* message box at middle right of plot */
			{
				MsgBox(6400-2*lgth,2270,p->message_str);
				break;
			}                                                    /* end case */
			case MSG_BOT_RIGHT:       /* message box at bottom right of plot */
			{
				MsgBox(6400-2*lgth,200,p->message_str);
				break;
			}                                                    /* end case */
		}                                                      /* end switch */
	}

	// Insert plot
	// No direct user interface such as checkbox and file edit box under options
	// Plot will be done if there is a data plot called p_insert.tkf in szPath
	// Make full insert filename
	strcpy(szInsertFile, szPath);
	strcat(szInsertFile,"p_insert.tkf"); 

	// Try to open it
	if((file=fopen(szInsertFile, "r"))!=NULL)
	{
		// Immediately release file pointer
		fclose(file);

		// Insert data plot at bottom left of phase plot
		InsertPlot(300, 200, szInsertFile);
	}

	// Write label
	if(bPlotLabel)
	{
		UnclipPlot();
		color(annotation_color);                         /* annotation color */
		width=(float)intopix(strwidth(szPlotLabel, (float)0.05));
		putlabel(szPlotLabel, 8192-2*(width+20), 20, (float)0.05, 0, PIXEL_UNITS);
	}

	// Activate mouse cursor
	if(p->do_mouse)
	{
		// Allocate memory for x and y point arrays
		fx=(float*)gpcalloc(64, sizeof(float));
		fy=(float*)gpcalloc(64, sizeof(float));

		curson(0, 0, &fx, &fy, &find);                /* turn on mouse cursor*/

		// Free memory - this assumes that point info isn't needed
		gpcfree((void**)&fx);
		gpcfree((void**)&fy);
	}                                                              /* end if */

	rightax(0);

	// Avoid unwanted clipping 
	UnclipPlot();

	endplot();
									/* finish plot and wait for instructions */
												/* to exit or draw next plot */
										/* resets file pointers and defaults */

	// Free plot arrays
	free(x);
	free(y);

	stopplot();            /* close files, return to text mode, exit program */
}

/*****************************************************************************/
