/*****************************************************************************/
/*                                                                           */
/*                           ReflectedTotalvarCalc()                         */
/*                                                                           */
/*      Function to calculate TOTALVAR using reflected phase data            */
/*                                                                           */
/*      Parameters:     F_TYPE x[]    	= standard phase data                */
/*                                      	x[0] = # data points             */
/*                                      	x[1] = analysis start            */
/*                                      	x[2] = analysis end              */
/*						F_TYPE *TotalDev= pointer to TOTALDEV                */
/*						F_TYPE fTau 	= interval between phase data points */
/*						int nAF			= averaging factor                   */
/*                                                                           */
/*      Return:         int           = # TOTALVAR analysis points,          */
/*                                      or -1 if error                       */
/*                                                                           */
/*      Notes:          As of now, for this implementation:                  */
/*						1. Entire phase data array is processed, regardless  */
/*                         of analysis limits.                               */
/*						2. No gap handling.                                  */
/*                      2. Data array size limited to 1/2 of normal.         */
/*						3. Calculation is done entirely with a new array     */
/*						   that is deleted after this function closes.       */
/*						4. No progress indicator.                            */
/*                                                                           */
/*		Reference:		D. Howe, "Methos of Improving the Estimation of      */
/*						Long-Term Frequency Variance", Proc. 11th EFTF,      */
/*						March, 1997, (to be published).                      */
/*                                                                           */
/*      Revision record:                                                     */
/*          03/31/97    Created                                              */
/*                                                                           */
/* (c) Copyright 1997   Hamilton Technical Services   All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

int FAR PASCAL _export ReflectedTotalvarCalc(F_TYPE fX[], F_TYPE *fTotalDev,
	F_TYPE fTau, int nAF)
{
	// Local variables
	int i;              // Index
	int nNum;           // # points
	int nCount;			// # analysis points
	F_TYPE *fP;     	// TOTALVAR phase array pointer
	F_TYPE fSum=0;		// TOTALVAR summation

	// Initializations
	nNum=fX[2]-fX[1]+1;   // Total # original phase data points = N

	// Check array size
	if(nNum>MAX_DATA_FILE_SIZE/2)
	{
		return(-1);  // Error - too many data points to process
	}

	// Allocate new phase data array to new size 2N-1
	if((fP=(F_TYPE*)GlobalAllocPtr(GHND,
		(2*nNum-1+ARRAY_OFFSET+EXTRA_MEM)*sizeof(F_TYPE)))==NULL)
	{
		AllocErrorMessage();
		return(-1);  // Error - memory reallocation failed
	}

	// Copy entire original phase data array to working array
	for(i=0; i<nNum+ARRAY_OFFSET; i++)
	{
		fTotal[i]=fTotal[i];
	}

	// Adjoin reflected phase data: x[N+i]=-x[N-i]
	for(i=1; i<nNum; i++)
	{
		fX[nNum+i+ARRAY_OFFSET]=-fX{nNum-i+ARRAY_OFFSET];
	}

	// Calc TOTALVAR - See Eq (4) of Reference
	for(i=0; i<nNum-nAF-1; i++)
	{
		fSum+=SQR(fP[i+ARRAY_OFFSET]-2*fP[i+nAF+ARRAY_OFFSET]+
			fP[nNum+2*nAF+ARRAY_OFFSET]);
		nCount++;
	}

	// Scale result - See Eq (4) of Reference
	fSum/=(fTau*fTau*(2*(nNum-nAF-1)));

	// Find TOTALDEV
	if(fSum>0)
	{
		*TotalDev=SQRT(fSum)
	}

	return(nCount);  // Return # phase analysis points
}

/***************************************************************************/

