/*****************************************************************************/
/*                                                                           */
/*                            STABLE_BB.C                                    */
/*                                                                           */
/*		S&E Functions for Polynomial Fitting                                 */
/*      Rev: 8.01 S/N 18146                                                  */
/*      Copied here from source disk                                         */
/*                                                                           */
/*		Functions:                                                           */
/*			PolyCurveFit()  in CURVEFIT.C                                    */
/*					GetPolyMem()                                             */
/*					CheckBadMem()                                            */
/*                  MultipleReg() in MULREG.C                                */
/*						GetPolyMem()                                         */
/*						MatTxTiX()                                           */
/*                      MatYTiX()                                            */
/*						ResAnamysis()                                        */
/*						GaussJordan() in GJ.C                                */
/*                          GetIntMem()                                      */
/*							CheckBadMem()                                    */
/*                          gjswap()                                         */
/*                                                                           */
/*      Non-standard headers included:                                       */
/*           polytype.h                                                      */
/*           mulreg.h                                                        */
/*           gj.h                                                            */
/*                                                                           */
/*      Reference:                                                           */
/*           The Science and Engineering Tools for Turbo/Borland C           */
/*           IPC-TC-006 Revision 8.0.1 Quinn-Curtis June 1992                */
/*                                                                           */
/*	    Revision record:                                                     */
/*			 12/15/07   Created                                              */
/*						Commented out unused functions                       */
/*                      Replaced alloc.h with malloc.h                       */
/*                      Compiles OK                                          */
/*                      Stable32 builds OK with this in project              */
/*                      Not used yet                                         */
/*                      Intend to call PolyCurveFit()                        */
/*						polytype.h replaced by reeltypr.h as in rest of      */
/*						Stable32 - this makes retype variables floats        */
/*           12/16/07	Changed realtype to polytype defined as double       */
/*                      Needed to improve dynamic range of curve fitting     */
/*                      This propagates into polytype.h, mulreg.h & gj.h     */
/*                      Changed GetMem() to GetPolyMem()                     */
/*						Changed CalcOverflow() to CalcPolyOverflow()         */
/*                      This file then builds w/o errors or warnings         */
/*                      With doubles, poly fit seems OK to order about 12    */
/*                                                                           */
/* (c) 2007  W. Riley   Hamilton Technical Services   All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#define polytype double
#include "Stable_w.h"

// File: CURVEFIT.C
/*-----------------------------------------------------------------

	 Copyright (c) Quinn-Curtis, 1992

	 Filename:  CURVEFIT.C
	 Revision:  8.0 LAB
	 Date:      6/1/92

	 Description: -Routines for polynomial & cubic spline
		   curvefitting

	 Functions:
	  PolyCurveFit  CubicSplines   CalcSpline
 *****************************************************************/

#include <stdlib.h>
#include <math.h>
#include <malloc.h>
#include "polytype.h"
#include "mulreg.h"
#define nearzero 1.0E-20

// Note: This is the top-level function we want to use
// It calls GetPolyMem(), CheckBadMem() and MultipleReg()

/* Calculate the least squares approximation of the original data */
void PolyCurveFit (polytype *indvar, polytype *depvar, int numobs, int order,
		 polytype *coef, polytype *yest, polytype *resid, polytype *see,
		 polytype *coefsig, polytype *rsq, polytype *r, char *cferror)
{
	int i, j, iord;
	polytype *regmat;

	regmat = GetPolyMem (numobs*order);  CheckBadMem(regmat);

	if (order + 1 < numobs)
	{
		for (i = 0; i < numobs; i++)  /* fill the regression matrix */
		{
			iord = i * order;
			regmat[iord] = indvar[i];
			for (j = 1; j < order; j++)
				regmat[iord+j] = indvar[i] * regmat[iord+j - 1];
		}
		MultipleReg (regmat, depvar, order, numobs, coef, yest,
			resid, see, coefsig, rsq, r, cferror);
	}
	else
		*cferror = 1;
	free(regmat);
}


/*--------------------------------------------------------------*/

#if(0) // Not used
void tridiag(polytype *mat, int n, polytype *vec)
{
	int i, i4, i14;
	polytype pivot, mult, ci, bi;

	for (i = 1; i < n; i++)
	{
		i4 = i * 4;
		i14 = (i+1) * 4;
		pivot = mat[i4];
		ci = mat[i4+1];
		bi = mat[i4+2];
		mult = mat[i14+3] / pivot;
		if (fabs(mult) > nearzero)
		{
			mat[i14+3] = mult;
			mat[i14]   = mat[i14] - mult * ci;
			mat[i14+2] = mat[i14+2] - mult * bi;
		}
		else
			mat[i14+3] = (polytype) 0.0;
	}
	vec[n] = mat[n*4+2] / mat[n*4+0];
	for (i = n - 1; i >= 1; --i)
	{
		i4 = i * 4;
		vec[i] = (mat[i4+2] - mat[i4+1] * vec[i+1]) / mat[i4];
	}
}
#endif

/*--------------------------------------------------------------*/

#if(0) // Not used
/* Interpolation using a cubic equation curve fitting */
void CubicSplines(polytype *x, polytype *y, int n, polytype *s)
{
	int i, i4;
	polytype bb, cc, hh, ff, fi, hi, nextf;
	polytype *eqn, *h, *bvec;

	eqn = GetPolyMem((n+1)*4); CheckBadMem(eqn);
	h = GetPolyMem(n+1);       CheckBadMem(h);
	bvec = GetPolyMem(n+1);    CheckBadMem(bvec);

	for (i = 0; i < n; i++)
		h[i] = x[i+1] - x[i];

	hh = h[0];
	ff = y[0];
	nextf = y[1];
	for (i = 1; i < n; i++)
	{
		i4 = i * 4;
		fi = nextf;
		nextf = y[i+1];
		hi = h[i];
		eqn[i4+2] = ((polytype) 3.0) * ((nextf - fi) / hi - (fi - ff) / hh);
		eqn[i4]   = ((polytype) 2.0) * (hh + hi);
		eqn[i4+3] = hh;
		eqn[i4+1] = hi;
		hh = hi;
		ff = fi;
	}
	tridiag (eqn, n - 1, bvec);
	bb = (polytype) 0.0;
	hh = h[0];
	s[2] = bb;
	s[0] = y[0];
	cc = (y[1] - s[0]) / hh - bvec[1] * hh / 3.0;
	s[1] = cc;
	for (i = 1; i < n; i++)
	{
		i4 = i * 4;
		s[i4+2] = bvec[i];
		s[i4+0] = y[i];
		cc = (s[i4+2] + bb) * hh + cc;
		s[i4+1] = cc;
		s[(i-1)*4+3] = (s[i4+2] - bb) / (3 * hh);
		bb = s[i4+2];
		hh = h[i];
	}
	s[(n-1)*4+3] = -bb / (3.0 * hh);
	free(eqn); free(h); free(bvec);
}
#endif

/*--------------------------------------------------------------*/

#if(0) // Not used
/* calculate the yvalue given the x value & the coefficient vector */
polytype PolyCalc(polytype XIn, polytype *CoefVector, int order)
{
	polytype YOut;
	int i;

	YOut = CoefVector[order];
	for (i = order - 1; i >= 0; i--)
		YOut = YOut * XIn + CoefVector[i];
	return(YOut);
}
#endif

/*--------------------------------------------------------------*/

#if(0) // Not used
/* Calculate the cubic spline interpolation of a y-value given an x-value
and the coefficient matrix */
void CalcSpline(polytype *xv, polytype *coef, int n, polytype x, polytype *y)
{
	int i, j, i4;
	polytype *CoefVector, dif;

	CoefVector = GetPolyMem(4);
	CheckBadMem(CoefVector);

	i = -1;
	if (x < xv[0])
	{
		i = 0;
		x = xv[i];
	}
	else
		if (x > xv[n])
		{
			i = n-1;
			x = xv[n];
		}
		else
		do
		{
			i++;
		} while (!((x >= xv[i]) && (x <= xv[i + 1])));
	dif = x - xv[i];
	i4 = i * 4;
	for (j = 0; j <= 3; j++)
	{
		CoefVector[j] = coef[i4+j];
	}
	*y = PolyCalc (dif, CoefVector, 3);
	free(CoefVector);
}
#endif

/*--------------------------------------------------------------*/

// File: MULREG.C
/*-----------------------------------------------------------------
	 Copyright (c) Quinn-Curtis, 1992

	 Filename:  MULREG.C
	 Revision:  8.01 LAB
	 Date:      4/01/92

	 Description: -Routines for multiple regression
	 Functions:  MultipleReg

-----------------------------------------------------------------*/

#include <stdlib.h>
#include <math.h>
//#include <alloc.h>
#include <malloc.h>
#include "gj.h"

// Called by MultipleReg()
/* Multiply the transpose of aryin by aryin */
void  MatTxTiX (polytype *aryin, int numrow, int numcol, polytype *aryout)
{
   int i, j, k, inum;

	for (i = 0; i < numcol; i++)
	{
		inum = i * numcol;
		for (j = 0; j < numcol; j++)
		{
			aryout[inum+j] = (polytype) 0.0;
			for (k = 0; k < numrow; k++ )
			{
				aryout[inum+j] = aryout[inum+j] +
								 aryin[k*numcol+j] * aryin[k*numcol+i];
			}
		}
   }
}

// Called by MultipleReg()
/*--------------------------------------------------------------*/
/* multiply dependent variable aryy by the transpose of aryx */
void MatYTiX (polytype *aryy, polytype *aryx, int numrow,
			int numcol, polytype *aryout)
{
   int i, k;

   for (i = 0; i < numcol; i++)
   {
	  aryout[i] = (polytype) 0.0;
	  for (k = 0; k < numrow; k++)
		 aryout[i] = aryout[i] + aryy[k] * aryx[k*numcol+i];
   }
}

// Called by MultipleReg()
/*--------------------------------------------------------------*/
/* Calculate solution vectory by multiplying the inverse of ymat
times regmat, the residuals, standard error of the estimates,
see, rsq, & r val are all returned */

void ResAnalysis (polytype *regmat, polytype *ymat, polytype *regcoef,
		 polytype *aryinv, int numrow, int numcol, polytype *yest,
		 polytype *resid, polytype *see, polytype *coefsig,
		 polytype *rsq, polytype *r)
{
   int i, j, nxx, inum;
   polytype sumressq, sumy, sumysq;

   sumressq = (polytype) 0.0;
   sumy = (polytype) 0.0;
   sumysq = (polytype) 0.0;
   for (i = 0; i < numrow; i++)
   {
	  yest[i] = (polytype) 0.0;
	  inum = i * numcol;
	  for (j = 0; j < numcol; j++)
		yest[i] = yest[i] + regcoef[j] * regmat[inum+j];
	  resid[i] = yest[i] - ymat[i];
	  sumressq = sumressq + resid[i] * resid[i];
	  sumy = sumy + ymat[i];
	  sumysq = sumysq + ymat[i] * ymat[i];
   }
   if (numrow == numcol)
	  nxx = 1;
   else
	  nxx = numrow - numcol;
   *see = (polytype) sqrt(sumressq / nxx);
   for (i = 0; i < numcol; i++)
	  coefsig[i] = *see * ((polytype) sqrt(fabs(aryinv[i*numcol+i])));
   *rsq = (polytype)fabs ((((polytype) 1.0) - sumressq / (sumysq - (sumy*sumy) / numrow)));
   *r = (polytype) sqrt(*rsq);
}


// Called by PloyCurveFit()
// It calls GetPolyMem(), MatTxTiX(), MatYYiX(), GaussJordan(), ResAnalys()
/*--------------------------------------------------------------*/
void MultipleReg (polytype *indvardat, polytype *depvardat, int numiv, int numobs,
		polytype *regcoef, polytype *yest, polytype *resid, polytype *see,
		polytype *coefsig, polytype *rsq, polytype *r, char *regerror)
{

	polytype  rxx, rsqxx, seexx, regdet;
	polytype  *regmat, *aryinv, *arya, *aryg;
	int numcol, inum, i, j, inumiv;

	numcol = numiv + 1;

	regmat = GetPolyMem(numobs*numcol);  CheckBadMem(regmat);
	aryinv = GetPolyMem(numcol*numcol);  CheckBadMem(aryinv);
	arya = GetPolyMem(numcol*numcol);    CheckBadMem(arya);
	aryg = GetPolyMem(numcol);           CheckBadMem(aryg);

	for (i = 0; i < numobs; i++)
	{
		inum = i * numcol;
		inumiv = i * numiv;
		for (j = 0; j < numiv; j++)
			regmat[inum+j+1] = indvardat[inumiv+j];
		regmat[inum] = 1;
	}

	/*Multiply the transpose of regmat by regmat*/
	MatTxTiX (regmat, numobs, numcol, arya);

	/* Multiply the dependent variable * result of MatTxTix */
	MatYTiX (depvardat, regmat, numobs, numcol, aryg);

	/* find the inverse of result of MatTxTix */
	GaussJordan (arya, aryg, numcol, regcoef, aryinv, &regdet);

	/* Calcualate solution */
	ResAnalysis (regmat, depvardat, regcoef, aryinv, numobs,
						numcol, yest, resid, &seexx, coefsig, &rsqxx, &rxx);
	*see = seexx;
	*rsq = rsqxx;
	*r   = rxx;
   if (regdet < (polytype) 1.0e-7)
	  *regerror = 1;
   else
	  *regerror = 0;
   free(regmat); free(aryinv); free(arya); free(aryg);
}

/*--------------------------------------------------------------*/

// File: GJ.C
/*-----------------------------------------------------------------
	 Copyright (c) Quinn-Curtis, 1992

	 Filename:  GJ.C
	 Revision:  8.01 LAB
	 Date:      6/1/92
	 Description: - Solves for simultaneous equations using
			the Gauss Jordan technique
	 Functions:   GaussJordan
-----------------------------------------------------------------*/

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "reeltype.h"

// Called by GaussJordan()
void gjswap (polytype *s1, polytype *s2)
{
   polytype temp;

   temp = *s1;
   *s1 = *s2;
   *s2 = temp;
}

// Called by MultipleReg()
// It calls GetIntMem(), CheckBadMem, gjswap(), CalcPolyOverflow()
/* ----------------------------------------------------------- */
void GaussJordan (polytype *coefary, polytype *constary,
				  int numcol, polytype *solcoef, polytype *invary,
				  polytype *det)
{
	int *pivlst;
	char *pivchk;
	int i, j, k, l, lerow, lecol, l1, inum, jnum, lecnum, l1num, l2;
	polytype piv, t, leval;

	pivlst = GetIntMem (numcol * 2);
	CheckBadMem(pivlst);
	pivchk = (char *) calloc (numcol, sizeof(char));
	CheckBadMem(pivchk);

	*det = (polytype) 1.0;
	for (i = 0; i < numcol; i++)
	{
		pivchk[i] = 0;
		inum = i * numcol;
		for (j = 0; j < numcol; j++)
			invary[inum+j] = coefary[inum+j];
	}

	for (i = 0; i < numcol; i++)
	{
		leval = (polytype) 0.0;
		for (j = 0; j < numcol; j++)
		{
			if (!pivchk[j])
			{
				jnum = j * numcol;
				for (k = 0; k < numcol; k++)
				{
					if (!pivchk[k])
					{
						if (fabs(invary[jnum+k]) > leval)
						{
							lerow = j;
							lecol = k;
							leval = (polytype) fabs(invary[jnum+k]);
						}
					}
				}
			}
		}
		pivchk[lecol] = 1;
		pivlst[i * 2] = lerow;
		pivlst[i * 2+1] = lecol;
		lecnum = lecol*numcol;
		if (lerow != lecol)
		{
			*det = -*det;
			for (l = 0; l < numcol; l++)
				gjswap (&invary[lerow * numcol + l], &invary[lecnum + l]);
			gjswap (&constary[lerow], &constary[lecol]);
		}
		piv = invary[lecnum+lecol];
		*det = *det * piv;
		*det = (polytype) CalcPolyOverflow(*det);

		invary[lecnum+lecol] =  (polytype) 1.0;
		for (l = 0; l < numcol; l++)
			invary[lecnum+l] = invary[lecnum+l] / piv;
		constary[lecol] = constary[lecol] / piv;
		for ( l1 = 0; l1 < numcol; l1++ )
		{
			if ( l1 != lecol )
			{
				l1num = l1 * numcol;
				t = invary[l1num + lecol];
				invary[l1num + lecol] = (polytype) 0.0;
				for (l = 0; l < numcol; l++)
					invary[l1num+l] = invary[l1num+l] - invary[lecnum+l] * t;
				constary[l1] = constary[l1] - constary[lecol] * t;
			}
		}
	}
	for (i = 0; i < numcol; i++)
	{
		l = numcol - i - 1;
		l2 = l * 2;
		if ( pivlst[l2] != pivlst[l2+1] )
		{
			lerow = pivlst[l2];
			lecol = pivlst[l2+1];
			for ( k = 0; k < numcol; k++ )
				gjswap(&invary[k * numcol + lerow], &invary[k * numcol + lecol]);
		}
	}
	for ( i = 0; i < numcol; i++ )
		solcoef[i] = constary[i];
	free(pivlst);  free(pivchk);
}
/* ----------------------------------------------------------- */

// File: polytype.C
/*---------------------------------------------------------------------
	 Copyright (c) Quinn-Curtis, 1992

	 Filename:  polytype.C
	 Revision:  8.01 LAB
	 Date:      6/01/92

	 Description: -Routines for dynamic memory
				  -Routines for handling allocation and graphics
				   errors
				  -polytype.h defines the constant 'defaultbgidir' which
				   specifies pathname for bgi files used in
				   all graphics demos.
-------------------------------------------------------------------*/


#include <stdio.h>
#include <stdlib.h>
#include "reeltype.h"

// Substitutite for SetQC_Error
void SetQC_Error (int new_err)
{
	;
}

#if(0) // Not used
#ifdef __cplusplus
extern "C" {
#endif


#ifndef QC_MEMERR
#define QC_MEMERR -20
#endif
int QC_error = 0;


/* this will print out memory allocation error messages which have occurred.
   Once an error is printed the program terminates.  Modify this function
   to handle memory errors the way your program requires */
void SetQC_Error (int new_err)
{
	QC_error = new_err;
	if (QC_error == QC_MEMERR)
	{
		printf("TC006 error: OUT OF MEMORY\n");
		exit (0);

	}
}


int CheckQC_Error (void)
{
	return (QC_error);
}
#endif

void CheckBadMem(void *p)
{
	if (p == NULL)
		SetQC_Error (QC_MEMERR);
}


polytype *GetPolyMem(int n)
{      polytype *xt;
	   xt = (polytype *) calloc(n, sizeof(polytype));
	   return(xt);
}

int *GetIntMem(int n)
{      int *xt;
	   xt = (int *) calloc(n, sizeof(int));
	   return(xt);
}

#if(0) // Not used
int BadMem(void *p)
{
	if (p == NULL)
	{
		printf("TC006 error: OUT OF MEMORY\n");
		return (-1);
	}
	else return (0);
}


void SuperSwap(polytype *s1, polytype *s2)
{
   polytype temp;

   temp = *s1;
   *s1 = *s2;
   *s2 = temp;
}
#endif

double CalcPolyOverflow(polytype num)
{
	int sze;
	sze = sizeof(polytype);
	if (sze == 4)
	{
		if (num > 1.0e+20)
			return(1.0e+20);
		else
			return(num);
	}
	else
	{
		if (num > 1.0e+100)
			return(1.0e+100);
		else
			return(num);
	}
}

#if(0) // Not used
#ifdef __cplusplus
}
#endif
#endif