/*****************************************************************************/
/*                                                                           */
/*                            STABLE_E.C                                     */
/*                                                                           */
/*		Stable Windows 95/NT Editing Dialog Box Functions (2nd Group)        */
/*                                                                           */
/*      EditDlgProc()                                                        */
/*      GetRowData()                                                         */
/*      FindDlgProc()                                                        */
/*      ReplaceDlgProc()                                                     */
/*                                                                           */
/*		Revision record:  												     */
/*      	08/24/96    Added #define STRICT                                 */
/*					   	Changed dialog procedure prototypes from             */
/*					    FAR PASCAL _export protos to CALLBACK                */
/*			10/12/96	Spun off non-editing functions into STABLE_M.C       */
/*			06/29/97	Use GET_WM_COMMAND_ID, etc. WIN32 compatibility      */
/*						macros to process WM_COMMAND messages:               */
/*						wParam <- GET_WM_COMMAND_ID(wParam, lParam)          */
/*						HIWORD(lParam) <- GET_WM_COMMAND_CMD(wParam, lParam) */
/*						Replaced WORD with UINT                              */
/*			06/30/97	Replaced FAR PASCAL _export with WINAPI              */
/*			07/10/97	Made numerous Win32 compatibility changes            */
/*			12/24/97	Made changes for error-free BC 32 bit compilation    */
/*			01/16/98	Added ShortenExp() usage for MSC VC++                */
/*			08/05/01	Added coloring of timetag gaps                       */
/*          07/22/07    Changed to safe string functions                     */
/*                                                                           */
/* (c) 1996-2007  W. Riley  Hamilton Technical Services  All Rights Reserved */
/*                                                                           */
/*****************************************************************************/

#define STRICT 					// For Win32 compatibility

#include "stable_w.h"     		// Stable/Win include file

/*****************************************************************************/
/*                                                                           */
/*                   Edit Dialog Procedure                                   */
/*                                                                           */
/* 		Uses ProtoView DataTable Spreadsheet Control 16-Bit Ver. 3.0         */
/*                                                                           */
/*		Dialog box idiom:	OK               = Accept changes and close      */
/*                          Cancel (Default) = Reject changes and close      */
/*                                                                           */
/*      This requires temporary working copy of data which replaces original */
/*      if changes are accepted.                                             */
/*                                                                           */
/*		Other controls just manipulate data within scope of dialog box.      */
/*                                                                           */
/*		Revision record:  												     */
/*			09/24/96	Fixed outlier code & added outlier finding           */
/*          09/25/96	Added code to handle CR in edit controls             */
/*                      IDOK message not used for OK button - it is          */
/*                      used to validate edit control entries.               */
/*						Note that no button is a DEFPUSHBUTTON.              */
/*          09/26/96	Added code insert & delete of timetags               */
/*			09/28/96	Disabled autoinsert                                  */
/*			10/30/96	Added fMed & fMedAbsDev calc in INITDIALOG           */
/*			11/13/96	Added code for handling new data                     */
/*			02/18/97	Added code for deleting blocks of data               */
/*			05/11/02	Changed to faster MAD calc using NewMedDev()         */
/*			11/18/06	Added CreateTab() code                               */
/*                                                                           */
/*****************************************************************************/

BOOL CALLBACK EditDlgProc(HWND hDlg, UINT message, WPARAM wParam,
    LPARAM lParam)
{
	// Local macros
	#define MIN_NUM_ROWS MIN_ARRAY_SIZE
									// Size of empty data array
	#define TAG_CHARS   17			// # characters in timetag column
									// 8 MJD digits + decimal point +
									// 8 decimal fraction digits
	#define NUM_CHARS   20			// # characters in data column
									// Sign + leading digit + decimal point +
									// 13 digit fractional part + e +
									// exponent sign + 2 exponent digits
	#define NO_GAP	    -1			// Flag for no gap found
	#define DTN_COLOR   57			// WJR made-up notification with unused #
	#define ONE_COL      0      	// Data col # for narrow format
	#define TIMETAG_COL  0     		// Timetag col # for wide format
	#define DATA_COL     1     		// Data col # for wide format
	#define ROW_CHARS	 6			// # characters in row # column

	// Local variables
	// Chars
	// Initial filenames for use in case of abort
	// Note: Memory is allocated for these strings
	static char *szInitialFile;
	static char *szInitialFileTitle;
	static char *szInitialPhaseFile;
	static char *szInitialPhaseFileTitle;
	static char *szInitialFreqFile;
	static char *szInitialFreqFileTitle;
	char szBuffer[BUFFER_SIZE+1];	// Text buffer

	// Misc
	DTCOLORSTRUCT SelColor;			// Color structure for selected cell
	DTCOLORSTRUCT GapColor;     	// Color structure for gap cells
	DTCOLORSTRUCT OutColor;    	 	// Color structure for outlier cells
	DTCOLORSTRUCT NormColor;    	// Color structure for normal cells
	DWORD dwExtWndStyle;	        // Data table extended window style
	static HWND hDataTbl;	        // Handle to data table
	LPVMSTRUCT lpData;				// Pointer to VMSTRUCT
	PENSTRUCT Pen;					// Pen structure
	COLSTYLE ColStyle;				// Column style
	// Ints
	int i;							// Index
	int nEntry;						// Entry to be validated
	static int nRows; 		   	    // # rows of data
									// NOT # visible table rows
	static int nCols=1;				// # table columns
	static int nColNum=0;			// Column #
	static int nRowNum;			   	// Row #
	static int nGapRow;			    // Gap row #
	static int nMatchRow;			// Find match row #
	static int nInsertRow;		    // # where row inserted
	static int nDeleteRow;          // # where row deleted
	static int nGaps;           	// # gaps to insert
	static int nTopRow;         	// Row # at top of visible table
	static int nNumVisRows;     	// # visible rows
	static int nSaveNum;        	// Original # data points
	static int nSaveStart;      	// Original analysis start
	static int nSaveEnd;        	// Original analysis end
	static int nChanges;			// # editing changes made
	// BOOLs
	static BOOL bNext;	  			// Search direction flag
	static BOOL bReplace=FALSE; 	// Replace data flag
	static BOOL bDoReplace=FALSE;   // Do replacement flag
	static BOOL bColorGaps=TRUE;    // Flag to color gaps
	static BOOL bColorOuts=TRUE;    // Flag to color outliers
	static BOOL	bNewFreqData=TRUE;  // Local flag that freq data changed
	static BOOL bNewPhaseData=TRUE; // Local flag that phase data changed
	static BOOL bErrorTrap=FALSE;	// Flag for entry error
	static BOOL bTags; 				// Timetags flag (phase or freq)
	static BOOL bAutoInsert=FALSE;  // Autoinsert flag
	static BOOL bEditingDone=FALSE;	// Flag to indicate that data was edited
	// These flags made TRUE so med abs dev is calc 1st time
	// Not used for phase data
	static BOOL bInitialFreqChange; // State of freq change flag at entry
	static BOOL bInitialPhaseChange;// State of phase change flag at entry
	static BOOL bInitialFreqScale;  // State of freq scale flag at entry
	static BOOL bInitialPhaseScale; // State of phase scale flag at entry
	// F_TYPESs
	F_TYPE	fVal;					// Misc value
	static F_TYPE fTagInterval;		// Timetag interval
	static F_TYPE *fEditData;   	// Pointer to temporary data array
	static F_TYPE *fEditTags;   	// Pointer to temporary timetag array
	static F_TYPE *fTemp;			// Temporary data pointer
	static F_TYPE fMed;				// Median value
	static F_TYPE fUpperLimit;  	// Upper outlier limit
	static F_TYPE fLowerLimit;  	// Lower outlier limit
	// Floats
	float fSigFactor;				// Sigma factor for outlier limits
	static float fMedAbsDev;		// Median absolute deviation

	// Start of code
	// To avoid complier warning
	lParam=lParam;

	// Set selected color parameters
	SelColor.BkgrdColor=GetSysColor(COLOR_HIGHLIGHT);
	SelColor.TextColor=GetSysColor(COLOR_HIGHLIGHTTEXT);
	SelColor.SelBkgrdColor=DEFAULTCOLOR;
	SelColor.SelTextColor=DEFAULTCOLOR;

	// Set gap color parameters
	GapColor.BkgrdColor=DEFAULTCOLOR;
	GapColor.TextColor=RGB(0,0,255); // Blue
	GapColor.SelBkgrdColor=DEFAULTCOLOR;
	GapColor.SelTextColor=DEFAULTCOLOR;

	// Set outlier color parameters
	OutColor.BkgrdColor=DEFAULTCOLOR;
	OutColor.TextColor=0x000000FF; // Red
	OutColor.SelBkgrdColor=DEFAULTCOLOR;
	OutColor.SelTextColor=DEFAULTCOLOR;

	// Set normal color parameters
	NormColor.BkgrdColor=DEFAULTCOLOR;
	NormColor.TextColor=DEFAULTCOLOR;
	NormColor.SelBkgrdColor=DEFAULTCOLOR;
	NormColor.SelTextColor=DEFAULTCOLOR;

	// Set pen parameters
	Pen.fPenStyle=PS_SOLID;
	Pen.nWidth=1;
	Pen.clr=GetSysColor(COLOR_HIGHLIGHT); // Makes black

	switch(message)
	{
		case WM_INITDIALOG:
		{
			// Zero changes count
			nChanges=0;

			// Initialize data pointers
			if(bDataType) // Freq data
			{
				fData=fFreqData;
				fTags=fFreqTimetags;
			}
			else // Phase data
			{
				fData=fPhaseData;
				fTags=fPhaseTimetags;
			}

			// Initialize editing done flag
			bEditingDone=FALSE;

			// Disable autoinsert
			ShowWindow(GETHCTL(IDC_EDIT_AUTO), SW_HIDE);
			EnableWindow(GETHCTL(IDC_EDIT_AUTO), FALSE);

			// Disable edit mode
			ShowWindow(GETHCTL(IDC_EDIT_MODE), SW_HIDE);
			EnableWindow(GETHCTL(IDC_EDIT_MODE), FALSE);

			// Allocate memory for filenames
			if((szInitialFile=(LPSTR)malloc((FILE_NAME_SIZE)*
				sizeof(char)))==NULL)
			{
				// Display error msg & quit
				AllocErrorMessage();
				EndDialog (hDlg, 0);
				return TRUE;
			}
			if((szInitialFileTitle=(LPSTR)malloc((FILE_TITLE_SIZE)*
				sizeof(char)))==NULL)
			{
				// Display error msg, free memory & quit
				AllocErrorMessage();
				free(szInitialFile);
				EndDialog (hDlg, 0);
				return TRUE;
			}
			if((szInitialPhaseFile=(LPSTR)malloc((FILE_NAME_SIZE)*
				sizeof(char)))==NULL)
			{
				// Display error msg, free memory & quit
				AllocErrorMessage();
				free(szInitialFile);
				free(szInitialFileTitle);
				EndDialog (hDlg, 0);
				return TRUE;
			}
			if((szInitialPhaseFileTitle=(LPSTR)malloc((FILE_TITLE_SIZE)*
				sizeof(char)))==NULL)
			{
				// Display error msg, free memory & quit
				AllocErrorMessage();
				free(szInitialFile);
				free(szInitialFileTitle);
				free(szInitialPhaseFile);
				EndDialog (hDlg, 0);
				return TRUE;
			}
			if((szInitialFreqFile=(LPSTR)malloc((FILE_NAME_SIZE)*
				sizeof(char)))==NULL)
			{
				// Display error msg, free memory & quit
				AllocErrorMessage();
				free(szInitialFile);
				free(szInitialFileTitle);
				free(szInitialPhaseFile);
				free(szInitialPhaseFileTitle);
				EndDialog (hDlg, 0);
				return TRUE;
			}
			if((szInitialFreqFileTitle=(LPSTR)malloc((FILE_TITLE_SIZE)*
				sizeof(char)))==NULL)
			{
				// Display error msg, free memory & quit
				AllocErrorMessage();
				free(szInitialFile);
				free(szInitialFileTitle);
				free(szInitialPhaseFile);
				free(szInitialPhaseFileTitle);
				free(szInitialFreqFile);
				EndDialog (hDlg, 0);
				return TRUE;
			}

			// Allocate working memory for data
			if((fEditData=
				(F_TYPE*)malloc(((int)fData[0]+ARRAY_OFFSET+EXTRA_MEM)*
				sizeof(F_TYPE)))==NULL)
			{
				// Display error msg, free memory & quit
				AllocErrorMessage();
				free(szInitialFile);
				free(szInitialFileTitle);
				free(szInitialPhaseFile);
				free(szInitialPhaseFileTitle);
				free(szInitialFreqFile);
				free(szInitialFreqFileTitle);
				EndDialog (hDlg, 0);
				return TRUE;
			}

			// Copy complete data array to working memory
			for(i=0; i<fData[0]+ARRAY_OFFSET; i++)
			{
				fEditData[i]=fData[i];
			}

			// Set the global data pointer to the temporary array
			// This will cause the related fetch, find and replace functions
			// to also use the temporary array
			// The global data pointer will be set back to the permanant
			// array when the Edit dialog box closes
			fData=fEditData;

			// Save initial state of global data change flag
			// Save the original # data points, analysis start & analysis end
			if(bDataType==FREQ_DATA)
			{
				GetWindowText(hwndFreqNum, szBuffer, BUFFER_SIZE);
				nSaveNum=atoi(szBuffer);
				GetWindowText(hwndFreqStart, szBuffer, BUFFER_SIZE);
				nSaveStart=atoi(szBuffer);
				GetWindowText(hwndFreqEnd, szBuffer, BUFFER_SIZE);
				nSaveEnd=atoi(szBuffer);
				bInitialFreqChange=bFreqDataChanged;
				bInitialFreqScale=bFreqScaleChanged;

				bTags=bFreqTimetags;

				// Save filenames
				strlcpy(szInitialFile, szFile, FILE_TITLE_SIZE);
				strlcpy(szInitialFileTitle, szFileTitle, FILE_TITLE_SIZE);
				strlcpy(szInitialFreqFile, szFreqFile, FILE_TITLE_SIZE);
				strlcpy(szInitialFreqFileTitle, szFreqFileTitle,
					FILE_TITLE_SIZE);
			}
			else // Phase data
			{
				GetWindowText(hwndPhaseNum, szBuffer,BUFFER_SIZE);
				nSaveNum=atoi(szBuffer);
				GetWindowText(hwndPhaseStart, szBuffer, BUFFER_SIZE);
				nSaveStart=atoi(szBuffer);
				GetWindowText(hwndPhaseEnd, szBuffer, BUFFER_SIZE);
				nSaveEnd=atoi(szBuffer);
				bInitialPhaseChange=bPhaseDataChanged;
				bInitialPhaseScale=bPhaseScaleChanged;

				bTags=bPhaseTimetags;

				// Save filenames
				strlcpy(szInitialFile, szFile, FILE_TITLE_SIZE);
				strlcpy(szInitialFileTitle, szFileTitle, FILE_TITLE_SIZE);
				strlcpy(szInitialPhaseFile, szPhaseFile, FILE_TITLE_SIZE);
				strlcpy(szInitialPhaseFileTitle, szPhaseFileTitle,
					FILE_TITLE_SIZE);
			}

			if(bTags)
			{
				// Allocate working memory for timetags
				if((fEditTags=(F_TYPE*)malloc(
					((int)fTags[0]+ARRAY_OFFSET+EXTRA_MEM)*sizeof(F_TYPE)))==
					NULL)
				{
					// Display error msg, free memory & quit
					AllocErrorMessage();
					free(szInitialFile);
					free(szInitialFileTitle);
					free(szInitialPhaseFile);
					free(szInitialPhaseFileTitle);
					free(szInitialFreqFile);
					free(szInitialFreqFileTitle);
					free(fEditData);
					EndDialog (hDlg, 0);
					return TRUE;
				}

				// Copy complete timetag array to working memory
				for(i=0; i<fTags[0]+ARRAY_OFFSET; i++)
				{
					fEditTags[i]=fTags[i];
				}

				// Set the global data pointer to the temporary array
				// This will cause the related fetch, find and replace
				// functions to also use the temporary array
				// The global data pointer will be set back to the permanant
				// array when the Edit dialog box closes
				fTags=fEditTags;
			}

			// Get handle for data table
			hDataTbl=GETHCTL(IDC_EDIT_TABLE);

			if(hDataTbl==0)
			{
				MessageBox(hwndMain, "hDataTbl==0", "Error", MB_OK);
			}

			// Allow global access to data table's handle
			hDialog=hDataTbl;

			// Initialize the controls
			// Go To point #
			SetWindowText(GETHCTL(IDC_EDIT_POINTNUM), "1");
			// Sigma factor
			SetWindowText(GETHCTL(IDC_EDIT_SIGMAFACTOR), "5.000");
			// # Gaps to insert
			SetWindowText(GETHCTL(IDC_EDIT_NUMGAPS), "0");

			// Restore previous Color Outliers state
			Button_SetCheck(GETHCTL(IDC_EDIT_COLOROUTS), bColorOuts);

			// Disable the outlier control and color outlier checkbox
			// Also clear and disable sigma factor edit control for phase data
			// And disable sigma factor label too
			if(bDataType==PHASE_DATA)
			{
				EnableWindow(GETHCTL(IDC_EDIT_OUTLIER), FALSE);
				EnableWindow(GETHCTL(IDC_EDIT_COLOROUTS), FALSE);
				Button_SetCheck(GETHCTL(IDC_EDIT_COLOROUTS), FALSE);
				SetWindowText(GETHCTL(IDC_EDIT_SIGMAFACTOR), "");
				EnableWindow(GETHCTL(IDC_EDIT_SIGMAFACTOR), FALSE);
				EnableWindow(GETHCTL(IDC_EDIT_SFLABEL), FALSE);
			}

			// Set search direction to Next
			bNext=TRUE;
			Button_SetCheck(GETHCTL(IDC_EDIT_NEXT), TRUE);
			Button_SetCheck(GETHCTL(IDC_EDIT_PREV), FALSE);

			// Restore previous Color Gaps state
			Button_SetCheck(GETHCTL(IDC_EDIT_COLORGAPS), bColorGaps);

			// Restore previous Autoinsert state
			Button_SetCheck(GETHCTL(IDC_EDIT_AUTO), bAutoInsert);

			// Restore previous Show Timetags state
			Button_SetCheck(GETHCTL(IDC_EDIT_TIMETAG), bWideEdit);

			// Check for valid timetag data
			if(!bTags)
			{
				EnableWindow(GETHCTL(IDC_EDIT_TIMETAG), FALSE);
				Button_SetCheck(GETHCTL(IDC_EDIT_TIMETAG), FALSE);
			}

			// Initialize the timetag format controls
			switch(nTagFormat)
			{
				case TAGFORMAT_MJD:
				{
					// Initialize the format radio buttons to MJD
					Button_SetCheck(GETHCTL(IDC_EDIT_MJD), TRUE);
					Button_SetCheck(GETHCTL(IDC_EDIT_DOY), FALSE);
					Button_SetCheck(GETHCTL(IDC_EDIT_DATE), FALSE);
					Button_SetCheck(GETHCTL(IDC_EDIT_EXP), FALSE);
				}
				break;

				case TAGFORMAT_DOY:
				{
					// Initialize the format radio buttons to DOY
					Button_SetCheck(GETHCTL(IDC_EDIT_MJD), FALSE);
					Button_SetCheck(GETHCTL(IDC_EDIT_DOY), TRUE);
					Button_SetCheck(GETHCTL(IDC_EDIT_DATE), FALSE);
					Button_SetCheck(GETHCTL(IDC_EDIT_EXP), FALSE);
				}
				break;

				case TAGFORMAT_DATE:
				{
					// Initialize the format radio buttons to Date
					Button_SetCheck(GETHCTL(IDC_EDIT_MJD), FALSE);
					Button_SetCheck(GETHCTL(IDC_EDIT_DOY), FALSE);
					Button_SetCheck(GETHCTL(IDC_EDIT_DATE), TRUE);
					Button_SetCheck(GETHCTL(IDC_EDIT_EXP), FALSE);
				}
				break;

				case TAGFORMAT_EXP:
				{
					// Initialize the format radio buttons to Exp
					Button_SetCheck(GETHCTL(IDC_EDIT_MJD), FALSE);
					Button_SetCheck(GETHCTL(IDC_EDIT_DOY), FALSE);
					Button_SetCheck(GETHCTL(IDC_EDIT_DATE), FALSE);
					Button_SetCheck(GETHCTL(IDC_EDIT_EXP), TRUE);
				}
				break;
			}

			// Set width of row # (left) column
			// Column width is in dialog units (4/char)
			// #  row number digit is ROW_CHARS
			SendMessage(hDataTbl, DTM_SETCOLWIDTH, (WPARAM)DTPOS_LABELS,
				(LPARAM)(4*ROW_CHARS));

			if(bWideEdit)
			{
				// Set # columns to 2
				nCols=2;

				// Set column # to data column
				nColNum=DATA_COL;

				// Create two columns
				SendMessage(hDataTbl, DTM_SETCOLCOUNT, (WPARAM)nCols,
					(LPARAM)0);

				// Configure the data column
				SendMessage(hDataTbl, DTM_SETCOLTYPE, (WPARAM)DATA_COL,
					(LPARAM)DTT_ZSTRING);

				// Configure the timetag column
				SendMessage(hDataTbl, DTM_SETCOLTYPE, (WPARAM)TIMETAG_COL,
					(LPARAM)DTT_ZSTRING);
			}
			else
			{
				// Set # columns to 1
				nCols=1;

				// Set column # to data column
				nColNum=ONE_COL;

				// Create one column
				SendMessage(hDataTbl, DTM_SETCOLCOUNT, (WPARAM)nCols,
					(LPARAM)0);

				// Configure the one (data) column
				SendMessage(hDataTbl, DTM_SETCOLTYPE, (WPARAM)ONE_COL,
					(LPARAM)DTT_ZSTRING);
			}

			// Set default data string
			_snprintf(szBuffer, sizeof(szBuffer), "%+20.13e", 0.0);
			ShortenExp(szBuffer);
			SendMessage(hDataTbl, DTM_SETCOLDEFVAL, (WPARAM)(COLNUM)nColNum,
				(LPARAM)(LPSTR)szBuffer);

			// So we will use 14-digit exponential format like STABLE/DOS
			// This requires 20 characters + 1 for EOS
			SendMessage(hDataTbl, DTM_SETCOLDATALEN, (WPARAM)nColNum,
				(LPARAM)(NUM_CHARS+1));

			// Column width is in dialog units (4/char)
			// So need 20*4=80
			SendMessage(hDataTbl, DTM_SETCOLWIDTH, (WPARAM)nColNum,
				(LPARAM)(4*NUM_CHARS));

			// label the column heading with the data type
			if(bDataType)
			{
				SendMessage(hDataTbl, DTM_SETCOLHEAD, (WPARAM)nColNum,
					(LPARAM)(LPSTR)"Frequency");

				// Set new freq data flag
				bNewFreqData=TRUE;

				// Recalc fMed & fMedAbsDev
				// Note:  This takes long time during startup with
				// large freq data set because it calculates
				// median absolute deviation
				SendMessage(hDlg, WM_COMMAND,
					GET_WM_COMMAND_MPS(IDC_EDIT_TABLE, hDataTbl, DTN_COLOR));
			}
			else
			{
				SendMessage(hDataTbl, DTM_SETCOLHEAD, nColNum,
					(LPARAM) (LPSTR) "Phase");
			}

			// Center column heading, make data editable, and make column
			// width fixed
			SendMessage(hDataTbl, DTM_SETCOLSTYLE, nColNum,
				(LPARAM) DTCS_HEADCENTER | DTCS_EDIT | DTCS_FIXED);

			if(bWideEdit)
			{
				// Set default timetag string to blank
				SendMessage(hDataTbl, DTM_SETCOLDEFVAL, (COLNUM)TIMETAG_COL,
					(LPARAM)(LPSTR)"");

				// Timetag will use 17.8 decimal format = XXXXXXXX.XXXXXXXX
				// or equivalent in HMS or EXP format
				// This requires 17 characters (no EOS)
				SendMessage(hDataTbl, DTM_SETCOLDATALEN, TIMETAG_COL,
					(LPARAM) TAG_CHARS);

				// Column width is in dialog units (4/char)
				// So need 11*4=44
				SendMessage(hDataTbl, DTM_SETCOLWIDTH, TIMETAG_COL,
					(LPARAM) 4*TAG_CHARS);

				// label the timetag column heading
				SendMessage(hDataTbl, DTM_SETCOLHEAD, TIMETAG_COL,
					(LPARAM) (LPSTR) "Timetag");

				// Center column heading, don't make data editable, and make
				// column width fixed
				SendMessage(hDataTbl, DTM_SETCOLSTYLE, TIMETAG_COL,
					(LPARAM) DTCS_HEADCENTER | DTCS_FIXED);

				// Lock the timetag column
				ColStyle=(COLSTYLE)SendMessage(hDataTbl, DTM_GETCOLSTYLE,
					TIMETAG_COL, 0L);
				ColStyle|=DTCS_LOCKED;
				SendMessage(hDataTbl, DTM_SETCOLSTYLE, TIMETAG_COL,
					(LPARAM) ColStyle);
			}

			// Set the extended window style to insert & delete rows
			// To always show vertical scrollbar , add DTES_SHOWVSCRL
			// Note: Don't use DTES_EEUPARROWS with DTES_AUTOINSERT
			// because ending an edit with a down arrow at bottom of table
			// will move into forbidden region w/o autoinserting
			// Autoinserting is good for new entry, but probably bad for
			// normal editing of existing data.  So we have a checkbox to
			// control autoinserting in Edit.  Might want a separate New
			// dialog procedure.
			SendMessage(hDataTbl, DTM_SETEXTWNDSTYLE, 0,
				(LPARAM) DTES_DELKEY | DTES_INSKEY);

			// Set color for currently selected cell
			SendMessage(hDataTbl, DTM_SETCURCELLCOLOR, 0,
				(LPARAM) (LPDTCOLORSTRUCT)&SelColor);

			// Change pen used for focus rectangle for better visibility
			SendMessage(hDataTbl, DTM_SETFOCUSPEN, 0,
				(LPARAM) (LPPENSTRUCT)&Pen);

			// Get # rows=size of data array
			nRows=(int)fData[0];

			// Initiate communications with data table
			SendMessage(hDataTbl, DTM_VM_INITIALIZE, FALSE, (LPARAM) nRows);

		return TRUE;
		}

		// DataTable virtual memory messages
		// Get the first row of data
		case DTM_VM_GETFIRST:
		{
			// Get pointer to VMSTRUCT
			lpData=(LPVMSTRUCT)lParam;

			// Set index to begining
			lpData->dwKey=0;

			// Get row data
			lpData->nStatus=(short)GetRowData(lpData, nRows);
		}
		break;

		// Get the last row of data
		case DTM_VM_GETLAST:
		{
			// Get pointer to VMSTRUCT
			lpData=(LPVMSTRUCT)lParam;

			// Set index to last row
			lpData->dwKey=nRows-1;	// Always OK

			// Get row data
			lpData->nStatus=(short)GetRowData(lpData, nRows);
		}
		break;

		// Get the next row of data (data after row=dwKey)
		case DTM_VM_GETNEXT:
		{
			// Get pointer to VMSTRUCT
			lpData=(LPVMSTRUCT)lParam;

			// Increment index
			lpData->dwKey+=1;

			// Get row data
			lpData->nStatus=(short)GetRowData(lpData, nRows);
		}
		break;

		// Get the previous row of data (data before row=dwKey)
		case DTM_VM_GETPREV:
		{
			// Get pointer to VMSTRUCT
			lpData=(LPVMSTRUCT)lParam;

			// Decrement index
			lpData->dwKey-=1;

			// Get row data
			lpData->nStatus=(short)GetRowData(lpData, nRows);
		}
		break;

		// Get the row of data that is specified % down table
		case DTM_VM_GETPERCENT:
		{
			// Get pointer to VMSTRUCT
			lpData=(LPVMSTRUCT)lParam;

			// Calc index
			lpData->dwKey=(lpData->dwKey)*(long)nRows/100L;

			// Get row data
			lpData->nStatus=(short)GetRowData(lpData, nRows);
		}
		break;

		// Get the row of data at dwKey
		case DTM_VM_GETROWDATA:
		{
			// Get pointer to VMSTRUCT
			lpData=(LPVMSTRUCT)lParam;

			// Get row data
			lpData->nStatus=(short)GetRowData(lpData, nRows);
		}
		break;

		case WM_COMMAND:
		{
			switch(GET_WM_COMMAND_ID(wParam, lParam))
			{
				case IDC_EDIT_TABLE:
				{
					switch(GET_WM_COMMAND_CMD(wParam, lParam))
					{
						// Processed DTN_s
						case DTN_ROWCHANGE:
						{
							// Process status plot cursor if plots enabled
							if(bStatusPlot)
							{
								// Get new row # (table index starts at 0)
								nRowNum=(int)SendMessage(hDataTbl,
									DTM_GETNOTIFYROW, (WPARAM)0, (LPARAM)0)+1;

								// At this point, nRowNum is same as point #
								// as shown in 1st column of table, which has
								// the same range as fData[0], etc.
								// The left edge of the cursor is going to be
								// positioned at left side of the point's time
								// interval.  So the numerator in the
								// nPosition calc starts at 0 for the lst
								// displayed point and the denominator is equal
								// to the # of plotted points.

								// Show plot cursor if in range of status plot
								if( (nRowNum>=(int)fData[1]) &&
									(nRowNum<=(int)fData[2]) )
								{
									DisplayPlotCursor((nRowNum-(int)fData[1])*
										1000/(int)(fData[2]-fData[1]+1), TRUE);
								}
								else // Erase cursor
								{
									DisplayPlotCursor(0, FALSE);
								}

								if(nRowNum<(int)fData[1]) // Offscale low
								{
									DisplayPlotCursor(-10, TRUE);
								}

								if(nRowNum>(int)fData[2]) // Offscale high
								{
									DisplayPlotCursor(1001, TRUE);
								}
							}
						}
						break;

						case DTN_BEFOREABORTEDIT:
						case DTN_BEFOREENDEDIT:
						{
							// Disable edit mode
							ShowWindow(GETHCTL(IDC_EDIT_MODE), SW_HIDE);
							EnableWindow(GETHCTL(IDC_EDIT_MODE), FALSE);
						}
						break;

						case DTN_BEFOREBEGINEDIT:
						{
							// Enable edit mode
							ShowWindow(GETHCTL(IDC_EDIT_MODE), SW_SHOWNORMAL);
							EnableWindow(GETHCTL(IDC_EDIT_MODE), TRUE);
						}
						break;

						// Detect column change & force position to DATA_COL
						case DTN_COLCHANGE:
						{
							nRowNum=(int)SendMessage(hDataTbl, DTM_GETCURROW,
								(WPARAM)0, (LPARAM)0);
							SendMessage(hDataTbl, DTM_SETCURCELL,
								(WPARAM)DATA_COL, (LPARAM)nRowNum);
						}
						break;

						// Notification before a row is deleted from table
						case DTN_BEFOREDELETEROW:
						{
							// Save previous data in tab
							if(nChanges==0)
							{
								if(bTabs && bTabEdit)
								{
									CreateTab();
								}
							}

							// Get row # of delete
							nDeleteRow=(int)SendMessage(hDataTbl,
								DTM_GETNOTIFYROW, (WPARAM)0, (LPARAM)0);

							// Update status display and array pointers
							// Note: For delete, move data 1st, then realloc
							_snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
								nRows-1);

							if(bDataType==FREQ_DATA)
							{
								// The #: status field is always decremented.
								SetWindowText(hwndFreqNum, szBuffer);

								// The Start: status field will be decremented
								// if it is beyond the deletion point.
								// and it is >1
								GetWindowText(hwndFreqStart, szBuffer,
									BUFFER_SIZE);
								nRowNum=atoi(szBuffer);

								if(( nRowNum>nDeleteRow) && (nRowNum>1) )
								{
									nRowNum--;
									_snprintf(szBuffer, sizeof(szBuffer),
										"%5.0d", nRowNum);
									SetWindowText(hwndFreqStart, szBuffer);
									fData[1]--;

									if(bTags)
									{
										fTags[1]--;
									}
								}

								// The End: status field will be decremented
								// if it is beyond the deletion point.
								GetWindowText(hwndFreqEnd, szBuffer,
									BUFFER_SIZE);
								nRowNum=atoi(szBuffer);
								if(nRowNum>nDeleteRow)
								{
									nRowNum--;
									_snprintf(szBuffer, sizeof(szBuffer),
										"%5.0d", nRowNum);
									SetWindowText(hwndFreqEnd, szBuffer);
									fData[2]--;

									if(bTags)
									{
										fTags[2]--;
									}
								}

								// Move original data up from delete point
								// Note:  Row index starts at 0
								// Data index starts at ARRAY_OFFSET
								for(i=nDeleteRow; i<nRows-1; i++)
								{
									fData[i+ARRAY_OFFSET]=
										fData[i+ARRAY_OFFSET+1];

									if(bTags)
									{
										fTags[i+ARRAY_OFFSET]=
											fTags[i+ARRAY_OFFSET+1];
									}
								}

								// Set change flags
								bNewFreqData=TRUE;
								bFreqDataChanged=TRUE;
								bFreqDataUnsaved=TRUE;
								sFPlot.new_file=TRUE;
								bFreqScaleChanged=TRUE;
								bSynch=FALSE;
								bEditingDone=TRUE;

								nChanges++;

								// Force new PSD plot scale
								sPSPlot.y_tic=0;

								// Increment filename
								if(bIncExtension)
								{
									IncExtension(szFile, FREQ_DATA);
									IncExtension(szFileTitle, FREQ_DATA);
									IncExtension(szFreqFile, FREQ_DATA);
									IncExtension(szFreqFileTitle, FREQ_DATA);
									SetWindowText(hwndFreqFilename,
										szFileTitle);
								}
							}
							else // Phase data
							{
								// The #: status field is always decremented.
								SetWindowText(hwndPhaseNum, szBuffer);

								// The Start: status field will be decremented
								// if it is beyond the deletion point
								// and it is >1
								GetWindowText(hwndPhaseStart, szBuffer,
									BUFFER_SIZE);
								nRowNum=atoi(szBuffer);

								if(( nRowNum>nDeleteRow) && (nRowNum>1) )
								{
									nRowNum--;
									_snprintf(szBuffer, sizeof(szBuffer),
										"%5.0d", nRowNum);
									SetWindowText(hwndPhaseStart, szBuffer);
									fData[1]--;

									if(bTags)
									{
										fTags[1]--;
									}
								}

								// The End: status field will be decremented
								// if it is beyond the deletion point.
								GetWindowText(hwndPhaseEnd, szBuffer,
									BUFFER_SIZE);
								nRowNum=atoi(szBuffer);
								if(nRowNum>nDeleteRow)
								{
									nRowNum--;
									_snprintf(szBuffer, sizeof(szBuffer),
										"%5.0d", nRowNum);
									SetWindowText(hwndPhaseEnd, szBuffer);
									fData[2]--;

									if(bTags)
									{
										fTags[2]--;
									}
								}

								// Move original data up from delete point
								// Note:  Row index starts at 0
								// Data index starts at ARRAY_OFFSET
								for(i=nDeleteRow; i<nRows-1; i++)
								{
									fData[i+ARRAY_OFFSET]=
										fData[i+ARRAY_OFFSET+1];

									if(bTags)
									{
										fTags[i+ARRAY_OFFSET]=
											fTags[i+ARRAY_OFFSET+1];
									}
								}

								// Set change flags
								bNewPhaseData=TRUE;
								bPhaseDataChanged=TRUE;
								bPhaseDataUnsaved=TRUE;
								bPhaseScaleChanged=TRUE;
								bSynch=FALSE;
								bEditingDone=TRUE;

								nChanges++;

								// Force new PSD plot scale
								sPSPlot.y_tic=0;

								// Increment filename
								if(bIncExtension)
								{
									IncExtension(szFile, PHASE_DATA);
									IncExtension(szFileTitle, PHASE_DATA);
									IncExtension(szPhaseFile, PHASE_DATA);
									IncExtension(szPhaseFileTitle, PHASE_DATA);
									SetWindowText(hwndPhaseFilename,
										szFileTitle);
								}
							}

							// Clear plot lines
							ClearLines();

							// Decrement # rows and data points
							nRows--;
							fData[0]--;

							if(bTags)
							{
								fTags[0]--;
							}

							// Reallocate memory
							// Remove one point from active data array
							if((fNew=(F_TYPE*)realloc((char *)fData,
								(nRows+ARRAY_OFFSET+EXTRA_MEM)*
								sizeof(F_TYPE)))==NULL)
							{
								// This should not ever fail
								// and even if it does it should not
								// cause a real problem, so just display
								// error msg w/o bailing out
								AllocErrorMessage();
							}
							else
							{
								fData=fNew;
							}

							if(bTags)
							{
								// Remove one point from active tag array
								if((fNew=(F_TYPE*)realloc((char *)
									fTags, (nRows+ARRAY_OFFSET+EXTRA_MEM)*
									sizeof(F_TYPE)))==NULL)
								{
									// This should not ever fail
									// and even if it does it should not
									// cause a real problem, so just display
									// error msg w/o bailing out
									AllocErrorMessage();
								}
								else
								{
									fTags=fNew;
								}
							}
							// Recolor gaps and outliers
							SendMessage(hDlg, WM_COMMAND,
								GET_WM_COMMAND_MPS(IDC_EDIT_TABLE, hDataTbl,
								DTN_COLOR));

							// Redraw status plots
							// SendMessage(hDlg, WM_COMMAND,
							// GET_WM_COMMAND_MPS(IDC_EDIT_REDRAW, 0, 0));
						}
						break;

						// Notification before a new row is inserted into table
						// Note: The insert operation inserts a 0 (gap) as
						// its default.  After inserting the 0, the highlight
						// remains there so it can be edited to another value.
						// A group of gaps can be inserted simply by holding
						// down the Ins key.

						// Note:  This code works with "blank" array (1 dummy
						// point), but it isn't worth much as a way to enter
						// data manually.  So the edit function is now dis-
						// abled until data is read from disk.

						case DTN_BEFOREINSERTROW:
						{
							// Save previous data in tab
							if(nChanges==0)
							{
								if(bTabs && bTabEdit)
								{
									CreateTab();
								}
							}

							// Check for too many points
							if(nRows>nMaxSize-1)
							{
								LoadString(hInst, IDS_MESSAGE_TOOMANY,
									szBuffer, BUFFER_SIZE);
								MessageBox(hwndMain, szBuffer, "Edit", MB_OK);

								// Abort row insert
								SendMessage(hDataTbl, DTM_NOTIFYRESULTS, 0,
									1L);
								break;
							}

							// Get row # of insert
							nInsertRow=(int)SendMessage(hDataTbl,
								DTM_GETNOTIFYROW, 0, 0L);

							// Increment # data points
							nRows++;
							fData[0]++;

							if(bTags)
							{
								fTags[0]++;
							}

							// Reallocate memory
							// Note: For delete, realloc 1st, then move data
							// Add one point to active data array

							if((fNew=(F_TYPE*)realloc((char *)fData,
								(nRows+ARRAY_OFFSET+EXTRA_MEM)*
								sizeof(F_TYPE)))==NULL)
							{
								// Dsp err msg, cancel edits & close dlg box
								AllocErrorMessage();
								PostMessage(hDlg, WM_COMMAND,
									GET_WM_COMMAND_MPS(IDCANCEL, NULL, 0));
								break;
							}
							else
							{
								fData=fNew;
							}

							if(bTags)
							{
								// Add one point to active timetag array
								if((fNew=(F_TYPE*)realloc((char *)
									fTags, (nRows+ARRAY_OFFSET+EXTRA_MEM)*
									sizeof(F_TYPE)))==NULL)
								{
									// Display error message, cancel edits
									// and close dialog box
									AllocErrorMessage();
									PostMessage(hDlg, WM_COMMAND,
										GET_WM_COMMAND_MPS(IDCANCEL, NULL, 0));
									break;
								}
								else
								{
									fTags=fNew;
								}
							}

							// Update status display and array pointers
							_snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
								nRows);

							if(bDataType==FREQ_DATA)
							{
								// The #: status field is always incremented.
								SetWindowText(hwndFreqNum, szBuffer);

								// The Start: status field will be incremented
								// if it is beyond the insertion point.
								GetWindowText(hwndFreqStart, szBuffer,
									BUFFER_SIZE);
								nRowNum=atoi(szBuffer);
								if(nRowNum>nInsertRow)
								{
									nRowNum++;
									_snprintf(szBuffer, sizeof(szBuffer),
										"%5.0d", nRowNum);
									SetWindowText(hwndFreqStart, szBuffer);
									fData[1]=nRowNum;

									if(bTags)
									{
										fTags[1]=nRowNum;
									}
								}

								// The End: status field will be incremented
								// if it is beyond the insertion point.
								GetWindowText(hwndFreqEnd, szBuffer,
									BUFFER_SIZE);
								nRowNum=atoi(szBuffer);
								if(nRowNum>nInsertRow)
								{
									nRowNum++;
									_snprintf(szBuffer, sizeof(szBuffer),
										"%5.0d", nRowNum);
									SetWindowText(hwndFreqEnd, szBuffer);
									fData[2]=nRowNum;

									if(bTags)
									{
										fTags[2]=nRowNum;
									}
								}

								// Move original data down from insert point
								// Note:  Row index starts at 0
								// Data index starts at ARRAY_OFFSET
								for(i=nRows-1; i>nInsertRow; i--)
								{
									fData[i+ARRAY_OFFSET]=
										fData[i+ARRAY_OFFSET-1];

									if(bTags)
									{
										fTags[i+ARRAY_OFFSET]=
											fTags[i+ARRAY_OFFSET-1];
									}
								}

								// Is this new data?
								if(bFreqData==FALSE)
								{
									// Assign generic filenames
									strlcpy(szFile, "NEW.DAT", FILE_NAME_SIZE);
									strlcpy(szFileTitle, "NEW.DAT",
										FILE_TITLE_SIZE);
									strlcpy(szFreqFile, "NEW.DAT",
										FILE_NAME_SIZE);
									strlcpy(szFreqFileTitle, "NEW.DAT",
										FILE_TITLE_SIZE);

									// Set data flag
									bFreqData=TRUE;

									// Activate data-dependent functions
									ActivateFunctions(MF_ENABLED);

									// Update status display
									SetWindowText(hwndFreqFilename,
										szFileTitle);
									_snprintf(szBuffer, sizeof(szBuffer),
										"%5.0d", nRows);
									SetWindowText(hwndFreqNum, szBuffer);
									SetWindowText(hwndFreqEnd, szBuffer);
									_snprintf(szBuffer, sizeof(szBuffer),
										"%5.0d", 1);
									SetWindowText(hwndFreqStart, szBuffer);
									_snprintf(szBuffer, sizeof(szBuffer),
										"%11.5e", 1.0);
									ShortenExp(szBuffer);
									SetWindowText(hwndFreqTau, szBuffer);
								}

								// Set change flags
								bNewFreqData=TRUE;
								bFreqDataChanged=TRUE;
								bFreqDataUnsaved=TRUE;
								sFPlot.new_file=TRUE;
								bFreqScaleChanged=TRUE;
								bSynch=FALSE;
								bEditingDone=TRUE;

								nChanges++;

								// Force new PSD plot scale
								sPSPlot.y_tic=0;

								// Increment filename
								if( (bFreqData) && (bIncExtension) )
								{
									IncExtension(szFile, FREQ_DATA);
									IncExtension(szFileTitle, FREQ_DATA);
									IncExtension(szFreqFile, FREQ_DATA);
									IncExtension(szFreqFileTitle, FREQ_DATA);
									SetWindowText(hwndFreqFilename,
										szFileTitle);
								}
							}
							else // Phase data
							{
								// The #: status field is always incremented.
								SetWindowText(hwndPhaseNum, szBuffer);

								// The Start: status field will be incremented
								// if it is beyond the insertion point.
								GetWindowText(hwndPhaseStart, szBuffer,
									BUFFER_SIZE);
								nRowNum=atoi(szBuffer);
								if(nRowNum>nInsertRow)
								{
									nRowNum++;
									_snprintf(szBuffer, sizeof(szBuffer),
										"%5.0d", nRowNum);
									SetWindowText(hwndPhaseStart, szBuffer);
									fData[1]=nRowNum;

									if(bTags)
									{
										fTags[1]=nRowNum;
									}
								}

								// The End: status field will be incremented
								// if it is beyond the insertion point.
								GetWindowText(hwndPhaseEnd, szBuffer,
									BUFFER_SIZE);
								nRowNum=atoi(szBuffer);
								if(nRowNum>nInsertRow)
								{
									nRowNum++;
									_snprintf(szBuffer, sizeof(szBuffer),
										"%5.0d", nRowNum);
									SetWindowText(hwndPhaseEnd,	szBuffer);
									fData[2]=nRowNum;

									if(bTags)
									{
										fTags[2]=nRowNum;
									}
								}

								// Move original data down from insert point
								// Note:  Row index starts at 0
								// Data index starts at ARRAY_OFFSET
								for(i=nRows-1; i>nInsertRow; i--)
								{
									fData[i+ARRAY_OFFSET]=
										fData[i+ARRAY_OFFSET-1];

									if(bTags)
									{
										fTags[i+ARRAY_OFFSET]=
											fTags[i+ARRAY_OFFSET-1];
									}
								}

								// Is this new data?
								if(bPhaseData==FALSE)
								{
									// Assign generic filenames
									strlcpy(szFile, "NEW.DAT", FILE_NAME_SIZE);
									strlcpy(szFileTitle, "NEW.DAT",
										FILE_TITLE_SIZE);
									strlcpy(szPhaseFile, "NEW.DAT",
										FILE_NAME_SIZE);
									strlcpy(szPhaseFileTitle, "NEW.DAT",
										FILE_TITLE_SIZE);

									// Set data flag
									bPhaseData=TRUE;

									// Activate data-dependent functions
									ActivateFunctions(MF_ENABLED);

									// Update status display
									SetWindowText(hwndPhaseFilename,
										szFileTitle);
									_snprintf(szBuffer, sizeof(szBuffer),
										"%5.0d", nRows);
									SetWindowText(hwndPhaseNum, szBuffer);
									SetWindowText(hwndPhaseEnd, szBuffer);
									_snprintf(szBuffer, sizeof(szBuffer),
										"%5.0d", 1);
									SetWindowText(hwndPhaseStart, szBuffer);
									_snprintf(szBuffer, sizeof(szBuffer),
										"%11.5e", 1.0);
									ShortenExp(szBuffer);
									SetWindowText(hwndPhaseTau, szBuffer);
								}

								// Set change flags
								bNewPhaseData=TRUE;
								bPhaseDataChanged=TRUE;
								bPhaseDataUnsaved=TRUE;
								bPhaseScaleChanged=TRUE;
								bSynch=FALSE;
								bEditingDone=TRUE;

								nChanges++;

								// Force new PSD plot scale
								sPSPlot.y_tic=0;

								// Increment filename
								if(bIncExtension)
								{
									IncExtension(szFile, PHASE_DATA);
									IncExtension(szFileTitle, PHASE_DATA);
									IncExtension(szPhaseFile, PHASE_DATA);
									IncExtension(szPhaseFileTitle, PHASE_DATA);
									SetWindowText(hwndPhaseFilename,
										szFileTitle);
								}
							}

							// Clear plot lines
							ClearLines();

							// Redraw status plots
							// SendMessage(hDlg, WM_COMMAND,
							// GET_WM_COMMAND_MPS(IDC_EDIT_REDRAW, 0, 0));
						}
						break;

						// Notification after a new row is inserted into table
						case DTN_INSERTEDROW:
						{
							// Get row # of insert
							nInsertRow=(int)SendMessage(hDataTbl,
								DTM_GETNOTIFYROW, 0, 0L);

							// Force value to 0
							fVal=0.0;

							// Update array data
							fData[nInsertRow+ARRAY_OFFSET]=fVal;

							// Update timetag data
							if(bTags)
							{
								fTags[nInsertRow+ARRAY_OFFSET]=fVal;
							}

							// Reinitiate communications with data table
							// This seems to clear up a number of display bugs
							// Insertion can now be chained easily
							// and display always seems "sane"
							SendMessage(hDataTbl, DTM_VM_INITIALIZE, FALSE,
								(LPARAM) nRows);

							// Reposition table to inserted cell
							SendMessage(hDataTbl, DTM_SETTOPINDEX, 0,
								(LPARAM) nInsertRow);

							// Set table focus to inserted cell 
							SendMessage(hDataTbl, DTM_SETCURCELL,
								(WPARAM)nColNum, (LPARAM)(nInsertRow));

							// Recolor gaps and outliers
							SendMessage(hDlg, WM_COMMAND,
								GET_WM_COMMAND_MPS(IDC_EDIT_TABLE, hDataTbl,
								DTN_COLOR));

							// Redraw status plots
							SendMessage(hDlg, WM_COMMAND,
								GET_WM_COMMAND_MPS(IDC_EDIT_REDRAW, 0, 0));
						}
						break;

						// Notification after a row is deteted from table
						case DTN_DELETEDROW:
						{
							// Redraw status plots
							SendMessage(hDlg, WM_COMMAND,
								GET_WM_COMMAND_MPS(IDC_EDIT_REDRAW, 0, 0));

							// Reposition focus wrt deleted row
							// nDeleteRow is 0-based index of deleted row
							// nRows is new 1-based # points
							// Message is 0-based row # to give focus to
							// Want to put focus on nDeleteRow unless
							// last row was the one deleted as indicated by
							// nDeleteRow==nRows
							// In that case, focus goes to nDeleteRow-1

							// Reinitiate communications with data table
							// This seems to clear up a number of display bugs
							// Deletion can now be chained easily
							// and display always seems "sane"
							SendMessage(hDataTbl, DTM_VM_INITIALIZE, FALSE,
								(LPARAM) nRows);

							// Reposition table to deleted cell and
							// Set table focus to deleted cell
							if(nDeleteRow!=nRows)
							{
								SendMessage(hDataTbl, DTM_SETTOPINDEX, 0,
									(LPARAM) (nDeleteRow));
								SendMessage(hDataTbl, DTM_SETCURCELL,
									(WPARAM)nColNum, (LPARAM)(nDeleteRow));
							}
							else
							{
								SendMessage(hDataTbl, DTM_SETTOPINDEX, 0,
									(LPARAM) (nDeleteRow-1));
								SendMessage(hDataTbl, DTM_SETCURCELL,
									(WPARAM)nColNum, (LPARAM)(nDeleteRow-1));
							}

							// Recolor gaps and outliers
							SendMessage(hDlg, WM_COMMAND,
								GET_WM_COMMAND_MPS(IDC_EDIT_TABLE, hDataTbl,
								DTN_COLOR));
						}
						break;

						// Cell contents have changed
						// Update data array
						case DTN_CHANGE:
						{
							// Save previous data in tab
							if(nChanges==0)
							{
								if(bTabs && bTabEdit)
								{
									CreateTab();
								}
							}

							// Get row # that changed
							// Note:  Row index starts at 0
							// Data index starts at ARRAY_OFFSET
							nRowNum=(int)SendMessage(hDataTbl,
								DTM_GETNOTIFYROW, 0, 0L);

							// Get column # that changed
							// This info needed if 2 column format is used
							nColNum=(int)SendMessage(hDataTbl,
								DTM_GETNOTIFYCOL, 0, 0L);

							// Get new value
							SendMessage(hDataTbl, DTM_SETACCESSPOS, nColNum,
								nRowNum);
							SendMessage(hDataTbl, DTM_READCELL, DTA_ACCESS,
								(LPARAM)(LPSTR)szBuffer);
							fVal=atof(szBuffer);

							// Reformat value
							_snprintf(szBuffer, sizeof(szBuffer), "%+20.13e",
								fVal);
							ShortenExp(szBuffer);

							// Rewrite table cell
							SendMessage(hDataTbl, DTM_SETACCESSPOS, nColNum,
								nRowNum);
							SendMessage(hDataTbl, DTM_UPDATECELL, DTA_ACCESS,
								(LPARAM)(LPSTR)szBuffer);

							// Update array data
							fData[nRowNum+ARRAY_OFFSET]=fVal;

							// Set change flags
							if(bDataType==FREQ_DATA)
							{
								bNewFreqData=TRUE;
								bFreqDataChanged=TRUE;
								bFreqDataUnsaved=TRUE;
								bFreqScaleChanged=TRUE;
								bSynch=FALSE;
								bEditingDone=TRUE;

								nChanges++;

								// Force new PSD plot scale
								sPSPlot.y_tic=0;

								// Increment filename
								if(bIncExtension)
								{
									IncExtension(szFile, FREQ_DATA);
									IncExtension(szFileTitle, FREQ_DATA);
									IncExtension(szFreqFile, FREQ_DATA);
									IncExtension(szFreqFileTitle, FREQ_DATA);
									SetWindowText(hwndFreqFilename,
										szFileTitle);
								}
							}
							else // Phase data
							{
								bNewPhaseData=TRUE;
								bPhaseDataChanged=TRUE;
								bPhaseDataUnsaved=TRUE;
								bPhaseScaleChanged=TRUE;
								bSynch=FALSE;
								bEditingDone=TRUE;

								nChanges++;

								// Force new PSD plot scale
								sPSPlot.y_tic=0;

								if(bIncExtension)
								{
									IncExtension(szFile, PHASE_DATA);
									IncExtension(szFileTitle, PHASE_DATA);
									IncExtension(szPhaseFile, PHASE_DATA);
									IncExtension(szPhaseFileTitle, PHASE_DATA);
									SetWindowText(hwndPhaseFilename,
										szFileTitle);
								}
							}

							// Redraw status plots
							SendMessage(hDlg, WM_COMMAND,
								GET_WM_COMMAND_MPS(IDC_EDIT_REDRAW, 0, 0));

							// Clear plot lines
							ClearLines();

							// Recolor gaps and outliers
							SendMessage(hDlg, WM_COMMAND,
								GET_WM_COMMAND_MPS(IDC_EDIT_TABLE, hDataTbl,
								DTN_COLOR));
						}
						break;

						case DTN_VSCROLL:
						{
							// Recolor gaps and outliers
							SendMessage(hDlg, WM_COMMAND,
								GET_WM_COMMAND_MPS(IDC_EDIT_TABLE, hDataTbl,
								DTN_COLOR));
						}
						break;

						case DTN_COLOR:
						{
							// Right mouse button marks visible gaps in blue

							// Get row # at top of visible table
							nTopRow=(int)SendMessage(hDataTbl, DTM_GETTOPINDEX,
								0, 0L);

							// Get # visible rows
							nNumVisRows=(int)SendMessage(hDataTbl,
								DTM_GETVISROWS, 0, 0L);

							// Calc outlier limits for freq data
							// Don't recalc if data hasn't changed
							if(bDataType==FREQ_DATA && bNewFreqData)
							{
								// Reset FreqDataChange flag
								bNewFreqData=FALSE;

								// Calc median value (via wrapper function)
								// Note: This is now quite fast with NR method
								// Part of FrequenC DLL as of 03/18/03
								FindMedian(fData, bDataType, &fMed);

								// Find median absolute deviation
								// with faster NR methos
								MedDev(fData, &fMedAbsDev);

								// Get sigma factor
								GetWindowText(GETHCTL(IDC_EDIT_SIGMAFACTOR),
									szBuffer, BUFFER_SIZE);
								fSigFactor=(float)atof(szBuffer);

								// Calc outlier limits
								fLowerLimit=fMed-fMedAbsDev*fSigFactor;
								fUpperLimit=fMed+fMedAbsDev*fSigFactor;
							}

							// Loop thru visible rows, coloring them all normal
							for(i=nTopRow; i<=nTopRow+nNumVisRows; i++)
							{
								// Get row access position
								SendMessage(hDataTbl, DTM_SETACCESSPOS,
									nColNum, i);

								// Color cell normal (black)
								SendMessage(hDialog, DTM_SETCELLCOLOR,
									DTA_ACCESS,
									(LPARAM)(LPDTCOLORSTRUCT)&NormColor);
							}

							// If gap or outlier coloring is activated
							if(bColorGaps || bColorOuts)
							{
								// Loop thru visible rows, looking for gaps
								for(i=nTopRow; i<=nTopRow+nNumVisRows; i++)
								{
									// Get row value
									SendMessage(hDataTbl, DTM_SETACCESSPOS,
										nColNum, i);
									SendMessage(hDataTbl, DTM_READCELL,
										DTA_ACCESS, (LPARAM)(LPSTR)szBuffer);
									fVal=atof(szBuffer);

									// Check for gap
									if(fVal==0.0 && bColorGaps)
									{
										if(bDataType==FREQ_DATA ||
											(i>0 && i<nRows-1))
										{
											// Color gap blue
											SendMessage(hDialog,
												DTM_SETCELLCOLOR,
												DTA_ACCESS, (LPARAM)
												(LPDTCOLORSTRUCT)&GapColor);
										}
									}

									// Check for outlier
									if(bDataType==FREQ_DATA && bColorOuts)
									{
										if(fVal<fLowerLimit || fVal>fUpperLimit)
										{
											if(fVal) // Gaps are not outliers
											{
												// Color outlier red
												SendMessage(hDialog,
													DTM_SETCELLCOLOR,
													DTA_ACCESS, (LPARAM)
													(LPDTCOLORSTRUCT)
													&OutColor);
											}
										}
									}

									if( (bDataType==FREQ_DATA &&
										bFreqTimetags) ||
										(bDataType==PHASE_DATA &&
										bPhaseTimetags) )
										// Test added 10/09/01
									{
										// Color timetag gaps
										// Calc beginning timetag interval
										fTagInterval=fTags[ARRAY_OFFSET+1]-
											fTags[ARRAY_OFFSET];

										// Check for timetag interruption
										// Current row # = i (i starts at 1)
										// Timetag value for previous row is
										// fTags[i+ARRAY_OFFSET-2]
										// Timetag value for current row is
										// fTags[i+ARRAY_OFFSET-1]
										// Tolerance on timetag interval is
										// +/- 1%
										if((((fTags[i+ARRAY_OFFSET-1]-
											fTags[i+ARRAY_OFFSET-2])>
											(1.01*fTagInterval))
											|| ((fTags[i+ARRAY_OFFSET-1]-
											fTags[i+ARRAY_OFFSET-2])<
											(0.99*fTagInterval)))
											&& bColorGaps && bWideEdit && i>2)
										{
											// Set access position
											SendMessage(hDataTbl,
												DTM_SETACCESSPOS, nColNum-1,
												i-1);

											// Color timetag gap blue
											SendMessage(hDialog,
												DTM_SETCELLCOLOR,
												DTA_ACCESS, (LPARAM)
												(LPDTCOLORSTRUCT)&GapColor);
										}
									}
								
								}
							}
						}
						break;
					}
				}
				break;

				case IDHELP:
				{
					#if(0)
					// Invoke help file
					HtmlHelp(hDlg, "Stable32.chm::EDIT_FUNCTION.HTM",
						HH_DISPLAY_TOPIC, 0L);
					#endif

					#if(0) // New code 11/29/11
					// Invoke help file
					strlcpy(szBuffer, szHelpFile, sizeof(szBuffer));
					strlcat(szBuffer, "::EDIT_FUNCTION.HTM", sizeof(szBuffer));
					HtmlHelp(hDlg, szBuffer,
						HH_DISPLAY_TOPIC, 0L);
					#endif

					#if(1) // Trial code 11/30/11
					// Invoke help file using CallHelp() function
					CallHelp(hDlg, "EDIT_FUNCTION");
					#endif
				}
				break;

				case IDC_EDIT_MJD:
				{
					// Redo timetag format
					nTagFormat=TAGFORMAT_MJD;

					// Get row # at top of visible table
					nTopRow=(int)SendMessage(hDataTbl, DTM_GETTOPINDEX, 0,
						0L);

					// Refresh table to update color state
					SendMessage(hDataTbl, DTM_SETTOPINDEX, 0,
						(LPARAM) nTopRow);
				}
				break;

				// Respond to CR in an edit control
				case IDOK:
				{
					// Send EN_KILLFOCUS message to POINTNUM edit control
					if(GetFocus()==GETHCTL(IDC_EDIT_POINTNUM))
					{
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_EDIT_POINTNUM, NULL,
							EN_KILLFOCUS));
					}

					// Send EN_KILLFOCUS message to SIGMAFACTOR edit control
					if(GetFocus()==GETHCTL(IDC_EDIT_SIGMAFACTOR))
					{
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_EDIT_SIGMAFACTOR, NULL,
							EN_KILLFOCUS));
					}

					// Send EN_KILLFOCUS message to NUMGAPS edit control
					if(GetFocus()==GETHCTL(IDC_EDIT_NUMGAPS))
					{
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_EDIT_NUMGAPS, NULL,
							EN_KILLFOCUS));
					}
				}
				break;

				case IDC_EDIT_SIGMAFACTOR:
				{
					// Respond to edit control when it loses the input focus
					// unless it loses it to the cancel button
					if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
						(GetFocus()!=GETHCTL(IDCANCEL)))
					{
						// Get sigma factor
						GetWindowText(GETHCTL(IDC_EDIT_SIGMAFACTOR), szBuffer,
							BUFFER_SIZE);
						fSigFactor=(float)fabs(atof(szBuffer)); // must be +
						_snprintf(szBuffer, sizeof(szBuffer), "%5.3f",
							fSigFactor);
						SetWindowText(GETHCTL(IDC_EDIT_SIGMAFACTOR), szBuffer);

						// Calc outlier limits
						fLowerLimit=fMed-fMedAbsDev*fSigFactor;
						fUpperLimit=fMed+fMedAbsDev*fSigFactor;

						// Get row # at top of visible table
						nTopRow=(int)SendMessage(hDataTbl, DTM_GETTOPINDEX,
							(WPARAM)0, (LPARAM)0);

						// Loop thru visible rows, coloring them all normal
						for(i=nTopRow; i<=nTopRow+nNumVisRows; i++)
						{
							// Get row access position
							SendMessage(hDataTbl, DTM_SETACCESSPOS,
								(WPARAM)nColNum, (LPARAM)i);

							// Color cell normal (black)
							SendMessage(hDialog, DTM_SETCELLCOLOR,
								(WPARAM)DTA_ACCESS,
								(LPARAM)(LPDTCOLORSTRUCT)&NormColor);
						}

						// If gap or outlier coloring is activated
						if(bColorGaps || bColorOuts)
						{
							// Loop thru visible rows, looking for gaps
							for(i=nTopRow; i<=nTopRow+nNumVisRows; i++)
							{
								// Get row value
								SendMessage(hDataTbl, DTM_SETACCESSPOS,
									(WPARAM)nColNum, (LPARAM)i);
								SendMessage(hDataTbl, DTM_READCELL,
									(WPARAM)DTA_ACCESS,
									(LPARAM)(LPSTR)szBuffer);
								fVal=atof(szBuffer);

								// Check for gap
								if(fVal==0.0 && bColorGaps)
								{
									if(bDataType==FREQ_DATA ||
										(i>0 && i<nRows-1))
									{
										// Color gap blue
										SendMessage(hDialog,
											DTM_SETCELLCOLOR,
											(WPARAM)DTA_ACCESS,
											(LPARAM)(LPDTCOLORSTRUCT)
											&GapColor);
									}
								}

								// Check for outlier
								if(bDataType==FREQ_DATA && bColorOuts)
								{
									if(fVal<fLowerLimit || fVal>fUpperLimit)
									{
										if(fVal) // Gaps are not outliers
										{
											// Color outlier red
											SendMessage(hDialog,
												DTM_SETCELLCOLOR,
												(WPARAM)DTA_ACCESS, (LPARAM)
												(LPDTCOLORSTRUCT)&OutColor);
										}
									}
								}
							}
						}
					}
				}
				break;

				case IDC_EDIT_NUMGAPS:
				{
					// Respond to edit control when it loses the input focus
					// unless it loses it to the cancel button
					if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
						(!bErrorTrap) &&
						(GetFocus()!=GETHCTL(IDCANCEL)))
					{
						// Set error flag to avoid multiple error messages
						bErrorTrap=TRUE;

						// Get # gaps
						GetWindowText(GETHCTL(IDC_EDIT_NUMGAPS), szBuffer,
							BUFFER_SIZE);
						nEntry=atoi(szBuffer);

						// Check nEntry entry - must be between 0 and
						// # that would make # rows > nMaxSize
						if(ValidateInteger(hDlg, nEntry, 0,
							nMaxSize-nRows)==TRUE)
						{
							// New value OK
							nGaps=nEntry;
							_snprintf(szBuffer, sizeof(szBuffer), "%d", nGaps);
							SetWindowText(GETHCTL(IDC_EDIT_NUMGAPS), szBuffer);
						}
						else
						{
							// Invalid entry - restore default value
							SetWindowText(GETHCTL(IDC_EDIT_NUMGAPS), "0");
							nGaps=0;
							SetFocus(GETHCTL(IDC_EDIT_NUMGAPS));
						}
					}
					else
					{
						// Reset error flag
						bErrorTrap=FALSE;
					}
				}
				break;

				case IDC_EDIT_POINTNUM:
				{
					// Respond to edit control when it loses the input focus
					// unless it loses it to the cancel button
					if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
						(!bErrorTrap) &&
						(GetFocus()!=GETHCTL(IDCANCEL)))
					{
						// Set error flag to avoid multiple error messages
						bErrorTrap=TRUE;

						// Get point #
						GetWindowText(GETHCTL(IDC_EDIT_POINTNUM), szBuffer,
							BUFFER_SIZE);
						nEntry=atoi(szBuffer);

						// Check nEntry entry - must be between 1 and nRows
						if(ValidateInteger(hDlg, nEntry, 1, nRows)==TRUE)
						{
							// New value OK
							nRowNum=nEntry;
							_snprintf(szBuffer, sizeof(szBuffer), "%d",
								nRowNum);
							SetWindowText(GETHCTL(IDC_EDIT_POINTNUM),
								szBuffer);
						}
						else
						{
							// Invalid entry - restore default value
							SetWindowText(GETHCTL(IDC_EDIT_POINTNUM), "1");
							nRowNum=1;
							SetFocus(GETHCTL(IDC_EDIT_POINTNUM));
						}
					}
					else
					{
						// Reset error flag
						bErrorTrap=FALSE;
					}
				}
				break;

				case IDC_EDIT_DOY:
				{
					// Redo timetag format
					nTagFormat=TAGFORMAT_DOY;

					// Get row # at top of visible table
					nTopRow=(int)SendMessage(hDataTbl, DTM_GETTOPINDEX,
						(WPARAM)0, (LPARAM)0);

					// Refresh table to update color state
						SendMessage(hDataTbl, DTM_SETTOPINDEX, (WPARAM)0,
							(LPARAM)nTopRow);
				}
				break;

				case IDC_EDIT_DATE:
				{
					// Redo timetag format
					nTagFormat=TAGFORMAT_DATE;

					// Get row # at top of visible table
					nTopRow=(int)SendMessage(hDataTbl, DTM_GETTOPINDEX,
						(WPARAM)0, (LPARAM)0);

					// Refresh table to update color state
						SendMessage(hDataTbl, DTM_SETTOPINDEX, (WPARAM)0,
						(LPARAM)nTopRow);
				}
				break;

				case IDC_EDIT_EXP:
				{
					// Redo timetag format
					nTagFormat=TAGFORMAT_EXP;

					// Get row # at top of visible table
					nTopRow=(int)SendMessage(hDataTbl, DTM_GETTOPINDEX,
						(WPARAM)0, (LPARAM)0);

					// Refresh table to update color state
						SendMessage(hDataTbl, DTM_SETTOPINDEX, (WPARAM)0,
							(LPARAM)nTopRow);
				}
				break;

				case IDC_EDIT_PREV:
				{
					bNext=FALSE;
				}
				break;

				case IDC_EDIT_NEXT:
				{
					bNext=TRUE;
				}
				break;

				// Go to cell
				// Note: Row index starts at 0
				case IDC_EDIT_POINT:
				{
					GetWindowText(GETHCTL(IDC_EDIT_POINTNUM), szBuffer,
						BUFFER_SIZE);
					nRowNum=atoi(szBuffer);
					if(SendMessage(hDataTbl, DTM_SETCURCELL, (WPARAM)nColNum,
						(LPARAM)(nRowNum-1))==DTE_ROWNUM)
					{
						// Invalid row #
						ValidateInteger(hDlg, nRowNum, 1, nRows);
						// Use default value
						SetWindowText(GETHCTL(IDC_EDIT_POINTNUM), "1");
						nRowNum=0;
						SetFocus(GETHCTL(IDC_EDIT_POINTNUM));
					}
					else
					{
						SendMessage(hDataTbl, DTM_SETTOPINDEX, (WPARAM)0,
							(LPARAM)(nRowNum-1));
					}
				}
				break;

				// Search for gap
				// Note: can't use DTM_SEARCHCOL with virtual memory
				case IDC_EDIT_GAP:
				{
					// Get current row number
					nRowNum=(int)SendMessage(hDataTbl, DTM_GETCURROW,
						(WPARAM)0, (LPARAM)0);
					nGapRow=NO_GAP;

					if(bNext)
					{
						// Search down array, starting with the next point
						// This allows the function to be chained
						for(i=nRowNum+1; i<nRows; i++)
						{
							if(!fData[i+ARRAY_OFFSET]) // Is point a gap?
							{
								// Os at 1st & last phase pts are not gaps
								if(bDataType==FREQ_DATA || (i>0 && i<nRows-1))
								{
									nGapRow=i;
									break;
								}
							}
						}
					}
					else
					{
						// Search up array, starting with the previous point
						// This allows the function to be chained
						for(i=nRowNum-1; i>=0; i--)
						{
							if(!fData[i+ARRAY_OFFSET]) // Is point a gap?
							{
								// Os at 1st & last phase pts are not gaps
								if(bDataType==FREQ_DATA || (i>0 && i<nRows-1))
								{
									nGapRow=i;
									break;
								}
							}
						}
					}

					if(nGapRow==NO_GAP)
					{
						MessageBeep(0); // No gap found
					}
					else
					{
						// Move to the gap
						SendMessage(hDataTbl, DTM_SETCURCELL, nColNum,
							(LPARAM) nGapRow); // Row index starts at 0
						SendMessage(hDataTbl, DTM_SETTOPINDEX, 0,
							(LPARAM) nGapRow); // Row index starts at 0
					}
				}
				break;

				case IDC_EDIT_FIND:
				{
					// Not needed with hDlgModeless2 usage
					// If the Welcome modeless dialog box is active we want
					// to destroy it so that the Find box can be opened
					// But still nice because it allows toggling between
					// the Find & Replace boxes
					// if(hDlgModeless!=0 && bShowWelcome)
					if(hDlgModeless!=0)
					{
						DestroyWindow(hDlgModeless);
						hDlgModeless=0;
					}

					// Open modeless dialog box
					if(hDlgModeless==0) // make sure not already open
					{
						hDlgModeless=CreateDialog(hInst, "FindBox", hDlg,
							(DLGPROC)FindDlgProc); 
					}
					else
					{
						MessageBeep(0);
					}
					// Note: The Find modeless dialog box stays open
					// until closed by user or when entire Edit dialog
					// box is closed.  The results of a Find operation
					// are passed with the IDC_FIND_ROWNUM message.
				}
				break;

				case IDC_EDIT_REPLACE:
				{
					// Not needed with hDlgModeless2 usage
					// If the Welcome modeless dialog box is active we want
					// to destroy it so that the Replace box can be opened
					// But still nice because it allows toggling between
					// the Find & Replace boxes
					// if(hDlgModeless!=0 && bShowWelcome)
					if(hDlgModeless!=0)
					{
						DestroyWindow(hDlgModeless);
						hDlgModeless=0;
					}

					// Open modeless dialog box
					if(hDlgModeless==0) // make sure not already open
					{
						hDlgModeless=CreateDialog(hInst, "ReplaceBox", hDlg,
							(DLGPROC)ReplaceDlgProc);
					}
					else
					{
						MessageBeep(0);
					}
					// Note: The Replace modeless dialog box stays open
					// until closed by user or when entire Edit dialog
					// box is closed.  The results of a Replace operation
					// are passed with the IDC_REPLACE_ROWNUM or
					// IDC_REPLACE_ROWCHANGE message.
				}
				break;

				case IDC_EDIT_OUTLIER:
				{
					// Note: Essentially same code as used for gap-finding
					// Get current row number
					nRowNum=(int)SendMessage(hDataTbl, DTM_GETCURROW, 0, 0L);
					nGapRow=NO_GAP;

					if(bNext)
					{
						// Search down array, starting with the next point
						// This allows the function to be chained
						for(i=nRowNum+1; i<nRows; i++)
						{
							 // Is point an outlier?
							if((fData[i+ARRAY_OFFSET]) && (
								(fData[i+ARRAY_OFFSET]<fLowerLimit) ||
								(fData[i+ARRAY_OFFSET]>fUpperLimit)))
							{
								{
									nGapRow=i;
									break;
								}
							}
						}
					}
					else
					{
						// Search up array, starting with the previous point
						// This allows the function to be chained
						for(i=nRowNum-1; i>=0; i--)
						{
							// Is point an outlier?
							if((fData[i+ARRAY_OFFSET]) && (
								(fData[i+ARRAY_OFFSET]<fLowerLimit) ||
								(fData[i+ARRAY_OFFSET]>fUpperLimit)))
							{
								{
									nGapRow=i;
									break;
								}
							}
						}
					}

					if(nGapRow==NO_GAP)
					{
						MessageBeep(0); // No outlier found
					}
					else
					{
						// Move to the outlier
						SendMessage(hDataTbl, DTM_SETCURCELL, nColNum,
							(LPARAM) nGapRow); // Row index starts at 0
						SendMessage(hDataTbl, DTM_SETTOPINDEX, 0,
							(LPARAM) nGapRow); // Row index starts at 0
					}
				}
				break;

				case IDC_EDIT_COLORGAPS:
				{
					// Get the state of the Color Gaps checkbox
					bColorGaps=
						(BOOL)Button_GetCheck(GETHCTL(IDC_EDIT_COLORGAPS));

					// Get row # at top of visible table
					nTopRow=(int)SendMessage(hDataTbl, DTM_GETTOPINDEX,
						(WPARAM)0, (LPARAM)0);

					// Refresh table to update color state
						SendMessage(hDataTbl, DTM_SETTOPINDEX, (WPARAM)0,
							(LPARAM)nTopRow);
				}
				break;

				case IDC_EDIT_COLOROUTS:
				{
					// Get the state of the Color Outliers checkbox
					bColorOuts=
						(BOOL)Button_GetCheck(GETHCTL(IDC_EDIT_COLOROUTS));

					// Get row # at top of visible table
					nTopRow=(int)SendMessage(hDataTbl, DTM_GETTOPINDEX,
						(WPARAM)0, (LPARAM)0);

					// Refresh table to update color state
					SendMessage(hDataTbl, DTM_SETTOPINDEX, (WPARAM)0,
						(LPARAM)nTopRow);
				}
				break;

				case IDC_EDIT_TIMETAG:
				{
					// Get the state of the Show Timetags checkbox
					bWideEdit=
						(BOOL)Button_GetCheck(GETHCTL(IDC_EDIT_TIMETAG));

					// NOTE:  Need to free memory allocations here
					// Do same things as done when box is closed with OK
					// thus accepting changes - this seems better choice
					// than rejecting them - must do one or the other

					// Refresh fEditData and fEditTags pointers
					fEditData=fData;
					if(bTags)
					{
						fEditTags=fTags;
					}

					// Reset global data pointers to the permanant data array
					if(bDataType==FREQ_DATA)
					{
						fData=fFreqData;
						if(bTags)
						{
							fTags=fFreqTimetags;
						}
					}
					else // Phase data
					{
						fData=fPhaseData;
						if(bTags)
						{
							fTags=fPhaseTimetags;
						}
					}

					// Reallocate permanant data array
					if((fNew=(F_TYPE*)realloc((char *)fData,
						(nRows+ARRAY_OFFSET+EXTRA_MEM)*
						sizeof(F_TYPE)))==NULL)
					{
						// Dsp err msg, cancel edits & close dlg box
						AllocErrorMessage();
						PostMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDCANCEL, NULL, 0));
						break;
					}
					else
					{
						fData=fNew;
					}

					// Copy entire temporary data array to permanent data array
					for(i=0; i<fEditData[0]+ARRAY_OFFSET; i++)
					{
						fData[i]=fEditData[i];
					}

					if(bTags)
					{
						// Reallocate permanant timetag array
						if((fNew=(F_TYPE*)realloc((char *)fTags,
							(nRows+ARRAY_OFFSET+EXTRA_MEM)*
							sizeof(F_TYPE)))==NULL)
						{
						// Dsp err msg, cancel edits & close dlg box
						AllocErrorMessage();
						PostMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDCANCEL, NULL, 0));
						break;
						}
						else
						{
							fTags=fNew;
						}

						// Copy entire temp tag array to permanent tag array
						for(i=0; i<fEditTags[0]+ARRAY_OFFSET; i++)
						{
							fTags[i]=fEditTags[i];
						}
					}

					// Reassign the pointers of the permanent data or
					// tags arrays to the new fData and fTags pointer values
					if(bDataType==FREQ_DATA)
					{
						fFreqData=fData;
						if(bTags)
						{
							fFreqTimetags=fTags;
						}
					}
					else
					{
						fPhaseData=fData;
						if(bTags)
						{
							fPhaseTimetags=fTags;
						}
					}

					// Free memory for filenames
					free(szInitialFile);
					free(szInitialFileTitle);
					free(szInitialPhaseFile);
					free(szInitialPhaseFileTitle);
					free(szInitialFreqFile);
					free(szInitialFreqFileTitle);

					// Free the temporary arrays
					free(fEditData);
					if(bTags)
					{
						free(fEditTags);
					}

					// Make sure modeless dialog box is closed
					if(hDlgModeless!=0)
					{
						DestroyWindow(hDlgModeless);
						hDlgModeless=0;
					}

					// Change to other edit dialog box template
					PostMessage(hwndMain, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDM_MAIN_EDIT, hDlg,
						IDM_MAIN_EDIT));

					EndDialog(hDlg, 0);
					return TRUE;
				}

				case IDC_EDIT_AUTO:
				{
					// Get the current style
					dwExtWndStyle=SendMessage(hDataTbl, DTM_GETEXTWNDSTYLE,
						(WPARAM)0, (LPARAM)0);

					// Get the state of the Autoinsert checkbox
					bAutoInsert=
						(BOOL)Button_GetCheck(GETHCTL(IDC_EDIT_AUTO));

					// Rearrange style bits
					if(bAutoInsert)
					{
						dwExtWndStyle|=DTES_AUTOINSERT;
					}
					else
					{
						dwExtWndStyle|=!DTES_AUTOINSERT;
					}

					// Set style
					SendMessage(hDataTbl, DTM_SETEXTWNDSTYLE, (WPARAM)0,
						(LPARAM)dwExtWndStyle);
				}
				break;

				case IDC_EDIT_GAPS:
				{
					// Display wait cursor
					SetCursor(hWait);

					// Get current row # where gap insertion starts
					nInsertRow=(int)SendMessage(hDataTbl, DTM_GETCURROW,
						(WPARAM)0, (LPARAM)0);

					// Get # gaps to insert
					GetWindowText(GETHCTL(IDC_EDIT_NUMGAPS), szBuffer,
						BUFFER_SIZE);
					nGaps=abs(atoi(szBuffer)); // Make sure it's positive
					_snprintf(szBuffer, sizeof(szBuffer), "%d", nGaps);
					// Redisplay (pos) #
					SetWindowText(GETHCTL(IDC_EDIT_NUMGAPS), szBuffer);

					// Save previous data in tab
					if(nGaps && nChanges==0)
					{
						if(bTabs && bTabEdit)
						{
							CreateTab();
						}
					}

					// Add nGaps to nRows and # data points
					nRows+=nGaps;
					fData[0]+=nGaps;

					if(bTags)
					{
						fTags[0]+=nGaps;
					}

					// Reallocate memory
					// Note: realloc 1st, then move data
					// Add nGaps points to active data array
					if((fNew=(F_TYPE*)realloc((char *)fData,
						(nRows+ARRAY_OFFSET+EXTRA_MEM)*
						sizeof(F_TYPE)))==NULL)
					{
						// Dsp err msg, cancel edits & close dlg box
						AllocErrorMessage();
						PostMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDCANCEL, NULL, 0));
						break;
					}
					else
					{
						fData=fNew;
					}

					if(bTags)
					{
						// Add nGaps points to active timetag array
						if((fNew=(F_TYPE*)realloc((char *)fTags,
							(nRows+ARRAY_OFFSET+EXTRA_MEM)*
							sizeof(F_TYPE)))==NULL)
						{
							// Gaps have been added to the data but now
							// there isn't enough room for the timetags
							// I suppose the best thing to do is display
							// error msg, clear timetags and bail out
							AllocErrorMessage();
							ClearTimetagArray(bDataType);
							if(bDataType==FREQ_DATA)
							{
								bFreqTimetags=FALSE;
							}
							else // Phase data
							{
								bPhaseTimetags=FALSE;
							}
							PostMessage(hDlg, WM_COMMAND,
								GET_WM_COMMAND_MPS(IDCANCEL, NULL, 0));
							break;
						}
						else
						{
							fTags=fNew;
						}
					}

					// Update status display and array pointers
					_snprintf(szBuffer, sizeof(szBuffer), "%5.0d", nRows);

					if(bDataType==FREQ_DATA)
					{
						// The #: status field is always increased to new nRows
						SetWindowText(hwndFreqNum, szBuffer);

						// The Start: status field will be increased by nGaps
						// if it is beyond the insertion point.
						GetWindowText(hwndFreqStart, szBuffer, BUFFER_SIZE);
						nRowNum=atoi(szBuffer);

						if(nRowNum>nInsertRow+1)
						{
							_snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
								nRowNum+nGaps);
							SetWindowText(hwndFreqStart, szBuffer);
							fData[1]=nRowNum+nGaps;

							if(bTags)
							{
								fTags[1]=nRowNum+nGaps;
							}
						}

						// The End: status field will be increased by nGaps
						// if it is beyond the insertion point.
						GetWindowText(hwndFreqEnd, szBuffer, BUFFER_SIZE);
						nRowNum=atoi(szBuffer);

						if(nRowNum>nInsertRow+1)
						{
							_snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
								nRowNum+nGaps);
							SetWindowText(hwndFreqEnd, szBuffer);
							fData[2]=nRowNum+nGaps;

							if(bTags)
							{
								fTags[2]=nRowNum+nGaps;
							}
						}

						// Move original data down from insert point
						// Start at new end of table and work up toward
						// smaller row #s to row below insert point
						// Note:  Row index goes from 0 to nRows-1
						// Data index starts at ARRAY_OFFSET
						// Index i is row #

						for(i=nRows-nGaps-1; i>nInsertRow; i--)
						{
							fData[i+ARRAY_OFFSET+nGaps]=
								fData[i+ARRAY_OFFSET];

							if(bTags)
							{
								fTags[i+ARRAY_OFFSET+nGaps]=
									fTags[i+ARRAY_OFFSET];
							}
						}

						if(nGaps>0)
						{
							// Set change flags
							bNewFreqData=TRUE;
							bFreqDataChanged=TRUE;
							bFreqDataUnsaved=TRUE;
							sFPlot.new_file=TRUE;
							bFreqScaleChanged=TRUE;
							bSynch=FALSE;
							bEditingDone=TRUE;

							nChanges++;

							// Force new PSD plot scale
							sPSPlot.y_tic=0;

							// Increment filename
							if(bIncExtension)
							{
								IncExtension(szFile, FREQ_DATA);
								IncExtension(szFileTitle, FREQ_DATA);
								IncExtension(szFreqFile, FREQ_DATA);
								IncExtension(szFreqFileTitle, FREQ_DATA);
								SetWindowText(hwndFreqFilename,
									szFileTitle);
							}

							// Clear plot lines
							ClearLines();
						}
					}
					else // Phase data
					{
						// The #: status field is always increased to new nRows
						SetWindowText(hwndPhaseNum, szBuffer);

						// The Start: status field will be increased by nGaps
						// if it is beyond the insertion point.
						GetWindowText(hwndPhaseStart, szBuffer, BUFFER_SIZE);
						nRowNum=atoi(szBuffer);

						if(nRowNum>nInsertRow+1)
						{
							_snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
								nRowNum+nGaps);
							SetWindowText(hwndPhaseStart, szBuffer);
							fData[1]=nRowNum+nGaps;

							if(bTags)
							{
								fTags[1]=nRowNum+nGaps;
							}
						}

						// The End: status field will be increased by nGaps
						// if it is beyond the insertion point.
						GetWindowText(hwndPhaseEnd, szBuffer, BUFFER_SIZE);
						nRowNum=atoi(szBuffer);

						if(nRowNum>nInsertRow+1)
						{
							_snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
								nRowNum+nGaps);
							SetWindowText(hwndPhaseEnd,	szBuffer);
							fData[2]=nRowNum+nGaps;

							if(bTags)
							{
								fTags[2]=nRowNum+nGaps;
							}
						}

						// Move original data down from insert point
						// Note:  Row index starts at 0
						// Data index starts at ARRAY_OFFSET
						// Index i is row #

						for(i=nRows-nGaps-1; i>nInsertRow; i--)
						{
							fData[i+ARRAY_OFFSET+nGaps]=
								fData[i+ARRAY_OFFSET];

							if(bTags)
							{
								fTags[i+ARRAY_OFFSET+nGaps]=
									fTags[i+ARRAY_OFFSET];
							}
						}

						if(nGaps>0)
						{
							// Set change flags
							bNewPhaseData=TRUE;
							bPhaseDataChanged=TRUE;
							bPhaseDataUnsaved=TRUE;
							bPhaseScaleChanged=TRUE;
							bSynch=FALSE;
							bEditingDone=TRUE;

							nChanges++;

							// Force new PSD plot scale
							sPSPlot.y_tic=0;

							// Increment filename
							if(bIncExtension)
							{
								IncExtension(szFile, PHASE_DATA);
								IncExtension(szFileTitle, PHASE_DATA);
								IncExtension(szPhaseFile, PHASE_DATA);
								IncExtension(szPhaseFileTitle, PHASE_DATA);
								SetWindowText(hwndPhaseFilename, szFileTitle);
							}

							// Clear plot lines
							ClearLines();
						}
					}

					// Put nGap 0s into data array
					for(i=nInsertRow+1; i<=nInsertRow+nGaps; i++)
					{
						fData[i+ARRAY_OFFSET]=0.0;

						// WHAT SHOULD GO INTO INSERTED TIMETAGS?

						if(bTags)
						{
							fTags[i+ARRAY_OFFSET]=0.0;
						}
					}

					// Update and reposition table
					SendMessage(hDataTbl, DTM_SETTOPINDEX, 0,
						(LPARAM) nInsertRow);

					// Redraw status plots
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_EDIT_REDRAW, 0, 0));

					// Display normal cursor
					SetCursor(hArrow);
				}
				break;

				// Delete a block of data
				case IDC_EDIT_DELS:
				{
					// Display wait cursor
					SetCursor(hWait);

					// Get current row # where deletion starts
					nInsertRow=(int)SendMessage(hDataTbl,
						DTM_GETCURROW, 0, 0L);

					// Get # points to delete
					GetWindowText(GETHCTL(IDC_EDIT_NUMGAPS), szBuffer,
						BUFFER_SIZE);
					nGaps=abs(atoi(szBuffer)); // Make sure its positive
					_snprintf(szBuffer, sizeof(szBuffer), "%d", nGaps);
					// Redisplay (pos) #
					SetWindowText(GETHCTL(IDC_EDIT_NUMGAPS), szBuffer);

					// Save previous data in tab
					if(nGaps && nChanges==0)
					{
						if(bTabs && bTabEdit)
						{
							CreateTab();
						}
					}

					// Check that # deletions is valid
					if(nGaps>(nRows-nInsertRow-1))
					{
						// Too many deletions
						_snprintf(szBuffer, sizeof(szBuffer),
							"Can not delete more than %d rows",
							nRows-nInsertRow-1);
						MessageBox(hDlg, szBuffer, "Edit", MB_ICONEXCLAMATION);
						break;
					}

					// Subtract nGaps from nRows and # data points
					nRows-=nGaps;
					fData[0]-=nGaps;

					if(bTags)
					{
						fTags[0]-=nGaps;
					}

					// Note: Move data first, then realloc

					// Move original data up to insert point
					// Starting at insert row & working down table
					// toward higher row #s
					// Note:  Row index starts at 0
					// Data index starts at ARRAY_OFFSET
					for(i=nInsertRow+1; i<nRows; i++)
					{
						fData[i+ARRAY_OFFSET]=
							fData[i+ARRAY_OFFSET+nGaps];

						if(bTags)
						{
							fTags[i+ARRAY_OFFSET]=
								fTags[i+ARRAY_OFFSET+nGaps];
						}
					}

					// Reallocate memory
					// Subtract nGaps points from active data array
					if((fNew=(F_TYPE*)realloc((char *)fData,
						(nRows+ARRAY_OFFSET+EXTRA_MEM)*
						sizeof(F_TYPE)))==NULL)
					{
						// This really shouldn't ever happen
						AllocErrorMessage();
					}
					else
					{
						fData=fNew;
					}

					if(bTags)
					{
						// Subtract nGaps points from active timetag array
						if((fNew=(F_TYPE*)realloc((char *)fTags,
							(nRows+ARRAY_OFFSET+EXTRA_MEM)*
							sizeof(F_TYPE)))==NULL)
						{
							// This really shouldn't ever happen either
							AllocErrorMessage();
						}
						else
						{
							fTags=fNew;
						}
					}

					// Update status display and array pointers
					_snprintf(szBuffer, sizeof(szBuffer), "%5.0d", nRows);

					if(bDataType==FREQ_DATA)
					{
						// The #: status field is always decreased to new nRows
						SetWindowText(hwndFreqNum, szBuffer);

						// The Start: status field will be decreased by nGaps
						// if it is beyond the insertion point.
						GetWindowText(hwndFreqStart, szBuffer,
							BUFFER_SIZE);
						nRowNum=atoi(szBuffer);

						if(nRowNum>nInsertRow+1)
						{
							_snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
								nRowNum-nGaps);
							SetWindowText(hwndFreqStart, szBuffer);
							fData[1]=nRowNum-nGaps;

							if(bTags)
							{
								fTags[1]=nRowNum-nGaps;
							}
						}

						// The End: status field will be decreased by nGaps
						// if it is beyond the insertion point.
						GetWindowText(hwndFreqEnd, szBuffer,
							BUFFER_SIZE);
						nRowNum=atoi(szBuffer);

						if(nRowNum>nInsertRow+1)
						{
							_snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
								nRowNum-nGaps);
							SetWindowText(hwndFreqEnd, szBuffer);
							fData[2]=nRowNum-nGaps;

							if(bTags)
							{
								fTags[2]=nRowNum-nGaps;
							}
						}

						if(nGaps>0)
						{
							// Set change flags
							bNewFreqData=TRUE;
							bFreqDataChanged=TRUE;
							bFreqDataUnsaved=TRUE;
							sFPlot.new_file=TRUE;
							bFreqScaleChanged=TRUE;
							bSynch=FALSE;
							bEditingDone=TRUE;

							nChanges++;

							// Force new PSD plot scale
							sPSPlot.y_tic=0;

							// Increment filename
							if(bIncExtension)
							{
								IncExtension(szFile, FREQ_DATA);
								IncExtension(szFileTitle, FREQ_DATA);
								IncExtension(szFreqFile, FREQ_DATA);
								IncExtension(szFreqFileTitle, FREQ_DATA);
								SetWindowText(hwndFreqFilename, szFileTitle);
							}

							// Clear plot lines
							ClearLines();
						}
					}
					else // Phase data
					{
						// The #: status field is always decreased to new nRows
						SetWindowText(hwndPhaseNum, szBuffer);

						// The Start: status field will be decreased by nGaps
						// if it is beyond the insertion point.
						GetWindowText(hwndPhaseStart, szBuffer,	BUFFER_SIZE);
						nRowNum=atoi(szBuffer);

						if(nRowNum>nInsertRow+1)
						{
							_snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
								nRowNum-nGaps);
							SetWindowText(hwndPhaseStart, szBuffer);
							fData[1]=nRowNum-nGaps;

							if(bTags)
							{
								fTags[1]=nRowNum-nGaps;
							}
						}

						// The End: status field will be decreased by nGaps
						// if it is beyond the insertion point.
						GetWindowText(hwndPhaseEnd, szBuffer, BUFFER_SIZE);
						nRowNum=atoi(szBuffer);

						if(nRowNum>nInsertRow+1)
						{
							_snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
								nRowNum-nGaps);
							SetWindowText(hwndPhaseEnd,	szBuffer);
							fData[2]=nRowNum-nGaps;

							if(bTags)
							{
								fTags[2]=nRowNum-nGaps;
							}
						}

						if(nGaps>0)
						{
							// Set change flags
							bNewPhaseData=TRUE;
							bPhaseDataChanged=TRUE;
							bPhaseDataUnsaved=TRUE;
							bPhaseScaleChanged=TRUE;
							bSynch=FALSE;
							bEditingDone=TRUE;

							nChanges++;

							// Force new PSD plot scale
							sPSPlot.y_tic=0;

							// Increment filename
							if(bIncExtension)
							{
								IncExtension(szFile, PHASE_DATA);
								IncExtension(szFileTitle, PHASE_DATA);
								IncExtension(szPhaseFile, PHASE_DATA);
								IncExtension(szPhaseFileTitle, PHASE_DATA);
								SetWindowText(hwndPhaseFilename, szFileTitle);
							}

							// Clear plot lines
							ClearLines();
						}
					}

					// Update and reposition table
					SendMessage(hDataTbl, DTM_SETTOPINDEX, 0,
						(LPARAM) nInsertRow);

					// Redraw status plots
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_EDIT_REDRAW, 0, 0));

					// Display normal cursor
					SetCursor(hArrow);
				}
				break;

				// Redraw status bar plot
				case IDC_EDIT_REDRAW:
				{
					if(bDataType==FREQ_DATA)
					{
						// Save main freq data pointer
						fTemp=fFreqData;

						// Reassign it to edit data
						fFreqData=fData;

						// Set freq redraw flag
						bFreqRedraw=TRUE;
					}
					else // Phase data
					{
						// Save main phase data pointer
						fTemp=fPhaseData;

						// Reassign it to edit data
						fPhaseData=fData;

						// Set phase redraw flag
						bPhaseRedraw=TRUE;
					}

					// Redraw status plot from edit data
					UpdateStatusBar(hwndMain, NEW_DATA);

					if(bDataType==FREQ_DATA)
					{
						// Restore main freq data pointer
						fFreqData=fTemp;
					}
					else // Phase data
					{
						// Restore main phase data pointer
						fPhaseData=fTemp;
					}
				}
				break;

				// Accept changes
				// Note: IDOK used to kill CR response of edit controls
				case IDC_EDIT_ACCEPT:
				{
					// Refresh fEditData and fEditTags pointers
					fEditData=fData;
					if(bTags)
					{
						fEditTags=fTags;
					}

					// Reset global data pointers to the permanant data array
					if(bDataType==FREQ_DATA)
					{
						// Put info into log file
						_snprintf(szBuffer, sizeof(szBuffer), "# Changes=%d",
							nChanges);
						WriteLog(24, szBuffer);

						fData=fFreqData;
						if(bTags)
						{
							fTags=fFreqTimetags;
						}
					}
					else // Phase data
					{
						// Put info into log file
						_snprintf(szBuffer, sizeof(szBuffer), "# Changes=%d",
							nChanges);
						WriteLog(23, szBuffer);

						fData=fPhaseData;
						if(bTags)
						{
							fTags=fPhaseTimetags;
						}
					}

					// Reallocate permanant data array
					if((fNew=(F_TYPE*)realloc((char *)fData,
						(nRows+ARRAY_OFFSET+EXTRA_MEM)*
						sizeof(F_TYPE)))==NULL)
					{
						// It would seem that the changes to the data will
						// have to be rejected
						AllocErrorMessage();
					}
					else
					{
						fData=fNew;

						// Copy entire temp data array to permanent data array
						for(i=0; i<fEditData[0]+ARRAY_OFFSET; i++)
						{
							fData[i]=fEditData[i];
						}

						// Reassign the pointer of the permanent data
						// array to the new fData pointer value
						if(bDataType==FREQ_DATA)
						{
							fFreqData=fData;
						}
						else // Phase data
						{
							fPhaseData=fData;
						}
					}

					if(bTags)
					{
						// Reallocate permanant timetag array
						if((fNew=(F_TYPE*)realloc((char *)fTags,
							(nRows+ARRAY_OFFSET+EXTRA_MEM)*
							sizeof(F_TYPE)))==NULL)
						{
							// Now what? The data array has been changed
							// but the timetag array can't be - display
							// error message, clear timetags & close dlg
							AllocErrorMessage();
							ClearTimetagArray(bDataType);
							if(bDataType==FREQ_DATA)
							{
								bFreqTimetags=FALSE;
							}
							else // Phase data
							{
								bPhaseTimetags=FALSE;
							}
						}
						else
						{
							fTags=fNew;

							// Copy entire temp tag array to perm tag array
							for(i=0; i<fEditTags[0]+ARRAY_OFFSET; i++)
							{
								fTags[i]=fEditTags[i];
							}
		
							// Reassign the pointers of the permanent timetag
							// array to the new fTags pointer value
							if(bDataType==FREQ_DATA)
							{
								fFreqTimetags=fTags;
							}
							else // Phase data
							{
								fPhaseTimetags=fTags;
							}
						}
					}

					// Free memory for filenames
					free(szInitialFile);
					free(szInitialFileTitle);
					free(szInitialPhaseFile);
					free(szInitialPhaseFileTitle);
					free(szInitialFreqFile);
					free(szInitialFreqFileTitle);

					// Free the temporary arrays
					free(fEditData);
					if(bTags)
					{
						free(fEditTags);
					}

					// Make sure modeless dialog box is closed
					if(hDlgModeless!=0)
					{
						DestroyWindow(hDlgModeless);
						hDlgModeless=0;
					}

					// Set redraw flag
					if(bDataType==FREQ_DATA)
					{
						bFreqRedraw=TRUE;
					}
					else // Phase data
					{
						bPhaseRedraw=TRUE;
					}

					// Redraw status plot from data
					UpdateStatusBar(hwndMain, NEW_DATA);

					EndDialog (hDlg, 0);
				}
				return TRUE;

				// Reject changes
				case IDCANCEL:
				{
					// Refresh fEditData and fEditTags pointers
					fEditData=fData;
					if(bTags)
					{
						fEditTags=fTags;
					}

					// Reset global data pointers to the permanant data array
					if(bDataType==FREQ_DATA)
					{
						fData=fFreqData;
						if(bTags)
						{
							fTags=fFreqTimetags;
						}
					}
					else // Phase data
					{
						fData=fPhaseData;
						if(bTags)
						{
							fTags=fPhaseTimetags;
						}
					}

					// Do not copy temporary data array to permanent data array
					// Just free the temporary arrays
					free(fEditData);
					if(bTags)
					{
						free(fEditTags);
					}

					// Restore the original # pts, analysis start & end values
					if(bDataType==FREQ_DATA)
					{
						_snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
							nSaveNum);
						SetWindowText(hwndFreqNum, szBuffer);
						_snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
							nSaveStart);
						SetWindowText(hwndFreqStart, szBuffer);
						_snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
							nSaveEnd);
						SetWindowText(hwndFreqEnd, szBuffer);

						// Restore filenames
						strlcpy(szFile, szInitialFile, FILE_NAME_SIZE);
						strlcpy(szFileTitle, szInitialFileTitle,
							FILE_TITLE_SIZE);
						strlcpy(szFreqFile, szInitialFreqFile,
							FILE_NAME_SIZE);
						strlcpy(szFreqFileTitle, szInitialFreqFileTitle,
							FILE_TITLE_SIZE);
						SetWindowText(hwndFreqFilename, szFileTitle);
					}
					else // Phase data
					{
						_snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
							nSaveNum);
						SetWindowText(hwndPhaseNum, szBuffer);
						_snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
							nSaveStart);
						SetWindowText(hwndPhaseStart, szBuffer);
						_snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
							nSaveEnd);
						SetWindowText(hwndPhaseEnd, szBuffer);

						// Restore filenames
						strlcpy(szFile, szInitialFile, FILE_NAME_SIZE);
						strlcpy(szFileTitle, szInitialFileTitle,
							FILE_TITLE_SIZE);
						strlcpy(szPhaseFile, szInitialFile,
							FILE_NAME_SIZE);
						strlcpy(szPhaseFileTitle, szInitialPhaseFileTitle,
							FILE_TITLE_SIZE);
						SetWindowText(hwndPhaseFilename, szFileTitle);
					}

					// Free memory for filenames
					free(szInitialFile);
					free(szInitialFileTitle);
					free(szInitialPhaseFile);
					free(szInitialPhaseFileTitle);
					free(szInitialFreqFile);
					free(szInitialFreqFileTitle);

					// Reset the data change flag
					// Note:  We don't use the local phase data change flag
					// Note:  Only reset the global data change flag if
					// it was not set before edit
					// Reset the global data pointer
					if(bDataType==FREQ_DATA)
					{
						fData=fFreqData;
						bNewFreqData=FALSE;
						bFreqDataChanged=bInitialFreqChange;
						bFreqScaleChanged=bInitialFreqScale;
					}
					else
					{
						fData=fPhaseData;
						bNewPhaseData=FALSE;
						bPhaseDataChanged=bInitialPhaseChange;
						bPhaseScaleChanged=bInitialPhaseScale;
					}

					// Make sure modeless dialog box is closed
					if(hDlgModeless!=0)
					{
						DestroyWindow(hDlgModeless);
						hDlgModeless=0;
					}

					if(bEditingDone) // Was any editing done?
					{
						// Set redraw flag
						if(bDataType==FREQ_DATA)
						{
							bFreqRedraw=TRUE;
						}
						else // Phase data
						{
							bPhaseRedraw=TRUE;
						}

						// Redraw status plot from data
						UpdateStatusBar(hwndMain, NEW_DATA);
					}

					EndDialog (hDlg, 0);
				}
				return TRUE;
			}
		}
		break;

		// Message from the Find or Replace modeless dialog box
		// Informing Edit procedure that a row # has been found
		// or that the Find or Replace box has been closed
		// Message format is like standard Windows WM_COMMAND
		// with Win 3.1 and Win32 compatibility:
		// Win 3.1:
		//     wParam = Child Window ID = 0 (not used)
		//     LOWORD (lParam) Child window handle
		//     HIWORD (lParam) Notification code
		// Win32:
		//     LOWORD wParam = Child Window ID = 0 (not used)
		//     HIWORD (wParam) = Notification code
		//     lParam = Child window handle
		// Notification code is either:
		//     nMatchRow (>=0)
		//     or FIND_NO_MATCH (-1)
		//     or FIND_CANCEL (-2) eg Find box closed
		// The GET_WM_COMMAND_CMD macro extracts the notification code
		// from either HIWORD(lparam) for Win 3.1
		// or HIWORD(wParam) for Win32

		// 02/06/00 This message format is NG - the row #
		// is passed as the 16-bit cmd argument in the
		// GET_WM_COMMAND_MPS macro, so it can't handle row #s
		// greater than 65K.				

		// The best approach is to use SendMessage() directly w/o
		// GET_WM_COMMAND_MPS macro since both params are 32-bits
		// Use lParam to pass hDlg and wParam to pass nRowNum

		// Replace message
		case IDC_REPLACE_ROWCHANGE:
		{
			bReplace=TRUE;
		}
		// Go right into following code for Find Next message
		case IDC_FIND_ROWNUM:
		case IDC_REPLACE_ROWNUM:
		{
			nMatchRow=(INT)wParam;

			// Test for valid row #
			if(nMatchRow>=nRows)
			{
				break;
			}
			
			if(nMatchRow==FIND_NO_MATCH)
			{
				MessageBeep(0); // search failed
			}
			else if(nMatchRow>FIND_NO_MATCH) // search succeded
			{
				// Move to the matching point
				SendMessage(hDataTbl, DTM_SETCURCELL, nColNum,
					(LPARAM) nMatchRow);
				SendMessage(hDataTbl, DTM_SETTOPINDEX, 0,
					(LPARAM) nMatchRow);

				if(bReplace)
				{
					bDoReplace=TRUE;

					// Ask replace modeless dlg whether confirm is on
					if(SendMessage(GetDlgItem((HWND)lParam,
						IDC_REPLACE_CONFIRM), BM_GETCHECK, 0, 0L)==TRUE)

					{
						// Ask for confirmation
						bDoReplace=(MessageBox(hwndMain, "Replace data?",
							"Replace", 	MB_ICONQUESTION | MB_OKCANCEL |
							 MB_DEFBUTTON2) ==IDOK);
					}

					if(bDoReplace)
					{
						// Get the replace value
						GetWindowText(GetDlgItem((HWND)lParam,
							IDC_REPLACE_WITH), szBuffer, BUFFER_SIZE);

						// Convert new value to a number
						fVal=atof(szBuffer);

						// Reformat value
						_snprintf(szBuffer, sizeof(szBuffer), "%+20.13e",
							fVal);
						ShortenExp(szBuffer);

						// Rewrite table cell
						SendMessage(hDataTbl, DTM_SETACCESSPOS, nColNum,
							nMatchRow);
						SendMessage(hDataTbl, DTM_UPDATECELL, DTA_ACCESS,
							(LPARAM)(LPSTR)szBuffer);

						// Update array data
						fData[nMatchRow+ARRAY_OFFSET]=fVal;

						bEditingDone=TRUE;
					
					}
				}
			}
			bReplace=FALSE;
		}
		break;
	 }
	 return FALSE;
}

/*****************************************************************************/
/*                                                                           */
/* 		Function to put 1 data point into row of data table                  */
/*                                                                           */
/*		Revision record:  												     */
/*			01/16/98	Added calls to ShortenExp() to elim 3-digit exps     */
/*                                                                           */
/*****************************************************************************/

int WINAPI GetRowData(LPVMSTRUCT lpData, int nRows)
{
	// Local variables
	// Ints
	int nYear;
	int nDOY;
	int nMonth;
	int nDay;
	int nHour;
	int nMin;
	int nSec;
	static int nSum[] = { 0, 31, 59, 90, 120, 151, 181, 212,
						  243, 273, 304, 334, 365 };
	// Longs
	long lnMJD;
	// F_TYPEs
	F_TYPE fMJD;
	F_TYPE fIntMJD;
	F_TYPE fFractMJD;

	// Check that row # is valid
	if((int)lpData->dwKey<0 || (int)lpData->dwKey>=nRows)
	{
		// Return error code
		return (int) DTE_ROWOUTOFRANGE;
	}

	if(bWideEdit) // Timetags
	{
		// Handle special case where timetag=0 (blank)
		if(fTags[(int)lpData->dwKey+ARRAY_OFFSET]==0)
		{
			sprintf(lpData->lpData, "                 %+20.13e",
				fData[(int)lpData->dwKey+ARRAY_OFFSET]);
			ShortenExp(lpData->lpData);
			return DTE_OKAY;
		}

		// Load data point into table
		// Note: row data is one big string
		// The width of the timetag data must always be the same
		switch(nTagFormat)
		{
			case TAGFORMAT_MJD:
			{
				// Format = %17.8 = 00000000.00000000
				sprintf(lpData->lpData, "%17.8f%+20.13e",
					fTags[(int)lpData->dwKey+ARRAY_OFFSET],
					fData[(int)lpData->dwKey+ARRAY_OFFSET]);
				ShortenExp(lpData->lpData);
			}
			break;

			case TAGFORMAT_DOY:
			{
				// MJD to date conversion
				// Get full MJD
				fMJD=fTags[(int)lpData->dwKey+ARRAY_OFFSET];

				// Round the MJD up by 0.5 sec
				fMJD+=5.78704e-6;

				// Split MJD into integer and fractional parts
				fFractMJD=modf(fMJD, &fIntMJD);
				lnMJD=(long)fIntMJD;

				// Convert integer MJD into date
				MJDToDate(lnMJD, &nDay, &nMonth, &nYear);
				nDOY=nSum[nMonth-1]+nDay;
				if(nYear%4==0 && nYear%100!=0 || nYear%400==0)
				{
					if(nMonth>2)
					{
						nDOY++;
					}
				}

				// Convert fractional MJD into time
				fFractMJD*=24.0;
				nHour=(int)fFractMJD;
				fFractMJD-=nHour;
				fFractMJD*=60.0;
				nMin=(int)fFractMJD;
				fFractMJD-=nMin;
				fFractMJD*=60.0;
				nSec=(int)(fFractMJD);

				// Format = YYYY:DDD:HH:MM:SS
				sprintf(lpData->lpData,
					"%04d:%03d:%02d:%02d:%02d%+20.13e",
					nYear, nDOY, nHour, nMin, nSec,
					fData[(int)lpData->dwKey+ARRAY_OFFSET]);
				ShortenExp(lpData->lpData);
			}
			break;

			case TAGFORMAT_DATE:
			{
				// MJD to date conversion
				// Get full MJD
				fMJD=fTags[(int)lpData->dwKey+ARRAY_OFFSET];

				// Round the MJD up by 0.5 sec
				fMJD+=5.78704e-6;

				// Split MJD into integer and fractional parts
				fFractMJD=modf(fMJD, &fIntMJD);
				lnMJD=(long)fIntMJD;

				// Convert integer MJD into date
				MJDToDate(lnMJD, &nDay, &nMonth, &nYear);

				// Convert fractional MJD into time
				fFractMJD*=24.0;
				nHour=(int)fFractMJD;
				fFractMJD-=nHour;
				fFractMJD*=60.0;
				nMin=(int)fFractMJD;
				fFractMJD-=nMin;
				fFractMJD*=60.0;
				nSec=(int)(fFractMJD);

				// Format = YY:MM:DD:HH:MM:SS
				sprintf(lpData->lpData,
					"%02d:%02d:%02d:%02d:%02d:%02d%+20.13e",
					nYear%100, nMonth, nDay, nHour, nMin, nSec,
					fData[(int)lpData->dwKey+ARRAY_OFFSET]);
				ShortenExp(lpData->lpData);
			}
			break;

			case TAGFORMAT_EXP:
			{
				// Format = %+17.10e = +0.0000000000e+00
				sprintf(lpData->lpData, "%+17.10e%+20.13e",
					fTags[(int)lpData->dwKey+ARRAY_OFFSET],
					fData[(int)lpData->dwKey+ARRAY_OFFSET]);
				ShortenExp(lpData->lpData);
			}
			break;
		}
	}
	else // Non-wide (no timetags)
	{
		// Load data point into table
		sprintf(lpData->lpData, "%+20.13e",
			fData[(int)lpData->dwKey+ARRAY_OFFSET]);
		ShortenExp(lpData->lpData);
	}

   // Return success
   return DTE_OKAY;
}

/*****************************************************************************/
/*                                                                           */
/*                   Find Modeless Dialog Procedure                          */
/*                                                                           */
/* 		Find row number in table where data matches search value             */
/*                                                                           */
/*		Control IDs:                                                         */
/*			IDC_FIND_TEST    Replace logic (<, =, >)                         */
/*			IDC_FIND_WHAT	 Search string	                                 */
/*			IDC_FIND_NEXT    Find Next button                                */
/*			IDC_FIND_REPLACE Replace button                                  */
/*			IDC_FIND_ALL     Replace All button                              */
/*			IDC_FIND_UP		 Up radio button                                 */
/*			IDC_FIND_DOWN	 Down radio button                               */
/*		Messages:                                                            */
/*			IDC_FIND_ROWNUM	Message to parent window with row # info         */
/*                                                                           */
/*                                                                           */
/*		Revision record:  												     */
/*			07/23/97	Fixed error in downwards = search with neg value     */
/*                                                                           */
/*****************************************************************************/

#ifndef _MSC_VER
	#pragma argsused
#endif

BOOL CALLBACK FindDlgProc(HWND hDlg, UINT message, WPARAM wParam,
	LPARAM lParam)
{
	// Local variables
	// Chars
	char szBuffer[BUFFER_SIZE+1];			// Text buffer
	static char szFind[BUFFER_SIZE+1]="";	// Find string
	// Ints
	int i;									// Index
	static int iTest=DATA_EQUAL;			// Test type
	static int nStartRow=0;		// Row number in data table to start search at
	static int nRowNum=FIND_NO_MATCH; 		// Row number where match is found
	// BOOLs
	static BOOL bUp=TRUE;		 			// Search direction
	// F_TYPEs
	static F_TYPE fFind=0.0;   				// Value to find
	static F_TYPE fLim=0.0;   				// Limit on value to find

	switch(message)
	{
		case WM_INITDIALOG:
		{
			// Fill up the list box
			nDummy=ComboBox_ResetContent(GETHCTL(IDC_FIND_TEST));
			if(bDataType)
			{
				nDummy=ComboBox_InsertString(GETHCTL(IDC_FIND_TEST),
					DATA_GREATERTHAN, " y >");
				nDummy=ComboBox_InsertString(GETHCTL(IDC_FIND_TEST),
					DATA_EQUAL, " y =");
				nDummy=ComboBox_InsertString(GETHCTL(IDC_FIND_TEST),
					DATA_LESSTHAN, " y <");
			}
			else
			{
				nDummy=ComboBox_InsertString(GETHCTL(IDC_FIND_TEST),
					DATA_GREATERTHAN, " x >");
				nDummy=ComboBox_InsertString(GETHCTL(IDC_FIND_TEST),
					DATA_EQUAL, " x =");
				nDummy=ComboBox_InsertString(GETHCTL(IDC_FIND_TEST),
					DATA_LESSTHAN, " x <");
			}

			// Select test type
			nDummy=ComboBox_SetCurSel(GETHCTL(IDC_FIND_TEST), iTest);

			// Restore the previous search direction
			Button_SetCheck(GETHCTL(IDC_FIND_UP), bUp);
			Button_SetCheck(GETHCTL(IDC_FIND_DOWN), !bUp);

			// Put the previous value back into Find What?
			// Don't reformat it
			SetWindowText(GETHCTL(IDC_FIND_WHAT), szFind);

			return TRUE;
		}

		case WM_CLOSE:
		{
			// Close modeless dialog box
			DestroyWindow(hDlg);
			hDlgModeless=0;
		}
		break;

		case WM_COMMAND:
		{
			switch(GET_WM_COMMAND_ID(wParam, lParam))
			{
				case IDHELP:
				{
					#if(0)
					// Invoke help file
					HtmlHelp(hDlg, "Stable32.chm::FIND_FUNCTION.HTM",
						HH_DISPLAY_TOPIC, 0L);
					#endif

					#if(0) // New code 11/29/11
					// Invoke help file
					strlcpy(szBuffer, szHelpFile, sizeof(szBuffer));
					strlcat(szBuffer, "::FIND_FUNCTION.HTM", sizeof(szBuffer));
					HtmlHelp(hDlg, szBuffer,
						HH_DISPLAY_TOPIC, 0L);
					#endif

					#if(1) // Trial code 11/30/11
					// Invoke help file using CallHelp() function
					CallHelp(hDlg, "FIND_FUNCTION");
					#endif
				}
				break;

				case IDC_FIND_UP:
				{
					bUp=TRUE;
				}
				break;

				case IDC_FIND_DOWN:
				{
					bUp=FALSE;
				}
				break;

				case IDC_FIND_NEXT:
				{
					// Get the value to find
					GetWindowText(GETHCTL(IDC_FIND_WHAT), szBuffer,
						BUFFER_SIZE);
					fFind=atof(szBuffer);
					// Save find string
					strlcpy(szFind, szBuffer, sizeof(szFind));

					// Get current row number
					// hDataTbl is passed via global handle variable hDialog
					nStartRow=(int)SendMessage(hDialog, DTM_GETCURROW,
						(WPARAM)0, (LPARAM)0);

					// Initialize nRow to NO_MATCH flag
					nRowNum=FIND_NO_MATCH;

					switch(iTest=
						ComboBox_GetCurSel(GETHCTL(IDC_FIND_TEST)))
					{
						case DATA_EQUAL:
						{
							// Get search limit
							fLim=CalcSearchLimit(szBuffer);

							// Search for match,
							// starting at current row number+1
							if(bUp) // Search upwards
							{
								for(i=nStartRow+1; i<fData[0]; i++)
								{
									if(fFind>0) // positive search value
									{
										if(fData[i+ARRAY_OFFSET]>=fFind &&
										   fData[i+ARRAY_OFFSET]<fLim)
										{
											nRowNum=i;
											break;
										}
									}
									if(fFind==0) // zero search value
									{
										if(fData[i+ARRAY_OFFSET]==0)
										{
											nRowNum=i;
											break;
										}
									}
									if(fFind<0) // negative search value
									{
										if(fData[i+ARRAY_OFFSET]<=fFind &&
										   fData[i+ARRAY_OFFSET]>fLim)
										{
											nRowNum=i;
											break;
										}
									}
								}
							}
							else // Search downwards same way
							{
								for(i=nStartRow-1; i>=0; i--)
								{
									if(fFind>0) // positive search value
									{
										if(fData[i+ARRAY_OFFSET]>=fFind &&
										   fData[i+ARRAY_OFFSET]<fLim)
										{
											nRowNum=i;
											break;
										}
									}
									if(fFind==0) // zero search value
										{
										if(fData[i+ARRAY_OFFSET]==0)
										{
											nRowNum=i;
											break;
										}
									}
									if(fFind<0) // negative search value
									{
										if(fData[i+ARRAY_OFFSET]<=fFind &&
										   fData[i+ARRAY_OFFSET]>fLim)
										{
											nRowNum=i;
											break;
										}
									}
								}
							}
						}
						break;

						case DATA_GREATERTHAN:
						{
							if(bUp) // Search upwards
							{
								for(i=nStartRow+1; i<fData[0]; i++)
								{
									if(fData[i+ARRAY_OFFSET]>fFind)
									{
										nRowNum=i;
										break;
									}
								}
							}
							else // Search downwards same way
							{
								for(i=nStartRow-1; i>=0; i--)
								{
									if(fData[i+ARRAY_OFFSET]>fFind)
									{
										nRowNum=i;
										break;
									}
								}
							}
						}
						break;

						case DATA_LESSTHAN:
						{
							if(bUp) // Search upwards
							{
								for(i=nStartRow+1; i<fData[0]; i++)
								{
									if(fData[i+ARRAY_OFFSET]<fFind)
									{
										nRowNum=i;
										break;
									}
								}
							}
							else // Search downwards same way
							{
								for(i=nStartRow-1; i>=0; i--)
								{
									if(fData[i+ARRAY_OFFSET]<fFind)
									{
										nRowNum=i;
										break;
									}
								}
							}
						}
						break;
					}
					// Send a message to the parent (Edit) window
					// Message format (like standard Windows WM_COMMAND):
					// Note: Has changes for Win 3.1 and Win32 compatibility
					// Win 3.1:
					//     wParam = Child Window ID = 0 (not used)
					//     LOWORD (lParam) = Child window handle
					//     HIWORD (lParam) = Notification code
					// Win32:
					//     LOWORD wParam = Child Window ID = 0 (not used)
					//     HIWORD (wParam) = Notification code
					//     lParam = Child window handle
					// Notification code = nRowNum is either:
					//     nMatchRow (>=0)
					//     or FIND_NO_MATCH (-1)
					//     or FIND_CANCEL (-2) e.g. Find box closed

					// 02/06/00 This message format is NG - the row #
					// is passed as the 16-bit cmd argument in the
					// GET_WM_COMMAND_MPS macro, so it can't handle row #s
					// greater than 65K.  The row # must be a 32-bit argument.					

					// The best approach is to use SendMessage() directly w/o
					// GET_WM_COMMAND_MPS macro since both params are 32-bit
					// Use lParam to pass hDlg and wParam to pass nRowNum

					// Send find nRowNum and hDlg to edit dialog
					SendMessage(GetParent(hDlg), IDC_FIND_ROWNUM,
						(WPARAM)nRowNum, (LPARAM)hDlg);

				}
				break;

				case IDCANCEL:
				{
					// Send a message to the parent window (see above)
					// Send cancel message to edit dialog
					SendMessage(GetParent(hDlg), IDC_FIND_ROWNUM,
						(WPARAM)FIND_CANCEL, 0);

					// Close modeless dialog box
					DestroyWindow(hDlg);
					hDlgModeless=0;
				}
				break;
			}
		}
		break;
	 }
	 return FALSE;
}

/*****************************************************************************/
/*                                                                           */
/*                   Replace Modeless Dialog Procedure                       */
/*                                                                           */
/*		Control IDs:                                                         */
/*			IDC_REPLACE_TEST    Replace logic (<, =, >)                      */
/*			IDC_REPLACE_WHAT	Search string	                             */
/*			IDC_REPLACE_WITH    Replace string                               */
/*			IDC_REPLACE_FIND    Find button                                  */
/*			IDC_REPLACE_REPLACE Replace button                               */
/*			IDC_REPLACE_ALL     Replace All button                           */
/*			IDC_REPLACE_UP		Up radio button                              */
/*			IDC_REPLACE_DOWN	Down radio button                            */
/*			IDC_REPLACE_CONFIRM Confirm replacements check box               */
/*		Messages:                                                            */
/*			IDC_REPLACE_ROWNUM		Msg to parent window with row # info     */
/*			IDC_REPLACE_ROWCHANGE	Msg to parent window with row # info     */
/*                                                                           */
/*		Revision record:  												     */
/*          12/30/96	Added Replace All action                             */
/*			07/23/97	Fixed error in downwards = search with neg value     */
/*			02/06/00	Changed IDC_REPLACE_ROWNUM and IDC_REPLACE_ROWCHANGE */
/*						message formats to allow passing 32-bit row numbers  */
/*                                                                           */
/*****************************************************************************/

#ifndef _MSC_VER
	#pragma argsused
#endif

BOOL CALLBACK ReplaceDlgProc(HWND hDlg, UINT message, WPARAM wParam,
	LPARAM lParam)
{
	// Local variables
	// Chars
	char szBuffer[BUFFER_SIZE+1];				// Text buffer
	static char szFind[BUFFER_SIZE+1]="";	    // Find string
	static char szReplace[BUFFER_SIZE+1]="0.0";	// Replace string
	// Ints
	int i;										// Index
	static int iTest=DATA_EQUAL;				// Test type
	static int nStartRow=0;		// Row # in data table to start search at
	static int nRowNum=FIND_NO_MATCH; 			// Row # where match is found
	// BOOLs
	static BOOL bUp=TRUE;		 				// Search direction
	static BOOL bReplace=FALSE;    				// Replace data flag
	static BOOL bConfirm=TRUE;					// Confirm replacements flag
	static BOOL bMore;							// Flag to control Replace All
	// F_TYPEs
	static F_TYPE fFind=0.0;   					// Value to find
	static F_TYPE fLim=0.0;   					// Limit on value to find
	static F_TYPE fReplace=0.0; 				// Replacement value

	switch(message)
	{
		case WM_INITDIALOG:
		{
			// Fill up the list box
			nDummy=ComboBox_ResetContent(GETHCTL(IDC_REPLACE_TEST));
			if(bDataType)
			{
				nDummy=ComboBox_InsertString(GETHCTL(IDC_REPLACE_TEST),
					DATA_GREATERTHAN, " y >");
				nDummy=ComboBox_InsertString(GETHCTL(IDC_REPLACE_TEST),
					DATA_EQUAL, " y =");
				nDummy=ComboBox_InsertString(GETHCTL(IDC_REPLACE_TEST),
					DATA_LESSTHAN, " y <");
			}
			else
			{
				nDummy=ComboBox_InsertString(GETHCTL(IDC_REPLACE_TEST),
					DATA_GREATERTHAN, " x >");
				nDummy=ComboBox_InsertString(GETHCTL(IDC_REPLACE_TEST),
					DATA_EQUAL, " x =");
				nDummy=ComboBox_InsertString(GETHCTL(IDC_REPLACE_TEST),
					DATA_LESSTHAN, " x <");
			}

			// Select test type
			nDummy=ComboBox_SetCurSel(GETHCTL(IDC_REPLACE_TEST), iTest);

			// Restore the previous search direction
			Button_SetCheck(GETHCTL(IDC_REPLACE_UP), bUp);
			Button_SetCheck(GETHCTL(IDC_REPLACE_DOWN), !bUp);

			// Restore the Confirm Replacements state
			Button_SetCheck(GETHCTL(IDC_REPLACE_CONFIRM), bConfirm);

			// Put the previous values back into Find What? and Replace With
			// Don't reformat it
			SetWindowText(GETHCTL(IDC_REPLACE_WHAT), szFind);
			SetWindowText(GETHCTL(IDC_REPLACE_WITH), szReplace);

			return TRUE;
		}

		case WM_CLOSE:
		{
			DestroyWindow(hDlg);
			hDlgModeless=0;
		}
		break;

		case WM_COMMAND:
		{
			switch(GET_WM_COMMAND_ID(wParam, lParam))
			{
				case IDHELP:
				{
					#if(0)
					// Invoke help file
					HtmlHelp(hDlg, "Stable32.chm::REPLACE_FUNCTION.HTM",
						HH_DISPLAY_TOPIC, 0L);
					#endif

					#if(0) // New code 11/29/11
					// Invoke help file
					strlcpy(szBuffer, szHelpFile, sizeof(szBuffer));
					strlcat(szBuffer, "::REPLACE_FUNCTION.HTM", sizeof(szBuffer));
					HtmlHelp(hDlg, szBuffer,
						HH_DISPLAY_TOPIC, 0L);
					#endif

					#if(1) // Trial code 11/30/11
					// Invoke help file using CallHelp() function
					CallHelp(hDlg, "REPLACE_FUNCTION");
					#endif
				}
				break;

				case IDC_REPLACE_UP:
				{
					bUp=TRUE;
				}
				break;

				case IDC_REPLACE_DOWN:
				{
					bUp=FALSE;
				}
				break;

				case IDC_REPLACE_CONFIRM:
				{
					bConfirm=
						(BOOL)Button_GetCheck(GETHCTL(IDC_REPLACE_CONFIRM));
				}
				break;

				case IDC_REPLACE_REPLACE:
				{
					// Set the replace flag
					bReplace=TRUE;

					// Get the replace value
					GetWindowText(GETHCTL(IDC_REPLACE_WITH), szBuffer,
						BUFFER_SIZE);
					fReplace=atof(szBuffer);
					// Save replace string
					strlcpy(szReplace, szBuffer, sizeof(szReplace));
				}
				// Go right into the following code
				case IDC_REPLACE_FIND:
				{
					// Get the value to find
					GetWindowText(GETHCTL(IDC_REPLACE_WHAT), szBuffer,
						BUFFER_SIZE);
					fFind=atof(szBuffer);
					// Save find string
					strlcpy(szFind, szBuffer, sizeof(szFind));  

					// Get current row number
					// hDataTbl is passed via global handle variable hDialog
					nStartRow=(int)SendMessage(hDialog, DTM_GETCURROW,
						(WPARAM)0, (LPARAM)0);

					// Initialize nRow to NO_MATCH flag
					nRowNum=FIND_NO_MATCH;

					switch(iTest=
						ComboBox_GetCurSel(GETHCTL(IDC_REPLACE_TEST)))
					{
						case DATA_EQUAL:
						{
							// Get search limit
							fLim=CalcSearchLimit(szBuffer);

							// Search for match,
							// starting at current row number+1
							if(bUp) // Search upwards
							{
								for(i=nStartRow+1; i<fData[0]; i++)
								{
									if(fFind>0) // positive search value
									{
										if(fData[i+ARRAY_OFFSET]>=fFind &&
										   fData[i+ARRAY_OFFSET]<fLim)
										{
											nRowNum=i;
											break;
										}
									}
									if(fFind==0) // zero search value
									{
										if(fData[i+ARRAY_OFFSET]==0)
										{
											nRowNum=i;
											break;
										}
									}
									if(fFind<0) // negative search value
									{
										if(fData[i+ARRAY_OFFSET]<=fFind &&
										   fData[i+ARRAY_OFFSET]>fLim)
										{
											nRowNum=i;
											break;
										}
									}
								}
							}
							else // Search downwards same way
							{
								for(i=nStartRow-1; i>=0; i--)
								{
									if(fFind>0) // positive search value
									{
										if(fData[i+ARRAY_OFFSET]>=fFind &&
										   fData[i+ARRAY_OFFSET]<fLim)
										{
											nRowNum=i;
											break;
										}
									}
									if(fFind==0) // zero search value
										{
										if(fData[i+ARRAY_OFFSET]==0)
										{
											nRowNum=i;
											break;
										}
									}
									if(fFind<0) // negative search value
									{
										if(fData[i+ARRAY_OFFSET]<=fFind &&
										   fData[i+ARRAY_OFFSET]>fLim)
										{
											nRowNum=i;
											break;
										}
									}
								}
							}
						}
						break;

						case DATA_GREATERTHAN:
						{
							if(bUp) // Search upwards
							{
								for(i=nStartRow+1; i<fData[0]; i++)
								{
									if(fData[i+ARRAY_OFFSET]>fFind)
									{
										nRowNum=i;
										break;
									}
								}
							}
							else // Search downwards same way
							{
								for(i=nStartRow-1; i>=0; i--)
								{
									if(fData[i+ARRAY_OFFSET]>fFind)
									{
										nRowNum=i;
										break;
									}
								}
							}
						}
						break;

						case DATA_LESSTHAN:
						{
							if(bUp) // Search upwards
							{
								for(i=nStartRow+1; i<fData[0]; i++)
								{
									if(fData[i+ARRAY_OFFSET]<fFind)
									{
										nRowNum=i;
										break;
									}
								}
							}
							else // Search downwards same way
							{
								for(i=nStartRow-1; i>=0; i--)
								{
									if(fData[i+ARRAY_OFFSET]<fFind)
									{
										nRowNum=i;
										break;
									}
								}
							}
						}
						break;
					}

					// Send a message to the parent (Edit) window
					// Message format (like standard Windows WM_COMMAND):
					// Note: Has changes for Win 3.1 and Win32 compatibility
					// Win 3.1:
					//     wParam = Child Window ID = 0 (not used)
					//     LOWORD (lParam) = Child window handle
					//     HIWORD (lParam) = Notification code
					// Win32:
					//     LOWORD wParam = Child Window ID = 0 (not used)
					//     HIWORD (wParam) = Notification code
					//     lParam = Child window handle
					// Notification code = nRowNum is either:
					//     nMatchRow (>=0)
					//     or FIND_NO_MATCH (-1)
					//     or FIND_CANCEL (-2) e.g. Find box closed

					// 02/06/00 This message format is NG - the row #
					// is passed as the 16-bit cmd argument in the
					// GET_WM_COMMAND_MPS macro, so it can't handle row #s
					// greater than 65K.
					
					// The best approach is to use SendMessage() directly w/o
					// GET_WM_COMMAND_MPS macro since both params are 32-bits
					// Use wParam to pass nRowNum and lParam to pass hDlg

					if(bReplace)
					{
						// Send replace nRowNum and hDlg to edit dialog
						SendMessage(GetParent(hDlg), IDC_REPLACE_ROWCHANGE,
							(WPARAM)nRowNum, (LPARAM)hDlg);

						// Terminate Replace All if done
						if(nRowNum==FIND_NO_MATCH)
						{
							bMore=FALSE;
						}
					}
					else
					{
						// Send replace nRowNum and hDlg to edit dialog
						SendMessage(GetParent(hDlg), IDC_REPLACE_ROWNUM,
							(WPARAM)nRowNum, (LPARAM)hDlg);
					}
					bReplace=FALSE;
				}
				break;

				case IDC_REPLACE_ALL:
				{
					bMore=TRUE;

					while(bMore)
					{
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_REPLACE_REPLACE, NULL, 0));
					}
				}
				break;

				case IDCANCEL:
				{
					DestroyWindow(hDlg);
					hDlgModeless=0;
				}
				break;
			}
		}
		break;
	 }
	 return FALSE;
}

/*****************************************************************************/

