/*****************************************************************************/
/*                                                                           */
/*                            STABLE_F.C                                     */
/*                                                                           */
/* 		Stable Windows 95/NT File Functions                                  */
/*                                                                           */
/*      FileInitialize()                                                     */
/*      FileOpenDlg()                                                        */
/*      FileSaveDlg()                                                        */
/*      FileOpenHookProc()                                                   */
/*		FileSaveHookProc()                                                   */
/*      FileLength()                                                         */
/*		MulticolumnDataRead()                                                */
/*		MulticolumnDataAdd()                                                 */
/*		FileSize()                                                           */
/*		FileInfo()                                                           */
/*      SigmaRead()                                                          */
/*		ReallocSigmaArrays()                                                 */
/*		DataWrite()                                                          */
/*		PSDWrite()                                                           */
/*		AutoWrite()                                                          */
/*		CommonDialogError()                                                  */
/*		TauInfo()                                                            */
/*      AddendInfo()                                                         */
/*      Multiplier Info()                                                    */
/*      TagScaleInfo()                                                       */
/*      TagOffsetInfo()                                                      */
/*                                                                           */
/* 		Note: Some of these functions are adapted from POPFILE.C by C.       */
/*		Petzold, "Programming Windows 3.1", Chapter 10.                      */
/*                                                                           */
/*      Overview of file reading functions:                                  */
/*      File reading starts with a call to ReadData(), which uses the        */
/*		FileInfo() function to determine the file size.  If it exceeds       */
/*      nMaxSize, the TooBig box opens.  If it has multiple columns,         */
/*      the Multicolumn or Multi box opens (the latter for >8 columns).      */
/*      The Multicolumn Details box can be opened from those.  Finally,      */
/*      the MulticolumnDataRead() function is used to read the data.         */
/*                                                                           */
/*      04/06/97:  Added code to convert slashes and colons to zeros         */
/*      in data reading functions.  This allows a line with either           */
/*      slashes (e.g. 04/06/97) or colons (e.g. 09:04) to be read.  While    */
/*      the date & time info is mangled, at least the data column can be     */
/*      selected and read, and the dates and times only take up 1 column.    */
/*                                                                           */
/*      08/11/08:  Some major changes were made to the file reading code.    */
/*      The Multicolumn Details box was added, and a different method        */
/*      of parsing the input lines was used in the FileInfo() and            */
/*      MulticolumnDataRead() functions.  The Multicolumn Details box        */
/*      allows certain date and time formats to be converted into MJD        */
/*      and used in the Timetags dialog to generate MJD timetags.            */
/*                                                                           */
/*		Revision record:                                                     */
/*                                                                           */
/*			06/29/97	Use GET_WM_COMMAND_ID, etc. WIN32 compatibility      */
/*						macros to process WM_COMMAND messages:               */
/*						wParam <- GET_WM_COMMAND_ID(wParam, lParam)          */
/*						Replaced WORD with UINT                              */
/*			06/30/97	Replaced FAR PASCAL _export with WINAPI              */
/*			07/12/97	More Win 32 compatibility changes                    */
/*			12/24/97	Made changes for AOK BC 32 bit compliation           */
/*			07/07/98	Added user-defined file extension option handling    */
/*			12/11/99	Added European comma decimal point handling          */
/*			05/01/01	Added *.phd filename extension                       */
/*			05/06/01	Added *.frd filename extension                       */
/*			08/06/01	Added TauInfo()                                      */
/*			08/22/01	Added ExplorerCornBox option                         */
/*			08/10/02	Added PSDWrite()                                     */
/*			08/10/02	Added AutoWrite()                                    */
/*          07/22/07    Changed to safe string functions                     */
/*          03/22/10	Added AddendInfo(), MultiplierInfo().                */
/*                      TagScaleInfo() and TagOffsetInfo()                   */
/*			06/03/11	Added *.txt filename extension                       */
/*			09/24/11	Store 1st UNIX comment read as subtitle              */
/*			11/03/11	Added *.csv filename extension                       */
/*                                                                           */
/*   (c) 1996-2011 W. Riley Hamilton Technical Services All Rights Reserved  */
/*                                                                           */
/*****************************************************************************/

#define STRICT							// For Win32 compatibility
#define OFN_ENABLESIZING 0x00800000		// To allow resizing option
										// in Windows 98 and 2000 or later

#include "stable_w.h"					// STABLE_W header file
#include <dlgs.h>						// Common dialog box header file

// This OPENFILENAME structure is local to this module
// Only the relevant members are passed to and from the calling module
// Modified to use custom template with phase/frequency filetype selection

static OPENFILENAME ofn ;
static char szFilePattern[BIG_BUFFER_SIZE];

/*****************************************************************************/
void FileInitialize (BOOL bData, BOOL bRead, HWND hwnd, HANDLE hInstance,
	LPSTR lpstrTitle, int nExt)
{
	int i=0;
	
	ofn.lStructSize = sizeof (OPENFILENAME);
	ofn.hwndOwner = hwnd;
	ofn.hInstance = hInstance;

	// Code to allow file pattern to be changed
	// Nominal file pattern
	// Note: | used as placeholder for EOS so strings can be manipulated
	if(bData)
	{
		// strlcpy(szFilePattern,"Data Files (*.dat)|*.dat|");
		#if(0)
		strlcpy(szFilePattern,
			"Data Files (*.dat)|*.dat|Phase Data Files (*.phd)|*.phd|"
			"Frequency Data Files (*.frd)|*.frd|",
			sizeof(szFilePattern));
		#endif
		#if(0) // *.txt added 06/03/11
		strlcpy(szFilePattern,
			"Data Files (*.dat)|*.dat|Text Files (*.txt)|*.txt|"
			"Phase Data Files (*.phd)|*.phd|"
			"Frequency Data Files (*.frd)|*.frd|",
			sizeof(szFilePattern));
		#endif
		#if(1) // *.csv added 11/03/11
		strlcpy(szFilePattern,
			"Data Files (*.dat)|*.dat|Text Files (*.txt)|*.txt|"
			"Comma-Separated Value Files (*.csv)|*.csv|"
			"Phase Data Files (*.phd)|*.phd|"
			"Frequency Data Files (*.frd)|*.frd|",
			sizeof(szFilePattern));
		#endif
	}
	else // Stability file
	{
		strlcpy(szFilePattern,"Stability Files (*.tau)|*.tau|",
			sizeof(szFilePattern));
	}
	
	// Add external filename pattern from config file (may be empty)
	strlcat(szFilePattern, szExternalPattern, sizeof(szFilePattern));

	// Add final standard one
	strlcat(szFilePattern, "All Files (*.*)|*.*|", sizeof(szFilePattern));

	// Replace placeholder characters with EOSs
	while(szFilePattern[i]!='\0')
	{
		if(szFilePattern[i]=='|')
		{
			szFilePattern[i]='\0';
		}
		i++;
	}

	// Assign pointer to file pattern
	ofn.lpstrFilter=(LPSTR)szFilePattern;

	// No user defined file extension handling
	ofn.lpstrCustomFilter = NULL;
	ofn.nMaxCustFilter    = 0;

	ofn.nFilterIndex      = 1;				// Was 0 w/o user defined choice
	ofn.lpstrFile         = NULL;           // Set in Open and Close functions
	ofn.nMaxFile          = _MAX_PATH;
	ofn.lpstrFileTitle    = NULL;           // Set in Open and Close functions
	ofn.nMaxFileTitle     = _MAX_FNAME + _MAX_EXT;
	ofn.lpstrInitialDir   = NULL;
	ofn.lpstrTitle        = lpstrTitle;     // Pointer to title bar string
	ofn.Flags             = 0;              // Set in Open and Close functions
	ofn.nFileOffset       = 0;
	ofn.nFileExtension    = 0;
	ofn.lpstrDefExt       = "";
	ofn.lCustData         = 0L;

	if(bData) // Phase, frequency or archive data
	{
		if(bRead)
		{
			if(bAdd)
			{
				ofn.lpTemplateName = "ExplorerAddBox";
			}
			else // Open
			{
				if((strcmp(szBoxTitle, "3-Cornered Hat")==0))
				{
					ofn.lpTemplateName = "ExplorerCornBox";
				}
				else
				{
					// Note:  Original dialog box template was
					// "ExplorerOpenBox" - can still use it
					// Or use "NewExplorerOpenBox" instead to allow
					// setting of averaging factor
					// No code changes needed for either choice
					ofn.lpTemplateName = "NewExplorerOpenBox";
				}
			}
			ofn.lpfnHook = FileOpenHookProc;
		}
		else // Save
		{
			ofn.lpTemplateName = "ExplorerSaveBox";
			ofn.lpfnHook = FileSaveHookProc;
		}
	}
	else // Stability file
	{
		ofn.lpTemplateName = "ExplorerStabBox";
		ofn.lpfnHook = StabOpenHookProc;
	}
}

/*****************************************************************************/

// This OPENFILENAME structure is local to this module
// Only the relevant members are passed to and from the calling module

static OPENFILENAME ofn ;
static char szFilePattern[BIG_BUFFER_SIZE];

/*****************************************************************************/
void InsertFileInitialize (HWND hwnd, HANDLE hInstance,	LPSTR lpstrTitle)
{
	int i=0;
	
	ofn.lStructSize = sizeof (OPENFILENAME);
	ofn.hwndOwner = hwnd;
	ofn.hInstance = hInstance;

	// TKF file pattern
	// Note: | used as placeholder for EOS so strings can be manipulated
	strlcpy(szFilePattern,"TKF Files (*.tkf)|*.tkf|",
			sizeof(szFilePattern));
	
	// Add standard one
	strlcat(szFilePattern, "All Files (*.*)|*.*|", sizeof(szFilePattern));

	// Replace placeholder characters with EOSs
	while(szFilePattern[i]!='\0')
	{
		if(szFilePattern[i]=='|')
		{
			szFilePattern[i]='\0';
		}
		i++;
	}

	// Assign pointer to file pattern
	ofn.lpstrFilter=(LPSTR)szFilePattern;

	// No user defined file extension handling
	ofn.lpstrCustomFilter = NULL;
	ofn.nMaxCustFilter    = 0;

	ofn.nFilterIndex      = 1;				// Was 0 w/o user defined choice
	ofn.lpstrFile         = NULL;           // Set in Open and Close functions
	ofn.nMaxFile          = _MAX_PATH;
	ofn.lpstrFileTitle    = NULL;           // Set in Open and Close functions
	ofn.nMaxFileTitle     = _MAX_FNAME + _MAX_EXT;
	ofn.lpstrInitialDir   = NULL;
	ofn.lpstrTitle        = lpstrTitle;     // Pointer to title bar string
	ofn.Flags             = 0;              // Set in Open and Close functions
	ofn.nFileOffset       = 0;
	ofn.nFileExtension    = 0;
	ofn.lpstrDefExt       = "";
	ofn.lCustData         = 0L;
	ofn.lpTemplateName = "ExplorerOpenBox";
	ofn.lpfnHook = FileOpenHookProc;
}

/*****************************************************************************/

BOOL FileOpenDlg (BOOL bCustom, HWND hwnd, LPSTR lpstrFileName,
					 LPSTR lpstrTitleName)
{
	// Important: If filename initialization is not necessary
	// The lpstrFileName must be a null string (e.g., "")
	// or the GetOpenFileName() dialog box will not open
	// Ref: Windows95 Common Controls & Messages API Bible, p 46
	// or lpstrFile member of OPENFILENAME strut reference

	// Local variables
	int nFlag=0;				 // Set to OFN_ENABLESIZING to allow resizing
    char szExt[BUFFER_SIZE];     // Text buffer for filename extension

	ofn.hwndOwner         = hwnd ;
	ofn.lpstrFile         = lpstrFileName ;
	ofn.lpstrFileTitle    = lpstrTitleName ;

	// Get the current filename extension (including the dot)
	_splitpath(ofn.lpstrFile, NULL, NULL, NULL, szExt);

	#if(0)
	// Set generic file type
	ofn.nFilterIndex=4;

	// Set the file type to agree with it for dat, phd & frd
	if( ((strcmp(szExt, ".dat"))==0) || ((strcmp(szExt, ".DAT"))==0) )
	{
		ofn.nFilterIndex=1;
	}
	if( ((strcmp(szExt, ".phd"))==0) || ((strcmp(szExt, ".PHD"))==0) )
	{
		ofn.nFilterIndex=2;
	}
	if( ((strcmp(szExt, ".frd"))==0) || ((strcmp(szExt, ".FRD"))==0) )
	{
		ofn.nFilterIndex=3;
	}
	#endif

	#if(0) // *.txt file extension added 06/03/11
	// Set generic file type
	ofn.nFilterIndex=5;

	// Set the file type to agree with it for dat, txt, phd & frd
	if( ((strcmp(szExt, ".dat"))==0) || ((strcmp(szExt, ".DAT"))==0) )
	{
		ofn.nFilterIndex=1;
	}
	if( ((strcmp(szExt, ".txt"))==0) || ((strcmp(szExt, ".TXT"))==0) )
	{
		ofn.nFilterIndex=2;
	}
	if( ((strcmp(szExt, ".phd"))==0) || ((strcmp(szExt, ".PHD"))==0) )
	{
		ofn.nFilterIndex=3;
	}
	if( ((strcmp(szExt, ".frd"))==0) || ((strcmp(szExt, ".FRD"))==0) )
	{
		ofn.nFilterIndex=4;
	}
	#endif

	#if(1) // *.csv file extension added 11/03/11
	// Set generic file type
	ofn.nFilterIndex=6;

	// Set the file type to agree with it for dat, txt, phd & frd
	if( ((strcmp(szExt, ".dat"))==0) || ((strcmp(szExt, ".DAT"))==0) )
	{
		ofn.nFilterIndex=1;
	}
	if( ((strcmp(szExt, ".txt"))==0) || ((strcmp(szExt, ".TXT"))==0) )
	{
		ofn.nFilterIndex=2;
	}
	if( ((strcmp(szExt, ".phd"))==0) || ((strcmp(szExt, ".PHD"))==0) )
	{
		ofn.nFilterIndex=3;
	}
	if( ((strcmp(szExt, ".frd"))==0) || ((strcmp(szExt, ".FRD"))==0) )
	{
		ofn.nFilterIndex=4;
	}
	if( ((strcmp(szExt, ".csv"))==0) || ((strcmp(szExt, ".CSV"))==0) )
	{
		ofn.nFilterIndex=5;
	}
	#endif

	if(bCustom)
	{
		ofn.Flags             = OFN_HIDEREADONLY | nFlag |
								OFN_ENABLETEMPLATE | OFN_ENABLEHOOK |
								OFN_SHAREAWARE | OFN_EXPLORER;
	}
	else // Use std Explorer-style dlg
	{
		ofn.Flags             = OFN_CREATEPROMPT | OFN_HIDEREADONLY |
								OFN_SHAREAWARE;
	}

	return GetOpenFileName (&ofn) ;
}

/*****************************************************************************/

BOOL FileSaveDlg (BOOL bCustom, HWND hwnd, LPSTR lpstrFileName,
					 LPSTR lpstrTitleName)
{
	// Local variables
	int nFlag=0;	// Set to OFN_ENABLESIZING to allow resizing
    char szExt[BUFFER_SIZE];     // Text buffer for filename extension

	ofn.hwndOwner         = hwnd ;
	ofn.lpstrFile         = lpstrFileName ;
	ofn.lpstrFileTitle    = lpstrTitleName ;

	// Get the current filename extension (including the dot)
	_splitpath(ofn.lpstrFile, NULL, NULL, NULL, szExt);

	#if(0)
	// Set generic file type
	ofn.nFilterIndex=4;

	// Set the file type to agree with it for dat, phd & frd
	if( ((strcmp(szExt, ".dat"))==0) || ((strcmp(szExt, ".DAT"))==0) )
	{
		ofn.nFilterIndex=1;
	}
	if( ((strcmp(szExt, ".phd"))==0) || ((strcmp(szExt, ".PHD"))==0) )
	{
		ofn.nFilterIndex=2;
	}
	if( ((strcmp(szExt, ".frd"))==0) || ((strcmp(szExt, ".FRD"))==0) )
	{
		ofn.nFilterIndex=3;
	}
	#endif

	#if(0) // *.txt added 06/03/11
	// Set generic file type
	ofn.nFilterIndex=5;

	// Set the file type to agree with it for dat, phd & frd
	if( ((strcmp(szExt, ".dat"))==0) || ((strcmp(szExt, ".DAT"))==0) )
	{
		ofn.nFilterIndex=1;
	}
	if( ((strcmp(szExt, ".txt"))==0) || ((strcmp(szExt, ".TXT"))==0) )
	{
		ofn.nFilterIndex=2;
	}
	if( ((strcmp(szExt, ".phd"))==0) || ((strcmp(szExt, ".PHD"))==0) )
	{
		ofn.nFilterIndex=3;
	}
	if( ((strcmp(szExt, ".frd"))==0) || ((strcmp(szExt, ".FRD"))==0) )
	{
		ofn.nFilterIndex=4;
	}
	#endif

	#if(1) // *.csv added 11/03/11
	// Set generic file type
	ofn.nFilterIndex=6;

	// Set the file type to agree with it for dat, phd & frd
	if( ((strcmp(szExt, ".dat"))==0) || ((strcmp(szExt, ".DAT"))==0) )
	{
		ofn.nFilterIndex=1;
	}
	if( ((strcmp(szExt, ".txt"))==0) || ((strcmp(szExt, ".TXT"))==0) )
	{
		ofn.nFilterIndex=2;
	}
	if( ((strcmp(szExt, ".phd"))==0) || ((strcmp(szExt, ".PHD"))==0) )
	{
		ofn.nFilterIndex=3;
	}
	if( ((strcmp(szExt, ".frd"))==0) || ((strcmp(szExt, ".FRD"))==0) )
	{
		ofn.nFilterIndex=4;
	}
	if( ((strcmp(szExt, ".csv"))==0) || ((strcmp(szExt, ".CSV"))==0) )
	{
		ofn.nFilterIndex=5;
	}
	#endif

	if(bCustom)
	{
		ofn.Flags             = OFN_HIDEREADONLY | nFlag |
								OFN_ENABLETEMPLATE | OFN_ENABLEHOOK |
								OFN_SHAREAWARE | OFN_EXPLORER;
	}
	else // Use std Explorer-stype dlg
	{
		ofn.Flags             = OFN_CREATEPROMPT | OFN_HIDEREADONLY |
								OFN_SHAREAWARE;
	}
	return GetSaveFileName (&ofn) ;
}

/*****************************************************************************/
/*                                                                           */
/*		Hook Function for Custom File Open Dialog Box                        */
/*                                                                           */
/*      Processes bDataType info from phase and frequency radio buttons      */
/*			12/20/96		Added help support                               */
/*			12/06/00		Trial code for Explorer type dialog box          */
/*			12/08/00		Added OFN_ENABLEWSIZING                          */
/*							Eliminated OFN_SHOWHELP                          */
/*							Added help button to custom template instead     */
/*							This makes overall dialog box more compact       */
/*							Changed hook proc code for new help button       */
/*			08/05/01		Added setting of data type to correspond with    */
/*							*.phd and *.frd filename extensions              */
/*			10/14/03		Added provisions for averaging data during read  */
/*                                                                           */
/*****************************************************************************/

#ifndef _MSC_VER
	#pragma argsused
#endif


UINT CALLBACK FileOpenHookProc(HWND hDlg, UINT msg, WPARAM wParam,
	LPARAM lParam)
{
	// Local variables
	// Chars
	char szBuffer[BUFFER_SIZE+1];	// General text buffer
	// BOOLs
	static BOOL bErrorTrap=FALSE;   // Flag for entry error
	// Ints
	static int nAvgFactor;          // Averaging factor
	int nEntry;                     // User avg factor entry to be validated
	int nPrev;                      // Previous value
	double fEntry;                     // Float to check for AF > int
	OFNOTIFY *ofnotify;
	LPOPENFILENAME ofn;
	NMHDR hdr;
	UINT code;
	RECT rect;
	INT nX;
	INT nY;
	
	switch(msg)
	{
		case WM_INITDIALOG:
		{
			// Initialize the dialog box radio buttons
			Button_SetCheck(GETHCTL(IDC_FILE_FREQ),
				(bDataType && !bArchiveData));
			Button_SetCheck(GETHCTL(IDC_FILE_PHASE),
				((1-bDataType) && !bArchiveData));
			Button_SetCheck(GETHCTL(IDC_FILE_ARCHIVE), bArchiveData);

			// Set averaging factor to default
			nBigAvgFactor=1;
			_snprintf(szBuffer, sizeof(szBuffer), "%d", nBigAvgFactor);
			SetWindowText(GETHCTL(IDC_FILE_AF), szBuffer);
			
			if(bAdd)
			{
				EnableWindow(GETHCTL(IDC_FILE_ARCHIVE), FALSE);
			}

			// Reposition dialog box inside client area of main window
			// Want new position at x=8 and y=45 dialog units wrt top left of
			// main window client area (0,0)
			// First convert those values to pixels
			rect.left=8;
			rect.top=45;
			rect.right=16; // Arbitrary
			rect.bottom=90; // Arbitrary
			MapDialogRect(hDlg, &rect);
			nX=rect.left;
			nY=rect.top;
			// Get current position and size of dialog box
			GetWindowRect(GetParent(hDlg), &rect);
			// Move dialog box to new position
			MoveWindow(GetParent(hDlg), rect.left+nX, rect.top+nY,
				rect.right-rect.left, rect.bottom-rect.top, TRUE);

			return TRUE;
		}

		case WM_NOTIFY:
		{
			// The lParam of a WM_NOTIFY msg is a pointer
			// to a OFNOTIFY struct
			// The hdr member of that struct specifies a NMHDR struct
			// The meaning of the msg is contained in the code member
			// of that struct
			ofnotify=(OFNOTIFY*)lParam;
			hdr=ofnotify->hdr;
			code=hdr.code;
			ofn=ofnotify->lpOFN;

			// Detect that file type has changed or dialog newly initialized
			// If file type is *.phd or *.frd, select corresponding data type
			// The nFilterIndex OPENFILENAME strut member indicates which file
			// type was just selected, as follows:
			// *.dat=1, *.phd=2, *.frd=3, *.*=4
			// After *.txt added 06/03/11:
			// *.dat=1, *.txt=2, *.phd=3, *.frd=4, *.*=5
			// After *.csv added 11/03/11:
			// *.dat=1, *.txt=2, *.csv=3, *.phd=4, *,frd=5, *.*=6

			#if(0) // For testing
			TRACE_INT(ofn->nFilterIndex);
			#endif

			if(code==CDN_TYPECHANGE || code==CDN_INITDONE)
			{
				// Set data type to agree with file type selection
				if(ofn->nFilterIndex==4)
				// *.phd (was 2 until 06/03/11, 3 until 11/03/11))
				{
					bDataType=PHASE_DATA;
					Button_SetCheck(GETHCTL(IDC_FILE_FREQ), bDataType);
					Button_SetCheck(GETHCTL(IDC_FILE_PHASE), (1-bDataType));
					Button_SetCheck(GETHCTL(IDC_FILE_ARCHIVE), FALSE);
				}
				if(ofn->nFilterIndex==5)
				// *.frd (was 3 until 06/03/11, 4 until 11/03/11)
				{
					bDataType=FREQ_DATA;
					Button_SetCheck(GETHCTL(IDC_FILE_FREQ), bDataType);
					Button_SetCheck(GETHCTL(IDC_FILE_PHASE), (1-bDataType));
					Button_SetCheck(GETHCTL(IDC_FILE_ARCHIVE), FALSE);
				}
			}

			if(code==CDN_FILEOK)
			{
				// Read data type info from radio buttons into bDataType
				bDataType=
					(BOOL)Button_GetCheck(GETHCTL(IDC_FILE_FREQ));

				// Handle archive data type as special case
				bArchiveData=
					(BOOL)Button_GetCheck(GETHCTL(IDC_FILE_ARCHIVE));

				if(bArchiveData)
				{
					// Clear phase & freq buttons
					Button_SetCheck(GETHCTL(IDC_FILE_FREQ), FALSE);
					Button_SetCheck(GETHCTL(IDC_FILE_PHASE), FALSE);
				}
				else
				{
					// Update the main status indicators
					Button_SetCheck(hwndFreqSelectButton, bDataType);
					Button_SetCheck(hwndPhaseSelectButton, (1-bDataType));
				}
			}
		}

		case WM_COMMAND:
		{
			if(GET_WM_COMMAND_ID(wParam, lParam)==IDC_OPEN_HELP)
			{
				// Invoke help file
				if(bAdd)
				{
					#if(0)
					HtmlHelp(hDlg, "Stable32.chm::ADD_FUNCTION.HTM",
						HH_DISPLAY_TOPIC, 0L);
					#endif

					#if(0) // New code 11/29/11
					// Invoke help file
					strlcpy(szBuffer, szHelpFile, sizeof(szBuffer));
					strlcat(szBuffer, "::ADD_FUNCTION.HTM", sizeof(szBuffer));
					HtmlHelp(hDlg, szBuffer,
						HH_DISPLAY_TOPIC, 0L);
					#endif

					#if(1) // Trial code 11/30/11
					// Invoke help file using CallHelp() function
					CallHelp(hDlg, "ADD_FUNCTION");
					#endif
				}
				else
				{
					#if(0)
					HtmlHelp(hDlg, "Stable32.chm::OPEN_FUNCTION.HTM",
						HH_DISPLAY_TOPIC, 0L);
					#endif

					#if(0) // New code 11/29/11
					// Invoke help file
					strlcpy(szBuffer, szHelpFile, sizeof(szBuffer));
					strlcat(szBuffer, "::OPEN_FUNCTION.HTM", sizeof(szBuffer));
					HtmlHelp(hDlg, szBuffer,
						HH_DISPLAY_TOPIC, 0L);
					#endif

					#if(1) // Trial code 11/30/11
					// Invoke help file using CallHelp() function
					CallHelp(hDlg, "OPEN_FUNCTION");
					#endif
				}
			}

			// Respond to avg factor edit control message
			if(GET_WM_COMMAND_ID(wParam, lParam)==IDC_FILE_AF)
			{
				// Respond to edit control when it loses the input focus
				// unless it loses it to the cancel button
				if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
					(!bErrorTrap) && (GetFocus()!=GETHCTL(IDC_FILE_CANCEL)))
				{
					// Set error flag to avoid multiple error messages
					bErrorTrap=TRUE;

					// Save previous value
					nPrev=nBigAvgFactor;

					// Get (new) avg factor
					GetWindowText(GETHCTL(IDC_FILE_AF), szBuffer,
						BUFFER_SIZE);
					fEntry=fabs(atof(szBuffer));

					// Make sure entry is a valid int
					if(fEntry>2147483651.) // Out of int range
					{
						// Invalid entry - restore default value
						_snprintf(szBuffer, sizeof(szBuffer), "%d", nPrev);
						SetWindowText(GETHCTL(IDC_FILE_AF), szBuffer);
						SetFocus(GETHCTL(IDC_FILE_AF));
						LoadString(hInst, IDS_MESSAGE_TOOBIGINT, szBuffer,
							BUFFER_SIZE);
						MessageBox(hDlg, szBuffer, "Invalid Entry",
							MB_ICONEXCLAMATION);
						break;
					}
					else
					{
						nEntry=abs(atoi(szBuffer));
					}

					// Check entry - must be between 1 and itself
					// Can't check max against file size - don't know it yet
					if(ValidateInteger(hDlg, nEntry, 1, nEntry)==TRUE)
					{
						// New value OK
						nBigAvgFactor=nEntry;
						_snprintf(szBuffer, sizeof(szBuffer), "%d",
							nBigAvgFactor);
						SetWindowText(GETHCTL(IDC_FILE_AF), szBuffer);
					}
					else
					{
						// Invalid entry - restore default value
						_snprintf(szBuffer, sizeof(szBuffer), "%d", nPrev);
						SetWindowText(GETHCTL(IDC_FILE_AF), szBuffer);
						SetFocus(GETHCTL(IDC_FILE_AF));
					}
				}
				else
				{
					// Reset error flag
					bErrorTrap=FALSE;
				}

				// If avg factor is 1 then use BIG_OK
				if(nBigAvgFactor==1)
				{
					nTooBigOption=BIG_OK;
				}
				else
				{
					// If avg factor not 1 then use BIG_DOAVG
					nTooBigOption=BIG_DOAVG;

					// Force use of Data File Opened dialog
					// to apply tau correction
					bShowFileOpened=TRUE;
				}
			}

			// Respond to CR in an edit control
			// Note: No button should be a DEFPUSHBUTTON
			// But I think that the Close button of the standard control is 
			if(GET_WM_COMMAND_ID(wParam, lParam)==IDOK)
			{
				// Does avg factor edit control have focus?
				if(GetFocus()==GETHCTL(IDC_FILE_AF))
				{
					// Send EN_KILLFOCUS message to factor edit control
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_FILE_AF, 0,
						EN_KILLFOCUS));
				}
			}
			break;
		}
	}
	return FALSE;
}

/*****************************************************************************/
/*                                                                           */
/*		Hook Function for Custom File Save Dialog Box                        */
/*                                                                           */
/*      Processes bDataType info from phase and frequency radio buttons      */
/*                                                                           */
/*      Revision record:                                                     */
/*      	08/24/96		Written - seems OK                               */
/*			12/20/96		Added help support                               */
/*			12/06/00		Trial code for Explorer type dialog box          */
/*                                                                           */
/*****************************************************************************/

#ifndef _MSC_VER
	#pragma argsused
#endif

UINT CALLBACK FileSaveHookProc(HWND hDlg, UINT msg, WPARAM wParam,
	LPARAM lParam)
{
	// Local variables
	OFNOTIFY *ofnotify;
	NMHDR hdr;
	UINT code;
	RECT rect;
	INT nX;
	INT nY;
	// char szBuffer[BUFFER_SIZE+1];
	
	switch(msg)
	{
		case WM_INITDIALOG:
		{
			// Initialize the timetags check box
			// Enable the timetags check box only if timetag data exists
			if(bDataType==FREQ_DATA)
			{
				EnableWindow(GETHCTL(IDC_SAVE_TIMETAGS), bFreqTimetags);
				Button_SetCheck(GETHCTL(IDC_SAVE_TIMETAGS), bFreqTimetags);
			}
			else // Phase data
			{
				EnableWindow(GETHCTL(IDC_SAVE_TIMETAGS), bPhaseTimetags);
				Button_SetCheck(GETHCTL(IDC_SAVE_TIMETAGS), bPhaseTimetags);
			}

			// Reposition dialog box inside client area of main window
			// Want new position at x=8 and y=45 dialog units wrt top left of
			// main window client area (0,0)
			// First convert those values to pixels
			rect.left=8;
			rect.top=45;
			rect.right=16; // Arbitrary
			rect.bottom=90; // Arbitrary
			MapDialogRect(hDlg, &rect);
			nX=rect.left;
			nY=rect.top;
			// Get current position and size of dialog box
			GetWindowRect(GetParent(hDlg), &rect);
			// Move dialon box to new position
			MoveWindow(GetParent(hDlg), rect.left+nX, rect.top+nY,
				rect.right-rect.left, rect.bottom-rect.top, TRUE);

			return TRUE;
		}

		case WM_NOTIFY:
		{
			// The lParam of a WM_NOTIFY msg is a pointer to a OFNOTIFY struct
			// The hdr member of that struct specifies a NMHDR struct
			// The meaning of the msg is contained in the code member
			// of that struct
			ofnotify=(OFNOTIFY*)lParam;
			hdr=ofnotify->hdr;
			code=hdr.code;
			
			if(code==CDN_FILEOK)
			{
				// Read timetags check box state
				bSaveTimetags=
					(BOOL)Button_GetCheck(GETHCTL(IDC_SAVE_TIMETAGS));
			}
		}

		case WM_COMMAND:
		{
			if(GET_WM_COMMAND_ID(wParam, lParam)==IDC_SAVE_HELP)
			{
				#if(0)		
				// Invoke help file
				HtmlHelp(hDlg, "Stable32.chm::SAVE_FUNCTION.HTM",
					HH_DISPLAY_TOPIC, 0L);
				#endif

				#if(0) // New code 11/29/11
				// Invoke help file
				strlcpy(szBuffer, szHelpFile, sizeof(szBuffer));
				strlcat(szBuffer, "::SAVE_FUNCTION.HTM", sizeof(szBuffer));
				HtmlHelp(hDlg, szBuffer,
					HH_DISPLAY_TOPIC, 0L);
				#endif

				#if(1) // Trial code 11/30/11
				// Invoke help file using CallHelp() function
				CallHelp(hDlg, "SAVE_FUNCTION");
				#endif
			}
		}
	}
	return FALSE;
}

/*****************************************************************************/
/*                                                                           */
/*		Hook Function for Custom Stability File Open Dialog Box              */
/*                                                                           */
/*      Processes bDataType info from phase and frequency radio buttons      */
/*			12/09/00		Created                                          */
/*                                                                           */
/*****************************************************************************/

#ifndef _MSC_VER
	#pragma argsused
#endif

UINT CALLBACK StabOpenHookProc(HWND hDlg, UINT msg, WPARAM wParam,
	LPARAM lParam)
{
	// Local variables
	OFNOTIFY *ofnotify;
	NMHDR hdr;
	UINT code;
	RECT rect;
	INT nX;
	INT nY;
	// char szBuffer[BUFFER_SIZE+1];
	
	switch(msg)
	{
		case WM_INITDIALOG:
		{
			// Reposition dialog box inside client area of main window
			// Want new position at x=8 and y=15 dialog units wrt top left of
			// main window client area (0,0)
			// First convert those values to pixels
			rect.left=8;
			rect.top=14;
			rect.right=16; // Arbitrary
			rect.bottom=28; // Arbitrary
			MapDialogRect(hDlg, &rect);
			nX=rect.left;
			nY=rect.top;
			// Get current position and size of dialog box
			GetWindowRect(GetParent(hDlg), &rect);
			// Move dialon box to new position
			MoveWindow(GetParent(hDlg), rect.left+nX, rect.top+nY,
				rect.right-rect.left, rect.bottom-rect.top, TRUE);

			return TRUE;
		}

		case WM_NOTIFY:
		{
			// The lParam of a WM_NOTIFY msg is a pointer to a OFNOTIFY struct
			// The hdr member of that struct specifies a NMHDR struct
			// The meaning of the msg is contained in the code member
			// of that struct
			ofnotify=(OFNOTIFY*)lParam;
			hdr=ofnotify->hdr;
			code=hdr.code;
			
			if(code==CDN_FILEOK)
			{
				// No action reqd
				;
			}
		}

		case WM_COMMAND:
		{
			if(GET_WM_COMMAND_ID(wParam, lParam)==IDC_STAB_HELP)
			{
				#if(0)
				// Invoke help file
				HtmlHelp(hDlg, "Stable32.chm::READ_STABILITY_FILE.HTM",
					HH_DISPLAY_TOPIC, 0L);
				#endif

				#if(0) // New code 11/29/11
				// Invoke help file
				strlcpy(szBuffer, szHelpFile, sizeof(szBuffer));
				strlcat(szBuffer, "::READ_STABILITY_FILE.HTM", sizeof(szBuffer));
				HtmlHelp(hDlg, szBuffer,
					HH_DISPLAY_TOPIC, 0L);
				#endif

				#if(1) // Trial code 11/30/11
				// Invoke help file using CallHelp() function
				CallHelp(hDlg, "READ_STABILITY_FILE");
				#endif
			}
		}
	}
	return FALSE;
}

/*****************************************************************************/

LONG FileLength (int hFile)
{
	long lCurrentPos = _llseek (hFile, 0L, 1);
	long lFileLength = _llseek (hFile, 0L, 2);

	_llseek (hFile, lCurrentPos, 0);

	return lFileLength;
}

/*****************************************************************************/
/*                                                                           */
/*                        MulticolumnDataRead()                              */
/*                                                                           */
/*  Function to read multicolumn data file                                   */
/*                                                                           */
/*  Note:  This version of DataRead() has full functionality for reading     */
/*         selected columns of timetags and data.                            */
/*                                                                           */
/*  Parameters:		LPSTR 	lpstrFileName	Name of data file                */
/*					int		nTagCol			Column # of timetags             */
/*					int		nDatCol			Column # of data                 */
/*					int		nTooBigOption	Options for large files          */
/*                                                                           */
/*	Globals:		F_TYPE	fData[]			Data array                       */
/*                                                                           */
/*  Return:			int                  	# data points                    */
/*										 	or -1 if error                   */
/*                                                                           */
/*	MulticolumnDataread() includes provisions for reading large files by     */
/*  averaging or using the first or last portions of the data file.  Phase   */
/*  averaging simply drops the intermediate values. Frequency averaging is   */
/*  done by forming the mean of the values in each group.  (This is done     */
/*  in-place and on-the-fly.)  Timetag averaging simply uses the last value  */
/*  of each group (like phase averaging).                                    */
/*                                                                           */
/*	Revision record:                                                         */
/*  	09/20/96	Added int nTooBigOption parameter - still needs code     */
/*		09/21/96	Code added for reading large files                       */
/*		09/22/96	Completed code for reading large files                   */
/*      11/19/96	Added code to handle invalid data points                 */
/*		11/25/97	Changed nDataArraySize from int to long                  */
/*					Changed i fron int to long & added (int) casts           */
/*		12/11/99	Added code to handle European comma deciaml point        */
/*		04/27/00	Added trap for blank lines                               */
/*		08/24/02	Added bDigit                                             */
/*					Changed to use of improved IsNumeric()                   */
/*		12/07/02	Added offsetting of timetags                             */
/*		10/14/03	Added BIG_DOAVG option                                   */
/*		08/13/08	Changed parsing method                                   */
/*		08/19/08	Added max col # bound                                    */
/*		08/23/08	Added BIG_ALL option                                     */
/*		08/23/08	Added progress indicator                                 */
/*		09/30/12	Added treatment of semicolon as comment line             */
/*                                                                           */
/*****************************************************************************/

int MulticolumnDataRead(LPSTR lpstrFileName, int nTagCol, int nDatCol,
	int nTooBigOption)
{
	// Local variables
	// Chars
	char szInputStr[BIG_BUFFER_SIZE+1];   // Input buffer
	char szBuffer[BUFFER_SIZE+1];		  // Parsing buffer
	// Ints
	int k;                 		// Secondary index
	int nFields;				// # sscanf() fields processed
	int nAF=1;					// Averaging factor
	int j;						// Count of # characters eliminated
	static BOOL bDigit=FALSE;	// Flag that previous character was digit
	// Longs
	long nDataArraySize;
	static long i;                 	// Index
	long ii;					// Auxilary index
	// F_TYPEs
	F_TYPE fval[MAX_NUM_COLS];  // Input values
	F_TYPE fTagOffset;			// Timetag offset value
	// Misc
	FILE *fhIn;           		// DOS file handle

	// Start of code
	// Display wait cursor
	SetCursor(hWait);

	// Open input file
	if((fhIn=fopen(lpstrFileName, "r"))==NULL)
	{
		// Display normal cursor
		SetCursor(hArrow);

		// Error - could not open input file
		return -1;
	}

	// Set the timetag offset value from the global value for the data type
	if(bDataType==FREQ_DATA)
	{
		fTagOffset=fFreqTagOffset;
	}
	else // Phase data
	{
		fTagOffset=fPhaseTagOffset;
	}

	// Calc averaging factor
	// Note: nAF=1 for case when file size is within normal bounds

	// Set data array size
	if(nTooBigOption==BIG_AVERAGE)
	{
		nAF=nBigAvgFactor;
		nDataArraySize=nDataFileSize;
	}
	else if(nTooBigOption==BIG_DOAVG)
	{
		// Make sure avg factor not too large
		//nBigAvgFactor=min(nDataFileSize, nBigAvgFactor);
		nAF=nBigAvgFactor;
		nDataArraySize=nDataFileSize;
	}
	else if(nTooBigOption==BIG_ALL) // Aded 08/23/08
	{
		nAF=1;
		nDataArraySize=nDataFileSize;
	}
	else
	{
		nDataArraySize=min(nDataFileSize, nMaxSize);
	}

	// Clear data array
	for(i=ARRAY_OFFSET; i<(nDataArraySize/nAF)+ARRAY_OFFSET; i++)
	{
		fData[(int)i]=0.0;
	}

	// Initialize index
	i=ARRAY_OFFSET; // Start entering values at 1st array points
					// (after #, Start, End)

	// Use last portion of big file
	// Note: No special processing for first portion of big file - it just
	//       hits the size limit and stops reading
	// Chew up leading lines to read last portion of data file
	if(nTooBigOption==BIG_LAST)
	{
		nAF=1;
		while(i<nDataFileSize-nMaxSize+ARRAY_OFFSET)
		{
			fgets(szInputStr, BIG_BUFFER_SIZE, fhIn);
			i++;
		}
		i=ARRAY_OFFSET; // Reset index
	}

	// Show message
	// WriteMessage("Reading Data File - Please Wait");
	
	// Label progress bar
	WriteProgressMessage(0, "Reading Data", FALSE); 

	// Read lines from input file
	// # data points bounded by nDataArraySize modulo nAF
	// Note: The cast was (int) before i changed to long_i
	while((fgets(szInputStr, BIG_BUFFER_SIZE, fhIn) != NULL) &&
		(i<((long)(nDataArraySize/nAF)*nAF)+ARRAY_OFFSET))
	{
		// Check if blank line
		// fgets() will put \n ahead of \0 of input string of a blank line
		// and the first character of the string will be either \r or \n
		if((szInputStr[0]=='\r') || (szInputStr[0]=='\n'))
		{
			// Skip everything for blank line
			continue;
		}

		// Is line a UNIX comment?
		if((bUnixComment) && (szInputStr[0]=='#'))
		{
			#if(1) // New code 09/24/11
			// Put first UNIX comment into plot subtitle string array
			if(nUserSubtitles==0)
			{
				// Store comment as 1st plot subtitle
				szInputStr[strlen(szInputStr)-1]='\0';
				strcpy(szPlotSubtitle[nUserSubtitles], &szInputStr[2]);
				nUserSubtitles++;
			}
			#endif

			// Reject line if it starts with a # (UNIX comment)
			continue;
		}
		
		#if(1) // New code 09/30/12
		// Does line start with a semicolon?
		// If so, treat it as a comment line and ignore it
		if(szInputStr[0]==';')
		{
			// Put first comment into plot subtitle string array
			if(nUserSubtitles==0)
			{
				// Store comment as 1st plot subtitle
				szInputStr[strlen(szInputStr)-1]='\0';
				strcpy(szPlotSubtitle[nUserSubtitles], &szInputStr[2]);
				nUserSubtitles++;
			}

			// Reject line if it starts with a semicolon
			continue;
		}
		#endif
		
		// Reject a line that begins with an alphabetic character (a-z or A-Z)
		// after skipping nSkipChar of them
		if(isalpha(szInputStr[nSkipChar]))
		{
			continue;
		}
		
		#if(0) // For testing 08/09/08
		// Test to remove all spaces from input string
		// to support pure comma delimiting
		// The RemoveWhitespace() function works fine
		// But other issues remain:
		// Don't want to remove spaces from comment line(s)
		// To use, must add flag and user interface
		// For example, Configuration/Data could have radiobuttons
		// to select file delimiter type:
		// Space, Comma, or Both (default)
		// So, instead of both delimiters now
		// which would parse a line like 1,2 3,4 as 1 2 3 4
		// with comma only delimiters it would parse as 1 23 4
		// (with space only delimiters the line couldn't be read)
		// As of now, there is no compelling reason for this
		// and it is not implemented
		RemoveWhitespace(szInputStr);
		#endif
		
		// Process commas		
		if(bComma) // European comma decimal point usage
		{
			// Convert commas into decimal points
			for(k=0; k<(int)strlen(szInputStr); k++)
			{
				if(szInputStr[k]==',')
				{
					szInputStr[k]='.';
				}
			}
		}
		else // Standard comma usage
		{
			// Convert commas into spaces so comma-delimited data can be read
			for(k=0; k<(int)strlen(szInputStr); k++)
			{
				if(szInputStr[k]==',')
				{
					szInputStr[k]=' ';
				}
			}
		}

		// Replace slashes and colons with zeros so date data can be read
		for(k=0; k<(int)strlen(szInputStr); k++)
		{
			if(szInputStr[k]=='/')
			{
				szInputStr[k]='0';
			}

			if(szInputStr[k]==':')
			{
				szInputStr[k]='0';
			}

			#if(1) // New code 08/10/08
			// Replace tabs with spaces so tab-delimited data can be read
			if(szInputStr[k]=='\t')
			{
				szInputStr[k]=' ';
			}
			#endif

			#if(1) // New code 08/11/08
			// Replace bars with spaces so bar-delimited data can be read
			if(szInputStr[k]=='|')
			{
				szInputStr[k]=' ';
			}
			#endif
		}

		j=nSkipChar;
		k=0;

		// Eliminate up to nSkipChar # of non-numeric characters
		bDigit=FALSE;
		while(j)
		{
			if(!IsNumeric(szInputStr[k], &bDigit))
			{
				szInputStr[k]=' ';
				j--;
			}
			k++;
		}

		// For testing
		// TRACE_STR(szInputStr);

		#if(0) // Original code
		// This method of reading space-delimited field values from an input line
		// doesn't work right if there is a dotted IP address in the line
		// It reads multiple values because of the extra "decimal points"
		// e.g., 192.168.2.1 becomes 192.168 and 0.2 and 0.3 (3 values)
		// Read the line input values (up to MAX_NUM_COLS=32)
		nFields=sscanf(szInputStr,"%le %le %le %le %le %le %le %le %le %le %le"
			"%le %le %le %le %le %le %le %le %le %le %le %le %le %le %le %le "
			"%le %le %le %le %le",
			&fval[0], &fval[1], &fval[2], &fval[3], &fval[4], &fval[5],
			&fval[6], &fval[7],	&fval[8], &fval[9], &fval[10], &fval[11],
			&fval[12], &fval[13], &fval[14], &fval[15], &fval[16], &fval[17],
			&fval[18], &fval[19], &fval[20], &fval[21], &fval[22], &fval[23],
			&fval[24], &fval[25], &fval[26], &fval[27], &fval[28], &fval[29],
			&fval[30], &fval[31]);
		#endif

		// For testing
		// TRACE_INT(i);

		// Save i in ii, then restore it later
		// This lets index i be used in code below
		// Original i is data line index
		// In new code below it is character index within a line
		// Should change index
		// Also should be able to make i an int rather than a long
		// which is probably something from the 16-bit version of Stable
		// But this all seems to be working fine as-is
		ii=i;

		#if(1) // New code 08/13/08
		// Parse szInput input line into numeric values fval[i]
		// Note: This is same parsing method as used by Multicomumn Details

		// Ignore leading whitespace
		i=0;
		while(i<(int)strlen(szInputStr))
		{
			if(isspace(szInputStr[i]))
			{
				i++;
			}
			else
			{
				break;
			}
		}

		// Note: i skips leading whitespace
		nFields=0;
		while(1)
		{
			// Scan for next space delimiter or EOS
			j=0;
			while((szInputStr[(int)i]!=' ')&&(szInputStr[(int)i]!='\0'))
			{
				// Copy characters of current item
				szBuffer[j]=szInputStr[(int)i];

				// Exclude characters like CR and LF
				if(szBuffer[j]=='\r' || szBuffer[j]=='\n' || szBuffer[j]=='\f')
				{
					szBuffer[j]=' ';
				}

				// Increment indices
				i++;
				j++;
			}

			// Insert EOS for this field
			szBuffer[j]='\0';

			// Don't process a blank line
			// Not sure if that can happen, but this prevents it
			if(strlen(szBuffer)==0)
			{
				break;
			}

			// For testing
			// TRACE_STR(szBuffer);

			// Turn field string into numeric value and save it
			fval[nFields]=atof(szBuffer);

			// Increment # fields
			nFields++;

			// Bound # fields
			if(nFields>MAX_NUM_COLS)
			{
				// For testing
				// TRACE_STR("Maximum # columns exceeded");
				break;
			}

			// Now at space or EOS
			// Quit if we're at EOS
			if(szInputStr[(int)i]=='\0')
			{
				break;
			}

			// Now at space
			// Move through any more spaces in this place
			while((szInputStr[(int)i+1]==' ')||(szInputStr[(int)i+1]=='\0'))
			{
				i++;
			}

			// Now at space or EOS
			// Quit if we're at EOS
			if(szInputStr[(int)i]=='\0')
			{
				break;
			}

			// Increment szInput index
			i++;

			// Done if at EOS or CR or LF (don't expect latter two)
			if((szInputStr[(int)i]=='\0')||(szInputStr[(int)i]=='\r')||(szInputStr[(int)i]=='\f'))
			{
				break;
			}
		}
		#endif
		
		// Restore i from ii
		i=ii;

		// Reject all input lines that do not parse correctly
		// They are probably header lines
		if(nFields<max(nTagCol, nDatCol))
		{
			i--;
		}
		
		// Put timetag value into timetag array
		if(nTagCol>0)
		{
			// Store the timetag value
			if(nTooBigOption==BIG_AVERAGE || nTooBigOption==BIG_DOAVG)
			{
				// Just use the last point of the group
				fTags[((int)((i-ARRAY_OFFSET)/nAF))+ARRAY_OFFSET]=
					fval[nTagCol-1]+fTagOffset;
			}
			else
			{
				fTags[(int)i]=fval[nTagCol-1]+fTagOffset;
				
				// For testing
				// TRACE_EXP(fTags[(int)i]);
			}
		}

		// Put data value into data array
		if(nDatCol>0)
		{
			if(nTooBigOption==BIG_AVERAGE || nTooBigOption==BIG_DOAVG)
			{
				if(bDataType==FREQ_DATA)
				{
					// The idea here is to form the averages in-place by
					// integer scaling of the array index and summation of the
					// scaled data.
					fData[((int)((i-ARRAY_OFFSET)/nAF))+ARRAY_OFFSET]+=
						fval[nDatCol-1]/nAF;
				}
				else // Phase data
				{
					// For phase data, just use the last point of the group
					fData[((int)((i-ARRAY_OFFSET)/nAF))+ARRAY_OFFSET]=
						fval[nDatCol-1];
				}
			}
			else
			{
				fData[(int)i]=fval[nDatCol-1];
				
				// For testing
				// TRACE_EXP(fData[(int)i]);
			}
		}

		// For testing
		// TRACE_EXP(fval[0]);
		// TRACE_EXP(fval[1]);
		
		// Increment index
		i++;

		// Show progress  - note test to avoid 0 as 2nd % arg
		if((nDataArraySize)/100)
		{
			if((i%((nDataArraySize)/100))==0)
			{
				WriteProgressMessage((int)(100.0*i/(nDataArraySize)), "Reading Data", TRUE);
			}
		}
	}

	// Hide progress bar
	WriteProgressMessage(0, "", FALSE); 

	// For testing
	// TRACE_INT(nFields);
	// TRACE_INT(i);

	// Notes: # data points read = i-ARRAY_OFFSET+1-1 = i-ARRAY_INDEX
	// because i gets incremented a final time at end
	// TRACE_INT(i-ARRAY_OFFSET);
	// Last data point is fData[i-1];
	// TRACE_EXP(fData[i-1]);
	
	// Set timetag array #, start and end values
	if(nTagCol>0)
	{
		if(nTooBigOption==BIG_AVERAGE || nTooBigOption==BIG_DOAVG)
		{
			fTags[0] = (i-ARRAY_OFFSET)/nAF; 	// #
			fTags[1] = 1;   			        // Start
			fTags[2] = (i-ARRAY_OFFSET)/nAF; 	// End
		}
		else
		{
			fTags[0] = i-ARRAY_OFFSET; 			// #
			fTags[1] = 1;   					// Start
			fTags[2] = i-ARRAY_OFFSET; 			// End
		}

		if(bDataType==FREQ_DATA)
		{
			bFreqTimetags=TRUE;
		}
		else
		{
			bPhaseTimetags=TRUE;
		}
	}
	else
	{
		if(bDataType==FREQ_DATA)
		{
			bFreqTimetags=FALSE;
		}
		else
		{
			bPhaseTimetags=FALSE;
		}
	}

	// Set data array #, start and end values
	if(nDatCol>0)
	{
		if(nTooBigOption==BIG_AVERAGE || nTooBigOption==BIG_DOAVG)
		{
			fData[0] = (i-ARRAY_OFFSET)/nAF; 	// #
			fData[1] = 1;   			        // Start
			fData[2] = (i-ARRAY_OFFSET)/nAF; 	// End
		}
		else
		{
			fData[0] = i-ARRAY_OFFSET; 			// #
			fData[1] = 1;   					// Start
			fData[2] = i-ARRAY_OFFSET; 			// End
		}
	}

	fclose(fhIn);

	// Display normal cursor
	SetCursor(hArrow);

	return((int)((i-ARRAY_OFFSET)/nAF));
}

#ifndef LITE_VER
/*****************************************************************************/
/*                                                                           */
/*                        MulticolumnDataAdd()                               */
/*                                                                           */
/*  Function to add multicolumn data file                                    */
/*                                                                           */
/*	Note: This is a generalization of MulticolumnDataRead(), so it can       */
/*  probably replace that function (setting nSize=0 for ordinary data read)  */
/*  after it is debugged.                                                    */
/*                                                                           */
/*  Note:  This version of DataRead() has full functionality for adding      */
/*         selected columns of timetags and data.                            */
/*                                                                           */
/*  Note:  This function does NOT do averaging.                              */
/*                                                                           */
/*  Parameters:		LPSTR 	lpstrFileName	Name of data file                */
/*					int		nTagCol			Column # of timetags             */
/*					int		nDatCol			Column # of data                 */
/*					int		nSize			Original array size              */
/*	Globals:		F_TYPE	fData[]			Data array                       */
/*  Return:			int                  	# data points                    */
/*										 	or -1 if error                   */
/*	Revision record:                                                         */
/*  	09/02/96	Adapted from MulticolumnDataRead()                       */
/*      11/19/96	Added code to handle invalid data points                 */
/*		04/27/00	Added trap for blank lines                               */
/*		08/24/02	Added bDigit                                             */
/*					Changed to use of improved IsNumeric()                   */
/*		12/07/02	Added offsetting of timetags                             */
/*		08/14/08	Changed parsing method                                   */
/*		08/19/08	Added max col # bound                                    */
/*		08/23/08	Added nDataFileSize arg and progress indicator           */
/*                                                                           */
/*****************************************************************************/

int MulticolumnDataAdd(LPSTR lpstrFileName, int nTagCol, int nDatCol,
	int nSize, int nDataFileSize)
{
	// Local variables
	// Chars
	char szInputStr[BIG_BUFFER_SIZE+1];   // Input buffer
	char szBuffer[BUFFER_SIZE+1];		  // Parsing buffer
	// Ints
	static int i;                 			// Index
	long ii;						// Auxilary index
	int k;                 			// Secondary index
	int nFields;					// # sscanf() fields processed
	int j;							// count of # characters eliminated
	static BOOL bDigit=FALSE;		// Flag that previous character was digit
	// F_TYPEs
	F_TYPE fval[MAX_NUM_COLS];      // Input values
	F_TYPE fTagOffset;				// Timetag offset value
	// Others
	FILE *fhIn;           			// DOS file handle

	// Open input file
	if((fhIn=fopen(lpstrFileName, "r"))==NULL)
	{
		// error - could not open input file
		return -1;
	}

	// Set the timetag offset value from the global value for the data type
	if(bDataType==FREQ_DATA)
	{
		fTagOffset=fFreqTagOffset;
	}
	else // Phase data
	{
		fTagOffset=fPhaseTagOffset;
	}

	// Set starting index
	i=nSize+ARRAY_OFFSET; // Start 1st new value after last original data
						  // point

	// Show message
	// WriteMessage("Reading Data File - Please Wait");

	// Label progress bar
	WriteProgressMessage(0, "Reading Data", FALSE); 

	// Read lines from input file
	while(fgets(szInputStr, BIG_BUFFER_SIZE, fhIn) != NULL)
	{
		// Check if blank line
		// fgets() will put \n ahead of \0 of input string of a blank line
		// and the first character of the string will be either \r or \n
		if((szInputStr[0]=='\r') || (szInputStr[0]=='\n'))
		{
			// Skip everything for blank line
			continue;
		}

		// Is line a UNIX comment?
		if((bUnixComment) && (szInputStr[0]=='#'))
		{
			// Reject line if it starts with a # (UNIX comment)
			continue;
		}
		
		// Reject a line that begins with an alphabetic character (a-z or A-Z)
		// after skipping nSkipChar of them
		if(isalpha(szInputStr[nSkipChar]))
		{
			continue;
		}
		
		// Process commas, slashes, colons, etc.
		for(k=0; k<(int)strlen(szInputStr); k++)
		{
			if(bComma) // European comma decimal point usage
			// Convert commas into decimal points
			{
				if(szInputStr[k]==',')
				{
					szInputStr[k]='.';
				}
			}
			else // Standard comma usage
			// Convert commas into spaces so comma-delimited data can be read
			{
				if(szInputStr[k]==',')
				{
					szInputStr[k]=' ';
				}
			}

			// Replace slashes & colons with zeros so dates & times can be read
			if(szInputStr[k]=='/')
			{
				szInputStr[k]='0';
			}
			if(szInputStr[k]==':')
			{
				szInputStr[k]='0';
			}

			#if(1) // New code 08/10/08
			// Replace tabs with spaces so tab-delimited data can be read
			if(szInputStr[k]=='\t')
			{
				szInputStr[k]=' ';
			}
			#endif

			#if(1) // New code 08/11/08
			// Replace bars with spaces so bar-delimited data can be read
			if(szInputStr[k]=='|')
			{
				szInputStr[k]=' ';
			}
			#endif
		}

		j=nSkipChar;
		k=0;

		// Eliminate up to nSkipChar # of non-numeric characters
		bDigit=FALSE;
		while(j)
		{
			if(!IsNumeric(szInputStr[k], &bDigit))
			{
				szInputStr[k]=' ';
				j--;
			}
			k++;
		}

		#if(0) // Original code
		// This method of reading space-delimited firld values from an input line
		// doesn't work right if there is a dotted IP address in the line
		// It reads multiple values because of the extra "decimal points"
		// e.g., 192.168.2.1 becomes 192.168 and 0.2 and 0.3 (3 values)
		// Read the line input values (up to MAX_NUM_COLS=32)
		nFields=sscanf(szInputStr,"%le %le %le %le %le %le %le %le %le %le %le"
			"%le %le %le %le %le %le %le %le %le %le %le %le %le %le %le %le "
			"%le %le %le %le %le",
			&fval[0], &fval[1], &fval[2], &fval[3], &fval[4], &fval[5],
			&fval[6], &fval[7], &fval[8], &fval[9], &fval[10], &fval[11],
			&fval[12], &fval[13], &fval[14], &fval[15],	&fval[16], &fval[17],
			&fval[18], &fval[19], &fval[20], &fval[21], &fval[22], &fval[23],
			&fval[24], &fval[25], &fval[26], &fval[27], &fval[28], &fval[29],
			&fval[30], &fval[31]);
		#endif

		// Save i in ii, then restore it later
		// This lets index i be used in code below
		// Original i is data line index
		// In new code below it is character index within a line
		// Should change index
		// Also should be able to make i an int rather than a long
		// which is probably something from the 16-bit version of Stable
		// But this all seems to be working fine as-is
		ii=i;

		#if(1) // New code 08/13/08
		// Parse szInput input line into numeric values fval[i]
		// Note: This is same parsing method as used by Multicomumn Details

		// Ignore leading whitespace
		i=0;
		while(i<(int)strlen(szInputStr))
		{
			if(isspace(szInputStr[i]))
			{
				i++;
			}
			else
			{
				break;
			}
		}

		// Note: i skips leading whitespace
		nFields=0;
		while(1)
		{
			// Scan for next space delimiter or EOS
			j=0;
			while((szInputStr[(int)i]!=' ')&&(szInputStr[(int)i]!='\0'))
			{
				// Copy characters of current item
				szBuffer[j]=szInputStr[(int)i];

				// Exclude characters like CR and LF
				if(szBuffer[j]=='\r' || szBuffer[j]=='\n' || szBuffer[j]=='\f')
				{
					szBuffer[j]=' ';
				}

				// Increment indices
				i++;
				j++;
			}

			// Insert EOS for this field
			szBuffer[j]='\0';

			// Don't process a blank line
			// Not sure if that can happen, but this prevents it
			if(strlen(szBuffer)==0)
			{
				break;
			}

			// For testing
			// TRACE_STR(szBuffer);

			// Turn field string into numeric value and save it
			fval[nFields]=atof(szBuffer);

			// Increment # fields
			nFields++;

			// Bound # fields
			if(nFields>MAX_NUM_COLS)
			{
				// For testing
				// TRACE_STR("Maximum # columns exceeded");
				break;
			}

			// Now at space or EOS
			// Quit if we're at EOS
			if(szInputStr[(int)i]=='\0')
			{
				break;
			}

			// Now at space
			// Move through any more spaces in this place
			while((szInputStr[(int)i+1]==' ')||(szInputStr[(int)i+1]=='\0'))
			{
				i++;
			}

			// Now at space or EOS
			// Quit if we're at EOS
			if(szInputStr[(int)i]=='\0')
			{
				break;
			}

			// Don't process a blank line
			// Not sure why that can happen, but this prevents it
			if(strlen(szBuffer)==0)
			{
				break;
			}

			// Increment szInput index
			i++;

			// Done if at EOS or CR or LF (don't expect latter two)
			if((szInputStr[(int)i]=='\0')||(szInputStr[(int)i]=='\r')||(szInputStr[(int)i]=='\f'))
			{
				break;
			}
		}
		#endif
		
		// Restore i from ii
		i=ii;

		// Reject input lines that do not parse correctly
		// They are probably header lines
		if(nFields<1)
		{
			i--;
		}

		// Put timetag value into timetag array
		if(nTagCol>0)
		{
			fTags[i]=fval[nTagCol-1]+fTagOffset;
		}

		// Put data value into data array
		if(nDatCol>0)
		{
			fData[i]=fval[nDatCol-1];
		}

		// Increment index
		i++;

		// Show progress  - note test to avoid 0 as 2nd % arg
		if((nSize+nDataFileSize)/100)
		{
			if((i%((nSize+nDataFileSize)/100))==0)
			{
				WriteProgressMessage((int)(100.0*i/(nSize+nDataFileSize)), "Reading Data", TRUE);
			}
		}
	}

	// Hide progress bar
	WriteProgressMessage(0, "", FALSE); 

	// Set timetag array #, start and end values
	if(nTagCol>0)
	{
		fTags[0] = i-ARRAY_OFFSET;	// #
		fTags[1] = 1;   			// Start
		fTags[2] = i-ARRAY_OFFSET; 	// End
		if(bDataType==FREQ_DATA)
		{
			bFreqTimetags=TRUE;
		}
		else
		{
			bPhaseTimetags=TRUE;
		}
	}
	else
	{
		if(bDataType==FREQ_DATA)
		{
			bFreqTimetags=FALSE;
		}
		else
		{
			bPhaseTimetags=FALSE;
		}
	}

	// Set data array #, start and end values
	if(nDatCol>0)
	{
		fData[0] = i-ARRAY_OFFSET; 	// #
		fData[1] = 1;   			// Start
		fData[2] = i-ARRAY_OFFSET; 	// End
	}

	fclose(fhIn);

	return(i-ARRAY_OFFSET);
}
#endif

/*****************************************************************************/
/*                                                                           */
/*                             FileSize()                                    */
/*                                                                           */
/*  Function to find # points in data file                                   */
/*                                                                           */
/*  Parameters:		LPSTR lpstrFileName  Name of data file                   */
/*  Return:			int                  # data points                       */
/*                                                                           */
/*	Revision record:                                                         */
/*                                                                           */
/*****************************************************************************/

int FileSize(LPSTR lpstrFileName)
{
	// Local variables
	FILE *in;
	int size=0;
	char input_str[81];

	if((in=fopen(lpstrFileName,"r"))!=NULL)
	{
		while(fgets(input_str,81,in) != NULL)
		{
			size++;                           /* count # lines in input file */
		}

		rewind(in);
	}
	return size;
}

/*****************************************************************************/
/*                                                                           */
/*                             FileInfo()                                    */
/*                                                                           */
/*  Function to find # points and # cols in data file                        */
/*                                                                           */
/*  Parameters:		LPSTR lpstrFileName  Name of data file                   */
/*					LPSTR lpstrText      Text of 1st line of data file       */
/*					*int  nCols			 # columns in last line              */
/*					*int  nRejects		 # lines rejected                    */
/*					*int  nComments		 # lines with UNIX # comments        */
/*  Return:			int   nSize          # data points                       */
/*                                       Note:  This is really a line count  */
/*                                       The number of valid data points     */
/*                                       may be less by the # of rejects     */
/*                                                                           */
/*	Revision record:                                                         */
/*  	11/19/96	Determine # cols from last rather than 1st line          */
/*  	11/25/96	Reject non-numeric lines                                 */
/*		06/20/97	Fix error in handling long input lines                   */
/*		11/25/97	Change return type to long                               */
/*					Change nSize to a long                                   */
/*		12/11/99	Add code to handle European comma=deciaml point data     */
/*		04/30/00	Added trap for blank lines                               */
/*		05/18/01	Rearranged code so that a comment line is not used as    */
/*					the last line to determine the # of columns              */
/*		08/24/02	Added bDigit                                             */
/*					Changed to use of improved IsNumeric()                   */
/*		08/11/08	Changed to improved file reading method                  */
/*                  Added nRejects argument and counting                     */
/*		08/19/08	Added max col # bound and message                        */
/*		08/19/09	Added provisions for writing comment lines to file       */
/*		01/06/12	Added support for # Size= and # Cols= header items       */
/*		01/07/12	Added support for # DatCol= and # TagCol= header items   */
/*                                                                           */
/*****************************************************************************/

long FileInfo(LPSTR lpstrFileName, int *nCols, int *nRejects,
	int *nComments, LPSTR lpstrText)
{
	// Local variables
	// Chars
	char szBuffer[BIG_BUFFER_SIZE+1];	// Text buffer
	char szInput[BIG_BUFFER_SIZE+1];	// Input string
	char szText[BUFFER_SIZE+1];			// Text message string

	// Bools
	BOOL flag=TRUE;					// Flag for saving first active line
	
	// Ints
	int i;							// Index
	int j;							// Count of # characters eliminated
									// and other secondary index usage
	int col;						// # columns
	int c=0;						// # columns set by # Cols= header item
	static BOOL bDigit=FALSE;		// Flag that previous character was digit
	
	// Longs
	long nSize=0;	   				// # lines in file
	long s=0;						// File size set by # Size= header item
	
	// F_TYPEs (for obsolete code)
	// F_TYPE n1, n2, n3, n4, n5, n6, n7, n8, n9;	// Input values (dummies, not
	// used, may not exist.  Extra 9th value used to detect # cols > 8.)
	
	//Misc
	FILE *in;                 		// Input file pointer

	#if(1) // New code 08/19/09
	FILE *txt;						// Text file pointer
	#endif

	// This code has the effect of determining the # of data columns from the
	// last (rather than first) line of the data file.  This is the better
	// choice, because it isn't confused by a header at the top of the file.

	// Initialize # rejected data file lines
	*nRejects=0;

	// Initialize # comment data file lines
	*nComments=0;

	// Initialize global column values
	nTCol=0;
	nDCol=0;

	// Full path added 05/28/11
	// Don't want to write this file where data read from
	// because may not have write permission to that folder
	// especially if it is on a network drive
	// Compose comments filename
	strncpy(szBuffer, szPath, BIG_BUFFER_SIZE);
	strncat(szBuffer, "Comments.txt", BIG_BUFFER_SIZE);
	
	// Open text file
	if((txt=fopen(szBuffer,"w"))!=NULL)
	{
		// Error message
	}
	
	// Open input file
	if((in=fopen(lpstrFileName,"r"))!=NULL)
	{
		// Show message
		WriteMessage("Getting Data File Information - Please Wait");
	
		// Read file line-by-line
		while(fgets(szInput, BIG_BUFFER_SIZE, in) != NULL)
		{
			// Check if blank line
			// fgets() will put \n ahead of \0 of input string of a blank line
			// and the first character of the string will be either \r or \n
			if((szInput[0]=='\r') || (szInput[0]=='\n'))
			{
				// Skip everything for blank line
				(*nRejects)++;

				#if(0) // For testing
				sprintf(szTest, "Blank Line: %s", szInput);
				TRACE_STR(szTest);
				#endif
				continue;
			}

			// Is line a UNIX comment?
			if((bUnixComment) && (szInput[0]=='#'))
			{
				// Reject line if it starts with a # (UNIX comment)
				// TRACE_STR("UNIX Comment");

				#if(1) // Trial code 01/06/12
				// Look for # Size= and # Cols= header items
				// If both present, these will determine nSize and col
				// and shorten file reading time by terminating file information gathering
				// Check for # Size=
				if(szInput[1]==' ' && szInput[2]=='S' && szInput[3]=='i' && szInput[4]=='z' && szInput[5]=='e' && szInput[6]=='=')
				{
					// Save size in s
					// TRACE_STR(szInput);
					s=(long)atoi(&szInput[7]);
					// TRACE_INT((int)s);
				}
				// Check for # Cols=
				if(szInput[1]==' ' && szInput[2]=='C' && szInput[3]=='o' && szInput[4]=='l' && szInput[5]=='s' && szInput[6]=='=')
				{
					// Save # columns in c
					// TRACE_STR(szInput);
					c=atoi(&szInput[7]);
					// TRACE_INT(c);
				}
				#endif
				
				#if(1) // Trial code 01/07/12
				// Look for # DatCol= and # TagCol= header items
				// If present, these will determine the data and timetag column #s
				// and eliminate the multicolumn data dialog
				// Check for # DCol=
				if(szInput[1]==' ' && szInput[2]=='D' && szInput[3]=='a' && szInput[4]=='t' && szInput[5]=='C' && szInput[6]=='o' && szInput[7]=='l' && szInput[8]=='=')
				{
					// Save data column # in nDatCol
					// TRACE_STR(szInput);
					nDCol=atoi(&szInput[9]);
					// TRACE_INT(nDCol);
				}
				// Check for # TagCol=
				if(szInput[1]==' ' && szInput[2]=='T' && szInput[3]=='a' && szInput[4]=='g' && szInput[5]=='C' && szInput[6]=='o' && szInput[7]=='l' && szInput[8]=='=')
				{
					// Save timetag column # in nTCol
					// TRACE_STR(szInput);
					nTCol=atoi(&szInput[9]);
					// TRACE_INT(nTCol);
				}
				#endif
				
				// This is a good place to count # comment lines
				// and to extract them to a file (Comments.txt)

				#if(1) // New code 08/19/09
				// Increment # UNIX # comment lines in file
				(*nComments)++;

				// Store comment line in Comments.txt file
				fputs(szInput, txt);
				#endif

				// Increment reject count
				// Note parentheses - subtle order-of-evaluation issue
				// Must increment the value not its pointer
				(*nRejects)++;

				#if(0) // For testing
				sprintf(szTest, "UNIX Comment: %s", szInput);
				TRACE_STR(szTest);
				#endif

				// Increment line count
				nSize++;

				continue;
			}
		
			// Save 1st line of input file
			if(nSize==0)
			{
				strncpy(lpstrText, szInput, BUFFER_SIZE);

				// Make sure last character is a NULL
				lpstrText[BUFFER_SIZE]='\0';

				// Set bLongLine flag if input line is >80 characters
				bLongLine=FALSE;
				if(strlen(szInput)>80)
				{
					bLongLine=TRUE;
				}
			}

			#if(1) // Trial code 01/06/12
			// Check if we have both size and # columns from header items
			// If so, set nSize to s and col to c and return
			// PROBLEM:  # TagCol= and # DatCol= header items may not have been read yet
			//           so globals nTCol and nDCol may not have been set
			// SOLUTION: Require that those header items preceed # Size= and # Cols=
			if(s && c)
			{
				// TRACE_INT(nSize);
				// TRACE_STR(szInput);

				// Set size and # columns
				nSize=s;
				col=c;
				*nCols=col;

				// Done
				rewind(in);

				// Close Comments.txt file
				fclose(txt);

				// TRACE_INT(nSize);
				// TRACE_INT(*nRejects);
				// TRACE_INT(*nComments);

				return nSize;
			}
			#endif

			// Save each numeric line in buffer
			// This test will reject blank lines, or lines w/o numbers
			// It will accept a line with any digit anywhere
			// The last such line will be used to determine the # of columns
			if(strpbrk(szInput, "0123456789"))
			{
				// Copy input line into working buffer
				strlcpy(szBuffer, szInput, sizeof(szBuffer));

				// Comment 08/10/08
				// Scan line for IP address and reformat them?
				// Logic is quite complex - probably not worth it
				// Special case - would slow down other cases
				// Comment 08/11/08
				// IP addresses are no problem with new column count method
				
				// Process the line that will determine the # of columns
				// This will be repeated until it is done for the last line
				// that is an actual data line - not a comment
				for(i=0; i<(int)strlen(szBuffer); i++)
				{
					// Handle European comma decimal point
					// Note: This excludes use of comma as field delimiter
					if(bComma)
					{
						// Convert commas into decimal points
						if(szBuffer[i]==',')
						{
							szBuffer[i]='.';
						}
					}
					else // Standard comma usage
					{
						// Convert commas into spaces so comma-delimited data
						// can be read
						if(szBuffer[i]==',')
						{
							szBuffer[i]=' ';
						}
					}		

					// Replace slashes & colons with zeros so dates & times
					// can be read
					if(szBuffer[i]=='/')
					{
						szBuffer[i]='0';
					}

					if(szBuffer[i]==':')
					{
						szBuffer[i]='0';
					}

					#if(1) // New code 08/10/08
					// Replace tabs with spaces so tab-delimited data
					// can be read
					if(szBuffer[i]=='\t')
					{
						szBuffer[i]=' ';
					}
					#endif

					#if(1) // New code 08/11/08
					// Replace bars with spaces so bar-delimited data
					// can be read
					if(szBuffer[i]=='|')
					{
						szBuffer[i]=' ';
					}
					#endif
				}

				i=0;
				j=nSkipChar;

				// Eliminate up to nSkipChar # of non-numeric characters
				bDigit=FALSE;
				while( (j) && (i<(int)strlen(szBuffer)))

				{
					if(!IsNumeric(szBuffer[i], &bDigit))
					{
						szBuffer[i]=' ';
						j--;
					}
					i++;
				}
				
				// Ignore leading whitespace
				while(i<(int)strlen(szBuffer))
				{
					if(isspace(szBuffer[i]))
					{
						i++;
					}
					else
					{
						break;
					}
				}

				// Is next character numeric?
				bDigit=FALSE;
				if(IsNumeric(szBuffer[i], &bDigit))
				{
					#if(0) // Obsolete code
					// Note 08/010/08
					// A new factor to consider: a field with a dotted IP address
					// e.g., 192.168.2.1
					// As-is, This will parse as three numeric values:
					// #1=192.168, #2=0.2 and #3=0.1
					// What to do?  We don't want to generate 3 values
					// but we don't want to change the # of characters in the line either
					// by removing the last 2 decimal points
					// Also, the European comma substitution has already created 2 extra values
					// So we have to deal with this before doing that
					// Perhaps the best thing to do is to turn the IP address into a numeric
					// value like 192.1682100, replacing the 2 extra dots with trailing zeros
					// Can't do it as integer because 255255255255 is too big for 32-bit int
					// 255.255.255.255 would become 255.255255255, which is fine
					// See above
					
					// Could break up the line into strings and read them individually
					// but that's a big change that could break current user procedures
					// unless the result was exactly the same
					
					// Count # columns in last non-blank line
					// That does not start with up to nSkipChar
					// non-numeric chars
					// Max # cols = 8 for normal case - more than 8 invokes
					// special handling
					// This statement allows up to 32 columns
					*nCols=sscanf(szBuffer,
						"%le %le %le %le %le %le %le %le %1e %le %le %le %le "
						"%le %le %le %le %1e %le %le %le %le %le %le %le %le "
						"%1e %le %le %le %le %le",
						&n1, &n2, &n3, &n4, &n5, &n6, &n7, &n8, &n9, &n9,
						&n9, &n9, &n9, &n9, &n9, &n9, &n9, &n9, &n9, &n9,
						&n9, &n9, &n9, &n9, &n9, &n9, &n9, &n9, &n9, &n9,
						&n9, &n9);
					#endif

					// Save raw and parsed first active line
					if(flag)
					{
						strncpy(szFirstRaw, szInput, BIG_BUFFER_SIZE);
						strncpy(szFirstParse, szBuffer, BIG_BUFFER_SIZE);
						flag=FALSE;
					}
					
					// Show last data line
					// Note 08/09/08: lpstrText made BIG_BUFFER_SIZE+1 to show entire long input line
					// But making size bigger here causes file reading to fail
					// The larger lpstrText size has other effects elsewhere in code
					// So original size restored
					strncpy(lpstrText, szInput, BUFFER_SIZE);

					// New code 08/09/08 - save last raw and parsed input lines in large global buffers
					strncpy(szRaw, szInput, BIG_BUFFER_SIZE);
					strncpy(szParse, szBuffer, BIG_BUFFER_SIZE);
				}
				else // Non-numeric line
				{
					(*nRejects)++;

					#if(0) // For testing
					sprintf(szText, "Non-numeric Character: %s", szInput);
					TRACE_STR(szText);
					#endif
				}
			}
			else // Line doesn't contain a numeric character
			{
				(*nRejects)++;

				#if(0) // For testing
				sprintf(szText, "Non-numeric Line: %s", szInput);
				TRACE_STR(szText);
				#endif
			}

			// Increment line count
			nSize++;
		}

		#if(1) // New code 08/11/08
		// Alternative (better) way to count columns
		// szParse has contents of last numeric line with space delimiters
		//i=0; // Don't initialize to zero - want to start after leading whitespace

		// For testing
		// TRACE_INT(i);

		col=0;
		while(1)
		{
			// At start of field
			// Scan for next space delimiter or EOS,LF,FF,CR,etc.
			while((szParse[i]!=' ')&&(!iscntrl(szParse[i])))
			{
				i++;
			}
			
			// Increment column count
			col++;

			// Check if max # cols exceeded
			if(col>MAX_NUM_COLS)
			{
				LoadString(hInst, IDS_MESSAGE_TOOMANYCOLS, szBuffer, BIG_BUFFER_SIZE);
				strlcat(szBuffer, " in file:\n", sizeof(szBuffer));
				strlcat(szBuffer, lpstrFileName, sizeof(szBuffer));
				strlcat(szBuffer, "\n", sizeof(szBuffer));
				_snprintf(szText, sizeof(szText), "Columns > %d ignored", MAX_NUM_COLS);
				strlcat(szBuffer, szText, sizeof(szBuffer));
				MessageBox(hwndMain, szBuffer, "File Open",
					MB_ICONEXCLAMATION);
				col--;
				break;
			}

			// Now at space or EOS etc.
			// Quit if we're at EOS etc.
			if(iscntrl(szParse[i]))
			{
				break;
			}

			// Now at space
			// Move through any more spaces in this place
			while((szParse[i+1]==' ')||(iscntrl(szParse[i+1])))
			{
				i++;
			}

			// Now at last space or EOS etc.
			// Quit if we're at EOS etc.
			if(iscntrl(szParse[i]))
			{
				break;
			}

			// Now at last space between fields
			// Increment szParser index
			i++;
					
			// Now at start of next field
		}

		// Show message
		WriteMessage("Data File Information");
	
		// Assign column count
		*nCols=col;
		#endif

	}
	else // Can't open file
	{
		LoadString(hInst, IDS_MESSAGE_CANTOPEN, szBuffer, BIG_BUFFER_SIZE);
		strlcat(szBuffer, lpstrFileName, sizeof(szBuffer));
		MessageBox(hwndMain, szBuffer, "File Open",
			MB_ICONEXCLAMATION);
		return -1;
	}
	
	// Done
	rewind(in);

	#if(1) // New code 08/19/09
	// Close Comments.txt file
	fclose(txt);
	#endif

	return nSize;
}

#ifndef LITE_VER
/*****************************************************************************/
/*                                                                           */
/*                             SigmaRead()                                   */
/*                                                                           */
/*  Function to read stability data file                                     */
/*                                                                           */
/*  Stability file format is same as Stable/DOS.  Columns are as follows:    */
/*                                                                           */
/*  | Tau | # Analysis Points | Sigma | Min Sigma | Max Sigma |              */
/*                                                                           */
/*  Parameters:		LPSTR 	lpstrFileName	Name of data file                */
/*					INT     nSigmaType		Sigma type:                      */
/*												NORMAL_RUN          0        */
/*												OVERLAPPING_RUN     1        */
/*												MODIFIED_RUN        2        */
/*												TIME_RUN            3        */
/*												HADAMARD_RUN        4        */
/*												OVERLAPHAD_RUN      5        */
/*												TOTAL_RUN           6        */
/*												MODTOTAL_RUN		7		 */
/*												TIMETOTAL_RUN		8        */
/*												BOTH_RUN            9        */
/*												TIERMS_RUN		   10        */
/*												MTIE_RUN		   11        */
/*                                                                           */
/*	Globals:		sNormSigData[]			Normal sigma data structure      */
/*                  sModSigData[]           Modified sigma data structure    */
/*												                             */
/*	The sigmadata structures have the following relevant items:              */
/*		int points;                             Number of sigma data points  */
/*		float tau[]                             Averaging time data array    */
/*		float num[]                             Number of data points array  */
/*		float sigma[]                           Sigma data array             */
/*		float sig_min[]                         Lower sigma error bar array  */
/*		float sig_max[]                         Upper sigma error bar array  */
/*		float std_dev[]					        Standard deviation           */
/*                                                                           */
/*  Return:			int		# sigma data points (dynamically allocated)      */
/*							or -1 if error                                   */
/*                                                                           */
/*	Revision record:                                                         */
/*		02/02/97	Updated Both sigma read format                           */
/*		06/06/98	Added provisions for TIE rms and MTIE                    */
/*		05/10/00	Added Overlapping Hadamard, MTOT and TTOT                */
/*		12/19/01    Added df                                                 */
/*		02/14/03	Added Tho1                                              */
/*		02/16/03	Added Overlapping plus Tho1                             */
/*		01/10/07	Added Tho1 and ThoH error bars                         */
/*		03/25/08	Make several fixes to ThoH case                         */
/*                                                                           */
/*****************************************************************************/

int WINAPI SigmaRead(LPSTR lpstrFileName, int nSigmaType)
{
	// Lcal variables
	// chars
	char szBuffer[BIG_BUFFER_SIZE+1];	// Text buffer
	char szInputStr[81];   				// Input buffer
	// Ints
	int i;                 				// Index
	// Misc
	FILE *in;              				// DOS file handle

	// Open inout file
	if((in=fopen(lpstrFileName, "r"))==NULL)
	{
		// error - could not open input file
		LoadString(hInst, IDS_MESSAGE_NGOPEN, szBuffer,	BUFFER_SIZE);
		MessageBox(hwndMain, szBuffer, "Stable32", MB_ICONEXCLAMATION);
		return -1;
	}

	i=0; // Initialize line index

	// Read lines from input file
	while(fgets(szInputStr,81,in) != NULL)
	{
		switch(nSigmaType)
		{
			case NORMAL_RUN:
			case OVERLAPPING_RUN:
			case OVERLAPHAD_RUN:
			case TOTAL_RUN:
			{
				// Initialize data to zero
				sNormSigData.sigma[i]=0.0;
				sNormSigData.sig_min[i]=0.0;
				sNormSigData.sig_max[i]=0.0;
				sNormSigData.df[i]=0.0;

				// Put value into data array
				sscanf(szInputStr,"%e%e%e%e%e%e", &sNormSigData.tau[i],
					&sNormSigData.num[i], &sNormSigData.sigma[i],
					&sNormSigData.sig_min[i], &sNormSigData.sig_max[i],
					&sNormSigData.df[i]);
				sNormSigData.points=i+1;
			}
			break;

			case MODIFIED_RUN:
			case TIME_RUN:
			case MODTOTAL_RUN:
			case TIMETOTAL_RUN:

			{
				// Initialize data to zero
				sModSigData.sigma[i]=0.0;
				sModSigData.sig_min[i]=0.0;
				sModSigData.sig_max[i]=0.0;
				sModSigData.df[i]=0.0;

				// Put value into data array
				sscanf(szInputStr,"%e%e%e%e%e%e", &sModSigData.tau[i],
					&sModSigData.num[i], &sModSigData.sigma[i],
					&sModSigData.sig_min[i], &sModSigData.sig_max[i],
					&sModSigData.df[i]);
				sModSigData.points=i+1;
				sNormSigData.df[i]=sModSigData.df[i];
			}
			break;

			case HADAMARD_RUN:
			case HADTOTAL_RUN:
			{
				// Initialize data to zero
				sNormSigData.sigma[i]=0.0;
				sNormSigData.sig_min[i]=0.0;
				sNormSigData.sig_max[i]=0.0;
				sNormSigData.df[i]=0.0;

				// Put value into data array
				sscanf(szInputStr,"%e%e%e%e", &sNormSigData.tau[i],
					&sNormSigData.num[i], &sNormSigData.sigma[i],
					&sNormSigData.df[i]);
				sNormSigData.points=i+1;
			}
			break;

			case BOTH_RUN:
			{
				// Initialize data to zero
				sNormSigData.sigma[i]=0.0;
				sNormSigData.sig_min[i]=0.0;
				sNormSigData.sig_max[i]=0.0;
				sNormSigData.df[i]=0.0;
				sModSigData.sigma[i]=0.0;
				sModSigData.sig_min[i]=0.0;
				sModSigData.sig_max[i]=0.0;
				sModSigData.df[i]=0.0;

				// Put value into data array
				sscanf(szInputStr,"%e%e%e%e%e%e", &sNormSigData.tau[i],
					&sNormSigData.num[i], &sNormSigData.std_dev[i],
					&sNormSigData.sigma[i], &sModSigData.sigma[i],
					&sNormSigData.df[i]);
				sModSigData.tau[i]=sNormSigData.tau[i];
				sModSigData.num[i]=sNormSigData.num[i];
				sModSigData.std_dev[i]=sNormSigData.std_dev[i];
				sModSigData.df[i]=sNormSigData.df[i];
				sNormSigData.points=i+1;
				sModSigData.points=i+1;
			}
			break;

			case TIERMS_RUN:
			{
				// Initialize data to zero
				sNormSigData.sigma[i]=0.0;
				sNormSigData.sig_min[i]=0.0;
				sNormSigData.sig_max[i]=0.0;
				sNormSigData.df[i]=0.0;

				// Put value into data array
				sscanf(szInputStr,"%e%e%e%e", &sNormSigData.tau[i],
					&sNormSigData.num[i], &sNormSigData.sigma[i],
					&sNormSigData.df[i]);
				sNormSigData.points=i+1;
			}
			break;

			case MTIE_RUN:
			// case THEO1_RUN: // Removed 01/10/07
			{
				// Initialize data to zero
				sNormSigData.sigma[i]=0.0;
				sNormSigData.sig_min[i]=0.0;
				sNormSigData.sig_max[i]=0.0;
				sNormSigData.df[i]=0.0;

				// Put value into data array
				sscanf(szInputStr,"%e%e%e%e", &sNormSigData.tau[i],
					&sNormSigData.num[i], &sNormSigData.sigma[i],
					&sNormSigData.df[i]);
				sNormSigData.points=i+1;
			}
			break;

			case THEO1_RUN:
			{
				// Initialize data to zero
				sNormSigData.sigma[i]=0.0;
				sNormSigData.sig_min[i]=0.0;
				sNormSigData.sig_max[i]=0.0;
				sNormSigData.df[i]=0.0;

				// Put value into data array
				sscanf(szInputStr,"%e%e%e%e%e%e", &sNormSigData.tau[i],
					&sNormSigData.num[i], &sNormSigData.sigma[i],
					&sNormSigData.sig_min[i], &sNormSigData.sig_max[i],
					&sNormSigData.df[i]);
				sNormSigData.points=i+1;
			}
			break;

			case FULL_RUN:
			{
				// Initialize data to zero
				sNormSigData.sigma[i]=0.0;
				sNormSigData.sig_min[i]=0.0;
				sNormSigData.sig_max[i]=0.0;
				sNormSigData.df[i]=0.0;

				// Clear any previous Tho1 error bar data
				sModSigData.sig_max[i]=0.0;
				sModSigData.sig_min[i]=0.0;

				// Put values into data array
				sscanf(szInputStr,"%e%e%e%e%e%e", &sNormSigData.tau[i],
					&sNormSigData.num[i], &sNormSigData.sigma[i],
					&sModSigData.sigma[i], &sNormSigData.sig_min[i],
					&sNormSigData.sig_max[i]);
				sNormSigData.points=i+1;
				sModSigData.points=i+1;

				// New code 03/25/08
				// AF is what?
				// Just set it to ADEV tau?
				// Or tau divided by 1st tau (presumably tau_0)
				// That sounds better
				sNormSigData.af[i]=
					(int)((sNormSigData.tau[i]/sNormSigData.tau[0])+0.5);

				if(sNormSigData.sigma[i]==0.0) // ThoBR value
				{
					sModSigData.sig_min[i]=sNormSigData.sig_min[i];
					sModSigData.sig_max[i]=sNormSigData.sig_max[i];
					sNormSigData.sig_min[i]=0.0;
					sNormSigData.sig_max[i]=0.0;

					// New code 03/25/08
					// 1st stability file column is ADEV tau
					// which was put into sNormSigData.tau[i]
					// The ThoBR tau is 75% of that
					sModSigData.tau[i]=(float)(0.75*sNormSigData.tau[i]);
					// Make the # points non-zero so get plot symbol
					sModSigData.num[i]=1;
				}
			}
			break;
		}
		i++;
	}

	fclose(in);

	return(i-1);
}
#endif

/*****************************************************************************/
/*                                                                           */
/*                        ReallocSigmaArrays()                               */
/*                                                                           */
/*  Function to reallocate size of stability data arrays                     */
/*                                                                           */
/*  Parameters:		int 	nSize			New array size                   */
/*                                                                           */
/*	Globals:		sNormSigData[]			Normal sigma data structure      */
/*                  sModSigData[]           Modified sigma data structure    */
/*												                             */
/*	The sigmadata structures have the following relevant items:              */
/*	Note: The actual structure elements are pointers to these arrays         */
/*		float tau[]                         Averaging time data array        */
/*		float num[]                         Number of data points array      */
/*		float sigma[]                       Sigma data array                 */
/*		float sig_min[]                     Lower sigma error bar array      */
/*		float sig_max[]                     Upper sigma error bar array      */
/*		float std_dev[]					    Standard deviation array         */
/*		int alpha[]                         Noise type alpha array           */
/*                                                                           */
/*  Return:			int		code			-1=NG (alloc error), 0=OK        */
/*                                                                           */
/* 	Revision record:                                                         */
/*  	06/17/99	Added error code                                         */
/*		08/19/01	Added corndata arrays                                    */
/*		12/19/01	Added df corndata items                                  */
/*		12/20/01	Added min & max corndata items                           */
/*		12/22/01	Added num items                                          */
/*		03/01/03	Added separate float and int error flags                 */
/*		04/10/10	Added sign struct member                                 */
/*		05/27/11	Added all tau arrays
/*                                                                           */
/*****************************************************************************/

int WINAPI ReallocSigmaArrays(int nSize)
{
	// Local variables
	// BOOLs
	BOOL bReAllocFloatError=FALSE;
	BOOL bReAllocIntError=FALSE;

	// Reallocate all sigmadata array sizes
	if((sNormSigData.tau=(float*)realloc(sNormSigData.tau,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sModSigData.tau=(float*)realloc(sModSigData.tau,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}

	if((sNormSigData.num=(float*)realloc(sNormSigData.num,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}

	if((sModSigData.num=(float*)realloc(sModSigData.num,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sNormSigData.sigma=(float*)realloc(sNormSigData.sigma,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sModSigData.sigma=(float*)realloc(sModSigData.sigma,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sNormSigData.ci=(float*)realloc(sNormSigData.ci,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sModSigData.ci=(float*)realloc(sModSigData.ci,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sNormSigData.sig_min=(float*)realloc(sNormSigData.sig_min,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sModSigData.sig_min=(float*)realloc(sModSigData.sig_min,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sNormSigData.sig_max=(float*)realloc(sNormSigData.sig_max,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sModSigData.sig_max=(float*)realloc(sModSigData.sig_max,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sNormSigData.std_dev=(float*)realloc(sNormSigData.std_dev,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sModSigData.std_dev=(float*)realloc(sModSigData.std_dev,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sNormSigData.df=(float*)realloc(sNormSigData.df,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sModSigData.df=(float*)realloc(sModSigData.df,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sNormSigData.alpha=(int*)realloc(sNormSigData.alpha,
		nSize*sizeof(int)))==NULL)
	{
		bReAllocIntError=TRUE;
	}
	if((sModSigData.alpha=(int*)realloc(sModSigData.alpha,
		nSize*sizeof(int)))==NULL)
	{
		bReAllocIntError=TRUE;
	}
	if((sNormSigData.af=(int*)realloc(sNormSigData.af,
		nSize*sizeof(int)))==NULL)
	{
		bReAllocIntError=TRUE;
	}
	if((sModSigData.af=(int*)realloc(sModSigData.af,
		nSize*sizeof(int)))==NULL)
	{
		bReAllocIntError=TRUE;
	}
	if((sNormSigData.sign=(int*)realloc(sNormSigData.sign,
		nSize*sizeof(int)))==NULL)
	{
		bReAllocIntError=TRUE;
	}
	if((sModSigData.sign=(int*)realloc(sModSigData.sign,
		nSize*sizeof(int)))==NULL)
	{
		bReAllocIntError=TRUE;
	}
	
	// Reallocate all all tau sigmadata array sizes
	if((sAllNormSigData.tau=(float*)realloc(sAllNormSigData.tau,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sAllModSigData.tau=(float*)realloc(sAllModSigData.tau,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}

	if((sAllNormSigData.num=(float*)realloc(sAllNormSigData.num,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}

	if((sAllModSigData.num=(float*)realloc(sAllModSigData.num,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sAllNormSigData.sigma=(float*)realloc(sAllNormSigData.sigma,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sAllModSigData.sigma=(float*)realloc(sAllModSigData.sigma,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sAllNormSigData.ci=(float*)realloc(sAllNormSigData.ci,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sAllModSigData.ci=(float*)realloc(sAllModSigData.ci,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sAllNormSigData.sig_min=(float*)realloc(sAllNormSigData.sig_min,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sAllModSigData.sig_min=(float*)realloc(sAllModSigData.sig_min,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sAllNormSigData.sig_max=(float*)realloc(sAllNormSigData.sig_max,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sAllModSigData.sig_max=(float*)realloc(sAllModSigData.sig_max,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sAllNormSigData.std_dev=(float*)realloc(sAllNormSigData.std_dev,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sAllModSigData.std_dev=(float*)realloc(sAllModSigData.std_dev,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sAllNormSigData.df=(float*)realloc(sAllNormSigData.df,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sAllModSigData.df=(float*)realloc(sAllModSigData.df,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sAllNormSigData.alpha=(int*)realloc(sAllNormSigData.alpha,
		nSize*sizeof(int)))==NULL)
	{
		bReAllocIntError=TRUE;
	}
	if((sAllModSigData.alpha=(int*)realloc(sAllModSigData.alpha,
		nSize*sizeof(int)))==NULL)
	{
		bReAllocIntError=TRUE;
	}
	if((sAllNormSigData.af=(int*)realloc(sAllNormSigData.af,
		nSize*sizeof(int)))==NULL)
	{
		bReAllocIntError=TRUE;
	}
	if((sAllModSigData.af=(int*)realloc(sAllModSigData.af,
		nSize*sizeof(int)))==NULL)
	{
		bReAllocIntError=TRUE;
	}
	if((sAllNormSigData.sign=(int*)realloc(sAllNormSigData.sign,
		nSize*sizeof(int)))==NULL)
	{
		bReAllocIntError=TRUE;
	}
	if((sAllModSigData.sign=(int*)realloc(sAllModSigData.sign,
		nSize*sizeof(int)))==NULL)
	{
		bReAllocIntError=TRUE;
	}
	
	// Reallocate all corndata array sizes
	if((sCornData.tau=(float*)realloc(sCornData.tau,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornData.num_a=(float*)realloc(sCornData.num_a,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornData.num_b=(float*)realloc(sCornData.num_b,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornData.num_c=(float*)realloc(sCornData.num_c,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornData.sig_a=(float*)realloc(sCornData.sig_a,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornData.sig_b=(float*)realloc(sCornData.sig_b,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornData.sig_c=(float*)realloc(sCornData.sig_c,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornData.sig_min_a=(float*)realloc(sCornData.sig_min_a,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornData.sig_min_b=(float*)realloc(sCornData.sig_min_b,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornData.sig_min_c=(float*)realloc(sCornData.sig_min_c,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornData.sig_max_a=(float*)realloc(sCornData.sig_max_a,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornData.sig_max_b=(float*)realloc(sCornData.sig_max_b,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornData.sig_max_c=(float*)realloc(sCornData.sig_max_c,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornData.df_a=(float*)realloc(sCornData.df_a,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornData.df_b=(float*)realloc(sCornData.df_b,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornData.df_c=(float*)realloc(sCornData.df_c,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornResults.tau=(float*)realloc(sCornResults.tau,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornResults.num_a=(float*)realloc(sCornResults.num_a,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornResults.num_b=(float*)realloc(sCornResults.num_b,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornResults.num_c=(float*)realloc(sCornResults.num_c,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornResults.sig_a=(float*)realloc(sCornResults.sig_a,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornResults.sig_b=(float*)realloc(sCornResults.sig_b,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornResults.sig_c=(float*)realloc(sCornResults.sig_c,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornResults.sig_min_a=(float*)realloc(sCornResults.sig_min_a,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornResults.sig_min_b=(float*)realloc(sCornResults.sig_min_b,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornResults.sig_min_c=(float*)realloc(sCornResults.sig_min_c,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornResults.sig_max_a=(float*)realloc(sCornResults.sig_max_a,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornResults.sig_max_b=(float*)realloc(sCornResults.sig_max_b,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornResults.sig_max_c=(float*)realloc(sCornResults.sig_max_c,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornResults.df_a=(float*)realloc(sCornResults.df_a,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornResults.df_b=(float*)realloc(sCornResults.df_b,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}
	if((sCornResults.df_c=(float*)realloc(sCornResults.df_c,
		nSize*sizeof(float)))==NULL)
	{
		bReAllocFloatError=TRUE;
	}

	// Announce memory reallocation error
	if(bReAllocFloatError)
	{
		// Display error message
		#ifdef MEM_DEBUG
		AllocErrorMessageEx(__FILE__, __LINE__,
			nSize*sizeof(float));
		#else
		AllocErrorMessage();
		#endif

		return -1; // Error code
	}
	else if(bReAllocIntError)
	{
		// Display error message
		#ifdef MEM_DEBUG
		AllocErrorMessageEx(__FILE__, __LINE__,
			nSize*sizeof(int));
		#else
		AllocErrorMessage();
		#endif

		return -1; // Error code
	}
	else
	{
		return 0;
	}
}

/*****************************************************************************/
/*                                                                           */
/*                             DataWrite()                                   */
/*                                                                           */
/*  Function to write 1 or 2 column data file                                */
/*                                                                           */
/*                                                                           */
/*  Parameters:		LPSTR 	lpstrFileName	Name of data file                */
/*	Globals:		F_TYPE	fData[]			Data array                       */
/*					F_TYPE	fTags[]			Timetag array                    */
/*					BOOL	bSaveTimetags	Save timetags flag               */
/*  Return:			int                  	# data points or -1 if error     */
/*                                                                           */
/* 	Revision record:                                                         */
/*  	08/24/96	Written - seems OK                                       */
/*		02/21/98	Changed code to use ShortenExp()                         */
/*		04/08/01	Added code for optional header                           */
/*      04/09/01	Changed end of header lines from "/r/n" to "/n"          */
/*					Added code to shorten tau exponent                       */
/*		                                                                     */
/*****************************************************************************/

int WINAPI DataWrite(LPSTR lpstrFileName)
{
	// Local macros
	#define ONECOL_FORMAT "%20.15e\n"		  // format for saving data
	#define TWOCOL_FORMAT "%20.15e %20.15e\n" // format for timetagged data

	// Local variables
    // Chars
    char szBuffer[BUFFER_SIZE]; // Text buffer
	// Ints
	int i;                 		// Index
	int nSize;					// Size of data array
	// Ftypes
	F_TYPE fTau;				// Tau of data
	// Misc
	FILE *hOut;              	// DOS file handle (NOT an HFILE*)

	// Open output file
	if((hOut=fopen(lpstrFileName, "w"))==NULL)
	{
		// Error - could not open output file
		return -1;
	}

	// Assign nSize to # data points
	nSize=(int)fData[0];

	// Write optional header at top of data file
	// Every line must begin with a non-numeric character
	// General form is Name: Value
	if(bHeader) // Global flag
	{
		// Filename
		// Note: Would be nice to add hostname at front
		// but not all machines have one, and there is no
		// standard Win32 call for getting it
		strlcpy(szBuffer, "File: ", sizeof(szBuffer));
		strlcat(szBuffer, lpstrFileName, sizeof(szBuffer));
		strlcat(szBuffer, "\n", sizeof(szBuffer));
		fputs(szBuffer, hOut);

		// Date
		strlcpy(szBuffer, "Date: ", sizeof(szBuffer));
		strlcat(szBuffer, szMDYHMS, sizeof(szBuffer));  // Global date & string
		strlcat(szBuffer, "\n", sizeof(szBuffer));
		fputs(szBuffer, hOut);

		// Data type
		strlcpy(szBuffer, "Type: ", sizeof(szBuffer));
		if(bDataType)
		{
			strlcat(szBuffer, "Freq", sizeof(szBuffer));
		}
		else // Phase data
		{
			strlcat(szBuffer, "Phase", sizeof(szBuffer));
		}
		strlcat(szBuffer, "\n", sizeof(szBuffer));
		fputs(szBuffer, hOut);

		// # data points
		_snprintf(szBuffer, sizeof(szBuffer), "Points: %d\n", nSize);
		fputs(szBuffer, hOut);

		// Tau
		if(bDataType)
		{
			GetWindowText(hwndFreqTau, szBuffer, BUFFER_SIZE);
		}
		else // Phase data
		{
			GetWindowText(hwndPhaseTau, szBuffer, BUFFER_SIZE);
		}
        fTau=atof(szBuffer);
		_snprintf(szBuffer, sizeof(szBuffer), "Tau: %e\n", fTau);
		ShortenExp(szBuffer);
		fputs(szBuffer, hOut);

		#if(1) // New code 03/22/10
		// Add addend, multiplier, tag scale & tag offset parameters to file header
		// Addend
		if(bDataType)
		{
			_snprintf(szBuffer, sizeof(szBuffer), "Addend: %e\n", fFreqAddendInput);
		}
		else // Phase data
		{
			_snprintf(szBuffer, sizeof(szBuffer), "Addend: %e\n", fPhaseAddendInput);
		}
		ShortenExp(szBuffer);
		fputs(szBuffer, hOut);

		// Multiplier
		if(bDataType)
		{
			_snprintf(szBuffer, sizeof(szBuffer), "Multiplier: %e\n", fFreqMultiplierInput);
		}
		else // Phase data
		{
			_snprintf(szBuffer, sizeof(szBuffer), "Multiplier: %e\n", fPhaseMultiplierInput);
		}
		ShortenExp(szBuffer);
		fputs(szBuffer, hOut);

		// Tag Scale
		if(bDataType)
		{
			_snprintf(szBuffer, sizeof(szBuffer), "Tag Scale: %e\n", fFreqTagScale);
		}
		else // Phase data
		{
			_snprintf(szBuffer, sizeof(szBuffer), "Tag Scale: %e\n", fPhaseTagScale);
		}
		ShortenExp(szBuffer);
		fputs(szBuffer, hOut);

		// Tag Offset
		if(bDataType)
		{
			_snprintf(szBuffer, sizeof(szBuffer), "Tag Offset: %e\n", fFreqTagOffset);
		}
		else // Phase data
		{
			_snprintf(szBuffer, sizeof(szBuffer), "Tag Offset: %e\n", fPhaseTagOffset);
		}
		ShortenExp(szBuffer);
		fputs(szBuffer, hOut);
		#endif

		#if(1) // New code 04/07/10
		// Add header end flag to file header
		strncpy(szBuffer, "# Header End\n", BUFFER_SIZE);
		fputs(szBuffer, hOut);
		#endif
	}
	
	// Write lines to output file
	// Use _snprintf() & fputs() instead of fprintf() so ShortenExp() can be
	// used to shorten exponent to 2 digits
	for(i=0; i<nSize; i++)
	{
		if(bSaveTimetags)
		{
			_snprintf(szBuffer, sizeof(szBuffer), TWOCOL_FORMAT,
				fTags[ARRAY_OFFSET+i],
				fData[ARRAY_OFFSET+i]);
			ShortenExp(szBuffer);
			fputs(szBuffer, hOut);
		}
		else // No timetags
		{
			_snprintf(szBuffer, sizeof(szBuffer), ONECOL_FORMAT,
				fData[ARRAY_OFFSET+i]);
			ShortenExp(szBuffer);
			fputs(szBuffer, hOut);
		}
	}

	// Close file
	fclose(hOut);

	// Done
	return(nSize);
}

/*****************************************************************************/
/*                                                                           */
/*                             PSDWrite()                                    */
/*                                                                           */
/*  Function to write 2 column PSD data file (Fourier freq & log PSD)        */
/*                                                                           */
/*                                                                           */
/*  Parameters:		LPSTR 	lpstrFileName	Name of data file                */
/*					float	fFreq[]			Freq array                       */
/*					float	fPSD[]			PSD array                        */
/*					float	fAF[]			Avg Factor array                 */
/*					int		nNum			# data points                    */
/*					int		bSmooth			Flag for PSD smoothing           */
/*  Return:			int                  	# data points or -1 if error     */
/*                                                                           */
/* 	Revision record:                                                         */
/*  	07/10/02	Created                                                  */
/*		07/11/02	Added header                                             */
/*		01/27/07	Added float fAF argument and file output                 */
/*					Added column headings                                    */
/*		01/29/07	Added bSmooth argument                                   */
/*		                                                                     */
/*****************************************************************************/

int WINAPI PSDWrite(LPSTR lpstrFileName, float fFreq[], float fPSD[],
	float fAF[], int nNum, BOOL bSmooth)
{
	// Local macros
	#define PSD_FORMAT1 "%15.8e %+15.8e %15.8e\n" // Periodogram PSD format
	#define PSD_FORMAT2 "%15.8e %+15.8e\n"        // Multitaper PSD format

	// Local variables
    // Chars
    char szBuffer[BUFFER_SIZE];     // Text buffer
    char szPSD[BUFFER_SIZE];        // PSD label
	// Ints
	int i;                 			// Index
	// Misc
	FILE *hOut;              		// DOS file handle (NOT an HFILE*)

	// Open output file
	if((hOut=fopen(lpstrFileName, "w"))==NULL)
	{
		// Error - could not open output file
		return -1;
	}

	// Write optional header at top of data file
	// Every line must begin with a non-numeric character
	// General form is Name: Value
	if(bHeader) // Global flag
	{
		// Filename
		// Note: Would be nice to add hostname at front
		// but not all machines have one, and there is no
		// standard Win32 call for getting it
		strlcpy(szBuffer, "File: ", sizeof(szBuffer));
		strlcat(szBuffer, lpstrFileName, sizeof(szBuffer));
		strlcat(szBuffer, "\n", sizeof(szBuffer));
		fputs(szBuffer, hOut);

		// Date
		strlcpy(szBuffer, "Date: ", sizeof(szBuffer));
		strlcat(szBuffer, szMDYHMS, sizeof(szBuffer));  // Global date & string
		strlcat(szBuffer, "\n", sizeof(szBuffer));
		fputs(szBuffer, hOut);

		// PSD type
		// Label depends on both bDataType and sPSPlot.plotype
		i=(bDataType)*3+sPSPlot.plotype;
		LoadString(hInst, IDS_PSPLOT_YLABEL_0+i, szPSD, BUFFER_SIZE);

		// Data type
		strlcpy(szBuffer, "Type: Fourier Freq, Hz & ", sizeof(szBuffer));
		strlcat(szBuffer, szPSD, sizeof(szBuffer));
		strlcat(szBuffer, "\n", sizeof(szBuffer));
		fputs(szBuffer, hOut);

		// PSD Info
		strlcpy(szBuffer, sPSPlot.right_y_label, sizeof(szBuffer));
		if(sPSPlot.win==MULTITAPER_WIN)
		{
			// Eliminate @ character from label
			i=0;
			while(szBuffer[i]!='\0')
			{
				if(szBuffer[i]=='@')
				{
					szBuffer[i]=' ';
				}
				i++;
			}
		}
		strlcat(szBuffer, "\n", sizeof(szBuffer));
		ShortenExp(szBuffer);
		fputs(szBuffer, hOut);

		// # data points
		_snprintf(szBuffer, sizeof(szBuffer), "Points: %d\n", nNum);
		fputs(szBuffer, hOut);

		// Column headings
		if(sPSPlot.win==MULTITAPER_WIN)
		{
			if(bSmooth)
			{
				fputs(" Fourier Freq        PSD         Avg Factor\n", hOut);
			}
			else
			{
				fputs(" Fourier Freq        PSD\n", hOut);
			}
		}
		else
		{
			fputs(" Fourier Freq        PSD         Avg Factor\n", hOut);
		}
	}
	
	// Write lines to output file
	// Use _snprintf() & fputs() instead of fprintf() so ShortenExp() can be
	// used to shorten exponent to 2 digits
	for(i=0; i<nNum; i++)
	{
		if((sPSPlot.win==MULTITAPER_WIN) && (!bSmooth))
		{
			_snprintf(szBuffer, sizeof(szBuffer), PSD_FORMAT2, fFreq[i],
				fPSD[i]);
		}
		else
		{
			_snprintf(szBuffer, sizeof(szBuffer), PSD_FORMAT1, fFreq[i],
				fPSD[i], fAF[i]);
		}
		ShortenExp(szBuffer);
		fputs(szBuffer, hOut);
	}

	fclose(hOut);

	return(nNum);
}

/*****************************************************************************/
/*                                                                           */
/*                             AutoWrite()                                   */
/*                                                                           */
/*  Function to write 2 column autocorrelation data file (lag & value)       */
/*                                                                           */
/*                                                                           */
/*  Parameters:		LPSTR 	lpstrFileName	Name of data file                */
/*					float	fAuto[]			Autocorrelation data array       */
/*					int		nNum			# data points                    */
/*  Return:			int                  	# data points or -1 if error     */
/*                                                                           */
/* 	Revision record:                                                         */
/*  	08/09/03	Created                                                  */
/*		                                                                     */
/*****************************************************************************/

int WINAPI AutoWrite(LPSTR lpstrFileName, float fAuto[], int nNum)
{
	// Local macros
	#define AUTO_FORMAT "%d %12.8e\n" // format for autocorrelation data

	// Local variables
    // Chars
    char szBuffer[BUFFER_SIZE];     // Text buffer
	// Ints
	int i;                 			// Index
	// Misc
	FILE *hOut;              		// DOS file handle (NOT an HFILE*)

	// Open output file
	if((hOut=fopen(lpstrFileName, "w"))==NULL)
	{
		// Error - could not open output file
		return -1;
	}

	// Write optional header at top of data file
	// Every line must begin with a non-numeric character
	// General form is Name: Value
	if(bHeader) // Global flag
	{
		// Filename
		// Note: Would be nice to add hostname at front
		// but not all machines have one, and there is no
		// standard Win32 call for getting it
		strlcpy(szBuffer, "File: ", sizeof(szBuffer));
		strlcat(szBuffer, lpstrFileName, sizeof(szBuffer));
		strlcat(szBuffer, "\n", sizeof(szBuffer));
		fputs(szBuffer, hOut);

		// Date
		strlcpy(szBuffer, "Date: ", sizeof(szBuffer));
		strlcat(szBuffer, szMDYHMS, sizeof(szBuffer));  // Global date & string
		strlcat(szBuffer, "\n", sizeof(szBuffer));
		fputs(szBuffer, hOut);

		// # data points
		_snprintf(szBuffer, sizeof(szBuffer), "Points: %d\n", nNum);
		fputs(szBuffer, hOut);
	}
	
	// Write lines to output file
	// Use _snprintf() & fputs() instead of fprintf() so ShortenExp() can be
	// used to shorten exponent to 2 digits
	for(i=0; i<nNum; i++)
	{
		_snprintf(szBuffer, sizeof(szBuffer), AUTO_FORMAT, i, fAuto[i]);
		ShortenExp(szBuffer);
		fputs(szBuffer, hOut);
	}

	fclose(hOut);

	return(nNum);
}

/*****************************************************************************/
/*                                                                           */
/*                             WaveWrite()                                   */
/*                                                                           */
/*  Function to write .WAV sound file of phase or frequency data             */
/*                                                                           */
/*  Selected phase or freq data is scaled to +/-32,768 range of a 16-bit     */
/*  signed short int and written to file with .WAV sound file header         */
/*                                                                           */
/*  Parameters:		LPSTR 	szFilename	Name of .WAV file                    */
/*                                                                           */
/*	Globals:		F_TYPE	fData[]		Data array                           */
/*                  BOOL	bDataType   Data type (0=Phase, 1=Freq)          */
/*                                                                           */
/*  Return:			int                 # data points written                */
/*										or -1 if error opening file          */
/*                                                                           */
/*  References:                                                              */
/*	  1.  Steve Moshier, "Pink Noise Digital Waveform Generator Program",    */
/*	      November, 2003, http://www.moshier.net/pink.html                   */
/*        Note:  Info only - this code not used                              */
/*    2.  WAVE PCM Sound File Format - good info                             */
/*        http://ccrma.stanford.edu/CCRMA/Courses/422/projects/WaveFormat    */
/*    3.  Wave File Format http://www.sonicspot.com/guide/wavefiles.html     */
/*                                                                           */
/*  Notes:                                                                   */
/*    1.  Click on Stable32.wav file to launch default audio player          */
/*    2.  Within program, use WinExec(Stable32.wav, SW_SHOWNORMAL)           */
/*        to launch default sound player with Stable32 wave file             */
/*                                                                           */
/* 	Revision record:                                                         */
/*  	07/26/07	Created                                                  */
/*		07/27/07	Working                                                  */
/*		07/28/07	Added nSampleRate                                        */
/*		08/03/07	Added log file write                                     */
/*		                                                                     */
/*****************************************************************************/

int WINAPI WaveWrite(LPSTR szFilename)
{
	// Local variables
    // Chars
    char szBuffer[BUFFER_SIZE]; // Text buffer
	// Ints
	int i;                 		// Index and general 4 bytes
	int nSize;					// Size of data array
	// Unsigned Ints
	unsigned int u;				// General unsigned int and 2 bytes
	// Ftypes
    F_TYPE fAvg;                // Average data value
    F_TYPE fMin;                // Minimum data value
    F_TYPE fMax;                // Maximum data value
	F_TYPE fRange;              // Range of data values
	// Shorts
	short *sdata;				// Ptr to short int WAV file data array 
	// Misc
	FILE *hOut;              	// DOS file handle

	// Open output file for writing in binary mode 
	if((hOut=fopen(szFilename, "wb"))==NULL)
	{
		// Error - could not open output file
		LoadString(hInst, IDS_MESSAGE_CANTOPEN, szBuffer, BIG_BUFFER_SIZE);
		strlcat(szBuffer, szFilename, sizeof(szBuffer));
		MessageBox(hwndMain, szBuffer, "Write WAV File",
			MB_ICONEXCLAMATION);
		return -1;
	}

	// Assign nSize to # data points (again)
	nSize=(int)fData[0];

	// Allocate memory for short int (16-bit) array
	if((sdata=(short*)malloc(nSize*sizeof(short)))==NULL)
	{
		// Nothing useful can be done, so announce error
		// and return 0 points
		AllocErrorMessage();
		return 0;
	}

	// Write 36-byte WAV header
	// Use fwrite(buffer, size(bytes), # items, file stream)
	// Write 4-byte ChunkID "RIFF" in ASCII big endian
	// R=0x52, I=0x49, F=0x46, R=0x46
	i=0x46464952;
	fwrite(&i, 4, 1, hOut);
	// Write 4-byte ChunkSize = 36 + # data bytes = 36 + 2*nSize
	// as little endian unsigned int
	u=36+2*nSize;
	fwrite(&u, 4, 1, hOut);
	// Write 4-byte Format "WAVE" in ASCII big endian
	// W=0x57, A=0x41, V=0x56, E=0x45
	i=0x45564157;
	fwrite(&i, 4, 1, hOut);
	//Write 4-byte Subchunk1ID "fmt " in ASCII big endian
	// f=0x66, m=0x6d, t=0x74, space=0x20
	i=0x20746d66;
	fwrite(&i, 4, 1, hOut);
	// Write 4-byte Subchunck1Size = 16 for PCM
	// as little endian unsigned int
	u=16;
	fwrite(&u, 4, 1, hOut);
	// Write 2-byte AudioFormat = PCM = 1
	// and 2-byte NumChannels = Mono = 1
	i=0x00010001;
	fwrite(&i, 4, 1, hOut);
	// Write 4-byte SampleRate (Default=8000)
	// as little endian unsigned int
	// Note:  Nothing magic about this sample rate
	// It has no relationship with the actual data tau
	// This rate just seems to sound good
	// Standard audio sampling rates (samples/sec) are:
	//  8000	Telephone
	// 11025
	// 22050	Radio
	// 32000
	// 44100	CD
	// 48000	Professional audio
	u=nSampleRate;
	fwrite(&u, 4, 1, hOut);
	// Write 4-byte ByteRate = 2X sample rate (Default=16000)
	// ByteRate = sample rate * num chans *bytes per sample = 8000*1*2 = 16000
	// as little endian unsigned int
	u=2*nSampleRate;
	fwrite(&u, 4, 1, hOut);
	// Write 2-byte BlockAlign = 2
	// BlockAlign = num chans * bytes per sample = 1*2 = 2 
	// and 2-byte BitsPerSample = 16
	i=0x00100002;
	fwrite(&i, 4, 1, hOut);

	// Write 8-byte data header
	// Sound data is 16-bit signed int
	// Write Subchunk2ID = "data" in ASCII big endian
	// d=0x64, a=0x61, t=0x74, a=0x61
	i=0x61746164;
	fwrite(&i, 4, 1, hOut);
	// Write Subchunk2Size = # bytes in data = 2*nSize
	u=2*nSize;
	fwrite(&u, 4, 1, hOut);

	// Find the average value
    CalcMean(fData, &fAvg, bDataType);

	// Get max, min, and range of fData array
    FindMinMax(fData, &fMin, &fMax, bDataType);
	
	// Remove average, calc range and scale it
	// Overall scaling is (Value-Avg)*(Span/Range)
	// Where Span is desired span of O/P file
	// For 16-bit signed short int, span is +/-32,768
	fRange=fMax-fMin;

	// Copy F_TYPE data to short array
	for(i=0; i<nSize; i++)
	{
		// Scale data to short int with range of about +/-32768
		// Make FS = 50,000 to provide overload margin
		// Copy data, scaling it on the fly
		sdata[i]=(short)((fData[i+ARRAY_OFFSET]-fAvg)*(50000/fRange));
	}

	// Write scaled data to file
	// fwrite() writes nSize items of sizeof(short) bytes
	// from buffer sdata to file stream hOut
	i=fwrite(sdata, sizeof(short), nSize, hOut);

	// Check for right # points written
	if(i<nSize)
	{
		_snprintf(szBuffer, sizeof(szBuffer),
			"Only %d of %d Points Written to WAV File", i, nSize);
		MessageBox(hwndMain, szBuffer, "Write WAV File",
			MB_ICONEXCLAMATION);
	}

	// Close file
	fclose(hOut);

	// Write message to log
	_snprintf(szBuffer, sizeof(szBuffer), " File=Stable32.wav, #=%d, Rate=%d",
		nSize, nSampleRate);
	WriteLog(76, szBuffer);
	
	// Done
	return(i);
}

/*****************************************************************************/

#ifndef LITE_VER
/*****************************************************************************/
/*                                                                           */
/*                            CommonDialogError()                            */
/*                                                                           */
/*  Function to handle common dialog error                                   */
/*  Calls CommDlgExtended Error()                                            */
/*  Returns error code value and string with name of error code              */
/*  Currently handles general and GetOpen/SaveFileName() errors only         */
/*                                                                           */
/*  Parameters:		LPSTR 	szErrorCode		Error code text                  */
/*                                                                           */
/*  Return:			int     nError         	Error code value                 */
/*                                                                           */
/* 	Revision record:                                                         */
/*  	09/15/98	Written                                                  */
/*                                                                           */
/*****************************************************************************/

int WINAPI CommonDialogError(LPSTR szErrorCode)
{
	// Local variables
	// Ints
	int nError;						// Error code value

	// Get error code
	nError=(int)CommDlgExtendedError();

	// Convert it to a string
	switch(nError)
	{
		// General error codes:
		case CDERR_FINDRESFAILURE:
		{
			strlcpy(szErrorCode, "CDERR_FINDRESFAILURE", sizeof(szErrorCode));
		}
		break;

		case CDERR_INITIALIZATION:
		{
			strlcpy(szErrorCode, "CDERR_INITIALIZATION", sizeof(szErrorCode));
		}
		break;

		case CDERR_LOCKRESFAILURE:
		{
			strlcpy(szErrorCode, "CDERR_LOCKRESFAILURE", sizeof(szErrorCode));
		}
		break;

		case CDERR_LOADSTRFAILURE:
		{
			strlcpy(szErrorCode, "CDERR_LOADSTRFAILURE", sizeof(szErrorCode));
		}
		break;

		case CDERR_MEMALLOCFAILURE:
		{
			strlcpy(szErrorCode, "CDERR_MEMALLOCFAILURE", sizeof(szErrorCode));
		}
		break;

		case CDERR_MEMLOCKFAILURE:
		{
			strlcpy(szErrorCode, "CDERR_MEMLOCKFAILURE", sizeof(szErrorCode));
		}
		break;

		case CDERR_NOHINSTANCE:
		{
			strlcpy(szErrorCode, "CDERR_NOHINSTANCE", sizeof(szErrorCode));
		}
		break;

		case CDERR_NOHOOK:
		{
			strlcpy(szErrorCode, "CDERR_NOHOOK", sizeof(szErrorCode));
		}
		break;

		case CDERR_NOTEMPLATE:
		{
			strlcpy(szErrorCode, "CDERR_NOTEMPLATE", sizeof(szErrorCode));
		}
		break;

		case CDERR_STRUCTSIZE:
		{
			strlcpy(szErrorCode, "CDERR_STRUCTSIZE", sizeof(szErrorCode));
		}
		break;

		// GetOpen/SaveFileName() error codes:
		case FNERR_BUFFERTOOSMALL:
		{
			strlcpy(szErrorCode, "FNERR_BUFFERTOOSMALL", sizeof(szErrorCode));
		}
		break;

		case FNERR_INVALIDFILENAME:
		{
			strlcpy(szErrorCode, "FNERR_INVALIDFILENAME", sizeof(szErrorCode));
		}
		break;

		case FNERR_SUBCLASSFAILURE:
		{
			strlcpy(szErrorCode, "FNERR_SUBCLASSFAILURE", sizeof(szErrorCode));
		}
		break;

		// Undefined error code
		// Perhaps from another common dialog function
		// Or one that was left out - by mistake or whatever
		default:
		{
			strlcpy(szErrorCode, "Undefined Error", sizeof(szErrorCode));
		}
		break;
	}

	// Return error code value
	return nError;
}

/*****************************************************************************/
#endif

/*****************************************************************************/
/*                                                                           */
/* General note regarding examining data file for file header information    */
/* with TauInfo(), AddendInfo(), MultiplierInfo(), TagScaleInfo() or         */
/* TagOffsetInfo():                                                          */
/*                                                                           */
/* These functions may be called in Stable_w.c and Stable_q.c depending      */
/* on whether their local bCheckFileHeader flag is set.  There is no         */
/* user inteface for that flag.  Checking for header information seems       */
/* quite fast compared with reading the data or plotting it on the status    */
/* bar or file opened dialog.  So there doesn't seem to be any reason not    */
/* to check.  The addend, multiplier, tag scale and tag offset items         */
/* are new in 03/22/10 but there is no backward compatibility issue with     */
/* checking for them.  If this header info is found, it "sticks" as  the     */
/* inputs settings even when the program is reopened.                        */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* We also added another header item to terminate checking for any more      */
/* header items in a file.  That speeds things up by eliminating the search  */
/* through the whole file.  The format of this flag is: # Header End         */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/*                              TauInfo()                                    */
/*                                                                           */
/*  Function to read tau info in data file header                            */
/*                                                                           */
/*  Parameters:		LPSTR  lpstrFileName  Name of data file                  */
/*					F_TYPE *fTau          Tau value (0 if none)              */
/*					                                                         */
/*  Return:			BOOL   bHeader        0=no tau, 1=tau value read         */
/*                                                                           */
/*	Revision record:                                                         */
/*  	08/06/01	Created                                                  */
/*		04/07/10	Added # Header End flag                                  */
/*                                                                           */
/*****************************************************************************/

BOOL TauInfo(LPSTR lpstrFileName, F_TYPE *fTau)
{
	// Local variables
	// Chars
	char szBuffer[BIG_BUFFER_SIZE+1];	// Text buffer
	char szInput[BIG_BUFFER_SIZE+1];	// Input string
	// Ints
	// Longs
	// F_TYPEs
	//Misc
	FILE *in;                 		  	// File pointer

	#if(0)
	TRACE_STR("TauInfo()");
	#endif
	
	// Open input file
	if((in=fopen(lpstrFileName,"r"))!=NULL)
	{
		// Read file line-by-line
		while(fgets(szInput, BIG_BUFFER_SIZE, in) != NULL)
		{
			// Check if blank line
			// fgets() will put \n ahead of \0 of input string of a blank line
			// and the first character of the string will be either \r or \n
			if((szInput[0]=='\r') || (szInput[0]=='\n'))
			{
				// Skip blank line
				continue;
			}

			// Parse line of input file

			// Check for # Header End flag to terminate search
			// Must be exactly like: # Header End
			if( (szInput[0]=='#') && (szInput[1]==' ') && (szInput[2]=='H') &&
				(szInput[3]=='e') && (szInput[4]=='a') && (szInput[5]=='d') &&
				(szInput[6]=='e') && (szInput[7]=='r') && (szInput[8]==' ') &&
				(szInput[9]=='E') && (szInput[10]=='n') && (szInput[11]=='d') )
			{
				rewind(in);
				return 1;
			}

			// Check for leading # (UNIX-style comment)
			// Must be exactly like: # Tau=1.0
			if( (szInput[0]=='#') && (szInput[1]==' ') && (szInput[2]=='T') &&
				(szInput[3]=='a') && (szInput[4]=='u') && (szInput[5]=='=') )
			{
				strlcpy(szBuffer, szInput, sizeof(szBuffer));
				szBuffer[0]=' ';
				szBuffer[1]=' ';
				szBuffer[2]=' ';
				szBuffer[3]=' ';
				szBuffer[4]=' ';
				szBuffer[5]=' ';
				*fTau=atof(szBuffer);
				rewind(in);
				return 1;
			}

			// Check for Tau: (Stable32-style header)
			// Must be exactly like: Tau: 1.0
			if( (szInput[0]=='T') && (szInput[1]=='a') && (szInput[2]=='u') &&
				(szInput[3]==':') )
			{
				strlcpy(szBuffer, szInput, sizeof(szBuffer));
				szBuffer[0]=' ';
				szBuffer[1]=' ';
				szBuffer[2]=' ';
				szBuffer[3]=' ';
				*fTau=atof(szBuffer);
				rewind(in);
				return 1;
			}
		}
	}
	else // Can't open file
	{
		LoadString(hInst, IDS_MESSAGE_CANTOPEN, szBuffer, BIG_BUFFER_SIZE);
		strlcat(szBuffer, lpstrFileName, sizeof(szBuffer));
		MessageBox(hwndMain, szBuffer, "File Open",
			MB_ICONEXCLAMATION);
		return -1;
	}
	
	// Done
	rewind(in);
	return 0;
}
/*****************************************************************************/

#if(1) // Drafts of new functions 03/22/10
/*****************************************************************************/
/*                                                                           */
/*                            AddendInfo()                                   */
/*                                                                           */
/*  Function to read addend info in data file header                         */
/*                                                                           */
/*  Parameters:		LPSTR  lpstrFileName  Name of data file                  */
/*					F_TYPE *fTau          Addend value (0 if none)           */
/*					                                                         */
/*  Return:			BOOL   bHeader        0=no addend, 1=addend value read   */
/*                                                                           */
/*	Revision record:                                                         */
/*  	03/22/10	Created from TauInfo()                                   */
/*		04/07/10	Added # Header End flag                                  */
/*		07/23/10	Fixed error in # Header End return value                 */
/*                                                                           */
/*****************************************************************************/

BOOL AddendInfo(LPSTR lpstrFileName, F_TYPE *fAdd)
{
	// Local variables
	// Chars
	char szBuffer[BIG_BUFFER_SIZE+1];	// Text buffer
	char szInput[BIG_BUFFER_SIZE+1];	// Input string
	// Ints
	// Longs
	// F_TYPEs
	//Misc
	FILE *in;                 		  	// File pointer

	#if(0) // For testing
	TRACE_STR("AddendInfo()");
	#endif
	
	// Open input file
	if((in=fopen(lpstrFileName,"r"))!=NULL)
	{
		// Read file line-by-line
		while(fgets(szInput, BIG_BUFFER_SIZE, in) != NULL)
		{
			// Check if blank line
			// fgets() will put \n ahead of \0 of input string of a blank line
			// and the first character of the string will be either \r or \n
			if((szInput[0]=='\r') || (szInput[0]=='\n'))
			{
				// Skip blank line
				continue;
			}

			// Parse line of input file

			// Check for # Header End flag to terminate search
			// Must be exactly like: # Header End
			if( (szInput[0]=='#') && (szInput[1]==' ') && (szInput[2]=='H') &&
				(szInput[3]=='e') && (szInput[4]=='a') && (szInput[5]=='d') &&
				(szInput[6]=='e') && (szInput[7]=='r') && (szInput[8]==' ') &&
				(szInput[9]=='E') && (szInput[10]=='n') && (szInput[11]=='d') )
			{
				rewind(in);
				return 0;
			}

			// Check for leading # (UNIX-style comment)
			// Must be exactly like: # Addend=1.0
			if( (szInput[0]=='#') && (szInput[1]==' ') && (szInput[2]=='A') &&
				(szInput[3]=='d') && (szInput[4]=='d') && (szInput[5]=='e') &&
				(szInput[6]=='n') && (szInput[7]=='d') &&(szInput[8]=='=') )
			{
				strlcpy(szBuffer, szInput, sizeof(szBuffer));
				szBuffer[0]=' ';
				szBuffer[1]=' ';
				szBuffer[2]=' ';
				szBuffer[3]=' ';
				szBuffer[4]=' ';
				szBuffer[5]=' ';
				szBuffer[6]=' ';
				szBuffer[7]=' ';
				szBuffer[8]=' ';
				*fAdd=atof(szBuffer);
				rewind(in);
				return 1;
			}

			// Check for Addend: (Stable32-style header)
			// Must be exactly like: Addend: 1.0
			if( (szInput[0]=='A') && (szInput[1]=='d') && (szInput[2]=='d') &&
				(szInput[3]=='e') && (szInput[4]=='n') && (szInput[5]=='d') &&
				(szInput[6]==':'))
			{
				strlcpy(szBuffer, szInput, sizeof(szBuffer));
				szBuffer[0]=' ';
				szBuffer[1]=' ';
				szBuffer[2]=' ';
				szBuffer[3]=' ';
				szBuffer[4]=' ';
				szBuffer[5]=' ';
				szBuffer[6]=' ';
				*fAdd=atof(szBuffer);
				rewind(in);
				return 1;
			}
		}
	}
	else // Can't open file
	{
		LoadString(hInst, IDS_MESSAGE_CANTOPEN, szBuffer, BIG_BUFFER_SIZE);
		strlcat(szBuffer, lpstrFileName, sizeof(szBuffer));
		MessageBox(hwndMain, szBuffer, "File Open",
			MB_ICONEXCLAMATION);
		return -1;
	}
	
	// Done
	rewind(in);
	return 0;
}

/*****************************************************************************/
/*                                                                           */
/*                          MultiplierInfo()                                 */
/*                                                                           */
/*  Function to read addend info in data file header                         */
/*                                                                           */
/*  Parameters:		LPSTR  lpstrFileName  Name of data file                  */
/*					F_TYPE *fMult         Multiplier value (0 if none)       */
/*					                                                         */
/*  Return:			BOOL   bHeader        0=no mult, 1=mult value read       */
/*                                                                           */
/*	Revision record:                                                         */
/*  	03/22/10	Created from TauInfo()                                   */
/*		04/07/10	Added # Header End flag                                  */
/*		07/23/10	Fixed error in # Header End return value                 */
/*                                                                           */
/*****************************************************************************/

BOOL MultiplierInfo(LPSTR lpstrFileName, F_TYPE *fMult)
{
	// Local variables
	// Chars
	char szBuffer[BIG_BUFFER_SIZE+1];	// Text buffer
	char szInput[BIG_BUFFER_SIZE+1];	// Input string
	// Ints
	// Longs
	// F_TYPEs
	//Misc
	FILE *in;                 		  	// File pointer

	#if(0) // For testing
	TRACE_STR("MultiplierInfo()");
	#endif
	
	// Open input file
	if((in=fopen(lpstrFileName,"r"))!=NULL)
	{
		// Read file line-by-line
		while(fgets(szInput, BIG_BUFFER_SIZE, in) != NULL)
		{
			// Check if blank line
			// fgets() will put \n ahead of \0 of input string of a blank line
			// and the first character of the string will be either \r or \n
			if((szInput[0]=='\r') || (szInput[0]=='\n'))
			{
				// Skip blank line
				continue;
			}

			// Parse line of input file

			// Check for # Header End flag to terminate search
			// Must be exactly like: # Header End
			if( (szInput[0]=='#') && (szInput[1]==' ') && (szInput[2]=='H') &&
				(szInput[3]=='e') && (szInput[4]=='a') && (szInput[5]=='d') &&
				(szInput[6]=='e') && (szInput[7]=='r') && (szInput[8]==' ') &&
				(szInput[9]=='E') && (szInput[10]=='n') && (szInput[11]=='d') )
			{
				rewind(in);
				return 0;
			}

			// Check for leading # (UNIX-style comment)
			// Must be exactly like: # Multiplier=1.0
			if( (szInput[0]=='#') && (szInput[1]==' ')  && (szInput[2] =='M') &&
				(szInput[3]=='u') && (szInput[4]=='l')  && (szInput[5] =='t') &&
				(szInput[6]=='i') && (szInput[7]=='p')  && (szInput[8] =='l') &&
				(szInput[9]=='i') && (szInput[10]=='e') && (szInput[11]=='r') &&
				(szInput[12]=='='))
			{
				strlcpy(szBuffer, szInput, sizeof(szBuffer));
				szBuffer[0]=' ';
				szBuffer[1]=' ';
				szBuffer[2]=' ';
				szBuffer[3]=' ';
				szBuffer[4]=' ';
				szBuffer[5]=' ';
				szBuffer[6]=' ';
				szBuffer[7]=' ';
				szBuffer[8]=' ';
				szBuffer[9]=' ';
				szBuffer[10]=' ';
				szBuffer[11]=' ';
				szBuffer[12]=' ';
				*fMult=atof(szBuffer);
				rewind(in);
				return 1;
			}

			// Check for Addend: (Stable32-style header)
			// Must be exactly like: Multiplier: 1.0
			if( (szInput[0]=='M') && (szInput[1] =='u') && (szInput[2]=='l') &&
				(szInput[3]=='t') && (szInput[4] =='i') && (szInput[5]=='p') &&
				(szInput[6]=='l') && (szInput[7] =='i') && (szInput[8]=='e') &&
				(szInput[9]=='r') && (szInput[10]==':'))
			{
				strlcpy(szBuffer, szInput, sizeof(szBuffer));
				szBuffer[0]=' ';
				szBuffer[1]=' ';
				szBuffer[2]=' ';
				szBuffer[3]=' ';
				szBuffer[4]=' ';
				szBuffer[5]=' ';
				szBuffer[6]=' ';
				szBuffer[7]=' ';
				szBuffer[8]=' ';
				szBuffer[9]=' ';
				szBuffer[10]=' ';
				*fMult=atof(szBuffer);
				rewind(in);
				return 1;
			}
		}
	}
	else // Can't open file
	{
		LoadString(hInst, IDS_MESSAGE_CANTOPEN, szBuffer, BIG_BUFFER_SIZE);
		strlcat(szBuffer, lpstrFileName, sizeof(szBuffer));
		MessageBox(hwndMain, szBuffer, "File Open",
			MB_ICONEXCLAMATION);
		return -1;
	}
	
	// Done
	rewind(in);
	return 0;
}

/*****************************************************************************/
/*                                                                           */
/*                             TagScaleInfo()                                */
/*                                                                           */
/*  Function to read addend info in data file header                         */
/*                                                                           */
/*  Parameters:		LPSTR  lpstrFileName  Name of data file                  */
/*					F_TYPE *fScale        Scale value (0 if none)            */
/*					                                                         */
/*  Return:			BOOL   bHeader        0=no scale, 1=scale value read     */
/*                                                                           */
/*	Revision record:                                                         */
/*  	03/22/10	Created from TauInfo()                                   */
/*		04/07/10	Added # Header End flag                                  */
/*		07/23/10	Fixed error in # Header End return value                 */
/*                                                                           */
/*****************************************************************************/

BOOL TagScaleInfo(LPSTR lpstrFileName, F_TYPE *fScale)
{
	// Local variables
	// Chars
	char szBuffer[BIG_BUFFER_SIZE+1];	// Text buffer
	char szInput[BIG_BUFFER_SIZE+1];	// Input string
	// Ints
	// Longs
	// F_TYPEs
	//Misc
	FILE *in;                 		  	// File pointer

	#if(0) // For testing
	TRACE_STR("TagScaleInfo()");
	#endif
	
	// Open input file
	if((in=fopen(lpstrFileName,"r"))!=NULL)
	{
		// Read file line-by-line
		while(fgets(szInput, BIG_BUFFER_SIZE, in) != NULL)
		{
			// Check if blank line
			// fgets() will put \n ahead of \0 of input string of a blank line
			// and the first character of the string will be either \r or \n
			if((szInput[0]=='\r') || (szInput[0]=='\n'))
			{
				// Skip blank line
				continue;
			}

			// Parse line of input file

			// Check for # Header End flag to terminate search
			// Must be exactly like: # Header End
			if( (szInput[0]=='#') && (szInput[1]==' ') && (szInput[2]=='H') &&
				(szInput[3]=='e') && (szInput[4]=='a') && (szInput[5]=='d') &&
				(szInput[6]=='e') && (szInput[7]=='r') && (szInput[8]==' ') &&
				(szInput[9]=='E') && (szInput[10]=='n') && (szInput[11]=='d') )
			{
				rewind(in);
				return 0;
			}

			// Check for leading # (UNIX-style comment)
			// Must be exactly like: # Tag Scale=1.0
			if( (szInput[0]=='#') && (szInput[1]==' ') && (szInput[2]=='T') &&
				(szInput[3]=='a') && (szInput[4]=='g') && (szInput[5]==' ') &&
				(szInput[6]=='S') && (szInput[7]=='c') && (szInput[8]=='a') &&
				(szInput[9]=='l')&& (szInput[10]=='e')&&(szInput[11]=='=') )
			{
				strlcpy(szBuffer, szInput, sizeof(szBuffer));
				szBuffer[0] =' ';
				szBuffer[1] =' ';
				szBuffer[2] =' ';
				szBuffer[3] =' ';
				szBuffer[4] =' ';
				szBuffer[5] =' ';
				szBuffer[6] =' ';
				szBuffer[7] =' ';
				szBuffer[8] =' ';
				szBuffer[9] =' ';
				szBuffer[10]=' ';
				szBuffer[11]=' ';
				*fScale=atof(szBuffer);
				rewind(in);
				return 1;
			}

			// Check for Addend: (Stable32-style header)
			// Must be exactly like: Tag Scale: 1.0
			if( (szInput[0]=='T') && (szInput[1]=='a') && (szInput[2]=='g') &&
				(szInput[3]==' ') && (szInput[4]=='S') && (szInput[5]=='c') &&
				(szInput[6]=='a') && (szInput[7]=='l') && (szInput[8]=='e') &&
				(szInput[9]==':'))
			{
				strlcpy(szBuffer, szInput, sizeof(szBuffer));
				szBuffer[0]=' ';
				szBuffer[1]=' ';
				szBuffer[2]=' ';
				szBuffer[3]=' ';
				szBuffer[4]=' ';
				szBuffer[5]=' ';
				szBuffer[6]=' ';
				szBuffer[7]=' ';
				szBuffer[8]=' ';
				szBuffer[9]=' ';
				*fScale=atof(szBuffer);
				rewind(in);
				return 1;
			}
		}
	}
	else // Can't open file
	{
		LoadString(hInst, IDS_MESSAGE_CANTOPEN, szBuffer, BIG_BUFFER_SIZE);
		strlcat(szBuffer, lpstrFileName, sizeof(szBuffer));
		MessageBox(hwndMain, szBuffer, "File Open",
			MB_ICONEXCLAMATION);
		return -1;
	}
	
	// Done
	rewind(in);
	return 0;
}

/*****************************************************************************/
/*                                                                           */
/*                            TagOffsetInfo()                                */
/*                                                                           */
/*  Function to read addend info in data file header                         */
/*                                                                           */
/*  Parameters:		LPSTR  lpstrFileName  Name of data file                  */
/*					F_TYPE *fOffset       Offset value (0 if none)           */
/*					                                                         */
/*  Return:			BOOL   bHeader        0=no offset, 1=offset value read   */
/*                                                                           */
/*	Revision record:                                                         */
/*  	03/22/10	Created from TauInfo()                                   */
/*		04/07/10	Added # Header End flag                                  */
/*		07/23/10	Fixed error in # Header End return value                 */
/*                                                                           */
/*****************************************************************************/

BOOL TagOffsetInfo(LPSTR lpstrFileName, F_TYPE *fOffset)
{
	// Local variables
	// Chars
	char szBuffer[BIG_BUFFER_SIZE+1];	// Text buffer
	char szInput[BIG_BUFFER_SIZE+1];	// Input string
	// Ints
	// Longs
	// F_TYPEs
	//Misc
	FILE *in;                 		  	// File pointer

	#if(0) // For testing
	TRACE_STR("TagOffsetInfo()");
	#endif
	
	// Open input file
	if((in=fopen(lpstrFileName,"r"))!=NULL)
	{
		// Read file line-by-line
		while(fgets(szInput, BIG_BUFFER_SIZE, in) != NULL)
		{
			// Check if blank line
			// fgets() will put \n ahead of \0 of input string of a blank line
			// and the first character of the string will be either \r or \n
			if((szInput[0]=='\r') || (szInput[0]=='\n'))
			{
				// Skip blank line
				continue;
			}

			// Parse line of input file

			// Check for # Header End flag to terminate search
			// Must be exactly like: # Header End
			if( (szInput[0]=='#') && (szInput[1]==' ') && (szInput[2]=='H') &&
				(szInput[3]=='e') && (szInput[4]=='a') && (szInput[5]=='d') &&
				(szInput[6]=='e') && (szInput[7]=='r') && (szInput[8]==' ') &&
				(szInput[9]=='E') && (szInput[10]=='n') && (szInput[11]=='d') )
			{
				rewind(in);
				return 0;
			}

			// Check for leading # (UNIX-style comment)
			// Must be exactly like: # Tag Offset=1.0
			if( (szInput[0]=='#') && (szInput[1]==' ')  && (szInput[2]=='T')  &&
				(szInput[3]=='a') && (szInput[4]=='g')  && (szInput[5]==' ')  &&
				(szInput[6]=='O') && (szInput[7]=='f')  && (szInput[8]=='f')  &&
				(szInput[9]=='s') && (szInput[10]=='e') && (szInput[11]=='t') &&
				(szInput[12]=='=') )
			{
				strlcpy(szBuffer, szInput, sizeof(szBuffer));
				szBuffer[0] =' ';
				szBuffer[1] =' ';
				szBuffer[2] =' ';
				szBuffer[3] =' ';
				szBuffer[4] =' ';
				szBuffer[5] =' ';
				szBuffer[6] =' ';
				szBuffer[7] =' ';
				szBuffer[8] =' ';
				szBuffer[9] =' ';
				szBuffer[10]=' ';
				szBuffer[11]=' ';
				szBuffer[12]=' ';
				*fOffset=atof(szBuffer);
				rewind(in);
				return 1;
			}

			// Check for Addend: (Stable32-style header)
			// Must be exactly like: Tag Offset: 1.0
			if( (szInput[0]=='T') && (szInput[1]=='a') && (szInput[2]=='g') &&
				(szInput[3]==' ') && (szInput[4]=='O') && (szInput[5]=='f') &&
				(szInput[6]=='f') && (szInput[7]=='s') && (szInput[8]=='e') &&
				(szInput[9]=='t') && (szInput[10]==':'))
			{
				strlcpy(szBuffer, szInput, sizeof(szBuffer));
				szBuffer[0] =' ';
				szBuffer[1] =' ';
				szBuffer[2] =' ';
				szBuffer[3] =' ';
				szBuffer[4] =' ';
				szBuffer[5] =' ';
				szBuffer[6] =' ';
				szBuffer[7] =' ';
				szBuffer[8] =' ';
				szBuffer[9] =' ';
				szBuffer[10]=' ';
				*fOffset=atof(szBuffer);
				rewind(in);
				return 1;
			}
		}
	}
	else // Can't open file
	{
		LoadString(hInst, IDS_MESSAGE_CANTOPEN, szBuffer, BIG_BUFFER_SIZE);
		strlcat(szBuffer, lpstrFileName, sizeof(szBuffer));
		MessageBox(hwndMain, szBuffer, "File Open",
			MB_ICONEXCLAMATION);
		return -1;
	}
	
	// Done
	rewind(in);
	return 0;
}
/*****************************************************************************/
#endif

#if(1)
/*****************************************************************************/
/*                                                                           */
/*                         RemoveWhitespace()                                */
/*                                                                           */
/*  Function to remove whitespace from string                                */
/*                                                                           */
/*  Parameters:		char	*str		Pointer to string to be processed    */
/*					                                                         */
/*  Return:			int		n			# whitespace characters removed      */
/*                                      or -1 if error                       */
/*                                                                           */
/*	Revision record:                                                         */
/*  	08/09/08	Created                                                  */
/*                                                                           */
/*****************************************************************************/

int RemoveWhitespace(char *str)
{
	// Local variables
	// Chars
	char c;								
	// Ints
	int i;								// Index
	int j;								// Secondary index
	int n;								// Count of chars removed


	// Initializations
	n=0;
	i=0;
	c=str[i];

	// Scan thru the string until its EOS
	while(c!='\0')
	{
		// Is current char a whitespace character?
		if(isspace(c))
		{
			// Shift characters up from current whitespace char to end
			for(j=i; j<(int)strlen(str); j++)
			{
				str[j]=str[j+1];
			}

			// Increment count
			n++;
		}
		else
		{
			// Increment index
			i++;
		}

		// Get next character
		c=str[i];
	}

	// Done
	return n;
}
#endif
