/*****************************************************************************/
/*                                                                           */
/*                            STABLE_H.C                                     */
/*                                                                           */
/* 		Stable Windows 95/NT Hammer Library Cloned Functions                 */
/*                                                                           */
/* 		Reference:		The Hammer Library - Software Tools in C             */
/*						O.E.S Systems, Pittsburgh, PA 15221                  */
/*                                                                           */
/*		Function            	Description                                  */
/*                                                                           */
/*		BreakDate()		Break Down Gregorian Date                            */
/*      DayOfWeek()		Return Day of Week for Gregorian Date                */
/*		DateToJulian()  Convert Gregorian Date to Julian Day Number          */
/*		JulianToDate()  Convert Julian Day Number to Gregorian Date          */
/* 		MakeDate()		Make Gregorian Date                                  */
/*		MJDtoGPS()      Convert MJD to GPS Week # (WJR function)             */
/*		MJDtoDOY()		Convert MJD to DOY (WJR function)                    */
/*		MJDtoDate()		Convert MJD to Calendar Date (WJR function)          */
/*                                                                           */
/*		Revision record:                                                     */
/*			07/17/97	Created from similar functions of Hammer Library     */
/*			01/25/98	Added MJDtoGPS()                                     */
/*					 	Added MJDtoDOY and MJDtoDate()                       */
/*			03/22/03	Removed from project - all in FrequenC DLL V 2.0     */
/*                                                                           */
/* 		(c) 1997-8 W. Riley Hamilton Technical Services All Rights Reserved  */
/*                                                                           */
/*****************************************************************************/

#define STRICT 					// For Win32 compatibility
#include "stable_w.h"           // STABLE_W header file

/*****************************************************************************/
/*                                                                           */
/*      						BreakDate()                                  */
/*                                                                           */
/*		Function to break down a Gregorian date into month, day and year     */
/*                                                                           */
/* 		Parameters:	LONG	gdate	Gregorian date (yyyymmdd)       	     */
/*      	        INT *	mo		month (1-12)                             */
/*                  INT *	da		day   (1-31)                             */
/*                  INT *	yr		year  (4-digit)                          */
/*                                                                           */
/*      Return:     void			None                                  	 */
/*                                                                           */
/*		Revision record:                                                     */
/*      	07/17/97	Cloned from brkdate() of Hammer Library              */
/*						Modified code so full 4-digit year is returned       */
/*                                                                           */
/*****************************************************************************/

void WINAPI BreakDate(LONG gdate, INT *mo,INT *da,INT *yr)
{
	*yr=(int) (gdate/10000L);
	*mo=(int) ((gdate/100L) % 100L);
	*da=(int) (gdate%100L);
}

/*****************************************************************************/
/*                                                                           */
/*      						DayOfWeek()                                  */
/*                                                                           */
/*		Function to find the day of the week for a given Gregorian date      */
/*                                                                           */
/*		Note: Calls Brkdate() above                                          */
/*                                                                           */
/* 		Parameters:	LONG	gdate	Gregorian date (yyyymmdd)       	     */
/*                                                                           */
/*      Return:     INT 			Day of week (0=Sunday, 6=Saturday)   	 */
/*                                                                           */
/*		Revision record:                                                     */
/*      	07/17/97	Cloned from dayofwk() of Hammer Library              */
/*						Changed error return to -1                           */
/*						Modified for 4-digit year from BreakDate()           */
/*                                                                           */
/*****************************************************************************/

INT WINAPI DayOfWeek(LONG gdate)
{
	// Local variables
	int a, b, mo, da, yr;
	static int mos[12] = { 0,3,3,6,1,4,6,2,5,0,3,5 };

	BreakDate(gdate, &mo, &da, &yr);

	// Convert 4-digit year into 2-digit year
	// yr=yr % 100;  // Only OK 1900-1999
	yr=yr-1900;      // Better; OK 1900-2099

	if ( mo<1 || mo>12 || da<1 || da>31 || yr<0 )
	{
		return (-1);	/* error - bad date */
	}

	a = (yr/4) + yr + da + mos[mo-1];
	b = (a % 7) + 1;

	if (mo<=2 && yr!=0 && (yr % 4)==0)
	{
		b = (b==0 ? 6 : b) - 1;
	}

	b = (b==0 ? 7 : b) - 1;

	return (b);	/* return day of week (0-6) */
}

/*****************************************************************************/
/*                                                                           */
/*      						DateToJulian()                               */
/*                                                                           */
/*		Function to convert Gregorian date to Julian day number              */
/*                                                                           */
/* 		Parameters:	LONG	gdate	Gregorian date (yyyymmdd)       	     */
/*                                                                           */
/*      Return:     LONG 	jdate	Julian day number                   	 */
/*                                                                           */
/*		Revision record:                                                     */
/*      	07/17/97	Cloned from datjul() of Hammer Library               */
/*						Changed error return to -1                           */
/*						Modofoed to use 4-digit year return from BreakDate() */
/*                                                                           */
/*****************************************************************************/

LONG WINAPI DateToJulian(LONG gdate)
{
	// Local variables
	LONG jdate;	/* Julian Day Number for given date */
	INT mo, da, yr, c, ya;

	if (gdate<=0L || gdate==19000000L)	/* if zero or negative, return -1 */
	{
		return (-1L); /* error */
	}

	BreakDate(gdate, &mo, &da, &yr);

	if (mo>2)
	{
		mo-=3;
	}
	else
	{
		mo+=9;
		yr--;
	}

	c=yr/100;
	ya=yr%100;
	jdate=(146097L*c)/4L + (1461L*ya)/4L + (153L*mo+2L)/5L + da + 1721119L;

	return (jdate);
}

/*****************************************************************************/
/*                                                                           */
/*      						JulianToDate()                               */
/*                                                                           */
/*		Function to convert Julian day number to Gregorian date              */
/*                                                                           */
/* 		Parameters:	LONG	jdate	Julian day number       	             */
/*                                                                           */
/*      Return:     LONG   			Gregorian date (yyyymmdd)                */
/*                                                                           */
/*		Revision record:                                                     */
/*      	07/17/97	Cloned from juldat() of Hammer Library               */
/*						Changed error code to -1                             */
/*                                                                           */
/*****************************************************************************/

LONG WINAPI JulianToDate(LONG jdate)
{
	extern long int makdate();
	long int gdate;	/* Gregorian Date (yyyymmdd) for given Julian Day */
	int mo, da, yr;
	long int td, tm, ty;

	if (jdate<=0L)	/* if zero or negative, return -1 */
	{
		return (-1L);
	}

	jdate-=1721119L;
	ty=(4L*jdate-1L)/146097L;
	jdate=4L*jdate - 1L - 146097L*ty;
	td=jdate/4L;

	jdate=(4L*td+3)/1461L;
	td=4L*td + 3L - 1461L*jdate;
	td=(td+4L)/4L;

	tm=(5L*td - 3L)/153L;
	td=5L*td - 3L - 153L*tm;
	td=(td+5L)/5L;

	ty=100L*ty + jdate;
	if (tm<10L)
	{
		tm+=3L;
	}
	else
	{
		tm-=9L, ty++;
	}

	da=(int) td; mo=(int) tm; yr=(int) ty;
	gdate=MakeDate(mo, da, yr);

	return(gdate);
}

/*****************************************************************************/
/*                                                                           */
/*      						MakeDate()                                   */
/*                                                                           */
/*		Function to make Gregorian date from month, day and year             */
/*                                                                           */
/* 		Parameters:	INT		mo		Month (1-12)            	             */
/*                  INT		da		Day (1-31)                               */
/*                  INT		yr      Year (2 or 4-digit)                      */
/*                                                                           */
/*      Return:     LONG   			Gregorian date (yyyymmdd)                */
/*                                                                           */
/*		Revision record:                                                     */
/*      	07/17/97	Cloned from makdate() of Hammer Library              */
/*                                                                           */
/*****************************************************************************/

LONG WINAPI MakeDate(INT mo, INT da, INT yr)
{
	yr = (yr<=100 ? yr+1900 : yr);
	return ((long int) yr*10000L + mo*100L + da);
}

#ifndef LITE_VER
/*****************************************************************************/
/*                                                                           */
/*      						MJDtoGPS()                                   */
/*                                                                           */
/*		Function to convert MJD to GPS Week #                                */
/*                                                                           */
/* 		Parameters:	LONG	MJD                               	             */
/*                                                                           */
/*      Return:     INT   	GPS Week #                                       */
/*                                                                           */
/*		Note:	The origin (time zero) of GPS System Time was 00:00:00 UTC   */
/*				6 January 1980, Julian Day 2,444,244.500, MJD 44244.  A      */
/*				GPS Cycle is 1024 weeks.                                     */
/*                                                                           */
/*		Revision record:                                                     */
/*      	01/25/98	Created                                              */
/*                                                                           */
/*****************************************************************************/

INT WINAPI MJDtoGPS(LONG MJD)
{
	return ((int)((MJD-44244L)/7)%1024);
}

/*****************************************************************************/
#endif

#if(0) // Now in FrequenC DLL 03/21/03
/*****************************************************************************/
/*                                                                           */
/*								MJDtoDOY()                                   */
/*                                                                           */
/* 		Function to convert long integer MJD to DOY                          */
/*                                                                           */
/*		Parameters:                                                          */
/*					long	nMJD	MJD value                                */
/*                                                                           */
/*		Return:		int		nDOY	Day of year	                             */
/*                                                                           */
/*      Dependencies:                                                        */
/*					MJDToDate()                                              */
/*                                                                           */
/*		Revision record:  												     */
/*			03/20/02	Created from GetRowData()                            */
/*                                                                           */
/*****************************************************************************/

int MJDtoDOY(long lnMJD)
{
	// Local variables
	// Ints
	int nYear;
	int nDOY;
	int nMonth;
	int nDay;
	static int nSum[] = { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304,
		334, 365 };

	// Start of code
	// Convert integer MJD to date
	MJDToDate(lnMJD, &nDay, &nMonth, &nYear);

	// Convert data to DOY
	nDOY=nSum[nMonth-1]+nDay;
	if(nYear%4==0 && nYear%100!=0 || nYear%400==0)
	{
		if(nMonth>2)
		{
			nDOY++;
		}
	}

	// Return DOY
	return nDOY;
}

/****************************************************************************/
#endif

#if(0) // Now in FrequenC DLL 03/21/03
/****************************************************************************/
/*                                                                          */
/*                             MJDtoDate()                                  */
/*                                                                          */
/*		Function for MJD to calendar date conversion                        */
/*                                                                          */
/*      Parameters:                                                         */
/*          long mjd     modified Julian day # to be converted into date    */
/*			int	*day	 pointer to day of date to be found from MJD        */
/*		    int *month   pointer to month of date to be found from MJD      */
/*          int *year    pointer to year of date to be found from MJD       */
/*                                                                          */
/*      Return: 														    */
/*			none (void)                                                     */
/*                                                                          */
/*		Function revision record:                                           */
/*			11/11/92	Created                                             */
/*                                                                          */
/*  (c) 1992	W. Riley  Hamilton Technical Services  All Rights Reserved  */
/*                                                                          */
/****************************************************************************/

void MJDtoDate(long mjd, int *day, int *month, int *year)
{
	mjd+=679019;
	*year=(int)((mjd-122.1)/365.25);
	*month=(int)((mjd-(long)(365.25**year))/30.6001);
	*day=mjd-(int)(365.25**year)-(int)(30.6001**month);

	if(*month<14)
	{
		*month-=1;
	}                                                             /* end if */
	else
	{
		*month-=13;
	}                                                           /* end else */

	if(*month<3)
	{
		*year+=1;
	}                                                             /* end if */

}                                                        /* end MjdToDate() */

/****************************************************************************/
#endif
