/*****************************************************************************/
/*                                                                           */
/*                            STABLE_N.C                                     */
/*                                                                           */
/* 		Stable Windows 95/NT Noise Generation Functions                      */
/*                                                                           */
/* 		Note:  Some of these functions could be added to FrequenC DLL        */
/*                                                                           */
/*      Function names and prototypes changed to Windows style               */
/*      Internal variables and style not changed                             */
/*                                                                           */
/*      GenerateNoise()                                                      */
/*      f_beta() - Note: Called GenNoise() in FrequenC DLL V2.0              */
/*      four1()                                                              */
/*      gasdev()                                                             */
/*      rand1()                                                              */
/*      realft()                                                             */
/*                                                                           */
/*		Revision record:                                                     */
/*			06/30/97	Replaced FAR PASCAL _export with WINAPI              */
/*			07/25/97	Change to HUGE usage for Win 32 compatibility        */
/*			12/24/97	Made changes for AOK BC 32 bit compliation           */
/*			01/16/98	Added ShortenExp() usage for MSC VC++                */
/*			06/16/02	Changed noise data generation floats to doubles      */
/*			01/13/06	Fixed problem with multiple noise types              */
/*			07/24/07    Changed to safe string functions                     */
/*                                                                           */
/*  (c) 1996-2002 W. Riley Hamilton Technical Services  All Rights Reserved  */
/*                                                                           */
/*****************************************************************************/

#define STRICT					// For Win32 compatibility

#include "stable_w.h"			// STABLE_W header file

#ifndef LITE_VER
/*****************************************************************************/
/*                                                                           */
/*                           GenerateNoise()                                 */
/*                                                                           */
/*	Top-level function to generate simulated power law phase noise data      */
/*  Note:  Many changes made for Windows version:                            */
/*		   Eliminated all Hammer and other user interface calls              */
/*		   These are now done via NoiseDlgProc()                             */
/*		   The noise data is put into the fPhaseData[] array                 */
/*                                                                           */
/*	Parameters:	F_TYPE	fSigma[] =	array of noise parameters                */
/*									index=alpha+2 (alpha -2 to +2)           */
/*				F_TYPE	fOffset	 =	frequency offset                         */
/*				F_TYPE	fDrift	 =	frequency drift per inverval             */
/*				F_TYPE	fTau	 =	tau value                                */
/*                                                                           */
/*	Return:		int		code		0=OK, -1=general error,                  */
/*								    -2=memory allocation error               */
/*                                                                           */
/*	Revision record:                                                         */
/*		03/29/96	Copied from STAB_N04.C                                   */
/*		06/12/99	Added return code to handle errors         		         */
/*		02/18/00	Fixed problem with W PM noise generation for tau!=1      */ 
/*		01/13/06	Fixed bug reported by Tom Parker/NIST re multiple noise  */
/*					type generation.  Seed did not change between noise      */
/*					types because it used time() with only 1 sec resolution. */
/*					Added index to assure new seed for each noise type.      */
/*                                                                           */
/*****************************************************************************/

int WINAPI GenerateNoise(F_TYPE fSigma[], F_TYPE fOffset,
	F_TYPE fDrift, F_TYPE fTau, int nNum)
{
	int i;                                                  /* general index */
	int num2;                /* # noise points rounded up to next power of 2 */
	int seed=1;                /* seed flag for peudo-random noise generator */
	int add=0;                                                   /* add flag */
	int code;			                        /* return code from b_beta() */
	float log2n;         /* log2(num) used to check that num is a power of 2 */
	double *x;                               /* pointer to phase noise array */
	float var;                                     /* variance of noise data */
	float beta;                            /* S(f) power law noise exponent */
	float t;                                                         /* time */
	float h[5];              /* array of noise coefficients vs index=alpha+2 */
	F_TYPE point;                                              /* data point */
	char szBuffer[BUFFER_SIZE+1];	                          /* text buffer */

	// calc power of 2 #
	num2=nNum;

	log2n=(float)(log(nNum)/log(2));       /* make sure nNum is a power of 2 */
	if(log2n!=(int)log2n)                /* if it is not, round it up to one */
	{
		num2=(int)pow(2, ((int)log2n)+1);
	}                                                              /* end if */

	// Allocate memory for noise data
	// Note that it is full power-of-2 size
	if((x=(double *)malloc((num2)*sizeof(double)))==NULL)
	{
		return(-2); // Error code
	}

	// Calc noise coefficients
	// alpha=-2 RW FM */
	h[0]=(float)(0.1520*fSigma[0]*fSigma[0]);

	// alpha=-1 F FM */
	h[1]=(float)(0.7213*fSigma[1]*fSigma[1]);

	// alpha=0 W FM */
	h[2]=(float)(2.0*fSigma[2]*fSigma[2]);

	// alpha=1 F PM */
	h[3]=(float)(8.828*fSigma[3]*fSigma[3]);

	//	alpha=2 W PM */
	h[4]=(float)(26.32*fSigma[4]*fSigma[4]);

	 // Generate noise
	for(i=0; i<5; i++)                          /* loop thru all noise types */
	{
		if(h[i])                                 /* want this type of noise? */
		{
			beta=(float)(i-2); // this is really alpha!
			var=h[i];

			// Generate new seed for random number generator 
			// Seed random number generator with current time
			// time() returns # seconds since 01/01/70
			// but doesnt have enough resolution to assure new seed for
			// multiple noise types, so also use index as part of
			// srand() argument
			srand((unsigned)(time(NULL)+i));
			
			seed=-rand();                           /* get new random # seed */

			var/=2.0;
			var/=(float)pow(6.283185308, beta);

			// New code 2/18/00
			// There seems to be a long standing error in the noise
			// generation alogrithm for the W PM case with tau !=1
			// It appears necessary to eliminate this tau-dependent factor
			// for the W PM case - after all, for W PM, the phase noise S/B
			// independent of tau.  But it is very strange that the
			// algorithm should need such an ugly patch.
			if(i!=4) // Exclude W PM case
			{
				var/=(float)pow(fTau, beta-1);
			}

			beta-=2;

			// Call noise generation function - was f_beta() until put into
			// FrequenC DLL V2.0 on 03/15/03

			code=GenNoise(num2, x, var, beta, &seed, add);

			// Test return code for error - Added 06/12/99
			if(code<0)
			{
				free(x);	  // Free memory
				return(code); // Return error code
			}

			add=1;                                   /* add=1 after 1st calc */
		}                                                          /* end if */
	}                                                             /* end for */

	// Write results to phase data array
	// Reallocate phase data array size required
	if((fNew=(F_TYPE*)realloc((char *)fPhaseData,
		(nNum+3)*sizeof(F_TYPE)))==NULL)
	{
		AllocErrorMessage();
		return -2;
	}
	else
	{
		fPhaseData=fNew;
	}

	fData=fPhaseData;

	// Set # phase data points & analysis limits
	fPhaseData[0]=nNum; // # points
	fPhaseData[1]=1;   // analysis start
	fPhaseData[2]=nNum; // analysis end

	// Put fTau value into phase status bar
	_snprintf(szBuffer, sizeof(szBuffer), "%10.5e", fTau);
	ShortenExp(szBuffer);
	SetWindowText(hwndPhaseTau, szBuffer);

	// Put NEW.DAT filename into phase status bar
	SetWindowText(hwndPhaseFilename, "NEW.DAT");

	// Set flags
	bDataType=0;
	bPhaseFlag=TRUE;
	bPhaseData=TRUE;
	bSynch=FALSE;

	// Copy noise to phase data array
	for(i=0; i<nNum; i++)
	{
		t=(float)i;                                         /* time interval */
		point=x[i]+(fOffset*fTau*t)+(fDrift*fTau*t*t/2);
								   /* add offset & drift to noise data point */
								    /* offset is fractional frequency offset */
				    /* drift is fractional frequency change per tau interval */
						  /* drift*tau is clock time change per tau interval */
	   /* which is multiplied by (t^2)/2 to get integrated clock time change */
		fPhaseData[i+3]=point;
	}                                                             /* end for */

	/* clean up and exit */
	free(x);
	return(0); // Normal return
}                                                             /* end noise() */

/*****************************************************************************/
#endif
