/*****************************************************************************/
/*                                                                           */
/*                            STABLE_U.C                                     */
/*                                                                           */
/* 		Stable Windows95 and above Utility Functions                         */
/*                                                                           */
/*		Function            	Description                                  */
/*                                                                           */
/*		InvokeDialogBox()		Wrapper Function to Invoke Dialog Box        */
/*		ClearDataArray()        Clear Phase or Frequency Data Array          */
/*      ClearTimetagArray()		Clear Phase or Frequency Timetag Array       */
/* 		CreateStatusBar()		Create Controls for Status Bar               */
/*		MoveStatusBarControls()	Move Status Bar Controls to Screen Position  */
/*      WriteProgressMessage()	Write Progress Message in Status Bar         */
/*      CalcSearchLimit()		Calculate Limits for Data Value Search       */
/*      DrawBitmap()            Draw Bitmap on a Display Surface             */
/*		DrawTransparentBitmap() Draw Tramsparent Bitmap on a Display Surface */
/*      WriteOutput()			Write Text on a Static Text Control          */
/*		ValidateInteger()		Validate integer input vs allowable limits   */
/*		ValidateFloat()			Validate float input vs allowable limits     */
/*		AllocErrorMessage()		Display memory allocation error message      */
/*		AllocErrorMessageEx()	Display verbose mem alloc error message      */
/*		TooFewMessage()         Display too few analysis points error msg    */
/*		ZeroRangeMessage()      Display zero range error msg                 */
/*		MustNormalizeMessage()  Display message to normalize data            */
/*		IncExtension()			Increment filename extension                 */
/*		CannotBeZero()          Display value cannot be zero error message   */
/*		YieldControl()			Allow Windows to process other messages      */
/*		ClearLines()            Clear all plot lines                         */
/*		ShortenExp()			Shorten 3-digit exp to 2 digits              */
/*      CheckPath()				Check that path exists, create it if not     */
/*      CheckFilename()			Check that filename is valid                 */
/*		DebugMessageBox()		Display debug message                        */
/*		WriteLog()				Write entry to log file                      */
/*		IsNumeric()				Check if a character is numeric              */
/*		Flash()					Flash scroll lock light                      */
/*		LinFit()				Calc weighted least squares linear fit       */
/*		SmoothPSD()				Smooth PSD data                              */
/*		SemiLogFit()			Fit data to a semilog line                   */
/*      strlcpy()				Safe version of strcpy()                     */
/*      strlcat()				Safe version of strcat()                     */
/*                                                                           */
/*		Revision record:                                                     */
/*			06/30/97	Replaced FAR PASCAL _export with WINAPI              */
/*						Replaced MoveTo() with MoveToEx()                    */
/*			07/12/97	Made numerous Win 32 compatibility changes           */
/*			07/26/07    Changed to safe string functions                     */
/*                                                                           */
/* (c) 1996-2007 W. Riley Hamilton Technical Services All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#define STRICT 					// For Win32 compatibility

#include "stable_w.h"           // STABLE_W header file

/*****************************************************************************/
/*                                                                           */
/*      InvokeDialogBox()                                                    */
/*                                                                           */
/*		Function to invoke dialog box                                        */
/*                                                                           */
/* 		Wrapper function to call to DialogBox()                              */
/* 		Creates and destroys instance thunk                                  */
/*                                                                           */
/* 		Globals:    None                                                                      */
/*                                                                           */
/* 		Parameters:	HANDLE	hInst			Application instance handle      */
/*      	        LPSTR	lpBoxName		Dialog box template name         */
/*                  HWND	hwnd			Parent window handle             */
/*                  DLGPROC	lpDlgProc		Pointer to dialog box procedure  */
/*                                                                           */
/*      Return:     int		nResult			Return value from DialogBox()	 */
/*                                                                           */
/*		Revision record:                                                     */
/*      	08/18/96	Changed type of 4th param from FARPROC to DLGPROC    */
/*                                                                           */
/*****************************************************************************/

int WINAPI InvokeDialogBox(HANDLE hInst, LPSTR lpBoxName,
	HWND hwnd, DLGPROC lpDlgProc)
{

	// Local variables
	int nResult;

	// Note: "Thunking" is not required for Win 3.1 or Win 32, so the simple
	// call to DialogBox() is OK

	nResult=DialogBox(hInst, lpBoxName, hwnd, lpDlgProc);

	return nResult;
}

/*****************************************************************************/
/*                                                                           */
/*      ClearDataArray()                                                     */
/*                                                                           */
/*		Function to clear phase or frequency data array                      */
/*                                                                           */
/* 		Data array is shrunk to MIN_ARRAY_SIZE+EXTRA_MEM                     */
/*                                                                           */
/* 		Globals:    HWND    hwndMain 			Main window handle           */
/*				    HWND    hwndFreqFilename    Handles to frequency         */
/*		  	        HWND    hwndFreqNum         and phase status bar         */
/*			        HWND    hwndFreqEnd         edit controls:               */
/*			        HWND    hwndFreqStart                                    */
/*			        HWMD    hwndFreqTau                                      */
/* 		            HWND    hwndPhaseFilename                                */
/*		  	        HWND    hwndPhaseNum                                     */
/*			        HWND    hwndPhaseEnd                                     */
/*			        HWND    hwndPhaseStart                                   */
/*			        HWMD    hwndPhaseTau                                     */
/*					HANDLE	hFreqData										 */
/*					HANDLE	hPhaseData										 */
/*                  F_TYPE  fFreqData  	    	Pointer to freq data array   */
/*                  F_TYPE  fPhaseData  	    Pointer to phase data array  */
/*                                                                           */
/* 		Parameters:	BOOL	bDatType   			Phase(0) or freq(1) array    */
/*                                                                           */
/*      Return:     int		     				Return status (-1 if error)  */
/*                                                                           */
/*		Revision record:                                                     */
/*      	01/04/97	Added more flags & globals                           */
/*                                                                           */
/*****************************************************************************/

int WINAPI ClearDataArray(BOOL bDatType)
{
	// No local variables

	if(bDatType)
	{
		// shrink freq data array
		if((fNew=(F_TYPE*)realloc((char *)fFreqData,
		(MIN_ARRAY_SIZE+ARRAY_OFFSET+EXTRA_MEM)*sizeof(F_TYPE)))==NULL)
		{
			// Shouldn't happen and not fatal anyway
			// so just announce error w/o bailing out
			AllocErrorMessage();
		}
		else
		{
			fFreqData=fNew;
		}

		// Initialize array size, start and end parameters
		fFreqData[0]=(F_TYPE)MIN_ARRAY_SIZE;
		fFreqData[1]=1.0;
		fFreqData[2]=(F_TYPE)MIN_ARRAY_SIZE;

		// Update status display
		SetWindowText(hwndFreqFilename, "            ");
		SetWindowText(hwndFreqNum, "");
		SetWindowText(hwndFreqEnd, "");
		SetWindowText(hwndFreqStart, "");
		SetWindowText(hwndFreqTau, "");
		bFreqData=FALSE;
		bOutRem=FALSE;
		bFreqGapFill=FALSE;
		bFreqNorm=FALSE;
		bLinDriftRem=FALSE;
		bLogDriftRem=FALSE;
		bDiffDriftRem=FALSE;
		bFreqAddend=FALSE;			
		bFreqScaled=FALSE;			
		nFreqGapsFilled=FALSE;
		nOutliersRemoved=FALSE;
	}
	else
	{
		// shrink phase data array
		if((fNew=(F_TYPE*)realloc((char *)fPhaseData,
			(MIN_ARRAY_SIZE+ARRAY_OFFSET+EXTRA_MEM)*sizeof(F_TYPE)))==NULL)
		{
			// Shouldn't happen and not fatal anyway
			// so just announce error w/o bailing out
			AllocErrorMessage();
		}
		else
		{
			fPhaseData=fNew;
		}

		// Initialize array size, start and end parameters
		fPhaseData[0]=(F_TYPE)MIN_ARRAY_SIZE;
		fPhaseData[1]=1.0;
		fPhaseData[2]=(F_TYPE)MIN_ARRAY_SIZE;

		// Update status display
		SetWindowText(hwndPhaseFilename, "            ");
		SetWindowText(hwndPhaseNum, "");
		SetWindowText(hwndPhaseEnd, "");
		SetWindowText(hwndPhaseStart, "");
		SetWindowText(hwndPhaseTau, "");
		bPhaseData=FALSE;
		bPhaseGapFill=FALSE;
		bPhaseNorm=FALSE;
		bQuadDriftRem=FALSE;
		nPhaseGapsFilled=FALSE;
		bPhaseAddend=FALSE;			
		bPhaseScaled=FALSE;		
	}

	// Deactivate data-dependent functions
	ActivateFunctions(MF_GRAYED);

	return 0;                   // OK
}

/*****************************************************************************/
/*                                                                           */
/*      ClearTimetagArray()                                                  */
/*                                                                           */
/*		Function to clear phase or frequency timetag array                   */
/*                                                                           */
/* 		Timetag array is shrunk to MIN_ARRAY_SIZE+EXTRA_MEM                  */
/*                                                                           */
/* 		Globals:    HWND    hwndMain 			Main window handle           */
/*                  F_TYPE  fFreqTimetags      	Ptr to freq timetag array    */
/*                  F_TYPE  fPhaseTimetags 	    Ptr to phase timetag array   */
/*                                                                           */
/* 		Parameters:	BOOL	bDatType   			Phase(0) or freq(1) array    */
/*                                                                           */
/*      Return:     int		     				Return status (-1 if error)  */
/*                                                                           */
/*****************************************************************************/

int WINAPI ClearTimetagArray(BOOL bDatType)
{
	// No local variables

	if(bDatType==FREQ_DATA)
	{
		// shrink freq timetag array
		if((fNew=(F_TYPE*)realloc((char *)fFreqTimetags,
		(MIN_ARRAY_SIZE+ARRAY_OFFSET+EXTRA_MEM)*sizeof(F_TYPE)))==NULL)
		{
			// Shouldn't happen and not fatal anyway
			// so just announce error w/o bailing out
			AllocErrorMessage();
		}
		else
		{
			fFreqTimetags=fNew;
		}

		// Initialize array size, start and end parameters
		fFreqTimetags[0]=0.0;
		fFreqTimetags[1]=0.0;
		fFreqTimetags[2]=0.0;

		// Clear freq timetags exist flag
		bFreqTimetags=FALSE;
	}
	else
	{
		// shrink phase timetag array
		if((fNew=(F_TYPE*)realloc((char *)fPhaseTimetags,
			(MIN_ARRAY_SIZE+ARRAY_OFFSET+EXTRA_MEM)*sizeof(F_TYPE)))==NULL)
		{
			// Shouldn't happen and not fatal anyway
			// so just announce error w/o bailing out
			AllocErrorMessage();
		}
		else
		{
			fPhaseTimetags=fNew;
		}

		// Initialize array size, start and end parameters
		fPhaseTimetags[0]=0.0;
		fPhaseTimetags[1]=0.0;
		fPhaseTimetags[2]=0.0;

		// Clear phase timetags exist flag
		bPhaseTimetags=FALSE;
	}

	// Deactivate regularize function
	ActivateFunctions(MF_ENABLED);

	return 0;                   // OK
}

/*****************************************************************************/
/*                                                                           */
/*             Function to calculate search test limit                       */
/*                                                                           */
/* 	This function accepts an input string and calculates a corresponding     */
/*  search test limit that, together with the value of the input string,     */
/*  provides a range to search over as determined by the precision of the    */
/*  input value.  For example, an input value of -1.23e-15 implies a search  */
/*  range of -1.24e-15 < x <= -1.23e-15.                                     */
/*                                                                           */
/*****************************************************************************/
F_TYPE WINAPI CalcSearchLimit(LPSTR szArg)
{
	#include <ctype.h>      // For isdigit()

	// Local variables
	F_TYPE fArg;			// Input value
	F_TYPE fExp;			// Exponent
	F_TYPE fMan;			// Mantissa
	F_TYPE fSign=1.0; 		// Sign
	int nLen;				// Length of string
	int nDig=0;             // # digits of precision
	int i=0;   				// Index
	BOOL bLZFlag=TRUE;      // Leading zero flag

	// Convert input string to a numeric value
	fArg=atof(szArg);

	// Zero is a special case - returns zero
	if(fArg==0)
	{
		return 0.0;
	}

	// Save sign
	if(fArg<0)
	{
		fSign=-1.0;
	}

	// Get absolute value
	fArg=fabs(fArg);

	// Identify the precision wanted
	// Count the # of digits of precision in the input string

	// strip off sign, if any, replacing it with a leading zero
	if(!isdigit(szArg[0]))
	{
		szArg[0]='0';
	}

	// Find length of string
	nLen=strlen(szArg);

	// Scan the string up to the decimal point, or, if none, to its end,
	// counting the # of digits, not including leading zeros
	while((isdigit(szArg[i])) && (i<nLen))
	{
		if((szArg[i]!='0') || (!bLZFlag))
		{
			nDig++;
			bLZFlag=FALSE;
		}
		i++;
	}

	// Now index points to decimal point or end
	// If not at dp, we are done
	// If at dp, we have to continue our count
	if(szArg[i]=='.')
	{
		// Move to 1st digit after dp (if any)
		i++;

		// Count how many digits there are directly after it,
		// stopping at exponent (if any)
		while(isdigit(szArg[i]))
		{
			i++;
			nDig++;
		}
	}

	// nDig now gives # of significant digits
	// Get mantissa and exponent
	fMan=log10(fArg);
	fMan=modf(fMan, &fExp);

	// Normalize the mantissa to a decimal fraction (eg 0.123e-15)
	if(fMan>0)
	{
		fMan-=1.0;
		fExp+=1.0;
	}
	fMan=pow(10.0, fMan);		// fMan is mantissa, fExp is exponent

	// Form the search limit by adding 1 in proper digit
	fMan+=pow(10.0, -nDig);

	// Return reconstructed number
	return fSign*fMan*pow(10.0, fExp);
}

/*****************************************************************************/
/*                                                                           */
/*                           DrawBitmap()                                    */
/*                                                                           */
/* 	Function to draw a bitmap on a display surface.                          */
/*                                                                           */
/*  See Petzold, "Programming Windows 3.1", p.631                            */
/*                                                                           */
/*****************************************************************************/

void WINAPI DrawBitmap(HDC hdc, HBITMAP hBitmap, short xStart,
	short yStart)
{
	BITMAP bm;
	HDC hdcMem;
	POINT ptSize, ptOrg;

	hdcMem=CreateCompatibleDC(hdc);
	SelectObject(hdcMem, hBitmap);
	SetMapMode(hdcMem, GetMapMode(hdc));

	GetObject(hBitmap, sizeof(BITMAP), (LPSTR)&bm);
	ptSize.x=bm.bmWidth;
	ptSize.y=bm.bmHeight;
	DPtoLP(hdc, &ptSize, 1);

	ptOrg.x=0;
	ptOrg.y=0;
	DPtoLP(hdcMem, &ptOrg, 1);

	BitBlt(hdc, xStart, yStart, ptSize.x, ptSize.y,
		hdcMem, ptOrg.x, ptOrg.y, SRCCOPY);

	DeleteDC(hdcMem);
}

/*****************************************************************************/
/*                                                                           */
/*                        DrawTransparentBitmap()                            */
/*                                                                           */
/*  Function to draw a transparent bitmap on a display surface               */
/*  given the source color bitmap with transparent areas black and a         */
/*  monochrome mask with transparent areas black and opaque areas white.     */
/*                                                                           */
/*  References:                                                              */
/*  1. C. Petzold, "Programming Windows 3.1", p.631                          */
/*                                                                           */
/*  2. R. Mashlan,  http://www.r2m.com/win-developer-FAQ/graphics/7.html     */
/*                                                                           */
/*  Q: How do I draw a transparent bitmap, that is, a bitmap with            */
/*     see-through areas?                                                    */
/*  A: You can use a technique known as the black source method. This        */
/*     technique involves using a  bitmap with a single color dedicated for  */
/*     marking the transparent areas. First, a monochrome  mask bitmap must  */
/*     be generated from the original bitmap, and the original bitmap has    */
/*     the transparent color turned to black. The mask bitmap contains black */
/*     for the transparent areas, and white for the opaque areas.  Once you  */
/*     have generated the mask, and modified the source bitmap, you can      */
/*     display the transparent by using a SRCAND BitBlt() from the mask      */
/*     bitmap to the destination, then using a  SRCPAINT BitBlt() of the     */
/*     source bitmap to the destination bitmap.                              */
/*                                                                           */
/*     Revision record:                                                      */
/*         05/18/00    Created from DrawBitmap() and references 1 & 2        */
/*					   Made a few corrections to get it working OK           */
/*                                                                           */
/*****************************************************************************/

void DrawTransparentBitmap(HDC hdc, HBITMAP hBmpSrc, HBITMAP hBmpMsk,
    short xStart, short yStart)
{
    // Local variables
    BITMAP bm;
    HDC hdcSrc;
    HDC hdcMsk;
    COLORREF tc;
    COLORREF bc;
    POINT ptSize;
	POINT ptOrg;

    // Initializations
    hdcSrc=CreateCompatibleDC(hdc);
	hdcMsk=CreateCompatibleDC(hdc);
	SelectObject(hdcSrc, hBmpSrc);
    SelectObject(hdcMsk, hBmpMsk);
    tc=SetTextColor(hdc, RGB(255, 255, 255));
    bc=SetBkColor(hdc, RGB(0, 0, 0));
    
	// Set source coordinates
	ptOrg.x=0;
    ptOrg.y=0;
    DPtoLP(hdcSrc, &ptOrg, 1);

    // Set bitmap size
	GetObject(hBmpSrc, sizeof(BITMAP), (LPSTR)&bm);
    ptSize.x=bm.bmWidth;
    ptSize.y=bm.bmHeight;
    DPtoLP(hdcSrc, &ptSize, 1);

    // Paint the mask
	BitBlt(hdc, xStart, yStart, ptSize.x, ptSize.y, hdcMsk, ptOrg.x, ptOrg.y,
		SRCAND);
   
    // Paint the source
    BitBlt(hdc, xStart, yStart, ptSize.x, ptSize.y, hdcSrc, ptOrg.x, ptOrg.y,
		SRCPAINT);
   
    // Restore text color and background color
    SetTextColor(hdc, tc);
    SetBkColor(hdc, bc);

	// Clean up
    DeleteDC(hdcSrc);
    DeleteDC(hdcMsk);
}

/*****************************************************************************/
/*                                                                           */
/*							ValidateInteger()                                */
/*                                                                           */
/*		Function to validate integer input versus allowable limits.          */
/*                                                                           */
/*      Parameters:                                                          */
/*      	HWND	hwnd		Handle to window calling function            */
/*          int		nVal		Integer value to be checked                  */
/*          int		nMin 		Minimum allowable value                      */
/*			int		nMax  	    Maximum allowable value                      */
/*                                                                           */
/*      Return:                                                              */
/*			BOOL	status		TRUE=Value OK, FALSE=Value NG                */
/*                                                                           */
/*      Note 1:	Calling function S/B able to restore variable to its         */
/*              previous or default value before the bad entry was made.     */
/*      Note 2:	Call in response to EN_KILLFOCUS message when using with     */
/*              an edit control.                                             */
/*      Note 3:	Don't call in response to pressing the Cancel button.        */
/*                                                                           */
/*****************************************************************************/
BOOL WINAPI ValidateInteger(HWND hwnd, int nVal, int nMin,
	int nMax)
{
	// Local variables
	char szBuffer[BUFFER_SIZE+1];			// Text buffer
	char szFormat[BUFFER_SIZE+1];	  	    // Format string

	if(nVal<nMin || nVal>nMax)
	{
		MessageBeep(0);
		LoadString(hInst, IDS_FORMAT_18, szFormat,	BUFFER_SIZE);
		_snprintf(szBuffer, sizeof(szBuffer), szFormat, nVal, nMin, nMax);
		MessageBox(hwnd, szBuffer, "Invalid Entry", MB_ICONEXCLAMATION);
		return FALSE;
	}
	else
	{
		return TRUE;
	}
}

/*****************************************************************************/
/*                                                                           */
/*							ValidateFloat()                                  */
/*                                                                           */
/*		Function to validate float input versus allowable limits.            */
/*                                                                           */
/*      Parameters:                                                          */
/*      	HWND	hwnd		Handle to window calling function            */
/*          F_TYPE	fVal		F_TYPE value to be checked                   */
/*          int		nMin 		Minimum allowable value                      */
/*			int		nMax  	    Maximum allowable value                      */
/*                                                                           */
/*      Return:                                                              */
/*			BOOL	status		TRUE=Value OK, FALSE=Value NG                */
/*                                                                           */
/*      Note 1:	Calling function S/B able to restore variable to its         */
/*              previous or default value before the bad entry was made.     */
/*      Note 2:	Call in response to EN_KILLFOCUS message when using with     */
/*              an edit control.                                             */
/*      Note 3:	Don't call in response to pressing the Cancel button.        */
/*                                                                           */
/*     Revision record:                                                      */
/*         07/17/08    Added ShortenExp()                                    */
/*                                                                           */
/*****************************************************************************/
BOOL WINAPI ValidateFloat(HWND hwnd, F_TYPE fVal, F_TYPE fMin,
	F_TYPE fMax)
{
	// Local variables
	char szBuffer[BUFFER_SIZE+1];			// Text buffer
	char szFormat[BUFFER_SIZE+1];	  	    // Format string
	F_TYPE fEps=1e-5;                       // Numerical margin factor

	if( ((fVal+fVal*fEps)<fMin) || ((fVal-fVal*fEps)>fMax) )
	{
		MessageBeep(0);
		LoadString(hInst, IDS_FORMAT_19, szFormat,	BUFFER_SIZE);
		_snprintf(szBuffer, sizeof(szBuffer), szFormat, fVal, fMin, fMax);
		ShortenExp(szBuffer);
		MessageBox(hwnd, szBuffer, "Invalid Entry", MB_ICONEXCLAMATION);
		return FALSE;
	}
	else
	{
		return TRUE;
	}
}
/*****************************************************************************/
/*                                                                           */
/*							ValidateMJD()                                    */
/*                                                                           */
/*		Function to validate MJD input versus allowable limits.              */
/*                                                                           */
/*      Parameters:                                                          */
/*      	HWND	hwnd		Handle to window calling function            */
/*          F_TYPE	fVal		F_TYPE value to be checked                   */
/*          int		nMin 		Minimum allowable value                      */
/*			int		nMax  	    Maximum allowable value                      */
/*                                                                           */
/*      Return:                                                              */
/*			BOOL	status		TRUE=Value OK, FALSE=Value NG                */
/*                                                                           */
/*      Note 1:	Calling function S/B able to restore variable to its         */
/*              previous or default value before the bad entry was made.     */
/*      Note 2:	Call in response to EN_KILLFOCUS message when using with     */
/*              an edit control.                                             */
/*      Note 3:	Don't call in response to pressing the Cancel button.        */
/*      Note 4:	Same as Validate Float() except for display format.          */
/*                                                                           */
/*****************************************************************************/
BOOL WINAPI ValidateMJD(HWND hwnd, F_TYPE fVal, F_TYPE fMin,
	F_TYPE fMax)
{
	// Local variables
	char szBuffer[BUFFER_SIZE+1];			// Text buffer
	char szFormat[BUFFER_SIZE+1];	  	    // Format string
	F_TYPE fEps=1e-5;                       // Numerical margin factor

	if( ((fVal+fVal*fEps)<fMin) || ((fVal-fVal*fEps)>fMax) )
	{
		MessageBeep(0);
		LoadString(hInst, IDS_FORMAT_22, szFormat,	BUFFER_SIZE);
		_snprintf(szBuffer, sizeof(szBuffer), szFormat, fVal, fMin, fMax);
		MessageBox(hwnd, szBuffer, "Invalid Entry", MB_ICONEXCLAMATION);
		return FALSE;
	}
	else
	{
		return TRUE;
	}
}
/*****************************************************************************/
/*                                                                           */
/*      Function to display memory allocation error message                  */
/*                                                                           */
/*                                                                           */
/*****************************************************************************/

void WINAPI AllocErrorMessage(void)
{
	// Local variables
	char szBuffer[BUFFER_SIZE+1];				// Text buffer

	LoadString(hInst, IDS_MESSAGE_MEMERR, szBuffer, BUFFER_SIZE);
	MessageBox(hwndMain, szBuffer, "Stable32", MB_ICONEXCLAMATION);
}

#ifdef MEM_DEBUG
/*****************************************************************************/
/*                                                                           */
/*      Function to display extended memory allocation error message         */
/*                                                                           */
/*      Parameters:                                                          */
/*      	char	*szFile		Pointer to source filename string            */
/*								Call with __FILE__                           */
/*          int		nLine       Line # in source file                        */
/*								Call with __LINE__                           */
/*          int		nSize       # memory bytes requested                     */
/*                                                                           */
/*      Return:                                                              */
/*			void	(none)                                                   */
/*                                                                           */
/*		Revision record:                                                     */
/*      	06/19/99	Created                                              */
/*                                                                           */
/*****************************************************************************/

void WINAPI AllocErrorMessageEx(char *szFile, int nLine, int nSize)
{
	// Local variables
	char szText[BIG_BUFFER_SIZE+1];			// Text buffer
	char szInfo[BUFFER_SIZE+1];				// File & line buffer
	MEMORYSTATUS ms;						// Memory status struct

	LoadString(hInst, IDS_MESSAGE_MEMERR, szText, BUFFER_SIZE);
	_snprintf(szInfo, sizeof(szInfo), "\nFile: %s, Line: %d", szFile, nLine);
	strlcat(szText, szInfo, sizeof(szText));
	_snprintf(szInfo, sizeof(szInfo), "\nMemory Requested=%d", nSize);
	strlcat(szText, szInfo, sizeof(szText));
	ms.dwLength=sizeof(MEMORYSTATUS);
	GlobalMemoryStatus(&ms);
	_snprintf(szInfo, sizeof(szInfo), "\nAvailable Physical Memory=%ld",
		ms.dwAvailPhys);
	strlcat(szText, szInfo, sizeof(szText));
	_snprintf(szInfo, sizeof(szInfo), "\nAvailable Page File=%ld",
		ms.dwAvailPageFile);
	strlcat(szText, szInfo, sizeof(szText));
	_snprintf(szInfo, sizeof(szInfo), "\nAvailable Virtual Memory=%ld",
		ms.dwAvailVirtual);
	strlcat(szText, szInfo, sizeof(szText));
	MessageBox(hwndMain, szText, "Stable32", MB_ICONEXCLAMATION);
}
#endif

/*****************************************************************************/
/*                                                                           */
/*       Function to display too few analysis points error message           */
/*                                                                           */
/*                                                                           */
/*****************************************************************************/

void WINAPI TooFewMessage(void)
{
	// Local variables
	char szBuffer[BUFFER_SIZE+1];			// Text buffer

	LoadString(hInst, IDS_MESSAGE_TOOFEW, szBuffer, BUFFER_SIZE);
	MessageBox(hwndMain, szBuffer, "Stable32", MB_ICONEXCLAMATION);
}

/*****************************************************************************/
/*                                                                           */
/*       Function to display zero range error message                        */
/*                                                                           */
/*                                                                           */
/*****************************************************************************/

void WINAPI ZeroRangeMessage(void)
{
	// Local variables
	char szBuffer[BUFFER_SIZE+1];			// Text buffer

	LoadString(hInst, IDS_MESSAGE_ZERORANGE, szBuffer, BUFFER_SIZE);
	MessageBox(hwndMain, szBuffer, "Stable32", MB_ICONEXCLAMATION);
}

/*****************************************************************************/
/*                                                                           */
/*       Function to display must normalize message                          */
/*                                                                           */
/*                                                                           */
/*****************************************************************************/

void WINAPI MustNormalizeMessage(void)
{
	// Local variables
	char szBuffer[BUFFER_SIZE+1];			// Text buffer

	LoadString(hInst, IDS_MESSAGE_MUSTNORM, szBuffer, BUFFER_SIZE);
	MessageBox(hwndMain, szBuffer, "Stable32", MB_ICONEXCLAMATION);
}

/*****************************************************************************/
/*                                                                           */
/*				               IncExtension()                                */
/*                                                                           */
/* 		Function to increment numerical filename extension                   */
/*		Parameters:		char *filename	   pointer to filename               */
/*										   whose numerical extension         */
/*										   is to be incremented, or,         */
/*                                         if non-numeric, made *.00#        */
/*						int type		   flag for type of file             */
/*										   0=phase, 1=frequency, and         */
/*										  -1=decrement extension             */
/*										  -2=make extension *.DAT            */
/*										  -3=make extension other datype     */
/*      Returns:		new extension #                                      */
/*                                                                           */
/*		Example:		IncExtension(szFile, FREQ_DATA);		             */
/*                                                                           */
/*		Revision record:                                                     */
/*			11/03/96	Adapted from Stable/DOS STAB_F23.C                   */
/*			11/10/96	Added type -2 & -3 options                           */
/*			06/03/97	Added clearing of plot lines                         */
/*          04/13/98	Revised to handle long filenames                     */
/*                                                                           */
/*****************************************************************************/

int WINAPI IncExtension( char *filename, int type)
{
	// Local variables
	int i;
	// Chars
	char drive[BUFFER_SIZE];
	char directory[FILENAMESIZE];
	char name[FILE_NAME_SIZE];
	char ext[BUFFER_SIZE];
	// Ints
	int ext_num;

	// Guard against blank filename
	if(strlen(filename)==0)
	{
		strlcpy(filename, "NONAME.DAT", FILE_NAME_SIZE);
	}

	// Split filename into its parts
	_splitpath(filename, drive, directory, name, ext);

	// Remove leading dot from ext (so it is an integer)
	for(i=0; i<(int)strlen(ext); i++)
	{
		// Move up all chars inc EOS
		ext[i]=ext[i+1]; 
	}

	ext_num = atoi(ext);                           /* convert ext to integer */
								   /* result will be 0 if ext is non-numeric */
	if((!ext_num) && (type==1))             /* if ext_num is 0 and type is 1 */
	{                                    /* then make it -1 so it becomes +1 */
		ext_num=-1;
	}
	if(type == -1)
	{
		ext_num--;                             /* decrement extension # by 1 */
	}
	else
	{
		ext_num+=2;                                      /* add 2 to ext_num */
	}

	// Subtract 1 from extension if type is -3
	// This changes extension to other data type
	if((type==-3) && ((ext_num>2) || (bDataType==PHASE_DATA)))
	{
		ext_num--;
	}

	// Convert it back to a 3-digit string with leading 0s as needed
	_snprintf(ext, BUFFER_SIZE, "%03d", ext_num); 
												

	// Make extension *.DAT if type is -2
	if(type == -2)
	{
		strlcpy(ext, "DAT", BUFFER_SIZE);
	}

	// Reassemble filename from its parts
	// Note: ext does not need to have a leading dot
	_makepath(filename, drive, directory, name, ext);

	return(ext_num);                           /* return the new extension # */
}

/*****************************************************************************/
/*                                                                           */
/*							CannotBeZero()                                   */
/*                                                                           */
/*		Function to display value cannot be zero error message.              */
/*                                                                           */
/*      Parameters:                                                          */
/*      	HWND	hwnd		Handle to window calling function            */
/*                                                                           */
/*      Return:                                                              */
/*			None	(void)                                                   */
/*                                                                           */
/*		Revision record:                                                     */
/*			11/09/96	Created                                              */
/*                                                                           */
/*****************************************************************************/
void WINAPI CannotBeZero(HWND hwnd)
{
	// Local variables
	char szBuffer[BUFFER_SIZE+1];				// Text buffer

	MessageBeep(0);
	LoadString(hInst, IDS_MESSAGE_NOZERO, szBuffer, BUFFER_SIZE);
	MessageBox(hwnd, szBuffer, "Invalid Entry", MB_ICONEXCLAMATION);
}

/*****************************************************************************/
/*                                                                           */
/*							YieldControl()                                   */
/*                                                                           */
/*		Function to allow Windows to take control & service other messages   */
/*                                                                           */
/*      Parameters:                                                          */
/*      	None 	(void)                                                   */
/*                                                                           */
/*      Return:                                                              */
/*			None	(void)                                                   */
/*                                                                           */
/*      Globals:                                                             */
/*			HWND hDialog	Handle of dialog box receiving messages          */
/*                                                                           */
/*		Revision record:                                                     */
/*			11/22/96	Created from GPCWINYieldControl()                    */
/*			10/05/98	Revised for better keyboard response                 */
/*			07/27/02	Added hDialog test                                   */
/*                                                                           */
/*****************************************************************************/
void WINAPI YieldControl(void)
{
	// Local variables
	MSG msg;
	
	if(hDialog) // Do only if valid dialog box handle
	{
		// Give Stop button the focus
		SetFocus(GetDlgItem(hDialog, IDCANCEL));
	}

	while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
	{
		if(!IsDialogMessage(hDialog, &msg))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
}

/*****************************************************************************/
/*                                                                           */
/*							ClearLines()                                     */
/*                                                                           */
/*		Function to clear all plot lines                                     */
/*      Clears phase or freq data and sigma plot lines                       */
/*      Called whenever data changes                                         */
/*                                                                           */
/*      Parameters:                                                          */
/*      	None 	(void)                                                   */
/*                                                                           */
/*      Return:                                                              */
/*			None	(void)                                                   */
/*                                                                           */
/*		Revision record:                                                     */
/*			06/04/97	Created                                              */
/*                                                                           */
/*****************************************************************************/
void WINAPI ClearLines(void)
{
	// Local variables
    int i;	// Index

	// Eliminate all sigma plot lines
	for(i=0; i<SLINE_NUMLINES; i++)
	{
		sSLine[i].nLineType=SLINE_NONE;
	}

	if(bDataType==PHASE_DATA)
	{
		// Eliminate all phase plot lines
		for(i=0; i<PLINE_NUMLINES; i++)
		{
			sPLine[i].nLineType=PLINE_NONE;
		}
	}

	if(bDataType==FREQ_DATA)
	{
		// Eliminate all freq plot lines
		for(i=0; i<FLINE_NUMLINES; i++)
		{
			sFLine[i].nLineType=FLINE_NONE;
		}
	}
}

/*****************************************************************************/
/*                                                                           */
/*							ShortenExp()                                     */
/*                                                                           */
/*		Function to shorten all 3-digit exponents in a string                */
/*      Use with Microsoft Visual C++ 5.0 for legacy display layouts         */
/*      Changes all numeric value formats of passed numeric value string     */
/*		from 3 digits to 2 digits                                            */
/*                                                                           */
/*      Parameters:                                                          */
/*      	szString		Numeric value string                             */
/*                                                                           */
/*      Return:                                                              */
/*			None	(void)                                                   */
/*                                                                           */
/*		Revision record:                                                     */
/*			01/16/98	Created                                              */
/*                                                                           */
/*****************************************************************************/
void WINAPI ShortenExp( char *szString)
{
	// Local variables
    int i;		// Main index
	int j;		// Secondary index
	int len;	// String length

	// Find string length
	len=strlen(szString);
	
	#ifdef _MSC_VER // Do only for Microsoft VC++
	// Scan for next 'e' in the string followed by a + or - and then a zero
	// Looking for pattern e+0dd or e-0dd
	// Scan stops 4 chars before end of string
	for(i=0; i<len-4; i++)
	{
		if( (szString[i]=='e') && (szString[i+1]=='+' || szString[i+1]=='-') &&
			(szString[i+2]=='0') )
		{
			// Original format is e+0dd or e-0dd
			// Want to eliminate the leading exp digit (always zero)
			// Modified format will be e+dd or e-dd
			// Shift up the string chars (and EOS) after the unwanted 0
			for(j=i+2; j<len; j++)
			{
				szString[j]=szString[j+1];
			}
			len--;
		}
	}
	#endif
}

#ifndef LITE_VER
/*****************************************************************************/
/*                                                                           */
/*							      CheckPath()                                */
/*                                                                           */
/*		Function to check whether string is valid path (drive + directory)   */
/*                                                                           */
/*      Accept szPath.  Add trailing backslash if necessary.  Test if        */
/*      a valid path.  If not, display message box in hWnd offering to       */
/*      create it (using label passed by szLabel).  Create new path if       */
/*		requested.  Return TRUE for OK path (existing or new) or FALSE if    */
/*      path NG.                                                             */
/*                                                                           */
/*      Parameters:                                                          */
/*      	szPath		Path string                                          */
/*      	hWnd		Parent window handle                                 */
/*      	szLabel		Label string for message box title                                        */
/*                                                                           */
/*      Return:                                                              */
/*			BOOL		Path status                                          */
/*                                                                           */
/*		Revision record:                                                     */
/*			05/09/98	Created                                              */
/*                                                                           */
/*****************************************************************************/
BOOL WINAPI CheckPath(char *szPath, HWND hWnd, char *szLabel)
{
	// Local variables
	char szBuffer[BUFFER_SIZE+1];		// General buffer
	char szCWD[FILENAMESIZE+1];			// Current working directory
	char szDrive[BUFFER_SIZE+1];		// Drive buffer
	char szDirectory[FILENAMESIZE+1];	// Directory buffer
	char szName[FILENAMESIZE+1];		// Filename buffer
	char szExt[BUFFER_SIZE+1];			// Extension buffer
	BOOL bPath;							// Path status flag

	// Split path into its parts
	_splitpath(szPath, szDrive, szDirectory, szName, szExt);

	// Make sure directory starts with a backslash
	if(szDirectory[0]!='\\')
	{
		strlcpy(szBuffer, "\\", sizeof(szBuffer));
		strlcat(szBuffer, szDirectory, sizeof(szBuffer));
		strlcpy(szDirectory, szBuffer, sizeof(szDirectory));
	}

	// Reassemble path in standard format
	_makepath(szPath, szDrive, szDirectory, szName, szExt);

	// Make sure path ends in backslash
	if(szPath[strlen(szPath)-1]!='\\')
	{
		strlcat(szPath, "\\", FILE_NAME_SIZE);
	}

	// Save current working directory
	_getcwd(szCWD, FILENAMESIZE);

	// Check if path exists
	if(_chdir(szPath)!=0)
	{
		// Display error message
		LoadString(hInst, IDS_MESSAGE_NOSUCHPATH, szBuffer,	BUFFER_SIZE);

		// Respond to user's yes/no choice
		if(MessageBox(hWnd, szBuffer, szLabel, MB_YESNO | MB_ICONQUESTION)==
			IDYES)
		{
			// Create new path
			_mkdir(szPath);

			// Path now exists
			bPath=TRUE;
		}
		else // Don't create new directory
		{
			// Path doesn't exist
			bPath=FALSE;
		}
	}
	else // Path already exists
	{
		bPath=TRUE;
	}

	// Restore CWD
	_chdir(szCWD);

	// Return path status
	return bPath;
}
#endif

/*****************************************************************************/
/*                                                                           */
/*							   CheckFilename()                               */
/*                                                                           */
/*		Function to check whether string is valid filename                   */
/*                                                                           */
/*      Accept szFile.  Test if a valid filename (with or w/o path).         */
/*      If (optional) path NG, display message box in hWnd offering to       */
/*      create it (using label passed by szLabel).  Create new path if       */
/*		requested.  Return TRUE for OK filename (existing or new) or FALSE   */
/*      if filename NG.  Also return new path, name & ext.                   */
/*                                                                           */
/*      Parameters:                                                          */
/*      	szFile		File name string                                     */
/*      	szTitle		File title string                                    */
/*      	szPath		File path string                                     */
/*      	hWnd		Parent window handle                                 */
/*      	szLabel		Label string for message box title                   */
/*                                                                           */
/*      Return:                                                              */
/*			BOOL		Path status                                          */
/*                                                                           */
/*		Revision record:                                                     */
/*			05/09/98	Created                                              */
/*                                                                           */
/*****************************************************************************/
BOOL WINAPI CheckFilename(char *szFile, char *szTitle, char *szPath, 
	char *szName, char *szExt, HWND hWnd, char *szLabel)
{
	// Local variables
	char szBuffer[BUFFER_SIZE+1];		// General buffer
	char szCWD[FILENAMESIZE+1];			// Current working directory
	char szDrive[BUFFER_SIZE+1];		// Drive buffer
	char szDirectory[FILENAMESIZE+1];	// Directory buffer
	BOOL bFile=TRUE;					// Filename status flag

	// Split filename into its parts
	_splitpath(szFile, szDrive, szDirectory, szName, szExt);

	
	// If drive is not NULL, make sure directory starts with a backslash
	if(strlen(szDrive))
	{
		if(szDirectory[0]!='\\')
		{
			strlcpy(szBuffer, "\\", sizeof(szBuffer));
			strlcat(szBuffer, szDirectory, sizeof(szBuffer));
			strlcpy(szDirectory, szBuffer, sizeof(szDirectory));
		}
	}

	// Reassemble full filename in standard format
	_makepath(szFile, szDrive, szDirectory, szName, szExt);

	// Reassemble path
	strlcpy(szPath, szDrive, FILE_NAME_SIZE);
	strlcat(szPath, szDirectory, FILE_NAME_SIZE);

	// Reassemble file title
	strlcpy(szTitle, szName, FILE_TITLE_SIZE);
	strlcat(szTitle, szExt, FILE_TITLE_SIZE);

	// Check if szTitle is a valid file title (name + ext)
	// A filename cannot be blank
	// Windows 95/NT filenames can contain any characters except:
	// ? " / \ < > * | :
	// Note that in a string constant the " and \ must be denoted
	// by their excape codes \" and \\ respectively

	if( (strlen(szTitle)==0) ||
		(strcspn(szTitle, "?\"/\\<>*|:") < strlen(szTitle)) )
	{
		// Display error message
		MessageBox(hWnd, "Invalid Filename", szLabel, MB_ICONEXCLAMATION);

		// Return FALSE if couldn't open file
		return FALSE;
	}

	// Check path if not null (backslash only)
	if(strlen(szPath)>1)
	{
		// Save current working directory
		_getcwd(szCWD, FILENAMESIZE);

		// Check if path exists
		if(_chdir(szPath)!=0)
		{
			// Display error message
			LoadString(hInst, IDS_MESSAGE_NOSUCHPATH, szBuffer,	BUFFER_SIZE);

			// Respond to user's yes/no choice
			if(MessageBox(hWnd, szBuffer, szLabel,
				MB_YESNO | MB_ICONQUESTION)==IDYES)
			{
				// Create new path
				_mkdir(szPath);

				// Path now exists
				bFile=TRUE;
			}
			else // Don't create new directory
			{
				// Path doesn't exist
				bFile=FALSE;
			}
		}
		else // Path already exists
		{
			bFile=TRUE;
		}

		// Restore CWD
		_chdir(szCWD);
	}

	// Return filename status
	return bFile;
}

#ifndef LITE_VER
/*****************************************************************************/
/*                                                                           */
/*						    DebugMessageBox()                                */
/*                                                                           */
/*		Function to display debug message                                    */
/*                                                                           */
/*      Parameters:                                                          */
/*          HWND	hWnd		Display window handle                        */
/*      	char *	szFile		File string (__FILE__)                       */
/*      	int		nLine		Line # (__LINE__)                            */
/*      	char *	szText		Message string                               */
/*                                                                           */
/*      Return:                                                              */
/*			None (void)                                                      */
/*                                                                           */
/*		Revision record:                                                     */
/*			10/11/98	Created                                              */
/*                                                                           */
/*****************************************************************************/
void WINAPI DebugMessageBox(HWND hWnd, char *szFile, int nLine,
	char *szText)
{
	// Local variables
	char szCaption[BUFFER_SIZE+1];		// Caption buffer

	// Compose caption
	_snprintf(szCaption, sizeof(szCaption), "File: %s, Line: %d", szFile,
		nLine);

	// Display debug message
	MessageBox(hWnd, szText, szCaption, MB_OK);
}

/*****************************************************************************/
#endif

/*****************************************************************************/
/*                                                                           */
/*						         WriteLog()                                  */
/*                                                                           */
/*		Function to write line to Stable32.log file                          */
/*                                                                           */
/*      Parameters:                                                          */
/*          int		nID		ID # of log message string                       */
/*      	char *	szText	Text string of additional log info               */
/*                                                                           */
/*      Return:                                                              */
/*			None (void)                                                      */
/*                                                                           */
/*		A new empty log file is created each time Stable32 starts.  A line   */
/*		is added to it with this function each time an operation is done.    */
/*		These lines contain the date, time, a "canned" message per the ID #  */
/*		and final string of additional variable information.  An example     */
/*		of the log file line when phase data file is opened is as follows:   */
/*                                                                           */
/*		01/16/99 0720 Phase data file opened: phase.dat                      */
/*                                                                           */
/*		A new log file is started if the ID # is zero.                       */
/*                                                                           */
/*		Revision record:                                                     */
/*			01/16/99	Created                                              */
/*			06/13/99	Changed szBuffer BUFFER_SIZE to BIG                  */
/*			01/29/00	Added special case for nID=0 to include version #    */
/*			06/01/00	Changed timetag to inclue seconds                    */
/*			07/28/08	Added ShortenExp()                                   */
/*                                                                           */
/*****************************************************************************/
void WINAPI WriteLog(int nID, char *szText)
{
	// Local variables
	char szBuffer[BIG_BUFFER_SIZE+1];	// Line buffer
	char szMessage[BUFFER_SIZE+1];	    // Message buffer
	FILE *hLog;              		    // DOS log file handle (NOT an HFILE*)

	// Update date & time display
	WriteDateMessage();

	// Get date & time in MM/DD/YY HH:MM:SS format from global buffer 
	strlcpy(szBuffer, szMDYHMS, sizeof(szBuffer));

	// Add space
	strlcat(szBuffer, " ", sizeof(szBuffer));

	// Get canned message string
	if(nID)
	{
		LoadString(hInst, (IDS_LOG_0 + nID), szMessage, BUFFER_SIZE);
		strlcat(szBuffer, szMessage, sizeof(szBuffer));
	}
	else // Special case for nID=0
	{
		LoadString(hInst, IDS_VERSION_NUMBER, szMessage, BUFFER_SIZE);
		strlcat(szBuffer, szMessage, sizeof(szBuffer));
		strlcat(szBuffer, " Log File", sizeof(szBuffer));
	}

	// Add space
	strlcat(szBuffer, " ", sizeof(szBuffer));

	// Add text string
	strlcat(szBuffer, szText, sizeof(szBuffer));

	// Add CR/LF
	strlcat(szBuffer, "\r\n", sizeof(szBuffer));

	// Shorten exponent
	ShortenExp(szBuffer);

	// Open log file
	if(nID==0) // New log file
	{
		if((hLog=fopen(szLogFile, "w"))==NULL)
		{
			// Error - could not open log file
			return;
		}
	}
	else // append
	{
		if((hLog=fopen(szLogFile, "a+"))==NULL)
		{
			// Error - could not open log file
			return;
		}
	}

	// Write line to log file
	fputs(szBuffer, hLog);

	// Close log file
	fclose(hLog);
}

/*****************************************************************************/
/*                                                                           */
/*						         IsNumeric()                                 */
/*                                                                           */
/*		Function to determine if a character is numeric:                     */
/*      '0' through '9', '+', '-', '.', and 'e', or 'E' as part of a number  */
/*                                                                           */
/*      Parameter:                                                           */
/*      	char 	cChar	Character to be tested                           */
/*			BOOL*	bDigit	Flag to indicate that previous character was a   */
/*							numeric digit.  This variable is defined in      */
/*							calling function.  It should be initialized to   */
/*							FALSE before first call for checking a buffer    */
/*							It will be updated to reflect current char.      */
/*                                                                           */
/*      Return:                                                              */
/*			BOOL    TRUE if numeric character, otherwise FALSE               */
/*                                                                           */
/*		Revision record:                                                     */
/*			02/03/99	Created                                              */
/*			08/24/02	Added *bDigit & changed treatment of e and E to      */
/*						non-numeric unless previous character was a digit    */
/*						e.g. rejects E in REJ, accepts E in 1E0              */
/*                                                                           */
/*****************************************************************************/

BOOL WINAPI IsNumeric(char cChar, BOOL *bDigit)
{
	// Test if numeric digit (0-9)
	if(isdigit((int)cChar))
    {
        *bDigit=TRUE;
		return TRUE;
    }

    // Test if +, - or .
	if( (cChar=='+') || (cChar=='-') || (cChar=='.') )
    {
        *bDigit=FALSE;
		return TRUE;
    }

    // Test if e or E and previous character was a digit
	if( ((cChar=='e') || (cChar=='E')) && *bDigit )
    {
        *bDigit=FALSE;
        return TRUE;
    }

    *bDigit=FALSE;
    return FALSE;
}

/*****************************************************************************/
/*                                                                           */
/*						           Flash()                                   */
/*                                                                           */
/*		Function to flash scroll lock light                                  */
/*                                                                           */
/*      Parameter:                                                           */
/*      	void	None                                                     */
/*                                                                           */
/*      Return:                                                              */
/*			void    None                                                     */
/*                                                                           */
/*		Revision record:                                                     */
/*			03/13/03	Created                                              */
/*                                                                           */
/*****************************************************************************/

void WINAPI Flash(void)
{
	// Toggle scroll lock key
	keybd_event(VK_SCROLL, 0, VK_PRESS, 0);
	keybd_event(VK_SCROLL, 0, VK_RELEASE, 0);

	// Set flash duration
	Sleep(100);

	// Untoggle scroll lock key
	keybd_event(VK_SCROLL, 0, VK_PRESS, 0);
	keybd_event(VK_SCROLL, 0, VK_RELEASE, 0);
}

/*****************************************************************************/
/*                                                                           */
/*						             LinFit()                                */
/*                                                                           */
/*		Numerical Recipes Function for Weighted Least-Squares Linear Fit     */
/*                                                                           */
/*      Given a set of points x[1...ndata], y[1...ndata] with standard       */
/*      deviations sig[1...ndata], fit them to a straight line y=a+bx        */
/*      by minimizing chi-square.  Returned are a, b and their               */
/*      respective probable uncertainities siga and sigb, the chi-square     */
/*      chi2, and the goodness-of-fit probability q (calc omitted).          */
/*                                                                           */
/*      If mwt=0 on input, then the standard deviations are assumed to be    */
/*      unavailable, and chi2 is normalized to unit std dev on all points.   */
/*                                                                           */
/*      Parameter:                                                           */
/*      	float	x[]		x data array (1-based)                           */
/*          float	y[]		y data array (1-based)                           */
/*          int		ndata	# data points                                    */
/*          float	sig[]	sigma (std dev) array (1-based)                  */
/*          int		mwt		sigma data available flag (0=No, 1=Yes)          */
/*          float*	a		a coefficient result                             */
/*          float*	b		b coefficient result                             */
/*          float*	siga	a coefficient std dev result                     */
/*          float*	sigb	b coefficient std dev result                     */
/*          float*	chi2	fit chi-square result                            */
/*          float*	q		goodness-of-fit result (always 1.0 here)         */
/*                                                                           */
/*      Return:                                                              */
/*			void    None                                                     */
/*                                                                           */
/*      Notes:                                                               */
/*			1. Original external dependencies: gammq, gcf, gser, gammln.     */
/*			2. Basic external call is to gammq(), which calls others.        */
/*			3. These were used to find goodness-of-fit probability, q        */
/*			   which we don't need.  So all elminated.                       */
/*			4. q result is always 1.0.                                       */
/*          5. gammq() usage eliminated.                                     */
/*			6. Note that it is used (but not exported) by CICS3.C            */
/*			   the CaclInvChiSqr() function in the FrequenC library.         */
/*          7. Could completely eliminate siga, sigb, chi2 & q.              */
/*          8. Don't forget to make input arrays 1-based.                    */
/*                                                                           */
/*		Revision record:                                                     */
/*			12/22/06	Created - Copied & adapted from NR fit()             */
/*                                                                           */
/*****************************************************************************/

// #include <math.h>
#define NRANSI
// #include "nrutil.h"

void WINAPI LinFit(float x[], float y[], int ndata, float sig[], int mwt,
	float *a, float *b, float *siga, float *sigb, float *chi2, float *q)
{
	// float gammq(float a, float x);
	int i;
	float wt,t,sxoss,sx=0.0,sy=0.0,st2=0.0,ss,sigdat;

	*b=0.0;
	if (mwt) {
		ss=0.0;
		for (i=1;i<=ndata;i++) {
			wt=(float)1.0/SQR(sig[i]);
			ss += wt;
			sx += x[i]*wt;
			sy += y[i]*wt;
		}
	} else {
		for (i=1;i<=ndata;i++) {
			sx += x[i];
			sy += y[i];
		}
		ss=(float)ndata;
	}
	sxoss=sx/ss;
	if (mwt) {
		for (i=1;i<=ndata;i++) {
			t=(x[i]-sxoss)/sig[i];
			st2 += t*t;
			*b += t*y[i]/sig[i];
		}
	} else {
		for (i=1;i<=ndata;i++) {
			t=x[i]-sxoss;
			st2 += t*t;
			*b += t*y[i];
		}
	}
	*b /= st2;
	*a=(sy-sx*(*b))/ss;
	*siga=(float)sqrt((1.0+sx*sx/(ss*st2))/ss);
	*sigb=(float)sqrt(1.0/st2);
	*chi2=0.0;
	*q=1.0;
	if (mwt == 0) {
		for (i=1;i<=ndata;i++)
			*chi2 += SQR(y[i]-(*a)-(*b)*x[i]);
		sigdat=(float)sqrt((*chi2)/(ndata-2));
		*siga *= sigdat;
		*sigb *= sigdat;
	} else {
		for (i=1;i<=ndata;i++)
			*chi2 += SQR((y[i]-(*a)-(*b)*x[i])/sig[i]);
		//if (ndata>2) *q=gammq(0.5*(ndata-2),0.5*(*chi2));
	}
}
#undef NRANSI

/*****************************************************************************/
/*                                                                           */
/*						          SmoothPSD()                                */
/*                                                                           */
/*     Smooth the results of PSD analysis by octaves                         */
/*                                                                           */
/*                                                                           */
/*     Algorithm:                                                            */
/*			1. Start at last point= fmax.                                    */
/*          2. Go back 1 octave to fmax/2.                                   */
/*          3. Calc # = points in highest octave & AF = # / PPO              */
/*             where PPO = # smoothed points per octave                      */
/*          4. Average data (in-place) in highest octave.                    */
/*          5. Go back another octave.  It will have half as many points.    */
/*             Average it by half of previous AF.                            */
/*          6. Iterate until AF becomes 1.                                   */
/*          7. Remove LF points prior to AF=1 octave                         */
/*          8. Smoothed data is stored at end of array, working forward      */
/*          9. When all smoothing done, smoothed data is shifted up          */
/*             to start of original array                                    */
/*                                                                           */
/*      Parameter:                                                           */
/*      	float	freq[]	Fourier frequency array (0-based)                */
/*          float	psd[]	PSD array (0-based)                              */
/*          float	avg[]	Averaging factor array (0-based)                 */
/*          int		*ndata	# data points                                    */
/*							Original # replaced by smoothed #                */
/*			int		ppo		# points per octave desired in smoothed data     */
/*          int		*maxaf	Highest AF used for smoothing                    */
/*                                                                           */
/*      Return:                                                              */
/*			void    None                                                     */
/*                                                                           */
/*		Revision record:                                                     */
/*			01/28/07	Created                                              */
/*                                                                           */
/*****************************************************************************/

void WINAPI SmoothPSD(float freq[], float psd[], float avg[], int *ndata,
	int ppo, int *maxaf)
{
	// Local variables
	int i;					// General and intermediate loop index
	int j;					// Inner index
	int n;					// Original # PSD points
	int points;				// # smoothed points
	int noctave;			// # points in octave
	int af;					// Averaging factor
	int octstart;			// Octave start index
	int octend;				// Octave end index
	int result;				// Result index
	int offset;				// Offset between initial and final results index
	float maxfreq;			// Original maximum Fourier frequency
	float freqsum;			// Frequency summation variable
	float psdsum;			// PSD summation variable

	// Copy original # PSD points
	n=*ndata;

	// Save original maximum Fourier frequency
	maxfreq=freq[n-1];

	// Search for PSD frequency one octave lower
	i=n-1;
	while(freq[i]>maxfreq/2.0)
	{
		i--;
	}

	// Calc # points in highest octave
	// Produces exact powers-of 2
	noctave=n-i; 

	// Calc averaging factor for highest octave
	af=noctave/ppo;

	// Return maximum averaging factor
	*maxaf=af;

	// First original data point in highest octave is y[n-noctave]
	// Last original data point in highest octave is y[n-1]
	// Set octave start index for highest octave
	octstart=n-noctave;

	// Set octave end index for highest octave
	octend=n-1;

	// Set first result index to top of array
	result=n-1;

	// Initialize # smoothed points to zero
	points=0;
	
	// Outer loop over all data - index=af
	while(af>0)
	{
		// Intermediate loop over 1 octave of data
		// Original octave start index is octstart
		// Original octave end index is octend=octstart+noctave-1.
		// Averaged (smoothed) result point index is result

		// Average data in place
		//for(i=0; i<noctave; i+=af) - freq order wrong in each af section
		for(i=noctave-1; i>=0; i-=af)
		{
			// Initialize sums to zero
			freqsum=0.0;
			psdsum=0.0;

			// Inner summation loop over one af interval
			for(j=0; j<af; j++)
			{
				#if(0) // Original code - last index (j) goes wrong way!
				freqsum+=freq[octstart+i+j];
				psdsum+=psd[octstart+i+j];
				#endif

				#if(1) // New code 02/07/07
				freqsum+=freq[octstart+i-j];
				psdsum+=psd[octstart+i-j];
				#endif
			}
			
			// Calc and store averages
			// Averages are stored at lowest unchanged array cells
			freq[result]=freqsum/af;
			psd[result]=psdsum/af;
			avg[result]=(float)af;

			// Decrement result index
			result--;

			// Increment # points
			points++;
		}

		// Halve averaging factor and octave size for next octave
		af/=2;
		noctave/=2;

		// Set start index for next lower octave
		octstart-=noctave;

		// Set end index for next lower octave
		octend=octstart+noctave-1;
	}

	// Shift averaged results up to start of their arrays
	// Result index was decremented a final time before outer loop terminated
	// # result points is n-result-1

	// Return # result points
	*ndata=points;

	// Calc offset between initial and final results index
	offset=n-points;

	// Shift results to start of arrays
	for(i=0; i<points; i++)
	{
		freq[i]=freq[i+offset];
		psd[i]=psd[i+offset];
		avg[i]=avg[i+offset];
	}
}

/*****************************************************************************/
/*                                                                           */
/*					             SemiLogFit()                                */
/*                                                                           */
/*		Numerical Recipes Function for Weighted Least-Squares Linear Fit     */
/*      mModified for Log X-Axix                                             */
/*                                                                           */
/*      Given a set of points x[1...ndata], y[1...ndata] with standard       */
/*      deviations sig[1...ndata], fit them to a straight line y=a+b*log(x)  */
/*      by minimizing chi-square.  Returned are a, b and their               */
/*      respective probable uncertainities siga and sigb, the chi-square     */
/*      chi2, and the goodness-of-fit probability q (calc omitted).          */
/*                                                                           */
/*      If mwt=0 on input, then the standard deviations are assumed to be    */
/*      unavailable, and chi2 is normalized to unit std dev on all points.   */
/*                                                                           */
/*      Parameter:                                                           */
/*      	float	x[]		x data array (1-based)                           */
/*          float	y[]		y data array (1-based)                           */
/*          int		ndata	# data points                                    */
/*          float	sig[]	sigma (std dev) array (1-based)                  */
/*          int		mwt		sigma data available flag (0=No, 1=Yes)          */
/*          float*	a		a coefficient result                             */
/*          float*	b		b coefficient result                             */
/*          float*	siga	a coefficient std dev result                     */
/*          float*	sigb	b coefficient std dev result                     */
/*          float*	chi2	fit chi-square result                            */
/*          float*	q		goodness-of-fit result (always 1.0 here)         */
/*                                                                           */
/*      Return:                                                              */
/*			void    None                                                     */
/*                                                                           */
/*      Notes:                                                               */
/*			1. Original external dependencies: gammq, gcf, gser, gammln.     */
/*			2. Basic external call is to gammq(), which calls others.        */
/*			3. These were used to find goodness-of-fit probability, q        */
/*			   which we don't need.  So all elminated.                       */
/*			4. q result is always 1.0.                                       */
/*          5. gammq() usage eliminated.                                     */
/*			6. Note that it is used (but not exported) by CICS3.C            */
/*			   the CaclInvChiSqr() function in the FrequenC library.         */
/*          7. Could completely eliminate siga, sigb, chi2 & q.              */
/*          8. Don't forget to make input arrays 1-based.                    */
/*                                                                           */
/*		Revision record:                                                     */
/*			01/29/07	Created - Adapted from LinFit()                      */
/*                                                                           */
/*****************************************************************************/

void WINAPI SemiLogFit(float x[], float y[], int ndata, float sig[], int mwt,
	float *a, float *b, float *siga, float *sigb, float *chi2, float *q)
{
	// float gammq(float a, float x);
	int i;
	float wt,t,sxoss,sx=0.0,sy=0.0,st2=0.0,ss,sigdat;

	*b=0.0;
	if (mwt) {
		ss=0.0;
		for (i=1;i<=ndata;i++) {
			wt=(float)1.0/SQR(sig[i]);
			ss += wt;
			sx += (float)((log10(x[i]))*wt);
			sy += y[i]*wt;
		}
	} else {
		for (i=1;i<=ndata;i++) {
			sx += (float)log10(x[i]);
			sy += y[i];
		}
		ss=(float)ndata;
	}
	sxoss=sx/ss;
	if (mwt) {
		for (i=1;i<=ndata;i++) {
			t=((float)log10(x[i])-sxoss)/sig[i];
			st2 += t*t;
			*b += t*y[i]/sig[i];
		}
	} else {
		for (i=1;i<=ndata;i++) {
			t=(float)log10(x[i])-sxoss;
			st2 += t*t;
			*b += t*y[i];
		}
	}
	*b /= st2;
	*a=(sy-sx*(*b))/ss;
	*siga=(float)sqrt((1.0+sx*sx/(ss*st2))/ss);
	*sigb=(float)sqrt(1.0/st2);
	*chi2=0.0;
	*q=1.0;
	if (mwt == 0) {
		for (i=1;i<=ndata;i++)
			*chi2 += SQR(y[i]-(*a)-(*b)*(float)log10(x[i]));
		sigdat=(float)sqrt((*chi2)/(ndata-2));
		*siga *= sigdat;
		*sigb *= sigdat;
	} else {
		for (i=1;i<=ndata;i++)
			*chi2 += SQR((y[i]-(*a)-(*b)*(float)log10(x[i]))/sig[i]);
	}
}

/*****************************************************************************/
/*                                                                           */
/*    Safe versions of strcpy() and strcat()                                 */
/*                                                                           */
/*    Refefence: http://en.wikipedia.org/wiki/Strlcpy                        */
/*                                                                           */
/*****************************************************************************/

/*      $OpenBSD: strlcpy.c,v 1.11 2006/05/05 15:27:38 millert Exp $        */

/*
 * Copyright (c) 1998 Todd C. Miller <Todd.Miller@courtesan.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <sys/types.h>
#include <string.h>

/*
 * Copy src to string dst of size siz.  At most siz-1 characters
 * will be copied.  Always NUL terminates (unless siz == 0).
 * Returns strlen(src); if retval >= siz, truncation occurred.
 */
size_t strlcpy(char *dst, const char *src, size_t siz)
{
        char *d = dst;
        const char *s = src;
        size_t n = siz;

        /* Copy as many bytes as will fit */
        if (n != 0) {
                while (--n != 0) {
                        if ((*d++ = *s++) == '\0')
                                break;
                }
        }

        /* Not enough room in dst, add NUL and traverse rest of src */
        if (n == 0) {
                if (siz != 0)
                        *d = '\0';                /* NUL-terminate dst */
                while (*s++)
                        ;
        }

        return(s - src - 1);        /* count does not include NUL */
}

/*****************************************************************************/

/*      $OpenBSD: strlcat.c,v 1.13 2005/08/08 08:05:37 espie Exp $        */

/*
 * Copyright (c) 1998 Todd C. Miller <Todd.Miller@courtesan.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <sys/types.h>
#include <string.h>

/*
 * Appends src to string dst of size siz (unlike strncat, siz is the
 * full size of dst, not space left).  At most siz-1 characters
 * will be copied.  Always NUL terminates (unless siz <= strlen(dst)).
 * Returns strlen(src) + MIN(siz, strlen(initial dst)).
 * If retval >= siz, truncation occurred.
 */
size_t strlcat(char *dst, const char *src, size_t siz)
{
        char *d = dst;
        const char *s = src;
        size_t n = siz;
        size_t dlen;

        /* Find the end of dst and adjust bytes left but don't go past end */
        while (n-- != 0 && *d != '\0')
                d++;
        dlen = d - dst;
        n = siz - dlen;

        if (n == 0)
                return(dlen + strlen(s));
        while (*s != '\0') {
                if (n != 1) {
                        *d++ = *s;
                        n--;
                }
                s++;
        }
        *d = '\0';

        return(dlen + (s - src));        /* count does not include NUL */
}

/*****************************************************************************/
