/*****************************************************************************/
/*                                                                           */
/*                            STABLE_Y.C                                     */
/*                                                                           */
/*      Stable Windows 95/NT Status Bar Functions                            */
/*                                                                           */
/*      Function                Description                                  */
/*                                                                           */
/*      CreateStatusBar()       Create Controls for Status Bar               */
/*      MoveStatusBarControls() Move Status Bar Controls to Screen Position  */
/*      UpdateStatusBar()       Update the Status Bar Display                */
/*      WriteProgressMessage()  Write Progress Message in Status Bar         */
/*      WriteDateMessage()      Write Date on Status Bar                     */
/*      DrawDataPlots()         Draw Phase & Freq Data Plots on Status Bar   */
/*      DisplayLims()           Display analysis limit #s on Status Bar      */
/*		DisplayPlotCursor()		Display cursor on status plot during Edit    */
/*                                                                           */
/*      Revision record:                                                     */
/*          01/06/98    Created from STABLE_U.C                              */
/*          01/19/98    Split MoveStatusBarControls()                        */
/*			07/30/07    Changed to safe string functions                     */
/*                                                                           */
/* (c) 1996-2007 W. Riley  Hamilton Technical  Services All Rights Reserved  */
/*                                                                           */
/*****************************************************************************/

#include "stable_w.h"           // STABLE_W header file
#include "location.h"			// Status bar control locations

#define STRICT                  // For Win32 compatibility

static HFONT hFont;             // Handle to font visible to all functions
								// in this file.  Font is created in
								// CreateStstusBar(), selected into
								// various device contexts, and deleted when
								// program closes in DeleteStatusBarFont()

/*****************************************************************************/
/*                                                                           */
/*      CreateStatusBar()                                                    */
/*                                                                           */
/*      Function to create controls for status bar                           */
/*      All controls are created with zero size at origin until moved        */
/*      Edit controls use special subclass procedure to handle Enter key     */
/*                                                                           */
/*      Globals:    Handles to status bar controls:                          */
/*                  HWND    hwndPhaseSelectButton,  hwndFreqSelectButton     */
/*                  HWND    hwndPhaseFileLabel,     hwndFreqFilelabel        */
/*                  HWND    hwndPhaseLimLabel,      hwndFreqLimLabel         */
/*                  HWND    hwndPhaseNumLabel,      hwndFreqNumLabel         */
/*                  HWND    hwndPhaseTauLabel,      hwndFreqTauLabel         */
/*                  HWMD    hwndPhaseStartLabel,    hwndFreqStartLabel       */
/*                  HWND    hwndPhaseEndLabel,      hwndFreqEndLabel         */
/*                  HWND    hwndPhaseLim,           hwndFreqLim              */
/*                  HWND    hwndPhaseNum,           hwndFreqNum              */
/*                  HWND    hwndPhaseFilename,      hwndFreqFilename         */
/*                  HWND    hwndPhaseTau,           hwndFreqTau              */
/*                  HWMD    hwndPhaseStart,         hwndFreqStart            */
/*                  HWND    hwndPhaseEnd,           hwndFreqEnd              */
/*                  FARPROC lpfnOldEdit             Original edit proc       */
/*                  FARPROC lpfnEditProc            Subclass edit proc       */
/*                                                                           */
/*      Parameters: HWND    hwnd                    Window handle            */
/*                  LPSTR   lpPhaseFile             Phase filename           */
/*                  LPSTR   lpFreqFile              Frequency filename       */
/*                                                                           */
/*      Return:     None    (void)                                           */
/*                                                                           */
/*****************************************************************************/

void WINAPI CreateStatusBar(HWND hwnd, LPSTR lpPhaseFile,
    LPSTR lpFreqFile)
{
	// Local variables
    HDC hdc;                // Handle to device context
    LOGFONT logfont;		// Font structure

	// Create font for all status bar controls
	memset(&logfont, 0, sizeof(LOGFONT));
    
	// Fill in size of font structure to match size of text in dialog boxes
    logfont.lfHeight=16;
    
	// Specify highest quality font
	logfont.lfQuality=PROOF_QUALITY;
	// Set font family
	logfont.lfPitchAndFamily=FF_SWISS;
	// Set the facename
	strlcpy(logfont.lfFaceName, "MS Sans Serif", LF_FACESIZE);
    
	// Create font
    hFont=CreateFontIndirect(&logfont);

	// Create invisible phase reset button
    hwndPhaseResetButton=CreateWindow("button", "",
        WS_CHILD | BS_PUSHBUTTON,
        0, 0, 0, 0, hwnd, (HMENU)IDC_PHASERESET, hInst, NULL);

    // Create invisible freq reset button
    hwndFreqResetButton=CreateWindow("button", "",
        WS_CHILD | BS_PUSHBUTTON,
        0, 0, 0, 0, hwnd, (HMENU)IDC_FREQRESET, hInst, NULL);

    // Create phase select button
    hwndPhaseSelectButton=CreateWindow("button", "Phase Data",
        WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON,
        0, 0, 0, 0, hwnd, (HMENU)IDC_PHASESELECT, hInst, NULL);

	// Obtain explicit hdc so it can be released to avoid resource leak
	hdc=GetDC(hwndPhaseSelectButton);
	SendMessage(GetParent(hwndPhaseSelectButton), WM_CTLCOLORBTN,
        (WPARAM)hdc, (LPARAM)hwndPhaseSelectButton);
	ReleaseDC(hwndPhaseSelectButton, hdc);

    // Create freq select button
    hwndFreqSelectButton=CreateWindow("button", "Freq Data",
        WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON,
        0, 0, 0, 0, hwnd, (HMENU)IDC_FREQSELECT, hInst, NULL);

	// Obtain explicit hdc so it can be released to avoid resource leak
	hdc=GetDC(hwndFreqSelectButton);
	SendMessage(GetParent(hwndFreqSelectButton), WM_CTLCOLORBTN,
        (WPARAM)hdc, (LPARAM)hwndFreqSelectButton);
	ReleaseDC(hwndFreqSelectButton, hdc);

    // Set the phase select button
    Button_SetCheck(hwndPhaseSelectButton, !bDataType);

    // Set the freq select button
    Button_SetCheck(hwndFreqSelectButton, bDataType);

    // Phase "File:" label window
    hwndPhaseFileLabel=CreateWindow("static", "  File:",
        WS_CHILD | WS_VISIBLE | SS_LEFTNOWORDWRAP | SS_NOPREFIX,
        0, 0, 0, 0, hwnd, (HMENU)0, hInst, NULL);

    // Phase "#:" label window
    hwndPhaseLimLabel=CreateWindow("static", "      #:",
        WS_CHILD | WS_VISIBLE | SS_LEFTNOWORDWRAP | SS_NOPREFIX,
        0, 0, 0, 0, hwnd, (HMENU)0, hInst, NULL);

    // Phase # window
    hwndPhaseLim=CreateWindow("static", "",
        WS_CHILD | WS_VISIBLE | SS_LEFTNOWORDWRAP | SS_NOPREFIX,
        0, 0, 0, 0, hwnd, (HMENU)0, hInst, NULL);

    // Phase "File #:" label window
    hwndPhaseNumLabel=CreateWindow("static", "   Total #:",
        WS_CHILD | WS_VISIBLE | SS_LEFTNOWORDWRAP | SS_NOPREFIX,
        0, 0, 0, 0, hwnd, (HMENU)0, hInst, NULL);

    // Phase File # window
    hwndPhaseNum=CreateWindow("static", "",
        WS_CHILD | WS_VISIBLE | SS_LEFTNOWORDWRAP | SS_NOPREFIX,
        0, 0, 0, 0, hwnd, (HMENU)0, hInst, NULL);

    // Phase "Tau:" label window
    hwndPhaseTauLabel=CreateWindow("static", "  Tau:",
        WS_CHILD | WS_VISIBLE | SS_LEFTNOWORDWRAP | SS_NOPREFIX,
        0, 0, 0, 0, hwnd, 0, hInst, NULL);

    // Phase "Start:" label window
    hwndPhaseStartLabel=CreateWindow("static", "  Start:",
        WS_CHILD | WS_VISIBLE | SS_LEFTNOWORDWRAP | SS_NOPREFIX,
        0, 0, 0, 0, hwnd, 0, hInst, NULL);

    // Phase "End:" label window
    hwndPhaseEndLabel=CreateWindow("static", "  End:",
        WS_CHILD | WS_VISIBLE | SS_LEFTNOWORDWRAP | SS_NOPREFIX,
        0, 0, 0, 0, hwnd, 0, hInst, NULL);

    // Frequency "File:" label window
    hwndFreqFileLabel=CreateWindow("static", "  File:",
    WS_CHILD | WS_VISIBLE | SS_LEFTNOWORDWRAP | SS_NOPREFIX,
    0, 0, 0, 0, hwnd, 0, hInst, NULL);

    // Frequency "#:" label window
    hwndFreqLimLabel=CreateWindow("static", "      #:",
        WS_CHILD | WS_VISIBLE | SS_LEFTNOWORDWRAP | SS_NOPREFIX,
        0, 0, 0, 0, hwnd, 0, hInst, NULL);

    // Frequency # window
    hwndFreqLim=CreateWindow("static", "",
        WS_CHILD | WS_VISIBLE | SS_LEFTNOWORDWRAP | SS_NOPREFIX,
        0, 0, 0, 0, hwnd, 0, hInst, NULL);

    // Frequency "File #:" label window
    hwndFreqNumLabel=CreateWindow("static", "   Total #:",
        WS_CHILD | WS_VISIBLE | SS_LEFTNOWORDWRAP | SS_NOPREFIX,
        0, 0, 0, 0, hwnd, (HMENU)0, hInst, NULL);

    // Frequency File # window
    hwndFreqNum=CreateWindow("static", "",
        WS_CHILD | WS_VISIBLE | SS_LEFTNOWORDWRAP | SS_NOPREFIX,
        0, 0, 0, 0, hwnd, (HMENU)0, hInst, NULL);

    // Frequency "Tau:" label window
    hwndFreqTauLabel=CreateWindow("static", "  Tau:",
    WS_CHILD | WS_VISIBLE | SS_LEFTNOWORDWRAP | SS_NOPREFIX,
    0, 0, 0, 0, hwnd, 0, hInst, NULL);

    // Frequency "Start:" label window
    hwndFreqStartLabel=CreateWindow("static", "  Start:",
    WS_CHILD | WS_VISIBLE | SS_LEFTNOWORDWRAP | SS_NOPREFIX,
    0, 0, 0, 0, hwnd, 0, hInst, NULL);

    // Frequency "End:" label window
    hwndFreqEndLabel=CreateWindow("static", "  End:",
        WS_CHILD | WS_VISIBLE | SS_LEFTNOWORDWRAP | SS_NOPREFIX,
        0, 0, 0, 0, hwnd, 0, hInst, NULL);

	// Create phase plot window
	hwndPhasePlot=CreateWindowEx(WS_EX_CLIENTEDGE, "static", "",
		WS_CHILD | WS_VISIBLE | SS_CENTER | SS_NOPREFIX | WS_BORDER,
		0, 0, 0, 0, hwnd, (HMENU)0, hInst, NULL);

	// Create freq plot window
	hwndFreqPlot=CreateWindowEx(WS_EX_CLIENTEDGE, "static", "",
		WS_CHILD | WS_VISIBLE | SS_CENTER | SS_NOPREFIX,
		0, 0, 0, 0, hwnd, (HMENU)0, hInst, NULL);

    
    // Create phase filename edit box
    hwndPhaseFilename=CreateWindowEx(WS_EX_CLIENTEDGE, "edit", lpPhaseFile,
        WS_CHILD | WS_VISIBLE | ES_LEFT | ES_OEMCONVERT | ES_AUTOHSCROLL,
        0, 0, 0, 0, hwnd, (HMENU)IDC_PHASEFILENAME, hInst, NULL);

    // Set up edit control subclass function
    // Save address of original edit procedure
    lpfnOldEdit=(FARPROC)GetWindowLong(hwndPhaseFilename, GWL_WNDPROC);

    // Set address of new edit subclass procedure
    SetWindowLong(hwndPhaseFilename, GWL_WNDPROC, (LONG)lpfnEditProc);

    // Create frequency filename edit box
    hwndFreqFilename=CreateWindowEx(WS_EX_CLIENTEDGE, "edit", lpFreqFile,
        WS_CHILD | WS_VISIBLE | ES_LEFT | ES_OEMCONVERT | ES_AUTOHSCROLL,
        0, 0, 0, 0, hwnd, (HMENU)IDC_FREQFILENAME, hInst, NULL);

    // Set address of new edit subclass procedure
    SetWindowLong(hwndFreqFilename, GWL_WNDPROC, (LONG)lpfnEditProc);

    // Create phase tau edit box
    hwndPhaseTau=CreateWindowEx(WS_EX_CLIENTEDGE, "edit", "",
        WS_CHILD | WS_VISIBLE | ES_LEFT | ES_LOWERCASE,
        0, 0, 0, 0, hwnd, (HMENU)IDC_PHASETAU, hInst, NULL);

    // Set address of new edit subclass procedure
    SetWindowLong(hwndPhaseTau, GWL_WNDPROC, (LONG)lpfnEditProc);

    // Create frequency tau edit box
    hwndFreqTau=CreateWindowEx(WS_EX_CLIENTEDGE, "edit", "",
        WS_CHILD | WS_VISIBLE | ES_LEFT | ES_LOWERCASE,
        0, 0, 0, 0, hwnd, (HMENU)IDC_FREQTAU, hInst, NULL);

    // Set address of new edit subclass procedure
    SetWindowLong(hwndFreqTau, GWL_WNDPROC, (LONG)lpfnEditProc);

    // Create phase start edit box
    hwndPhaseStart=CreateWindowEx(WS_EX_CLIENTEDGE, "edit", "",
        WS_CHILD | WS_VISIBLE | ES_LEFT | ES_AUTOHSCROLL,
        0, 0, 0,0, hwnd, (HMENU)IDC_PHASESTART, hInst, NULL);

    // Set address of new edit subclass procedure
    SetWindowLong(hwndPhaseStart, GWL_WNDPROC, (LONG)lpfnEditProc);

    // Create frequency start edit box
    hwndFreqStart=CreateWindowEx(WS_EX_CLIENTEDGE, "edit", "",
        WS_CHILD | WS_VISIBLE | ES_LEFT | ES_AUTOHSCROLL,
        0, 0, 0, 0, hwnd, (HMENU)IDC_FREQSTART, hInst, NULL);

    // Set address of new edit subclass procedure
    SetWindowLong(hwndFreqStart, GWL_WNDPROC, (LONG)lpfnEditProc);

    // Create phase end edit box
    hwndPhaseEnd=CreateWindowEx(WS_EX_CLIENTEDGE, "edit", "",
        WS_CHILD | WS_VISIBLE | ES_LEFT | ES_AUTOHSCROLL,
        0, 0, 0, 0, hwnd, (HMENU)IDC_PHASEEND, hInst, NULL);

    // Set address of new edit subclass procedure
    SetWindowLong(hwndPhaseEnd, GWL_WNDPROC, (LONG)lpfnEditProc);

    // Create frequency end edit box
    hwndFreqEnd=CreateWindowEx(WS_EX_CLIENTEDGE, "edit", "",
        WS_CHILD | WS_VISIBLE | ES_LEFT | ES_AUTOHSCROLL,
        0, 0, 0, 0, hwnd, (HMENU)IDC_FREQEND, hInst, NULL);

    // Set address of new edit subclass procedure
    SetWindowLong(hwndFreqEnd, GWL_WNDPROC, (LONG)lpfnEditProc);
  
    return;
}

/*****************************************************************************/
/*                                                                           */
/*      MoveStatusBarControls()                                              */
/*                                                                           */
/*      Function to move status bar controls to bottom of client area        */
/*                                                                           */
/*      Globals:    Handles to status bar controls:                          */
/*                  HWND    hwndPhaseSelectButton,  hwndFreqSelectButton     */
/*                  HWND    hwndPhaseTauLabel,      hwndFreqTauLabel         */
/*                  HWMD    hwndPhaseStartLabel,    hwndFreqStartLabel       */
/*                  HWND    hwndPhaseEndLabel,      hwndFreqEndLabel         */
/*                  HWND    hwndPhaseLim,           hwndFreqLim              */
/*                  HWND    hwndPhaseNum,           hwndFreqNum              */
/*                  HWND    hwndPhaseFilename,      hwndFreqFilename         */
/*                  HWND    hwndPhaseTau,           hwndFreqTau              */
/*                  HWMD    hwndPhaseStart,         hwndFreqStart            */
/*                  HWND    hwndPhaseEnd,           hwndFreqEnd              */
/*                  RECT    rc                                               */
/*                  int     cxClient,               cyClient                 */
/*                  int     cxChar,                 cyChar                   */
/*                                                                           */
/*      Parameters: HWND    hwnd                    Parent window handle     */
/*                                                                           */
/*      Return:     None    (void)                                           */
/*                                                                           */
/*      Revision record:                                                     */
/*          11/15/96    Added too few points bitmap logic                    */
/*          11/16/96    Added limits bitmap logic                            */
/*                      Added limits reset button logic                      */
/*          11/19/96    Improved limits reset button logic                   */
/*          06/06/97    Shorten size of Start & End fields so no int O/F     */
/*          12/25/97    Changes field widths for 6-digit #s                  */
/*          01/19/98    General cleanup & spinoff to UpdateStatusBar()       */
/*                                                                           */
/*****************************************************************************/

void WINAPI MoveStatusBarControls(HWND hwnd)
{
    // Local variables
    HDC hdc;                // Handle to device context 
	HDC hdc2;				// Temporary DC handle
    
	// Skip this function if status bar window handles not valid
	// as appears to be the case during 1st call(s)
	if(hwndPhaseFilename==NULL)
	{
		return;
	}

	// Get handle to main window device context
    hdc=GetDC(hwnd);

    // Select font into device context
	SelectObject(hdc, hFont);

    // Get size of client area
    GetClientRect(hwnd, &rc);
    cxClient=rc.right-rc.left;
    cyClient=rc.bottom-rc.top;
   
    // Color entire status bar region
    SetRect(&rc, 0, (int)(SW_BAR_Y), cxClient, cyClient);
    FillRect(hdc, &rc, hBrush);

	// Set phase plot background color
	// Obtain explicit hdc so it can be released to avoid resource leak
	hdc2=GetDC(hwndPhasePlot);
	SendMessage(GetParent(hwndPhasePlot), WM_CTLCOLORBTN,
		(WPARAM)hdc2, (LPARAM)hwndPhasePlot);
	ReleaseDC(hwndPhasePlot, hdc2);

	// Set freq plot background color
	// Obtain explicit hdc so it can be released to avoid resource leak
	hdc2=GetDC(hwndFreqPlot);
	SendMessage(GetParent(hwndFreqPlot), WM_CTLCOLORBTN,
		(WPARAM)hdc2, (LPARAM)hwndFreqPlot);
	ReleaseDC(hwndFreqPlot, hdc2);

	// Set fonts & move all status bar controls
    SendMessage(hwndPhaseSelectButton, WM_SETFONT, (WPARAM)hFont,
		(LPARAM)TRUE);
    MoveWindow(hwndPhaseSelectButton, SW_PSB_X, SW_PSB_Y, SW_PSB_W,
		SW_PSB_H, TRUE);

    SendMessage(hwndFreqSelectButton, WM_SETFONT, (WPARAM)hFont,
		(LPARAM)TRUE);
    MoveWindow(hwndFreqSelectButton, SW_FSB_X, SW_FSB_Y, SW_FSB_W,
		SW_FSB_H, TRUE);

    SendMessage(hwndPhaseFilename, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndPhaseFilename, SW_PFE_X, SW_PFE_Y, SW_PFE_W,
		SW_PFE_H, TRUE);

    SendMessage(hwndFreqFilename, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndFreqFilename, SW_FFE_X, SW_FFE_Y, SW_FFE_W,
		SW_FFE_H, TRUE);

    SendMessage(hwndPhaseTau, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndPhaseTau, SW_PTE_X, SW_PTE_Y, SW_PTE_W, SW_PTE_H, 1);

    SendMessage(hwndFreqTau, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndFreqTau, SW_FTE_X, SW_FTE_Y, SW_FTE_W, SW_FTE_H, 1);

    SendMessage(hwndPhaseFileLabel, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndPhaseFileLabel, SW_PFL_X, SW_PFL_Y, SW_PFL_W, SW_PFL_H, 1);

    SendMessage(hwndPhaseLimLabel, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndPhaseLimLabel, SW_PLL_X, SW_PLL_Y, SW_PLL_W, SW_PLL_H, 1);

    SendMessage(hwndPhaseLim, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndPhaseLim, SW_PLT_X, SW_PLT_Y, SW_PLT_W, SW_PLT_H, 1);
    
    SendMessage(hwndPhaseNumLabel, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndPhaseNumLabel, SW_PNL_X, SW_PNL_Y, SW_PNL_W, SW_PNL_H, 1);

    SendMessage(hwndPhaseNum, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndPhaseNum, SW_PNT_X, SW_PNT_Y, SW_PNT_W, SW_PNT_H, 1);

    SendMessage(hwndFreqFileLabel, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndFreqFileLabel, SW_FFL_X, SW_FFL_Y, SW_FFL_W, SW_FFL_H, 1);

    SendMessage(hwndFreqLimLabel, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndFreqLimLabel, SW_FLL_X, SW_FLL_Y, SW_FLL_W, SW_FLL_H, 1);

    SendMessage(hwndFreqLim, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndFreqLim, SW_FLT_X, SW_FLT_Y, SW_FLT_W, SW_FLT_H, 1);

    SendMessage(hwndFreqNumLabel, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndFreqNumLabel, SW_FNL_X, SW_FNL_Y, SW_FNL_W, SW_FNL_H, 1);

    SendMessage(hwndFreqNum, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndFreqNum, SW_FNT_X, SW_FNT_Y, SW_FNT_W, SW_FNT_H, 1);

    SendMessage(hwndPhaseStart, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndPhaseStart, SW_PSE_X, SW_PSE_Y, SW_PSE_W, SW_PSE_H, 1);

    SendMessage(hwndFreqStart, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndFreqStart, SW_FSE_X, SW_FSE_Y, SW_FSE_W, SW_FSE_H, 1);

    SendMessage(hwndPhaseEnd, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndPhaseEnd, SW_PEE_X, SW_PEE_Y, SW_PEE_W, SW_PEE_H, 1);

    SendMessage(hwndFreqEnd, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndFreqEnd, SW_FEE_X, SW_FEE_Y, SW_FEE_W, SW_FEE_H, 1);

	SendMessage(hwndPhaseTauLabel, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndPhaseTauLabel, SW_PTL_X, SW_PTL_Y, SW_PTL_W, SW_PTL_H, 1);

    SendMessage(hwndPhaseStartLabel, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndPhaseStartLabel, SW_PSL_X, SW_PSL_Y, SW_PSL_W, SW_PSL_H, 1);

    SendMessage(hwndPhaseEndLabel, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndPhaseEndLabel, SW_PEL_X, SW_PEL_Y, SW_PEL_W, SW_PEL_H, 1);

    SendMessage(hwndFreqTauLabel, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndFreqTauLabel, SW_FTL_X, SW_FTL_Y, SW_FTL_W, SW_FTL_H, 1);

    SendMessage(hwndFreqStartLabel, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndFreqStartLabel, SW_FSL_X, SW_FSL_Y, SW_FSL_W, SW_FSL_H, 1);

    SendMessage(hwndFreqEndLabel, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
    MoveWindow(hwndFreqEndLabel, SW_FEL_X, SW_FEL_Y, SW_FEL_W, SW_FEL_H, 1);

	// Determine screen resolution
	if((GetSystemMetrics(SM_CXSCREEN))>1000) // XGA is 1024
	{
		bShowPlot=TRUE; // XGA or better
	}
	else
	{
		bShowPlot=FALSE; // SVGA or lower
	}
	
	// New code 02/10/00
	// Should totally eliminate the status bar data plot boxes when they are
	// disabled.  Should also disable them when screen is too narrow (cxMain
	// too small) even if XGA or better when main window is shrunk.
	// A width of 950 pixels is about the limit for which the plots are OK.

	if(cxMain<950)
	{
		bShowPlot=FALSE;
	}
	
	// Move or fill status bar plots
	if(bShowPlot)
	{
		// Move status bar plots
		SendMessage(hwndPhasePlot, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
		MoveWindow(hwndPhasePlot, SW_PPT_X, SW_PPT_Y, SW_PPT_W, SW_PPT_H, 1);
	    SendMessage(hwndFreqPlot, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);
		MoveWindow(hwndFreqPlot, SW_FPT_X, SW_FPT_Y, SW_FPT_W, SW_FPT_H, 1);
	}
	else
	{
		// Fill status bar plot holes if they are killed
		SetRect(&rc, SW_PPT_X, SW_PPT_Y, (SW_PPT_X)+(SW_PPT_W),
			(SW_PPT_Y)+(SW_PPT_H));
		FillRect(hdc, &rc, hBrush);
		SetRect(&rc, SW_FPT_X, SW_FPT_Y, (SW_FPT_X)+(SW_FPT_W),
			(SW_FPT_Y)+(SW_FPT_H));
		FillRect(hdc, &rc, hBrush);

		// Reset status plot control flag - avoid useless invisible plotting
		// bStatusPlot=FALSE; // Removed 2/11/00 - Don't want to prohibit
		// status plots from being reactivated when main windows is enlarged
	}

    MoveWindow(hwndPhaseResetButton, 5+14*cxChar+46,
        (int)(cyClient-3.02*cyChar), 3*cxChar/2, 3*cyChar/4, 1);
	MoveWindow(hwndFreqResetButton, cxClient/2+5+14*cxChar+45,
        (int)(cyClient-3.02*cyChar), 3*cxChar/2, 3*cyChar/4, 1);
    
	// Draw lines to outline the status box
    // Horizontal line at top
	// This is the Topmost Line above the message region
    SetRect(&rc, -1, (int)(cyClient-3.25*cyChar-4-2.5*cyChar/2.5),
        cxClient+1, (int)(cyClient-3.25*cyChar-2.5*cyChar/2.5));
    FillRect(hdc, &rc, hBrush);
    FrameRect(hdc, &rc, GetStockObject(BLACK_BRUSH));

    // Vertical Line at middle
    SetRect(&rc, cxClient/2-2, (int)(cyClient-3.25*cyChar-2),
        cxClient/2-1, cyClient);
    FillRect(hdc, &rc, hBrush);

    SetRect(&rc, cxClient/2, (int)(cyClient-3.25*cyChar-2),
        cxClient/2+1, cyClient);
    FillRect(hdc, &rc, GetStockObject(BLACK_BRUSH));

    // Message bar
    SetRect(&rc, -1, (int)(cyClient-3.25*cyChar-1-2.5*cyChar/2.5),
        cxClient+1, (int)(cyClient-3.25*cyChar-1));
    FillRect(hdc, &rc, hBrush);
    FrameRect(hdc, &rc, GetStockObject(BLACK_BRUSH));
    
	// Release main window device context
	ReleaseDC(hwnd, hdc);

 	return;
}

/*****************************************************************************/
/*                                                                           */
/*                      UpdateStatusBar()                                    */
/*                                                                           */
/*      Function to update status bar                                        */
/*                                                                           */
/*      Globals:    Handles to status bar controls:                          */
/*                  HWND    hwndPhaseSelectButton,  hwndFreqSelectButton     */
/*                  HWND    hwndPhaseTauLabel,      hwndFreqTauLabel         */
/*                  HWMD    hwndPhaseStartLabel,    hwndFreqStartLabel       */
/*                  HWND    hwndPhaseEndLabel,      hwndFreqEndLabel         */
/*                  HWND    hwndPhaseLim,           hwndFreqLim              */
/*                  HWND    hwndPhaseNum,           hwndFreqNum              */
/*                  HWND    hwndPhaseFilename,      hwndFreqFilename         */
/*                  HWND    hwndPhaseTau,           hwndFreqTau              */
/*                  HWMD    hwndPhaseStart,         hwndFreqStart            */
/*                  HWND    hwndPhaseEnd,           hwndFreqEnd              */
/*                  RECT    rc                                               */
/*                  int     cxClient,               cyClient                 */
/*                  int     cxChar,                 cyChar                   */
/*                                                                           */
/*      Parameters: HWND    hwnd                    Parent window handle     */
/*                                                                           */
/*      Return:     None    (void)                                           */
/*                                                                           */
/*      Revision record:                                                     */
/*          01/19/98    Split off from MoveStatusBarControls()               */
/*			08/27/98	Added code for Options/Status Bar menu items         */
/*			09/10/98	Added bNewData argument                              */
/*                                                                           */
/*****************************************************************************/

void WINAPI UpdateStatusBar(HWND hwnd, BOOL bNewData)
{
    // Local variables
    HDC hdc;							// Handle to device context 
    RECT rc;							// Client area rectangle

	// Abort if window handles are NULL
	if(hwndPhaseFilename==NULL)
	{
		return;
	}

    // Get handle to main window device context
    hdc = GetDC(hwnd);

    // Get size of client area
    GetClientRect(hwnd, &rc);
    cxClient=rc.right-rc.left;
    cyClient=rc.bottom-rc.top;
   
    // Enable/Disable status bar controls depending on whether or not there
    // is valid phase or freq data as indicated by bPhaseData & bFreqData
    EnableWindow(hwndPhaseFilename, bPhaseData);
    EnableWindow(hwndPhaseTau, bPhaseData);
    EnableWindow(hwndPhaseStart, bPhaseData);
    EnableWindow(hwndPhaseEnd, bPhaseData);
    EnableWindow(hwndFreqFilename, bFreqData);
    EnableWindow(hwndFreqTau, bFreqData);
    EnableWindow(hwndFreqStart, bFreqData);
    EnableWindow(hwndFreqEnd, bFreqData);

	// Fill white regions - must do individually
	// filling whole ststus bar wrecks CLIENTEDGE borders
	// Region to left of phase select button
    SetRect(&rc, 0, SW_BUT_Y, SW_BUT_X, ((SW_BUT_Y)+(SW_BUT_H)));
    FillRect(hdc, &rc, hBrush);

	// Region to left of freq select button
    SetRect(&rc, SW_CENTER, SW_BUT_Y, ((SW_CENTER)+(SW_BUT_X)),
		((SW_BUT_Y)+(SW_BUT_H)));
    FillRect(hdc, &rc, hBrush);

	// Region between phase file & tau control rows
	SetRect(&rc, 0, ((SW_FILE_Y)+(SW_EDIT_H)), SW_PLOT_X, SW_TAU_Y);
	FillRect(hdc, &rc, hBrush);

	// Region between freq file & tau control rows
	SetRect(&rc, SW_CENTER, ((SW_FILE_Y)+(SW_EDIT_H)),
		((SW_CENTER)+(SW_PLOT_X)), SW_TAU_Y);
	FillRect(hdc, &rc, hBrush);

	// Region between phase #s & plot
	SetRect(&rc, ((SW_END_X)+(SW_NUM_W)), SW_BUT_Y, SW_PLOT_X,
		((SW_TAU_Y)+(SW_EDIT_H)));
	FillRect(hdc, &rc, hBrush);

	// Region between freq #s & plot
	SetRect(&rc, ((SW_CENTER)+(SW_END_X)+(SW_NUM_W)), SW_BUT_Y,
		((SW_CENTER)+(SW_PLOT_X)),
		((SW_TAU_Y)+(SW_EDIT_H)));
	FillRect(hdc, &rc, hBrush);

	// Region to right of phase plot
	SetRect(&rc, ((SW_PLOT_X)+(SW_PLOT_W)), SW_BAR_Y, SW_CENTER-1,
		((SW_BAR_Y)+(SW_BAR_H)));
	FillRect(hdc, &rc, hBrush);

	// Region to right of freq plot
	SetRect(&rc, ((SW_CENTER)+(SW_PLOT_X)+(SW_PLOT_W)),
		SW_BAR_Y, cxClient-1, ((SW_BAR_Y)+(SW_BAR_H)));
	FillRect(hdc, &rc, hBrush);

	// Region above phase plot
	SetRect(&rc, SW_PLOT_X, SW_BAR_Y, SW_CENTER-1, SW_PLOT_Y);
	FillRect(hdc, &rc, hBrush);

	// Region above freq plot
	SetRect(&rc, ((SW_CENTER)+(SW_PLOT_X)), SW_BAR_Y, cxClient-1,
		SW_PLOT_Y);
	FillRect(hdc, &rc, hBrush);

	// Region to right of phase select button
	SetRect(&rc, ((SW_BUT_X)+(SW_BUT_W)), SW_BUT_Y,
		((SW_ENDLABEL_X)-(2*cxChar)), SW_FILE_Y);
	FillRect(hdc, &rc, hBrush);

	// Region to right of freq select button
	SetRect(&rc, ((SW_CENTER)+(SW_BUT_X)+(SW_BUT_W)), SW_BUT_Y,
		((SW_CENTER)+(SW_ENDLABEL_X)-(2*cxChar)),
		SW_FILE_Y);
	FillRect(hdc, &rc, hBrush);

	// Region above phase total #
	SetRect(&rc, ((SW_ENDLABEL_X)-(2*cxChar)), SW_BUT_Y,
		((SW_END_X)+(SW_NUM_W)), ((SW_BUT_Y)+(3)));
	FillRect(hdc, &rc, hBrush);

	// Region above freq total #
	SetRect(&rc, ((SW_CENTER)+(SW_ENDLABEL_X)-(2*cxChar)), SW_BUT_Y,
		((SW_CENTER)+(SW_END_X)+(SW_NUM_W)), ((SW_BUT_Y)+(3)));
	FillRect(hdc, &rc, hBrush);

	// Draw lines to outline the status box
    // Horizontal line at top
	// This is the Topmost Line above the message region
    SetRect(&rc, -1, (int)(cyClient-3.25*cyChar-4-2.5*cyChar/2.5),
        cxClient+1, (int)(cyClient-3.25*cyChar-2.5*cyChar/2.5));
    FillRect(hdc, &rc, hBrush);
    FrameRect(hdc, &rc, GetStockObject(BLACK_BRUSH));

	// New code 11/16/06
	// Draw line above file tabs
	if(bTabs)
	{
		MoveToEx(hdc, 0, (int)(cyClient-3.25*cyChar-33-2.5*cyChar/2.5), NULL);
		LineTo(hdc, cxClient, (int)(cyClient-3.25*cyChar-33-2.5*cyChar/2.5));
	}
	else // Erase line
	{
		SetRect(&rc, 0, (int)(cyClient-3.25*cyChar-32-2.5*cyChar/2.5),
			cxClient, (int)(cyClient-3.25*cyChar-34-2.5*cyChar/2.5));
		FillRect(hdc, &rc, GetStockObject(WHITE_BRUSH));
	}

    // Vertical Line at middle
    SetRect(&rc, cxClient/2-2, (int)(cyClient-3.25*cyChar-2),
        cxClient/2-1, cyClient);
    FillRect(hdc, &rc, hBrush);

    SetRect(&rc, cxClient/2, (int)(cyClient-3.25*cyChar-2),
        cxClient/2+1, cyClient);
    FillRect(hdc, &rc, GetStockObject(BLACK_BRUSH));

    // Message bar
    SetRect(&rc, -1, (int)(cyClient-3.25*cyChar-1-2.5*cyChar/2.5),
        cxClient+1, (int)(cyClient-3.25*cyChar-1));
    FillRect(hdc, &rc, hBrush);
    FrameRect(hdc, &rc, GetStockObject(BLACK_BRUSH));

    // Display file unsaved bitmap
    // if(bPhaseData && bPhaseDataChanged) // Replaced 11/22/06
    if(bPhaseData && bPhaseDataUnsaved)
    {
        DrawBitmap(hdc, hUnsavedBitmap, (short)(5+12*cxChar+6),
            (short)(cyClient-3.00*cyChar-2));
    }
	
	// if(bFreqData && bFreqDataChanged) // Replaced 11/22/06
	if(bFreqData && bFreqDataUnsaved)
    {
        DrawBitmap(hdc, hUnsavedBitmap, (short)(cxClient/2+5+12*cxChar+5),
            (short)(cyClient-3.00*cyChar-2));
    }

    // Display timetags exist bitmap
    if(bPhaseData && bPhaseTimetags)
    {
        DrawTransparentBitmap(hdc, hTagSrcBitmap, hTagMskBitmap,
			(short)(5+12*cxChar+26), (short)(cyClient-3.00*cyChar-2));
    }
    if(bFreqData && bFreqTimetags)
    {
        DrawTransparentBitmap(hdc, hTagSrcBitmap, hTagMskBitmap,
			(short)(cxClient/2+5+12*cxChar+25),
			(short)(cyClient-3.00*cyChar-2));
    }

    // Display synch or unsynch bitmap
    if(bPhaseData && bFreqData)
    {
        if(bSynch)
        {
			DrawTransparentBitmap(hdc, hSynchSrcBitmap, hSynchMskBitmap,
				(short)((cxClient/2)-8), (short)(SW_BAR_Y-cyChar));
        }
     }
        
    // Display analysis limits or too few bitmaps
    // Also display reset button next to bitmap
    if(bPhaseData)
    {
        if((fPhaseData[2]-fPhaseData[1]+1)<MIN_DATA_FILE_SIZE)
        {
            if( (fPhaseData[2]<fPhaseData[0]) || (fPhaseData[1]>1) )
            {
                ShowWindow(hwndPhaseResetButton, SW_SHOWNORMAL);

	            // Redraw the phase reset button
	            InvalidateRect(hwndPhaseResetButton, NULL, TRUE);

				// Activate Options/Status Bar/Phase All menu item
				EnableMenuItem(hMenu, IDC_PHASERESET, MF_ENABLED);
          }
            else
            {
                ShowWindow(hwndPhaseResetButton, SW_HIDE);
                // Phase reset button "plug" region
                SetRect(&rc, 5+14*cxChar+46, (int)(cyClient-2.30*cyChar),
                    5+14*cxChar+3*cxChar/2+46, cyClient-3*cyChar-1);
                FillRect(hdc, &rc, hBrush);

				// Deactivate Options/Status Bar/Phase All menu item
				EnableMenuItem(hMenu, IDC_PHASERESET, MF_GRAYED);

            }

            DrawTransparentBitmap(hdc, hFewSrcBitmap, hFewMskBitmap,
				(short)(5+12*cxChar+46), (short)(cyClient-3.00*cyChar-2));
        }
        else if( (fPhaseData[2]<fPhaseData[0]) || (fPhaseData[1]>1) )
        {
            DrawTransparentBitmap(hdc, hLimitSrcBitmap, hLimitMskBitmap,
				(short)(5+12*cxChar+46), (short)(cyClient-3.00*cyChar-2));

            ShowWindow(hwndPhaseResetButton, SW_SHOWNORMAL);

           // Redraw the phase reset button
            InvalidateRect(hwndPhaseResetButton, NULL, TRUE);

 			// Activate Options/Status Bar/Phase All menu item
			EnableMenuItem(hMenu, IDC_PHASERESET, MF_ENABLED);
       }
        else
        {
            ShowWindow(hwndPhaseResetButton, SW_HIDE);
            // Phase reset button "plug" region
            SetRect(&rc, 5+14*cxChar+46, (int)(cyClient-2.30*cyChar),
                5+14*cxChar+3*cxChar/2+46, cyClient-3*cyChar-1);
            FillRect(hdc, &rc, hBrush);

 			// Deactivate Options/Status Bar/Phase All menu item
			EnableMenuItem(hMenu, IDC_PHASERESET, MF_GRAYED);
       }
    }
    else
    {
        ShowWindow(hwndPhaseResetButton, SW_HIDE);
        // Phase reset button "plug" region
        SetRect(&rc, 5+14*cxChar+46, (int)(cyClient-2.30*cyChar),
            5+14*cxChar+3*cxChar/2+46, cyClient-3*cyChar-1);
        FillRect(hdc, &rc, hBrush);

		// Deactivate Options/Status Bar/Phase All menu item
		EnableMenuItem(hMenu, IDC_PHASERESET, MF_GRAYED);
    }

    if(bFreqData)
    {
        if((fFreqData[2]-fFreqData[1]+1)<MIN_DATA_FILE_SIZE)
        {
            if( (fFreqData[2]<fFreqData[0]) || (fFreqData[1]>1) )
            {
                ShowWindow(hwndFreqResetButton, SW_SHOWNORMAL);

	            // Redraw the freq reset button
	            InvalidateRect(hwndFreqResetButton, NULL, TRUE);

				// Activate Options/Status Bar/Freq All menu item
				EnableMenuItem(hMenu, IDC_FREQRESET, MF_ENABLED);
          }
            else
            {
                ShowWindow(hwndFreqResetButton, SW_HIDE);
                // Freq reset button "plug" region
                SetRect(&rc, (int)(5+cxClient/2+12*cxChar+45),
                    (int)(cyClient-2.30*cyChar),
                    (int)(5+cxClient/2+14*cxChar+3*cxChar/2+45),
                    (int)(cyClient-3*cyChar-1));
                FillRect(hdc, &rc, hBrush);
 
				// Deactivate Options/Status Bar/Freq All menu item
				EnableMenuItem(hMenu, IDC_FREQRESET, MF_GRAYED);
			}

            DrawTransparentBitmap(hdc, hFewSrcBitmap, hFewMskBitmap,
				(short)(cxClient/2+5+12*cxChar+45),
				(short)(cyClient-3.00*cyChar-2));
        }
        else if( (fFreqData[2]<fFreqData[0]) || (fFreqData[1]>1) )
        {
			DrawTransparentBitmap(hdc, hLimitSrcBitmap, hLimitMskBitmap,
				(short)(cxClient/2+5+12*cxChar+45),
				(short)(cyClient-3.00*cyChar-2));

            ShowWindow(hwndFreqResetButton, SW_SHOWNORMAL);

            // Redraw the freq reset button
            InvalidateRect(hwndFreqResetButton, NULL, TRUE);

 			// Activate Options/Status Bar/Freq All menu item
			EnableMenuItem(hMenu, IDC_FREQRESET, MF_ENABLED);
        }
        else
        {
            ShowWindow(hwndFreqResetButton, SW_HIDE);
            // Freq reset button "plug" region
            SetRect(&rc, (int)(5+cxClient/2+12*cxChar+45),
                (int)(cyClient-2.30*cyChar),
                (int)(5+cxClient/2+14*cxChar+3*cxChar/2+45),
                (int)(cyClient-3*cyChar-1));
            FillRect(hdc, &rc, hBrush);

			// Deactivate Options/Status Bar/Freq All menu item
			EnableMenuItem(hMenu, IDC_FREQRESET, MF_GRAYED);
        }
    }
    else
    {
        ShowWindow(hwndFreqResetButton, SW_HIDE);
        // Freq reset button "plug" region
        SetRect(&rc, (int)(5+cxClient/2+12*cxChar+45),
            (int)(cyClient-2.30*cyChar),
            (int)(5+cxClient/2+14*cxChar+3*cxChar/2+45),
            (int)(cyClient-3*cyChar-1));
        FillRect(hdc, &rc, hBrush);

		// Deactivate Options/Status Bar/Freq All menu item
		EnableMenuItem(hMenu, IDC_FREQRESET, MF_GRAYED);
    }

	if(bPhaseData)
	{
		EnableMenuItem(hMenu, IDC_PHASETAUSELECT, MF_ENABLED);
		EnableMenuItem(hMenu, IDC_PHASEFILESELECT, MF_ENABLED);
		EnableMenuItem(hMenu, IDC_PHASESTARTSELECT, MF_ENABLED);
		EnableMenuItem(hMenu, IDC_PHASEENDSELECT, MF_ENABLED);
	}
	else
	{
		EnableMenuItem(hMenu, IDC_PHASETAUSELECT, MF_GRAYED);
		EnableMenuItem(hMenu, IDC_PHASEFILESELECT, MF_GRAYED);
		EnableMenuItem(hMenu, IDC_PHASESTARTSELECT, MF_GRAYED);
		EnableMenuItem(hMenu, IDC_PHASEENDSELECT, MF_GRAYED);
	}

	if(bFreqData)
	{
		EnableMenuItem(hMenu, IDC_FREQTAUSELECT, MF_ENABLED);
		EnableMenuItem(hMenu, IDC_FREQFILESELECT, MF_ENABLED);
		EnableMenuItem(hMenu, IDC_FREQSTARTSELECT, MF_ENABLED);
		EnableMenuItem(hMenu, IDC_FREQENDSELECT, MF_ENABLED);
	}
	else
	{
		EnableMenuItem(hMenu, IDC_FREQTAUSELECT, MF_GRAYED);
		EnableMenuItem(hMenu, IDC_FREQFILESELECT, MF_GRAYED);
		EnableMenuItem(hMenu, IDC_FREQSTARTSELECT, MF_GRAYED);
		EnableMenuItem(hMenu, IDC_FREQENDSELECT, MF_GRAYED);
	}

	// Control avtivation of Audio item in Utilities menu
	if(bDataType==FREQ_DATA)
	{
		if(bFreqData)
		{
			EnableMenuItem(hMenu, IDM_AUDIO, MF_ENABLED);
		}
		else
		{
			EnableMenuItem(hMenu, IDM_AUDIO, MF_GRAYED);
		}
	}
	else // Phase data
	{
		if(bPhaseData)
		{
			EnableMenuItem(hMenu, IDM_AUDIO, MF_ENABLED);
		}
		else
		{
			EnableMenuItem(hMenu, IDM_AUDIO, MF_GRAYED);
		}
	}

	// Display analysis limit #s
    DisplayLims();

	// Draw phase & freq data plots
	DrawDataPlots(bNewData);

	// Validate plots so they don't get overwritten by default
	// Windows painting
	GetClientRect(hwndPhasePlot, &rc);
	ValidateRect(hwndPhasePlot, NULL);
	GetClientRect(hwndFreqPlot, &rc);
	ValidateRect(hwndFreqPlot, NULL);

	// Write date message
	WriteDateMessage();

	#ifdef LITE_VER
	if(bShowPlot) // XGA or higher screen resolution
	{
		WriteAd(
			"This complementary Lite version of the Stable program "
			"for frequency stability analysis",
			"is provided by Datum, Inc., Beverly, MA 01915 USA.",
			"See the Hamilton Technical Services and Datum web sites "
			"for more information.");
	}
	else // SVGA or lower screen resolution
	{
		WriteAd("This complementary Lite version of the",
			    "Stable program is provided by Datum, Inc.",
				"Frequency and Time Systems, Beverly, MA.");
	}
	#endif
   
	// Free device context 
	ReleaseDC(hwnd, hdc);
    return;
}

/*****************************************************************************/
/*                                                                           */
/*             Function to write progress message in status bar              */
/*                                                                           */
/*      Revision record:                                                     */
/*          10/08/96    Added GPCWINYieldControl() to allow aborting         */
/*			03/08/03	Removed WINAPI from function type                    */
/*			01/25/06	Added bUpdateMsg param to control message writing    */
/*                                                                           */
/*****************************************************************************/
void WriteProgressMessage(int nPercentDone, char *szMessage, BOOL bUpdateMsg)
{
    #define PROGRESSBAR_SIZE 20
	
    // Local variables
    static RECT rProgressRect;
    static RECT rMessageRect;
    HDC hdc;
    char szBuffer[BUFFER_SIZE+1];           // Text buffer
    char szProgress[PROGRESSBAR_SIZE+1];    // Progress bar string

	// Get main window device context
    hdc=GetDC(hwndMain);

    // Select font into device context
	SelectObject(hdc, hFont);
    
	// Initializations during 1st call with nPercentDone=0
	// Update message if bUpdateMsg TRUE
    if((nPercentDone==0) || (bUpdateMsg))
    {
        // Define rect to full message area
        SetRect(&rMessageRect, 0, (int)(cyClient-3.25*cyChar-2.5*cyChar/2.5),
            (int)(cxClient), (int)(cyClient-3.25*cyChar-2));

		// Clear the message area
		FillRect(hdc, &rMessageRect, hBrush);
 
		// Draw separator line at center of screen
        MoveToEx(hdc, cxClient/2, (int)(cyClient-3.25*cyChar-2), NULL);
        LineTo(hdc, cxClient/2, (int)(cyClient-3.25*cyChar-2.5*cyChar/2.5-2));

        // Lines at right side of progress bar
        MoveToEx(hdc, (int)(cxClient/2+14.5*cxChar+3),
            (int)(cyClient-3.25*cyChar-2), NULL);
        LineTo(hdc, (int)(cxClient/2+14.5*cxChar+3),
            (int)(cyClient-3.25*cyChar-2.5*cyChar/2.5-2));
        MoveToEx(hdc, (int)(cxClient/2+14.5*cxChar+3),
            (int)(cyClient-3.25*cyChar-2), NULL);
        LineTo(hdc, (int)(cxClient/2+14.5*cxChar+3),
            (int)(cyClient-3.25*cyChar-2.5*cyChar/2.5-2));

        // Define message rect
        SetRect(&rMessageRect, 2, (int)(cyClient-3.25*cyChar-2.5*cyChar/2.5),
            (int)(cxChar*30), (int)(cyClient-3.25*cyChar));

        // Change text background color to gray
        SetBkColor(hdc, GetSysColor(COLOR_BTNFACE));

        // Show message
        ExtTextOut(hdc, 2, (int)(cyClient-3.25*cyChar-2.5*cyChar/2.5),
            ETO_CLIPPED, &rMessageRect, szMessage, strlen(szMessage), NULL);

        // Redefine message rect
        SetRect(&rMessageRect, (int)(cxClient/2+14.5*cxChar+4),
            (int)(cyClient-3.25*cyChar-2.5*cyChar/2.5),
            (int)(cxClient/2+14.5*cxChar+4+cxChar*5),
			(int)(cyClient-3.25*cyChar));

        // Define progress bar rect
        SetRect(&rProgressRect, (int)(cxClient/2+3),
            (int)(cyClient-3.25*cyChar-2.5*cyChar/2.5+3),
            (int)(cxClient/2+cxChar*13), (int)(cyClient-3.25*cyChar-3));
	}

    // Yield control to allow aborting
    YieldControl();

    // Prepare progress bar string
    // This string has PROGRESSBAR_SIZE spaces
    strlcpy(szProgress, "                    ", sizeof(szProgress));
    strnset(szProgress, 0x6c, nPercentDone/5); // Was 0x10
    // Compose message
    if(nPercentDone>0)
    {
        _snprintf(szBuffer, sizeof(szBuffer), "%3d%%", nPercentDone);
    }
    else
    {
        strlcpy(szBuffer, "", sizeof(szBuffer));
    }

    // Change text background color to gray
    SetBkColor(hdc, GetSysColor(COLOR_BTNFACE));

	// Show message
    ExtTextOut(hdc, (int)(cxClient/2+14.5*cxChar+5),
        (int)(cyClient-3.25*cyChar-2.5*cyChar/2.5),
        ETO_CLIPPED, &rMessageRect, szBuffer, strlen(szBuffer), NULL);

    // Set text color to pure red
    SetTextColor(hdc, 0x000000FF);

    // Show progress bar string in message area of status display
    ExtTextOut(hdc, cxClient/2+4,
        (int)(cyClient-3.25*cyChar-2.5*cyChar/2.5),
        ETO_OPAQUE, &rProgressRect, szProgress, strlen(szProgress), NULL);

	// Deselect custom font before deleting it
	SelectFont(hdc, GetStockObject(SYSTEM_FONT));

	// Release main window device context
    ReleaseDC(hwndMain, hdc);
}

/*****************************************************************************/
/*                                                                           */
/*             Function to write date message in status bar                  */
/*                                                                           */
/*      Revision record:                                                     */
/*          07/18/97    Used new date functions to eliminate Hammer Library  */
/*			01/31/98	Added display of full filename                       */
/*			06/01/00	Added another timetag string that includes seconds   */
/*                                                                           */
/*****************************************************************************/
void WINAPI WriteDateMessage(void)
{
    #include <time.h>           // Header for Std C time functions & tm struct

    // Local variables
    static RECT rMessageRect;               // Message rectangle
    HDC hdc;                                // Device context
	HFONT hFont;							// Font handle
    char szBuffer[BUFFER_SIZE+1];           // Text buffer

    // Time & date variables
    static struct tm *datetime; // Standard C time structure
								// Minute=tm_min(0-59)
								// Hour=tm_hour(0-23)
                                // Day of Month=tm_mday (1-31)
                                // Month of Year=tm_mon (0-11)
                                // Year since 1900=tm_year
    static long lTime;          // Time value
    int nMonth;                 // Month of year
    int nYear;                  // Year
    int nDay;                   // Day of month
    long int lnGDate;           // Gregorian date
    long int lnJDate;           // Julian day #
    LOGFONT logfont;			// Font structure

	// Make sure hwndMain handle is still valid - won't be as program ends
	if(!hwndMain)
	{
		return;
	}

	// Create font for date message
	memset(&logfont, 0, sizeof(LOGFONT));
    
	// Fill in size of font structure to match size of text in dialog boxes
    logfont.lfHeight=16;
    
	// Specify highest quality font
	logfont.lfQuality=PROOF_QUALITY;
	// Set font family
	logfont.lfPitchAndFamily=FF_SWISS;
	// Set the facename
	strlcpy(logfont.lfFaceName, "MS Sans Serif", LF_FACESIZE);
   
	// Create font
    hFont=CreateFontIndirect(&logfont);

	// Get current date and time using Std C lib finctions
    time(&lTime);
    datetime=localtime(&lTime);
    nDay=datetime->tm_mday;                         // Today's day # (1-31)
    nMonth=datetime->tm_mon+1;                      // Month of year(1-12)
    nYear=datetime->tm_year+1900;                   // Year

    // Get MJD using Hammer Library replacement functions
    // Stated MJD limits from Jan 1, 1900 to Dec 31, 1999
    // But seems OK for any date after 1/1/1900
    lnGDate=MakeDate(nMonth, nDay, nYear);
    lnJDate=DateToJulian(lnGDate);

    lnMJD=lnJDate-2400001L;

    // Put date & MJD into buffer
    _snprintf(szBuffer, sizeof(szBuffer), "Date: %02d/%02d/%02d   MJD: %ld",
		nMonth, nDay, nYear%100, lnMJD);

    // Put date into global buffer
    _snprintf(szDate, sizeof(szDate), "Date: %02d/%02d/%02d",
		nMonth, nDay, nYear%100);

	// Put MM/DD/YY HH:MM into global buffer
    _snprintf(szMDYHM, sizeof(szMDYHM), "%02d/%02d/%02d %02d:%02d", nMonth,
		nDay, nYear%100, datetime->tm_hour, datetime->tm_min);
    
	// Put MM/DD/YY HH:MM:SS into global buffer
    _snprintf(szMDYHMS, sizeof(szMDYHMS), "%02d/%02d/%02d %02d:%02d:%02d",
		nMonth, nDay, nYear%100, datetime->tm_hour, datetime->tm_min,
		datetime->tm_sec);

    // Get main window device context
	hdc=GetDC(hwndMain);

    // Select custom font into device context
    SelectObject(hdc, hFont);

	// Line at left side of date on progress bar
    MoveToEx(hdc, cxClient-23*cxChar,
        (int)(cyClient-3.25*cyChar-2), NULL);
    LineTo(hdc, cxClient-23*cxChar,
        (int)(cyClient-3.25*cyChar-2.5*cyChar/2.5-2));

    // Define date message rect
    SetRect(&rMessageRect, cxClient-cxChar*22,
        (int)(cyClient-3.25*cyChar-2.5*cyChar/2.5),
        cxClient-2, (int)(cyClient-3.25*cyChar));

    // Change text background color to gray
    SetBkColor(hdc, GetSysColor(COLOR_BTNFACE));

    // Show date
    ExtTextOut(hdc, cxClient-cxChar*22,
        (int)(cyClient-3.25*cyChar-2.5*cyChar/2.5),
        ETO_CLIPPED, &rMessageRect, szBuffer, strlen(szBuffer), NULL);

	// Display full filename at left side of message bar
	if( (bDataType==FREQ_DATA && bFreqData==TRUE) ||
		(bDataType==PHASE_DATA && bPhaseData==TRUE) )
	{
		// Define filename message rect - full area not used by date message
		SetRect(&rMessageRect, 0,
			(int)(cyClient-3.25*cyChar-2.5*cyChar/2.5),
			cxClient-cxChar*24, (int)(cyClient-3.25*cyChar));

		// Change text background color to gray
		SetBkColor(hdc, GetSysColor(COLOR_BTNFACE));
    
		// Show filename
		ExtTextOut(hdc, 0,
			(int)(cyClient-3.25*cyChar-2.5*cyChar/2.5),
			ETO_CLIPPED, &rMessageRect, szFile, strlen(szFile), NULL);
	}

	// Deselect custom font before deleting it
	SelectFont(hdc, GetStockObject(SYSTEM_FONT));

	// Delete font
	DeleteObject(hFont);

	// Release main window device context
    ReleaseDC(hwndMain, hdc);
}

/*****************************************************************************/
/*                                                                           */
/*             Function to draw data plots on status bar                     */
/*                                                                           */
/*      Uses two plotting windows created in CreateStatusBar() and moved to  */
/*      proper location in MoveStatusBarControls().  These static text       */
/*      windows have the global handles hPhasePlot & hFreqPlot.              */
/*      Separate plotting code is used for phase and frequency data to give  */
/*      the right appearance for each data type.                             */
/*      Plotting is controlled by the global flags bPhaseData & bFreqData.   */
/*      Plot for active data type is in red, inactive in black.              */
/*      If there is no data, a message is displayed in the plot window.      */
/*      Plotting is only done when the data has changed as indicated by the  */
/*      global flags bPhaseDataChanged & bFreqDataChanged.                   */
/*      Plotting is also skipped if client area is too small for OK plot.    */
/*      An error trap is included to handle zero range which writes an error */
/*      message in the plot window.                                          */
/*      No fancy plot scales are used or shown. Just uses simple 4x4 grid    */
/*      scaled to the full x & y data range.                                 */
/*      Purpose is just to give a rough picture of the data.                 */
/*      Plotting is inhibited by a global config flag bStatusPlot.           */
/*      Plot area also responds to mouse clicks:  Clicking inactive data     */
/*      makes it the active data type.  Clicking one of the four x-axis      */
/*      divisions zooms the data in and changes the analysis limits.  The    */
/*      full analysis limit button is then used to unzoom the plot.          */
/*                                                                           */
/*      Revision record:                                                     */
/*          01/06/98    Created                                              */
/*			09/07/98	Added code for bitmap usage                          */
/*			09/08/98	Used phase plotting method for large freq data sets  */
/*			09/10/98	Added bNewData argument                              */
/*			02/11/00	Added code to hide data plots when bShowPlot=0       */
/*			12/07/00	Fixed hdc errors when no room for plots              */
/*                                                                           */
/*****************************************************************************/
void WINAPI DrawDataPlots(BOOL bNewData)
{
	// Local macros
	#define POINTLIMIT 256

	// Local variables
	// Windows types
    HDC hdcPhasePlot;               // Handle to phase plot device context
    HDC hdcFreqPlot;                // Handle to freq plot device context
	HDC hdcMain;					// Handle to main window device context
    RECT rect;                      // Plot rectangle
	HPEN hRedPen;					// Handle to red pen
    HPEN hBlackPen;					// Handle to black (stock) pen
	// Ints
	int i;                          // Data index
    int j;                          // X-axis index
    int nLastY;                     // Last point plotted
    int nPoints;                    // # data points to plot
    // F_TYPEs
	F_TYPE fMin, fMax;              // Data min & max values
    F_TYPE fRange;                  // Plot range
    F_TYPE fXScale, fYScale;        // Plot scale factors
	// Bitmap variables
	HDC hdcPhaseMem;
	HDC hdcFreqMem;
	int xSize;
	int ySize;

	// Get handle to phase plot device context
    hdcPhasePlot=GetDC(hwndPhasePlot);

    // Get handle to freq plot device context
    hdcFreqPlot=GetDC(hwndFreqPlot);

    // Get handle to main window device context
    hdcMain=GetDC(hwndMain);

	// The purpose of this code is to hide the status plot area
	// of the status bar when the bShowPlot flag is FALSE
	// because the main window has been made too small to support the plots
	if(!bShowPlot)
	{
		// MessageBeep(1);  // For testing
		// Hide data plot windows
		ShowWindow(hwndPhasePlot, SW_HIDE);
		ShowWindow(hwndFreqPlot, SW_HIDE);

		// Fill status bar plot holes if they are killed
		SetRect(&rc, SW_PPT_X, SW_PPT_Y, (SW_PPT_X)+(SW_PPT_W),
			(SW_PPT_Y)+(SW_PPT_H));
		FillRect(hdcMain, &rc, hBrush);
		SetRect(&rc, SW_FPT_X, SW_FPT_Y, (SW_FPT_X)+(SW_FPT_W),
			(SW_FPT_Y)+(SW_FPT_H));
		FillRect(hdcMain, &rc, hBrush);

		// Release plot window device contexts
		ReleaseDC(hwndPhasePlot, hdcPhasePlot);
		ReleaseDC(hwndFreqPlot, hdcFreqPlot);
		ReleaseDC(hwndMain, hdcMain);
		
		return;
	}
	else  // Restore data plot windows if main window gets bigger again
	{
		// Show data plot windows
		ShowWindow(hwndPhasePlot, SW_SHOW);
		ShowWindow(hwndFreqPlot, SW_SHOW);
	}


	// Create red & black pens
	hRedPen=CreatePen(PS_SOLID, 1, 0x000000FF);
	hBlackPen=GetStockObject(BLACK_PEN);

    // Display message if no phase data and plot activated
    if( (!bPhaseData) && (bStatusPlot) )
    {
        SetWindowText(hwndPhasePlot, "\nNo Phase Data");
    }
    
    // Display blank message if no phase data and plot deactivated
	if( (!bPhaseData) && (!bStatusPlot) )
    {
        SetWindowText(hwndPhasePlot, "");
    }

    // Display message if phase data
	// and either plot deactivated or phase redraw
	if( (bPhaseData) && ( (!bStatusPlot) || (bPhaseRedraw) ) )
    {
        SetWindowText(hwndPhasePlot, "\nPhase Data");
    }

    // Display message if no freq data and plot activated
    if( (!bFreqData) & (bStatusPlot) )
    {
        SetWindowText(hwndFreqPlot, "\nNo Freq Data");
    }

    // Display message if no freq data and plot activated
    if( (!bFreqData) & (bStatusPlot) )
    {
        SetWindowText(hwndFreqPlot, "\nNo Freq Data");
    }

    // Display message if freq data
	// and either plot deactivated or freq redraw
    if( (bFreqData) & ( (!bStatusPlot) || (bFreqRedraw) ) )
    {
        SetWindowText(hwndFreqPlot, "\nFreq Data");
    }

    // Plot phase data
    // bPhaseRedraw flag set by any operation
	// that could require phase data
	// plot to be redrawn.  Otherwise don't do needless redraw
	// Redraw flag reset in all cases of phase status plot processing
    if( (bPhaseData) && (bStatusPlot) && (bPhaseRedraw))                
    {
		// Reset bPhaseRedraw flag 
		bPhaseRedraw=FALSE;

		// Don't redraw from data if bNewData FALSE
		// unless bitmap doesn't exist
		if( (bNewData) || (!hPhaseColorBitmap) )
		{
			// Get size of phase plot area
			GetClientRect(hwndPhasePlot, &rect);

			//  Make plot background white
			FillRect(hdcPhasePlot, &rect, GetStockObject(WHITE_BRUSH));

			// Find data range
			FindMinMax(fPhaseData, &fMin, &fMax, PHASE_DATA);

			// Calc phase plot scales
			fRange=fMax-fMin;

			// Check for zero range
			if(fRange)
			{
				// Set scales - the -1 avoids stary overrange pixels at bottom
				fYScale=(F_TYPE)(rect.bottom-rect.top-1);
				fXScale=(F_TYPE)(rect.right-rect.left-1);

				// Select red pen - always red - monochrome bitmap does black
				SelectObject(hdcPhasePlot, hRedPen);
 
				// Calc # phase data points to plot
				nPoints=(int)(fPhaseData[2]-fPhaseData[1]+1);

				// Plot phase data points
				j=0;
				for(i=(int)fPhaseData[1]-1; i<(int)fPhaseData[2]-1; i++)
				{
					// Skip gaps
                    if( (fPhaseData[i+ARRAY_OFFSET] &&
						fPhaseData[i+ARRAY_OFFSET+1]) ||
						((i==(int)fPhaseData[1]-1 ||
						i==(int)fPhaseData[2]-2)) )
					{
						// Move to current point
						MoveToEx(hdcPhasePlot,
							(int)(((F_TYPE)j/(F_TYPE)(nPoints-1))*fXScale),
							(int)(((fMax-fPhaseData[i+ARRAY_OFFSET])/
							(fRange))*(fYScale)), NULL);

						// Draw line from there to next point
						LineTo(hdcPhasePlot, (int)(((F_TYPE)(j+1)/
							(F_TYPE)(nPoints-1))*fXScale),
							(int)(((fMax-fPhaseData[i+ARRAY_OFFSET+1])/
							(fRange))*(fYScale)));
					}
					else // Gap
					{                
					;
					}
					j++;
				}

				// Select black pen for grid
				SelectObject(hdcPhasePlot, hBlackPen);

				// Draw horizontal grid
				// Note: Use (i=0; i<5; i++) for border too
				// Border wanted with plain style
				// No border for CLIENTEDGE style
				for(i=1; i<4; i++)
				{
					MoveToEx(hdcPhasePlot, 0, (int)((i/4.0)*fYScale), NULL);
					LineTo(hdcPhasePlot, (int)fXScale, (int)((i/4.0)*fYScale));
				}

				// Draw vertical grid
				// Note: Use (i=0; i<5; i++) for border too
				// Border wanted with plain style
				// No border for CLIENTEDGE style
				for(i=1; i<4; i++)
				{
					MoveToEx(hdcPhasePlot, (int)((i/4.0)*fXScale), 0, NULL);
					LineTo(hdcPhasePlot, (int)((i/4.0)*fXScale), (int)fYScale);
				}

				// Save phase plot as both color and monochrome bitmaps
				// Plot was generated in color
				// Use color for same data type, mono for other data type

				// Get size of phase plot
				xSize=rect.right-rect.left;
				ySize=rect.bottom-rect.top;

				// Create memory device context
				hdcPhaseMem=CreateCompatibleDC(hdcPhasePlot);

				// Detete existing phase plot bitmaps (if any)
				if(hPhaseColorBitmap)
				{
					DeleteObject(hPhaseColorBitmap);
				}
				if(hPhaseMonoBitmap)
				{
					DeleteObject(hPhaseMonoBitmap);
				}
				
				// Create an empty device dependent color bitmap for plot
				hPhaseColorBitmap=CreateCompatibleBitmap(hdcPhasePlot, xSize,
					ySize);

				// Create an empty device dependent monochrome bitmap for plot
				hPhaseMonoBitmap=CreateCompatibleBitmap(hdcPhaseMem, xSize,
					ySize);

				// Select color bitmap into memory device context
				SelectObject(hdcPhaseMem, hPhaseColorBitmap);

				// MessageBeep(1); // For testing

				// Copy pixels from screen to memory
				BitBlt(hdcPhaseMem, 0, 0, xSize, ySize, hdcPhasePlot, 0, 0,
					SRCCOPY);

				// Select monochrome bitmap into memory device context
				SelectObject(hdcPhaseMem, hPhaseMonoBitmap);

				// MessageBeep(1); // For testing

				// Copy pixels from screen to memory
				BitBlt(hdcPhaseMem, 0, 0, xSize, ySize, hdcPhasePlot, 0, 0,
					SRCCOPY);

				// Refresh screen from bitmap in order to get right color
				if(bDataType==PHASE_DATA)
				{
					// Select color bitmap into memory device context
					SelectObject(hdcPhaseMem, hPhaseColorBitmap);
				}
				else // Freq data
				{
					// Select monochrome bitmap into memory device context
					SelectObject(hdcPhaseMem, hPhaseMonoBitmap);
				}

				// Display bitmap
				BitBlt(hdcPhasePlot, 0, 0, xSize, ySize, hdcPhaseMem, 0, 0,
					SRCCOPY);

				// Delete memory device context
				DeleteDC(hdcPhaseMem);
			}
			else
			{
				// Display error message instead of plot
				GetClientRect(hwndPhasePlot, &rect);
				SetWindowText(hwndPhasePlot, "\nZero Range");
				FrameRect(hdcPhasePlot, &rect, GetStockObject(BLACK_BRUSH));
			}
		}
		else
		// Phase data same - use bitmap to redraw plot
		{
			// Create memory device context
			hdcPhaseMem=CreateCompatibleDC(hdcPhasePlot);

			// Refresh screen from bitmap
			if(bDataType==PHASE_DATA)
			{
				// Select color bitmap into memory device context
				SelectObject(hdcPhaseMem, hPhaseColorBitmap);
			}
			else // Freq data
			{
				// Select monochrome bitmap into memory device context
				SelectObject(hdcPhaseMem, hPhaseMonoBitmap);
			}

			// Get size of phase plot area
			GetClientRect(hwndPhasePlot, &rect);

			// Get size of phase plot
			xSize=rect.right-rect.left;
			ySize=rect.bottom-rect.top;

			// Display bitmap
			BitBlt(hdcPhasePlot, 0, 0, xSize, ySize, hdcPhaseMem, 0, 0,
				SRCCOPY);

			// Delete memory device context
			DeleteDC(hdcPhaseMem);
		}
	}

    // Plot freq data
    // bFreqRedraw flag set by any operation that could require freq data
	// plot to be redrawn.  Otherwise don't do needless redraw         
	// Redraw flag reset in all cases of freq status plot processing
    if( (bFreqData) && (bStatusPlot) && (bFreqRedraw) )
    {
		// Reset bFreqRedraw flag
		bFreqRedraw=FALSE;

		// Don't redraw from data if bNewData FALSE unless bitmap doesn't exist
		if( (bNewData) || (!hFreqColorBitmap) )
		{

			// Get size of plot area
			GetClientRect(hwndFreqPlot, &rect);

			// Make plot background white
			FillRect(hdcFreqPlot, &rect, GetStockObject(WHITE_BRUSH));
 
			// Find data range
			FindMinMax(fFreqData, &fMin, &fMax, FREQ_DATA);

			// Calc freq plot scales
			fRange=fMax-fMin;

			// Check for zero range
			if(fRange)
			{
				// Set scales - the -1 avoids stary overrange pixels at bottom
				fYScale=(F_TYPE)(rect.bottom-rect.top-1);
				fXScale=(F_TYPE)(rect.right-rect.left-1);

				// Select red pen - always red - monochrome bitmap does black
				SelectObject(hdcFreqPlot, hRedPen);
 
				// Calc # freq data points to plot
				nPoints=(int)(fFreqData[2]-fFreqData[1]+1);

				if(nPoints<POINTLIMIT)
				{
					// Plot freq data points as freq data with horizontal lines
					j=0;
					nLastY=0;
					for(i=(int)fFreqData[1]-1; i<(int)fFreqData[2]-1; i++)
					{
						// Skip gaps
						if(fFreqData[i+ARRAY_OFFSET] &&
							fFreqData[i+ARRAY_OFFSET+1])
						{
							// Move to current point
							MoveToEx(hdcFreqPlot,
								(int)(((F_TYPE)j/(F_TYPE)nPoints)*fXScale),
								(int)(((fMax-fFreqData[i+ARRAY_OFFSET])/
								(fRange))*(fYScale)), NULL);

							// Draw horiz line from there to next point
							LineTo(hdcFreqPlot,
								(int)(((F_TYPE)(j+1)/(F_TYPE)nPoints)*fXScale),
								(int)(((fMax-fFreqData[i+ARRAY_OFFSET])/
								(fRange))*(fYScale)));

							// Draw vert line from there to next point
							nLastY=(int)(((fMax-fFreqData[i+ARRAY_OFFSET+1])/
								(fRange))*(fYScale));
							LineTo(hdcFreqPlot,
								(int)(((F_TYPE)(j+1)/(F_TYPE)nPoints)*
								fXScale), nLastY);
						}
						else // Gap
						{
							nLastY=0;
							if(fFreqData[i+ARRAY_OFFSET])
							{
								// Draw horiz line from there to next point
								LineTo(hdcFreqPlot,
									(int)(((F_TYPE)(j+1)/(F_TYPE)nPoints)*
									fXScale),
									(int)(((fMax-fFreqData[i+ARRAY_OFFSET])/
									(fRange))*(fYScale)));
							}
						}
						j++;
					}
                
					// Draw final horiz line - test added 09/14/98
					// Avoids trouble with zoomed gaps
					if(nLastY)
					{
						LineTo(hdcFreqPlot,
							(int)(((F_TYPE)(j+1)/(F_TYPE)nPoints)*fXScale),
							nLastY);
					}
				}
				else
				{
					// Plot freq data as phase data if # points>POINTLIMIT
					// This speeds up freq data plotting by x2 & looks same
					j=0;
					for(i=(int)fFreqData[1]-1; i<(int)fFreqData[2]-1; i++)
					{
						// Skip gaps
						if(fFreqData[i+ARRAY_OFFSET] &&
							fFreqData[i+ARRAY_OFFSET+1])
						{
							// Move to current point
							MoveToEx(hdcFreqPlot,
								(int)(((F_TYPE)j/(F_TYPE)(nPoints-1))*fXScale),
								(int)(((fMax-fFreqData[i+ARRAY_OFFSET])/
								(fRange))*(fYScale)), NULL);

							// Draw line from there to next point
							LineTo(hdcFreqPlot, (int)(((F_TYPE)(j+1)/
								(F_TYPE)(nPoints-1))*fXScale),
								(int)(((fMax-fFreqData[i+ARRAY_OFFSET+1])/
								(fRange))*(fYScale)));
						}
						else // Gap
						{                
							;
						}
						j++;
					}
				}

				// Select black pen for grid
				SelectObject(hdcFreqPlot, hBlackPen);

				// Draw horizontal grid
				// Note: Use (i=0; i<5; i++) for border too
 				// Border wanted with plain style
				// No border for CLIENTEDGE style
				for(i=1; i<4; i++)
				{
					MoveToEx(hdcFreqPlot, 0, (int)((i/4.0)*fYScale), NULL);
					LineTo(hdcFreqPlot, (int)fXScale, (int)((i/4.0)*fYScale));
				}

				// Draw vertical grid
				// Note: Use (i=0; i<5; i++) for border too
				// Border wanted with plain style
				// No border for CLIENTEDGE style
				for(i=1; i<4; i++)
				{
					MoveToEx(hdcFreqPlot, (int)((i/4.0)*fXScale), 0, NULL);
					LineTo(hdcFreqPlot, (int)((i/4.0)*fXScale), (int)fYScale);
				}

				// Save freq plot as both color and monochrome bitmaps
				// Plot was generated in color
				// Use color for same data type, mono for other data type

				// Get size of freq plot
				xSize=rect.right-rect.left;
				ySize=rect.bottom-rect.top;

				// Create memory device context
				hdcFreqMem=CreateCompatibleDC(hdcFreqPlot);

				// Detete existing freq plot bitmaps (if any)
				if(hFreqColorBitmap)
				{
					DeleteObject(hFreqColorBitmap);
				}
				if(hFreqMonoBitmap)
				{
					DeleteObject(hFreqMonoBitmap);
				}

				// Create an empty device dependent color bitmap for plot
				hFreqColorBitmap=CreateCompatibleBitmap(hdcFreqPlot, xSize,
					ySize);

				// Create an empty device dependent monochrome bitmap for plot
				hFreqMonoBitmap=CreateCompatibleBitmap(hdcFreqMem, xSize,
					ySize);

				// Select color bitmap into memory device context
				SelectObject(hdcFreqMem, hFreqColorBitmap);

				// Copy pixels from screen to memory
				BitBlt(hdcFreqMem, 0, 0, xSize, ySize, hdcFreqPlot, 0, 0,
					SRCCOPY);

				// Select monochrome bitmap into memory device context
				SelectObject(hdcFreqMem, hFreqMonoBitmap);

				// Copy pixels from screen to memory
				BitBlt(hdcFreqMem, 0, 0, xSize, ySize, hdcFreqPlot, 0, 0,
					SRCCOPY);

				// Refresh screen from bitmap in order to get right color
				if(bDataType==FREQ_DATA)
				{
					// Select color bitmap into memory device context
					SelectObject(hdcFreqMem, hFreqColorBitmap);
				}
				else // Phase data
				{
					// Select monochrome bitmap into memory device context
					SelectObject(hdcFreqMem, hFreqMonoBitmap);
				}

				// Display bitmap
				BitBlt(hdcFreqPlot, 0, 0, xSize, ySize, hdcFreqMem, 0, 0,
					SRCCOPY);

				// Delete memory device context
				DeleteDC(hdcFreqMem);
			}
			else
			{
				// Display error message instead of plot
				GetClientRect(hwndFreqPlot, &rect);
				SetWindowText(hwndFreqPlot, "\nZero Range");
				FrameRect(hdcFreqPlot, &rect, GetStockObject(BLACK_BRUSH));
			}
		}
		else
		// Freq data same - use bitmap to redraw plot
		{
			// Create memory device context
			hdcFreqMem=CreateCompatibleDC(hdcFreqPlot);

			// Refresh screen from bitmap
			if(bDataType==FREQ_DATA)
			{
				// Select color bitmap into memory device context
				SelectObject(hdcFreqMem, hFreqColorBitmap);
			}
			else // Phase data
			{
				// Select monochrome bitmap into memory device context
				SelectObject(hdcFreqMem, hFreqMonoBitmap);
			}

			// Get size of freq plot area
			GetClientRect(hwndFreqPlot, &rect);

			// Get size of phase plot
			xSize=rect.right-rect.left;
			ySize=rect.bottom-rect.top;

			// Display bitmap
			BitBlt(hdcFreqPlot, 0, 0, xSize, ySize, hdcFreqMem, 0, 0,
				SRCCOPY);

			// Delete memory device context
			DeleteDC(hdcFreqMem);
		}
    }

	// Delete red pen (black pen is stock object)
    DeleteObject(hRedPen);

	// Release plot window device contexts
    ReleaseDC(hwndPhasePlot, hdcPhasePlot);
    ReleaseDC(hwndFreqPlot, hdcFreqPlot);
	ReleaseDC(hwndMain, hdcMain);
}

/*****************************************************************************/
/*                                                                           */
/*             Function to display analysis limit #s in status bar           */
/*                                                                           */
/*      Revision record:                                                     */
/*          01/17/98    Created                                              */
/*                                                                           */
/*****************************************************************************/
void WINAPI DisplayLims(void)
{
    // Local variables
    // Chars
    char szBuffer[BUFFER_SIZE+1];       // General text buffer
    
    // Update phase analysis limit # display
    if(bPhaseData)
    {
        _snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
			(int)(fPhaseData[2]-fPhaseData[1]+1));
    }
    else
    {
        strlcpy(szBuffer, "", sizeof(szBuffer));
    }
    SetWindowText(hwndPhaseLim, szBuffer);

    // Update freq analysis limit # display
    if(bFreqData)
    {
        _snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
			(int)(fFreqData[2]-fFreqData[1]+1));
    }
    else
    {
        strlcpy(szBuffer, "", sizeof(szBuffer));
    }
    SetWindowText(hwndFreqLim, szBuffer);
}

/*****************************************************************************/
/*                                                                           */
/*             Function to delete status bar font                            */
/*                                                                           */
/*      Revision record:                                                     */
/*          09/03/98    Created                                              */
/*                                                                           */
/*****************************************************************************/
void WINAPI DeleteStatusBarFont(void)
{
	// Delete font object
	DeleteObject(hFont);
}

/*****************************************************************************/
/*                                                                           */
/*             Function to display cursor on status bar plot                 */
/*                                                                           */
/*		Parameters:                                                          */
/*			int nPosition	Cursor position as %% of plot full scale         */
/*          BOOL bMode      0=Erase cursor, 1=Write cursor                   */
/*                                                                           */
/*      Return:				none (void)                                      */
/*                                                                           */
/*      Revision record:                                                     */
/*          09/13/98    Created                                              */
/*                                                                           */
/*****************************************************************************/
void WINAPI DisplayPlotCursor(int nPosition, BOOL bMode)
{
	// Local variables
    HDC hdcMain;                // Handle to main screen device context
    RECT rc;					// Cursor area rectangle
	HPEN hRedPen;				// Handle to red pen
    HPEN hBlackPen;				// Handle to black (stock) pen

	// Get handle to main screen device context
    hdcMain=GetDC(hwndMain);

	// Create red & black pens
	hRedPen=CreatePen(PS_SOLID, 1, 0x000000FF);
	hBlackPen=GetStockObject(BLACK_PEN);

	// Select pen color: black=normal, red=beyond limits
	if( (nPosition<0) || (nPosition>1000) )
	{
		// Select red pen
		SelectObject(hdcMain, hRedPen);
	}
	else
	{
		// Select black pen
		SelectObject(hdcMain, hBlackPen);
	}

	// Process display plot cursor function
	if(bDataType==FREQ_DATA)
	{
		// Erase freq cursor region by filling it with light gray brush
	    SetRect(&rc, SW_FPT_X-10, SW_FPT_Y-2, ((SW_FPT_X)+(SW_FPT_W)+10),
			SW_FPT_Y);
	    FillRect(hdcMain, &rc, hBrush);

		if(bMode)
		{
			// Draw freq plot cursor line
			MoveToEx(hdcMain, SW_FPT_X +2 +nPosition*(SW_FPT_W-4)/1000,
				SW_FPT_Y-2, NULL);
			LineTo(hdcMain, SW_FPT_X +2 +nPosition*(SW_FPT_W-4)/1000,
				SW_FPT_Y);
		}
	}
	else // Phase data
	{
		// Erase phase cursor region by filling it with light gray brush
	    SetRect(&rc, SW_PPT_X-10, SW_PPT_Y-2, ((SW_PPT_X)+(SW_PPT_W)+10),
			SW_PPT_Y);
	    FillRect(hdcMain, &rc, hBrush);

		if(bMode)
		{
			// Draw phase plot cursor line
			MoveToEx(hdcMain, SW_PPT_X + nPosition*SW_PPT_W/1000,
				SW_PPT_Y-2, NULL);
			LineTo(hdcMain, SW_PPT_X + nPosition*SW_PPT_W/1000, SW_PPT_Y);
		}
	}

	// Delete red pen (black pen is stock object)
    DeleteObject(hRedPen);

	// Release main screen device contexts
    ReleaseDC(hwndMain, hdcMain);
}

/*****************************************************************************/
