/*****************************************************************************/
/*                                                                           */
/*                            STABLE_Z.C                                     */
/*                                                                           */
/*      Functions to read & parse TS-3020 archive file                       */
/*      Extracts phase data for clock and reference channels                 */
/*                                                                           */
/*      ReadArchiveFile()   ArchiveDlgProc()                                 */
/*                                                                           */
/*      The archive file has lines that start with either a $ or #.          */
/*      The $ lines have 1 additional field, the MJD for the data on the     */
/*      following # lines.  The MJD is a 18-character string comprised of    */
/*      a 5-digit integer part, a decimal point, and a 12-digit fraction-    */
/*      al part.  A sample $ line is as follows ($ is in 1st column):        */
/*                                                                           */
/*      $ 48888.612500000000                                                 */
/*                                                                           */
/*      The # lines have 5 additional fields, a 1 or 2-digit channel #       */
/*      between 1 and MAXCHAN, a 20-char clock ID string, and 3 numerical    */
/*      fields.  The 1st number is the nominal clock frequency in MHz,       */
/*      the "reference" frequency for that channel.  It is expressed         */
/*      as a 1 or 2-digit integer part, a decimal point, and a 15-digit      */
/*      fractional part, thus forming a 17 or 18-character string.  The      */
/*      2nd and 3rd numbers are clock phase in cycles wrt the offset ref-    */
/*      erence oscillator.  The 2nd number is a rounded integer tag,         */
/*      while the 3rd has the full precision with a decimal point and a      */
/*      12-digit fractional part.  The integer parts of these numbers grow   */
/*      with the beat note cycles of the offset oscillator, about 10 per     */
/*      second, so a 9-digit field is sufficient for about 3 years.  Thus    */
/*      the 2nd number string can have 9 characters, and the 3rd number      */
/*      string can have 22 characters.  The number fields are each separat-  */
/*      ed by 2 spaces.  Including spaces, the maximum length of a # line    */
/*      is 79 characters.  A sample # line is as follows (# is in 1st col):  */
/*                                                                           */
/*      # Clock S/N 0123456789 5.000000000000000  12007355  12007354.***     */
/*          where *** denotes 12 more digits after the decimal point         */
/*                                                                           */
/*      Raw clock time data (in seconds) is calculated by dividing the       */
/*      phase number by the nominal frequency (in MHz).  Corrected clock     */
/*      time is obtained from that value by subtracting the corresponding    */
/*      value for the reference channel.  Fractional frequency is, of        */
/*      course, the difference between two adjacent time values divided by   */
/*      the time interval between them.                                      */
/*                                                                           */
/*      Revision record:                                                     */
/*          04/24/98    Created from CLKPLT.C                                */
/*          04/30/98	Now working                                          */
/*			07/30/07    Changed to safe string functions                     */
/*                                                                           */
/* (c) 1998-2007 W. Riley  Hamilton Technical Services  All Rights Reserved  */
/*                                                                           */
/*****************************************************************************/

#define STRICT                      // For Win32 compatibility

#include "stable_w.h"               // STABLE_W header file

#ifndef LITE_VER

// Pointers to data arrays with file scope
F_TYPE *fX;                         // Phase data array pointer
F_TYPE *fY;                         // Freq data array pointer

/*****************************************************************************/
/*                                                                           */
/*                           ReadArchiveFile()                               */
/*                                                                           */
/*      Function to extract clock data from TS-3020 archive file             */
/*                                                                           */
/*      The archive filename is given by the global string sxFile.           */
/*      The clock data will be put into both the phase or frequency          */
/*      data arrays.  No conversions, averaging, outlier removal,            */
/*      file writing, analysis or plotting is done. The usual Stable         */
/*      functions are available for that.                                    */
/*                                                                           */
/*      Parameters:                                                          */
/*          int     nSelChan        Selected channel #                       */
/*          int     nRefChan        Reference channel #                      */
/*          int		nRecord			Record # to find (0=last, -1=MJD)        */
/*          F_TYPE  fTau            Measurement interval, sec                */
/*          F_TYPE	fMJDStart		MJD to start data extraction             */
/*          F_TYPE	fMJDEnd			MJD to end data extraction               */
/*                                                                           */
/*      Return:                                                              */
/*          F_TYPE	fDataMJD		MJD where data begins                    */
/*									or 0 if error                            */
/*                                                                           */
/*      Function revision record:                                            */
/*          04/24/98    Created from CLKPLT.C.                               */
/*			06/18/99	Added nCode testing                                  */
/*			10/18/00	Discovered that can't read archive file data with    */
/*						the nCode testing.  This function has been NG for    */
/*						over a year - nobody must use it!  Problem was       */
/*						if statements followed by a semicolon - fixed.       */
/*                                                                           */
/*****************************************************************************/

F_TYPE WINAPI ReadArchiveFile(int nSelChan, int nRefChan, int nRecord,
    F_TYPE fTau, F_TYPE fFirstMJD, F_TYPE fLastMJD, F_TYPE fMJDStart,
	F_TYPE fMJDEnd) 
{
    // Local variables
	// CHARs
	char szBuffer[BUFFER_SIZE+1];       // General text buffer
	char szFileBuffer[BUFFER_SIZE+1];   // Archive file line buffer
	char szMJD[20];                     // MJD string
	char szID[22];                      // Clock ID string
	char szMeasID[22];                  // Meas clock ID string
	// char szRefID[22];                // Ref clock ID string
	char szTitle[22];                   // Clock title string
	char szMessage[BUFFER_SIZE+1];      // Message buffer
    char cSymbol;                       // First character of line: $ or # 
    char szPhaseTag[11];                // Phase tag string
    char szNomFreq[20];                 // Nominal frequency string
    char szPhaseData[24];               // Phase data string
    char szPrevMJD[20];                 // MJD string
	char szChan[4];                     // Channel # string
    char szArchiveDrive[BUFFER_SIZE];	// Filename drive
    char szArchiveDir[FILENAMESIZE];	// Filename directory
    char szArchiveName[FILE_NAME_SIZE];	// Filename name
    char szArchiveExt[BUFFER_SIZE];		// Filename extension
	// BOOLs
	BOOL bUsed;                         // Channel used flag
	BOOL bRefUsed;                      // Ref used flag
	BOOL bFirstGap=TRUE;                // First gap flag (not used??)
	BOOL bGapFlag=FALSE;                // Flag for gap being processed
	BOOL bWritefile=FALSE;              // Flag to write data file
	BOOL bCheck=FALSE;                  // Flag for bad symbol checking
	BOOL bNoRef=FALSE;                  // Flag for no reference subtraction
    BOOL bActive;                       // Channel active flag
    BOOL bMeas;                         // Channel measured flag
    BOOL bRemoved=FALSE;                // Clock removed flag
    BOOL bRefActive;                    // Ref active flag
    BOOL bRefMeas;                      // Ref measured flag
    BOOL bRefRemoved=FALSE;             // Ref removed flag
    BOOL bRev;							// Flag for ref & meas ch reversal
    BOOL bSign;							// Flag for output data sense reversal
	BOOL bTest1;						// Flag for testing
	BOOL bTest2;						// Flag for testing
	BOOL bMJDOK=FALSE;					// MJD within selected range
	BOOL bUseMJD;						// Use MJD (not record #)
	// INTs
	int nMeasRecord=0;                  // # meas file records
	int nRefRecord=0;                   // # ref file records 
	int nPoint=0;                       // # fY[] data points
	int nGap=0;                         // # gaps in group
	int nNumGap=0;                      // # of gaps in clock data
    int i;                              // Index
    int k;								// # digits in ch # corr factor
    int nChan;                          // Channel # 
    int nEnd=0;                         // Record to process
	int nPercentDone;					// Progress = % of archive file read
	int nCode;							// Return code from ResizeArrays() 
    // LONGs
    long nLineNum=0;                    // Line # of archive file
 	// F_TYPEs
    F_TYPE fTime=0;                     // Clock time
    F_TYPE fPreTime;                    // Previous clock time
    F_TYPE fRefTime=0;                  // Reference time
    F_TYPE fRefPreTime;                 // Reference previous time
    F_TYPE fNomFreq;                    // Nominal frequency value
    F_TYPE fRefFreq;                    // Reference frequency
    F_TYPE fFreq;                       // Clock fractional frequency
    F_TYPE fMJD;                        // Numeric value of szMJD
    F_TYPE fDataMJD;                    // MJD where data begins
    F_TYPE fPrevMJD=0;                  // Numeric value of szPrevMJD
	F_TYPE fTagStart=0;					// Numeric value of MJD at start
    F_TYPE fInterval;                   // Measurement interval in days
    F_TYPE *fNew;                       // New data array pointer
	// MISC
    FILE *fpArch;                       // File pointer 

    // Initializations

	// Calc meas interval in days
	fInterval=fTau/86400.0;

    // Parse nRecord
	if(nRecord<0) // MJD mode
	{
		bUseMJD=TRUE;
		nRecord=1;
	}
	else
	{
		bUseMJD=FALSE;
	}
	
	// Set Record #
    nEnd=nRecord;

	bTest1=TRUE;
	bTest2=TRUE;

	// Set nPoint to zero
	nPoint=0;

	// Assign default clock ID
	strlcpy(szMeasID, "NONAME", sizeof(szMeasID));

    // Allocate fX and fY arrays for storing phase & freq data
    // They will have # points,  start &  end cells
    // just like their main analysis counterparts
    // Use dynamic realloc as points are read from disk
    // (No good way to predict their size in advance)
 
    // Allocate memory for minimum-sized phase data array
    if((fX=(F_TYPE*)malloc(
        (MIN_ARRAY_SIZE+1+ARRAY_OFFSET+EXTRA_MEM)*sizeof(F_TYPE)))==NULL)
    {
        // Error message & bailout
        AllocErrorMessage();
        return 0.0;
    }

    // Allocate memory for minimum-sized freq data array
    if((fY=(F_TYPE*)malloc(
        (MIN_ARRAY_SIZE+ARRAY_OFFSET+EXTRA_MEM)*sizeof(F_TYPE)))==NULL)
    {
        // Error message, free mem & bailout
        AllocErrorMessage();
		free(fY);
        return 0.0;
    }

	// Reverse sense of output data if neg meas ch #
    if(nSelChan<0) 
    {
        bSign=TRUE;
        nSelChan=-nSelChan;
    }
    else
    {
        bSign=FALSE;
    }

	// Set no ref with ref chan # = 0
    if(nRefChan==0)
    {
        bNoRef=TRUE;
    }
    else
    {
        bNoRef=FALSE;
    }
        
    if(bNoRef) // make ref ch # = meas ch # 
    {
        nRefChan=nSelChan;
    }

    /* code to handle ref chan # > meas chan # */
    /* data in archive file is in numerical order by channel # */
    /* so must always read lower chan # first */
    /* whether it is ref or meas channel */
    /* set rev flag and reverse ref & neas chan #s */
    /* besides this code, other changes for this case are: */
    /*     (1) reverse sign of freq data */
    /*     (2) change which clock id is stored for display */

    if(nRefChan>nSelChan)
    {
        bRev=TRUE;

        // Swap sel & ref ch #s
        i=nRefChan;
        nRefChan=nSelChan;
        nSelChan=i;
    }
	else
	{
		bRev=FALSE;
	}

	// Flag usage:
	// Meas Ch = Ref Chan if bRev=TRUE (when ref ch # > meas ch #)
    // Meas Ch = Sel Chan if bRev=FALSE (when ref ch # < meas ch #) 
    // Seek = End if nEnd=TRUE (seek certain record #)
    // Seek = Last if nEnd=FALSE (seek last record)                                                      
    // No Ref Channel if bNoRef=TRUE (denoted by ref ch # = 0)
	// Reverse sign if bSign=TRUE (denoted by neg ref ch #)

    // Clear clock flags */
    bUsed=FALSE;
    bMeas=FALSE;
    bActive=FALSE;

    // Clear ref flags */
    bRefUsed=FALSE;
    bRefMeas=FALSE;
    bRefActive=FALSE;

    // Open archive data file
    if((fpArch=fopen(szFile, "r")) == NULL)
    {
		// Error message
        _snprintf(szBuffer, sizeof(szBuffer), "Cannot open %s\n", szFile);
        MessageBox(hwndMain, szBuffer, "Read Archive", MB_ICONEXCLAMATION);
        return 0.0; // Error exit
    }                                                              

    // Set up message
    if(bRev)
    {
        // Status message
        _snprintf(szMessage, sizeof(szMessage),
            "Reading data for Ch # %d from %s"
            , nRefChan, szFileTitle);
    }                                                              
    else
    {
        // Status message
        _snprintf(szMessage, sizeof(szMessage),
            "Reading data for Ch # %d from %s",
            nSelChan, szFileTitle);
    }                                                            

    // Display simple message in message bar
    WriteMessage(szMessage);

    // Archive file data reading starts here
    while(fgets(szFileBuffer, 81, fpArch) != NULL)    /* read 1 line of file */
    {
        sscanf(szFileBuffer, "%c", &cSymbol);                   /* scan line */
        nLineNum++;              /* archive file increment line number count */
        if(cSymbol=='#')                                  /* channel # line? */
        {
            sscanf(szFileBuffer, "%c %s", &cSymbol, szChan);
                                                               /* scan again */
            nChan=atoi(szChan);                             /* get channel # */

            if(nChan==nSelChan || nChan==nRefChan)   /* is chan one we want? */
            {
                if( (strlen(szChan)==1) )                 /* 1-digit chan #? */
                {
                    k=0;
                }                                                  
                else                                       /* 2-digit chan # */
                {
                    k=1;
                }                                                

                /* read clock szID */
                for(i=0; i<20; i++)          /* 20-character clock id string */
                {
                    szID[i]=szFileBuffer[i+4+k];
                    szID[20]='\0';
                } 
                
                if( (nChan==nSelChan) && (!bRev) )
                {
                    // Display ID of measurement channel clock
                    if(nPoint==1)
                    {
						strlcpy(szMeasID, szID, sizeof(szMeasID));
                    }
                }

                if( (nChan==nRefChan) && (bRev) )
                {
                    // Display ID of measurement channel clock
                    if(nPoint==1)
                    {
 						strlcpy(szMeasID, szID, sizeof(szMeasID));
                    }
                }
                
                /* remove '#', szChan and szID from szFileBuffer */
                /* '#' is 1 character followed by 1 space */
                /* szChan is k+1 characters followed by 1 space */
                /* szID is 20 characters followed by 1 space */
                /* so we want to remove 25+k characters from szFileBuffer */

                for(i=0; i<55-k; i++)
                {
                    szFileBuffer[i]=szFileBuffer[i+25+k];
                }                                                 

                // Parse the rest of szFileBuffer
                sscanf(szFileBuffer, "%s %s %s", szNomFreq, szPhaseTag,
                       szPhaseData);              /* scan rest of input line */

                // Process reference time
                // Ref chan data comes before corresponding clock data
                if(nChan==nRefChan)
                {
                    fRefPreTime=fRefTime;              /* save previous time */
                    fNomFreq=atof(szNomFreq);       /* get nominal frequency */
                    fRefTime=atof(szPhaseData);           /* get phase value */
  
                    if(fNomFreq)                 /* check for divide by zero */
                    {
                        fRefTime/=fNomFreq*1e6;        /* conv to time value */
                    }                                              
                    else
                    {
						// Error message
						LoadString(hInst, IDS_MESSAGE_NGREFPROC, szBuffer,
						   BUFFER_SIZE);
				        MessageBox(hwndMain, szBuffer, "Read Archive",
							MB_ICONEXCLAMATION);
                        return 0.0;
                    }                                            

                    bRefMeas=TRUE;                    /* raise measured flag */
                    bRefUsed=TRUE;                        /* raise used flag */

                    if(bRefActive==FALSE)              /* newly entered ref? */
                    {
                        if(bRefRemoved)         /* was there a previous ref? */
                        {
                            fPrevMJD=0;       /* reinitialize previous mjd # */

                            nPoint=0;               /* reset point count to 0 */
                            bFirstGap=TRUE;     /* reset first_gap flag to 1 */
                        }                                          

                        bRefRemoved=FALSE;
                        bRefActive=TRUE;                /* mark it as active */

                        if(bRev)
                        {
							nRefRecord++;
							fDataMJD=atof(szMJD);
                        }
                        else if(!bNoRef)
                        {
 							nRefRecord++;
							fDataMJD=atof(szMJD);
                        }
                    }                                             
                }                                                 

                // Calculate clock time & frequency
                if(nChan==nSelChan)
                {
                    fPreTime=fTime;                    /* save previous time */
                    fNomFreq=atof(szNomFreq);       /* get nominal frequency */
                    fTime=atof(szPhaseData);              /* get phase value */
                    if(fNomFreq)                 /* check for divide by zero */
                    {
                        fTime/=fNomFreq*1e6;           /* conv to time value */
                    }                                             
                    else
                    {
 						// Error message
						LoadString(hInst, IDS_MESSAGE_NGCLKPROC, szBuffer,
							BUFFER_SIZE);
						MessageBox(hwndMain, szBuffer, "Read Archive",
							MB_ICONEXCLAMATION);
                        return 0.0;
                    }                                           

                    // Calc clock frequency
                    fRefFreq=(fRefTime-fRefPreTime)/fTau;
                    fFreq=(fTime-fPreTime)/fTau;

                    if(bNoRef)      /* are we supposed to subtract ref data? */
                           /* are reference & measurement channels reversed? */
					{
                        if(bRev==TRUE)
                        {
                            fFreq=-fRefFreq;                 /* use ref data */
                        }
                    }
                    else
                    {
                        fFreq-=fRefFreq;                /* subtract ref data */
                    }

                    if(bSign)
                    {
                        // Reverse sign
                        fFreq=-fFreq;
                    }

                    if(bActive==TRUE && bRefActive==TRUE)         /* active? */
                    {
                        if(nGap)
                        {
                            // Process gap in data
							// This is a "normal" gap within data
							// Not space between records
                            bGapFlag=TRUE;
                            while(nGap--)
                            {
								if(bMJDOK)
								{
									// Store gap data
									nCode=ResizeArrays(nPoint);
									if(nCode==0)
									{
										return 0.0;
									}

									fY[nPoint+ARRAY_OFFSET]=0.0;
									fX[nPoint+ARRAY_OFFSET]=0.0;
									nPoint++;
								}
                            }                                                           
                        } 
						
                        if(bRev)
                        {
                            if(!bGapFlag)
                            {
 								if(bMJDOK)
								{
									// Store freq point
									nCode=ResizeArrays(nPoint);
									if(nCode==0)
									{
										return 0.0;
									}

									fY[nPoint+ARRAY_OFFSET]=-fFreq;
								}
                            }                                     
                            else
                            {
 								if(bMJDOK)
								{
									// Store freq gap
									nCode=ResizeArrays(nPoint);
									if(nCode==0)
									{
										return 0.0;
									}

									fY[nPoint+ARRAY_OFFSET]=0.0;
								}
                            }                                   

 							if(bMJDOK)
							{
								// Store phase point
								// New test 05/03/98
								if(bNoRef)
								{
									fX[nPoint+ARRAY_OFFSET]=-fTime;
								}
								else
								{
									fX[nPoint+ARRAY_OFFSET]=fRefTime-fTime;
								}
							}
                        }                                         
                        else // !bRev
                        {
                            if(!bGapFlag)
                            {
 								if(bMJDOK)
								{
									// Store freq point
									nCode=ResizeArrays(nPoint);
									if(nCode==0)
									{
										return 0.0;
									}

									fY[nPoint+ARRAY_OFFSET]=fFreq;
								}
                            }                                     
                            else
                            {
  								if(bMJDOK)
								{
									// Store freq gap
									nCode=ResizeArrays(nPoint);
									if(nCode==0)
									{
										return 0.0;
									}

									fY[nPoint+ARRAY_OFFSET]=0.0;
								}
                            }                                   

 							if(bMJDOK)
							{
								// Store phase point                            
								// New test 05/03/98
								if(bNoRef)
								{
									fX[nPoint+ARRAY_OFFSET]=fTime;
								}
								else
								{
									fX[nPoint+ARRAY_OFFSET]=fTime-fRefTime;
								}
							}
                        }                                       

                        if(bSign)
                        {
 							if(bMJDOK)
							{
								// Reverse sign
								fX[nPoint+ARRAY_OFFSET]=
									-fX[nPoint+ARRAY_OFFSET];
							}
                        }

 						if(bMJDOK)
						{
							nPoint++;
						}

                        bGapFlag=FALSE;
                    }                                             

                    bMeas=TRUE;                       /* raise measured flag */
                    bUsed=TRUE;                           /* raise used flag */

                    if(bActive==FALSE)               /* newly entered clock? */
                    {
                        if(bRemoved)          /* was there a previous clock? */
                        {
                            nPoint=0;               /* reset point count to 0 */
                            bFirstGap=TRUE;     /* reset first_gap flag to 1 */
                            fPrevMJD=0;       /* reinitialize previous mjd # */
                        }                                         
                        bRemoved=FALSE;

                        if( (bRev) && (!bNoRef) )
                        {
							nMeasRecord++;

 							// Save MJD where data begins
							fDataMJD=atof(szMJD);

                        }
                        else if(!bRev)
                        {
 							nMeasRecord++;

 							// Save MJD where data begins
							fDataMJD=atof(szMJD);
                        }
                    }                                             

                    if(bActive==FALSE && bRefActive==TRUE) /* newly entered? */
                    {
                        if(bRemoved)          /* was there a previous clock? */
                        {
                            nPoint=0;              /* reset point count to 0 */
                            bFirstGap=TRUE;     /* reset first_gap flag to 1 */
                            fPrevMJD=0;       /* reinitialize previous mjd # */
                        }                                         
                        bRemoved=FALSE;
                        bActive=TRUE;                   /* mark it as active */

                        if(bRev)
                        {
 							if(bMJDOK)
							{
								// Store 1st phase data point
								nCode=ResizeArrays(nPoint);
								if(nCode==0)
								{
									return 0.0;
								}

								fX[nPoint+ARRAY_OFFSET]=fRefTime-fTime;
								fY[nPoint+ARRAY_OFFSET]=0.0;
							}
                        }                                         
                        else
                        {
 							if(bMJDOK)
							{
								// Store 1st phase data point
								nCode=ResizeArrays(nPoint);
								if(nCode==0)
								{
									return 0.0;
								}

								fX[nPoint+ARRAY_OFFSET]=fTime-fRefTime;
								fY[nPoint+ARRAY_OFFSET]=0.0;
							}
                        }                                       

                        if(bSign)
                        {
  							if(bMJDOK)
							{
								// Reverse sign
								fX[nPoint+ARRAY_OFFSET]=
									-fX[nPoint+ARRAY_OFFSET];
							}
                        }

 						if(bMJDOK)
						{
							nPoint++;
						}
                    }                                             
                }                                                 
            }                                                     
        }                                                         

        if(cSymbol=='$')                                        /* MJD line? */
        {
            sscanf(szFileBuffer, "%c %s", &cSymbol, szMJD);
                                                          /* scan line again */

			// Use current MJD % read as status indicator
			fMJD=atof(szMJD);
			nPercentDone=(int)((fMJD-fFirstMJD)*100/(fLastMJD-fFirstMJD));
			_snprintf(szBuffer, sizeof(szBuffer), "%d", nPercentDone);
			WriteProgressMessage(nPercentDone, szMessage, FALSE);

			// Code for MJD control
			if(fMJD>=fMJDStart-fInterval/2)
			{
				if(bTest1) // Do this 1st time only
				{
					bMJDOK=TRUE;
					fTagStart=atof(szMJD);
					bTest1=FALSE;
				}
			}

			if(fMJD>=fMJDEnd+fInterval/2)
			{
				if(bTest2) // Do this 1st time only
				{
					bMJDOK=FALSE;
					bTest2=FALSE;
				}
			}
			
			if( (bActive==TRUE) && (bMeas==FALSE) ||             /* clock or */
				(bRefActive==TRUE) && (bRefMeas==FALSE) )    /* ref missing? */
            {
				// Detect a gap point between records in MJD mode
				if(bUseMJD) 
				{
					if(bMJDOK)
					{
						// Store gap data
						nCode=ResizeArrays(nPoint);
						if(nCode==0)
						{
							return 0.0;
						}

						fY[nPoint+ARRAY_OFFSET]=0.0;
						fX[nPoint+ARRAY_OFFSET]=0.0;
						nPoint++;
					}
				}
			}

			if( (bActive==TRUE) && (bMeas==FALSE) )        /* clock removed? */
            {
				// Skip if using MJD limits
				if(!bUseMJD)
				{
					bActive=FALSE;                          /* deactivate it */
					bRemoved=TRUE;                 /* set clock removed flag */

					if( (bRev) && (!bNoRef) )
					{
					}
					else if(!bRev)
					{
						if(nMeasRecord==nEnd)
						{
							break;
						}
					}
				}

            }                                                     

            if( (bRefActive==TRUE) && (bRefMeas==FALSE) )    /* ref removed? */
            {
				// Skip if using MJD limits
				if(!bUseMJD)
				{
					bRefActive=FALSE;                       /* deactivate it */
					bRefRemoved=TRUE;                /* set ref removed flag */

					if(bRev)
					{
						if(nMeasRecord==nEnd)
						{
							break;
						}
					}                                             
				}
            }                                                     

            bMeas=FALSE;                           /* clear measurement flag */
            bRefMeas=FALSE;                    /* clear ref measurement flag */

            fMJD=atof(szMJD);                      /* calc numeric MJD value */

			
			if( ( bActive && fPrevMJD!=0) &&
                ((fMJD-fPrevMJD-(fInterval/2))>fInterval) )
			{
                nGap=(int)(((fMJD-fPrevMJD+(fInterval/2))/fInterval)-1);
                nNumGap+=nGap;
            }                                                     
            else
            {
                nGap=0;
            }
					
			// Save previous MJD string and value
            strlcpy(szPrevMJD, szMJD, sizeof(szPrevMJD));            
            fPrevMJD=fMJD; 
        }                                                    
    }
    
    // Close archive file
    fclose(fpArch);

    // Check if record not found
    if( (nEnd) && (nMeasRecord!=nEnd) )
    {
		// Error message
        _snprintf(szBuffer, sizeof(szBuffer), "Record %d Not Found", nEnd);
        MessageBox(hwndMain, szBuffer, "Read Archive", MB_ICONEXCLAMATION);
        return 0.0; // Error exit */
    }
    
    // Check if data not found
    if(nPoint==0)
    {
		// Error message
        LoadString(hInst, IDS_MESSAGE_NODATA, szBuffer, BUFFER_SIZE);
        MessageBox(hwndMain, szBuffer, "Read Archive", MB_ICONEXCLAMATION);
        return 0.0; // Error exit */
    }

    // Write archive data to both main analysis & timetag arrays
    // All we need to do generate timetags is use start MJD and tau interval
    // Phase & freq (almost) the same (1 more phase point)

    // Realloc phase data
    // Have nPoint phase points
    if((fNew=(F_TYPE*)realloc(fPhaseData, (nPoint+ARRAY_OFFSET+EXTRA_MEM)
        *sizeof(F_TYPE)))==NULL)
    {
		// Dsp msg, free mem & return error code
        AllocErrorMessage();
	    free(fX);
		free(fY);
		return 0.0;
    }
    else
    {
        fPhaseData=fNew;
    }

    if(bDataType==PHASE_DATA)
    {
        fData=fPhaseData;
    }

    // Realloc phase timetags
    // Have nPoint phase points
    if((fNew=(F_TYPE*)realloc(fPhaseTimetags, (nPoint+ARRAY_OFFSET+EXTRA_MEM)
        *sizeof(F_TYPE)))==NULL)
    {
		// Dsp msg, free mem & return error code
        AllocErrorMessage();
	    free(fX);
		free(fY);
		return 0.0;
    }
    else
    {
        fPhaseTimetags=fNew;
    }

    if(bDataType==PHASE_DATA)
    {
        fTags=fPhaseTimetags;
    }


 	// Copy phase data (not header)
    // Phase data index goes from 0 to nPoint-1
    for(i=0; i<nPoint; i++)
    {
        fPhaseData[i+ARRAY_OFFSET]=fX[i+ARRAY_OFFSET];
    }

	// Set 1st MJD value to where data actually starts
	if(bUseMJD)
	{
		fDataMJD=fTagStart;
	}

	// Generate phase timetags
    // Phase data index goes from 0 to nPoint-1
    for(i=0; i<nPoint; i++)
    {
        fPhaseTimetags[i+ARRAY_OFFSET]=fDataMJD+(F_TYPE)(i*fInterval);
    }
    
    // Set phase #, start & end values
    fPhaseData[0]=(F_TYPE)nPoint;
    fPhaseData[1]=(F_TYPE)1;
    fPhaseData[2]=(F_TYPE)nPoint;
    fPhaseTimetags[0]=(F_TYPE)nPoint;
    fPhaseTimetags[1]=(F_TYPE)1;
    fPhaseTimetags[2]=(F_TYPE)nPoint;

    // Update phase status display
    _snprintf(szBuffer, sizeof(szBuffer), "%5.0d", nPoint);
    SetWindowText(hwndPhaseNum, szBuffer);
    SetWindowText(hwndPhaseEnd, szBuffer);
    _snprintf(szBuffer, sizeof(szBuffer), "%5.0d", 1);
    SetWindowText(hwndPhaseStart, szBuffer);
    _snprintf(szBuffer, sizeof(szBuffer), "%5.0d", (int)fTau);
    SetWindowText(hwndPhaseTau, szBuffer);

    // Set phase flags
    bPhaseData=TRUE;
    bPhaseDataChanged=TRUE;
	bPhaseDataUnsaved=TRUE;
    bPhaseScaleChanged=TRUE;
    bPhaseRedraw=TRUE;
    bPhaseTimetags=TRUE;

	// Define clock title
	strlcpy(szTitle, szMeasID, sizeof(szTitle));

    // Strip leading spaces from szTitle
    while(szTitle[0]==' ')
    {
        for(i=0; i<(int)strlen(szTitle); i++)
        {
            // Move up all chars inc EOS
            szTitle[i]=szTitle[i+1]; 
        }
    }
    
    // Strip trailing spaces from szTitle
    while(szTitle[strlen(szTitle)-1]==' ')
    {
        szTitle[strlen(szTitle)-1]='\0';
    }

    // Check if szTitle is a valid filename
    // A filename cannot be blank
    // Windows 95/NT filenames can contain any characters except:
    // ? " / \ < > * | :
    // Note that in a string constant the " and \ must be denoted
    // by their excape codes \" and \\ respectively

    if( (strlen(szTitle)==0) ||
        (strcspn(szTitle, "?\"/\\<>*|:") < strlen(szTitle)) )
    {
        // NG filename - Use default
        strlcpy(szTitle, "NONAME", sizeof(szTitle));
    }
    
    // Set phase filenames
    // Make a full filename from path + clock ID
    // Split filename into its parts
    _splitpath(szFile, szArchiveDrive, szArchiveDir, szArchiveName,
        szArchiveExt);
    // Define the phase filenames
    strlcpy(szPhaseFile, szArchiveDrive, FILE_NAME_SIZE);
    strlcat(szPhaseFile, szArchiveDir, FILE_NAME_SIZE);
    strlcat(szPhaseFile, szTitle, FILE_NAME_SIZE);
    strlcat(szPhaseFile, szArchiveExt, FILE_NAME_SIZE);
    strlcpy(szPhaseFileTitle, szTitle, FILE_TITLE_SIZE);
    IncExtension(szPhaseFile, PHASE_DATA);
    IncExtension(szPhaseFileTitle, PHASE_DATA);
    SetWindowText(hwndPhaseFilename, szPhaseFileTitle);

    // Rebuild the main filename
    _makepath(szFile, szArchiveDrive, szArchiveDir, szTitle,
        szArchiveExt);
    // Rename the main file title
    strlcpy(szFileTitle, szTitle, FILE_TITLE_SIZE);

    // Realloc freq data
    // Have nPoint-1 freq points
    if((fNew=(F_TYPE*)realloc(fFreqData, (nPoint-1+ARRAY_OFFSET+EXTRA_MEM)
        *sizeof(F_TYPE)))==NULL)
    {
		// Dsp msg, free mem & return error code
        AllocErrorMessage();
	    free(fX);
		free(fY);
		return 0.0;
    }
    else
    {
        fFreqData=fNew;
    }

    if(bDataType==FREQ_DATA)
    {
        fData=fFreqData;
    }

    // Realloc freq timetags
    // Have nPoint-1 freq points
    if((fNew=(F_TYPE*)realloc(fFreqTimetags, (nPoint-1+ARRAY_OFFSET+EXTRA_MEM)
        *sizeof(F_TYPE)))==NULL)
    {
		// Dsp msg, free mem & return error code
        AllocErrorMessage();
	    free(fX);
		free(fY);
		return 0.0;
    }
    else
    {
        fFreqTimetags=fNew;
    }

    if(bDataType==FREQ_DATA)
    {
        fTags=fFreqTimetags;
    }

    // Copy freq data (skips header)
	// Freq data index goes from 1 to nPoint-1
    for(i=1; i<nPoint; i++)
    {
        fFreqData[i+ARRAY_OFFSET-1]=fY[i+ARRAY_OFFSET];
    }                                                 

    // Generate freq timetags
    // Freq data index goes from 1 to nPoint-1
    for(i=1; i<nPoint; i++)
    {
        fFreqTimetags[i+ARRAY_OFFSET-1]=fDataMJD+(F_TYPE)(i*fInterval);
    }
    
    // Set freq #, start & end values
    fFreqData[0]=(F_TYPE)(nPoint-1);
    fFreqData[1]=(F_TYPE)1;
    fFreqData[2]=(F_TYPE)(nPoint-1);
    fFreqTimetags[0]=(F_TYPE)(nPoint-1);
    fFreqTimetags[1]=(F_TYPE)1;
    fFreqTimetags[2]=(F_TYPE)(nPoint-1);

    // Update freq status display
    _snprintf(szBuffer, sizeof(szBuffer), "%5.0d", nPoint-1);
    SetWindowText(hwndFreqNum, szBuffer);
    SetWindowText(hwndFreqEnd, szBuffer);
    _snprintf(szBuffer, sizeof(szBuffer), "%5.0d", 1);
    SetWindowText(hwndFreqStart, szBuffer);
    _snprintf(szBuffer, sizeof(szBuffer), "%5.0d", (int)fTau);
    SetWindowText(hwndFreqTau, szBuffer);
    
    // Set freq filenames
    // Make a full filename from path + clock ID
    // Split filename into its parts
    _splitpath(szFile, szArchiveDrive, szArchiveDir, szArchiveName,
        szArchiveExt);
    strlcpy(szFreqFile, szArchiveDrive, FILE_NAME_SIZE);
    strlcat(szFreqFile, szArchiveDir, FILE_NAME_SIZE);
    strlcat(szFreqFile, szTitle, FILE_NAME_SIZE);
    strlcat(szFreqFile, szArchiveExt, FILE_NAME_SIZE);
    strlcpy(szFreqFileTitle, szTitle, FILE_TITLE_SIZE);
    IncExtension(szFreqFile, FREQ_DATA);
    IncExtension(szFreqFileTitle, FREQ_DATA);
    SetWindowText(hwndFreqFilename, szFreqFileTitle);

    // Set freq flags
    bFreqData=TRUE;
    bFreqDataChanged=TRUE;
	bFreqDataUnsaved=TRUE;
	sFPlot.new_file=TRUE;
    bFreqScaleChanged=TRUE;
    bFreqRedraw=TRUE;
    bFreqTimetags=TRUE;
    
    // Set data synch flag
    bSynch=TRUE;

    // Update analysis limit display
    DisplayLims();

    // Clear plot lines
    ClearLines();

    // Update status display
    UpdateStatusBar(hwndMain, NEW_DATA);

    // Activate data-dependent functions
    ActivateFunctions(MF_ENABLED);

    // Free all memory allocations
    free(fX);
    free(fY);

    // Done
    return fDataMJD; // Normal exit
}
                                                              
/*****************************************************************************/
/*                                                                           */
/*                            ResizeArrays()                                 */
/*                                                                           */
/*      Function to resize phase and frequency data arrays                   */
/*                                                                           */
/*      Parameters:                                                          */
/*          int nSize   New array size                                       */
/*                                                                           */
/*      Return:                                                              */
/*          int nCode	1=OK, 0=NG                                           */
/*                                                                           */
/*      Function revision record:                                            */
/*          04/26/98        Created                                          */
/*			06/18/98		Added error code return                          */
/*                                                                           */
/*****************************************************************************/

int WINAPI ResizeArrays(int nSize)
{
    // Local variables
    F_TYPE *fNew;       // New data array pointer

    // Realloc phase & freq archive file data arrays
    // to new size nSize (plus ARRAY_OFFSET plus EXTRA_MEM)
    // plus 1 for phase array
    // The array pointers are global to this source file

	// Realloc phase data
    if((fNew=(F_TYPE*)realloc(fX, (nSize+ARRAY_OFFSET+EXTRA_MEM+1)
        *sizeof(F_TYPE)))==NULL)
    {
        AllocErrorMessage();
		return 0;
    }
    else
    {
        fX=fNew;
    }

    // Realloc freq data
    if((fNew=(F_TYPE*)realloc(fY, (nSize+1+ARRAY_OFFSET+EXTRA_MEM)
        *sizeof(F_TYPE)))==NULL)
    {
        AllocErrorMessage();
		return 0;
    }
    else
    {
        fY=fNew;
    }

	return 1;
}                                                      

/*****************************************************************************/
/*                                                                           */
/*                            IndexArchive()                                 */
/*                                                                           */
/*      Function to create an index for a TSC archive file                   */
/*		Extracts clock description, channel # & MJD in/out information       */
/*		Shows all clock transactions                                         */
/*		Calculates clock time and frequency                                  */
/*		Writes clock index file ARCINDEX.DAT using szArcIndexFile filename   */
/*		Uses notepad to display results                                      */
/*                                                                           */
/*      Parameters:                                                          */
/*          firstMJD	numeric value of first MJD in archive file           */
/*          lastMJD		numeric value of last MJD in archive file            */
/*			msg_str		message stiring to be displayed in progress bar      */
/*                                                                           */
/*      Return:                                                              */
/*          int Status code: 0=OK, 1=error                                   */
/*                                                                           */
/*      Function revision record:                                            */
/*          04/30/98    Created from CLKPLT.C                                */
/*			05/08/98	Changed date code for Y2K compliance                 */	
/*                                                                           */
/*****************************************************************************/

int WINAPI IndexArchive(double firstMJD, double lastMJD, char *msg_str)
{
	#define MAXCHAN 24                                 /* default # channels */
	#define TAU 900.0                        /* default measurement interval */

     /* Note: each character string has space for maximum # characters, '\0' */
								    /* EOS character plus 1 extra for safety */

	struct clock_info                  /* definition of clock_info structure */
	{
		BOOL used;                                      /* used channel flag */
		BOOL active;                                  /* active channel flag */
		BOOL meas;                                  /* measured channel flag */
		BOOL sign;    /* data sign rev flag: FALSE=normal, TRUE=reverse sign */
		char name[22];                                         /* clock name */
		char in[20];                                               /* MJD in */
		char out[20];                                             /* MJD out */
		double time;                                            /* time data */
		double pre_time;                               /* previous time data */
		int refchan;                                  /* reference channel # */
	};

	FILE *arch, *out;                                       /* file pointers */
	int i;                                                          /* index */
	int k;              /* correction factor to account for # digits in ch # */
	int chan;                                                   /* channel # */
	int year;                                                 /* year of MDJ */
	int month;                                               /* month of MJD */
	int day;                                                   /* day of MJD */
	int hour;                                                /* current hour */
	int min;                                               /* current minute */
	int numchan;                                     /* # channels in system */
	int record[MAXCHAN];      /* array for current record # for each channel */
	int percent;			                               /* % of file read */
	long MJD;                                                       /* MJD # */
	long daynum;                             /* day # for date calc from MJD */
	// long lday;                                     /* long version of day */
	float time;                       /* current time as a fraction of a MJD */
	float tau;                                       /* measurement interval */
	double nom_freq;                             /* nominal frerquency value */
	double MJD_val;                                  /* numeric value of MJD */
	struct clock_info **id;       /* pointer to array of clock_info pointers */
	char input_str[81];               /* one line of input from archive file */
	char symbol;                          /* first character of line: $ or # */
	char phase_tag_str[11];                              /* phase tag string */
	char nom_freq_str[20];                       /* nominal frequency string */
	char phase_data_str[24];                            /* phase data string */
	char MJD_str[20];                                          /* MJD string */
	char previous_MJD_str[20];                                 /* MJD string */
	char id_str[22];                                      /* clock id string */
	char chan_str[4];                                    /* channel # string */
	char filenam[12];                                       /* data filename */
    char szBuffer[BUFFER_SIZE+1];                             /* text buffer */

	/* assign default value to # channels */
	numchan=MAXCHAN;

	/* initialize record #s */
	for(i=0; i<MAXCHAN; i++)
	{
		record[i]=0;
	}

	/* allocate memory for array of pointers to clock_info structures */
	if((id=(struct clock_info **)
		  malloc(numchan*sizeof(struct clock_info*)))==NULL)
	{
		TRACE_STR("Insufficient memory for clock data pointers.");
		return 1;                                              /* error exit */
	}                                                              /* end if */

	/* allocate memory for array of clock_info structures */
	for(i=0; i<numchan; i++)
	{
		if((id[i]=(struct clock_info *)
			  malloc(1*sizeof(struct clock_info)))==NULL)
		{
			TRACE_STR("Insufficient memory for clock data structure.");
			return 1;                                          /* error exit */
		}                                                          /* end if */
	}                                                             /* end for */

	/* store default filename & tau */
	strlcpy(filenam, "ARCHIVE.BAK", sizeof(filenam));
	tau=TAU;

	/* clear clock flags & enter defaults */
	for(i=0; i<numchan; i++)
	{
		id[i]->used=FALSE;
		id[i]->active=FALSE;
		id[i]->meas=FALSE;
		id[i]->sign=FALSE;
		id[i]->refchan=1;
	}                                                             /* end for */

	/* open input file */
	if((arch=fopen(szFile, "r")) == NULL)      /* open archive file */
	{
		_snprintf(szBuffer, sizeof(szBuffer), "Cannot Open %s", filenam);
		TRACE_STR(szBuffer);
		return 1;                                              /* error exit */
	}                                                              /* end if */

	/* open output file */
	if((out=fopen(szArcIndexFile, "w")) == NULL)         /* open output file */
	{
		_snprintf(szBuffer, sizeof(szBuffer),
			"Cannot Open Archive Index Data File");
		TRACE_STR(szBuffer);
		return 1;                                              /* error exit */
	}                                                              /* end if */

	/* write heading */
	fprintf(out,
	"  DATE          MJD         HR MIN  CH      CLOCK ID        I/O"
	" REC #\n");

	/* read input data */
	while(fgets(input_str, 81, arch) != NULL)         /* read 1 line of file */
	{
		sscanf(input_str, "%c", &symbol);                       /* scan line */
		if(symbol=='#')                                   /* channel # line? */
		{
			sscanf(input_str, "%c %s", &symbol, &chan_str);
			chan=atoi(chan_str);                            /* get channel # */
			if(chan>numchan)                      /* test for chan # too big */
			{
				_snprintf(szBuffer, sizeof(szBuffer),
					"Error: Channel # > MAXCHAN");
				TRACE_STR(szBuffer);
				return 1;                                      /* error exit */
			}                                                      /* end if */
			if( (strlen(chan_str)==1) )                   /* 1-digit chan #? */
			{
				k=0;
			}                                                      /* end if */
			else                                           /* 2-digit chan # */
			{
				k=1;
			}                                                    /* end else */

			/* read clock id_str */
			for(i=0; i<20; i++)              /* 20-character clock id string */
			{
				id_str[i]=input_str[i+3+k];
				id_str[20]='\0';
			}                                                     /* end for */

			strlcpy(id[chan-1]->name, id_str, 22);      /* put id into table */

			/* remove '#', chan_str and id_str from input_str */
			/* '#' is 1 character followed by 1 space */
			/* chan_str is k+1 characters followed by 1 space */
			/* id_str is 20 characters followed by 1 space */
			/* so we want to remove 25+k characters from input_str */

			for(i=0; i<55-k; i++)
			{
				input_str[i]=input_str[i+25+k];
			}

			/* now we can easily parse the rest of input_str */
			sscanf(input_str, "%s %s %s", &nom_freq_str, &phase_tag_str,
				   &phase_data_str);          	  /* scan rest of input line */

			/* calculate clock time & frequency */
			id[chan-1]->pre_time=id[chan-1]->time;     /* save previous time */
			nom_freq=atof(nom_freq_str);            /* get nominal frequency */

			id[chan-1]->time=atof(phase_data_str);        /* get phase value */
			if(nom_freq)                         /* check for divide by zero */
			{
				id[chan-1]->time/=nom_freq*1e6;        /* conv to time value */
			}

			id[chan-1]->used=TRUE;                        /* raise used flag */
			id[chan-1]->meas=TRUE;                    /* raise measured flag */

			if(id[chan-1]->active==FALSE)            /* newly entered clock? */
			{
				id[chan-1]->active=TRUE;                /* mark it as active */
				strlcpy(id[chan-1]->in, MJD_str, 20); /* put MJD In in table */
				MJD=atol(MJD_str);

				// Note: This code assumes (MJD>48622L) && (MJD<51543L)
				// e.g. 1992 thru 2000

				time=(float)atof(MJD_str)-MJD;
				hour=(int)(time*24);
				min=(int)((((time*24)-hour)*60)+0.5);

				/* convert MJD to date */
				daynum=MJD+2400001L; // Julian day #
				daynum=JulianToDate(daynum); // Get Gregorian date (YYYYMMDD)
				BreakDate(daynum, &month, &day, &year); // Get date components
				year=year%100;

				/* increment record # */
				record[chan-1]++;

				/* write output */
				fprintf(out, "%02d/%02d/%02d %s %2d  %2d  %2d %s  in  %3d\n",
						month, day, year, MJD_str, hour, min, chan,
						id_str, record[chan-1]);                /* write o/p */
			}                                                      /* end if */
		}                                                          /* end if */

		else if(symbol=='$')                                    /* MJD line? */
		{
			sscanf(input_str, "%c %s", &symbol, &MJD_str);
														  /* scan line again */
			for(i=0; i<numchan; i++)                /* any channels removed? */
			/* scan all channels, looking for any active clocks that were  */
			/* not measured, and therefore were removed since previous     */
			/* measurement time                                            */
			{

				// Use current MJD % read as status indicator
				MJD_val=atof(MJD_str);
				percent=(int)((MJD_val-firstMJD)*100/(lastMJD-firstMJD));
				_snprintf(szBuffer, sizeof(szBuffer), "%d", percent);
				WriteProgressMessage(percent, msg_str, FALSE);

				if( (id[i]->active==TRUE) &&
					(id[i]->meas==FALSE) )                 /* clock removed? */
				{
					strlcpy(id[i]->out, previous_MJD_str, 20);
												   /* put MJD Out into table */
					id[i]->active=FALSE;                    /* deactivate it */
					MJD=atol(MJD_str);                   /* calc numeric MJD */
					time=(float)atof(previous_MJD_str)-MJD;
					                                    /* time as fract day */
					hour=(int)(time*24);                       /* calc hours */
					min=(int)((((time*24)-hour)*60)+0.5);    /* calc minutes */

					/* convert MJD to date */
					daynum=MJD+2400001L; // Julian day #
					// Get Gregorian date (YYYYMMDD)
					daynum=JulianToDate(daynum); 
					// Get date components
					BreakDate(daynum, &month, &day, &year);
											
					year=year%100;

					/* write output */
					fprintf(out,
							"%02d/%02d/%02d %s %2d  %2d  %2d %s out  %3d\n",
							month, day, year, previous_MJD_str, hour,
							min, i+1, id[i]->name, record[i]);
				}                                                  /* end if */
			}                                                   /* end while */

			for(i=0; i<numchan; i++)       /* prepare for next meas interval */
			{
				id[i]->meas=FALSE;            /* clear all measurement flags */
			}                                                     /* end for */

			strlcpy(previous_MJD_str, MJD_str, sizeof(previous_MJD_str));
														/* save previous MJD */
		}                                                        /* end else */
	}                                                           /* end while */

	/* clean up & end */
	fclose(arch);                                         /* close all files */
	fclose(out);

	// Display index
    strlcpy(szBuffer, szNotepad, sizeof(szBuffer));
    strlcat(szBuffer, " ", sizeof(szBuffer));
    strlcat(szBuffer, szArcIndexFile, sizeof(szBuffer));
    WinExec(szBuffer, SW_SHOWNORMAL);
	
	return 0;                                                 /* normal exit */
}                                                            

#ifndef LITE_VER
/*****************************************************************************/
/*                                                                           */
/*                    Archive Dialog Procedure                               */
/*                                                                           */
/*      Function to control reading of a TSC archive file                    */
/*                                                                           */
/*      Revision record:                                                     */
/*          04/24/98    Created                                              */
//*                                                                          */
/*****************************************************************************/

BOOL CALLBACK ArchiveDlgProc(HWND hDlg, UINT message, WPARAM wParam,
    LPARAM lParam)
{
    // Local variables
    // Chars
    char cSymbol;                   // Line ID symbol
    char szBuffer[BUFFER_SIZE+1];   // Text buffer
    char szMJD[BUFFER_SIZE+1];      // MJD buffer
    char szSection[BUFFER_SIZE+1];  // INI file section name
    // Ints
    // int i;                       // Index - not used
    int nEntry;                     // # entered by user
    int nPrev;                      // Previous value
    static int nSec;                // Tau in seconds
    static int nSize;               // # data points
    static int nMax;                // Maximum allowable channel #
    static int nMeas=1;             // Measurement channel #
    static int nRef=1;              // Reference channel #
    static int nRecord=1;           // Record #
    static int nReadError;          // Read error count
    // LONGs
    static long lOffset;            // File position offset
    // BOOLs
    static BOOL bFirstTime=TRUE;    // Flag for 1st time initialization
    static BOOL bErrorTrap=FALSE;   // Flag for entry error
    static BOOL bLastRecord=TRUE;   // Flag for last record checkbox
    static BOOL bUseEnd=FALSE;      // Flag for end MJD checkbox
    static BOOL bRecordNum=TRUE;    // Flag for record number (not MJD)
	static BOOL bNewArchive=TRUE;	// Flag that index needs to be created
    // F_TYPEs
    F_TYPE fEntry;                  // # entered by user
    F_TYPE fPrev;                   // Previous value
    F_TYPE fMJD1;                   // 1st MJD in file
    F_TYPE fMJD2;                   // 2nd MJD in file
    static F_TYPE fFirstMJD;        // Archive file first MJD
    static F_TYPE fLastMJD;         // Archive file last MJD
    static F_TYPE fMJDStart;        // MJD start date
    static F_TYPE fMJDEnd;          // MJD end date
    // Misc
    FILE *fpArch;                   // Archive file pointer

    switch(message)
    {
        case WM_INITDIALOG:
        {
            // Set global dialog box handle
            hDialog=hDlg;

            // Get first & last MDJ & tau from archive file
            SendMessage(hDlg, WM_COMMAND,
                GET_WM_COMMAND_MPS(IDC_ARCHIVE_READ, 0, 0));

            // First time initializations
            if(bFirstTime)
            {
                // Set MJD start & end dates to defaults
                fMJDStart=fFirstMJD;                    
                fMJDEnd=fLastMJD;

                // Reset first time flag
                bFirstTime=FALSE;

				// Set new archive index flag
				bNewArchive=TRUE;

				// Initial nMax, nMeas & nRref values from INI file
				strlcpy(szSection, "Archive", sizeof(szSection));
				nMax=GetPrivateProfileInt(szSection, "Num", 4, szIniFile);
				nMeas=GetPrivateProfileInt(szSection, "Meas", 2, szIniFile);
				nRef=GetPrivateProfileInt(szSection, "Ref", 1, szIniFile);
           }
            
            // Display the archive filename
            strlcpy(szBuffer, "File: ", sizeof(szBuffer));
            strlcat(szBuffer, szFileTitle, sizeof(szBuffer));
            SetWindowText(GETHCTL(IDC_ARCHIVE_FILE), szBuffer);
            
            // Initialize the measurement channel #
			if(nMeas>0)
			{
				_snprintf(szBuffer, sizeof(szBuffer), "%d", nMeas,
					sizeof(szBuffer));
			}
			else // Sense reversed
			{
				_snprintf(szBuffer, sizeof(szBuffer), "%d (Rev)", nMeas,
					sizeof(szBuffer));
			}

            SetWindowText(GETHCTL(IDC_ARCHIVE_MEAS), szBuffer);

            // Initialize the reference channel #
			if(nRef)
			{
				_snprintf(szBuffer, sizeof(szBuffer), "%d", nRef,
					sizeof(szBuffer));
			}
			else // 0=NOREF case
			{
				_snprintf(szBuffer, sizeof(szBuffer), "%d (No Ref)", nRef,
					sizeof(szBuffer));
			}

            SetWindowText(GETHCTL(IDC_ARCHIVE_REF), szBuffer);

			// Make sure record # not 0 from previous last record search
			if(nRecord==0)
			{
				nRecord=1;
			}

           // Initialize the record #
            _snprintf(szBuffer, sizeof(szBuffer), "%d", abs(nRecord));
            SetWindowText(GETHCTL(IDC_ARCHIVE_RECORD), szBuffer);

            // Initialize the start MJD value 
            _snprintf(szBuffer, sizeof(szBuffer), "%14.8f", fMJDStart);
            SetWindowText(GETHCTL(IDC_ARCHIVE_START), szBuffer);

            // Initialize the end MJD value
            _snprintf(szBuffer, sizeof(szBuffer), "%14.8f", fMJDEnd);
            SetWindowText(GETHCTL(IDC_ARCHIVE_END), szBuffer);

            // Initialize the time span radio buttons to Record #
            Button_SetCheck(GETHCTL(IDC_ARCHIVE_NUM), bRecordNum);
            Button_SetCheck(GETHCTL(IDC_ARCHIVE_MJD), !bRecordNum);

            // Initialize the last record checkbox state
            Button_SetCheck(GETHCTL(IDC_ARCHIVE_LAST), bLastRecord);

            // Set the activation state of the record # edit box
            // It is inactive if the last record box is checked
            // or if the MJD radio button is true
            EnableWindow(GETHCTL(IDC_ARCHIVE_RECORD),
                (!bLastRecord)&&(bRecordNum));

            // Initialize the end MJD checkbox state
            Button_SetCheck(GETHCTL(IDC_ARCHIVE_ALL), bUseEnd);

            // Set the activation state of the start MJD edit box
            // It is inactive if the record # radio button is true
            EnableWindow(GETHCTL(IDC_ARCHIVE_START), !bRecordNum);

            // Set the activation state of the end MJD edit box
            // It is active if the end MJD box is checked
            // and the record # radio button is false
            EnableWindow(GETHCTL(IDC_ARCHIVE_END),
                (bUseEnd)&&(!bRecordNum));

            // Set the activation state of the last record checkbox
            // It is active if the record # radio button is true
            EnableWindow(GETHCTL(IDC_ARCHIVE_LAST), bRecordNum);

            // Display the tau value
            _snprintf(szBuffer, sizeof(szBuffer), "Tau: %d", nSec);
            SetWindowText(GETHCTL(IDC_ARCHIVE_TAU), szBuffer);

            // Display the first MJD value 
            _snprintf(szBuffer, sizeof(szBuffer), "First MJD: %14.8f",
				fFirstMJD);
            SetWindowText(GETHCTL(IDC_ARCHIVE_FIRST), szBuffer);

            // Set fMJDStart to fFirstMJD
            fMJDStart=fFirstMJD;
            
            // Display the last MJD value 
            _snprintf(szBuffer, sizeof(szBuffer), "Last MJD: %14.8f",
				fLastMJD);
            SetWindowText(GETHCTL(IDC_ARCHIVE_LASTMJD), szBuffer);

            // Set fMJDEnd to fLastMJD
            fMJDEnd=fLastMJD;

            // Reinitialize the start MJD value 
            _snprintf(szBuffer, sizeof(szBuffer), "%14.8f", fMJDStart);
            SetWindowText(GETHCTL(IDC_ARCHIVE_START), szBuffer);

			// Initialize the meas chan # spin control base to 10
			SendMessage(GETHCTL(IDC_ARCHIVE_MSPIN), UDM_SETBASE,
				(DWORD)10, 0);

			// Init the meas chan # spin control range from -nMax to nMax
			// Note: The "max" & "min" usages are reversed
			SendMessage(GETHCTL(IDC_ARCHIVE_MSPIN), UDM_SETRANGE, 1,
				MAKELONG((WORD)nMax, (WORD)-nMax));

			// Initialize the meas chan # spin control value to nMeas
			SendMessage(GETHCTL(IDC_ARCHIVE_MSPIN), UDM_SETPOS, 0,
				MAKELONG((WORD)nMeas, 0));

			// Initialize the ref chan # spin control base to 10
			SendMessage(GETHCTL(IDC_ARCHIVE_RSPIN), UDM_SETBASE,
				(DWORD)10, 0);

			// Init the ref chan # spin control range from 0 to nMax
			// Note: The "max" & "min" usages are reversed
			SendMessage(GETHCTL(IDC_ARCHIVE_RSPIN), UDM_SETRANGE, 0,
				MAKELONG((WORD)nMax, (WORD)0));

			// Initialize the ref chan # spin control value to nRef
			SendMessage(GETHCTL(IDC_ARCHIVE_RSPIN), UDM_SETPOS, 0,
				MAKELONG((WORD)nRef, 0));

            return TRUE;
        }

		case WM_VSCROLL:
		{
			// Is message from measurement channel spin control?
			if((HWND)lParam==GETHCTL(IDC_ARCHIVE_MSPIN))
			{
				// Get new position (value)
				nEntry=HIWORD(wParam);

				// Convert negative value to signed value
				if(nEntry>32767)
				{
					nEntry=nEntry-65536;
				}

				// Value=0 or nRef not allowed
				if(nEntry==0 || abs(nEntry)==nRef)
				{
					break;
				}

				// Entry OK
				nMeas=nEntry;

				if(nMeas>0)
				{
					_snprintf(szBuffer, sizeof(szBuffer), "%d", nMeas);
				}
				else // Sense reversed
				{
					_snprintf(szBuffer, sizeof(szBuffer), "%d (Rev)",
						nMeas);
				}

				SetWindowText(GETHCTL(IDC_ARCHIVE_MEAS), szBuffer);
			}

			// Is message from reference channel spin control?			
			if((HWND)lParam==GETHCTL(IDC_ARCHIVE_RSPIN))
			{
				// Get new position (value)
				nEntry=HIWORD(wParam);

				// Value=nMeas not allowed
				if(nEntry==nMeas)
				{
					break;
				}

				// Entry OK
				nRef=nEntry;

				if(nRef)
				{
					_snprintf(szBuffer, sizeof(szBuffer), "%d", nRef);
				}
				else // 0=NOREF case
				{
					_snprintf(szBuffer, sizeof(szBuffer), "%d (No Ref)",
						nRef);
				}

				SetWindowText(GETHCTL(IDC_ARCHIVE_REF), szBuffer);
			}
		}
		break;

        case WM_COMMAND:
        {
            switch(GET_WM_COMMAND_ID(wParam, lParam))
            {
                // Respond to Help pushbutton
                case IDHELP:
                {
					#if(0)
					// Invoke help file
					HtmlHelp(hDlg, "Stable32.chm::READ_ARCHIVE_FUNCTION.HTM",
						HH_DISPLAY_TOPIC, 0L);
					#endif

					#if(0) // New code 11/29/11
					// Invoke help file
					strlcpy(szBuffer, szHelpFile, sizeof(szBuffer));
					strlcat(szBuffer, "::READ_ARCHIVE_FUNCTION.HTM", sizeof(szBuffer));
					HtmlHelp(hDlg, szBuffer,
						HH_DISPLAY_TOPIC, 0L);
					#endif
 
					#if(1) // Trial code 11/30/11
					// Invoke help file using CallHelp() function
					CallHelp(hDlg, "READ_ARCHIVE_FUNCTION");
					#endif
               }
                break;

                // Respond to MJD start edit control message
                case IDC_ARCHIVE_START:
                {
                    // Respond to edit control when it loses the input focus
                    // unless it loses it to the cancel button
                    if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
                        (!bErrorTrap) &&
                        (GetFocus()!=GETHCTL(IDC_ARCHIVE_CANCEL)))
                    {
                        // Set error flag to avoid multiple error messages
                        bErrorTrap=TRUE;

                        // Save previous value
                        fPrev=fMJDStart;

                        // Get (new) MJD start value
                        GetWindowText(GETHCTL(IDC_ARCHIVE_START), szBuffer,
                            BUFFER_SIZE);
                        fEntry=fabs(atof(szBuffer));

                        // Check entry - must be between fFirstMJD and fMJDEnd
                        // Numeric params are value, min & max
                        if(ValidateMJD(hDlg, fEntry, fFirstMJD, fMJDEnd)
                            ==TRUE)
                        {
                            // New value OK
                            fMJDStart=fEntry;

                            // Redisplay value
                            _snprintf(szBuffer, sizeof(szBuffer), "%14.8f",
								fMJDStart);
                            SetWindowText(GETHCTL(IDC_ARCHIVE_START), szBuffer);
                        }
                        else
                        {
                            // Invalid entry - restore previous value
                            _snprintf(szBuffer, sizeof(szBuffer), "%14.8f",
								fPrev);
                            SetWindowText(GETHCTL(IDC_ARCHIVE_START), szBuffer);
                            SetFocus(GETHCTL(IDC_ARCHIVE_START));
                        }
                    }
                    else
                    {
                        // Reset error flag
                        bErrorTrap=FALSE;
                    }
                }
                break;

                // Respond to MJD end edit control message
                case IDC_ARCHIVE_END:
                {
                    // Respond to edit control when it loses the input focus
                    // unless it loses it to the cancel button
                    if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
                        (!bErrorTrap) &&
                        (GetFocus()!=GETHCTL(IDC_ARCHIVE_CANCEL)))
                    {
                        // Set error flag to avoid multiple error messages
                        bErrorTrap=TRUE;

                        // Save previous value
                        fPrev=fMJDEnd;

                        // Get (new) MJD end value
                        GetWindowText(GETHCTL(IDC_ARCHIVE_END), szBuffer,
                            BUFFER_SIZE);
                        fEntry=fabs(atof(szBuffer));

                        // Check entry
                        // Must be between fMJDStart and fLastMJD
                        // Numeric params are value, min & max
                        if(ValidateMJD(hDlg, fEntry, fMJDStart,
                            fLastMJD)==TRUE)
                        {
                            // New value OK
                            fMJDEnd=fEntry;

                            // Redisplay value
                            _snprintf(szBuffer, sizeof(szBuffer), "%14.8f",
								fMJDEnd);
                            SetWindowText(GETHCTL(IDC_ARCHIVE_END), szBuffer);
                        }
                        else
                        {
                            // Invalid entry - restore previous value
                            _snprintf(szBuffer, sizeof(szBuffer), "%14.8f",
								fPrev);
                            SetWindowText(GETHCTL(IDC_ARCHIVE_END), szBuffer);
                            SetFocus(GETHCTL(IDC_ARCHIVE_END));
                        }
                    }
                    else
                    {
                        // Reset error flag
                        bErrorTrap=FALSE;
                    }
                }
                break;

                // Respond to measurement channel # edit control message
                case IDC_ARCHIVE_MEAS:
                {
                    // Respond to edit control when it loses the input focus
                    // unless it loses it to the cancel button
                    if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
                        (!bErrorTrap) &&
                        (GetFocus()!=GETHCTL(IDC_ARCHIVE_CANCEL)))
                    {
                        // Set error flag to avoid multiple error messages
                        bErrorTrap=TRUE;

                        // Save previous value
                        nPrev=nMeas;

                        // Get (new) measurement channel #
                        GetWindowText(GETHCTL(IDC_ARCHIVE_MEAS), szBuffer,
                            BUFFER_SIZE);
                        nEntry=(atoi(szBuffer));

                        // Check entry - abs value must be between 1 and
                        // max # channels
                        // Numeric params are value, min & max
                        if(ValidateInteger(hDlg, abs(nEntry), 1, nMax)==TRUE)
                        {
							// Check that entry not equal ref channel #
							if(abs(nEntry)==nRef)
							{
								// Error message
								LoadString(hInst, IDS_MESSAGE_CHANSEQUAL,
								   szBuffer, BUFFER_SIZE);
								MessageBox(hwndMain, szBuffer, "Read Archive",
									MB_ICONEXCLAMATION);

								// Invalid entry - restore previous value
								_snprintf(szBuffer, sizeof(szBuffer), "%d",
									nPrev);
								SetWindowText(GETHCTL(IDC_ARCHIVE_MEAS),
									szBuffer);
								SetFocus(GETHCTL(IDC_ARCHIVE_MEAS));
								break;
							}

                            // New value OK
                            nMeas=nEntry;

                            // Redisplay value
							if(nMeas>0)
							{
								_snprintf(szBuffer, sizeof(szBuffer), "%d",
									nMeas);
							}
							else // Sense reversed
							{
								_snprintf(szBuffer, sizeof(szBuffer),
									"%d (Rev)", nMeas);
							}

                            SetWindowText(GETHCTL(IDC_ARCHIVE_MEAS), szBuffer);
                        }
                        else
                        {
                            // Invalid entry - restore previous value
                            _snprintf(szBuffer, sizeof(szBuffer), "%d", nPrev);

                            SetWindowText(GETHCTL(IDC_ARCHIVE_MEAS), szBuffer);
                            SetFocus(GETHCTL(IDC_ARCHIVE_MEAS));
                        }
                    }
                    else
                    {
                        // Reset error flag
                        bErrorTrap=FALSE;
                    }
                }
                break;

                // Respond to reference channel # edit control message
                case IDC_ARCHIVE_REF:
                {
                    // Respond to edit control when it loses the input focus
                    // unless it loses it to the cancel button
                    if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
                        (!bErrorTrap) &&
                        (GetFocus()!=GETHCTL(IDC_ARCHIVE_CANCEL)))
                    {
                        // Set error flag to avoid multiple error messages
                        bErrorTrap=TRUE;

                        // Save previous value
                        nPrev=nRef;

                        // Get (new) reference channel #
                        GetWindowText(GETHCTL(IDC_ARCHIVE_REF), szBuffer,
                            BUFFER_SIZE);
                        nEntry=abs(atoi(szBuffer));

                        // Check entry - must be between 0 and
                        // max channel # (0=NOREF option)
                        // Numeric params are value, min & max
                        if(ValidateInteger(hDlg, nEntry, 0, nMax)==TRUE)
                        {
							// Check that entry not equal meas channel #
							if(nEntry==nMeas)
							{
								// Error message
								LoadString(hInst, IDS_MESSAGE_CHANSEQUAL,
								   szBuffer, BUFFER_SIZE);
								MessageBox(hwndMain, szBuffer, "Read Archive",
									MB_ICONEXCLAMATION);

								// Invalid entry - restore previous value
								_snprintf(szBuffer, sizeof(szBuffer), "%d",
									nPrev);
								SetWindowText(GETHCTL(IDC_ARCHIVE_REF),
									szBuffer);
								SetFocus(GETHCTL(IDC_ARCHIVE_REF));
								break;
							}
                            
							// New value OK
                            nRef=nEntry;

                            // Redisplay value
							if(nRef)
							{
								_snprintf(szBuffer, sizeof(szBuffer), "%d",
									nRef);
							}
							else // 0=NOREF case
							{
								_snprintf(szBuffer, sizeof(szBuffer),
									"%d (No Ref)", nRef);
							}

							SetWindowText(GETHCTL(IDC_ARCHIVE_REF), szBuffer);
                        }
                        else
                        {
                            // Invalid entry - restore previous value
                            _snprintf(szBuffer, sizeof(szBuffer), "%d", nPrev);

                            SetWindowText(GETHCTL(IDC_ARCHIVE_REF), szBuffer);
                            SetFocus(GETHCTL(IDC_ARCHIVE_REF));
                        }
                    }
                    else
                    {
                        // Reset error flag
                        bErrorTrap=FALSE;
                    }
                }
                break;

                // Respond to record # edit control message
                case IDC_ARCHIVE_RECORD:
                {
                    // Respond to edit control when it loses the input focus
                    // unless it loses it to the cancel button
                    if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
                        (!bErrorTrap) &&
                        (GetFocus()!=GETHCTL(IDC_ARCHIVE_CANCEL)))
                    {
                        // Set error flag to avoid multiple error messages
                        bErrorTrap=TRUE;

                        // Save previous value
                        nPrev=nRecord;

                        // Get (new) record #
                        GetWindowText(GETHCTL(IDC_ARCHIVE_RECORD), szBuffer,
                            BUFFER_SIZE);
                        nEntry=abs(atoi(szBuffer));

                        // Check entry - must be between 1 and ?
                        // Use 9999 for max record # for now
                        // Numeric params are value, min & max
                        if(ValidateInteger(hDlg, nEntry, 1, 9999)==TRUE)
                        {
                            // New value OK
                            nRecord=nEntry;

                            // Redisplay value
                            _snprintf(szBuffer, sizeof(szBuffer), "%d",
								nRecord);
                            SetWindowText(GETHCTL(IDC_ARCHIVE_RECORD),
                                szBuffer);
                        }
                        else
                        {
                            // Invalid entry - restore previous value
                            _snprintf(szBuffer, sizeof(szBuffer), "%d",
								nPrev);

                            SetWindowText(GETHCTL(IDC_ARCHIVE_RECORD),
                                szBuffer);
                            SetFocus(GETHCTL(IDC_ARCHIVE_RECORD));
                        }
                    }
                    else
                    {
                        // Reset error flag
                        bErrorTrap=FALSE;
                    }
                }
                break;

                // Respond to last record checkbox
                case IDC_ARCHIVE_LAST:
                {
                    // Get state of last record checkbox
                    bLastRecord=
                        (BOOL)Button_GetCheck(GETHCTL(IDC_ARCHIVE_LAST));
                    
                    // Set the activation state of the record # edit box
                    // It is inactive if the last record box is checked
                   EnableWindow(GETHCTL(IDC_ARCHIVE_RECORD), !bLastRecord);
                }
                break;

                // Respond to end MJD checkbox
                case IDC_ARCHIVE_ALL:
                {
                    // Get state of end MJD checkbox
                    bUseEnd=
                        (BOOL)Button_GetCheck(GETHCTL(IDC_ARCHIVE_ALL));
                    
                    // Set the activation state of the end MJD edit box
                    // It is active if the end MJD box is checked
                    // and the record # flag is false
                   EnableWindow(GETHCTL(IDC_ARCHIVE_END),
                       (bUseEnd)&&(!bRecordNum));
                }
                break;

                // Respond to record # radio button
                case IDC_ARCHIVE_NUM:
                {
                    // Get state of record # radio button
                    bRecordNum=
                        (BOOL)Button_GetCheck(GETHCTL(IDC_ARCHIVE_NUM));

					// If bRecordNum is TRUE, want to set the MJD values
					// to the full file limits
					if(bRecordNum)
					{
			            // Initialize the start MJD value 
			            _snprintf(szBuffer, sizeof(szBuffer), "%14.8f",
							fFirstMJD);
			            SetWindowText(GETHCTL(IDC_ARCHIVE_START), szBuffer);

			            // Initialize the end MJD value
			            _snprintf(szBuffer, sizeof(szBuffer), "%14.8f",
							fLastMJD);
			            SetWindowText(GETHCTL(IDC_ARCHIVE_END), szBuffer);
					}
                    
                    // Set the activation state of the record # edit box
                    // It is active if the record # radio button is true
                    // and the last record checkbox is false
                    EnableWindow(GETHCTL(IDC_ARCHIVE_RECORD),
                        (bRecordNum)&&(!bLastRecord));

                    // Set the activation state of the start MJD edit box
                    // It is active if the record # radio button is false
                    EnableWindow(GETHCTL(IDC_ARCHIVE_START), !bRecordNum);

                    // Set the activation state of the end MJD edit box
                    // It is active if the record # radio button is false
                    // and the end MJD checkbox is true
                    EnableWindow(GETHCTL(IDC_ARCHIVE_END),
                        (!bRecordNum)&&(bUseEnd));

                    // Set the activation state of the last record checkbox
                    // It is active if the record # radio button is true
                   EnableWindow(GETHCTL(IDC_ARCHIVE_LAST), bRecordNum);
                }
                break;

                // Respond to MJD radio button
                case IDC_ARCHIVE_MJD:
                {
                    // Get state of record # radio button
                    bRecordNum=
                        (BOOL)Button_GetCheck(GETHCTL(IDC_ARCHIVE_NUM));
                    
                    // Set the activation state of the record # edit box
                    // It is active if the record # radio button is true
                    // and the last record checkbox is flase
                    EnableWindow(GETHCTL(IDC_ARCHIVE_RECORD),
                        (bRecordNum)&&(!bLastRecord));

                    // Set the activation state of the start MJD edit box
                    // It is active if the record # radio button is false
                    EnableWindow(GETHCTL(IDC_ARCHIVE_START), !bRecordNum);

                    // Set the activation state of the end MJD edit box
                    // It is active if the record # radio button is false
                    // and the end MJD checkbox is true
                    EnableWindow(GETHCTL(IDC_ARCHIVE_END),
                        (!bRecordNum)&&(bUseEnd));

                    // Set the activation state of the last record checkbox
                    // It is active if the record # radio button is true
                   EnableWindow(GETHCTL(IDC_ARCHIVE_LAST), bRecordNum);
                }
                break;

                // Read archive file to get tau and start MJD
                case IDC_ARCHIVE_READ:
                {
                    // Open archive data file
                    if((fpArch=fopen(szFile, "r")) == NULL)
                    {
                        // Error message
                        LoadString(hInst, IDS_MESSAGE_NOARCHIVE, szBuffer,
                            BUFFER_SIZE);
                        MessageBox(hwndMain, szBuffer, "Read Archive",
                            MB_ICONEXCLAMATION);

                        // Bail out
                        EndDialog (hDlg, 0);
                        break;
                    } 

                    // Read archive file to determine tau
                    // Set error count
                    nReadError=3;

                    // Read 1 line of file 
                    while(fgets(szBuffer, 81, fpArch) != NULL) 
                    {
                        // Scan line
                        sscanf(szBuffer, "%c", &cSymbol);
                        if(cSymbol=='$') // MJD line
                        {
                            // Scan again to parse 1st MJD string
                            sscanf(szBuffer, "%c %s", &cSymbol, szMJD);
                            fMJD1=atof(szMJD);

                            // Save start MJD
                            fFirstMJD=fMJD1;

                            // Decrement error count
                            nReadError-=1;

                            break;
                        }                                                         
                    }  

                    // Read another line of file
                    while(fgets(szBuffer, 81, fpArch) != NULL)
                    {
                        // Scan line
                        sscanf(szBuffer, "%c", &cSymbol);                      
                        if(cSymbol=='$') // MJD line
                        {
                            // Scan again to parse 2nd MJD string
                            sscanf(szBuffer, "%c %s", &cSymbol, szMJD);
                            fMJD2=atof(szMJD);

                            // Decrement error count again
                            nReadError-=1;

                            break;
                        }                                                         
                    }                                                          

                    // Find last MJD
                    // Position file pointer at end of file
                    // Decrement it until $ character is found
                    lOffset=0;
                    cSymbol=0;
                    while(cSymbol!='$')
                    {
                        if(fseek(fpArch, lOffset, SEEK_END))
                        {
                            // Seek error
                            // Note: This error trap can take a long time
                            // with a big NG file - so we also set a trap
                            // for a unreasonably large negative lOffset
                            break;
                        }
                        cSymbol=fgetc(fpArch);
                        lOffset--;
                        if(lOffset<-10000L)
                        {
                            // Too big an offset error
                            // The 10000L value corresponds to 125
                            // 80-character lines - more channels than
                            // any known TSC clock measuring system
                            break;
                        }
                    }

                    // reposition file pointer to read last MJD line
                    lOffset--;
                    fseek(fpArch, lOffset, SEEK_END);

                    // Read last MJD line of file
                    while(fgets(szBuffer, 81, fpArch) != NULL)
                    {
                        // Scan line
                        sscanf(szBuffer, "%c", &cSymbol);                      
                        if(cSymbol=='$') // MJD line 
                        {
                            // Scan again to parse last MJD string
                            sscanf(szBuffer, "%c %s", &cSymbol, szMJD);
                            fLastMJD=atof(szMJD);
                            break;
                        }                                                         
                    }

                    // Calc measurement interval in seconds
                    nSec=(int)(((fMJD2-fMJD1)*86400)+0.5);

                    // Check for OK interval
                    if(nSec>0)
                    {
                            // Decrement error count
                            nReadError-=1;
                    }

                    // Display error message if read NG
                    if(nReadError>0)
                    {
                        // Error message
                        LoadString(hInst, IDS_MESSAGE_NGARCHIVE, szBuffer,
                            BUFFER_SIZE);
                        MessageBox(hwndMain, szBuffer, "Read Archive",
                            MB_ICONEXCLAMATION);

                        // Bail out
                        EndDialog (hDlg, 0);
                        break;
                    }

                    // Close archive file
                    fclose(fpArch); 
                }
                break;
    
                // Respond to CR in an edit control
                // Note: No button is a DEFPUSHBUTTON
                case IDOK:
                {
                    // Does measurement channel # edit control have focus?
                    if(GetFocus()==GETHCTL(IDC_ARCHIVE_MEAS))
                    {
                        // Send EN_KILLFOCUS message to meas ch # edit ctrl
                        SendMessage(hDlg, WM_COMMAND,
                            GET_WM_COMMAND_MPS(IDC_ARCHIVE_MEAS, NULL,
                            EN_KILLFOCUS));
                    }

                    // Does reference channel # edit control have focus?
                    if(GetFocus()==GETHCTL(IDC_ARCHIVE_REF))
                    {
                        // Send EN_KILLFOCUS message to ref ch # edit control
                        SendMessage(hDlg, WM_COMMAND,
                            GET_WM_COMMAND_MPS(IDC_ARCHIVE_REF, NULL,
                            EN_KILLFOCUS));
                    }

                    // Does record # edit control have focus?
                    if(GetFocus()==GETHCTL(IDC_ARCHIVE_RECORD))
                    {
                        // Send EN_KILLFOCUS message to rec # edit control
                        SendMessage(hDlg, WM_COMMAND,
                            GET_WM_COMMAND_MPS(IDC_ARCHIVE_RECORD, NULL,
                            EN_KILLFOCUS));
                    }

                    // Does start edit control have focus?
                    if(GetFocus()==GETHCTL(IDC_ARCHIVE_START))
                    {
                        // Send EN_KILLFOCUS message to start edit control
                        SendMessage(hDlg, WM_COMMAND,
                            GET_WM_COMMAND_MPS(IDC_ARCHIVE_START, NULL,
                            EN_KILLFOCUS));
                    }

                    // Does end edit control have focus?
                    if(GetFocus()==GETHCTL(IDC_ARCHIVE_END))
                    {
                        // Send EN_KILLFOCUS message to end edit control
                        SendMessage(hDlg, WM_COMMAND,
                            GET_WM_COMMAND_MPS(IDC_ARCHIVE_END, NULL,
                            EN_KILLFOCUS));
                    }
                }
                break;

                case IDC_ARCHIVE_DISPLAY:
                {
                    strlcpy(szBuffer, szNotepad, sizeof(szBuffer));
                    strlcat(szBuffer, " ", sizeof(szBuffer));
                    strlcat(szBuffer, szFile, sizeof(szBuffer));
                    WinExec(szBuffer, SW_SHOWNORMAL);
                }
                break;

                case IDC_ARCHIVE_INDEX:
                {
					strlcpy(szBuffer, "Show index for archive file.",
						sizeof(szBuffer));
					WriteMessage(szBuffer);
					if(bNewArchive)
					{
						IndexArchive((double)fFirstMJD, (double)fLastMJD,
						szBuffer);
						bNewArchive=FALSE;
					}
					else // Show existing index file
					{
						strlcpy(szBuffer, szNotepad, sizeof(szBuffer));
						strlcat(szBuffer, " ", sizeof(szBuffer));
						strlcat(szBuffer, szArcIndexFile, sizeof(szBuffer));
						WinExec(szBuffer, SW_SHOWNORMAL);
					}	
               }
                break;

                case IDC_ARCHIVE_CONFIG:
                {
                    // Open dialog box to read & write config info
                    // for archive function in INI file
                    // Includes such items as # channels in meas system,
                    // default meas & ref chans, etc.

					// Open dialog box
					InvokeDialogBox(hInst, "ArcConfigBox", hDlg,
						(DLGPROC)ArcConfigDlgProc);

					// Update nMax from INI file
					strlcpy(szSection, "Archive", sizeof(szSection));
					nMax=GetPrivateProfileInt(szSection, "Num", 4, szIniFile);
               }
                break;

                case IDC_ARCHIVE_ACCEPT:
                {
                    // Here's where we call ReadArchiveFile() to read the
                    // archive data into the phase or freq data arrays
 
 					// Set nRecord=0 to seek last record
					if(bLastRecord)
					{
						nRecord=0;
					}

					// If MJD limits are chosen, force Record # to -1
					if(!bRecordNum)
					{
						nRecord=-1;
					}

                    // Read data from archive file
                    ReadArchiveFile(nMeas, nRef, nRecord, (F_TYPE)nSec,
                        fFirstMJD, fLastMJD, fMJDStart, fMJDEnd);
                }

                // Fall into cancel
                // Respond to esc key or cancel button
                case IDCANCEL:
                case IDC_ARCHIVE_CANCEL:
                {
					bNewArchive=TRUE; // Force new arcindex next time
					EndDialog (hDlg, 0);
                }
                return TRUE;
            }
        }
        break;
     }
     return FALSE;
}

/*****************************************************************************/
#endif
#endif