/*****************************************************************************/
/*                                                                           */
/*                                CalcStarB1()                               */
/*                                                                           */
/*   Function to calculate the Barnes B1 ratio of the N-sample to 2-sample   */
/*   (standard to Allan) variance for frequency data treated as phase data   */
/*   which is used to resolve alpha=3 or -4 ambiguity for Hadamard sigmas.   */
/*   This is called *B1 to distinguish it from the normal B1 ratio.          */
/*                                                                           */
/*   Parameters:    F_TYPE  *fPhase         Phase data array                 */
/*                                          or NULL if freq array used       */
/*                  F_TYPE  *fFreq          Frequency data array             */
/*                                          or NULL if phase array used      */
/*                  int     fTau              
/*                  int     nAF             Averaging factor                 */
/*                                                                           */
/*   Return:        float   fB             *B1 ratio                        */
/*                                          or -1.0 if error                 */
/*                                                                           */
/*   Notes:         What we are doing here is to use Chuck Greenhall's       */
/*                  idea to calculate the B1 ratio for the freq data         */
/*                  as phase data, and use the standard noise ID method      */
/*                  except for adding 2 to the resulting alpha.              */
/*                                                                           */
/*   References:    1.  C.A. Greenhall, (private communication)              */
/*                  2.  D. Howe, et al PTTI 2000 paper                       */
/*                                                                           */
/*****************************************************************************/
 
float WINAPI CalcStarB1(F_TYPE *fPhaseData, F_TYPE *fFreqData, F_TYPE fTau,
    int nAF)
{
    F_TYPE *fWork;   // Pointer to working data array
    F_TYPE fSig;     // Allan deviation
    F_TYPE fStdfDev; // Standard deviation
    int nNum         // # data points 
    
    // Get # data points in active data array
    if(fPhase) // Phase data
    {
        nNum=fPhase[0];
    }
    else //Freq data
    {
        nNum=fFreq[0];
    }

    // Allocate working array of freq data
    // Data is subject to being averaged, so must use a working array
    if((fWork=(F_TYPE*)malloc((nNum+ARRAY_OFFSET)*sizeof(F_TYPE)))==NULL)
    {
        // Error
        return -1.0;
    }

    // For phase data, convert it to freq data in working array
    if(fPhase)
    {
        // Conv phase data to freq data
        ConvPhaseToFreq(fPhase, fWork, fTau);
    }
    else // Freq data - no conversion needed
    {
        // Copy freq data to working array
        for(i=0; i<nNum+ARRAY_OFFSET; i++)
        {
            fWork[i]=fFreq[i];
        }
    }
                        
    // Average working freq data
    AverageFreqData(fWork, nAF);

    // Calc its sigma and std dev as if it were phase data
    CalcPhaseSigma(fWork, &fSig, fTau);
    CalcPhaseStdDev(fWork, &fStdDev, fTau);

    // Calc & return *B1 = ratio of standard to Allan variance
    return (float)(SQR(fStdDev/fSig));
}

/*****************************************************************************/


