/*****************************************************************************/
/*                                                                           */
/*                            STABLE_G.C                                     */
/*                                                                           */
/* 		Stable Windows 95/NT GraphiC Functions                               */
/*                                                                           */
/* 		Adapted from GPCWAPP.C                                               */
/*                                                                           */
/*      GPCWndProc()                                                         */
/*                                                                           */
/* (c) 1996-2007  W. Riley Hamilton Technical Services  All Rights Reserved  */
/*                                                                           */
/*****************************************************************************/

#define STRICT 					// For Win32 compatibility

#include "stable_w.h"			// STABLE_W header file

/*****************************************************************************/
/*                                                                           */
/*      GPCWndProc()                                                         */
/*                                                                           */
/* 		GraphiC Window Procedure                                             */
/*                                                                           */
/* 		Adapted from GPCWndProc() of GPCWAPP.C                               */
/*                                                                           */
/*      (c) 1996  W. Riley Hamilton Technical Services All Rights Reserved   */
/*                                                                           */
/*		Revision record:                                                     */
/*			12/22/96	Added code to switch to Stable/Win working directory */
/*                      during GraphiC plotting                              */
/*			06/29/97	Use GET_WM_COMMAND_ID, etc. WIN32 compatibility      */
/*						macros to process WM_COMMAND messages:               */
/*						wParam <- GET_WM_COMMAND_ID(wParam, lParam)          */
/*			07/12/97	Made more Win 32 compatibility changes               */
/*						Will need 32-bit version of GraphiC to complete      */
/*			12/24/97	Made changes for OK BC 32 bit compliation            */
/*			09/26/02	Added provisions for timetag x-y plot                */
/*			05/20/04	Eliminate copying bitmaps and metafiles to clipboard */
/*			05/27/04	Added new code for copying bitmaps and metafiles     */
/*			11/19/05	Activated DVAR plotting                              */
/*          07/22/07    Changed to safe string functions                     */
/*                                                                           */
/*****************************************************************************

	FUNCTION: GPCWndProc(HWND, UINT, UINT, LONG)

	PURPOSE:  Processes messages

	MESSAGES:

	WM_COMMAND    - application menu (About dialog box)
	WM_CREATE     - create window and objects
	WM_PAINT      - update window, draw objects
	WM_DESTROY    - destroy window

	COMMENTS:

	Handles to the objects you will use are obtained when the WM_CREATE
	message is received, and deleted when the WM_DESTROY message is
	received.  The actual drawing is done whenever a WM_PAINT message is
	received.

****************************************************************************/
LONG WINAPI GPCWndProc(HWND hWnd, UINT message, UINT wParam, LONG lParam)
{
	#include "gpcwin.h" // Include to use struct gpcwin_data
	
	// External GPCWDRVN.DLL GraphiC function prototype
	// (copied from GPCWIN.H)
	extern void FTYPE CopyMetaq(HWND hWnd);
	extern void WMFCopyq(HWND hWnd);
	extern struct _GPCWIN_DATA gpcwin_data; // Needed to use drawflgq, etc.

	// Local variables
	// Ints
	int i;						// General index
	static int nSize;			// Full original data array size
	static int nInterruptions;	// # data interruptions
	static int nMissingPts;		// # missing points
	#if(1) // New code 08/24/07
	static nFloat;				// Size of float arrays
	#endif
	// BOOLs
	static BOOL bHistoFirstTime=TRUE;
	static BOOL bSpecFirstTime=TRUE;
	static BOOL bAutoFirstTime=TRUE;
	static BOOL bScatterFirstTime=TRUE;
	// Flags to allow use of CopyMetaq() for only one GraphiC plot
	// bCopyMetaUsed is set when copy metafile to clipboard is used
	// If that flag is set, bCopyMetaAvailable is cleared when plot is closed
	// Copy metafile is then no longer available for use
	static BOOL	bCopyMetaAvailable=TRUE;
	static BOOL bCopyMetaUsed=FALSE;
	// F_TYPES
	static F_TYPE *fD;				// Pointer to saved data array
	static F_TYPE *fT;				// Pointer to saved timetag array
	// char szBuffer[BUFFER_SIZE+1];   // Text buffer

	// Windows Types
	HWND hDlg;				// Local version of hDialog for GETHCTL macro
	//HWND hPlot;			// Plot window handle
							// Removed 11/28/05 - Don't want this local hPlot
							// to replace global one
	// DWORD dwVer;			// For op system version test

	// Start of code
	hPlot=hWnd;             // Save plot window handle as global
    hDlg=hDialog;			// Copy global dialog box handle
	hGPC=hWnd;				// Copy global Graphic main window handle

	switch(message)
	{
		// Process GraphiC/Win COMMAND messages
		case WM_COMMAND:
			// Both COPY and COPYMETA have problems
			// Apparently due to bugs in GraphiC itself
			// Better to trap them rather than have them defective

			// This is alternative to GraphiC copy plot to clipboard
			// as wmf (which has problem that causes program to hang & crash)
			// This code has a trap to prevent copying to clipboard
			// after first GraphiC plot
			if(wParam==IDM_COPYMETA)
			{
				// This also hangs up when used more than once
				// It's the same function that causes trouble in GraphiC
				if(bCopyMetaAvailable)
				{
					WMFCopyq(hWnd);  // Full function with progress indicator	

					// Set used flag
					// Can only copy metafile for one plot w/o trouble
					bCopyMetaUsed=TRUE; 
				}
				else
				{
					MessageBox(NULL, "Copy Metafile not available",
						"Copy Metafile", MB_OK | MB_ICONEXCLAMATION);
				}
				break;
			}
			
			// This is alternative to GraphiC copy plot to clipboard
			// as bitmap (which doesn't work - gives all black area)
			if(wParam==IDM_COPY)
			{
				hPlot=GetForegroundWindow();
				CopyDialog(hPlot, 1000, 1000);
				break;
			}

			// Intercept GraphiC help command
			if(wParam==IDM_HELP_INDEX)
			{
				// Call HTML GraphiC help
				// This opens the Stable32 Microsoft HTML help file
				// to the GraphiC topic
				#if(0)
				HtmlHelp(hWnd, "Stable32.chm::GRAPHIC.HTM", HH_DISPLAY_TOPIC,
				0L);
				#endif

				#if(0) // New code 11/29/11
				// Invoke help file
				strlcpy(szBuffer, szHelpFile, sizeof(szBuffer));
				strlcat(szBuffer, "::GRAPHIC.HTM", sizeof(szBuffer));
				HtmlHelp(hDlg, szBuffer, HH_DISPLAY_TOPIC, 0L);
				#endif

				#if(1) // Trial code 11/30/11
				// Invoke help file using CallHelp() function
				CallHelp(hDlg, "GRAPHIC");
				#endif

				break;
			}

			// Intercept GraphiC using help command
			if(wParam==IDM_HELP_HELP)
			{
				// Call HTML using help
				// This opens a Microsoft HTML help file
				// that provides help on help
				// HonH.chm is based on MS Viewhlp.chm
				HtmlHelp(hWnd, "HonH.chm", HH_DISPLAY_TOPIC, 0L);
				break;
			}

			// Process non GraphiC/Win WM_COMMAND messages by intercepting
			// other wParam messages
			switch(GET_WM_COMMAND_ID(wParam, lParam))
			{
				// Process other messages in normal way
				default:
				{
					// Process other messages in BAUXWIN.C
					GPCWINCommProc(hWnd, LOWORD(wParam), lParam);
				}
				break;
			}
			break;

		case WM_CREATE:
		{
			// Post message to start GraphiC application function
			PostMessage(hWnd, WM_USER_START, (WPARAM)0, (LPARAM)0);
		}
		break;

		case WM_USER_START:
		{
			GPCWINStart(hWnd);
		}
		break;

		case WM_USER_REDRAW:
		{
			GPCWINReDraw(hWnd, wParam, lParam);
		}
		break;

		case WM_USER_CANCEL:
		{
			GPCWINCancel();
		}
		break;

		case WM_USER_MAIN:
		{
			// Before plot, reset path to Stable/Win working directory.
			// GraphiC reads its fonts and write its *.TKF files here.

			// Prevent GraphiC use of floppy for temp file
			// Get current drive # (1=A, etc.)
			// Display drive where program opened (letter + colon e.g. C:)
			// Convert drive letter into int
			// Change drive to one where program opened
			_chdrive((int)((szDrive[0]-'A')+1));

			// Here's the story as I understand it:
			// Under Windows 98, when doing a GraphiC plot conversion
			// the operating system appears to set the path in the Save As
			// dialog box for the WMF file to the documents folder
			// which defaults to My Documents
			// The old GraphiC code can't handle the long directory name
			// and it isn't automatically shortened to mydocu~1
			// so it can't write the metafile.
			// The user must change/shorten the path to do the conversion
			// or just use the Edit Copy (Metafile) to write it to the
			// clipboard.  What is really needed is a way to make the
			// operating system not change the path - don't know how
			// especially within the GraphiC code.
			// Note: It is not absolutely necessary to restore the original
			// current drive, especially since it is rare that it is changed

			// Call selected GraphiC plotting function
			switch(nPlotType)
			{
				case PHASE_PLOT:
				{
					// Make sure plot info exists
					if((strcmp(sFPlot.x_label, ""))==0)
					{
						ReadIni(INI_FREQ);
					}

					// Check if we are doing a timetag x-y plot
					if(nPPlotX==X_TIMETAG)
					{
						// Find full size of original data array
						nSize=(int)fData[0]+ARRAY_OFFSET;

						// Allocate memory to save original data
						// and timetag arrays
						if((fD=(F_TYPE*)malloc(nSize*
							sizeof(F_TYPE)))==NULL)
						{
							AllocErrorMessage();
							break;
						}
						if((fT=(F_TYPE*)malloc(nSize*
							sizeof(F_TYPE)))==NULL)
						{
							AllocErrorMessage();
							free(fD);
							break;
						}

						// Save the original data & timetags
						for(i=0; i<nSize; i++)
						{
							fD[i]=fPhaseData[i];
							fT[i]=fPhaseTimetags[i];
						}

						// Find the # of missing points
						RegularizeCount(fPhaseData, fPhaseTimetags,
							&nInterruptions, &nMissingPts);
						
						// Realloc tags to new # required
						if((fNew=(F_TYPE*)realloc(fPhaseTimetags,
							(nSize+nMissingPts+ARRAY_OFFSET+EXTRA_MEM)
							*sizeof(F_TYPE)))==NULL)
						{
							AllocErrorMessage();
							break;
 						}
						else
						{
							fPhaseTimetags=fNew;
						}

						// Realloc data to new # required
						if((fNew=(F_TYPE*)realloc(fPhaseData,
							(nSize+nMissingPts+ARRAY_OFFSET+EXTRA_MEM)
							*sizeof(F_TYPE)))==NULL)
						{
							AllocErrorMessage();
							break;
 						}
						else
						{
							fPhaseData=fNew;
						}

						// Insert missing points in data & timetags
						RegularizeInsert(fPhaseData, fPhaseTimetags);

						// Force new scale determination
						bPhaseScaleChanged=TRUE;

						// Clear default flag
						bPXDefault=FALSE;
						
						// Clear Lines
						// Note: This done to avoid incorrect lines
						// because of unregularized data
						// Line fits from options box can still be used
						ClearLines();
					}

					// Make sure plot scale exists
					if( (sPPlot.y_min==0.0) && (sPPlot.y_max==0.0) )
					{
						bPhaseScaleChanged=TRUE;
					}

					// Reinitialize when data changes
					if(bPhaseScaleChanged)
					{
						// Initialize phase plot parameters
						InitPhasePlotParams(1);

						// Reset scale recalc flag
						bPhaseScaleChanged=FALSE;
					}

					// Redetermine X1000 x-scale after data is "expanded"
					// and scale reinitialized
					if((nPPlotX==X_TIMETAG) &&
						((sPPlot.x_max-sPPlot.x_min)>10000))
					{
						sPPlot.x_mult=(float)0.001;
						strlcpy(sPPlot.x_label, "Timetag Point x1000",
							XLABELSIZE);
						sPPlot.x_max=(float)(sPPlot.x_mult*(fPhaseData[2]+
							sPPlot.x_offset));
						sPPlot.x_min=(float)(sPPlot.x_mult*(fPhaseData[1]+
							sPPlot.x_offset));

						// Determine x-axis plot scale
						DetermineXScale(&sPPlot);
					}

					// Do phase plot
					PhasePlot(fPhaseData, &sPPlot);

					// Check if we are doing a timetag x-y plot
					if(nPPlotX==X_TIMETAG)
					{
						// Free regularized arrays
						free(fPhaseData);
						free(fPhaseTimetags);

						// Reassign array pointers to saved data
						fPhaseData=fD;
						fPhaseTimetags=fT;
						fData=fD;
						fTags=fT;

						// Force new scale determination
						bPhaseScaleChanged=TRUE;
					}

					// Redraw status plots in case covered by GraphiC plot
					bPhaseRedraw=TRUE;
					bFreqRedraw=TRUE;

					// Redraw status plots from bitmap
					UpdateStatusBar(hwndMain, OLD_DATA);

					// Reset redraw flags
					bPhaseRedraw=FALSE;
					bFreqRedraw=FALSE;
				}
				break;

				case FREQ_PLOT:
				{
					// Make sure plot info exists
					if((strcmp(sFPlot.x_label, ""))==0)
					{
						ReadIni(INI_FREQ);
					}

					// Check if we are doing a timetag x-y plot
					if(nFPlotX==X_TIMETAG)
					{
						// Find full size of original data array
						nSize=(int)fData[0]+ARRAY_OFFSET;

						// Allocate memory to save original data
						// and timetag arrays
						if((fD=(F_TYPE*)malloc(nSize*
							sizeof(F_TYPE)))==NULL)
						{
							AllocErrorMessage();
							break;
						}
						if((fT=(F_TYPE*)malloc(nSize*
							sizeof(F_TYPE)))==NULL)
						{
							AllocErrorMessage();
							free(fD);
							break;
						}

						// Save the original data & timetags
						for(i=0; i<nSize; i++)
						{
							fD[i]=fFreqData[i];
							fT[i]=fFreqTimetags[i];
						}

						// Find the # of missing points
						RegularizeCount(fFreqData, fFreqTimetags,
							&nInterruptions, &nMissingPts);
						
						// Realloc tags to new # required
						if((fNew=(F_TYPE*)realloc(fFreqTimetags,
							(nSize+nMissingPts+ARRAY_OFFSET+EXTRA_MEM)
							*sizeof(F_TYPE)))==NULL)
						{
							AllocErrorMessage();
							break;
 						}
						else
						{
							fFreqTimetags=fNew;
						}

						// Realloc data to new # required
						if((fNew=(F_TYPE*)realloc(fFreqData,
							(nSize+nMissingPts+ARRAY_OFFSET+EXTRA_MEM)
							*sizeof(F_TYPE)))==NULL)
						{
							AllocErrorMessage();
							break;
 						}
						else
						{
							fFreqData=fNew;
						}

						// Insert missing points in data & timetags
						RegularizeInsert(fFreqData, fFreqTimetags);

						// Force new scale determination
						bFreqScaleChanged=TRUE;

						// Clear default flag
						bFXDefault=FALSE;
						
						// Clear Lines
						// Note: This done to avoid incorrect lines
						// because of unregularized data
						// Line fits from options box can still be used
						ClearLines();
					}

					// Make sure plot scale exists
					if( (sFPlot.y_min==0.0) && (sFPlot.y_max==0.0) )
					{
						bFreqScaleChanged=TRUE;
					}

					// Reinitialize when data changes
					if(bFreqScaleChanged)
					{
						// Initialize freq plot parameters
						InitFreqPlotParams(1);

						// Reset scale recalc flag
						bFreqScaleChanged=FALSE;
					}

					// Redetermine X1000 x-scale after data is "expanded"
					// and scale reinitialized
					if((nFPlotX==X_TIMETAG) &&
						((sFPlot.x_max-sFPlot.x_min)>10000))
					{
						sFPlot.x_mult=(float)0.001;
						strlcpy(sFPlot.x_label, "Timetag Point x1000",
							XLABELSIZE);
						sFPlot.x_max=(float)(sFPlot.x_mult*(fFreqData[2]+
							sFPlot.x_offset));
						sFPlot.x_min=(float)(sFPlot.x_mult*(fFreqData[1]+
							sFPlot.x_offset));

						// Determine x-axis plot scale
						DetermineXScale(&sFPlot);
					}

					// Do freq plot
					FreqPlot(fFreqData, fPhaseData, &sFPlot);

					// Check if we are doing a timetag x-y plot
					if(nFPlotX==X_TIMETAG)
					{
						// Free regularized arrays
						free(fFreqData);
						free(fFreqTimetags);

						// Reassign array pointers to saved data
						fFreqData=fD;
						fFreqTimetags=fT;
						fData=fD;
						fTags=fT;

						// Force new scale determination
						bFreqScaleChanged=TRUE;
					}

					// Redraw status bar plots in case they got covered
					// by GraphiC plot
					bPhaseRedraw=TRUE;
					bFreqRedraw=TRUE;

					// Redraw status plots from bitmap
					UpdateStatusBar(hwndMain, OLD_DATA);

					// Reset redraw flags
					bPhaseRedraw=FALSE;
					bFreqRedraw=FALSE;
				}
				break;

				case SIGMA_PLOT:
				case DOMAIN_SIGMA_PLOT:
				{
					// Disable buttons
					EnableWindow(GETHCTL(IDC_RUN_RUN), FALSE);
					EnableWindow(GETHCTL(IDCANCEL), FALSE);
					EnableWindow(GETHCTL(IDC_RUN_PLOT), FALSE);
					EnableWindow(GETHCTL(IDC_RUN_PRINT), FALSE);
					EnableWindow(GETHCTL(IDC_RUN_OPTIONS), FALSE);
					EnableWindow(GETHCTL(IDC_RUN_LINES), FALSE);
					EnableWindow(GETHCTL(IDC_RUN_NOTES), FALSE);
					EnableWindow(GETHCTL(IDC_RUN_REMOVE), FALSE);
					EnableWindow(GETHCTL(IDHELP), FALSE);

					// Do sigma plot w/o autoscaling - scales set in Options
					SigmaPlot(&sSPlot, FALSE, FALSE, FALSE);

					// Redraw status bar plots in case they got covered
					// by GraphiC plot
					bPhaseRedraw=TRUE;
					bFreqRedraw=TRUE;

					// Redraw status plots from bitmap
					UpdateStatusBar(hwndMain, OLD_DATA);

					// Reset redraw flags
					bPhaseRedraw=FALSE;
					bFreqRedraw=FALSE;

					// Enable buttons
					EnableWindow(GETHCTL(IDC_RUN_RUN), TRUE);
					EnableWindow(GETHCTL(IDCANCEL), TRUE);
					EnableWindow(GETHCTL(IDC_RUN_PLOT), TRUE);

					// Added 2nd test 01/28/01
					if(!bAllTau || nSigmaPlotType==MTIE_RUN)
					{
						EnableWindow(GETHCTL(IDC_RUN_PRINT), TRUE);
					}
					EnableWindow(GETHCTL(IDC_RUN_OPTIONS), TRUE);
					EnableWindow(GETHCTL(IDC_RUN_LINES), TRUE);
					EnableWindow(GETHCTL(IDC_RUN_NOTES), TRUE);
					EnableWindow(GETHCTL(IDC_RUN_REMOVE), TRUE);
					EnableWindow(GETHCTL(IDHELP), TRUE);
				}
				break;

				#ifndef LITE_VER
				case READ_PLOT:
				{
					// Disable buttons
					EnableWindow(GETHCTL(IDOK), FALSE);
					EnableWindow(GETHCTL(IDCANCEL), FALSE);
					EnableWindow(GETHCTL(IDC_READ_PLOT), FALSE);
					EnableWindow(GETHCTL(IDC_READ_PRINT), FALSE);
					EnableWindow(GETHCTL(IDC_READ_OPTIONS), FALSE);
					EnableWindow(GETHCTL(IDC_READ_LINES), FALSE);
					EnableWindow(GETHCTL(IDC_READ_NOTES), FALSE);
					EnableWindow(GETHCTL(IDHELP), FALSE);

					// Do sigma plot w/o autoscaling - scales set in Options
					SigmaPlot(&sSPlot, FALSE, FALSE, FALSE);

					// Redraw status bar plots in case they got covered
					// by GraphiC plot
					bPhaseRedraw=TRUE;
					bFreqRedraw=TRUE;

					// Redraw status plots from bitmap
					UpdateStatusBar(hwndMain, OLD_DATA);

					// Reset redraw flags
					bPhaseRedraw=FALSE;
					bFreqRedraw=FALSE;

					// Enable buttons
					EnableWindow(GETHCTL(IDOK), TRUE);
					EnableWindow(GETHCTL(IDCANCEL), TRUE);
					EnableWindow(GETHCTL(IDC_READ_PLOT), TRUE);
					EnableWindow(GETHCTL(IDC_READ_PRINT), TRUE);
					EnableWindow(GETHCTL(IDC_READ_OPTIONS), TRUE);
					EnableWindow(GETHCTL(IDC_READ_LINES), TRUE);
					EnableWindow(GETHCTL(IDC_READ_NOTES), TRUE);
					EnableWindow(GETHCTL(IDHELP), TRUE);
				}
				break;
				#endif

				#ifndef LITE_VER
				case HISTO_PLOT:
				{
					// Disable buttons
					EnableWindow(GETHCTL(IDC_HPLOT_CANCEL), FALSE);
					EnableWindow(GETHCTL(IDC_HPLOT_PLOT), FALSE);
					EnableWindow(GETHCTL(IDC_HPLOT_RESET), FALSE);
					EnableWindow(GETHCTL(IDHELP), FALSE);

					// Initializations
					if(bHistoFirstTime)
					{
						// Variable initializations

						// Reset initialization flag
						bHistoFirstTime=FALSE;
					}

					// Gray the plot menu items & toolbutton
					// Only one plot instance allowed
					EnableMenuItem(hMenu, IDM_HIST, MF_GRAYED);
					tb[TB_HIST].state=TOOLBUTTON_DIM;
					RedrawToolbar(tb, FALSE);

					HistoPlot(fBins, fHist, sHPlot.num, &sHPlot);

					// Redraw status bar plots in case they got covered
					// by GraphiC plot
					bPhaseRedraw=TRUE;
					bFreqRedraw=TRUE;

					// Redraw status plots from bitmap
					UpdateStatusBar(hwndMain, OLD_DATA);

					// Reset redraw flags
					bPhaseRedraw=FALSE;
					bFreqRedraw=FALSE;

					// Reenable the plot menu items & toolbutton
					EnableMenuItem(hMenu, IDM_HIST, MF_ENABLED);
					tb[TB_HIST].state=TOOLBUTTON_OUT;
					RedrawToolbar(tb, FALSE);

					// Reenable buttons
					EnableWindow(GETHCTL(IDC_HPLOT_CANCEL), TRUE);
					EnableWindow(GETHCTL(IDC_HPLOT_PLOT), TRUE);
					EnableWindow(GETHCTL(IDC_HPLOT_RESET), TRUE);
					EnableWindow(GETHCTL(IDHELP), TRUE);
				}
				break;
				#endif

				#ifndef LITE_VER
				case SPECT_PLOT:
				case DOMAIN_PSD_PLOT:
				{
					// Disable buttons
					EnableWindow(GETHCTL(IDC_PSPLOT_CANCEL), FALSE);
					EnableWindow(GETHCTL(IDC_PSPLOT_PLOT), FALSE);
					EnableWindow(GETHCTL(IDC_PSPLOT_RESET), FALSE);
					EnableWindow(GETHCTL(IDHELP), FALSE);

					// Initializations
					if(bSpecFirstTime)
					{
						// Variable initializations

						// Reset initialization flag
						bSpecFirstTime=FALSE;
					}

					// Gray the plot menu items & toolbutton
					// Only one plot instance allowed
					EnableMenuItem(hMenu, IDM_POWER, MF_GRAYED);
					tb[TB_POWER].state=TOOLBUTTON_DIM;
					RedrawToolbar(tb, FALSE);

					// Plot power spectrum
					// freq data in Hz is in ydata array from 0 to n/2
					// log power spectrum data is in xdata array from 0 to n/2
					// tau is interval between frequency data samples
					// n is # frequency data points
					// total time of data record is T=n*tau
					// Fourier frequency spacing is 1/T
					// maximum Fourier frequency is 1/(2*tau)
					// probably want to ignore the dc term at xdata and
					// ydata [0]
					// fFreq[]   = array of Fourier freq data
					// fPSD[]    = array of power spectral data
					// .nfft     = # power spectrum points
					// .plotype  = PSD type: 0=Sx(f), 1=S(f), 2=L(f), 0=Sy(f)
					// The Sy(f) code of 0 is converted to 3 in call
					// &sPSPlot  = plotdata structure

					// Do power spectrum plot
					PowerSpectrumPlot(fFreq, fPSD, (sPSPlot.nfft/2)-1,
						sPSPlot.plotype+bDataType*3, &sPSPlot);

					// Redraw status bar plots in case they got covered
					// by GraphiC plot
					bPhaseRedraw=TRUE;
					bFreqRedraw=TRUE;

					// Redraw status plots from bitmap
					UpdateStatusBar(hwndMain, OLD_DATA);

					// Reset redraw flags
					bPhaseRedraw=FALSE;
					bFreqRedraw=FALSE;

					// Reenable the plot menu items & toolbutton
					EnableMenuItem(hMenu, IDM_POWER, MF_ENABLED);
					tb[TB_POWER].state=TOOLBUTTON_OUT;
					RedrawToolbar(tb, FALSE);

					// Reenable buttons
					EnableWindow(GETHCTL(IDC_PSPLOT_CANCEL), TRUE);
					EnableWindow(GETHCTL(IDC_PSPLOT_PLOT), TRUE);
					EnableWindow(GETHCTL(IDC_PSPLOT_RESET), TRUE);
					EnableWindow(GETHCTL(IDHELP), TRUE);
				}
				break;
				#endif

				#ifndef LITE_VER
				case AUTO_PLOT:
				{
					// Disable buttons
					EnableWindow(GETHCTL(IDC_APLOT_CANCEL), FALSE);
					EnableWindow(GETHCTL(IDC_APLOT_PLOT), FALSE);
					EnableWindow(GETHCTL(IDC_APLOT_RESET), FALSE);
					EnableWindow(GETHCTL(IDHELP), FALSE);

					// Initializations
					if(bAutoFirstTime)
					{
						// Variable initializations

						// Reset initialization flag
						bAutoFirstTime=FALSE;
					}

					// Gray the plot menu items & toolbutton
					// Only one plot instance allowed
					EnableMenuItem(hMenu, IDM_AUTO, MF_GRAYED);
					tb[TB_ACS].state=TOOLBUTTON_DIM;
					RedrawToolbar(tb, FALSE);

					if(bCmdA)
					{
						sAPlot.do_table=FALSE;
					}
					
					// Plot autocorrelation
					AutocorrelationPlot(fAuto, &sAPlot);

					// Redraw status bar plots in case they got covered
					// by GraphiC plot
					bPhaseRedraw=TRUE;
					bFreqRedraw=TRUE;

					// Redraw status plots from bitmap
					UpdateStatusBar(hwndMain, OLD_DATA);

					// Reset redraw flags
					bPhaseRedraw=FALSE;
					bFreqRedraw=FALSE;

					// Reenable the plot menu items & toolbutton
					EnableMenuItem(hMenu, IDM_AUTO, MF_ENABLED);
					tb[TB_ACS].state=TOOLBUTTON_OUT;
					RedrawToolbar(tb, FALSE);

					// Reenable buttons
					EnableWindow(GETHCTL(IDC_APLOT_CANCEL), TRUE);
					EnableWindow(GETHCTL(IDC_APLOT_PLOT), TRUE);
					EnableWindow(GETHCTL(IDC_APLOT_RESET), TRUE);
					EnableWindow(GETHCTL(IDHELP), TRUE);
				}
				break;
				#endif

				#ifndef LITE_VER
				case SCATTER_PLOT:
				{
					// Disable buttons
					EnableWindow(GETHCTL(IDC_APLOT_CANCEL), FALSE);
					EnableWindow(GETHCTL(IDC_APLOT_PLOT), FALSE);
					EnableWindow(GETHCTL(IDC_APLOT_SCATTER), FALSE);
					EnableWindow(GETHCTL(IDC_APLOT_RESET), FALSE);
					EnableWindow(GETHCTL(IDHELP), FALSE);

					// Initializations
					if(bAutoFirstTime)
					{
						// Variable initializations

						// Reset initialization flag
						bScatterFirstTime=FALSE;
					}

					// Gray the plot menu items & toolbutton
					// Only one plot instance allowed
					EnableMenuItem(hMenu, IDM_AUTO, MF_GRAYED);
					tb[TB_ACS].state=TOOLBUTTON_DIM;
					RedrawToolbar(tb, FALSE);

					// Plot lag scatter
					LagScatterPlot(fData, &sAPlot);

					// Redraw status bar plots in case they got covered
					// by GraphiC plot
					bPhaseRedraw=TRUE;
					bFreqRedraw=TRUE;

					// Redraw status plots from bitmap
					UpdateStatusBar(hwndMain, OLD_DATA);

					// Reset redraw flags
					bPhaseRedraw=FALSE;
					bFreqRedraw=FALSE;

					// Reenable the plot menu items & toolbutton
					EnableMenuItem(hMenu, IDM_AUTO, MF_ENABLED);
					tb[TB_ACS].state=TOOLBUTTON_OUT;
					RedrawToolbar(tb, FALSE);

					// Reenable buttons
					EnableWindow(GETHCTL(IDC_APLOT_CANCEL), TRUE);
					EnableWindow(GETHCTL(IDC_APLOT_PLOT), TRUE);
					EnableWindow(GETHCTL(IDC_APLOT_SCATTER), TRUE);
					EnableWindow(GETHCTL(IDC_APLOT_RESET), TRUE);
					EnableWindow(GETHCTL(IDHELP), TRUE);
				}
				break;
				#endif

				case DAVAR_PLOT:
				{
					DynamicSigmaPlot(&sDPlot);
				}
				break;
			}
		}
		break;

		case WM_CHAR:
		{
			GPCWINCharinq(wParam);
		}
		break;

		case WM_KEYDOWN:
		{
			GPCWINCurkeyq(wParam);
		}
		break;

		case WM_LBUTTONDOWN:
		{
			GPCWINLButtProc(hWnd, lParam);
		}
		break;

		case WM_MOUSEMOVE:
		{
			GPCWINMouseMProc(hWnd, lParam);
		}
		break;

		case WM_RBUTTONDOWN:
		{
			GPCWINRButtProc(hWnd);
		}
		break;

		case WM_SIZE:
		{
			GPCWINSizeProc(hWnd, wParam);
		}
		break;

		case WM_PAINT:
		{
			GPCWINPaintProc(hWnd);
		}
		break;

		case WM_ACTIVATE:
		{
			GPCWINActProc(hWnd, wParam, lParam);
		}
		break;

		case WM_SYSCOMMAND:
		{
			return(GPCWINSysCommand(hWnd, message, wParam, lParam));
		}

		case WM_CLOSE:
		{
			// This invoked first when plot window closed - window still
			// visible here

			if( (nPlotType==PHASE_PLOT) || (nPlotType==FREQ_PLOT) )
			{
				// Enable plot dialog box buttons
				EnableWindow(GETHCTL(IDCANCEL), TRUE);
				EnableWindow(GETHCTL(IDC_PLOT_PLOT), TRUE);
				EnableWindow(GETHCTL(IDC_PLOT_OPTIONS), TRUE);
				EnableWindow(GETHCTL(IDC_PLOT_LINES), TRUE);
				EnableWindow(GETHCTL(IDC_PLOT_NOTES), TRUE);
				EnableWindow(GETHCTL(IDHELP), TRUE);
			}

			GPCWINClose(hWnd);
		}
		break;

		case WM_DESTROY:
		{
			// This invoked second when plot window closed - window gone here

			if( (nPlotType==PHASE_PLOT) || (nPlotType==FREQ_PLOT) )
			{
				// Enable plot dialog box buttons
				EnableWindow(GETHCTL(IDCANCEL), TRUE);
				EnableWindow(GETHCTL(IDC_PLOT_PLOT), TRUE);
				EnableWindow(GETHCTL(IDC_PLOT_OPTIONS), TRUE);
				EnableWindow(GETHCTL(IDC_PLOT_LINES), TRUE);
				EnableWindow(GETHCTL(IDC_PLOT_NOTES), TRUE);
				EnableWindow(GETHCTL(IDHELP), TRUE);
			}
			
			GPCWINDestroy(hWnd);

			// IMPORTANT!!!
			// This seems to fix the "disappearing Stable program" problem
			// Without this, if Stable is opened, a file opened and plotted,
			// and then the GraphiC Print setup is invoked, when the plot is
			// closed Stable/Win "goes away" and Windows has to be restarted
			// to run it again.  With this SetFocus() all seems OK.  Whew!
			// Note: Doesn't seem necessary any more
			SetFocus(hDialog);

			if(bCopyMetaUsed)
			{
				bCopyMetaAvailable=FALSE;
			}
		}
		break;

		case WM_QUIT:
		{
			if( (nPlotType==PHASE_PLOT) || (nPlotType==FREQ_PLOT) )
			{
				// Enable plot dialog box buttons
				EnableWindow(GETHCTL(IDCANCEL), TRUE);
				EnableWindow(GETHCTL(IDC_PLOT_PLOT), TRUE);
				EnableWindow(GETHCTL(IDC_PLOT_OPTIONS), TRUE);
				EnableWindow(GETHCTL(IDC_PLOT_LINES), TRUE);
				EnableWindow(GETHCTL(IDC_PLOT_NOTES), TRUE);
				EnableWindow(GETHCTL(IDHELP), TRUE);
			}

			return(wParam);
		}

		default:
		{
			return (DefWindowProc(hWnd, message, wParam, lParam));
		}
	}
	return ((long)NULL);
}

/*****************************************************************************/
