/*****************************************************************************/
/*                                                                           */
/*                            STABLE_I.C                                     */
/*                                                                           */
/*      Stable Windows 95/NT Support Functions                               */
/*                                                                           */
/*      RegularizeData()                                                     */
/*                                                                           */
/*      Revision record:                                                     */
/*          09/24/02    Created source file                                  */
/*					    Added RegularizeData()                               */
/*			09/26/02	Added ConvPhaseToFreqUsingTimetags()                 */
/*			10/01/02	Added DetermineXScale()                              */
/*			10/02/02	Added DetermineXMult()                               */
/*          07/22/07    Changed to safe string functions (none)              */
/*                                                                           */
/* (c) 2002-7  W. Riley  Hamilton Technical Services  All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#define STRICT                  // For Win32 compatibility

#include "stable_w.h"           // Stable/Win include file

/*****************************************************************************/
/*                                                                           */
/* 		                      RegularizeCount()                              */
/*                                                                           */
/* 	Function to count gaps to data file on basis of timetags                 */
/*                                                                           */
/*  Parameters:		F_TYPE	fData[]			Data array                       */
/*   				F_TYPE	fTags[]			Timetag array                    */
/*                  int *	nInterruptions	# interruptions                  */
/*                  int *	nMissings	    # missing points                 */
/*                  int nMaxSize			Maximum allowable array size     */
/*                                                                           */
/*	Return			BOOL	bBlankTag		Flag for blank timetag           */
/*                                                                           */
/*  Note:	This function only processes the points within the array limits  */
/*                                                                           */
/*	Revision record:                                                         */
/*			09/25/02	Created                                              */
/*			                                                                 */
/* 	(c) 2002  W. Riley Hamilton Technical Services All Rights Reserved       */
/*                                                                           */
/*****************************************************************************/

BOOL WINAPI RegularizeCount(F_TYPE fData[], F_TYPE fTags[],
	int *nInterruptions, int *nMissings)
{
	// Local variables
	// Ints
	int i;						// General index
	int	nGaps=0;				// # interruptions
	int	nMissingPts=0;			// # missing points
	int nStart;					// Start index #
	int nEnd;					// End index #
	int nNum;                   // # points to process
	// BOOLs
	BOOL bGapAdded=FALSE;       // Flag for gap(s) added to data
	BOOL bBlankTag=FALSE;		// Flag for blank timetag 
	// F_TYPEs
	F_TYPE fInterval;    // Tau interval of timetags

	// Start of code

	// Set the array analysis limits
	nStart=(int)fData[1];
	nEnd=(int)fData[2];
	nNum=nEnd-nStart+1;

	// Find timetag interval
	fInterval=fTags[ARRAY_OFFSET+1]-fTags[ARRAY_OFFSET];

	// Check entire timetag array for gaps
	for(i=nStart; i<nEnd; i++)
	{
		// Skip blank timetags (those==0)
		if((fTags[ARRAY_OFFSET+i-1]) && (fTags[ARRAY_OFFSET+i]))
		{
			// Search for interruptions (gaps in data)
			if((fTags[ARRAY_OFFSET+i]-fTags[ARRAY_OFFSET+i-1]-
				(fInterval/2))>fInterval)
			{
				nGaps++;

				// Find # of missing points
				// Subtracting 1 gives "exact" value, which would be
				// subject to int truncation.  Subtracting 0.5
				// provides rounding
				nMissingPts+=(int)(((fTags[ARRAY_OFFSET+i]-
					fTags[ARRAY_OFFSET+i-1])/fInterval)-0.5);
			}
		}
		else
		{
			// Set blank timetag flag
			bBlankTag=TRUE;
		}
	}

	// Assign results
	*nInterruptions=nGaps;
	*nMissings=nMissingPts;

	// Done
	return bBlankTag;
}

/*****************************************************************************/
/*                                                                           */
/* 		                    RegularizeInsert()                               */
/*                                                                           */
/* 	Function to add gaps to data file on basis of timetags so it has         */
/*  regular spacing between data points.                                     */
/*                                                                           */
/*  Parameters:		F_TYPE	fData[]			Data array                       */
/*   				F_TYPE	fTags[]			Timetag array                    */
/*                                                                           */
/*	Return			BOOL	bGapAdded		Flag that gap was added          */
/*                                                                           */
/*	Note:	The input arrays must be pre-allocated to the size of the        */
/*			regularized data, but with the original header values.           */
/*                                                                           */
/*	Revision record:                                                         */
/*			09/25/02	Created                                              */
/*			                                                                 */
/* 	(c) 2002  W. Riley Hamilton Technical Services All Rights Reserved       */
/*                                                                           */
/*****************************************************************************/

BOOL WINAPI RegularizeInsert(F_TYPE fData[], F_TYPE fTags[])
{
	// Local variables
	// Ints
	int i;						// General index
	int j;                      // Shifting index
	int nSize;					// # points in regularized data file
	int	nGaps=0;				// # interruptions
	int	nMissingPts=0;			// # missing points
	int nStart;					// Start index #
	int nEnd;					// End index #
	int nNum;                   // # points to process
	int nBlock;					// # missing points in a block
	// BOOLs
	BOOL bGapAdded=FALSE;       // Flag for gap(s) added to data
	BOOL bBlankTag=FALSE;       // Flag for blank timetag
	// F_TYPEs
	F_TYPE fInterval;    // Tau interval of timetags

	// Start of code	
	// Set the array analysis limits
	nSize=(int)fData[0];
	nStart=(int)fData[1];
	nEnd=(int)fData[2];
	nNum=nEnd-nStart+1;

	// Find timetag interval
	fInterval=fTags[ARRAY_OFFSET+1]-fTags[ARRAY_OFFSET];

	// Insert missing points in timetag data
	for(i=nStart; i<nNum; i++)
	{
		// Skip blank timetags (those==0)
		if((fTags[ARRAY_OFFSET+i-1]) && (fTags[ARRAY_OFFSET+i]))
		{
			// Search for interruptions (gaps in data)
			// Find # of missing points in block
			// Subtracting 1 gives "exact" value, which would be
			// subject to int truncation.  Subtracting 0.5
			// provides rounding
			nBlock=(int)(((fTags[ARRAY_OFFSET+i]-
				fTags[ARRAY_OFFSET+i-1])/fInterval)-0.5);

			if(nBlock>0)
			{
				// Insert gap into timetag & data arrays
				// Shift data block after gap up in arrays
				for(j=nNum+nBlock-1; j>i+nBlock-1; j--)
				{
					fTags[ARRAY_OFFSET+j]=fTags[ARRAY_OFFSET+j-nBlock];
					fData[ARRAY_OFFSET+j]=fData[ARRAY_OFFSET+j-nBlock];
				}

				// Insert new timetag & gap data points
				for(j=0; j<nBlock; j++)
				{
					fTags[ARRAY_OFFSET+i+j]=
						fTags[ARRAY_OFFSET+i-1]+(j+1)*fInterval;
					fData[ARRAY_OFFSET+i+j]=0.0;
				}

				// Increment counts
				bGapAdded=TRUE;
				nEnd+=nBlock;
				nNum+=nBlock;
				nSize+=nBlock;
				i+=nBlock-1;
			}
		}
	}

	// Update array bounds
	fData[0]=nSize;
	fTags[0]=nSize;
	fData[2]=nEnd;
	fTags[2]=nEnd;

	// Done
	return bGapAdded;
}

/***************************************************************************/
/*                                                                         */
/*					          DetermineXScale()                            */
/*                                                                         */
/*		Subroutine to determine x-axis scale for phase or freq data plot   */
/*                                                                         */
/*		Parameter:	struct plotdata *sPlot	Pointer to plotdata struct     */
/*                                                                         */
/*		Return:     none (void)                                            */
/*                                                                         */
/*		Revision record:                                                   */
/*			10/01/02	Created                                            */
/*                                                                         */
/* (c) Copyright 2002 Hamilton Technical Services All Rights Reserved      */
/*                                                                         */
/***************************************************************************/

void WINAPI DetermineXScale(struct plotdata *sPlot)
{
	/* local variables */
	F_TYPE fMin;				// Scale min
	F_TYPE fMax;				// Scale max
	F_TYPE fDiv;				// Scale div

	// Determine x-axis plot scale
	if(((sPlot->x_max-sPlot->x_min)/
		((fabs(sPlot->x_max)+fabs(sPlot->x_min)*2)))>0.05)
	{
		// Determine plot scale using RoundAxes() function
		fMin=sPlot->x_min;
		fMax=sPlot->x_max;
		RoundAxes(&fMin, &fMax, &fDiv);
		fDiv*=5;
	}
	else // Small relative range
	{
		// Determine plot scale using BasScale() function
		BasScale(sPlot->x_min, sPlot->x_max, 12, &fMin, &fDiv);
		sPlot->x_div=(int)ceil((sPlot->x_max-fMin)/fDiv);
		fMax=fMin+(sPlot->x_div*fDiv);
	}

	// Set # divs
	sPlot->x_div=(int)ceil((fMax-fMin)/fDiv);

	// Tighten scale as much as possible
	while(sPlot->x_min>=(fMin+fDiv))
	{
		fMin+=fDiv;
		sPlot->x_div-=1;
	}
	while(sPlot->x_max<=(fMax-fDiv))
	{
		fMax-=fDiv;
		sPlot->x_div-=1;
	}
						
	// Widen scale if necessary
	// BasScale() may need this sometimes
	while(sPlot->x_min<fMin)
	{
		fMin-=fDiv;
		sPlot->x_div+=1;
	}
	while(sPlot->x_max>fMax)
	{
		fMax+=fDiv;
		sPlot->x_div+=1;
	}

	// Set scale limits
	sPlot->x_min=(float)fMin;
	sPlot->x_max=(float)fMax;
}

/***************************************************************************/
/*                                                                         */
/*					         DetermineXMult()                              */
/*                                                                         */
/*		Subroutine to determine x-axis multiplier for phase or freq plot   */
/*                                                                         */
/*		Parameters:	struct plotdata *sPlot	Pointer to plotdata struct     */
/*                  int             nPlotX	X-Axis type                    */
/*                  F_TYPE			fTau	Measurement interval           */
/*                                                                         */
/*		Return:     none (void)                                            */
/*                                                                         */
/*		Revision record:                                                   */
/*			10/02/02	Created                                            */
/*                                                                         */
/* (c) Copyright 2002 Hamilton Technical Services All Rights Reserved      */
/*                                                                         */
/***************************************************************************/

void WINAPI DetermineXMult(struct plotdata *sPlot, int nPlotX, F_TYPE fTau)
{
	// Set the x-axis multiplier factor
	switch(nPlotX)
	{
		case X_POINT:
		case X_TIMETAG:
		{
			sPlot->x_mult=(float)1.0;
		}
		break;

		case X_KPOINT:
		{
			sPlot->x_mult=(float)0.001;
		}
		break;

		case X_MILLISECOND:
		{
			sPlot->x_mult=(float)(fTau*1000.0);
		}
		break;

		case X_SECOND:
		{
			sPlot->x_mult=(float)fTau;
		}
		break;

		case X_MINUTE:
		{
			sPlot->x_mult=(float)(fTau/60.0);
		}
		break;

		case X_HOUR:
		{
			sPlot->x_mult=(float)(fTau/3600.0);
		}
		break;

		case X_DAY:
		case X_DOY:
		case X_MJD:
		{
			sPlot->x_mult=(float)(fTau/86400.0);
		}
		break;

		case X_WEEK:
		{
			sPlot->x_mult=(float)(fTau/(86400.0*7));
		}
		break;
	}
}

/***************************************************************************/
	