/*****************************************************************************/
/*                                                                           */
/*                             STABLE_II.C                                   */
/*                                                                           */
/* Files for the Lomb periodogram method for spectral analysis of unevenly   */
/* sampled data with the objective of using it to handle unevenly sampled    */
/* time domain data by finding the complex Lomb periodogram and then using   */
/* the inverse FFT to reconstruct a set of equvalent evenly sampled time     */
/* domain data.                                                              */
/*                                                                           */
/* The starting point is the NR function period() modified to return the     */
/* amplitude and phase spectra per K. Hocke, Ann. Geophysicae 16,356-358,    */
/* 1998.                                                                     */
/*                                                                           */
/* Other references include: K. Hicke and N. Kampfer, Atmos. Chem. Discuss.  */
/* 8, 4603-4623, (2008), J.D. Scargle, Astrophysical Journal, 343, 874-887,  */
/* 1989, and http://en.wikipedia.org/wiki/Least-squares spectral analysis    */
/*                                                                           */
/* The period() function fills the py[] array with the values of the         */
/* normalized Lomb periodogram at frequencies px[]                           */
/*                                                                           */
/* Hocke 1998 gives the corresponding amplitude spectrum as:                 */
/* sqrt( (2/n) * (2*var) * px[i] )                                           */
/* and the phase spectrum as (I think):                                      */
/* -pnow*2*pi*xave -wtau -atan2(sumsy/sqrt(sums), sumcy/sqrt(sumc))          */
/*                                                                           */
/* Note that the Lomb periodogram functions are not in my copy of the NR     */
/* book, but they are on my licensed CD-ROM of the NR functions              */
/*                                                                           */
/* Credit for pointing out the Lomb periodogram method goes to:              */
/* Mark W. Keller, Magnetics Group, Electromagnetics Division, NIST          */
/* Phone: 303-497-5430, Email: mark.keller@boulder.nist.gov                  */
/* Mail: NIST/818.03, 325 Broadway, Boulder, CO 80305-3328                   */
/* in a 12/15/09 e-mail to me.                                               */
/*                                                                           */
/* Revision record:                                                          */
/*		12/20/09    Created                                                  */
/*		12/21/09	Compiles w/o error, Stable32 builds OK, not yet used     */
/*                                                                           */
/* (c) Copyright 2009   Hamilton Technical Services  All Rights Reserved     */
/*                                                                           */
/*****************************************************************************/

/* period.c from Numerical Recipes in C */

#if(1) // Revised version
// Returns amplitude and phase spectrum as amp[] and phase[] respectively
#include <windows.h> // For MessageBox() usage
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <math.h>
#include "nrutil.h"
#define NR_END 1
#define FREE_ARG char*
#define NRANSI
#define TWOPID 6.2831853071795865

// Given n data points with abscissas x[1..n] (which need not be equally spaced)
// and ordinates y[1..n], and given a desired oversampling factor ofac
// (a typical value being 4o r larger), this routine fills array px[1..np]
// with an increasing sequence of frequencies (not angularfrequencies)
// up to hifac times the average Nyquist frequency, and fills array py[1..np]
// with the values of the Lomb normalized periodogram at those frequencies.
// The arrays x and y are not altered. np, the dimension of px and py,
// must be large enough to contain the output, or an error results.
// The routine also returns jmax such that py[jmax] is the maximum element
// in py, and prob, an estimate of the significance of that maximum
// against the hypothesis of random noise. A small value of prob indicates
// that a significant periodic signal is present.

void period2(float x[], float y[], int n, float ofac, float hifac, float px[],
	float py[], int np, int *nout, int *jmax, float *prob,
	float amp[], float phase[])
{
	void avevar(float data[], unsigned long n, float *ave, float *var);
	int i,j;
	float ave,c,cc,cwtau,effm,expy,pnow,pymax,s,ss,sumc,sumcy,sums,sumsh,
		sumsy,swtau,var,wtau,xave,xdif,xmax,xmin,yy;
	double arg,wtemp,*wi,*wpi,*wpr,*wr;

	wi=dvector(1,n);
	wpi=dvector(1,n);
	wpr=dvector(1,n);
	wr=dvector(1,n);
	*nout=(int)(0.5*ofac*hifac*n);
	if (*nout > np) nrerror("output arrays too short in period");
	avevar(y,n,&ave,&var);
	if (var == 0.0) nrerror("zero variance in period");
	xmax=xmin=x[1];
	for (j=1;j<=n;j++) {
		if (x[j] > xmax) xmax=x[j];
		if (x[j] < xmin) xmin=x[j];
	}
	xdif=xmax-xmin;
	xave=(float)(0.5*(xmax+xmin));
	pymax=0.0;
	pnow=(float)(1.0/(xdif*ofac));
	for (j=1;j<=n;j++) {
		arg=TWOPID*((x[j]-xave)*pnow);
		wpr[j]=-2.0*sin(0.5*arg)*sin(0.5*arg);
		wpi[j]=sin(arg);
		wr[j]=cos(arg);
		wi[j]=wpi[j];
	}
	for (i=1;i<=(*nout);i++) {
		px[i]=pnow;
		sumsh=sumc=0.0;
		for (j=1;j<=n;j++) {
			c=(float)(wr[j]);
			s=(float)(wi[j]);
			sumsh += (float)(s*c);
			sumc += (c-s)*(c+s);
		}
		wtau=(float)(0.5*atan2(2.0*sumsh,sumc));
		swtau=(float)(sin(wtau));
		cwtau=(float)(cos(wtau));
		sums=sumc=sumsy=sumcy=0.0;
		for (j=1;j<=n;j++) {
			s=(float)(wi[j]);
			c=(float)(wr[j]);
			ss=s*cwtau-c*swtau;
			cc=c*cwtau+s*swtau;
			sums += ss*ss;
			sumc += cc*cc;
			yy=y[j]-ave;
			sumsy += yy*ss;
			sumcy += yy*cc;
			wr[j]=((wtemp=wr[j])*wpr[j]-wi[j]*wpi[j])+wr[j];
			wi[j]=(wi[j]*wpr[j]+wtemp*wpi[j])+wi[j];
		}
		py[i]=(float)(0.5*(sumcy*sumcy/sumc+sumsy*sumsy/sums)/var);

		// Here we add determination of the spectral amplitude 
		amp[i] = (float)(sqrt((2/n)* (2*var*var)*py[i]));
		// and the spectral phase
		phase[i]=(float)(-pnow*TWOPID*xave -wtau -atan2(sumsy/sqrt(sums),
			sumcy/sqrt(sumc)));

		if (py[i] >= pymax) pymax=py[(*jmax=i)];
		pnow += (float)(1.0/(ofac*xdif));
	}
	expy=(float)(exp(-pymax));
	effm=(float)(2.0*(*nout)/ofac);
	*prob=effm*expy;
	if (*prob > 0.01) *prob=(float)(1.0-pow(1.0-expy,effm));
	free_dvector(wr,1,n);
	free_dvector(wpr,1,n);
	free_dvector(wpi,1,n);
	free_dvector(wi,1,n);
}
#undef TWOPID
#undef NRANSI
#endif

#if(0) // Customized version - compiles w/o error or warning
#include <math.h>
#define NRANSI
#include "nrutil.h"
#define TWOPID 6.2831853071795865

void period(float x[], float y[], int n, float ofac, float hifac, float px[],
	float py[], int np, int *nout, int *jmax, float *prob)
{
	void avevar(float data[], unsigned long n, float *ave, float *var);
	int i,j;
	float ave,c,cc,cwtau,effm,expy,pnow,pymax,s,ss,sumc,sumcy,sums,sumsh,
		sumsy,swtau,var,wtau,xave,xdif,xmax,xmin,yy;
	double arg,wtemp,*wi,*wpi,*wpr,*wr;

	wi=dvector(1,n);
	wpi=dvector(1,n);
	wpr=dvector(1,n);
	wr=dvector(1,n);
	*nout=(int)(0.5*ofac*hifac*n);
	if (*nout > np) nrerror("output arrays too short in period");
	avevar(y,n,&ave,&var);
	if (var == 0.0) nrerror("zero variance in period");
	xmax=xmin=x[1];
	for (j=1;j<=n;j++) {
		if (x[j] > xmax) xmax=x[j];
		if (x[j] < xmin) xmin=x[j];
	}
	xdif=xmax-xmin;
	xave=(float)(0.5*(xmax+xmin));
	pymax=0.0;
	pnow=(float)(1.0/(xdif*ofac));
	for (j=1;j<=n;j++) {
		arg=TWOPID*((x[j]-xave)*pnow);
		wpr[j]=-2.0*sin(0.5*arg)*sin(0.5*arg);
		wpi[j]=sin(arg);
		wr[j]=cos(arg);
		wi[j]=wpi[j];
	}
	for (i=1;i<=(*nout);i++) {
		px[i]=pnow;
		sumsh=sumc=0.0;
		for (j=1;j<=n;j++) {
			c=(float)(wr[j]);
			s=(float)(wi[j]);
			sumsh += (float)(s*c);
			sumc += (c-s)*(c+s);
		}
		wtau=(float)(0.5*atan2(2.0*sumsh,sumc));
		swtau=(float)(sin(wtau));
		cwtau=(float)(cos(wtau));
		sums=sumc=sumsy=sumcy=0.0;
		for (j=1;j<=n;j++) {
			s=(float)(wi[j]);
			c=(float)(wr[j]);
			ss=s*cwtau-c*swtau;
			cc=c*cwtau+s*swtau;
			sums += ss*ss;
			sumc += cc*cc;
			yy=y[j]-ave;
			sumsy += yy*ss;
			sumcy += yy*cc;
			wr[j]=((wtemp=wr[j])*wpr[j]-wi[j]*wpi[j])+wr[j];
			wi[j]=(wi[j]*wpr[j]+wtemp*wpi[j])+wi[j];
		}
		py[i]=(float)(0.5*(sumcy*sumcy/sumc+sumsy*sumsy/sums)/var);
		if (py[i] >= pymax) pymax=py[(*jmax=i)];
		pnow += (float)(1.0/(ofac*xdif));
	}
	expy=(float)(exp(-pymax));
	effm=(float)(2.0*(*nout)/ofac);
	*prob=effm*expy;
	if (*prob > 0.01) *prob=(float)(1.0-pow(1.0-expy,effm));
	free_dvector(wr,1,n);
	free_dvector(wpr,1,n);
	free_dvector(wpi,1,n);
	free_dvector(wi,1,n);
}
#undef TWOPID
#undef NRANSI
#endif

#if(0) // Original version
#include <math.h>
#define NRANSI
#include "nrutil.h"
#define TWOPID 6.2831853071795865

void period(float x[], float y[], int n, float ofac, float hifac, float px[],
	float py[], int np, int *nout, int *jmax, float *prob)
{
	void avevar(float data[], unsigned long n, float *ave, float *var);
	int i,j;
	float ave,c,cc,cwtau,effm,expy,pnow,pymax,s,ss,sumc,sumcy,sums,sumsh,
		sumsy,swtau,var,wtau,xave,xdif,xmax,xmin,yy;
	double arg,wtemp,*wi,*wpi,*wpr,*wr;

	wi=dvector(1,n);
	wpi=dvector(1,n);
	wpr=dvector(1,n);
	wr=dvector(1,n);
	*nout=0.5*ofac*hifac*n;
	if (*nout > np) nrerror("output arrays too short in period");
	avevar(y,n,&ave,&var);
	if (var == 0.0) nrerror("zero variance in period");
	xmax=xmin=x[1];
	for (j=1;j<=n;j++) {
		if (x[j] > xmax) xmax=x[j];
		if (x[j] < xmin) xmin=x[j];
	}
	xdif=xmax-xmin;
	xave=0.5*(xmax+xmin);
	pymax=0.0;
	pnow=1.0/(xdif*ofac);
	for (j=1;j<=n;j++) {
		arg=TWOPID*((x[j]-xave)*pnow);
		wpr[j] = -2.0*SQR(sin(0.5*arg));
		wpi[j]=sin(arg);
		wr[j]=cos(arg);
		wi[j]=wpi[j];
	}
	for (i=1;i<=(*nout);i++) {
		px[i]=pnow;
		sumsh=sumc=0.0;
		for (j=1;j<=n;j++) {
			c=wr[j];
			s=wi[j];
			sumsh += s*c;
			sumc += (c-s)*(c+s);
		}
		wtau=0.5*atan2(2.0*sumsh,sumc);
		swtau=sin(wtau);
		cwtau=cos(wtau);
		sums=sumc=sumsy=sumcy=0.0;
		for (j=1;j<=n;j++) {
			s=wi[j];
			c=wr[j];
			ss=s*cwtau-c*swtau;
			cc=c*cwtau+s*swtau;
			sums += ss*ss;
			sumc += cc*cc;
			yy=y[j]-ave;
			sumsy += yy*ss;
			sumcy += yy*cc;
			wr[j]=((wtemp=wr[j])*wpr[j]-wi[j]*wpi[j])+wr[j];
			wi[j]=(wi[j]*wpr[j]+wtemp*wpi[j])+wi[j];
		}
		py[i]=0.5*(sumcy*sumcy/sumc+sumsy*sumsy/sums)/var;
		if (py[i] >= pymax) pymax=py[(*jmax=i)];
		pnow += 1.0/(ofac*xdif);
	}
	expy=exp(-pymax);
	effm=2.0*(*nout)/ofac;
	*prob=effm*expy;
	if (*prob > 0.01) *prob=1.0-pow(1.0-expy,effm);
	free_dvector(wr,1,n);
	free_dvector(wpr,1,n);
	free_dvector(wpi,1,n);
	free_dvector(wi,1,n);
}
#undef TWOPID
#undef NRANSI
#endif

#if(0)
/* fasper.c from Numerical Recipes in C */
/* Hocke does not give the amplitude and phase expressions */
/* for this function */
/* So try it with period() first */
/* "First get it right, then get it fast" */

#include <math.h>
#define NRANSI
#include "nrutil.h"
#define MOD(a,b) 	while(a >= b) a -= b;
#define MACC 4

void fasper(float x[], float y[], unsigned long n, float ofac, float hifac,
	float wk1[], float wk2[], unsigned long nwk, unsigned long *nout,
	unsigned long *jmax, float *prob)
{
	void avevar(float data[], unsigned long n, float *ave, float *var);
	void realft(float data[], unsigned long n, int isign);
	void spread(float y, float yy[], unsigned long n, float x, int m);
	unsigned long j,k,ndim,nfreq,nfreqt;
	float ave,ck,ckk,cterm,cwt,den,df,effm,expy,fac,fndim,hc2wt;
	float hs2wt,hypo,pmax,sterm,swt,var,xdif,xmax,xmin;

	*nout=0.5*ofac*hifac*n;
	nfreqt=ofac*hifac*n*MACC;
	nfreq=64;
	while (nfreq < nfreqt) nfreq <<= 1;
	ndim=nfreq << 1;
	if (ndim > nwk) nrerror("workspaces too small in fasper");
	avevar(y,n,&ave,&var);
	if (var == 0.0) nrerror("zero variance in fasper");
	xmin=x[1];
	xmax=xmin;
	for (j=2;j<=n;j++) {
		if (x[j] < xmin) xmin=x[j];
		if (x[j] > xmax) xmax=x[j];
	}
	xdif=xmax-xmin;
	for (j=1;j<=ndim;j++) wk1[j]=wk2[j]=0.0;
	fac=ndim/(xdif*ofac);
	fndim=ndim;
	for (j=1;j<=n;j++) {
		ck=(x[j]-xmin)*fac;
		MOD(ck,fndim)
		ckk=2.0*(ck++);
		MOD(ckk,fndim)
		++ckk;
		spread(y[j]-ave,wk1,ndim,ck,MACC);
		spread(1.0,wk2,ndim,ckk,MACC);
	}
	realft(wk1,ndim,1);
	realft(wk2,ndim,1);
	df=1.0/(xdif*ofac);
	pmax = -1.0;
	for (k=3,j=1;j<=(*nout);j++,k+=2) {
		hypo=sqrt(wk2[k]*wk2[k]+wk2[k+1]*wk2[k+1]);
		hc2wt=0.5*wk2[k]/hypo;
		hs2wt=0.5*wk2[k+1]/hypo;
		cwt=sqrt(0.5+hc2wt);
		swt=SIGN(sqrt(0.5-hc2wt),hs2wt);
		den=0.5*n+hc2wt*wk2[k]+hs2wt*wk2[k+1];
		cterm=SQR(cwt*wk1[k]+swt*wk1[k+1])/den;
		sterm=SQR(cwt*wk1[k+1]-swt*wk1[k])/(n-den);
		wk1[j]=j*df;
		wk2[j]=(cterm+sterm)/(2.0*var);
		if (wk2[j] > pmax) pmax=wk2[(*jmax=j)];
	}
	expy=exp(-pmax);
	effm=2.0*(*nout)/ofac;
	*prob=effm*expy;
	if (*prob > 0.01) *prob=1.0-pow(1.0-expy,effm);
}
#undef MOD
#undef MACC
#undef NRANSI
#endif

// NR Subroutines
// Note that some of these may be in the FrequenC library
// and would be overwritten here

#if(0)
/*****************************************************************************/
/*                                                                           */
/*                                  spread()                                 */
/*                                                                           */
/* This version of spread() is from my Version 2.10/2.11 CD                  */
/*                                                                           */
/* Given an array yy[1..n], extirpolate (spread) a value y into m actual     */
/* array elements that best approximate the "fictional" (i.e., possibly      */
/* noninteger) array element number x. The weights used are coefficients of  */
/* the Lagrange interpolating polynomial.                                    */
/*                                                                           */
/*****************************************************************************/

#include "nrutil.h"

void spread(float y, float yy[], unsigned long n, float x, int m)
{
	int ihi,ilo,ix,j,nden;
	static long nfac[11]={0,1,1,2,6,24,120,720,5040,40320,362880};
	float fac;

	if (m > 10) nrerror("factorial table too small in spread");
	ix=(int)x;
	if (x == (float)ix) yy[ix] += y;
	else {
		ilo=LMIN(LMAX((long)(x-0.5*m+1.0),1),n-m+1);
		ihi=ilo+m-1;
		nden=nfac[m];
		fac=x-ilo;
		for (j=ilo+1;j<=ihi;j++) fac *= (x-j);
		yy[ihi] += y*fac/(nden*(x-ihi));
		for (j=ihi-1;j>=ilo;j--) {
			nden=(nden/(j+1-ilo))*(j-ihi);
			yy[j] += y*fac/(nden*(x-j));
		}
	}
}
#endif

#if(0) // Original terminal application version
void nrerror(char error_text[])
/* Numerical Recipes standard error handler */
{
	fprintf(stderr,"Numerical Recipes run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"...now exiting to system...\n");
	exit(1);
}
#endif

#if(1) // Replacement Windows version
void nrerror(char error_text[])
{
	MessageBox(NULL, error_text, "NR Error", MB_OK);
}
#endif

double *dvector(long nl, long nh)
/* allocate a double vector with subscript range v[nl..nh] */
{
	double *v;

	v=(double *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(double)));
	if (!v) nrerror("allocation failure in dvector()");
	return v-nl+NR_END;
}

void free_dvector(double *v, long nl, long nh)
/* free a double vector allocated with dvector() */
{
	free((FREE_ARG) (v+nl-NR_END));
}

void avevar(float data[], unsigned long n, float *ave, float *var)
{
	unsigned long j;
	float s,ep;

	for (*ave=0.0,j=1;j<=n;j++) *ave += data[j];
	*ave /= n;
	*var=ep=0.0;
	for (j=1;j<=n;j++) {
		s=data[j]-(*ave);
		ep += s;
		*var += s*s;
	}
	*var=(*var-ep*ep/n)/(n-1);
}
