/*****************************************************************************/
/*									 								    	 */
/*  					 CalcPhaseCrossSigma()                   		     */
/*																		     */
/*		Function to calculate cross correlation Allan variance using         */
/*		overlapping samplesof phase data                                     */
/*                                                                           */
/*      Parameters:    	F_TYPE x1[]   = phase data array #1                  */
/*                                      x1[0] =  # phase data points         */
/*                                      x1[1] = analysis start               */
/*                                      x1[2] = analysis end                 */
/*					   	F_TYPE x2[]   = phase data array #2                  */
/*                                      x2[0] =  # phase data points         */
/*                                      x2[1] = analysis start               */
/*                                      x2[2] = analysis end                 */
/*  		            F_TYPE *p_sig = pointer to Allan deviation           */
/*			            F_TYPE tau    = averaging time of phase data         */
/*      	            int m         = averaging factor                     */
/*						int *sign     = sign of sigma result                 */
/*                                                                           */
/*		Return:         int           = # analysis points,  			     */
/*										or -1 if error                       */
/*									 									     */
/*		Notes:          Embedded zeros treated as gaps in phase data.        */
/*						Must have at least 1 analysis point; this            */
/*                      requires at least 3 adjacent non-gap phase data      */
/*                      points.  Averaging time for sigma calculation is     */
/*                      m times the tau parameter of the phase data.         */
/*                      m must be >0 , tau must be >0, and the # non-gap     */
/*                      phase data points N must be > 2m.                    */
/*                                                                           */
/*                      The x1[] and x2[] phase data arrays have the same    */
/*                      header as do other such Stable32 data arrays         */
/*                      They must represent simutaneous data taken with two  */
/*                      separate measurement systems whose noise will be     */
/*                      cancelled by the cross-correlation process.          */
/*                                                                           */
/*                      The two x[] arrays must have the same # of points.   */
/*                      The x1[] analysis limits are used.                   */
/*                                                                           */
/*                      Cross sigma is calculated exactly like normal one    */
/*                      except that the products of separate x[] array       */
/*						values are used instead of the square of values      */
/*                      from a single array.                                 */
/*                                                                           */
/*                      Sigma can have either sign, which is returned by     */
/*                      a separate argument.                                 */
/*                                                                           */
/*		Reference:		A. Lepek & F.L. Walls, "Cross Correlation Analysis   */
/*                      Improves Time Domain Measurements", Proc. 1993 IEEE  */
/*                      Freq. Ctrl. Symp., pp.312-320.                       */
/*                                                                           */
/*		Revision record:                                                     */
/*			02/15/10	Adapted from CalcPhaseOverlapSigma() of FrequenC     */
/*						Library                                              */
/*																		     */
/* (c) Copyright 2010  Hamilton Technical Services  All Rights Reserved      */
/*                                                                           */
/*****************************************************************************/

#include "Stable_w.h"

int WINAPI CalcPhaseCrossSigma(F_TYPE x1[],
	F_TYPE x2[], F_TYPE *p_sig, F_TYPE tau, int m, int *sign)
{
	double sum=0;                          /* summing variable to calc sigma */
	float fm=(float)m;                                 /* float version of m */
	int i;                                            /* index for summation */
	int num=0;                                          /* # analysis points */
	int n1;                          /* total # phase data points in array 1 */
	int n2;                          /* total # phase data points in array 2 */

	// Set total # phase data points inn each array
	n1=(int)(*(x1+2)-*(x1+1)+1);
	n2=(int)(*(x2+2)-*(x2+1)+1);

	// Check for equal # points in each data array
	if( n1 != n2 )
	{
		return(-1);                                                 /* error */
	}

	// Check parameters
	if( (m<1) || (n1<2*m) || (n2<2*m) || (tau<=0) )
	{
		return(-1);                                                 /* error */
	}


	for( i=(int)(*(x1+1)+2); i<=(int)(*(x1+2)-2*m+2); i++)   /* summing loop */
	/* i basically goes from 1 to N-2m where N is # phase data points        */
	/* and m is averaging factor.  Term *(x+1) is the analysis start limit   */
	/* (1 if all data used) & the 2 accounts for the data array offset       */
	{
		if(    ( i==*(x1+1)+2 && *(x1+i+m) && *(x1+i+(2*m)) )
		   ||  ( *(x1+i)      && *(x1+i+m) && *(x1+i+(2*m)) )
		   ||  ( i==n1-2*m+2  && *(x1+i+m) && *(x1+i)        )
		   ||  ( i==*(x2+1)+2 && *(x2+i+m) && *(x2+i+(2*m)) )
		   ||  ( *(x2+i)      && *(x2+i+m) && *(x2+i+(2*m)) )
		   ||  ( i==n2-2*m+2  && *(x2+i+m) && *(x2+i)        ) )
		   /* skip if any imbedded phase data value is zero */
		{
			sum += ( *(x1+i+(2*m)) - (2 * (*(x1+i+m))) + *(x1+i) )*
			       ( *(x2+i+(2*m)) - (2 * (*(x2+i+m))) + *(x2+i) );
			num++;
		}                                                          /* end if */
	}

	if(num>0)
	{
		// Note we use fabs(sum) because it can have either sign
		// Sign is returned separately
		*p_sig=sqrt( fabs(sum) / (2.0*tau*tau*fm*fm*num));
		if(sum<0.0)
		{
			*sign=-1;
		}
		else
		{
			*sign=1;
		}
		return(num);                                    /* # analysis points */
	}
	else
	{
		return(-1);                                                 /* error */
	}  
}                                               /* end CalcPhaseCrossSigma() */

/*****************************************************************************/
