/*****************************************************************************/
/*                                                                           */
/*                            STABLE_K.C                                     */
/*                                                                           */
/* 		Stable32 Speed-Optimized Functions                                   */
/*                                                                           */
/*		CalcFastMTIE()                                                       */
/*                                                                           */
/*		Revision record:                                                     */
/*			01/28/01	File created for CalcFastMTIE()                      */
/*			02/10/01	Added NewModTotvarCalc()                             */
/*						Added HadTotvarCalc()  *                             */
/*			05/18/02	Added NewFillGaps() *                                */
/*			07/25/02	Added NewFillFloatGaps()  *                          */
/*			03/14/03	* Eliminated - now in FrequenC DLL                   */
/*          07/22/07    Changed to safe string functions (none)              */
/*			                                                                 */
/* 	(c) 2001-2007 W. Riley Hamilton Technical Services All Rights Reserved   */
/*                                                                           */
/*****************************************************************************/

#define STRICT					// For Win32 compatibility

#include "stable_w.h"           // STABLE_W header file

/*****************************************************************************/
/*                                                                           */
/*  				            CalcFastMTIE()                               */
/*                                                                           */
/*		Function to calc MTIE (maximum time interval error) for phase data   */
/*		Parameters:                                                          */
/* 			F_TYPE fX[]        = data array, where:                          */
/*										     F_TYPE fX[0] = # data points    */
/*											 F_TYPE fX[1] = analysis start   */
/*											 F_TYPE fX[2] = analysis end     */
/*			plotdata *pResults = MTIE results sigmadata structure            */
/*                                                                           */
/*		Return:                                                              */
/*			k   = Outer index (# MTIE points), or -1 if error                */
/*                                                                           */
/* 		sigmadata structure elements for sigma data:					     */
/*			int		points        number of sigma data points                */
/*			float	*tau          pointer to averaging time data array       */
/*			float	*num          pointer to number of data points array     */
/*			float	*sigma        pointer to sigma data array                */
/*			float	*ci           pointer to confidence interval             */
/*			float	*sig_min      pointer to lower sigma error bar array     */
/*			float	*sig_max      pointer to upper sigma error bar array     */
/*			float	*std_dev      pointer to standard deviation array        */
/*			int		*alpha        pointer to noise type exponent array       */
/*			int		*af           pointer to averaging factor                */
/*                                                                           */
/*		Note: Signature of this function is necessarily different from       */
/*		CalcMTIE() because it includes the outer averaging factor loop.      */
/*		It is used differently in main program.                              */
/*                                                                           */
/*		Basic Algorithm:                                                     */
/*			An n-point wide window is moved thru the phase data and the      */
/*			difference between the maximum and minimum phase (time) values   */
/*			is found at each window position.  MTIE is the overall maximum   */
/*			of this time interval error over the whole data set.             */
/*                                                                           */
/*		Fast Algorithm:                                                      */
/*			Calc MTIE in octave steps only.  Store max and min for each tau  */
/*			= AF*tau0 in two vectors.  For next AF, just search these        */
/*			vectors, not all data, for next maxs and mins.                   */
/*                                                                           */
/*		Notes:                                                               */
/*			(1) This function does not handle gaps.                          */
/*			(2) No abort option or progress message for this function        */
/*          (4) # time intervals is N-1, where N = # phase data points, so   */
/*				AF can go from 1 to N-1.                                     */
/*          (3) B&M algorithm uses AF sequence 1,3,7,15,...2^k -1            */
/*				where k is outer index that goes from 1 to <= log2(N)        */
/*                                                                           */
/*		Reference 1:                                                         */
/*			S. Bregni, "Clock Stability Characterization and Measurement     */
/*			in Telecommunications", IEEE Transactions on Instrumentation     */
/*			and Measurement, Vol. 46, No. 6, Dec. 1997, pp. 1284-1294,       */
/*			Eq. 14.                                                          */
/*		Reference 2:                                                         */
/*			S. Bregni and S. Maccabruni, "Fast Computation of Maximum Time   */
/*			Interval Error by Binary Decomposition", IEEE Transactions on    */
/*			Instrumentation and Measurement, Vol. 49, No. 6, Dec. 2000, pp.  */
/*			1240-1244.                                                       */
/*                                                                           */
/*		Revision record:                                                     */
/*			01/26/01	Drafted from CalcMTIE()                              */
/*			01/27/01	Running                                              */
/*			01/28/01	Added progress message                               */
/*						Changed 2nd param to plotdata struct                 */
/*						Changed min & max arrays from F_TYPE to float        */
/*						Tried various speed enhancements                     */
/*						Moved here from stable_r.c to use compiler speed     */
/*						optimizations - they work the best                   */
/*						Copy in stable_r.c still has debug code if needed.   */
/*			01/30/01	Added results on-the-fly & progress display          */
/*                                                                           */
/*****************************************************************************/

int WINAPI CalcFastMTIE(F_TYPE fX[], struct sigmadata *pResults)
{
	// Local variables
	// Chars
	char szBuffer[BUFFER_SIZE+1];   // Text buffer
	// Ints
	int i;							// Inner and general inner index
	int j;							// Middle index
	int last_j;          			// Previous middle index
	int k;							// Outer index
	int p;							// Span
	int nStart;						// Analysis start
	int nEnd;						// Analysis end
	int nNum;	  					// # analysis points
	int nAF;						// Averaging factor
	int nStop;						// Max value of outer index
	int tic;						// Progress interval
	// Floats 
	float fMTIE;					// Current MTIE value
	float *fSmall;					// Min phase data value array
	float *fBig;					// Max phase data value array

	// Initialize progress display
	LoadString(hInst, IDS_MESSAGE_MTIECALC, szBuffer,
		BUFFER_SIZE);
	WriteProgressMessage(0, szBuffer, FALSE);

	// Set analysis limits
	nStart=(int)fX[1];
	nEnd=(int)fX[2];
	nNum=nEnd-nStart+1;

	// Allocate working memory for max and min vectors
	// First pass they must be same size as analysis data array (w/o header)
	// Together they hold all points in overlapping big/small pairs
	// No point in reallocing as they get smaller

	// Allocate maximum array
	if((fBig=(float*)malloc((nNum)*sizeof(float)))==NULL)
	{
		AllocErrorMessage();
		return -1;
	}

	// Allocate minimum array
	if((fSmall=(float*)malloc((nNum)*sizeof(float)))==NULL)
	{
		free(fBig);
		AllocErrorMessage();
		return -1;
	}

	// Initialize the max and min arrays by copying the phase data into them
	for(i=0; i<nNum; i++)
	{
		// Copy data
		fBig[i]=(float)fX[nStart+ARRAY_OFFSET+i-1];
		fSmall[i]=(float)fX[nStart+ARRAY_OFFSET+i-1];
	}
	
	// Calc MTIE
	// Outer k loop over octave AFs up to max nAF = nNum-1
	// nAF is 2^k -1 and max k is log2(nNum)

	// Find maximum outer index = log2(nNum) as truncated integer
	nStop=(int)(log(nNum)/log(2));

	// Calc MITE points at octave averaging factors up to largest possible
	for(k=1; k<=nStop; k++)
	{
		// Find the current averaging factor nAF = 2^k -1
		nAF=(int)(pow(2, k))-1;

		// Find the current span p = 2^(k-1)
		p=(int)(pow(2, k-1));

		// Initialize previous middle index
		last_j=0;

		// Update the min & max vectors for this AF
		// Loop through all nNum-AF points
		for(j=0; j<nNum-nAF; j++)
		{
			/* Set 1% interval for progress report */
			tic=(int)ceil((nNum-nAF)/100);

			// The max and min values are already initialized				
			// Find max & min values for each step in this window of span p
			// Note: This is apparently the section of code that takes the
			// most time as nAF and k get larger, and p gets much larger
			// But its hard to see how one can do any better than the
			// max() and min() macros.
			// This code was tried with index arithmitic outside loop
			// and just incrementing inside - little difference in speed
			// Pointer arithmitic was also tried - ditto
			// What does help is to ask the complier to optimize for speed
			// That makes 5:1 improvement for any of the code versions
			for(i=1; i<=p; i++)
			{
				fBig[j]=max(fBig[j+i], fBig[j]);
				fSmall[j]=min(fSmall[j+i], fSmall[j]);
			}

			// Display progress
			// Note:  This code provides updates only if index has changed
			// by at least tic amount (1%)
			if((j-last_j)>=tic)
			{
				WriteProgressMessage((int)(100.0*j/(nNum-nAF)),
					szBuffer, FALSE);
				last_j=j;
			}
		}

		// Initialize the MTIE value
		// Note that the results array is zero-based
		fMTIE=(float)(fBig[0]-fSmall[0]);
		
		// Loop thru the rest of the nNum-nAF points in each vector
		for(i=1; i<nNum-nAF; i++)
		{
			// MTIE is the overall maximum p-p excursion
			fMTIE=(float)(max((fBig[i]-fSmall[i]), fMTIE));
		}

		// Store results
		pResults->sigma[k-1]=fMTIE;
		pResults->points=k;
		pResults->af[k-1]=nAF;
		pResults->num[k-1]=(float)(nNum-nAF);
		// Note: tau calc external to this function

		// Display results
		SendMessage(hDialog, WM_COMMAND,
			GET_WM_COMMAND_MPS(IDC_RUN_DISPLAY, NULL, 0));

		// Check for abort
		if(bAbort)
		{
			// Bail out - program flow goes to code below, then to IDCANCEL
			// handler, and finally back to where this function was called
			break;
		}
	}

	// Free working memory
	free(fBig);
	free(fSmall);

	// Done
	// Clear progress display
	WriteProgressMessage(0, "", FALSE); 
	return k;
}

/*****************************************************************************/

#if(0) // Another (more general) fast MTIE calc function - doesn't work right
/*****************************************************************************/
/*                                                                           */
/*  				            CalcFastMTIE()                               */
/*                                                                           */
/*		Function to calc MTIE (maximum time interval error) from phase data  */
/*      at power-of-2 averaging factors (1, 2, 4 ***, 2                      */
/*																			 */
/*		Parameters:                                                          */
/* 			F_TYPE fX[]        = data array, where:                          */
/*										     F_TYPE fX[0] = # data points    */
/*											 F_TYPE fX[1] = analysis start   */
/*											 F_TYPE fX[2] = analysis end     */
/*			plotdata *pResults = MTIE results sigmadata structure            */
/*                                                                           */
/*		Return:                                                              */
/*			k   = Outer index (# MTIE points), or -1 if error                */
/*                                                                           */
/* 		sigmadata structure elements for sigma data:					     */
/*			int		points        number of sigma data points                */
/*			float	*tau          pointer to averaging time data array       */
/*			float	*num          pointer to number of data points array     */
/*			float	*sigma        pointer to sigma data array                */
/*			float	*ci           pointer to confidence interval             */
/*			float	*sig_min      pointer to lower sigma error bar array     */
/*			float	*sig_max      pointer to upper sigma error bar array     */
/*			float	*std_dev      pointer to standard deviation array        */
/*			int		*alpha        pointer to noise type exponent array       */
/*			int		*af           pointer to averaging factor                */
/*                                                                           */
/*		Note: Signature of this function is same as previous CalcFastMTIE()  */
/*		It is probably best suited for use by the Sigma function for any one */
/*		averaging factor.                                                    */
/*                                                                           */
/*		Basic Algorithm:                                                     */
/*			An n-point wide window is moved thru the phase data and the      */
/*			difference between the maximum and minimum phase (time) values   */
/*			is found at each window position.  MTIE is the overall maximum   */
/*			of this time interval error over the whole data set.             */
/*                                                                           */
/*		Fast Algorithm #2 of Reference:                                      */
/*			Recursive binary tree MTIE calc.  Start witn x[1] thru x[N).     */
/*          Find and store max and min for adjacent pair, recursively        */
/*			reducing the # values from N (at n=0) to 1 (at n=N-1).           */
/*          Difference of last max and min values is MTIE.                   */
/*                                                                           */
/*		Notes:                                                               */
/*			(1) This function does not handle gaps.                          */
/*			(2) No abort option or progress message for this function        */
/*          (4) # time intervals is N-1, where N = # phase data points, so   */
/*				AF can go from 1 to N-1.                                     */
/*          (3) Works for any averaging factor.                              */
/*                                                                           */
/*		Reference:                                                           */
/*			 M. Li and C. Liao, "A Fast Computation Approach for Maximum     */
/*			 Time Interval Error in Telecommunications Measurement",  Proc.  */
/*			 15th EFTF, March 2001 (to be published).                        */
/*			                                                                 */                                                                         */
/*		Revision record:                                                     */
/*			01/26/01	Drafted from FastCalcMTIE()                          */
/*                                                                           */
/*****************************************************************************/

int WINAPI CalcFastMTIE(F_TYPE fX[], struct sigmadata *pResults)
{
	// Local variables
	// Chars
	char szBuffer[BUFFER_SIZE+1];   // Text buffer
	// Ints
	int i;							// Inner and general inner index
	int j;							// Middle index
	int last_j;          			// Previous middle index
	int k;							// Outer index
	int p;							// Span
	int nStart;						// Analysis start
	int nEnd;						// Analysis end
	int nNum;	  					// # analysis points
	int nAF;						// Averaging factor
	int nStop;						// Max value of outer index
	int tic;						// Progress interval
	// Floats 
	float fMTIE;					// Current MTIE value
	float *fSmall;					// Min phase data value array
	float *fBig;					// Max phase data value array

	// Initialize progress display
	LoadString(hInst, IDS_MESSAGE_MTIECALC, szBuffer,
		BUFFER_SIZE);
	WriteProgressMessage(0, szBuffer);

	// Set analysis limits
	nStart=(int)fX[1];
	nEnd=(int)fX[2];
	nNum=nEnd-nStart+1;

	// Allocate working memory for max and min vectors
	// First pass they must be same size as analysis data array (w/o header)
	// Together they hold all points in overlapping big/small pairs
	// No point in reallocing as they get smaller

	// Allocate maximum array
	if((fBig=(float*)malloc((nNum)*sizeof(float)))==NULL)
	{
		AllocErrorMessage();
		return -1;
	}

	// Allocate minimum array
	if((fSmall=(float*)malloc((nNum)*sizeof(float)))==NULL)
	{
		free(fBig);
		AllocErrorMessage();
		return -1;
	}

	// Initialize the max and min arrays by copying the phase data into them
	// Note: This is the for(1<=j<=N-1) loop at the top of the pseudocode
	for(i=0; i<nNum; i++)
	{
		// Copy data
		fBig[i]=(float)fX[nStart+ARRAY_OFFSET+i-1];
		fSmall[i]=(float)fX[nStart+ARRAY_OFFSET+i-1];
	}
	
	// Calc MTIE

	// Initialize i
	i=1;

	// Outer while loop over i while 2^i <= nNum-1
	// Calc MITE points at octave averaging factors up to largest possible
	// nAF is 2^i and max i is log2(nNum)
	while(nAF=(int)(pow(2, i)) <= nNum-1)
	{
		// Middle loop
		// Update the min & max vectors for this AF
		// Loop through all nNum-AF points
		for(j=1; j<nAF; j++)
		{
			// 
			/* Set 1% interval for progress report */
			tic=(int)ceil((nNum-nAF)/100);

			// Inner loop to find max and min
			// This loop appears to be degenerate
			for(k=1; i<=1; i++)
			{
				fBig[j]=max(fBig[j+i], fBig[j]);
				fSmall[j]=min(fSmall[j+i], fSmall[j]);
			}

			// Display progress
			// Note:  This code provides updates only if index has changed
			// by at least tic amount (1%)
			if((j-last_j)>=tic)
			{
				WriteProgressMessage((int)(100.0*j/(nNum-nAF)),
					szBuffer);
				last_j=j;
			}
		}

		// Initialize the MTIE value
		// Note that the results array is zero-based
		fMTIE=(float)(fBig[0]-fSmall[0]);
		
		// Loop thru the rest of the nNum-nAF points in each vector
		for(i=1; i<nNum-nAF; i++)
		{
			// MTIE is the overall maximum p-p excursion
			fMTIE=(float)(max((fBig[i]-fSmall[i]), fMTIE));
		}

		// Store results
		pResults->sigma[k-1]=fMTIE;
		pResults->points=k;
		pResults->af[k-1]=nAF;
		pResults->num[k-1]=(float)(nNum-nAF);
		// Note: tau calc external to this function

		// Display results
		SendMessage(hDialog, WM_COMMAND,
			GET_WM_COMMAND_MPS(IDC_RUN_DISPLAY, NULL, 0));

		// Check for abort
		if(bAbort)
		{
			// Bail out - program flow goes to code below, then to IDCANCEL
			// handler, and finally back to where this function was called
			break;
		}
	}

	// Free working memory
	free(fBig);
	free(fSmall);

	// Done
	// Clear progress display
	WriteProgressMessage(0, ""); 
	return k;
}

/*****************************************************************************/
#endif
