/*****************************************************************************/
/*                                                                           */
/*                            STABLE_O.C                                     */
/*                                                                           */
/*  Stable32 SQL Database Reading Functions:                                 */
/*      SQLDlgProcData()                                                     */
/*		SQLDetailsDlgProc()                                                  */
/*		GetSQLTableData()                                                    */
/*		SQLConfigDlgProc()                                                   */
/*      DoSQL()                                                              */
/*		AllocCharArray()                                                     */
/*		ReallocCharArray()                                                   */
/*		ClearCharArray()                                                     */
/*		FreeCharArray()                                                      */
/*		TimestampToMJD()                                                     */
/*		FractionalDayToHMS()                                                 */
/*		MJDtoTimestamp()                                                     */
/*																			 */
/*	Uses PostgreSQL 8.0 libpq C library								    	 */
/*	The following special dlls are needed:                 				     */
/*		libq.dll															 */
/*		libeay32.dll														 */
/*		libintl-2.dll														 */
/*		libiconv-2.dll														 */
/*		ssleay32.dll														 */
/*																			 */
/*	 Dialog box & misc procedures for Stable32 database reading function	 */
/*	 Note:  Three types of data are retreived from the TSC MMS database:     */
/*		(1)	Clocks															 */
/*		(2) Measurement records												 */
/*		(3) Measurement data												 */
/*																			 */
/*	The clock and measurement record information is relatively small, and	 */
/*	is accessed via simple SELECT commands.                 				 */
/*	The measurement data, however, can be quite large, and is accessed in	 */
/*	batches (between certain start and end times) with a cursor transaction. */
/*  The reference clock data is subtracted from the measurement clock data,  */
/*  and then divided by the nominal frequency, as part of the query.         */
/*                                                                           */
/*  References:																 */
/*		1.	PostgreSQL Global Development Group, www.postgresql.org.		 */
/*		2.	PostgreSQL 8.0 Documentation, Chapter 27, libpq C Library.		 */
/*		3.	Neil Matthew and Richard Stones, "Beginning Databases with		 */
/*			PostgreSQL", 2nd Edition, Apress, 2005, ISBN 1-59059-478-9.		 */
/*		4.	Korry Douglas and Susan Douglas, "PostgreSQL", 2nd Edition,		 */
/*			Developer's Library, ISBN 0-672-32756-2.						 */
/*                                                                           */
/*	Acknowledgments:														 */
/*		1.	PerkinElmer, Inc., Rubidium Clock Department, Salem, MA, for	 */
/*			developmental support.											 */
/*		2.	Matthew Phillips, Timing Solutions Corp., Boulder, CO, for		 */
/*			programming and testing support.								 */
/*                                                                           */
/*  Revision record:														 */
/*      12/03/05    Created source file	from MMSDBase application			 */
/*		05/07/06	Fixed bReadOK logic (became Stable32 Version 1.46c)      */
/*		07/24/07    Changed to safe string functions                         */
/*		08/04/07	Several fixes:                                           */
/*						1. Span display update moved                         */
/*                      2. fSize rounded                                     */
/*                      3. AF>1 progress bar set up                          */
/*                      4. AF>1 while loop limit changed                     */
/*		11/21/07  Changed progress indicator range to 32 bits                */
/*				  Used AF>1 method to read AF=1                              */
/*				  Added database, host & user to title                       */
/*				  Added choice of not stopping at gap                        */
/*                                                                           */
/* (c) 2005-7  W. Riley  Hamilton Technical Services  All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

// Defines & macros
#define STRICT                  // For Win32 compatibility
#define BIG_BUF_SIZE	1024

// Includes
#include "stable_w.h"           // Stable32 include file
#include "sql.h"				// Subsume this into Stable_w.h?

// Subroutine labels
// SQL controls have numbers between 7000-7099
// SQL subroutines, etc. have numbers between 7100-7199
#define IDC_SQL_MEASTABLE			7100
#define IDC_SQL_READTIMES			7101
#define IDC_SQL_READIMES			7102
#define IDC_SQL_FREETABLES			7103
#define IDC_SQL_READSIZE			7104
#define IDC_SQL_FREQ				7105
#define IDC_SQL_CURRENTMJD			7106
#define IDC_SQL_ENABLE				7107
#define IDC_SQL_CLEARTABLES			7108

// Local variables
// Note: All string allocations have extra character for EOS
HICON hIcon;					// Application icon
PGconn	  *conn;				// Database connection structure pointer
int nMeasClkID=0;				// Measurement clock if # in database
int nRefClkID=0;				// Reference clock id # in database
int nMeasSourceID=0;			// Measurement source id # in database
int nRefSourceID=0;				// Reference source id # in database
int	nMeasMeasID=0;				// Measurement meas id # in database
int	nRefMeasID=0;				// Reference meas id # in database
int nMeasID=0;					// Measurement id # in database
int nRefRec;					// Ref clock selected record table index
int nMeasRec;					// Meas clock selected record table index
// Local flags
BOOL bHideNGMMS=FALSE;			// Flag to hide wrong MMS in record table
BOOL bNoRefRecords=FALSE;		// Flag for no records in ref record table
BOOL bNoMeasRecords=FALSE;		// Flag for no records in meas record table
// Handle for accelerators
HACCEL hAccel;					// Accelerator handle

/*****************************************************************************/
/*																			 */
/*					 SQL Dialog Procedure									 */
/*																			 */
/*		Revision record:													 */
/*			12/03/05	Created from SQL4DlgProc() of MMSDBase				 */
/*			12/04/05	Removed About and Stable buttons & code              */
/*			05/07/06	Fixed bReadOK logic                                  */
/*			07/25/06	Added faster clock data reading method for AF>1      */
/*						with display of # points as they are read            */
/*			08/29/06    Added reading meas ch source_id at IDC_SQL_READ      */
/*			11/22/07	Updated read code per MMSDbase changes               */
/*						(single read method, etc.)                           */
/*			11/23/07	Added PQclear() in read to avoid pagefile overflow   */
/*			01/15/08	Fixed table scrolling problem                        */
/*			02/16/08	Change dababase connection parameter parsing         */
/*						Added PostgreSQL help button                         */
/*			02/17/08	Added single quotes for user name with space         */
/*			02/22/08	Fixed connection information in title                */
/*			03/21/08	Changed read loop limit to read last point           */
/*																			 */
/*****************************************************************************/

BOOL CALLBACK SQLDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	// Local variables
	PENSTRUCT Pen;						// Pen structure
	static char szQuery[BIG_BUF_SIZE+1];		// Query text buffer
	static char szTable[BUFFER_SIZE+1];	// Table text buffer
	char szBuffer[BIG_BUF_SIZE+1];		// General text buffer
	char szBegin[BUFFER_SIZE+1];		// Record start time text buffer
	char szEnd[BUFFER_SIZE+1];			// Record end time text buffer
	char szBeginMJD[BUFFER_SIZE+1];		// Record start MJD text buffer
	char szEndMJD[BUFFER_SIZE+1];		// Record end MJD text buffer
	char szBeginUTC[BUFFER_SIZE+1];		// Record start UTC text buffer
	char szEndUTC[BUFFER_SIZE+1];		// Record end UTC text buffer
	char szUTC[BUFFER_SIZE+1];			// UTC of AF>1 query end
	char szPQdb[BUFFER_SIZE+1];			// String returned by PQdb()
	char szPQhost[BUFFER_SIZE+1];		// String returned by PQhost()
	char szPQuser[BUFFER_SIZE+1];		// String returned by PQuser()
	static char szNow[BUFFER_SIZE+1];	// Current timestamp
	static char szNowMJD[BUFFER_SIZE+1];// Current MJD
	char szEncryptedPassword[BUFFER_SIZE+1];	// Encrypted password
	int i;								// Index
	int j;								// Secondary index
	int n;								// size
	int nPoints;						// # data points read
	int nWrite;							// # points written to file
	int nDays;							// # days in record
	int nHours;							// # hours in record
	int nMinutes;						// # minutes in record
	int nSeconds;						// # seconds in record
	static int nSize;					// # data points to be read
	static int nRefRows;				// # reference database rows
	static int nMeasRows;				// # measurement database rows
	static int nClkRows;				// # clocks in database
	int nDummy;							// To avoid complier warning
	static int nRefClk=-1;				// Ref clock list index
										// Note: Set to -1 so selecting any
										// combo box name will be a change to
										// activate processing
	static int nMeasClk=-1;				// Meas clock list index
										// Note: Set to -1 so selecting any
										// combo box name will be a change to
										// activate processing
	static int nSys=0;					// system_id of selected MMS system
										// Initialized to 0 to indicate no
										// ref record selected
	static int nTau=1;					// Measurement tau
	static int nAF=1;					// Averaging factor
	static double fReadStart;			// Read start time as MJD
	static double fReadEnd;				// Read end time as MJD
	static double fMinStartMJD;			// Min allowable start MJD
										// for ref & meas data overlap
	static double fMaxEndMJD;			// Max allowable end MJD
										//for ref & meas data overlap
	static double fNomMeasFreq;			// Nominal measurement clock frequency
	static double fNomRefFreq;			// Nominal referfence clock frequency
	static double fMJD;					// MJD variable for AF>1 data read
	static double fEndMJD;				// End MJD for AF>1 data read
	double fSize;						// Size of results file
	double fVal;						// MJD double value
	int nEntry;							// User entry to be validated
	static BOOL bMJD=FALSE;				// 0=Date/Time format, 1=MJD format
										// in record table
	static BOOL bDateFormat=TRUE;		// 0=Date/Time format, 1=MJD format
										// in output file
	static BOOL bRefData=FALSE;			// Flag for have valid ref data
	static BOOL bMeasData=FALSE;		// Flag for have valid meas data
	static BOOL bDetailsOK=FALSE;		// Flag for OK details data
	static BOOL bReadOK=FALSE;			// Flag for OK read data
	static BOOL bErrorTrap=FALSE;		// Flag for edit box entry logic
	static BOOL bStableOK=FALSE;		// Flag for data available for Stable32
	static BOOL bAutoFilename=TRUE;		// Flag for auto or user-entered output
										// filename
	static BOOL bReading=FALSE;			// Flag that data is being read
	static BOOL bAbort=FALSE;			// Flag to abort read
	static BOOL bEnable=TRUE;			// BOOL to enable/disable controls
	static BOOL bStopAtGap=TRUE;		// Flag to stop reading at gap in data
	// static BOOL bStart=TRUE;			// First time flag for testing

	// Character table pointers
	static char **RefMMSTab;			// Ref MMS string table
	static char **RefChTab;				// Ref Ch # string table
	static char **MeasMMSTab;			// Meas MMS string table
	static char **MeasChTab;			// Meas Ch # string table
	static char **RefStartDateTab;		// Ref start date string table
	static char **RefEndDateTab;		// Ref end date string table
	static char **MeasStartDateTab;		// Meas start date string table
	static char **MeasEndDateTab;		// Meas end date string table
	static char **RefStartMJDTab;		// Ref start MJD string table
	static char **RefEndMJDTab;			// Ref end MJD string table
	static char **MeasStartMJDTab;		// Meas start MJD string table
	static char **MeasEndMJDTab;		// Meas end MJD string table
	static char **RefSourceTab;			// Ref source id string table
	static char **MeasSourceTab;		// Meas source id string table
	static char **RefMeasTab;			// Ref meas id string table
	static char **MeasMeasTab;			// Meas meas id string table
	static char **ClockIDTab;			// Clock id string table

	// Character table sizes
	// # rows = # records (RefTab or MeasTab) or # clocks (ClockTab)
	// # cols = length of string -1 (MJD biggest for RefTab or MeasTab,
	// ID int for ClockTab)
	static nRefTabRows=10;				// Initialize ref table to 10 rows
	static nRefTabCols=20;				// Initialize ref table to 20 cols
	static nMeasTabRows=10;				// Initialize meas table to 10 rows
	static nMeasTabCols=20;				// Initialize meas table to 20 cols
	static nClockTabRows=10;			// Initialize clock table to 10 rows
	static nClockTabCols=10;			// Initialize clock table to 10 cols

	// PostgreSQL variables
	static char szConninfo[BUFFER_SIZE];	// Connection parameter text buffer
	static PGresult   *res;					// Query result pointer
	static PGresult   *res2;				// 2nd level query result pointer
	// Note: The *conn database connection structure pointer is global
	
	// DataTable variables
	static HWND hRefTab=NULL;			// Handle to reference clock table
	static HWND hMeasTab=NULL;			// Handle to measurement clock table
	static HWND hTable;					// Handle to either table
	static LPVMSTRUCT lpVMData;			// Pointer to VMSTRUCT
	static int nNumRefRows=0;			// # rows in ref table
	static int nNumMeasRows=0;			// # rows in meas table
	DWORD dwStyle;

	// Database connection parameters
	static char szHost[BUFFER_SIZE+1];		// Host IP address
	static char szDatabase[BUFFER_SIZE+1];	// Database name
	static char szUser[BUFFER_SIZE+1];		// User name
	static char szPassword[BUFFER_SIZE+1];	// Password (unencrypted)

	// DataTable color structure
	static DTCOLORSTRUCT MMSColor;

	// TEA variables
	static int k[]={1, 10, 100, 1000};		// 128-bit encryption key
	unsigned int v[2];						// 64-bit encryption variable

	// Dialog box message switch statement
	switch(msg)
	{
		case WM_INITDIALOG:
		{
			// Set the application system menu icon
			SetClassLong(hDlg, GCL_HICON, (LONG)hIcon);

			// Initialize the time/date format radiobox
			Button_SetCheck(GETHCTL(IDC_SQL_UTC), TRUE);
			
			// Deactivate Details button (bDetailsOK initialized to FALSE)
			EnableWindow(GETHCTL(IDC_SQL_DETAILS), bDetailsOK);

			// Deactivate Read button (bReadOK initialized to FALSE)
			EnableWindow(GETHCTL(IDC_SQL_READ), bReadOK);

			// Show hidden record note
			// if different MMS system records are hidden
			ShowWindow(GETHCTL(IDC_SQL_NOTE), bHideNGMMS);

			// Define color for wrong MMS row
			MMSColor.BkgrdColor=DEFAULTCOLOR;
			MMSColor.TextColor=0x000000FF;
			MMSColor.SelBkgrdColor=0x000000FF;
			MMSColor.SelTextColor=DEFAULTCOLOR;

			// Set up MMS & Ch #, start & end, ref & meas
			// date/time/MJD DataTables
			// Get handle for reference table
			hRefTab=GETHCTL(IDC_SQL_REFTAB);

			// Create four columns
			SendMessage(hRefTab, DTM_SETCOLCOUNT, (WPARAM) 4,
				(LPARAM)0);

			// Configure the MMS column to a string
			SendMessage(hRefTab, DTM_SETCOLTYPE, (WPARAM) 0,
				(LPARAM)DTT_ZSTRING);

			// Configure the Ch # column to a string
			SendMessage(hRefTab, DTM_SETCOLTYPE, (WPARAM) 1,
				(LPARAM)DTT_ZSTRING);

			// Configure the Start Date column to a string
			SendMessage(hRefTab, DTM_SETCOLTYPE, (WPARAM) 2,
				(LPARAM)DTT_ZSTRING);

			// Configure the End Date column to a string
			SendMessage(hRefTab, DTM_SETCOLTYPE, (WPARAM) 3,
				(LPARAM)DTT_ZSTRING);

			// Use ##_ format for MMS and space
			// This requires 3 characters (for title)
			SendMessage(hRefTab, DTM_SETCOLDATALEN, (WPARAM)0,
				(LPARAM) 3);

			// Column width is in dialog units (3/char seems enough)
			// So need 3*3=9 - But title requires more
			SendMessage(hRefTab, DTM_SETCOLWIDTH, (WPARAM)0,
				(LPARAM)(15));

			// Use ##_ format for Ch #
			// This requires 3 characters (for title)
			SendMessage(hRefTab, DTM_SETCOLDATALEN, (WPARAM)1,
				(LPARAM) 3);

			// Column width is in dialog units (3/char seems enough)
			// So need 3*3=9 - But title requires more
			SendMessage(hRefTab, DTM_SETCOLWIDTH, (WPARAM)1,
				(LPARAM)(15));

			// Use YYYY-MM-DD HH:MM:SS format for Start Date/Time
			// This requires 19 characters
			// Also plenty of room for XXXXX.XXXXXX MJD format
			SendMessage(hRefTab, DTM_SETCOLDATALEN, (WPARAM)2,
				(LPARAM) 19);

			// Column width is in dialog units (3/char seems enough)
			// So need 19*3=57
			SendMessage(hRefTab, DTM_SETCOLWIDTH, (WPARAM)2,
				(LPARAM)(57));

			// Also use YYYY-MM-DD HH:MM:SS format for End Date/Time
			// This requires 19 characters
			// Also plenty of room for XXXXX.XXXXXX MJD format
			SendMessage(hRefTab, DTM_SETCOLDATALEN, (WPARAM)3,
			(LPARAM) 19);

			// Column width is in dialog units (3/char seems enough)
			// So need 19*3=57
			SendMessage(hRefTab, DTM_SETCOLWIDTH, (WPARAM)3,
				(LPARAM)(57));

			// Label the column headings
			SendMessage(hRefTab, DTM_SETCOLHEAD, (WPARAM)0,
				(LPARAM)(LPSTR)"MMS");
			SendMessage(hRefTab, DTM_SETCOLHEAD, (WPARAM)1,
				(LPARAM)(LPSTR)"Ch #");
			SendMessage(hRefTab, DTM_SETCOLHEAD, (WPARAM)2,
				(LPARAM)(LPSTR)"Start Time");
			SendMessage(hRefTab, DTM_SETCOLHEAD, (WPARAM)3,
				(LPARAM)(LPSTR)"End Time");

			// Center MMS column heading, no data editing, 
			// and makecolumn width fixed
			SendMessage(hRefTab, DTM_SETCOLSTYLE, (WPARAM)0,
				(LPARAM)(DTCS_HEADCENTER | DTCS_FIXED));

			// Center Ch# column heading, no data editing, 
			// and makecolumn width fixed
			SendMessage(hRefTab, DTM_SETCOLSTYLE, (WPARAM)1,
				(LPARAM)(DTCS_HEADCENTER | DTCS_FIXED));

			// Center Start Time column heading, no data editing,
			//  and makecolumn width fixed
			SendMessage(hRefTab, DTM_SETCOLSTYLE, (WPARAM)2,
				(LPARAM)(DTCS_HEADCENTER | DTCS_FIXED));

			// Center End Time column heading, no data editing, 
			// and makecolumn width fixed
			SendMessage(hRefTab, DTM_SETCOLSTYLE, (WPARAM)3,
				(LPARAM)(DTCS_HEADCENTER | DTCS_FIXED));

			// Get handle for measurement table
			hMeasTab=GETHCTL(IDC_SQL_MEASTAB);

			// Create four columns
			SendMessage(hMeasTab, DTM_SETCOLCOUNT, (WPARAM) 4,
				(LPARAM)0);

			// Configure the MMS column to a string
			SendMessage(hMeasTab, DTM_SETCOLTYPE, (WPARAM) 0,
				(LPARAM)DTT_ZSTRING);

			// Configure the Ch # column to a string
			SendMessage(hMeasTab, DTM_SETCOLTYPE, (WPARAM) 1,
				(LPARAM)DTT_ZSTRING);

			// Configure the Start Time column to a string
			SendMessage(hMeasTab, DTM_SETCOLTYPE, (WPARAM) 2,
				(LPARAM)DTT_ZSTRING);

			// Configure the End Time column to a string
			SendMessage(hMeasTab, DTM_SETCOLTYPE, (WPARAM) 3,
				(LPARAM)DTT_ZSTRING);

			// Use ##_ format for MMS and space
			// This requires 3 characters (for title)
			SendMessage(hMeasTab, DTM_SETCOLDATALEN, (WPARAM)0,
				(LPARAM) 3);

			// Column width is in dialog units (3/char seems enough)
			// So need 3*3=9 - But title requires more
			SendMessage(hMeasTab, DTM_SETCOLWIDTH, (WPARAM)0,
				(LPARAM)(15));

			// Use ##_ format for Ch #
			// This requires 3 characters (for title)
			SendMessage(hMeasTab, DTM_SETCOLDATALEN, (WPARAM)1,
				(LPARAM) 3);

			// Column width is in dialog units (3/char seems enough)
			// So need 3*3=9 - But title requires more
			SendMessage(hMeasTab, DTM_SETCOLWIDTH, (WPARAM)1,
				(LPARAM)(15));

			// Use YYYY-MM-DD HH:MM:SS format for Start Time
			// This requires 19 characters
			// Also plenty of room for XXXXX.XXXXXX MJD format
			SendMessage(hMeasTab, DTM_SETCOLDATALEN, (WPARAM)2,
				(LPARAM) 19);

			// Column width is in dialog units (3/char seems enough)
			// So need 19*3=57
			// Also plenty of room for XXXXX.XXXXXX MJD format
			SendMessage(hMeasTab, DTM_SETCOLWIDTH, (WPARAM)2,
				(LPARAM)(57));

			// Also use YYYY-MM-DD HH:MM:SS format for End Time
			// This requires 19 characters
			SendMessage(hMeasTab, DTM_SETCOLDATALEN, (WPARAM)3,
			(LPARAM) 19);

			// Column width is in dialog units (3/char seems enough)
			// So need 19*3=57
			SendMessage(hMeasTab, DTM_SETCOLWIDTH, (WPARAM)3,
				(LPARAM)(57));

			// Label the column headings
			SendMessage(hMeasTab, DTM_SETCOLHEAD, (WPARAM)0,
				(LPARAM)(LPSTR)"MMS");
			SendMessage(hMeasTab, DTM_SETCOLHEAD, (WPARAM)1,
				(LPARAM)(LPSTR)"Ch #");
			SendMessage(hMeasTab, DTM_SETCOLHEAD, (WPARAM)2,
				(LPARAM)(LPSTR)"Start Time");
			SendMessage(hMeasTab, DTM_SETCOLHEAD, (WPARAM)3,
				(LPARAM)(LPSTR)"End Time");

			// Center MMS column heading, no data editing, and make
			// column width fixed
			SendMessage(hMeasTab, DTM_SETCOLSTYLE, (WPARAM)0,
				(LPARAM)(DTCS_HEADCENTER | DTCS_FIXED));

			// Center Ch# column heading, no data editing, and make
			// column width fixed
			SendMessage(hMeasTab, DTM_SETCOLSTYLE, (WPARAM)1,
				(LPARAM)(DTCS_HEADCENTER | DTCS_FIXED));

			// Center Start Time column heading, no data editing, 
			// and makecolumn width fixed
			SendMessage(hMeasTab, DTM_SETCOLSTYLE, (WPARAM)2,
				(LPARAM)(DTCS_HEADCENTER | DTCS_FIXED));

			// Center End Time column heading, no data editing, 
			// and makecolumn width fixed
			SendMessage(hMeasTab, DTM_SETCOLSTYLE, (WPARAM)3,
				(LPARAM)(DTCS_HEADCENTER | DTCS_FIXED));

			// Null pen used for no focus rectangle
			SendMessage(hRefTab, DTM_SETFOCUSPEN, (WPARAM)0,
				(LPARAM)(LPPENSTRUCT)&Pen);

			// Null pen used for no focus rectangle
			SendMessage(hMeasTab, DTM_SETFOCUSPEN, (WPARAM)0,
				(LPARAM)(LPPENSTRUCT)&Pen);

			// Disable multirow & column selection
			dwStyle=SendMessage(hRefTab, DTM_GETWNDSTYLE, 0, 0L);
			dwStyle&=~(DTS_MULTIROW|DTS_COLSELECT|DTS_MULTICOL);
			SendMessage(hRefTab, DTM_SETWNDSTYLE, 0, dwStyle);

			// Disable multirow & column selection
			dwStyle=SendMessage(hMeasTab, DTM_GETWNDSTYLE, 0, 0L);
			dwStyle&=~(DTS_MULTIROW|DTS_COLSELECT|DTS_MULTICOL);
			SendMessage(hMeasTab, DTM_SETWNDSTYLE, 0, dwStyle);

			// Do not initiate communications with data table yet

			// Allocate 2 char tables for ref records start time and
			// end date/time
			// Both will start with 10 rows of 20 chaqracters
			// The # rows will be adjusted dynamically if # records is larger
			// The width fits YYYY-MM-DD HH:MM:SS time format plus EOS
			RefStartDateTab=AllocCharArray(nRefTabRows, nRefTabCols);
			RefEndDateTab=AllocCharArray(nRefTabRows, nRefTabCols);

			// Allocate similar char tables for meas records
			MeasStartDateTab=AllocCharArray(nMeasTabRows, nMeasTabCols);
			MeasEndDateTab=AllocCharArray(nMeasTabRows, nMeasTabCols);

			// Allocate 2 char tables for ref records start time and end MJD
			// Both will start with 10 rows of 20 chaqracters
			// The # rows will be adjusted dynamically if # records is larger
			// The width fits XXXXX.XXXXXX MJD format plus EOS
			RefStartMJDTab=AllocCharArray(nRefTabRows, nRefTabCols);
			RefEndMJDTab=AllocCharArray(nRefTabRows, nRefTabCols);

			// Allocate similar char tables for meas records
			MeasStartMJDTab=AllocCharArray(nMeasTabRows, nMeasTabCols);
			MeasEndMJDTab=AllocCharArray(nMeasTabRows, nMeasTabCols);

			// Allocate similar char tables for ref & meas MMS & Ch # records
			RefMMSTab=AllocCharArray(nRefTabRows, nRefTabCols);
			RefChTab=AllocCharArray(nRefTabRows, nRefTabCols);
			MeasMMSTab=AllocCharArray(nMeasTabRows, nMeasTabCols);
			MeasChTab=AllocCharArray(nMeasTabRows, nMeasTabCols);

			// Allocate meas id char tables
			MeasMeasTab=AllocCharArray(nMeasTabRows, nMeasTabCols);
			RefMeasTab=AllocCharArray(nRefTabRows, nRefTabCols);

			// Allocate source id char tables
			MeasSourceTab=AllocCharArray(nMeasTabRows, nMeasTabCols);
			RefSourceTab=AllocCharArray(nRefTabRows, nRefTabCols);

			// Allocate clock id char table
			ClockIDTab=AllocCharArray(nClockTabRows, nClockTabCols);

			// Clear all tables
			SendMessage(hDlg, WM_COMMAND,
				GET_WM_COMMAND_MPS(IDC_SQL_CLEARTABLES, 0, 0));

			// Clear combo boxes
			ComboBox_ResetContent(GETHCTL(IDC_SQL_REFCLK));
			ComboBox_ResetContent(GETHCTL(IDC_SQL_MEASCLK));

			// Clear details flag
			bDetailsOK=FALSE;

			// Display initial error message
			SetWindowText(GETHCTL(IDC_SQL_START), "Select clocks");
			SetWindowText(GETHCTL(IDC_SQL_END), "Then select records");

			// Initialize averaging factor
			SetWindowText(GETHCTL(IDC_SQL_AF), "1");

			// Get stop at gap flag setting from INI file = MMSDbase.ini
			bStopAtGap=(BOOL)GetPrivateProfileInt("Database", "StopAtGap",
				1, szIniFile);

			// Activate buttons (in case we are returning from reset
			// Reset and Close already enabled
			bEnable=TRUE;
			SendMessage(hDlg, WM_COMMAND,
				GET_WM_COMMAND_MPS(IDC_SQL_ENABLE, 0, 0));

			// Get date format from INI file = MMSDbase.ini
			bDateFormat=(BOOL)GetPrivateProfileInt("Date", "Format", 1,
				szIniFile);

			// Get database connection parameters
			// from INI file = MMSDbase.ini
			GetPrivateProfileString("Database", "Host", "192.168.1.1",
				szHost,	BUFFER_SIZE, szIniFile);
			GetPrivateProfileString("Database", "Database", "tsc-postgres",
				szDatabase,	BUFFER_SIZE, szIniFile);
			GetPrivateProfileString("Database", "User", "tsc", szUser,
				BUFFER_SIZE, szIniFile);
			GetPrivateProfileString("Database", "Password", "tsc",
				szEncryptedPassword, BUFFER_SIZE, szIniFile);

			#if(1) // New code 02/17/08
			// Enclose user name in single quotes if it contains a space
			if(strchr(szUser, ' '))
			{
				// Don't add single quotes if they are already there
				if(szUser[0] != '\'')
				{
					strncpy(szBuffer, "'", BUFFER_SIZE);
					strncat(szBuffer, szUser, BUFFER_SIZE);
					strncat(szBuffer, "'", BUFFER_SIZE);
					strncpy(szUser, szBuffer, BUFFER_SIZE);
				}
			}
			#endif

			// Get length of encrypted password
			n=strlen(szEncryptedPassword);

			// Parse encrypted password
			for(j=0; j<n/16; j++)
			{
				// Scan a word of the the encrypted password
				sscanf(&szEncryptedPassword[16*j], "%08X%08X", &v[0], &v[1]);

				// Decrypt
				Decode(v, k);

				// Conv v[0] back to substring
				for(i=0; i<4; i++)
				{
					szBuffer[i+8*j]=v[0]/(1<<(8*(4-i)));
					v[0]-=(1<<(8*(4-i)));
				}

				// Conv v[1] back to substring
				for(i=0; i<4; i++)
				{
					szBuffer[i+4+8*j]=v[1]/(1<<(8*(4-i)));
					v[1]-=(1<<(8*(4-i)));
				}
			}

			// Add EOS at end
			szBuffer[n/2]='\0';

			// Save recovered password
			strlcpy(szPassword, szBuffer, sizeof(szPassword));

			#if(0) // Obsolete code
			// Make a connection to the database
			// Copy connection parameters into conninfo
			// A blank host IP address will connect to localhost
			if(strlen(szHost)==0)
			{
				// No host address - leave it out - connect to localhost
				_snprintf(szConninfo, sizeof(szConninfo),
					"dbname=%s user=%s password=%s", szDatabase, szUser,
					szPassword);
			}
			else
			{
				// Include host address
				_snprintf(szConninfo, sizeof(szConninfo),
					"dbname=%s user=%s hostaddr=%s password=%s",
					szDatabase, szUser, szHost, szPassword);
			}
			#endif

			#if(1) // New code 02/16/08
			// Make a connection to the database
			// Copy connection parameters into conninfo
			// Database argument required, others optional
			// A blank host IP address will connect to localhost
			// Spaces inserted at end of each argument
			if(strlen(szHost)==0)
			{
				// No host address - leave it out - connect to localhost
				sprintf(szConninfo, "dbname=%s ", szDatabase);
			}
			else // Use host info
			{
				// Include host address
				sprintf(szConninfo, "dbname=%s hostaddr=%s ", szDatabase, szHost);
			}

			// A blank user name will use default
			if(strlen(szUser)==0)
			{
				; // Omit user name - use default
			}
			else // Use user name
			{
				// Include user name
				strcat(szConninfo, "user=");
				strcat(szConninfo, szUser);
				strcat(szConninfo, " ");
			}

			// A blank password will omit it
			if(strlen(szPassword)==0)
			{
				; // Omit password
			}
			else // Use password
			{
				// Include password
				strcat(szConninfo, "password=");
				strcat(szConninfo, szPassword);
			}
			#endif

			// Display connecting message
			WriteMessage("Connecting to database server");
			
			// Connect to PostgreSQL server
			// Note:  This connection will be kept alive for whole program
			conn=PQconnectdb(szConninfo);
		
			// Check to see that the backend connection was successfully made
			if(PQstatus(conn) != CONNECTION_OK)
			{
				// Not connected - re-enter configuration
				{
					// Display not connected message
					WriteMessage("Not connected to database server");
			
					// Show error message and quit 
					_snprintf(szBuffer, sizeof(szBuffer),
						"Connection as user %s to database %s at host"
						" %s failed.\n"
						"Check connection and server.\n"
						"Configuration box will open "
						"to enter connection parameters.",
						szUser, szDatabase, szHost);
						MessageBox(NULL, szBuffer, "Database Error",
						MB_OK | MB_ICONHAND);

						// Open configuration dialog box
						SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_SQL_CONFIG, NULL, 0));

					// Show error message and quit 
					_snprintf(szBuffer, sizeof(szBuffer),
						"Restart Database\nwith new connection parameters.",
						szUser,	szDatabase, szHost);
						MessageBox(NULL, szBuffer, "Database Error",
							MB_OK | MB_ICONHAND);

					// Close database dialog box
                    EndDialog (hDlg, 0);
					return TRUE;
				}
				
			}

			// Get connection information
			// Database
			if(PQdb(conn))
			{
				strcpy(szPQdb, PQdb(conn));
			}
			else // This should never happen
			{
				strcpy(szPQdb, szDatabase);
			}

			// Host
			// Note: PQhost() always returns a null
			// when a connection is established by a hostaddr
			// so can't use that function here
			if( (strcmp(szHost, "127.0.0.1")==0) || (strcmp(szHost, "")==0))
			{
				strcpy(szPQhost, "localhost");
			}
			else // Not local host
			{
				strcpy(szPQhost, szHost);
			}

			// User
			if(PQuser(conn))
			{
				strcpy(szPQuser, PQuser(conn));
			}
			else // This should never happen
			{
				strcpy(szPQuser, szUser);
			}
			
			// Put connection information in title
			sprintf(szBuffer, "MMS Database (Database=%s, Host=%s, User=%s)",
				szPQdb, szPQhost, szPQuser);
			SetWindowText(hDlg, szBuffer);
			
			// Display connected message
			WriteMessage("Connected to database server");
			
			// Do snapshot (until possible reset) of clock names & ids
			// Names will be put into ref & meas clock list combo boxes with
			// nClkRow as index
			// Clock IDs (clock_id) will be put into ClockIDTab[] with same
			// nClkRow as index
			// Can get id for any clock by looking it up in this local table
			// vs combo box index
			// w/o subsequent query that might reflect database changes
			// Can get clock description later from database
			// vs primary key = clock_id 
			
			// Fetch clock names & ids from clock_names database table
			if((res=DoSQL(conn, "SELECT clock_name, clock_id"
				" FROM clock_names ORDER BY clock_name"))==0)
			{
				return 0; // Close program
			}
			
			// Put clock names into reference & measurement combo boxs
			// Put clock ids into local ClockIDTab[] table

			// Save # clocks
			nClkRows = PQntuples(res);

			// No clocks?
			if(nClkRows==0)
			{
				// Load and display no clocks message in clock combo boxes
				nDummy=ComboBox_InsertString(GETHCTL(IDC_SQL_REFCLK), 0,
					"No clocks");
				nDummy=ComboBox_SetCurSel(GETHCTL(IDC_SQL_REFCLK), 0);
				nDummy=ComboBox_InsertString(GETHCTL(IDC_SQL_MEASCLK), 0,
					"No clocks");
				nDummy=ComboBox_SetCurSel(GETHCTL(IDC_SQL_MEASCLK), 0);

				// Deactivate buttons - nothing to do
				// Allow Reset and Close to remain enabled
				bEnable=FALSE;
				SendMessage(hDlg, WM_COMMAND,
					GET_WM_COMMAND_MPS(IDC_SQL_ENABLE, 0, 0));

				return TRUE;
			}

			// Enlarge clock id array as necessary
			if(nClkRows>nClockTabRows)
			{
				// Reallocate clock id table
				// # cols remains at original 10
				// Assumed large enough for any
				// clock_id string
				ClockIDTab=ReallocCharArray(ClockIDTab, nClockTabRows,
					nClkRows, nClockTabCols);

				// Save new measurement table size
				nClockTabRows=nClkRows;
			}

			// Put clock names into reference clock combo box
			// The reversed indexing puts top name at top
			for(i=nClkRows-1; i>=0; i--)
			{
				// Load names into combo box
				nDummy=ComboBox_InsertString(GETHCTL(IDC_SQL_REFCLK), 0,
					PQgetvalue(res, i, 0));
			}

			// Put clock names into measurement clock combo box
			// The reversed indexing puts top name at top
			for(i=nClkRows-1; i>=0; i--)
			{
				// Load names into combo box
				nDummy=ComboBox_InsertString(GETHCTL(IDC_SQL_MEASCLK), 0,
					PQgetvalue(res, i, 0));
			}

			// Put clock ids into ClockIDTab[] array
			// The reversed indexing puts top name at top
			for(i=nClkRows-1; i>=0; i--)
			{
				// Load ids into local table
				strlcpy(ClockIDTab[i], PQgetvalue(res, i, 1), nClockTabCols);
			}

			// Clear result
			PQclear(res);

			// Select 1st reference clock 
			nDummy=ComboBox_SetCurSel(GETHCTL(IDC_SQL_REFCLK), 0);

			// Select 1st measurement clock 
			nDummy=ComboBox_SetCurSel(GETHCTL(IDC_SQL_MEASCLK), 0);

			return TRUE;
		}

		// DataTable virtual memory messages
		// Note: nSys=0 for ref clock (always shown)
		// Get the first row of data
		case DTM_VM_GETFIRST:
		{
			// Get pointer to VMSTRUCT
			lpVMData=(LPVMSTRUCT)lParam;

			// Get row data
			if(hTable==hRefTab)
			{
				// Set index to begining
				lpVMData->dwKey=0;

				if(bMJD)
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						RefMMSTab, RefChTab, RefStartMJDTab,	RefEndMJDTab,
						bMJD, 0, 1);
				}
				else // Date/Time format
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						RefMMSTab, RefChTab, RefStartDateTab, RefEndDateTab,
						bMJD, 0, 1);
				}
			}
			else // Measurement table
			{
				// Set index to begining
				lpVMData->dwKey=0;

				if(bMJD)
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						MeasMMSTab,	MeasChTab, MeasStartMJDTab, MeasEndMJDTab,
						bMJD, nSys, 0);
				}
				else // Date/Time format
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						MeasMMSTab,	MeasChTab, MeasStartDateTab,
						MeasEndDateTab, bMJD, nSys, 0);
				}
			}
		}
		break;

		case DTM_VM_GETLAST:
		{
			// Get pointer to VMSTRUCT
			lpVMData=(LPVMSTRUCT)lParam;

			if(hTable==hRefTab)
			{
				// Set index to last row
				lpVMData->dwKey=nNumRefRows-1; // Always OK

				if(bMJD)
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						RefMMSTab, RefChTab, RefStartMJDTab, RefEndMJDTab,
						bMJD, 0, 1);
				}
				else // Date/Time format
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						RefMMSTab, RefChTab, RefStartDateTab, RefEndDateTab,
						bMJD, 0, 1);
				}
			}
			else // Measurement table
			{
				// Set index to last row
				lpVMData->dwKey=nNumMeasRows-1; // Always OK

				if(bMJD)
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						MeasMMSTab,	MeasChTab, MeasStartMJDTab,	MeasEndMJDTab,
						bMJD, nSys, 0);
				}
				else // Date/Time format
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						MeasMMSTab,	MeasChTab, MeasStartDateTab,
						MeasEndDateTab, bMJD, nSys, 0);
				}
			}
		}
		break;

		// Get the next row of data (data after row=dwKey)
		case DTM_VM_GETNEXT:
		{
			// Get pointer to VMSTRUCT
			lpVMData=(LPVMSTRUCT)lParam;

			if(hTable==hRefTab)
			{
				// Increment index
				lpVMData->dwKey+=1;

				// Avoid crash at bottom of table
				if(lpVMData->dwKey>=(unsigned)nRefRows)
				{
					break;
				}

				if(bMJD)
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						RefMMSTab, RefChTab, RefStartMJDTab, RefEndMJDTab,
						bMJD, 0, 1);
				}
				else // Date/Time format
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						RefMMSTab, RefChTab, RefStartDateTab, RefEndDateTab,
						bMJD, 0, 1);
				}
			}
			else // Measurement table
			{
				// Increment index
				lpVMData->dwKey+=1;

				// Avoid crash at bottom of table
				if(lpVMData->dwKey>=(unsigned)nMeasRows)
				{
					break;
				}

				if(bMJD)
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						MeasMMSTab,	MeasChTab, MeasStartMJDTab, MeasEndMJDTab,
						bMJD, nSys, 0);
				}
				else // Date/Time format
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						MeasMMSTab,	MeasChTab, MeasStartDateTab,
						MeasEndDateTab, bMJD, nSys, 0);
				}
			}
		}
		break;

		// Get the previous row of data (data before row=dwKey)
		case DTM_VM_GETPREV:
		{
			// Get pointer to VMSTRUCT
			lpVMData=(LPVMSTRUCT)lParam;

			if(hTable==hRefTab)
			{
				// Decrement index
				lpVMData->dwKey-=1;

				if(bMJD)
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						RefMMSTab, RefChTab, RefStartMJDTab, RefEndMJDTab,
						bMJD, 0, 1);
				}
				else // Date/Time format
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						RefMMSTab, RefChTab, RefStartDateTab, RefEndDateTab,
						bMJD, 0, 1);
				}
			}
			else // Measurement table
			{
				// Decrement index
				lpVMData->dwKey-=1;

				if(bMJD)
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						MeasMMSTab,	MeasChTab, MeasStartMJDTab, MeasEndMJDTab,
						bMJD, nSys, 0);
				}
				else // Date/Time format
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						MeasMMSTab,	MeasChTab, MeasStartDateTab,
						MeasEndDateTab, bMJD, nSys, 0);
				}
			}
		}
		break;

		// Get the row of data that is specified % down table
		case DTM_VM_GETPERCENT:
		{
			// Get pointer to VMSTRUCT
			lpVMData=(LPVMSTRUCT)lParam;

			if(hTable==hRefTab)
			{
				// Calc index
				lpVMData->dwKey=(lpVMData->dwKey)*(long)nNumRefRows/100L;

				if(bMJD)
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						RefMMSTab, RefChTab, RefStartMJDTab,	RefEndMJDTab,
						bMJD, 0, 1);
				}
				else // Date/Time format
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						RefMMSTab, RefChTab, RefStartDateTab, RefEndDateTab,
						bMJD, 0, 1);
				}
			}
			else // Measurement table
			{
				// Calc index
				lpVMData->dwKey=(lpVMData->dwKey)*(long)nNumMeasRows/100L;

				if(bMJD)
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						MeasMMSTab,	MeasChTab, MeasStartMJDTab, MeasEndMJDTab,
						bMJD, nSys, 0);
				}
				else // Date/Time format
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						MeasMMSTab,	MeasChTab, MeasStartDateTab,
						MeasEndDateTab, bMJD, nSys, 0);
				}
			}
		}
		break;

		// Get the row of data at dwKey
		case DTM_VM_GETROWDATA:
		{
			// Get pointer to VMSTRUCT
			lpVMData=(LPVMSTRUCT)lParam;

			if(hTable==hRefTab)
			{
				if(bMJD)
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						RefMMSTab, RefChTab,RefStartMJDTab, RefEndMJDTab,
						bMJD, 0, 1);
				}
				else // Date/Time format
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						RefMMSTab, RefChTab,RefStartDateTab, RefEndDateTab,
						bMJD, 0, 1);
				}
			}
			else // Measurement table
			{
				if(bMJD)
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						MeasMMSTab,	MeasChTab, MeasStartMJDTab, MeasEndMJDTab,
						bMJD, nSys, 0);
				}
				else // Date/Time format
				{
					lpVMData->nStatus=(short)GetSQLTableData(lpVMData,
						MeasMMSTab,	MeasChTab, MeasStartDateTab,
						MeasEndDateTab, bMJD, nSys, 0);
				}
			}
		}
		break;

		case WM_COMMAND:
		{
			switch(HIWORD(wParam))
			{
				// Detect vertical scroll in DataTable
				case DTN_VMBEGINFETCH:
				{
					// Message could be from either table
					// Table's window handle is in lParam
					hTable=(HWND)lParam;
				}
				break;

				// Detect vertical scroll in DataTable
				case DTN_VSCROLL:
				{
					// Message could be from either table
					// Table's window handle is in lParam
					hTable=(HWND)lParam;
				}
				break;

				// Detect row select in DataTable
				case DTN_BEFOREROWSEL:
				{
					if(bReading)
					{
						// Message could be from either table
						// Table's window handle is in lParam
						hTable=(HWND)lParam;

						// Prohibit row selection during reading
						// Note: This clears row selections and prevents
						// new ones - must reinitialize table after reading
						// to make table normal again
						SendMessage(hTable, DTM_NOTIFYRESULTS, (WPARAM)0,
							(LPARAM)(1));

						// Redisplay record display tables
						// so appearance doesn't change
						// No harm doing both
						SendMessage(hRefTab, DTM_VM_INITIALIZE, (WPARAM)FALSE,
							(LPARAM)nRefRows);
						SendMessage(hMeasTab, DTM_VM_INITIALIZE, (WPARAM)FALSE,
							(LPARAM)nMeasRows);
					}
				}
				// Detect row select in DataTable
				case DTN_ROWSEL:
				{
					if(!bReading) // Can't change selection during read
					{
						// Deactivate Details button
						// Note:  This doesn't seem necessary
						// and flashing Details button looks bad
						// bDetailsOK=FALSE;
						// EnableWindow(GETHCTL(IDC_SQL_DETAILS), bDetailsOK);
						
						// Reinitialize message
						SetWindowText(GETHCTL(IDC_SQL_MESSAGE),
							"Select ref and meas clocks,  "
							"choose their records,"
							" and press Read to extract data "
							"from MMS database.");

						// Message could be from either table
						// Table's window handle is in lParam
						hTable=(HWND)lParam;

						// Get row #
						i=SendMessage(hTable, DTM_GETCURROW, 0, 0L);

						// Save corresponding selected record #
						if(hTable==hRefTab)
						{ 
							// Save selected ref record #
							nRefRec=i;

							// Note: Want to be able to select for reading
							// only pairs of reference and measurement records
							// from the same MMS system
							// (they also have to have overlapping times).
							// Can do that by showing only measurement records
							// for same MMS as selected for the reference clock
							// or by highlighting different MMS records in red.
							// Must be able to change the selection, so can't
							// show records for only one MMS in both tables.
							// Approach is to show all records for reference
							// clock, and highlight those measurement records
							// that aren't from same MMS as selected reference
							// clock. Then if selected reference clock changes
							// to another MMS, another set of measurement
							// records will be shown.  If measurement clock
							// chosen first, all records regardless of MMS will
							// be shown until a reference clock is picked.
							// Get system_id of MMS system for selected ref row
							// Put it into nSys where it may be used in
							// GetTableData[] to determine which measurement
							// records are displayed.
							nSys=atoi(RefMMSTab[i]);

							// Update measurement record table
							// MMS # may be different
							// Get current meas table row #
							i=SendMessage(hMeasTab, DTM_GETCURROW, 0, 0L);

							// Redraw meas table
							SendMessage(hDlg, WM_COMMAND,
								GET_WM_COMMAND_MPS(IDC_SQL_MEASTABLE, NULL,
								0));

							// Go back to previous row
							SendMessage(hMeasTab, DTM_SETCURCELL, 0, i);
						}
						else // Meas table
						{
							// Save selected meas record #
							nMeasRec=i;

							// Get description from measurement_list for
							// measurement clock with meas_id as index
							// Compose search string
							_snprintf(szQuery, sizeof(szQuery),
								"SELECT description FROM measurement_list"
								" WHERE meas_id=%d",
								atoi(MeasMeasTab[nMeasRec]));

							// Do query				
							if((res=DoSQL(conn, szQuery))==0)
							{
								return 0; // Close program
							}

							// Make sure we got OK response
							if(PQntuples(res)!=1)
							{
								_snprintf(szBuffer, sizeof(szBuffer),
									"Error in getting measurement description"
									"\nUnexpected # of rows (%d) returned "
									"from query",
									PQntuples(res));
								TRACE_STR(szBuffer);
								SetWindowText(GETHCTL(IDC_SQL_MEASINFO), "");
								break;
							}

							// Display description of measurement
							SetWindowText(GETHCTL(IDC_SQL_MEASINFO), 
								PQgetvalue(res, 0, 0));

							// Clear result
							PQclear(res);
						}

						// Calc & display read times
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_SQL_READTIMES, NULL, 0));

						// Update read size & #
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_SQL_READSIZE, NULL, 0));
					}
				}
			}
			
			switch(GET_WM_COMMAND_ID(wParam, lParam))
			{

				// Respond to UTC radiobox
				case IDC_SQL_UTC:
				{
					// Clear MJD flag
					bMJD=FALSE;

					// Display Ref dates in table
					if(nRefRows)
					{
						// Set table handle
						hTable=hRefTab;

						// Get row #
						i=SendMessage(hTable, DTM_GETCURROW, 0, 0L);

						// Redraw table with UTC
						SendMessage(hRefTab, DTM_VM_INITIALIZE, (WPARAM)FALSE,
							(LPARAM)nRefRows);

						// Go back to previous row
						SendMessage(hTable, DTM_SETCURCELL, 0, i);
					}

					// Display Meas dates in table
					if(nMeasRows)
					{
						// Set table handle
						hTable=hMeasTab;

						// Get row #
						i=SendMessage(hTable, DTM_GETCURROW, 0, 0L);

						// Redraw table with UTC
						SendMessage(hMeasTab, DTM_VM_INITIALIZE, (WPARAM)FALSE,
							(LPARAM)nMeasRows);

						// Go back to previous row
						SendMessage(hTable, DTM_SETCURCELL, 0, i);
					}
				}
				break;

				// Respond to MJD radiobox
				case IDC_SQL_MJD:
				{
					// Set MJD flag
					bMJD=TRUE;

					// Display Ref dates in table
					if(nRefRows)
					{
						// Set table handle
						hTable=hRefTab;

						// Get row #
						i=SendMessage(hTable, DTM_GETCURROW, 0, 0L);

						// Redraw table with MJD
						SendMessage(hRefTab, DTM_VM_INITIALIZE, (WPARAM)FALSE,
							(LPARAM)nRefRows);

						// Go back to previous row
						SendMessage(hTable, DTM_SETCURCELL, 0, i);
					}

					// Display Meas dates in table
					if(nMeasRows)
					{
						// Set table handle
						hTable=hMeasTab;

						// Get row #
						i=SendMessage(hTable, DTM_GETCURROW, 0, 0L);

						// Redraw table with MJD
						SendMessage(hMeasTab, DTM_VM_INITIALIZE, (WPARAM)FALSE,
							(LPARAM)nMeasRows);

						// Go back to previous row
						SendMessage(hTable, DTM_SETCURCELL, 0, i);
					}
				}
				break;

				// Respond to change in reference clock name
				case IDC_SQL_REFCLK:
				{
					// Get new ref clock index
					nEntry=ComboBox_GetCurSel(GETHCTL(IDC_SQL_REFCLK));

					// Has reference clock changed?
					if(nEntry!=nRefClk)
					{
						// Reinitialize message
						SetWindowText(GETHCTL(IDC_SQL_MESSAGE),
							"Select ref and meas clocks,  choose their "
							"records, and press Read to extract data from "
							"MMS database.");

						// Save the new ref clock index
						nRefClk=nEntry;

						// nRefClk is combo box index and ClockIDTab[] row #
						// (both 0-based)
						// It is not meaningful for MMS database
						// nRefClkID is read from ClockIDTab[]
						// and used as clock_id in
						// clock_names table queries

						// Get clock_id of selected clock
						// from ClockIDTab with nRefClk as index
						nRefClkID=atoi(ClockIDTab[nRefClk]);

						// Fetch description from clock_names table
						// of MMS database
						// Expect 1 row (clock_id is primary key)
						_snprintf(szQuery, sizeof(szQuery),
							"SELECT description FROM clock_names "
							"WHERE clock_id=%d",
							nRefClkID);
						if((res=DoSQL(conn, szQuery))==0)
						{
							return 0; // Close program
						}

						// Make sure we got OK response
						if(PQntuples(res)!=1)
						{
							_snprintf(szBuffer, sizeof(szBuffer),
								"Error in getting reference clock description"
								"\nUnexpected # of rows (%d) returned "
								"from query",
								PQntuples(res));
							TRACE_STR(szBuffer);
							SetWindowText(GETHCTL(IDC_SQL_REFCLKDES), "");
							break;
						}

						// Display description of reference clock
						SetWindowText(GETHCTL(IDC_SQL_REFCLKDES),
							PQgetvalue(res, 0, 0));

						// Clear result
						PQclear(res);

						// Fill reference table
						SendMessage(hDlg, WM_COMMAND,
								GET_WM_COMMAND_MPS(IDC_SQL_REFTABLE, NULL, 0));

						// Calc & display read times
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_SQL_READTIMES, NULL, 0));
					}
				}
				break;

				// Respond to change in measurement clock name
				case IDC_SQL_MEASCLK:
				{
					// Get new meas clock index
					nEntry=ComboBox_GetCurSel(GETHCTL(IDC_SQL_MEASCLK));

					// Has reference clock changed?
					if(nEntry!=nMeasClk)
					{
						// Reinitialize message
						SetWindowText(GETHCTL(IDC_SQL_MESSAGE),
							"Select ref and meas clocks,  choose their "
							"records, and press Read to extract data from "
							"MMS database.");

						// Save the new meas clock index
						nMeasClk=nEntry;

						// nMeasClk is combo box index and ClockIDTab row #
						// (both 0-based)
						// It is not meaningful for MMS database
						// nRefClkID is read from ClockIDTab[]
						// and used as clock_id
						// in clock_names table queries

						// Get clock_id of selected clock from ClockIDTab with
						// nMeasClk as index
						nMeasClkID=atoi(ClockIDTab[nMeasClk]);

						// Fetch description from clock_names table
						// of MMS database
						// Expect 1 row (clock_id is primary key)
						_snprintf(szQuery, sizeof(szQuery),
							"SELECT description FROM clock_names "
							"WHERE clock_id=%d",
							nMeasClkID);
						if((res=DoSQL(conn, szQuery))==0)
						{
							return 0; // Close program
						}

						// Make sure we got OK response
						if(PQntuples(res)!=1)
						{
							_snprintf(szBuffer, sizeof(szBuffer),
								"Error in getting measurement clock "
								"description\nUnexpected # of rows (%d) "
								"returned from query",
								PQntuples(res));
							TRACE_STR(szBuffer);
							SetWindowText(GETHCTL(IDC_SQL_MEASCLKDES), "");
							break;
						}

						// Display description of measurement clock
						SetWindowText(GETHCTL(IDC_SQL_MEASCLKDES),
							PQgetvalue(res, 0, 0));

						// Clear result
						PQclear(res);

						// Fill measurement table
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_SQL_MEASTABLE, NULL, 0));

						// Calc & display read times
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_SQL_READTIMES, NULL, 0));
					}
				}
				break;

				// Internal subroutine to fill reference clock record table
				// Connection to database is established before entering this
				// subroutine
				case IDC_SQL_REFTABLE:
				{
					// Get Start & End Dates
					// We have clock_id, and want list of all measurements for it
					// In particular, we want start and end times
					// begin_utc and end_utc
					// and the MMS and channel #s to put in display table

					// Start a transaction block
					res=PQexec(conn, "BEGIN");
					if(PQresultStatus(res) != PGRES_COMMAND_OK)
					{
						// Display query error message
						_snprintf(szBuffer, sizeof(szBuffer),
							"BEGIN command failed\nProgram will close.");
						MessageBox(hwndMain, szBuffer, "Database Error",
							MB_ICONHAND);

						// Quit
						PQclear(res);
						PQfinish(conn);
						PostQuitMessage(0);
					}
					PQclear(res);

					// Get start time ordered by start date
					// (so row # is consistent)
					// Also get source_id to use to get MMS & Ch #s
					// and meas_id (the primary key to measurement_list table)
					// Compose search string
					_snprintf(szQuery, sizeof(szQuery),
						"SELECT begin_utc, source_id, meas_id FROM "
						"measurement_list WHERE clock_id=%d "
						"ORDER BY begin_utc DESC, meas_id",
						nRefClkID);

					// Do start time query				
					if((res=DoSQL(conn, szQuery))==0)
					{
						return 0; // Close program
					}

					// Check for NULL (No records)
					if(PQgetisnull(res, 0, 0))
					{
						nRefRows=1;
						bNoRefRecords=TRUE;
						hTable=hRefTab;
						SendMessage(hRefTab, DTM_VM_INITIALIZE, (WPARAM)FALSE,
							(LPARAM)nRefRows);
						break;
					}
					else
					{
						bNoRefRecords=FALSE;
					}

					// Get # rows returned for start reference records
					nRefRows=PQntuples(res);

					// Make sure we got a response
					if(PQntuples(res)==0)
					{
						TRACE_STR("Error in getting reference start times\n"
							"No rows returned from query");
						SendMessage(hRefTab, DTM_RESETDATA, 0, 0L);
						break;
					}

					// Enlarge reference arrays as necessary
					if(nRefRows>nRefTabRows)
					{
						// Reallocate reference tables
						RefMMSTab=ReallocCharArray(RefMMSTab, nRefTabRows,
							nRefRows, nRefTabCols);
						RefChTab=ReallocCharArray(RefChTab, nRefTabRows,
							nRefRows, nRefTabCols);
						RefStartDateTab=ReallocCharArray(RefStartDateTab,
							nRefTabRows, nRefRows, nRefTabCols);
						RefStartMJDTab=ReallocCharArray(RefStartMJDTab,
							nRefTabRows, nRefRows, nRefTabCols);
						RefEndDateTab=ReallocCharArray(RefEndDateTab,
							nRefTabRows, nRefRows, nRefTabCols);
						RefEndMJDTab=ReallocCharArray(RefEndMJDTab,
							nRefTabRows, nRefRows, nRefTabCols);
						RefSourceTab=ReallocCharArray(RefSourceTab,
							nRefTabRows, nRefRows, nRefTabCols);
						RefMeasTab=ReallocCharArray(RefMeasTab, nRefTabRows,
							nRefRows, nRefTabCols);

						// Save new measurement table size
						nRefTabRows=nRefRows;
					}
					
					// Put start dates, etc. into reference array
					if(nRefRows)
					{
						for(i=0; i<nRefRows; i++)
						{
							// Check for NULL?
							
							// Get source id of this record
							nRefSourceID=atoi(PQgetvalue(res, i, 1));

							// Put source id into its table
							strlcpy(RefSourceTab[i], PQgetvalue(res, i, 1),
								nRefTabCols);

							// Put meas id into its table
							strlcpy(RefMeasTab[i], PQgetvalue(res, i, 2),
								nRefTabCols);

							_snprintf(szQuery, sizeof(szQuery),
								"SELECT system_id, channel, active"
								" FROM measurement_sources"
								" WHERE source_id=%d", nRefSourceID);

							// Do 2nd level query for MMS & Ch #				
							if((res2=DoSQL(conn, szQuery))==0)
							{
								return 0; // Close program
							}

							// Make sure we got a result back
							if(PQntuples(res2))
							{
								// Put MMS system_id into its table
								// Format it to 2 digits (e.g. 01)
								// with leading space
								_snprintf(szQuery, sizeof(szQuery),
									" %02d", atoi(PQgetvalue(res2, 0, 0)));
								strlcpy(RefMMSTab[i], szQuery, nRefTabCols);

								// Put channel # into its table
								// Format it to 2 digits (e.g. 01)
								// with leading space
								_snprintf(szQuery, sizeof(szQuery),
									" %02d", atoi(PQgetvalue(res2, 0, 1)));
								strlcpy(RefChTab[i], szQuery, nRefTabCols);
							}
							else // No result
							{
								_snprintf(szBuffer, sizeof(szBuffer),
									"No result returned fron 2nd level "
									"query with i=%d", i);
								TRACE_STR(szBuffer);

								// Fill in table items with placeholders
								strlcpy(RefMMSTab[i], " ??", nRefTabCols);
								strlcpy(RefChTab[i], " ??", nRefTabCols);
							}

							// Get date from database
							strlcpy(RefStartDateTab[i], PQgetvalue(res, i, 0),
								nRefTabCols);

							// Convert date/time to MJD and store it too
							TimestampToMJD(RefStartDateTab[i], RefStartMJDTab[i],
								&fVal);

							// Clear result
							PQclear(res2);
						}
						bRefData=TRUE;
					}
					else // No ref records
					{
						nRefRows=1; // This makes empty table look better
						strlcpy(RefStartDateTab[0], "", nRefTabCols);
						bRefData=FALSE;
					}

					// Clear result
					PQclear(res);

					// Get end dates ordered by start date
					// (so row # is consistent)
					// These can be null (blank strings) - that's OK
					// Compose search string
					_snprintf(szQuery, sizeof(szQuery),
						"SELECT end_utc FROM measurement_list "
						"WHERE clock_id=%d"
						" ORDER BY begin_utc DESC, meas_id", nRefClkID);

					// Do end time query				
					if((res=DoSQL(conn, szQuery))==0)
					{
						return 0; // Close program
					}

					// Make sure we got a response
					if(PQntuples(res)==0)
					{
						TRACE_STR("Error in getting measurement end times\n"
							"No rows returned from query");
						SendMessage(hMeasTab, DTM_RESETDATA, 0, 0L);
						break;
					}

					// Get # rows returned for reference end records
					nRefRows=PQntuples(res);

					// Put end dates into reference array
					if(nRefRows)
					{
						for(i=0; i<nRefRows; i++)
						{
							// Get date from database
							strlcpy(RefEndDateTab[i], PQgetvalue(res, i, 0),
								nRefTabCols);

							// Check for NULL = run continuing
							if(PQgetisnull(res, i, 0))
							{
								// NULL Ref End
								// Put special values into end date
								// to indicate active meas
								strlcpy(RefEndDateTab[i], "Run Continuing",
									nRefTabCols);
								strlcpy(RefEndMJDTab[i], "0", nRefTabCols);
							}
							else // Not NULL
							{
								// Convert date/time to MJD and store it too
								TimestampToMJD(RefEndDateTab[i],
									RefEndMJDTab[i], &fVal);
							}
						}
						bRefData=TRUE;
					}
					else // No Meas Records

					{
						nRefRows=1; // This makes empty table look better
						strlcpy(RefEndDateTab[0], "", nRefTabCols);
						bRefData=FALSE;
					}

					// Clear result
					PQclear(res);

					// End transaction
					res = PQexec(conn, "END");
					if(PQresultStatus(res) != PGRES_COMMAND_OK)
					{
						// Display query error message
						_snprintf(szBuffer, sizeof(szBuffer),
							"END command failed\nProgram will close.");
						MessageBox(hwndMain, szBuffer, "Database Error",
							MB_ICONHAND);

						// Quit
						PQclear(res);
						PQfinish(conn);
						PostQuitMessage(0);
					}
					PQclear(res);

					// Display dates in table
					hTable=hRefTab;
					SendMessage(hRefTab, DTM_VM_INITIALIZE, (WPARAM)FALSE,
						(LPARAM)nRefRows);

					// Save selected ref record #
					nRefRec=0;

					// Calc & display read times
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_SQL_READTIMES, NULL, 0));
				}
				break;

				// Internal subroutine to fill measurement clock record table
				// Connection to database is established before entering this
				// subroutine
				case IDC_SQL_MEASTABLE:
				{
					// Get Start & End Dates
					// Start a transaction block
					res=PQexec(conn, "BEGIN");
					if(PQresultStatus(res) != PGRES_COMMAND_OK)
					{
						// Display query error message
						_snprintf(szBuffer, sizeof(szBuffer),
							"BEGIN command failed\nProgram will close.");
						MessageBox(hwndMain, szBuffer, "Database Error",
							MB_ICONHAND);

						// Quit
						PQclear(res);
						PQfinish(conn);
						PostQuitMessage(0);
					}
					PQclear(res);

					// Get start times, etc. for selected measurement clock
					// Ordered by start date so row # is consistent
					// Also get source id and meas id
					// Compose search string
					_snprintf(szQuery, sizeof(szQuery),
						"SELECT begin_utc, source_id, meas_id FROM "
						"measurement_list WHERE clock_id=%d "
						"ORDER BY begin_utc DESC, meas_id",
						nMeasClkID);

					// Do query					
					if((res=DoSQL(conn, szQuery))==0)
					{
						return 0; // Close program
					}

					// Check for NULL (No records)
					if(PQgetisnull(res, 0, 0))
					{
						nMeasRows=1;
						bNoMeasRecords=TRUE;
						hTable=hMeasTab;
						SendMessage(hMeasTab, DTM_VM_INITIALIZE, (WPARAM)FALSE,
							(LPARAM)nMeasRows);
						break;
					}
					else
					{
						bNoMeasRecords=FALSE;
					}

					// Make sure we got a response
					if(PQntuples(res)==0)
					{
						TRACE_STR("Error in getting measurement start times\n"
							"No rows returned from query");
						SendMessage(hMeasTab, DTM_RESETDATA, 0, 0L);
						break;
					}

					// Display start dates
					// Get # records returned for this clock
					nMeasRows=PQntuples(res);

					// Enlarge measurement arrays as necessary
					if(nMeasRows>nMeasTabRows)
					{
						// Reallocate measurement tables
						MeasMMSTab=ReallocCharArray(MeasMMSTab,
							nMeasTabRows, nMeasRows, nMeasTabCols);
						MeasChTab=ReallocCharArray(MeasChTab,
							nMeasTabRows, nMeasRows, nMeasTabCols);
						MeasStartDateTab=ReallocCharArray(MeasStartDateTab,
							nMeasTabRows, nMeasRows, nMeasTabCols);
						MeasStartMJDTab=ReallocCharArray(MeasStartMJDTab,
							nMeasTabRows, nMeasRows, nMeasTabCols);
						MeasEndDateTab=ReallocCharArray(MeasEndDateTab,
							nMeasTabRows, nMeasRows, nMeasTabCols);
						MeasEndMJDTab=ReallocCharArray(MeasEndMJDTab,
							nMeasTabRows, nMeasRows, nMeasTabCols);
						MeasSourceTab=ReallocCharArray(MeasSourceTab,
							nMeasTabRows, nMeasRows, nMeasTabCols);
						MeasMeasTab=ReallocCharArray(MeasMeasTab,
							nMeasTabRows, nMeasRows, nMeasTabCols);

						// Save new measurement table size
						nMeasTabRows=nMeasRows;
					}
					
					// Put start dates into measurement array
					if(nMeasRows)
					{
						for(i=0; i<nMeasRows; i++)
						{
							// Get source id of this record
							nMeasSourceID=atoi(PQgetvalue(res, i, 1));

							// Put source id into its table
							strlcpy(MeasSourceTab[i], PQgetvalue(res, i, 1),
								nMeasTabCols);

							// Put meas id into its table
							strlcpy(MeasMeasTab[i], PQgetvalue(res, i, 2),
								nMeasTabCols);

							// Compose query
							_snprintf(szQuery, sizeof(szQuery),
								"SELECT system_id, channel FROM "
								"measurement_sources WHERE source_id=%d",
								nMeasSourceID);

							// Do 2nd level query for MMS & Ch #				
							if((res2=DoSQL(conn, szQuery))==0)
							{
								return 0; // Close program
							}

							// Make sure we got a result back
							// 11/04/05 PM testing could produce 0
							// indicating no result and causing crash
							// in atoi()
							if(PQntuples(res2))
							{
								// Put MMS system_id into its table
								// Format it to 2 digits (e.g. 01)
								// with leading space
								_snprintf(szQuery, sizeof(szQuery), " %02d",
									atoi(PQgetvalue(res2, 0, 0)));
								strlcpy(MeasMMSTab[i], szQuery, nMeasTabCols);

								// Put channel # into its table
								// Format it to 2 digits (e.g. 01)
								// with leading space
								_snprintf(szQuery, sizeof(szQuery), " %02d",
									atoi(PQgetvalue(res2, 0, 1)));
								strlcpy(MeasChTab[i], szQuery, nMeasTabCols);
							}
							else  // No result
							{
								_snprintf(szBuffer, sizeof(szBuffer),
									"No result returned fron 2nd level "
									"query with i=%d", i);
								TRACE_STR(szBuffer);

								// Fill in table items with placeholders
								strlcpy(MeasMMSTab[i], " ??", nMeasTabCols);
								strlcpy(MeasChTab[i], " ??", nMeasTabCols);
							}

							// Get date from database
							strlcpy(MeasStartDateTab[i], PQgetvalue(res, i, 0),
								nMeasTabCols);

							// Convert date/time to MJD and store it too
							TimestampToMJD(MeasStartDateTab[i],
								MeasStartMJDTab[i], &fVal);

							// Set meas data flag
							bMeasData=TRUE;
						}
					}
					else
					{
						nMeasRows=1; // This makes empty table look better
						strlcpy(MeasStartDateTab[0], "", nMeasTabCols);
						bMeasData=FALSE;
					}

					// Clear result
					PQclear(res);

					// Get end times for selected measurement clock
					// Ordered by start date so row # is consistent
					// These can be null (blank strings) - that's OK
					// Compose search string
					_snprintf(szQuery, sizeof(szQuery),
						"SELECT end_utc FROM measurement_list "
						"WHERE clock_id=%d ORDER BY begin_utc DESC, meas_id",
						nMeasClkID);

					// Do query
					if((res=DoSQL(conn, szQuery))==0)
					{
						return 0; // Close program
					}
					 
					// Make sure we got a response
					if(PQntuples(res)==0)
					{
						TRACE_STR("Error in getting measurement end times\n"
							"No rows returned from query");
						SendMessage(hMeasTab, DTM_RESETDATA, 0, 0L);
						break;
					}

					// Display end times
					// Get # records (rows) returned for this clock
					nMeasRows=PQntuples(res);

					// Put end times into array
					if(nMeasRows)
					{
						for(i=0; i<nMeasRows; i++)
						{
							// Get date from database
							strlcpy(MeasEndDateTab[i], PQgetvalue(res, i, 0),
								nMeasTabCols);

							// Check for NULL = run continuing
							if(PQgetisnull(res, i, 0))
							{
								// NULL Meas End
								// Put special values into end date
								// to indicate active meas
								strlcpy(MeasEndDateTab[i], "Run Continuing",
									nMeasTabCols);
								strlcpy(MeasEndMJDTab[i], "0", nMeasTabCols);
							}
							else // Not NULL
							{
								// Convert date/time to MJD and store it too
								TimestampToMJD(MeasEndDateTab[i],
									MeasEndMJDTab[i], &fVal);

								// Set meas data flag
								bMeasData=TRUE;
							}
						}
					}
					else
					{
						nMeasRows=1; // This makes empty table look better
						strlcpy(MeasEndDateTab[0], "", nMeasTabCols);
						bMeasData=FALSE;
					}

					// Clear result
					PQclear(res);

					// Display dates in table
					hTable=hMeasTab;
					SendMessage(hMeasTab, DTM_VM_INITIALIZE, (WPARAM)FALSE,
						(LPARAM)nMeasRows);

					// Save selected meas record #
					nMeasRec=0;

					// Get description from measurement_list
					// for measurement clock with meas_id as index
					// Compose search string
					_snprintf(szQuery, sizeof(szQuery),
						"SELECT description FROM measurement_list "
						"WHERE meas_id=%d",
						atoi(MeasMeasTab[nMeasRec]));

					// Do query				
					if((res=DoSQL(conn, szQuery))==0)
					{
						return 0; // Close program
					}

					// Make sure we got OK response
					if(PQntuples(res)!=1)
					{
						_snprintf(szBuffer, sizeof(szBuffer),
							"Error in getting measurement description\n"
							"Unexpected # of rows (%d) returned from query",
							PQntuples(res));
						TRACE_STR(szBuffer);
						SetWindowText(GETHCTL(IDC_SQL_MEASINFO), "");
					}

					// Display description of measurement
					SetWindowText(GETHCTL(IDC_SQL_MEASINFO),
						PQgetvalue(res, 0, 0));

					// Clear result
					PQclear(res);

					// End transaction
					res = PQexec(conn, "END");
					if(PQresultStatus(res) != PGRES_COMMAND_OK)
					{
						// Display query error message
						_snprintf(szBuffer, sizeof(szBuffer),
							"END command failed\nProgram will close.");
						MessageBox(hwndMain, szBuffer, "Database Error",
							MB_ICONHAND);

						// Quit
						PQclear(res);
						PQfinish(conn);
						PostQuitMessage(0);
					}
					PQclear(res);

					// Calc & display read times
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_SQL_READTIMES, NULL, 0));
				}
				break;

				// Internal subroutine to find & display
				// read start & end times
				case IDC_SQL_READTIMES:
				{
					// Initialize bDetailsOK to FALSE
					bDetailsOK=FALSE;

					// Initialize bReadOK to FALSE
					bReadOK=FALSE;
					
					// Clear read time displays
					SetWindowText(GETHCTL(IDC_SQL_START), "");
					SetWindowText(GETHCTL(IDC_SQL_END), "");
					SetWindowText(GETHCTL(IDC_SQL_STARTMJD), "");
					SetWindowText(GETHCTL(IDC_SQL_ENDMJD), "");

					// Check if have valid ref data
					if(!bRefData)
					{
						// Deactivate Read button
						EnableWindow(GETHCTL(IDC_SQL_READ), bReadOK);

						// Display error message
						SetWindowText(GETHCTL(IDC_SQL_START),
							"No ref clock data");
						SetWindowText(GETHCTL(IDC_SQL_END), "Read impossible");

						break;
					}

					// Check if have valid meas data
					if(!bMeasData)
					{
						// Deactivate Read button
						EnableWindow(GETHCTL(IDC_SQL_READ), bReadOK);

						// Display error message
						SetWindowText(GETHCTL(IDC_SQL_START),
							"No meas clock data");
						SetWindowText(GETHCTL(IDC_SQL_END), "Read impossible");

						break;
					}

					// Have valid ref & meas data so can observe details
					bDetailsOK=TRUE;

					// Activate Details button
					EnableWindow(GETHCTL(IDC_SQL_DETAILS), bDetailsOK);
					
					// Check that selected records have same MMS
					// Only do this if ref MMS is selected (non-zero)
					if(atoi(RefMMSTab[nRefRec]))
					{
						if((atoi(RefMMSTab[nRefRec])) !=
							(atoi(MeasMMSTab[nMeasRec])))
						{
							// Deactivate Read button
							EnableWindow(GETHCTL(IDC_SQL_READ), bReadOK);

							// Display error message
							SetWindowText(GETHCTL(IDC_SQL_START), "Different MMS");
							SetWindowText(GETHCTL(IDC_SQL_END), "Read impossible");

							// Set row text red
							SendMessage(hMeasTab, DTM_SETACCESSPOS, 0, nMeasRec);
							SendMessage(hMeasTab, DTM_SETROWCOLOR, DTA_ACCESS,
								(LPARAM)(LPDTCOLORSTRUCT)&MMSColor);

							break;
						}
					}

					// Check if ref & meas clock are the same
					if((nRefClkID==nMeasClkID)||(nRefClkID==0)||
						(nMeasClkID==0))
					{
						// Deactivate Read button
						EnableWindow(GETHCTL(IDC_SQL_READ), bReadOK);

						// Display error message
						SetWindowText(GETHCTL(IDC_SQL_START), "Clocks same");
						SetWindowText(GETHCTL(IDC_SQL_END), "Read impossible");

						break;
					}
					
					// Find overlapping times from selected ref & meas records
					// Selected # of records are nRefRec & nMeasRec
					// Use MJD format for easier calc

					// Find later start time
					if(atof(RefStartMJDTab[nRefRec])>=
						atof(MeasStartMJDTab[nMeasRec]))
					{
						// Ref start later						
						// Get read start MJD
						fReadStart=atof(RefStartMJDTab[nRefRec]);

						// Display selected ref start time
						SetWindowText(GETHCTL(IDC_SQL_STARTMJD),
							RefStartMJDTab[nRefRec]);
						SetWindowText(GETHCTL(IDC_SQL_START),
							RefStartDateTab[nRefRec]);
					}
					else // Meas start time is later
					{
						// Meas start later
						// Get read start MJD
						fReadStart=atof(MeasStartMJDTab[nMeasRec]);

						// Display selected meas start time
						SetWindowText(GETHCTL(IDC_SQL_STARTMJD),
							MeasStartMJDTab[nMeasRec]);
						SetWindowText(GETHCTL(IDC_SQL_START),
							MeasStartDateTab[nMeasRec]);
					}
						
					// Find earlier end time
					// This test valid only if neither run continuing
					if((atof(RefEndMJDTab[nRefRec])) &&
						(atof(MeasEndMJDTab[nMeasRec])))
					{
						if(atof(RefEndMJDTab[nRefRec])<=
							atof(MeasEndMJDTab[nMeasRec]))
						{
							// Ref end earlier						
							// Get read end MJD
							fReadEnd=atof(RefEndMJDTab[nRefRec]);

							// Display selected ref end time
							SetWindowText(GETHCTL(IDC_SQL_ENDMJD),
								RefEndMJDTab[nRefRec]);
							SetWindowText(GETHCTL(IDC_SQL_END),
								RefEndDateTab[nRefRec]);
						}
						else // Meas end time is earlier
						{
							// Meas end earlier
							// Get read end MJD
							fReadEnd=atof(MeasEndMJDTab[nMeasRec]);

							// Display selected meas end time
							SetWindowText(GETHCTL(IDC_SQL_ENDMJD),
								MeasEndMJDTab[nMeasRec]);
							SetWindowText(GETHCTL(IDC_SQL_END),
								MeasEndDateTab[nMeasRec]);
						}
					}

					// If ref run still continuing
					if(atof(RefEndMJDTab[nRefRec])==0)
					{
						// Use meas end time
						if(atof(MeasEndMJDTab[nMeasRec])==0)
						{
							// Both still continuing
							SetWindowText(GETHCTL(IDC_SQL_ENDMJD), "");
							SetWindowText(GETHCTL(IDC_SQL_END),
								"Both Runs Continuing");
							
							// Set bReadOK
							bReadOK=TRUE;
							// Activate Read button
							EnableWindow(GETHCTL(IDC_SQL_READ), bReadOK);
						}
						else // Meas run has stopped
						{
							// Make sure meas end time is after ref start time
							// or there is no overlapping data
							fReadEnd=atof(MeasEndMJDTab[nMeasRec]);

							if(fReadEnd>=fReadStart)
							{
								// Use meas end time
								SetWindowText(GETHCTL(IDC_SQL_ENDMJD),
									MeasEndMJDTab[nMeasRec]);
								SetWindowText(GETHCTL(IDC_SQL_END),
									MeasEndDateTab[nMeasRec]);

								// Set bReadOK
								bReadOK=TRUE;

								// Activate Read button
								EnableWindow(GETHCTL(IDC_SQL_READ), bReadOK);

							}
							else // No overlap
							{
								// Deactivate Read button
								EnableWindow(GETHCTL(IDC_SQL_READ), bReadOK);

								// Display error messages
								SetWindowText(GETHCTL(IDC_SQL_START),
									"No overlapping data");
								SetWindowText(GETHCTL(IDC_SQL_END),
									"Read impossible");

								// Clear read MJD time displays
								SetWindowText(GETHCTL(IDC_SQL_STARTMJD), "");
								SetWindowText(GETHCTL(IDC_SQL_ENDMJD), "");

								// Clear span windows
								SetWindowText(GETHCTL(IDC_SQL_LENGTH), "");
								SetWindowText(GETHCTL(IDC_SQL_SIZE), "");
							}
						}
					}
					else if(atof(MeasEndMJDTab[nMeasRec])==0) 
					// If meas run still continuing
					{
						// Use ref end time
						if(atof(RefEndMJDTab[nRefRec])==0)
						{
							// Both still continuing
							SetWindowText(GETHCTL(IDC_SQL_ENDMJD), "");
							SetWindowText(GETHCTL(IDC_SQL_END),
								"Both Runs Continuing");
							
							// Set bReadOK
							bReadOK=TRUE;
							
							// Activate Read button
							EnableWindow(GETHCTL(IDC_SQL_READ), bReadOK);
						}
						else // Ref run has stopped
						{
							// Make sure ref end time is after meas start time
							// or there is no overlapping data
							fReadEnd=atof(RefEndMJDTab[nRefRec]);

							if(fReadEnd>=fReadStart)
							{
								// Use ref end time
								SetWindowText(GETHCTL(IDC_SQL_ENDMJD),
									RefEndMJDTab[nRefRec]);
								SetWindowText(GETHCTL(IDC_SQL_END),
									RefEndDateTab[nRefRec]);
								
								// Set bReadOK
								bReadOK=TRUE;

								// Activate Read button
								EnableWindow(GETHCTL(IDC_SQL_READ), bReadOK);
							}
							else // No overlap
							{
								// Deactivate Read button
								EnableWindow(GETHCTL(IDC_SQL_READ), bReadOK);

								// Display error messages
								SetWindowText(GETHCTL(IDC_SQL_START),
									"No overlapping data");
								SetWindowText(GETHCTL(IDC_SQL_END),
									"Read impossible");

								// Clear read MJD time displays
								SetWindowText(GETHCTL(IDC_SQL_STARTMJD), "");
								SetWindowText(GETHCTL(IDC_SQL_ENDMJD), "");

								// Clear span windows
								SetWindowText(GETHCTL(IDC_SQL_LENGTH), "");
								SetWindowText(GETHCTL(IDC_SQL_SIZE), "");
							}
						}
					}
					else
					// But, if read start is after read end,
					// there is no overlap
					// and can't extract data from this pair of records
					{
						if(fReadStart>fReadEnd)
						{
							// Deactivate Read button
							EnableWindow(GETHCTL(IDC_SQL_READ), bReadOK);

							// Display error messages
							SetWindowText(GETHCTL(IDC_SQL_START),
								"No overlapping data");
							SetWindowText(GETHCTL(IDC_SQL_END),
								"Read impossible");

							// Clear bReadOK
							bReadOK=FALSE;

							// Deactivate Read button
							EnableWindow(GETHCTL(IDC_SQL_READ), bReadOK);

							// Clear read MJD time displays
							SetWindowText(GETHCTL(IDC_SQL_STARTMJD), "");
							SetWindowText(GETHCTL(IDC_SQL_ENDMJD), "");

							// Clear span windows
							SetWindowText(GETHCTL(IDC_SQL_LENGTH), "");
							SetWindowText(GETHCTL(IDC_SQL_SIZE), "");
						}
						else
						{
							// Set bReadOK
							bReadOK=TRUE;

							// Ref & Meas data overlaps - valid to read
							// Activate Read button
							EnableWindow(GETHCTL(IDC_SQL_READ), bReadOK);
						}
					}

					if(bReadOK)
					{
						// Save allowable min & max MJD
						// The Read database query to fetch the clock data
						// uses the start and end timestamps as limits
						// It is important that these not become wider
						// because of rounding errors, etc.
						// If that can happen, it might be wise
						// to pad them by a second
						// But seems OK with MJD rounding in TimestampToMJD()
						GetWindowText(GETHCTL(IDC_SQL_STARTMJD), szQuery,
							BUFFER_SIZE);
						fMinStartMJD=atof(szQuery);
						GetWindowText(GETHCTL(IDC_SQL_ENDMJD), szQuery,
							BUFFER_SIZE);
						fMaxEndMJD=atof(szQuery);
						if(fMaxEndMJD==0) // Will be 0 if runs continuing 
						{
							// Use current timestamp
							if((res=DoSQL(conn,
								"SELECT current_timestamp"))==0)
							{
								return 0; // Close program
							}

							// Get end timestamp
							strlcpy(szQuery, PQgetvalue(res, 0, 0),
								sizeof(szQuery));

							// Truncate it to remove fractional seconds
							// and time zone
							szQuery[19]='\0';

							// Convert it to MJD
							TimestampToMJD(szQuery, szBuffer, &fVal);
							fMaxEndMJD=atof(szBuffer);

							// Clear result
							PQclear(res);
						}

						// Calc & display read size
						SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_SQL_READSIZE, NULL, 0));
					}

					// Set tau display
					// The measurement tau is same for all channels within a
					// database system
					// All measurements are apparently taken at a rate of
					// 1 per second and are stored at a retention rate
					// that can be at any multiple of that as set by
					// the retention_rate item in the measdb_system table
					// It would seem that the tau can therefore be determined
					// by that value and that it is always
					// an integer # of seconds

					// Query retention_rate					
					if((res=DoSQL(conn,
						"SELECT retention_rate FROM measdb_system"))==0)
					{
						return 0; // Close program
					}

					// Make sure we got OK response
					if(PQntuples(res)!=1)
					{
						_snprintf(szBuffer, sizeof(szBuffer),
							"Error in getting retention rate\n"
							"Unexpected # of rows (%d) returned from query",
							PQntuples(res));
						TRACE_STR(szBuffer);
						SetWindowText(GETHCTL(IDC_SQL_MEASTAU), "");
						nTau=1;  // Better than nothing
						_snprintf(szBuffer, sizeof(szBuffer), "%d", nAF*nTau);
						SetWindowText(GETHCTL(IDC_SQL_READTAU), szBuffer);
						break;
					}

					// Set meas tau
					// Note that tau is an int (always a multiple of 1 sec)
					// Check for NULL?
							
					nTau=atoi(PQgetvalue(res, 0, 0));
					_snprintf(szQuery, sizeof(szQuery), "%d", nTau);
					SetWindowText(GETHCTL(IDC_SQL_MEASTAU), szQuery);

					// Read AF
					GetWindowText(GETHCTL(IDC_SQL_AF), szBuffer,
						BUFFER_SIZE);
					nAF=abs(atoi(szBuffer));
					
					// Set read tau
					_snprintf(szBuffer, sizeof(szBuffer), "%d", nAF*nTau);
					SetWindowText(GETHCTL(IDC_SQL_READTAU), szBuffer);

					// Clear result
					PQclear(res);
				}
				break;

				// Internal subroutine to get current MJD
				case IDC_SQL_CURRENTMJD:
				{
					// Do query	to get current timestamp				
					if((res=DoSQL(conn, "SELECT current_timestamp"))==0)
					{
						return 0; // Close program
					}

					// Get current timestamp
					strlcpy(szNow, PQgetvalue(res, 0, 0), sizeof(szNow));

					// Clear result
					PQclear(res);

					// Truncate it to remove fractional seconds & time zone
					szNow[19]='\0';

					// Convert it to MJD
					TimestampToMJD(szNow, szNowMJD, &fVal);
				}
				break;

				// Internal subroutine to calc length & size of read
				case IDC_SQL_READSIZE:
				{
					// Clear span windows if read is impossible
					GetWindowText(GETHCTL(IDC_SQL_END), szBuffer, BUFFER_SIZE);
					if((strcmp(szBuffer, "Read impossible")==0))
					{
						// Clear span windows and return
						SetWindowText(GETHCTL(IDC_SQL_LENGTH), "");
						SetWindowText(GETHCTL(IDC_SQL_SIZE), "");
						break;
					}

					// Estimate the size of the file that will be read for AF=1
					// from the times and meas tau
					GetWindowText(GETHCTL(IDC_SQL_STARTMJD), szBeginMJD,
						BUFFER_SIZE);
					GetWindowText(GETHCTL(IDC_SQL_ENDMJD), szEndMJD,
						BUFFER_SIZE);
					fSize=atof(szEndMJD);

					if(fSize==0) // Runs continuing - use current MJD
					{
						// Use current date/time as end_utc
						// Do query	to get current timestamp				
						if((res=DoSQL(conn, "SELECT current_timestamp"))==0)
						{
							return 0; // Close program
						}

						// Get end timestamp
						strlcpy(szEnd, PQgetvalue(res, 0, 0), sizeof(szEnd));

						// Cleas result
						PQclear(res);

						// Truncate it to remove fractional seconds & time zone
						szEnd[19]='\0';

						// Convert it to MJD
						TimestampToMJD(szEnd, szEndMJD, &fVal);

						// Calc fSize again
						fSize=atof(szEndMJD);
					}

					// Calc size based on start and end times
					fSize-=atof(szBeginMJD);	// Record length in days
					fSize*=((86400.0)/(nTau));  // Record length in points for AF=1
					fSize++;					// # is 1 more than span

					// Round fSize to int
					nSize=(int)(fSize+0.5);
					sprintf(szBuffer, "%d", nSize);

					SetWindowText(GETHCTL(IDC_SQL_SIZE), szBuffer);
					// Estimate the length of the record
					// in days, hours, minutes & seconds
					fSize*=(double)nTau; // seconds
					nDays=(int)(fSize/86400.);
					fSize-=(double)(nDays*86400);
					nHours=(int)(fSize/3600.);
					fSize-=(double)(nHours*3600);
					nMinutes=(int)(fSize/60.);
					fSize-=(double)(nMinutes*60);
					nSeconds=(int)fSize;
					_snprintf(szBuffer, sizeof(szBuffer),
						"%dd, %dh, %dm, %ds", nDays, nHours, nMinutes,
						nSeconds);
					SetWindowText(GETHCTL(IDC_SQL_LENGTH), szBuffer);
				}
				break;

				// Internal subroutine to get nominal clock frequencies
				case IDC_SQL_FREQ:
				{
					// Nominal ref & meas clock frequencies
					// fNomRefFreq & fNomMeasFreq
					// are stored in the measurement_list table versus meas_id
					// They cannot be NULL
					// Get frequency from measurement_list for reference clock
					// with meas_id as index
					// Compose search string
					_snprintf(szQuery, sizeof(szQuery),
						"SELECT frequency FROM measurement_list "
						"WHERE meas_id=%d",	atoi(RefMeasTab[nRefRec]));

					// Do query				
					if((res=DoSQL(conn, szQuery))==0)
					{
						return 0; // Close program
					}

					// Make sure we got OK response
					if(PQntuples(res)!=1)
					{
						_snprintf(szBuffer, sizeof(szBuffer),
							"Error in getting reference frequency\n"
							"Unexpected # of rows (%d) returned from query",
							PQntuples(res));
						TRACE_STR(szBuffer);
						fNomRefFreq=5e6;  // Better than nothing
						break;
					}

					// Get ref clock frequency
					fNomRefFreq=atof(PQgetvalue(res, 0, 0));

					// Clear result
					PQclear(res);

					// Get frequency from measurement_list for meas clock
					// with meas_id as index
					// Compose search string
					_snprintf(szQuery, sizeof(szQuery),
						"SELECT frequency FROM measurement_list "
						"WHERE meas_id=%d", atoi(MeasMeasTab[nMeasRec]));

					// Do query				
					if((res=DoSQL(conn, szQuery))==0)
					{
						return 0; // Close program
					}

					// Make sure we got OK response
					if(PQntuples(res)!=1)
					{
						_snprintf(szBuffer, sizeof(szBuffer),
							"Error in getting measurement frequency\n"
							"Unexpected # of rows (%d) returned from query",
							PQntuples(res));
						TRACE_STR(szBuffer);
						fNomMeasFreq=5e6;  // Better than nothing
						break;
					}

					// Get meas clock frequency
					fNomMeasFreq=atof(PQgetvalue(res, 0, 0));

					// Clear result
					PQclear(res);
				}
				break;

				// Internal subroutine to clear character tables
				case IDC_SQL_CLEARTABLES:
				{
					// Clear all tables
					ClearCharArray(RefStartDateTab, nRefTabRows);
					ClearCharArray(RefEndDateTab, nRefTabRows);
					ClearCharArray(MeasStartDateTab, nMeasTabRows);
					ClearCharArray(MeasEndDateTab, nMeasTabRows);
					ClearCharArray(RefStartMJDTab, nRefTabRows);
					ClearCharArray(RefEndMJDTab, nRefTabRows);
					ClearCharArray(MeasStartMJDTab, nMeasTabRows);
					ClearCharArray(MeasEndMJDTab, nMeasTabRows);
					ClearCharArray(RefMMSTab, nRefTabRows);
					ClearCharArray(RefChTab, nRefTabRows);
					ClearCharArray(MeasMMSTab, nMeasTabRows);
					ClearCharArray(MeasChTab, nMeasTabRows);
					ClearCharArray(MeasSourceTab, nMeasTabRows);
					ClearCharArray(RefSourceTab, nRefTabRows);
					ClearCharArray(MeasMeasTab, nMeasTabRows);
					ClearCharArray(RefMeasTab, nRefTabRows);
					ClearCharArray(ClockIDTab, nClockTabRows);
				}
				break;

				// Internal subroutine to free character tables
				// Called during normal program shutdown, or if database error
				case IDC_SQL_FREETABLES:
				{
					// Free tables
					FreeCharArray(RefStartDateTab, nRefTabRows);
					FreeCharArray(RefEndDateTab, nRefTabRows);
					FreeCharArray(MeasStartDateTab, nMeasTabRows);
					FreeCharArray(MeasEndDateTab, nMeasTabRows);
					FreeCharArray(RefStartMJDTab, nRefTabRows);
					FreeCharArray(RefEndMJDTab, nRefTabRows);
					FreeCharArray(MeasStartMJDTab, nMeasTabRows);
					FreeCharArray(MeasEndMJDTab, nMeasTabRows);
					FreeCharArray(RefMMSTab, nRefTabRows);
					FreeCharArray(RefChTab, nRefTabRows);
					FreeCharArray(MeasMMSTab, nMeasTabRows);
					FreeCharArray(MeasChTab, nMeasTabRows);
					FreeCharArray(MeasSourceTab, nMeasTabRows);
					FreeCharArray(RefSourceTab, nRefTabRows);
					FreeCharArray(MeasMeasTab, nMeasTabRows);
					FreeCharArray(RefMeasTab, nRefTabRows);
					FreeCharArray(ClockIDTab, nClockTabRows);
				}
				break;

				// Internal subroutine to enable/disable controls
				case IDC_SQL_ENABLE:
				{
					// Enable/disable buttons
					// Note: Reset and Close not included
					//EnableWindow(GETHCTL(IDC_SQL_DETAILS), bEnable);
					EnableWindow(GETHCTL(IDC_SQL_DETAILS), bDetailsOK);
					EnableWindow(GETHCTL(IDC_SQL_CONFIG), bEnable);
					EnableWindow(GETHCTL(IDC_SQL_HELP), bEnable);
					EnableWindow(GETHCTL(IDC_SQL_UTC), bEnable);
					EnableWindow(GETHCTL(IDC_SQL_MJD), bEnable);

					// Enasble/disable combo boxes
					ComboBox_Enable(GETHCTL(IDC_SQL_REFCLK), bEnable);
					ComboBox_Enable(GETHCTL(IDC_SQL_MEASCLK), bEnable);

					// Enable/disable edit boxes
					Edit_Enable(GETHCTL(IDC_SQL_START), bEnable);
					Edit_Enable(GETHCTL(IDC_SQL_STARTMJD), bEnable);
					Edit_Enable(GETHCTL(IDC_SQL_END), bEnable);
					Edit_Enable(GETHCTL(IDC_SQL_ENDMJD), bEnable);
					Edit_Enable(GETHCTL(IDC_SQL_AF), bEnable);
				}
				break;

				// Respond to Start read time edit box
				case IDC_SQL_START:
				{
                    // Respond to edit control when it loses the input focus
					// unless it loses it to the cancel button
                    if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
                        (!bErrorTrap) && (GetFocus()!=GETHCTL(IDC_SQL_CANCEL)))
					{
						// Set error flag to avoid multiple error messages
                        bErrorTrap=TRUE;

						// Get new start timestamp
						GetWindowText(GETHCTL(IDC_SQL_START), szBuffer,
							BUFFER_SIZE);

						// Get limits
						GetWindowText(GETHCTL(IDC_SQL_STARTMJD), szQuery,
							BUFFER_SIZE);
						fMinStartMJD=atof(szQuery);
						GetWindowText(GETHCTL(IDC_SQL_ENDMJD), szQuery,
							BUFFER_SIZE);
						fMaxEndMJD=atof(szQuery);
						
						// Blank box and zero value is special case
						// when both runs are continuing
						if(fMaxEndMJD==0)
						{
							// Use current MJD as end MJD
							SendMessage(hDlg, WM_COMMAND,
								GET_WM_COMMAND_MPS(IDC_SQL_CURRENTMJD, NULL,
								0));
							fMaxEndMJD=atof(szNowMJD);
						}

						// Validate timestamp entry
						if((ValidateTimestamp(hDlg, szBuffer, fMinStartMJD,
							fMaxEndMJD))==TRUE)
						{
							// Use new timestamp
							// Reinitialize message
							SetWindowText(GETHCTL(IDC_SQL_MESSAGE),
								"Select ref and meas clocks,  choose their "
								"records, and press Read to extract data "
								"from MMS database.");

							// Put corresponding MJD into STARTMJD edit control
							TimestampToMJD(szBuffer, szQuery, &fVal);
							SetWindowText(GETHCTL(IDC_SQL_STARTMJD), szQuery);

							// Refresh display
							SetWindowText(GETHCTL(IDC_SQL_START), szBuffer);
						}
						else
						{
							// Restore original start timestamp from its MJD
							GetWindowText(GETHCTL(IDC_SQL_STARTMJD), szBuffer,
								BUFFER_SIZE);
							MJDtoTimestamp(atof(szBuffer), szQuery);
							SetWindowText(GETHCTL(IDC_SQL_START), szQuery);
						}
						// Calc & display read size
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_SQL_READSIZE, NULL, 0));
					}
					else
					{
						// Reset error flag
						bErrorTrap=FALSE;
					}
				}
				break;

				// Respond to Start MJD read time edit box
				case IDC_SQL_STARTMJD:
				{
                    // Respond to edit control when it loses the input focus
					// unless it loses it to the cancel button
                    if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
                        (!bErrorTrap) && (GetFocus()!=GETHCTL(IDC_SQL_CANCEL)))
					{
						// Set error flag to avoid multiple error messages
                        bErrorTrap=TRUE;

						// Get new start mjd
						GetWindowText(GETHCTL(IDC_SQL_STARTMJD), szBuffer,
							BUFFER_SIZE);

						// Get limits
						GetWindowText(GETHCTL(IDC_SQL_STARTMJD), szQuery,
							BUFFER_SIZE);
						fMinStartMJD=atof(szQuery);
						GetWindowText(GETHCTL(IDC_SQL_ENDMJD), szQuery,
							BUFFER_SIZE);
						fMaxEndMJD=atof(szQuery);

						// Blank box and zero value is special case
						// when both runs are continuing
						if(fMaxEndMJD==0)
						{
							// Use current MJD as end MJD
							SendMessage(hDlg, WM_COMMAND,
								GET_WM_COMMAND_MPS(IDC_SQL_CURRENTMJD, NULL,
								0));
							fMaxEndMJD=atof(szNowMJD);
						}

						if(fMaxEndMJD==0)
						{
							// Set end MJD to current MJD
						}
						
						// Validate mjd entry
						if((ValidateMJD(hDlg, atof(szBuffer), fMinStartMJD,
							fMaxEndMJD))==TRUE)
						{
							// Use new mjd

							// Reinitialize message
							SetWindowText(GETHCTL(IDC_SQL_MESSAGE),
								"Select ref and meas clocks,  choose their "
								"records, and press Read to extract data "
								"from MMS database.");

							// Put corresponding timestamp
							// into START edit control
							MJDtoTimestamp(atof(szBuffer), szQuery);
							SetWindowText(GETHCTL(IDC_SQL_START), szQuery);

							// Refresh mjd display
							TimestampToMJD(szQuery, szBuffer, &fVal);
							SetWindowText(GETHCTL(IDC_SQL_STARTMJD), szBuffer);
						}
						else
						{
							// Restore original start mjd from its timestamp
							GetWindowText(GETHCTL(IDC_SQL_START), szBuffer,
								BUFFER_SIZE);
							TimestampToMJD(szBuffer, szQuery, &fVal);
							SetWindowText(GETHCTL(IDC_SQL_STARTMJD), szQuery);
						}
						// Calc & display read size
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_SQL_READSIZE, NULL, 0));
					}
					else
					{
						// Reset error flag
						bErrorTrap=FALSE;
					}
				}
				break;

				// Respond to End read time edit box
				case IDC_SQL_END:
				{
                    // Respond to edit control when it loses the input focus
					// unless it loses it to the cancel button
                    if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
                        (!bErrorTrap) && (GetFocus()!=GETHCTL(IDC_SQL_CANCEL)))
					{
						// Set error flag to avoid multiple error messages
                        bErrorTrap=TRUE;

						// Get new end timestamp
						GetWindowText(GETHCTL(IDC_SQL_END), szBuffer,
							BUFFER_SIZE);

						// Get limits
						GetWindowText(GETHCTL(IDC_SQL_STARTMJD), szQuery,
							BUFFER_SIZE);
						fMinStartMJD=atof(szQuery);
						GetWindowText(GETHCTL(IDC_SQL_ENDMJD), szQuery,
							BUFFER_SIZE);
						fMaxEndMJD=atof(szQuery);

						// Blank box and zero value is special case
						// when both runs are continuing
						if(fMaxEndMJD==0)
						{
							// Use current MJD as end MJD
							SendMessage(hDlg, WM_COMMAND,
								GET_WM_COMMAND_MPS(IDC_SQL_CURRENTMJD, NULL,
								0));
							fMaxEndMJD=atof(szNowMJD);
						}

						// Validate timestamp entry
						// Check for special case
						if(strcmp(szBuffer, "Both Runs Continuing")==0)
						{
							// Both Runs Continuing
						}
						else if((ValidateTimestamp(hDlg, szBuffer,
							fMinStartMJD, fMaxEndMJD))==TRUE)
						{
							// Use new timestamp
							// Reinitialize message
							SetWindowText(GETHCTL(IDC_SQL_MESSAGE),
								"Select ref and meas clocks,  choose their "
								"records, and press Read to extract data "
								"from MMS database.");

							// Put corresponding MJD
							// into ENDMJD edit control
							TimestampToMJD(szBuffer, szQuery, &fVal);
							SetWindowText(GETHCTL(IDC_SQL_ENDMJD), szQuery);

							// Refresh display
							SetWindowText(GETHCTL(IDC_SQL_END), szBuffer);
						}
						else
						{
							// Restore original start timestamp from its MJD
							GetWindowText(GETHCTL(IDC_SQL_ENDMJD), szBuffer,
								BUFFER_SIZE);
							MJDtoTimestamp(atof(szBuffer), szQuery);
							SetWindowText(GETHCTL(IDC_SQL_END), szQuery);
						}
						// Calc & display read size
						SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_SQL_READSIZE, NULL, 0));
					}
					else
					{
						// Reset error flag
						bErrorTrap=FALSE;
					}
				}
				break;

				// Respond to End MJD read time edit box
				case IDC_SQL_ENDMJD:
				{
                    // Respond to edit control when it loses the input focus
					// unless it loses it to the cancel button
                    if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
                        (!bErrorTrap) && (GetFocus()!=GETHCTL(IDC_SQL_CANCEL)))
					{
						// Set error flag to avoid multiple error messages
                        bErrorTrap=TRUE;

						// Get new end mjd
						GetWindowText(GETHCTL(IDC_SQL_ENDMJD), szBuffer,
							BUFFER_SIZE);

						// Get limits
						GetWindowText(GETHCTL(IDC_SQL_STARTMJD), szQuery,
							BUFFER_SIZE);
						fMinStartMJD=atof(szQuery);
						GetWindowText(GETHCTL(IDC_SQL_ENDMJD), szQuery,
							BUFFER_SIZE);
						fMaxEndMJD=atof(szQuery);

						// Blank box and zero value is special case
						// when both runs are continuing
						if(fMaxEndMJD==0)
						{
							// Use current MJD as end MJD
							SendMessage(hDlg, WM_COMMAND,
								GET_WM_COMMAND_MPS(IDC_SQL_CURRENTMJD, NULL,
								0));
							fMaxEndMJD=atof(szNowMJD);
						}

						// Validate mjd entry
						else if((ValidateMJD(hDlg, atof(szBuffer),
							fMinStartMJD, fMaxEndMJD))==TRUE)
						{
							// Use new mjd

							// Reinitialize message
							SetWindowText(GETHCTL(IDC_SQL_MESSAGE),
								"Select ref and meas clocks,  choose their "
								"records, and press Read to extract data "
								"from MMS database.");

							// Put corresponding timestamp
							// into END edit control
							MJDtoTimestamp(atof(szBuffer), szQuery);
							SetWindowText(GETHCTL(IDC_SQL_END), szQuery);

							// Refresh mjd display
							TimestampToMJD(szQuery, szBuffer, &fVal);
							SetWindowText(GETHCTL(IDC_SQL_ENDMJD), szBuffer);
						}
						else
						{
							// Restore original end mjd from its timestamp
							GetWindowText(GETHCTL(IDC_SQL_END), szBuffer,
								BUFFER_SIZE);
							if((TimestampToMJD(szBuffer, szQuery, &fVal)==0))
							{
								SetWindowText(GETHCTL(IDC_SQL_ENDMJD),
									szQuery);
							}
							else // Error: Continuing text in end timestamp box
							{
								SetWindowText(GETHCTL(IDC_SQL_ENDMJD), "");
							}
						}
						// Calc & display read size
						SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_SQL_READSIZE, NULL, 0));
					}
					else
					{
						// Reset error flag
						bErrorTrap=FALSE;
					}
				}
				break;

				// Respond to AF edit box
				case IDC_SQL_AF:
				{
                    // Respond to edit control when it loses the input focus
					// unless it loses it to the cancel button
                    if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
                        (!bErrorTrap) && (GetFocus()!=GETHCTL(IDC_SQL_CANCEL)))
					{
						// Set error flag to avoid multiple error messages
                        bErrorTrap=TRUE;

						// Get (new) avg factor
						GetWindowText(GETHCTL(IDC_SQL_AF), szBuffer,
							BUFFER_SIZE);
						nEntry=abs(atoi(szBuffer));

						// Has AF changed
						if(nEntry != nAF)
						{
							// Reinitialize message
							SetWindowText(GETHCTL(IDC_SQL_MESSAGE),
								"Select ref and meas clocks,  choose their "
								"records, and press Read to extract data "
								"from MMS database.");
						}

						// Check entry
						// Edit control set to numbers only
						// abs() insures positive number - so does numbers only
						// Need trap to exclude 0
						// Any positive value S/B OK
						// Won't get any more than 1 analysis point after
						// decimation

						// Exclude 0
						if(nEntry==0)
						{
							nEntry=1;
						}

						// Accept entry
						nAF=nEntry;
						_snprintf(szBuffer, sizeof(szBuffer), "%d", nAF);
						SetWindowText(GETHCTL(IDC_SQL_AF), szBuffer);

						// Display read tau
						_snprintf(szBuffer, sizeof(szBuffer), "%d", nAF*nTau);
						SetWindowText(GETHCTL(IDC_SQL_READTAU), szBuffer);
					}
					else
					{
						// Reset error flag
						bErrorTrap=FALSE;
					}
				}
				break;

				// Respond to Read button
				case IDC_SQL_READ:
				{
					// Get measurement source_id
					nMeasSourceID=atoi(MeasSourceTab[nMeasRec]);

					// Get stop at gap flag setting from INI file
					bStopAtGap=(BOOL)GetPrivateProfileInt("Database",
						"StopAtGap", 1, szIniFile);

					// Detect abort button during reading
					if(bReading)
					{
						bAbort=TRUE;
						break;
					}
					
					// Read must read difference of
					// overlapping reference and clock
					// data (divided by nominal frequency) and timetags
					// and store them in phase data array
					// Database connection is still alive

					#if(0) // Removed 03/21/08 - No longer used
					// Read ref data using cursor
					// Start a transaction block
					res=PQexec(conn, "BEGIN");
					if(PQresultStatus(res) != PGRES_COMMAND_OK)
					{
						// Display query error message
						_snprintf(szBuffer, sizeof(szBuffer),
							"BEGIN command failed\nProgram will close.");
						MessageBox(hwndMain, szBuffer, "Database Error",
							MB_ICONHAND);

						// Quit
						PQclear(res);
						PQfinish(conn);
						PostQuitMessage(0);
					}
					PQclear(res);
					#endif

					// Selected record has index nRefRec
					// Selected ref source_id is RefSourceTab[nRefRec]
					// Selected begin_utc is RefStartDateTab[nRefRec]
					// Selected end_utc is either RefEndDataTab[nRefRec]
					// or current_date if both ref & meas runs
					// are still continuing in which case
					// atof(RefEndMJD[nRefRec]) will be 0

					// Get reference source_id
					nRefSourceID=atoi(RefSourceTab[nRefRec]);

					// Get start & end timestamps from Read Times edit controls
					GetWindowText(GETHCTL(IDC_SQL_START), szBegin,
						BUFFER_SIZE);
					GetWindowText(GETHCTL(IDC_SQL_END), szEnd, BUFFER_SIZE);

					// Test for continuing ref record 
					// Note: End MJD=0 is flag for both runs continuing
					// User may have entered another value into end edit box
					// In that case use it, not current time

					// Get end MJD from edit box
					GetWindowText(GETHCTL(IDC_SQL_ENDMJD), szQuery,
						BUFFER_SIZE);
					if((atof(szQuery))==0)
					{
						// Both runs continuing and no end time entered by user
						// Use current timestamp as end time
						// if both runs continuing

						if(atof(RefEndMJDTab[nRefRec])==0)
						{
							// Continuing ref record
							// Test for continuing meas record also
							if(atof(MeasEndMJDTab[nMeasRec])==0)
							{
								// Continuing meas record also
								// Use current date/time as end_utc
								// Do query	to get current timestamp				
								if((res=DoSQL(conn,
									"SELECT current_timestamp"))==0)
								{
									return 0; // Close program
								}

								// Get end timestamp
								strlcpy(szEnd, PQgetvalue(res, 0, 0),
									sizeof(szEnd));

								// Truncate it to remove
								// fractional seconds & time zone
								szEnd[19]='\0';

								PQclear(res);
							}
							else // Meas record has ended
							{
								// Use meas end_utc
								strlcpy(szEnd, MeasEndDateTab[nMeasRec],
									sizeof(szEnd));
							}
						}
					}

					// Save szBegin and szEnd for use later in AF>1 section
					strlcpy(szBeginUTC, szBegin, sizeof(szBeginUTC));
					strlcpy(szEndUTC, szEnd, sizeof(szEndUTC));

					// Put single quotes around begin & end timestamps
					strlcpy(szBuffer, "'", sizeof(szBuffer));
					strlcat(szBuffer, szBegin, sizeof(szBuffer));
					strlcat(szBuffer, "'", sizeof(szBuffer));
					strlcpy(szBegin, szBuffer, sizeof(szBegin));
					strlcpy(szBuffer, "'", sizeof(szBuffer));
					strlcat(szBuffer, szEnd, sizeof(szBuffer));
					strlcat(szBuffer, "'", sizeof(szBuffer));
					strlcpy(szEnd, szBuffer, sizeof(szEnd));
					
					// Now we're ready to do the measurements table query
					// What kind of pathological cases are possible?
					// Can we count on there being valid data based on
					// start & end dates?
					// What happens if a clock is disconnected (or fails)
					// without being logged out?

					// Get nominal freqs
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_SQL_FREQ, NULL, 0));

					// Initialize data point & write counts
					nPoints=0; // Needed for decimation logic
					nWrite=0;  // Needed for display & progress bar
					
					// Display reading message
					SetWindowText(GETHCTL(IDC_SQL_MESSAGE),
						"Reading data\nPress Abort to abort");

					// Deactivate buttons during read
					// Can't press Reset or Close during read
					// Ditto clock, clock record, etc.
					// Don't want to allow others to respond either
					EnableWindow(GETHCTL(IDC_SQL_RESET), FALSE);
					EnableWindow(GETHCTL(IDC_SQL_CANCEL), FALSE);
					bEnable=FALSE;
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_SQL_ENABLE, 0, 0));

					// Change label of Read button to Abort
					SetWindowText(GETHCTL(IDC_SQL_READ), "A&bort");

					// Set reading flag
					bReading=TRUE;

					// Reset abort flag
					bAbort=FALSE;

					// Reallocate minimum-sized phase data and timetags arrays
					ClearDataArray(PHASE_DATA);
					ClearTimetagArray(PHASE_DATA);

					// Set busy flag
					bBusy=FALSE;

					// Display wait cursor
					SetCursor(hWait);

					// Clear phase data flag
					bPhaseData=FALSE;

					// Update status bar to show no phase data while reading
					UpdateStatusBar(hwndMain, 0);

					// Get high-resolution start and end MJDs as doubles
					// Start MJD is fMJD
					// Get incremented during read process
					// End MJD is fEndMJD
					TimestampToMJD(szBeginUTC, szBuffer, &fMJD);
					TimestampToMJD(szEndUTC, szBuffer, &fEndMJD);
						
					// Size of data set to be read is fSize/nAF
					// Realloc phase data
					if((fNew=(F_TYPE*)realloc(fPhaseData,
						((nSize/nAF)+ARRAY_OFFSET+EXTRA_MEM+1)
						*sizeof(F_TYPE)))==NULL)
					{
						// Display error message
						AllocErrorMessage();

						// Reset busy flag
						bBusy=FALSE;
							
						// Display normal cursor
						SetCursor(hArrow);

						// Abort
						return 0;
					}
					else
					{
						fPhaseData=fNew;
					}
					fData=fPhaseData;
					bPhaseRedraw=TRUE;

					// Realloc phase timetags
					if((fNew=(F_TYPE*)realloc(fPhaseTimetags,
						((nSize/nAF)+ARRAY_OFFSET+EXTRA_MEM+1)
						*sizeof(F_TYPE)))==NULL)
					{
						// Display error message
						AllocErrorMessage();
						// Bailing out here will leave the phase data array
						// reallocated, so the safest thing to do is
						// to clear it along with the phase timetag array
						ClearDataArray(PHASE_DATA);
						ClearTimetagArray(PHASE_DATA);
								
						// Reset busy flag
						bBusy=FALSE;
						
						// Display normal cursor
						SetCursor(hArrow);

						// Abort
						return 0;
					}
					else
					{
						fPhaseTimetags=fNew;
					}
					fTags=fPhaseTimetags;

					// Set up the 32-bit progress bar
					// Ref: See Win32 Help Info "Using Progress Bars"
					// Set its range as the file size
					// divided by the averaging factor
					SendMessage(GETHCTL(IDC_SQL_PROGRESS), PBM_SETRANGE32, 0,
						nSize/nAF);

					// Set the step size as 1
					SendMessage(GETHCTL(IDC_SQL_PROGRESS), PBM_SETSTEP,
						(WPARAM)1,0);

					// For testing
					// TRACE_INT(nSize/nAF);

					// Read clock data
					// Use point count as while loop limit
					#if(0) // Original code
					while(nPoints<nSize/nAF)
					#endif
					#if(1) // New code 03/21/08
					while(nPoints<=nSize/nAF)
					#endif
					{
						// For testing
						// TRACE_INT(nPoints);

						// Compose timestamp
						MJDtoTimestamp(fMJD, szUTC);
						strlcpy(szBuffer, "'", sizeof(szBuffer));
						strlcat(szBuffer, szUTC, sizeof(szBuffer));
						strlcat(szBuffer, "'", sizeof(szBuffer));

						// For testing
						// TRACE_STR(szBuffer);

						// Query to get phase data from meas of measurement table
						// Params are meas nom freq, ref nom freq, meas source id,
						// ref source id, begin utc and end utc.
						// Compose query
						_snprintf(szQuery, sizeof(szQuery),
							"SELECT a.utc, a.meas/%f - b.meas/%f"
							" FROM measurements a, measurements b"
							" WHERE a.source_id=%d"
							" AND b.source_id=%d AND a.utc=b.utc"
							" AND a.utc=%s",
							fNomMeasFreq, fNomRefFreq, nMeasSourceID,
							nRefSourceID, szBuffer);

						// Execute query
						// Don't use DoSQL() call
						// Need to handle bad response
						// That is expected at end of "stale" database
						res=PQexec(conn, szQuery);

						// Make sure we got OK response
						if(PQntuples(res)==1)
						{
							// Convert timestamp to MJD
							TimestampToMJD(PQgetvalue(res, 0, 0), szBuffer,
								&fVal);

							// Provide better timetag resolution -
							// about 100 nsec
							_snprintf(szBuffer, sizeof(szBuffer),
								"%20.12f", fVal);

							// Store timetag in phase timetag array
							// Data array index is nWrite (0-based)
							// plus ARRAY_OFFSET
							fPhaseTimetags[nWrite+ARRAY_OFFSET]=
								(F_TYPE)atof(szBuffer);

							// Store data point in phase array
							// Data array index is nWrite (0-based)
							// plus ARRAY_OFFSET
							fPhaseData[nWrite+ARRAY_OFFSET]=
								(F_TYPE)atof(PQgetvalue(res, 0, 1));
						}
						else // No data
						{
							if(bStopAtGap) // Stop reading
							{
								// Break out of while loop
								break;
							}
							else // Continue reading
							{
								// Store missing timetag in phase timetag array
								// Data array index is nWrite (0-based)
								// plus ARRAY_OFFSET
								fPhaseTimetags[nWrite+ARRAY_OFFSET]=
									(F_TYPE)fMJD;

								// Store zero data point in phase array
								// Data array index is nWrite (0-based)
								// plus ARRAY_OFFSET
								fPhaseData[nWrite+ARRAY_OFFSET]=
									(F_TYPE)0.0;
							}
						}

						// Keep score of # points written
						nWrite++;

						// Keep score of # points read (same)
						nPoints++;

						// Step progress bars 
						// Note: For "stale" test database,
						// where both runs are "continuing",
						// the current date is too late,
						// making the progress bar too long
						SendMessage(GETHCTL(IDC_SQL_PROGRESS), PBM_STEPIT,
							0,0);

						// Show count as data is read
						// Nice user feedback
						_snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
							nWrite);
						SetWindowText(hwndPhaseNum, szBuffer);

						// Clear result structure
						PQclear(res);
							
						// Increment time as determined by nAF
						// Time increment is MMS retention rate nTau
						// whose value is in integer seconds
						// and 1 sec = 1/86400 MJD = 0.00001157407407+
						// multiplied by nAF
						fMJD+=0.00001157407407407407*nTau*nAF;

						// Process any pending messages
						YieldControl();

						// Check for abort
						if(bAbort)
						{
							bAbort=FALSE;
							break;
						}

						// Set phase data array limits & flags
						fPhaseData[0]=(F_TYPE)nWrite;
						fPhaseData[1]=1.0;
						fPhaseData[2]=(F_TYPE)nWrite;

						// Create filename from meas clock name
						GetWindowText(GETHCTL(IDC_SQL_MEASCLK), szBuffer,
							BUFFER_SIZE);
								
						// Add .phd extension
						strlcat(szBuffer, ".phd", sizeof(szBuffer));

						// Put this filename in various global strings
						strlcpy(szFile, szBuffer, FILE_NAME_SIZE);
						strlcpy(szFileTitle, szBuffer, FILE_TITLE_SIZE);
						strlcpy(szPhaseFile, szBuffer, FILE_NAME_SIZE);
						strlcpy(szPhaseFileTitle, szBuffer, FILE_TITLE_SIZE);

						// Update status display
						SetWindowText(hwndPhaseFilename, szBuffer);
						_snprintf(szBuffer, sizeof(szBuffer), "%5.0d", nWrite);
						SetWindowText(hwndPhaseNum, szBuffer);
						SetWindowText(hwndPhaseEnd, szBuffer);
						_snprintf(szBuffer, sizeof(szBuffer), "%5.0d", 1);
						SetWindowText(hwndPhaseStart, szBuffer);
						_snprintf(szBuffer, sizeof(szBuffer), "%5.0d",
							(int)(fPhaseData[2]-fPhaseData[1]+1));
						SetWindowText(hwndPhaseLim, szBuffer);
						_snprintf(szBuffer, sizeof(szBuffer), "%11.5e",
							(F_TYPE)nTau*nAF);
						ShortenExp(szBuffer);
						SetWindowText(hwndPhaseTau, szBuffer);
						bPhaseData=TRUE;
						bPhaseGapFill=FALSE;
						bPhaseNorm=FALSE;
						bQuadDriftRem=FALSE;
						nPhaseGapsFilled=FALSE;
						bPhaseAddend=FALSE;			
						bPhaseScaled=FALSE;		

						// Set phase timetag array limits & flags
						fPhaseTimetags[0]=(F_TYPE)nWrite;
						fPhaseTimetags[1]=1.0;
						fPhaseTimetags[2]=(F_TYPE)nWrite;

						// Clear phase timetags exist flag
						bPhaseTimetags=TRUE;
					}
					// Read done

					// Clear progress bar 
					SendMessage(GETHCTL(IDC_SQL_PROGRESS), PBM_SETPOS, 0,0);

					// Display message
					if(bAbort)
					{
						_snprintf(szBuffer, sizeof(szBuffer),
							"Read Aborted\n\n# Points Read =\n%d", nWrite);
					}
					else // Read not aborted
					{
						_snprintf(szBuffer, sizeof(szBuffer),
							"Read Complete\n\n# Points Read =\n%d", nWrite);
					}
					SetWindowText(GETHCTL(IDC_SQL_MESSAGE), szBuffer);

					// Activate buttons
					EnableWindow(GETHCTL(IDC_SQL_RESET), TRUE);
					EnableWindow(GETHCTL(IDC_SQL_CANCEL), TRUE);
					bEnable=TRUE;
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_SQL_ENABLE, 0, 0));

					// Change label of Abort button back to Read
					SetWindowText(GETHCTL(IDC_SQL_READ), "Re&ad");

					// Reset reading flag
					bReading=FALSE;

					// Setect phase data
					bDataType=PHASE_DATA;

					// This closes database dialog box at end of read
					// Free tables
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_SQL_FREETABLES, NULL, 0));

					// Close database connection
					PQfinish(conn);

					// Close database dialog box
                    EndDialog (hDlg, 0);
					return TRUE;
				}
				break;

                // Respond to CR in an edit control
                // Note: No button is a DEFPUSHBUTTON
                case IDOK:
                {
                    // Does Start control have focus?
                    if(GetFocus()==GETHCTL(IDC_SQL_START))
                    {
						// Send EN_KILLFOCUS message to factor edit control
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_SQL_START, 0,
							EN_KILLFOCUS));
					}

                    // Does Start MJD edit control have focus?
                    if(GetFocus()==GETHCTL(IDC_SQL_STARTMJD))
                    {
						// Send EN_KILLFOCUS message to factor edit control
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_SQL_STARTMJD, 0,
							EN_KILLFOCUS));
					}

                    // Does End edit control have focus?
                    if(GetFocus()==GETHCTL(IDC_SQL_END))
                    {
						// Send EN_KILLFOCUS message to factor edit control
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_SQL_END, 0,
							EN_KILLFOCUS));
					}

                    // Does End MJD edit control have focus?
                    if(GetFocus()==GETHCTL(IDC_SQL_ENDMJD))
                    {
						// Send EN_KILLFOCUS message to factor edit control
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_SQL_ENDMJD, 0,
							EN_KILLFOCUS));
					}

                    // Does AF edit control have focus?
                    if(GetFocus()==GETHCTL(IDC_SQL_AF))
                    {
						// Send EN_KILLFOCUS message to factor edit control
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_SQL_AF, 0,
							EN_KILLFOCUS));
					}
				}
                break;

				// Respond to Configure button
				case IDC_SQL_CONFIG:
				{
                    // Open dialog box to enter database config info
                    // Config info is saved in INI file MMSDbase.ini

					// Open dialog box
					DialogBox(hInst, "SQLConfigBox", hDlg, SQLConfigDlgProc);
				}
				break;
				
				// Respond to Details button
				case IDC_SQL_DETAILS:
				{
                    // Open dialog box to display ref & meas clock details
					// Note that the Dialog Properties ID text must be inside 
					// double quotes: "DETAILSBOX"
					// The Details pushbutton should be deactivated until the
					// ref and meas clock records have been selected

					// Set values for details display
					// Source tables have source_id values for all clk records
					// Meas tables have meas_id values
					// nRefRec & nMeasRec tell which record is selected
					// They are gobal variables that SQLDetailsDlgProc() uses
					nRefSourceID=atoi(RefSourceTab[nRefRec]);
					nMeasSourceID=atoi(MeasSourceTab[nMeasRec]);
					nRefMeasID=atoi(RefMeasTab[nRefRec]);
					nMeasMeasID=atoi(MeasMeasTab[nMeasRec]);

					// Invoke Details dialog box
					DialogBox(hInst, "SQLDetailsBox", hDlg, SQLDetailsDlgProc);
				}
				break;
				
				// Respond to Reset button
				// Reset useful if user suspests that database may have changed
				// or database connection may have been lost
				// (perhaps because program has been open for a while)
				// It avoids need to reopen program
				case IDC_SQL_RESET:
				{
					// First check that the database connection is still OK
					if(PQstatus(conn) != CONNECTION_OK)
					{
						// Try again to establish connection
						PQreset(conn);

						// Bail out if connection is still bad
						if(PQstatus(conn) != CONNECTION_OK)
						{
							// Show error message 
							_snprintf(szBuffer, sizeof(szBuffer),
								"Connection as user %s to database %s"
								" at host %s failed.\n"
								"Check connection and server.",
								szUser, szDatabase, szHost);
							MessageBox(NULL, szBuffer, "Database Error",
								MB_OK | MB_ICONHAND);
							
							// Return to main program
							// Let user decide what to do
							break;
						}
					}

					// Clear combo boxes
					ComboBox_ResetContent(GETHCTL(IDC_SQL_REFCLK));
					ComboBox_ResetContent(GETHCTL(IDC_SQL_MEASCLK));

					// Clear edit boxes
					SetWindowText(GETHCTL(IDC_SQL_START), "");
					SetWindowText(GETHCTL(IDC_SQL_STARTMJD), "");
					SetWindowText(GETHCTL(IDC_SQL_END), "");
					SetWindowText(GETHCTL(IDC_SQL_ENDMJD), "");
					SetWindowText(GETHCTL(IDC_SQL_LENGTH), "");
					SetWindowText(GETHCTL(IDC_SQL_SIZE), "");
					SetWindowText(GETHCTL(IDC_SQL_MEASINFO), "");

					// Reinitialize message
					SetWindowText(GETHCTL(IDC_SQL_MESSAGE),
						"Select ref and meas clocks,  choose their records,"
						" and press Read to extract data from MMS database.");

					// Reinitialize AF & AF box
					nAF=1;
					nTau=1; // New 07/25/06
					SetWindowText(GETHCTL(IDC_SQL_AF), "1");

					// Set read tau
					_snprintf(szBuffer, sizeof(szBuffer), "%d", nAF*nTau);
					SetWindowText(GETHCTL(IDC_SQL_READTAU), szBuffer);

					// Reinitialize flags
					bMJD=FALSE;
					bDateFormat=TRUE;
					bRefData=FALSE;
					bMeasData=FALSE;
					bDetailsOK=FALSE;
					bReadOK=FALSE;
					bErrorTrap=FALSE;
					bStableOK=FALSE;

					// Initialize the time/date format radiobox
					Button_SetCheck(GETHCTL(IDC_SQL_UTC), !bMJD);
					Button_SetCheck(GETHCTL(IDC_SQL_MJD), bMJD);
			
					// Deactivate Details button
					// (bDetailsOK initialized to FALSE)
					EnableWindow(GETHCTL(IDC_SQL_DETAILS), bDetailsOK);

					// Deactivate Read button (bReadOK initialized to FALSE)
					EnableWindow(GETHCTL(IDC_SQL_READ), bReadOK);

					// Clear all tables
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_SQL_CLEARTABLES, 0, 0));

					// No need to reinitialize those items read from ini file

					// Fetch clock names & ids from clock_names database table
					if((res=DoSQL(conn, "SELECT clock_name, clock_id"
						" FROM clock_names ORDER BY clock_name"))==0)
					{
						return 0; // Close program
					}
					
					// Put clock names into reference & measurement combo boxes
					// Put clock ids into local ClockIDTab[] table

					// Save # clocks
					nClkRows = PQntuples(res);

					// Check for no clocks
					if(nClkRows)
					{
						// Enlarge clock id array as necessary
						if(nClkRows>nClockTabRows)
						{
							// Reallocate clock id table
							// # cols remains at original 10 -
							// assumed large enough for any clock_id string
							ClockIDTab=ReallocCharArray(ClockIDTab,
								nClockTabRows, nClkRows, nClockTabCols);

							// Save new measurement table size
							nClockTabRows=nClkRows;
						}

						// Put clock names into reference clock combo box
						// The reversed indexing puts top name at top
						for(i=nClkRows-1; i>=0; i--)
						{
							// Load names into combo box
							nDummy=
								ComboBox_InsertString(GETHCTL(IDC_SQL_REFCLK),
								0, PQgetvalue(res, i, 0));
						}

						// Put clock names into measurement clock combo box
						// The reversed indexing puts top name at top
						for(i=nClkRows-1; i>=0; i--)
						{
							// Load names into combo box
							nDummy=
								ComboBox_InsertString(GETHCTL(IDC_SQL_MEASCLK),
								0, PQgetvalue(res, i, 0));
						}

						// Put clock ides into ClockIDTab[] array
						// The reversed indexing puts top name at top
						for(i=nClkRows-1; i>=0; i--)
						{
							// Load ids into local table
							strlcpy(ClockIDTab[i], PQgetvalue(res, i, 1),
								nClockTabCols);
						}

						// Clear result
						PQclear(res);

						// Select 1st reference clock 
						nDummy=ComboBox_SetCurSel(GETHCTL(IDC_SQL_REFCLK), 0);

						// Select 1st measurement clock 
						nDummy=ComboBox_SetCurSel(GETHCTL(IDC_SQL_MEASCLK), 0);
					}
					else // No clocks
					{
						// Load and display no clocks message
						// in clock combo boxes
						nDummy=ComboBox_InsertString(GETHCTL(IDC_SQL_REFCLK),
							0, "No clocks");
						nDummy=ComboBox_SetCurSel(GETHCTL(IDC_SQL_REFCLK),
							0);
						nDummy=ComboBox_InsertString(GETHCTL(IDC_SQL_MEASCLK),
							0,
							"No clocks");
						nDummy=ComboBox_SetCurSel(GETHCTL(IDC_SQL_MEASCLK),
							0);
					}

					// Our choice here is either to wait for user to select
					// a ref & meas clock again, or to select the previous
					// ones automatically and reload their records into the
					// display tables
					// For now, let user reselect clocks

					// Reset state
					nMeasClkID=0;
					nRefClkID=0;
					nMeasSourceID=0;
					nRefSourceID=0;
					nMeasMeasID=0;
					nRefMeasID=0;
					nMeasID=0;
					nRefRows=0;
					nMeasRows=0;
					nRefClk=-1;
					nMeasClk=-1;
					//nSys=0;

					// Clear record display tables
					SendMessage(hRefTab, DTM_RESETDATA, 0, 0);
					SendMessage(hMeasTab, DTM_RESETDATA, 0, 0);

					// Reinitialize
					SendMessage(hDlg, WM_INITDIALOG, (WPARAM)0, (LPARAM)0);
				}
				break;

				// Respond to Help button
				case IDC_SQL_HELP:
				{
					#if(0)
					// Invoke help file
					HtmlHelp(hDlg, "Stable32.chm::DATABASE_FUNCTION.HTM",
						HH_DISPLAY_TOPIC, 0L);
					#endif

					#if(0) // New code 11/29/11
					// Invoke help file
					strlcpy(szBuffer, szHelpFile, sizeof(szBuffer));
					strlcat(szBuffer, "::DATABASE_FUNCTION.HTM", sizeof(szBuffer));
					HtmlHelp(hDlg, szBuffer,
						HH_DISPLAY_TOPIC, 0L);
					#endif

					#if(1) // Trial code 11/30/11
					// Invoke help file using CallHelp() function
					CallHelp(hDlg, "DATABASE_FUNCTION");
					#endif
				}
				break;

				// Respond to PostgreSQL Help button
				case IDC_SQL_SQLHELP:
				{
					// Invoke help file
					HtmlHelp(hDlg, "postgresql.chm",
						HH_DISPLAY_TOPIC, 0L);
				}
				break;

				// Respond to Cancel button - close database dialog box
				case IDCANCEL:
				case IDC_SQL_CANCEL:
				{
					// Free tables
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_SQL_FREETABLES, NULL, 0));

					// Close database connection
					PQfinish(conn);

					// Close database dialog box
                    EndDialog (hDlg, 0);
					return TRUE;
				}
				return TRUE;
			}
		}
		break;
	}
	return FALSE;
}

/*****************************************************************************/
/*																			 */
/*					 SQL Details Dialog Procedure							 */
/*																			 */
/*																			 */
/*		Global variables used: conn, nMeasClkID, nRefClkID, nRefRec,         */
/*		nMeasRec, nRefSourceID, nMeasSourceID, nRefMeasID, nMeasMeasID		 */
/*																			 */
/*		Revision record:													 */
/*			12/03/05	  Created from DetailsDlgProc() of MMSDbase			 */
/*																			 */
/*****************************************************************************/

BOOL CALLBACK SQLDetailsDlgProc(HWND hDlg, UINT message, WPARAM wParam,
	LPARAM lParam)
{
	// Local variables
	char szBuffer[BUFFER_SIZE+1];	// Text buffer
	char szQuery[BUFFER_SIZE+1];	// Query text buffer
	int nRefSystemID;				// Ref MMS system ID #
	int nMeasSystemID;				// Meas MMS system ID #
	PGresult   *res;				// Query result pointer
	BOOL bRefRecords=TRUE;			// Flag for ref records
	BOOL bMeasRecords=TRUE;			// Flag for meas records

	switch(message)
	{
		case WM_INITDIALOG:
		{
			// Database connection exists using conn
			// Check for reference records
			// Compose search string
			_snprintf(szQuery, sizeof(szQuery),
				"SELECT meas_id FROM measurement_list WHERE clock_id=%d",
				nRefClkID);

			// Do query				
			if((res=DoSQL(conn, szQuery))==0)
			{
				return 0; // Close program
			}

			// Check for NULL (No records)
			if(PQgetisnull(res, 0, 0))
			{
				bRefRecords=FALSE;
			}
			else // Have  ref record(s)
			{
				bRefRecords=TRUE;
			}

			// Check for measurement records
			// Compose search string
			_snprintf(szQuery, sizeof(szQuery),
				"SELECT meas_id FROM measurement_list WHERE clock_id=%d",
				nMeasClkID);

			// Do query				
			if((res=DoSQL(conn, szQuery))==0)
			{
				return 0; // Close program
			}

			// Check for NULL (No records)
			if(PQgetisnull(res, 0, 0))
			{
				bMeasRecords=FALSE;
			}
			else // Have meas records(s)
			{
				bMeasRecords=TRUE;
			}

			// Load clock info already known into text boxes
			_snprintf(szBuffer, sizeof(szBuffer), "%d", nRefClkID);
			SetWindowText(GETHCTL(IDC_SQL_REFCLOCKID), szBuffer);
			_snprintf(szBuffer, sizeof(szBuffer), "%d", nMeasClkID);
			SetWindowText(GETHCTL(IDC_SQL_MEASCLOCKID), szBuffer);

			// Get ref clock name from clock_names table vs ref clock_id
			_snprintf(szQuery, sizeof(szQuery),
				"SELECT clock_name FROM clock_names"
				" WHERE clock_id=%d", nRefClkID);

			// Do query					
			if((res=DoSQL(conn, szQuery))==0)
			{
				return 0; // Close program
			}

			// Make sure we got OK response
			if(PQntuples(res)!=1)
			{
				_snprintf(szBuffer, sizeof(szBuffer),
					"Error in getting reference clock name\n"
					"Unexpected # of rows (%d) returned from query",
					PQntuples(res));
				TRACE_STR(szBuffer);
				break;
			}

			// Show ref clock name
			SetWindowText(GETHCTL(IDC_SQL_REFCLOCKNAME), PQgetvalue(res, 0,
				0));

			// Clear result
			PQclear(res);

			// Get meas clock name from clock_names table vs meas clock_id
			_snprintf(szQuery, sizeof(szQuery),
				"SELECT clock_name FROM clock_names"
				" WHERE clock_id=%d", nMeasClkID);

			// Do query					
			if((res=DoSQL(conn, szQuery))==0)
			{
				return 0; // Close program
			}

			// Make sure we got OK response
			if(PQntuples(res)!=1)
			{
				_snprintf(szBuffer, sizeof(szBuffer),
					"Error in getting measurement clock name\n"
					"Unexpected # of rows (%d) returned from query",
					PQntuples(res));
				TRACE_STR(szBuffer);
				break;
			}

			// Show meas clock name
			SetWindowText(GETHCTL(IDC_SQL_MEASCLOCKNAME), PQgetvalue(res, 0,
				0));

			// Clear result
			PQclear(res);

			if(bRefRecords)
			{
				_snprintf(szBuffer, sizeof(szBuffer), "%d", nRefSourceID);
				SetWindowText(GETHCTL(IDC_SQL_REFSOURCEID), szBuffer);

				// Get ref system id & channel from measurement_sources table
				// with ref source ID as key
				_snprintf(szQuery, sizeof(szQuery),
					"SELECT system_id, channel FROM measurement_sources"
					" WHERE source_id=%d", nRefSourceID);

				// Do query					
				if((res=DoSQL(conn, szQuery))==0)
				{
					return 0; // Close program
				}

				// Make sure we got OK response
				if(PQntuples(res)!=1)
				{
					_snprintf(szBuffer, sizeof(szBuffer),
						"Error in getting reference system id & channel #\n"
						"Unexpected # of rows (%d) returned from query",
						PQntuples(res));
					TRACE_STR(szBuffer);
					break;
				}

				// Save ref system id
				nRefSystemID=atoi(PQgetvalue(res, 0, 0));

				// Display ref system ID
				SetWindowText(GETHCTL(IDC_SQL_REFSYSTEMID), PQgetvalue(res, 0,
					0));

				// Display ref channel #
				SetWindowText(GETHCTL(IDC_SQL_REFCHANNEL), PQgetvalue(res, 0,
					1));

				// Clear result
				PQclear(res);

				// Get meas_id, begun_by, ended_by & frequency from
				// measurement_list for reference clock with meas_id as index
				// Compose search string
				_snprintf(szQuery, sizeof(szQuery),
					"SELECT meas_id, begun_by, ended_by, frequency"
					" FROM measurement_list WHERE meas_id=%d", nRefMeasID);

				// Do query				
				if((res=DoSQL(conn, szQuery))==0)
				{
					return 0; // Close program
				}

				// Make sure we got OK response
				if(PQntuples(res)!=1)
				{
					_snprintf(szBuffer, sizeof(szBuffer),
						"Error in getting reference meas id, etc.\n"
						"Unexpected # of rows (%d) returned from query",
						PQntuples(res));
					TRACE_STR(szBuffer);
					SetWindowText(GETHCTL(IDC_SQL_REFMEASID), "");
					SetWindowText(GETHCTL(IDC_SQL_REFBEGUNBY), "");
					SetWindowText(GETHCTL(IDC_SQL_REFENDEDBY), "");
					SetWindowText(GETHCTL(IDC_SQL_REFFREQ), "");
					break;
				}

				// Display ref meas_id
				SetWindowText(GETHCTL(IDC_SQL_REFMEASID), PQgetvalue(res, 0,
					0));

				// Display begun_by
				SetWindowText(GETHCTL(IDC_SQL_REFBEGUNBY), PQgetvalue(res, 0,
					1));

				// Display ended_by
				SetWindowText(GETHCTL(IDC_SQL_REFENDEDBY), PQgetvalue(res, 0,
					2));

				// Display frequency
				SetWindowText(GETHCTL(IDC_SQL_REFFREQ), PQgetvalue(res, 0,
					3));

				// Clear result
				PQclear(res);

				// Get ref MMS system name from measurement_sources table
				// with system_id as key
				// Compose query
				_snprintf(szQuery, sizeof(szQuery),
					"SELECT system_name FROM measurement_systems"
					" WHERE system_id=%d", nRefSystemID);

				// TRACE_INT(nRefSystemID);

				// Do query					
				if((res=DoSQL(conn, szQuery))==0)
				{
					return 0; // Close program
				}

				// Make sure we got OK response
				if(PQntuples(res)!=1)
				{
					_snprintf(szBuffer, sizeof(szBuffer),
						"Error in getting reference reference MMS name\n"
						"Unexpected # of rows (%d) returned from query",
						PQntuples(res));
					TRACE_STR(szBuffer);
					SetWindowText(GETHCTL(IDC_SQL_REFSYSNAME), "");
					break;
				}

				// Display ref system name
				SetWindowText(GETHCTL(IDC_SQL_REFSYSNAME), PQgetvalue(res, 0,
					0));

				// TRACE_STR(szBuffer);

				// Clear result
				PQclear(res);
			}
			else // No ref record
			{
				SetWindowText(GETHCTL(IDC_SQL_REFSOURCEID), "");
				SetWindowText(GETHCTL(IDC_SQL_REFSYSTEMID), "");
				SetWindowText(GETHCTL(IDC_SQL_REFCHANNEL), "");
				SetWindowText(GETHCTL(IDC_SQL_REFMEASID), "");
				SetWindowText(GETHCTL(IDC_SQL_REFBEGUNBY), "");
				SetWindowText(GETHCTL(IDC_SQL_REFENDEDBY), "");
				SetWindowText(GETHCTL(IDC_SQL_REFFREQ), "");
				SetWindowText(GETHCTL(IDC_SQL_REFSYSNAME), "");
			}

			if(bMeasRecords)
			{
				_snprintf(szBuffer, sizeof(szBuffer), "%d", nMeasSourceID);
				SetWindowText(GETHCTL(IDC_SQL_MEASSOURCEID), szBuffer);

				// Get meas system id & channel from measurement_sources table
				// with meas source ID as key
				_snprintf(szQuery, sizeof(szQuery),
					"SELECT system_id, channel FROM measurement_sources"
					" WHERE source_id=%d", nMeasSourceID);

				// Do query					
				if((res=DoSQL(conn, szQuery))==0)
				{
					return 0; // Close program
				}

				// Make sure we got OK response
				if(PQntuples(res)!=1)
				{
					_snprintf(szBuffer, sizeof(szBuffer),
						"Error in getting measurement system id & channel #\n"
						"Unexpected # of rows (%d) returned from query",
						PQntuples(res));
					TRACE_STR(szBuffer);
					break;
				}

				// Save meas system id
				nMeasSystemID=atoi(PQgetvalue(res, 0, 0));

				// Display meas system ID
				SetWindowText(GETHCTL(IDC_SQL_MEASSYSTEMID), PQgetvalue(res, 0,
					0));

				// Display meas channel #
				SetWindowText(GETHCTL(IDC_SQL_MEASCHANNEL), PQgetvalue(res, 0,
					1));

				// Clear result
				PQclear(res);

				// Get meas_id, begun_by, ended_by & frequency
				// from measurement_list
				// for measurement clock with meas_id as index
				// Compose search string
				_snprintf(szQuery, sizeof(szQuery),
					"SELECT meas_id, begun_by, ended_by, frequency"
					" FROM measurement_list WHERE meas_id=%d", nMeasMeasID);

				// Do query				
				if((res=DoSQL(conn, szQuery))==0)
				{
					return 0; // Close program
				}

				// Make sure we got OK response
				if(PQntuples(res)!=1)
				{
					_snprintf(szBuffer, sizeof(szBuffer),
						"Error in getting measurement reference meas id, etc."
						"\nUnexpected # of rows (%d) returned from query",
						PQntuples(res));
					TRACE_STR(szBuffer);
					SetWindowText(GETHCTL(IDC_SQL_MEASMEASID), "");
					SetWindowText(GETHCTL(IDC_SQL_MEASBEGUNBY), "");
					SetWindowText(GETHCTL(IDC_SQL_MEASENDEDBY), "");
					SetWindowText(GETHCTL(IDC_SQL_MEASFREQ), "");
					break;
				}

				// Display ref meas_id
				SetWindowText(GETHCTL(IDC_SQL_MEASMEASID), PQgetvalue(res, 0,
					0));

				// Display begun_by
				SetWindowText(GETHCTL(IDC_SQL_MEASBEGUNBY), PQgetvalue(res, 0,
					1));

				// Display ended_by
				SetWindowText(GETHCTL(IDC_SQL_MEASENDEDBY), PQgetvalue(res, 0,
					2));

				// Display frequency
				SetWindowText(GETHCTL(IDC_SQL_MEASFREQ), PQgetvalue(res, 0,
					3));

				// Clear result
				PQclear(res);

				// Get meas MMS system name from measurement_sources table
				// with system_id as key
				// Compose query
				_snprintf(szQuery, sizeof(szQuery),
					"SELECT system_name FROM measurement_systems"
					" WHERE system_id=%d", nMeasSystemID);

				// Do query					
				if((res=DoSQL(conn, szQuery))==0)
				{
					return 0; // Close program
				}

				// Make sure we got OK response
				if(PQntuples(res)!=1)
				{
					_snprintf(szBuffer, sizeof(szBuffer),
						"Error in getting measurement MMS name\n"
						"Unexpected # of rows (%d) returned from query",
						PQntuples(res));
					TRACE_STR(szBuffer);
					SetWindowText(GETHCTL(IDC_SQL_REFSYSNAME), "");
					break;
				}

				// Display meas system name
				SetWindowText(GETHCTL(IDC_SQL_MEASSYSNAME),
					PQgetvalue(res, 0, 0));

				// Clear result
				PQclear(res);
			}
			else // No meas record
			{
				SetWindowText(GETHCTL(IDC_SQL_MEASSOURCEID), "");
				SetWindowText(GETHCTL(IDC_SQL_MEASSYSTEMID), "");
				SetWindowText(GETHCTL(IDC_SQL_MEASCHANNEL), "");
				SetWindowText(GETHCTL(IDC_SQL_MEASMEASID), "");
				SetWindowText(GETHCTL(IDC_SQL_MEASBEGUNBY), "");
				SetWindowText(GETHCTL(IDC_SQL_MEASENDEDBY), "");
				SetWindowText(GETHCTL(IDC_SQL_MEASFREQ), "");
				SetWindowText(GETHCTL(IDC_SQL_REFSYSNAME), "");
			}

			// Done
			return TRUE;
		}

		case WM_COMMAND:
		{
			switch(GET_WM_COMMAND_ID(wParam, lParam))
			{
				case IDOK:
				case IDCANCEL:
				{
					EndDialog (hDlg, 0);
				}
				return TRUE;
			}
		}
		break;
	 }
	 return FALSE;
}

/*****************************************************************************/
/*																			 */
/*		Function to put MMS, Ch #, start & end dates into row of data table	 */
/*																			 */
/*		There is a VMSTRUCT inside the DataTable control that contains the	 */
/*		elements nStatus, dwKey and lpData.	 A pointer to it is passed to	 */
/*		us via the lParam of each DTM message.	We assign this to a			 */
/*		LPVMSTRUCT pointer called lpVMData.	 We then use dwKey to access our */
/*		query data for display in the table.								 */
/*																			 */
/*		Parameters:															 */
/*			LPVMSTRUCT	*lpData			Pointer to row data structure		 */
/*			char		*StartTab		Start date/time or MJD table    	 */
/*			char		*EndTab			End date/time or MJD table           */
/*			BOOL		bMJD			Flag to show MJD, not timestamp 	 */
/*			int			nSys			MMS system # or code				 */
/*										0 = code to ignore system #          */
/*										>0 = MMS system # to compare with    */
/*			BOOL		bRef			Flag for which table 0=Meas, 1=Ref	 */
/*																			 */
/*		Revision record:													 */
/*			10/07/05	Adapted from GetOutlierData()						 */
/*			10/18/05	Added provisions for MMS & Ch # columns              */
/*						Added test for right MMS system                      */
/*																			 */
/*****************************************************************************/

int WINAPI GetSQLTableData(LPVMSTRUCT lpData, char ** MMSTab, char ** ChTab,
	char ** StartTab, char ** EndTab, BOOL bMJD, int nSys, BOOL bRef)
{
	// Store start & end times in ref or meas display table
	// They come from their respective string tables, indexed by row #
	// with whitespace between them as required
	// nSys is system_id of MMS system for selected left (ref) clock record
	// It is 0 before a ref record is selected (all meas records shown then)
	// Activate if statement to only write rows that have right (meas)
	// system_id = nSys (unless nSys=0).
	// nSys is set to 0 for all reference table rows, so all ref records are
	// always shown
	// I prefer to also show all meas rows and flag different MMS with error
	// message in read times boxes, especially if the wrong MMS meas rows
	// are shown in a different color
	// But set bHideNGMMS if that is wanted
	// Most users will have only one MMS system anyway

	// Ref and meas data tables each have 4 columns
	// Columns have 3+3+19+19=44 characters (3 chars hard-coded) + EOS
	// Local macro for # chars in table row
	#define NUM_SQL_TAB_CHARS 45

	if( (!bHideNGMMS) || (atoi(MMSTab[lpData->dwKey])==nSys) || (nSys==0) )
	{
		if(bNoRefRecords && bRef)
		{
			strlcpy(lpData->lpData, "      No Ref Records", NUM_SQL_TAB_CHARS);
		    return DTE_OKAY;
		}
		
		if(bNoMeasRecords  & !bRef)
		{
			strlcpy(lpData->lpData, "      No Meas Records",
				NUM_SQL_TAB_CHARS);
		    return DTE_OKAY;
		}
		
		// Show MMS
		strlcpy(lpData->lpData, MMSTab[lpData->dwKey], NUM_SQL_TAB_CHARS);

		// Show Channel #
		strlcat(lpData->lpData, ChTab[lpData->dwKey], NUM_SQL_TAB_CHARS);

		// Show start date
		strlcat(lpData->lpData, StartTab[lpData->dwKey], NUM_SQL_TAB_CHARS);

		// Add whitespace to separate columns for MJD format
		if(bMJD)
		{
			strlcat(lpData->lpData, "       ", NUM_SQL_TAB_CHARS); // 7 spaces
		}

		// Show end date
		if(atof(EndTab[lpData->dwKey])==0) // O MJD
		{
			strlcat(lpData->lpData, "Run Continuing", NUM_SQL_TAB_CHARS);
		}
		else
		{
			strlcat(lpData->lpData, EndTab[lpData->dwKey], NUM_SQL_TAB_CHARS);
		}
	}

    // Return success
    return DTE_OKAY;
}

/*****************************************************************************/
/*                                                                           */
/*                     Configuration Dialog Procedure                        */
/*                                                                           */
/*	This function reads, edits & stores default database configuration       */
/*  parameters: Host IP Address, Database Name, User Name & Password,        */
/*	data file date format & filename, autofilename state, Stable32 path      */
/*	and cmd.                                                                 */
/*                                                                           */
/*  Revision record:                                                         */
/*      12/03/05    Created from DetailDlgProc() of MMSDBase                 */
/*		12/04/05	Removed data file and Stable controls and code           */
/*		02/17/08	Added single quotes for user name with space             */
/*                                                                           */
/*****************************************************************************/

BOOL CALLBACK SQLConfigDlgProc(HWND hDlg, UINT message, WPARAM wParam,
	LPARAM lParam)
{
	// Local subroutines
	#define IDC_SQLCONFIG_READINI 7200
	#define IDC_SQLCONFIG_WRITEINI 7201

	// Local variables
	// Ints
	int i;										// Index
	int j;										// Secondary index
	int n;										// Size
	// Chars
	char szBuffer[BUFFER_SIZE+1];				// General text buffer
	char szEncryptedPassword[BUFFER_SIZE+1];	// Encrypted password
	static char szHost[BUFFER_SIZE+1];			// Host IP address
	static char szDatabase[BUFFER_SIZE+1];		// Database name
	static char szUser[BUFFER_SIZE+1];			// Username
	static char szPassword[BUFFER_SIZE+1];		// Password
	// BOOLs
	static BOOL bErrorTrap=FALSE;				// Flag for entry error
	static BOOL bStopAtGap=TRUE;				// Flag to stop read at gap
	// TEA variables
	static int k[]={1, 10, 100, 1000};		// 128-bit encryption key
	unsigned int v[2];						// 64-bit encryption variable

	switch(message)
	{
		case WM_INITDIALOG:
		{
			// Read current config values from INI file
			SendMessage(hDlg, WM_COMMAND,
				GET_WM_COMMAND_MPS(IDC_SQLCONFIG_READINI, 0, 0));

			#if(1) // New code 02/17/08
			// Enclose user name in single quotes if it contains a space
			if(strchr(szUser, ' '))
			{
				// Don't add single quotes if they are already there
				if(szUser[0] != '\'')
				{
					strncpy(szBuffer, "'", BUFFER_SIZE);
					strncat(szBuffer, szUser, BUFFER_SIZE);
					strncat(szBuffer, "'", BUFFER_SIZE);
					strncpy(szUser, szBuffer, BUFFER_SIZE);
				}
			}
			#endif

			// Don't have to save original values
			// They will remain in ini file for cancel key

			// Show config info in edit boxes
			SetWindowText(GETHCTL(IDC_SQLCONFIG_HOST), szHost);
			SetWindowText(GETHCTL(IDC_SQLCONFIG_DATABASE), szDatabase);
			SetWindowText(GETHCTL(IDC_SQLCONFIG_USER), szUser);
			SetWindowText(GETHCTL(IDC_SQLCONFIG_PASSWORD), szPassword);

			// Get stop at gap flag setting from INI file = MMSDbase.ini
			bStopAtGap=(BOOL)GetPrivateProfileInt("Database", "StopAtGap", 1, szIniFile);

			// Display the stop at gap checkbox state
			Button_SetCheck(GETHCTL(IDC_SQLCONFIG_STOP), bStopAtGap);
		}
		return TRUE;

		case WM_COMMAND:
		{
			switch(GET_WM_COMMAND_ID(wParam, lParam))
			{
				// Respond to help button message
				case IDC_SQLCONFIG_HELP:
				{
					#if(0)
					// Invoke help file
					HtmlHelp(hDlg, "Stable32.chm::DATABASE_CONFIGURE.HTM",
						HH_DISPLAY_TOPIC, 0L);
					#endif

					#if(0) // New code 11/29/11
					// Invoke help file
					strlcpy(szBuffer, szHelpFile, sizeof(szBuffer));
					strlcat(szBuffer, "::DATABASE_CONFIGURE.HTM", sizeof(szBuffer));
					HtmlHelp(hDlg, szBuffer,
						HH_DISPLAY_TOPIC, 0L);
					#endif

					#if(1) // Trial code 11/30/11
					// Invoke help file using CallHelp() function
					CallHelp(hDlg, "DATABASE_CONFIGURE");
					#endif
				}
				break;

				// Respond to host IP address edit control message
				case IDC_SQLCONFIG_HOST:
				{
					// Respond to edit control when it loses the input focus
					// unless it loses it to the cancel button
					if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
						(!bErrorTrap) &&
						(GetFocus()!=GETHCTL(IDC_SQLCONFIG_CANCEL)))
					{
						// Set error flag to avoid multiple error messages
						bErrorTrap=TRUE;

						// Get (new) host IP address
						GetWindowText(GETHCTL(IDC_SQLCONFIG_HOST), szHost,
							BUFFER_SIZE);

						// Rewrite it to privide user feedback
						SetWindowText(GETHCTL(IDC_SQLCONFIG_HOST), szHost);
                    }
                    else
                    {
                        // Reset error flag
                        bErrorTrap=FALSE;
                    }
                }
                break;

				// Respond to database name edit control message
				case IDC_SQLCONFIG_DATABASE:
				{
					// Respond to edit control when it loses the input focus
					// unless it loses it to the cancel button
					if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
						(!bErrorTrap) &&
						(GetFocus()!=GETHCTL(IDC_SQLCONFIG_CANCEL)))
					{
						// Set error flag to avoid multiple error messages
						bErrorTrap=TRUE;

						// Get (new) database name
						GetWindowText(GETHCTL(IDC_SQLCONFIG_DATABASE),
							szDatabase, BUFFER_SIZE);

						// Rewrite it to provide user feedback
						SetWindowText(GETHCTL(IDC_SQLCONFIG_DATABASE),
							szDatabase);
                    }
                    else
                    {
                        // Reset error flag
                        bErrorTrap=FALSE;
                    }
                }
                break;

				// Respond to username edit control message
				case IDC_SQLCONFIG_USER:
				{
					// Respond to edit control when it loses the input focus
					// unless it loses it to the cancel button
					if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
						(!bErrorTrap) &&
						(GetFocus()!=GETHCTL(IDC_SQLCONFIG_CANCEL)))
					{
						// Set error flag to avoid multiple error messages
						bErrorTrap=TRUE;

						// Get (new) user name
						GetWindowText(GETHCTL(IDC_SQLCONFIG_USER), szUser,
							BUFFER_SIZE);

						#if(1) // New code 02/17/08
						// Enclose user name in single quotes if it contains a space
						if(strchr(szUser, ' '))
						{
							// Don't add single quotes if they are already there
							if(szUser[0] != '\'')
							{
								strncpy(szBuffer, "'", BUFFER_SIZE);
								strncat(szBuffer, szUser, BUFFER_SIZE);
								strncat(szBuffer, "'", BUFFER_SIZE);
								strncpy(szUser, szBuffer, BUFFER_SIZE);
							}
						}
						#endif

						// Rewrite it to provide user feedback
						SetWindowText(GETHCTL(IDC_SQLCONFIG_USER), szUser);
                    }
                    else
                    {
                        // Reset error flag
                        bErrorTrap=FALSE;
                    }
                }
                break;

				// Respond to password edit control message
				case IDC_SQLCONFIG_PASSWORD:
				{
					// Respond to edit control when it loses the input focus
					// unless it loses it to the cancel button
					if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
						(!bErrorTrap) &&
						(GetFocus()!=GETHCTL(IDC_SQLCONFIG_CANCEL)))
					{
						// Set error flag to avoid multiple error messages
						bErrorTrap=TRUE;

						// Get (new) password
						GetWindowText(GETHCTL(IDC_SQLCONFIG_PASSWORD),
							szPassword, BUFFER_SIZE);

						// Rewrite it to provide user feedback
						SetWindowText(GETHCTL(IDC_SQLCONFIG_PASSWORD),
							szPassword);
                    }
                    else
                    {
                        // Reset error flag
                        bErrorTrap=FALSE;
                    }
                }
                break;

				// Respond to stop at gap checkbox message
				case IDC_SQLCONFIG_STOP:
				{
					// Get the state of the stop at gap checkbox
					bStopAtGap=
						(BOOL)Button_GetCheck(GETHCTL(IDC_SQLCONFIG_STOP));
                }
                break;

                // Respond to CR in an edit control
                // Note: No button is a DEFPUSHBUTTON
                case IDOK:
                {
                    // Does host edit control have focus?
                    if(GetFocus()==GETHCTL(IDC_SQLCONFIG_HOST))
                    {
						// Send EN_KILLFOCUS message to factor edit control
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_SQLCONFIG_HOST, 0,
							EN_KILLFOCUS));
					}

                    // Does database edit control have focus?
                    if(GetFocus()==GETHCTL(IDC_SQLCONFIG_DATABASE))
                    {
						// Send EN_KILLFOCUS message to factor edit control
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_SQLCONFIG_DATABASE, 0,
							EN_KILLFOCUS));
					}

                    // Does username edit control have focus?
                    if(GetFocus()==GETHCTL(IDC_SQLCONFIG_USER))
                    {
						// Send EN_KILLFOCUS message to factor edit control
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_SQLCONFIG_USER, 0,
							EN_KILLFOCUS));
					}

                    // Does password edit control have focus?
                    if(GetFocus()==GETHCTL(IDC_SQLCONFIG_PASSWORD))
                    {
						// Send EN_KILLFOCUS message to factor edit control
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_SQLCONFIG_PASSWORD, 0,
							EN_KILLFOCUS));
					}
				}
                break;

				// Subfunction to read config data from INI file
				case IDC_SQLCONFIG_READINI:
				{
					// Get database connection parameters from INI file
					GetPrivateProfileString("Database", "Host", "192.168.1.1",
						szHost, BUFFER_SIZE, szIniFile);
					GetPrivateProfileString("Database", "Database",
						"tsc-postgres",	szDatabase,	BUFFER_SIZE, szIniFile);
					GetPrivateProfileString("Database", "User", "tsc", szUser,
						BUFFER_SIZE, szIniFile);
					GetPrivateProfileString("Database", "Password", "tsc",
						szEncryptedPassword, BUFFER_SIZE, szIniFile);

					// Get length of encrypted password
					n=strlen(szEncryptedPassword);

					// Parse encrypted password
					for(j=0; j<n/16; j++)
					{
						// Scan a word of the the encrypted password
						sscanf(&szEncryptedPassword[16*j], "%08X%08X", &v[0],
							&v[1]);

						// Decrypt
						Decode(v, k);

						// Conv v[0] back to substring
						for(i=0; i<4; i++)
						{
							szBuffer[i+8*j]=v[0]/(1<<(8*(4-i)));
							v[0]-=(1<<(8*(4-i)));
						}

						// Conv v[1] back to substring
						for(i=0; i<4; i++)
						{
							szBuffer[i+4+8*j]=v[1]/(1<<(8*(4-i)));
							v[1]-=(1<<(8*(4-i)));
						}
					}

					// Add EOS at end
					szBuffer[n/2]='\0';

					// Save recovered password
					strlcpy(szPassword, szBuffer, sizeof(szPassword));
				}
				break;

				// Subfunction to write config data to INI file
				case IDC_SQLCONFIG_WRITEINI:
				{
					// Get parameters
					GetWindowText(GETHCTL(IDC_SQLCONFIG_HOST), szHost,
						BUFFER_SIZE);
					GetWindowText(GETHCTL(IDC_SQLCONFIG_DATABASE), szDatabase,
						BUFFER_SIZE);
					GetWindowText(GETHCTL(IDC_SQLCONFIG_USER), szUser,
						BUFFER_SIZE);
					GetWindowText(GETHCTL(IDC_SQLCONFIG_PASSWORD), szPassword,
						BUFFER_SIZE);

					// TEA password encryption code
					// Get password length
					i=strlen(szPassword);

					// Length must be a multiple of 8 characters (bytes)
					n=((int)ceil((double)i/8.0))*8;

					// Pad string with EOS until length a multiple of 8
					// szBuffer[n] is original EOS
					// Want to add more until last one is past multiple of 8
					while(i<=n)
					{
						i++;
						szPassword[i]='\0';
					}

					// Clear the encrypted password
					strlcpy(szEncryptedPassword, "",
						sizeof(szEncryptedPassword));

					// Parse the padded password string
					for(j=0; j<n/8; j++)
					{
						// Calc v[0]
						v[0]=0;
						for(i=0; i<4; i++)
						{
							v[0]+=(szPassword[i+8*j]<<(8*(4-i)));
						}

						// Calc v[1]
						v[1]=0;
						for(i=0; i<4; i++)
						{
							v[1]+=(szPassword[i+4+8*j]<<(8*(4-i)));
						}
					
						// Encrypt
						Encode(v, k);

						// Save v[0] and v[1] as hex characters
						// Can't save them as character strings
						// because some won't be valid characters
						// Saved in ini file as:
						// Password=string where string is groups of v[0]v[1]
						// each expressed as 8 hex digits: XXXXXXXXXXXXXXXX
						// w/o any delimiting
						// (double length of original password)

						// Compose encrypted password block
						_snprintf(szBuffer, sizeof(szBuffer), "%08X%08X", v[0],
							v[1]);

						// Add it to complete encrypted password
						strlcat(szEncryptedPassword, szBuffer,
							sizeof(szEncryptedPassword));
					}

					// Save parameters in ini file
					WritePrivateProfileString("Database", "Host", szHost,
						szIniFile);
					WritePrivateProfileString("Database", "Database",
						szDatabase,
						szIniFile);
					WritePrivateProfileString("Database", "User", szUser,
						szIniFile);
					WritePrivateProfileString("Database", "Password",
						szEncryptedPassword, szIniFile);

					// Save new stop at gap flag setting in ini file
					_snprintf(szBuffer, sizeof(szBuffer), "%d",
						bStopAtGap);
					WritePrivateProfileString("Database", "StopAtGap",
						szBuffer, szIniFile);
				}
				break;

				// Respond to OK buttom message
                case IDC_SQLCONFIG_ACCEPT:
                {
					// Write cconfig values to INI file
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_SQLCONFIG_WRITEINI, 0, 0));
                    EndDialog (hDlg, 0);
				}
				break;

                 // Respond to esc key or cancel button
                case IDCANCEL:
                case IDC_SQLCONFIG_CANCEL:
                {
					// No need to restore original values
					// They are already in ini file
					EndDialog (hDlg, 0);
                }
                return TRUE;
            }
        }
        break;
	 }
     return FALSE;
}

/*****************************************************************************/
/*																			 */
/*		Wrapper function to execute simple PostgreSQL query                  */
/*		Use only when PGRES_TUPLES_OK is expected query status		         */
/*		If query fails, program checks connection.  If NG, will attempt to   */
/*		reset it, and, if successful, will retry query.  Otherwise, will     */
/*		display error message and close program.                             */
/*																			 */
/*		Usage (within main dialog procedure):								 */
/*																			 */
/*				if((res=DoSQL(conn, "SELECT id FROM customer"))==0)          */
/*				{                                                            */
/*					return 0; // Close program                               */
/*				}                                                            */
/*																			 */
/*		Note: If error, message sent to main dialog proc to free tables      */
/*																			 */
/*		Parameters:															 */
/*			PGconn	 *conn	Pointer to connection structure             	 */
/*			char	 *cmd	Pointer to command string						 */
/*														    				 */
/*		Return:                                                              */
/*		    PGresult *      Pointer to result structure, or 0=Error   	     */
/*																			 */
/*		Revision record:													 */
/*			12/03/05	Copied from DoSQL() of MMSDBase						 */
/*																			 */
/*****************************************************************************/
 
PGresult *DoSQL(PGconn *conn, char *cmd)
{
	// Local variables
	char szError[BUFFER_SIZE+1];		// Error message
	PGresult *res;						// Query result pointer

	// Local database connection parameters
	char szHost[BUFFER_SIZE+1];			// Host IP address
	char szDatabase[BUFFER_SIZE+1];		// Database name
	char szUser[BUFFER_SIZE+1];			// User name

	// Execute query
	res=PQexec(conn, cmd);

	// Check status
	if(PQresultStatus(res) != PGRES_TUPLES_OK)
	{
		// Check connection
		if((PQstatus(conn)==CONNECTION_OK))
		{
			// Display query error message
			_snprintf(szError, sizeof(szError),
				"Database query \"%s\" failed.\n"
				"Program will close.", cmd);
			MessageBox(hwndMain, szError, "Database Error", MB_ICONHAND);

			// Close down PostgreSQL
			PQclear(res);
			PQfinish(conn);

			// Free tables
			SendMessage(hwndMain, WM_COMMAND,
				GET_WM_COMMAND_MPS(IDC_SQL_FREETABLES, NULL, 0));

			// Close down program
			PostQuitMessage(0);
			return 0;
		}
		else // Bad connection
		{
			// Try to reset connection
			PQreset(conn);

			// Bail out if connection is still bad
			if(PQstatus(conn) != CONNECTION_OK)
			{
				// Get connection parameters
				strlcpy(szHost, PQhost(conn), sizeof(szHost));
				strlcpy(szDatabase, PQdb(conn), sizeof(szDatabase));
				strlcpy(szUser, PQuser(conn), sizeof(szUser));

				// Show error message and quit
				_snprintf(szError, sizeof(szError),
					"Connection as user %s to database %s"
					" at host %s failed.\n"
					"Check connection and server.\n"
					"Program will close.", szUser, szDatabase, szHost);
					MessageBox(NULL, szError, "Database Error",
						MB_OK | MB_ICONHAND);

				// Close down PostgreSQL
				PQclear(res);
				PQfinish(conn);

				// Free tables
				SendMessage(hwndMain, WM_COMMAND,
					GET_WM_COMMAND_MPS(IDC_SQL_FREETABLES, NULL, 0));

				// Close down program
				PostQuitMessage(0);
				return 0;
			}
			else // Connection restored
			{
				// Try query again
				res=PQexec(conn, cmd);

				// Check status
				if(PQresultStatus(res) != PGRES_TUPLES_OK)
				{
					// Display query error message
					_snprintf(szError, sizeof(szError),
						"Database query \"%s\" failed.\n"
						"Program will close.", cmd);
					MessageBox(hwndMain, szError, "Database Error",
						MB_ICONHAND);

					// Close down PostgreSQL
					PQclear(res);
					PQfinish(conn);

					// Free tables
					SendMessage(hwndMain, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_SQL_FREETABLES, NULL, 0));

					// Close down program
					PostQuitMessage(0);
					return 0;
				}
			}
		}
	}

	// Query successful
	return res;
}

/*****************************************************************************/
/*																			 */
/*		Functions to allocate, reallocate & free an array of characters		 */
/*		of variable size (a matrix of characters with r rows of c columns)	 */
/*		Typically used to hold a table of strings whose number and size		 */
/*		can be set and changed at run time.	 Access is simply by table[i]	 */
/*		where i is the row number of the string in the table.				 */
/*		Both # rows and # characters can be set & changed at run time		 */
/*		Use static ints to hold row & column sizes							 */
/*																			 */
/*		Row indices go from 0 to r-1										 */
/*		Column indices go from 0 to c-1										 */
/*		Strings are a maximum of c-1 characters long plus EOS at end		 */
/*																			 */
/*		Usage:	static char **table;		   // Define table pointer		 */
/*				table=AllocCharArray(5, 10);   // Dims variable at run time	 */
/*				strcpy(table[i], "123");	   // i=row #					 */
/*				ClearCharArray(table, 5, 10);  // Fill with/ empty strings	 */
/*				table=ReallocCharArray(8, 20); // Dims variable at run time	 */
/*				FreeCharArray(table, 8, 20);   // Free memory				 */
/*																			 */
/*		Parameters:	  char ** p	  pointer to character pointer				 */
/*					  int r		  # rows									 */
/*					  int c		  # columns									 */
/*					  int oldr	  old # rows								 */
/*					  int newr	  new # rows								 */
/*																			 */
/*		Return: Pointer to array of pointers to rows						 */
/*																			 */
/*		Ref:  Numerical Recipes in C  Section 1.2 & Utility Routines		 */
/*			  for similar scheme involving numeric data arrays				 */
/*																			 */
/*		Revision record:													 */
/*			12/03/05	Copied from same functions of MMSDBase application	 */
/*																			 */
/*****************************************************************************/

// Allocate a character array with r rows of c characters
// Return pointer to pointers to array rows
// Access contents with table[i], where i if row #
// All rows are same length
// Strings can have up to r-1 characters plus EOS at end

// File scope variable to store # array columns
// Set during array alloc or realloc
static nNumArrayCols;

char ** AllocCharArray(int r, int c)
{
	// Local variables
	int i;			// Index
	char **p;		// Pointer to pointer to array

	// Set # cols
	nNumArrayCols=c;
	
	// Allocate pointers to rows
	p=(char **)malloc((unsigned)r*sizeof(char *));
	// Error check here

	// Allocate rows and set pointers to them
	for(i=0; i<r; i++)
	{
		p[i]=(char *)malloc((unsigned)c*sizeof(char));
		// Error check here
	}

	return p;
}

// Reallocate existing character array from oldr # rows
// to newr # rows and c characters per row (new value)
// Old content is preserved in rows that remain
char ** ReallocCharArray(char **p, int oldr, int newr, int c)
{
	// Local variables
	int i;			// Index

	// Set # cols
	nNumArrayCols=c;
	
	// Reallocate pointers to rows
	p=(char **)realloc((char **)p, (unsigned)newr*sizeof(char *));
	// Error check here

	// More rows
	if(newr>oldr)
	{
		// Rellocate old rows and set pointers to them
		for(i=0; i<oldr; i++)
		{
			p[i]=(char *)realloc((char *)p[i], (unsigned)c*sizeof(char));
			// Error check here
		}

		// Allocate new rows and set pointers to them
		for(i=oldr; i<newr; i++)
		{
			p[i]=(char *)malloc((unsigned)c*sizeof(char));
			// Error check here
		}
	}

	// Same or fewer rows
	if(newr<=oldr)
	{
		// Rellocate some of old rows and set pointers to them
		for(i=0; i<newr; i++)
		{
			p[i]=(char *)realloc((char *)p[i], (unsigned)c*sizeof(char));
			// Error check here
		}
	}

	return p;
}


// This doesn't actually clear the character array,
// but rather puts null strings into it
void ClearCharArray(char ** p, int r)
{
	// Local variables
    int i;			// Index

    // Clear character array by inserting empty strings
    for(i=0; i<r; i++)
    {
        strlcpy(p[i], "", nNumArrayCols);
    }
}

// Note:  This function doesn't free memory that was abandoned
// due to table size reduction by ReallocCharArray()
// but it is the best I know how to do simply.
// Want to preserve table contents when changing size,
// so can't free and allocate all over again w/o elaborate
// copying (e.g. allocate new table, copy old contents into it,
// and free old table).
void FreeCharArray(char **p, int r)
{
	int i;

	// Free character array
	for(i=0; i<r; i++)
	{
		free((char*)p[i]);
	}
	free((char*)p);
}

/*****************************************************************************/
/*																			 */
/*		Function to convert SQL TIMESTAMP to MJD                             */
/*																			 */
/*		Example: 2005-10-09 12:00:00 = 	63652.500000						 */
/*		The timestamp seconds may have a fractional part e.g. :00.123456	 */
/*		which is used in determining the fractional part of the MJD.         */
/*		The fractional part of the MJD always has 5 digits.      			 */
/*		The timnestamptz format includes a +XX timezone value at the end	 */
/*		which is ignored for this conversion.        						 */
/*		The returned MJD string has 6 decimal places - about 1 second        *
/*		The returned MJD double has 12 decimal places - about 100 nsec res   */
/*																			 */
/*		Parameters:															 */
/*			const char   *timestamp		Timestamp string to be converted	 */
/*			      char   *mjd			MJD string for timestamp             */
/*				  double *mjdval		MJD value for higher resolution      */
/*																			 */
/*		Return:   int					0=OK, >0=Error (bad date/time)	     */
/*																			 */
/*		Revision record:													 */
/*			06/13/07	Copied from MMSDbase								 */
/*			07/17/07	Eliminated fractional seconds                        */
/*			07/24/07	Recopied from MMSDbase								 */
/*			07/27/09	Ertror codes changed to specific values >0           */
/*																			 */
/*****************************************************************************/
 
int TimestampToMJD(char *timestamp, char *mjd, double *mjdval)
{
	char year[5];		// 4-digit year string YYYY (e.g. 2005)
	char month[3];		// 2-digit month string MM (01=Jan, etc.)
	char day[3];		// 2-digit day of month string DD (01-31)
	char hour[3];		// 2-digit hour string HH (00-23)
	char minute[3];		// 2-digit minute string MM (00-59)
	char second[10];	// 2-digit seconds plus 6-digit fractional seconds
						// string (e.g. 01.234567)
	const int days[13]={0,31,29,31,30,31,30,31,31,30,31,30,31};
						// # days in month (leap year)
	int yr;				// Year as number (1900-2099 valid)
	int mo;				// Month as number (1-12 valid)
	int da;				// Day as number (1-31 valid depending on month)
	int modjd;			// MJD as number (15020-88068 covered)
	double dy;			// Fractional day as double
	double hr;			// Hour as double
	double min;			// Minutes as double
	double sec;			// Seconds as double
	
	// Truncate the timestamp to eliminate fractional seconds
	timestamp[19]='\0';
	
	// Break down timestamp into its parts as strings
	// Get numeric values of the yr, mo, da, dy, hr, min & sec
	// Parse year
	year[0]=timestamp[0];
	year[1]=timestamp[1];
	year[2]=timestamp[2];
	year[3]=timestamp[3];
	year[4]='\0';
	yr=atoi(year);

	// Check for valid year
	// MJD covers beyond these limits
	if((yr<1900)|(yr>2099))
	{
		// Bad year
		// Error
		return 1;
	}

	// Check for dash after year
	if(timestamp[4] != '-')
	{
		// No dash after year
		// Error
		// return 1;
		return 2;
	}

	// Parse month
	month[0]=timestamp[5];
	month[1]=timestamp[6];
	month[2]='\0';
	mo=atoi(month);

	// Check for valid month
	if((mo<1)|(mo>12))
	{
		// Bad month
		// Error
		// return 1;
		return 3;
	}

	// Check for dash after month
	if(timestamp[7] != '-')
	{
		// No dash after month
		// Error
		// return 1;
		return 4;
	}

	// Parse day
	day[0]=timestamp[8];
	day[1]=timestamp[9];
	day[2]='\0';
	da=atoi(day);

	// Check for valid day
	if((da<1)||(da>days[mo]))
	{
		// Error
		// return 1;
		return 5;
	}

	// Check for February 29
	if((mo==2) && (da==29))
	{
		// Leap year?
		if(yr%4==0 && yr%100!=0 || yr%400==0)
		{
			// Leap year
		}
		else
		{
			// Bad day
			// Error
			// return 1;
			return 6;
		}
	}

	// Check for space after day
	if(timestamp[10] != ' ')
	{
		// No space after year
		// Error
		// return 1;
		return 7;
	}

	// Parse hour
	hour[0]=timestamp[11];
	hour[1]=timestamp[12];
	hour[2]='\0';
	hr=atof(hour);

	// Check for valid hour
	if((hr<0)|(hr>23))
	{
		// Bad hour
		// Error
		// return 1;
		return 8;
	}

	// Check for colon after hour
	if(timestamp[13] != ':')
	{
		// No colon after hour
		// Error
		return 1;
		return 9;
	}

	// Parse minute
	minute[0]=timestamp[14];
	minute[1]=timestamp[15];
	minute[2]='\0';
	min=atof(minute);

	// Check for valid minute
	if((min<0)|(min>59))
	{
		// Bad minute
		// Error
		// return 1;
		return 10;
	}

	// Check for colon after minute
	if(timestamp[16] != ':')
	{
		// No colon after minute
		// Error
		// return 1;
		return 11;
	}

	// Check for valid character after whole seconds
	// It can be a space, decimal point, or EOS
	if((timestamp[19]!=' ') && (timestamp[19]!='.') && (timestamp[19]!='\0'))
	{
		// Bad character after whole seconds
		// Error
		// return 1;
		return 12;
	}

	// Parse seconds
	// Test for fractional seconds removed
	second[0]=timestamp[17];
	second[1]=timestamp[18];
	//second[2]=timestamp[19];
	//second[3]=timestamp[20];
	//second[4]=timestamp[21];
	//second[5]=timestamp[22];
	//second[6]=timestamp[23];
	//second[7]=timestamp[24];
	//second[8]=timestamp[25];
	//second[9]='\0';
	second[2]='\0';
	sec=atof(second);

	// Check for valid second
	// Leap second not allowed
	if((sec<0)|(sec>59))
	{
		// Bad second
		// Error
		// return 1;
		return 13;
	}
	
	// Find the MJD
	modjd=DateToMJD(da, mo, yr);

	// Calc the fractional day
	dy=(hr/24)+(min/1440)+(sec/86400);

	// Check for no fractional seconds
	if(timestamp[19]!=' ')
	{
		// Integer seconds
		// Add 0.5 sec to fractional day for rounding
		// dy+=0.000005787;  // Seems better w/o this
	}

	// Add on the whole mjd
	dy+=(double)modjd;

	*mjdval=dy;

	// Convert it to a string
	// Limit mjd string to width expected from %f format
	// MJD format = 54025.351713 = 12 char + EOS
	_snprintf(mjd, 13, "%f", dy);

	// Return success
	return 0;
}

/****************************************************************************/
/*                                                                          */
/*                             MJDtoDate()                                  */
/*                                                                          */
/*		Function for MJD to calendar date conversion                        */
/*                                                                          */
/*      Parameters:                                                         */
/*          long mjd     modified Julian day # to be converted into date    */
/*			int	*day	 pointer to day of date to be found from MJD        */
/*		    int *month   pointer to month of date to be found from MJD      */
/*          int *year    pointer to year of date to be found from MJD       */
/*                                                                          */
/*      Return: 														    */
/*			none (void)                                                     */
/*                                                                          */
/*		Function revision record:                                           */
/*			11/11/92	Created                                             */
/*			10/13/05	Argument mjd changed from long to int (32 bit)      */
/*			02/19/08	Added missing (int) cast                            */
/*                                                                          */
/*  (c) 1992-2005 W. Riley Hamilton Technical Services All Rights Reserved  */
/*                                                                          */
/****************************************************************************/

void MJDtoDate(int mjd, int *day, int *month, int *year)
{
	mjd+=679019;
	*year=(int)((mjd-122.1)/365.25);
	*month=(int)((mjd-(int)(365.25**year))/30.6001);
	*day=mjd-(int)(365.25**year)-(int)(30.6001**month);

	if(*month<14)
	{
		*month-=1;
	}                                                             
	else
	{
		*month-=13;
	}                                                          
	if(*month<3)
	{
		*year+=1;
	}                                                             

}                                                        

/****************************************************************************/
/*                                                                          */
/*                            FractionalDayToHMS                            */
/*                                                                          */
/*		Function to convert fractional day (e.g. the fractional  part of    */
/*		a MJD) to HMS in 24-hour format                                     */
/*                                                                          */
/*      Parameters:                                                         */
/*			double fd		Fractional day (0.0 to <1.0)					*/
/*			int	 *hr		Hours (0-23)                                    */
/*		    int  *min		Minutes (0-59)                                  */
/*          int  *sec		Seconds (0-59)                                  */
/*                                                                          */
/*      Return: 														    */
/*          void (none)                                                     */
/*                                                                          */
/*      Example:                                                            */
/*		    0.5 day is 12:00:00 as HH:MM:SS                                 */
/*                                                                          */
/*		Function revision record:                                           */
/*			06/15/07	Copied from MMSDbase                                */
/*			06/25/07	Added rounding of seconds                           */
/*			07/24/07	Recopied from MMSDbase                              */
/*                                                                          */
/* (c) Copyright 2005-7 Hamilton Technical Services  All Rights Reserved    */
/*                                                                          */
/****************************************************************************/

void FractionalDayToHMS(double fd, int *hr, int *min, int *sec)
{
	*hr=(int)(fd*24);
	fd=fd*24-(double)*hr;
	*min=(int)(fd*60);
	fd=fd*60-(double)*min;
	// Round seconds - this keeps start & end utc & mjd consistent
	// as their edit boxes are repeatedly pressed
	*sec=(int)((fd*60)+0.5);
}                                                        

/*****************************************************************************/
/*																			 */
/*		Function to convert numeric MJD to SQL TIMESTAMP string              */
/*		Timestamp format is YYYY-MM-YY HH:MM:SS                              */
/*		It does not include fractional seconds                               */
/*																			 */
/*		Example:  63652.500000 = 2005-10-09 12:00:00						 */
/*																			 */
/*		Parameters:															 */
/*			      double mjd			MJD numeric value to be converted    */
/*				  char *timestamp		Timestamp string                 	 */
/*																			 */
/*		Return:   none (void)									     	     */
/*																			 */
/*		Revision record:													 */
/*			12/03/05	Copied from MJDtoTimestamp() of MMSDBase             */
/*																			 */
/*****************************************************************************/
 
void MJDtoTimestamp(double mjd, char *timestamp)
{
	int year;			// Year (YYYY)
	int month;			// Month (MM, 00-12)
	int day;			// Day (DD, 00-31)
	int hour;			// Hours (HH, 00-24)
	int minute;			// Minutes (MM, 00-59
	int second;			// Seconds (SS, 00-59)
	double fmjd;		// Fractional part of MJD
	
	// Use interger part of MJD to find date
	MJDtoDate((int)mjd, &day, &month, &year);

	// Find fractional part of MJD
	fmjd = mjd - (int)mjd;

	// Use the fractional part of MJD to find time
	FractionalDayToHMS(fmjd, &hour, &minute, &second);
	
	// Create the timestamp string
	// Timestamp string has format 2007-07-24 17:43:16 (19 chars)
	// So limit timestamp string to 20 chars (19 + EOS)
	_snprintf(timestamp, 20, "%04d-%02d-%02d %02d:%02d:%02d",
		year, month, day,hour, minute, second);
}

/*****************************************************************************/
/*                                                                           */
/*							ValidateTimestamp                                */
/*                                                                           */
/*		Function to validate SQL timestamp string input versus allowable     */
/*		MJD limits.  Checks for OK timestamp format and date/time values.    */
/*		Error message shows allowable timestamp range                        */
/*                                                                           */
/*      Parameters:                                                          */
/*      	HWND	hwnd			Handle to window calling function        */
/*          char	*szTimestamp	Timestamp string to be checked           */
/*          int		nMinMJD 		Minimum allowable MJD value              */
/*			int		nMaxMJD  		Maximum allowable MJD value              */
/*                                                                           */
/*      Return:                                                              */
/*			BOOL	status		TRUE=Value OK, FALSE=Value NG                */
/*                                                                           */
/*      Note 1:	Calling function S/B able to restore variable to its         */
/*              previous or default value before the bad entry was made.     */
/*      Note 2:	Call in response to EN_KILLFOCUS message when using with     */
/*              an edit control.                                             */
/*      Note 3:	Don't call in response to pressing the Cancel button.        */
/*                                                                           */
/*****************************************************************************/
BOOL ValidateTimestamp(HWND hwnd, char *szTimestamp, double fMinMJD,
	double fMaxMJD)
{
	// Local variables
	char szMJD[BUFFER_SIZE+1];		// MJD Text buffer
	char szMinTS[BUFFER_SIZE+1];	// Min allowable timestamp
	char szMaxTS[BUFFER_SIZE+1];	// Max allowable timestamp
	double fMJDVal;					// Numeric value of full MJD being tested

	F_TYPE fEps=1e-11;              // Numerical margin factor
									// XXXXX.XXXXXX MJD has e-11 resolution
	
	// Convert timestamp to MJD string
	if(TimestampToMJD(szTimestamp, szMJD, &fMJDVal))
	{
		// Bad date/time format
			// Display error message
			MessageBeep(0);
			_snprintf(szMJD, sizeof(szMJD),
				"Timestamp %s format invalid, should be YYYY-MM-DD HH:MM:SS",
				szTimestamp);
			MessageBox(hwnd, szMJD, "Invalid Entry", MB_ICONEXCLAMATION);
		return FALSE;
	}
	else // Timestamp format valid - check its MJD value against limits
	{
		// Compare MJD against limits
		if( ((fMJDVal+fMJDVal*fEps)<fMinMJD) ||
			((fMJDVal-fMJDVal*fEps)>fMaxMJD) )
		{
			// Find allowable timestamps
			MJDtoTimestamp(fMinMJD, szMinTS);
			MJDtoTimestamp(fMaxMJD, szMaxTS);
			
			// Display error message
			MessageBeep(0);
			_snprintf(szMJD, sizeof(szMJD),
				"Timestamp %s must be between %s and %s",
				szTimestamp, szMinTS, szMaxTS);
			MessageBox(hwnd, szMJD, "Invalid Entry", MB_ICONEXCLAMATION);
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}
}

/*****************************************************************************/
/*                                                                           */
/*							SQLYieldControl()                                */
/*                                                                           */
/*		Function to allow Windows to take control & service other messages   */
/*                                                                           */
/*      Parameters:                                                          */
/*			HWND hDialog	Handle of dialog box receiving messages          */
/*                                                                           */
/*      Return:                                                              */
/*			None	(void)                                                   */
/*                                                                           */
/*      Globals:                                                             */
/*           None                                                            */
/*                                                                           */
/*		Revision record:                                                     */
/*			12/03/05	Created from YieldControl() in MMSDBase              */
/*						Has hDialog as parameter instead of global           */
/*						as in original Stable32 YieldControl()               */
/*                                                                           */
/*****************************************************************************/
void WINAPI SQLYieldControl(HWND hDialog)
{
	// Local variables
	MSG msg;
	
	while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
	{
		if(!IsDialogMessage(hDialog, &msg))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
}

/*****************************************************************************/
/*                                                                           */
/*							Encode() and Decode()                            */
/*                                                                           */
/*		Tiny encription algorithm (TEA) - Assumes 32 bit words               */
/*		Ref: http://www.ftp.cl.cam.ac.uk/ftp/papers/djw-rmn/djw-rmn-tea.html */
/*		Credit: D. Wheeler & R. Needham, Cambridge Univ. Computer Lab        */
/*                                                                           */
/*      Parameters:                                                          */
/*      	long *v		Data in v[0] and v[1]   (64 bits)                    */
/*			long *k		Key  in k[0] thru k[3]  (128 bits)                   */
/*                                                                           */
/*      Return:                                                              */
/*			None (void)                                                      */
/*                                                                           */
/*		Revision record:                                                     */
/*			07/25/03	Created, modified slightly for our purposes          */
/*                                                                           */
/* (c) Copyright 2003 Hamilton Technical Services  All Rights Reserved       */
/*                                                                           */
/*****************************************************************************/

void WINAPI Encode(unsigned int *v, unsigned int *k)
{              
	unsigned int y=v[0], z=v[1], sum=0,			  /* set up */
		delta=0x9e3779b9, n=32;				      /* a key schedule constant */

	while (n-->0)
	{											  /* basic cycle start */
		sum += delta;
		y += (z<<4)+k[0] ^ z+sum ^ (z>>5)+k[1];
		z += (y<<4)+k[2] ^ y+sum ^ (y>>5)+k[3];   /* end cycle */
    }
	
	v[0]=y;
	v[1]=z;
}

void WINAPI Decode(unsigned int *v, unsigned int *k)
{
	unsigned int n=32, sum, y=v[0], z=v[1],
		delta=0x9e3779b9;
	
	sum=delta<<5;
												  /* start cycle */
	while(n-->0)
	{
		z-= (y<<4)+k[2] ^ y+sum ^ (y>>5)+k[3]; 
		y-= (z<<4)+k[0] ^ z+sum ^ (z>>5)+k[1];
		sum-=delta;
	}
												  /* end cycle */
	v[0]=y;
	v[1]=z;
}

/*****************************************************************************/
