/*****************************************************************************/
/*                                                                           */
/*                            STABLE_S.C                                     */
/*                                                                           */
/*	Plot Scale Functions for Stable32                                        */
/*                                                                           */
/*	Functions:		                                                         */
/*              PlotScale()   Not used     							         */
/*              BasScale()    Not used     							         */
/*              RoundAxes()   Used                                           */
/*                                                                           */
/*	Revision record:                                                         */
/*                                                                           */
/*	03/10/02    Created                                                      */
/*	03/19/03	All these functions moved to FrequenC DLL                    */
/*                                                                           */
/*  (c)1996-2002 W. Riley Hamilton Technical Services All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#define STRICT 					// For Win32 compatibility

// Include the stable_w header file
#include "stable_w.h"           // STABLE_W header file

#if(0) // Now in FrequenC DLL as of 3/19/03
/*****************************************************************************/
/*                                                                           */
/*  							PlotScale()                                  */
/*                                                                           */
/* 	    Function to find scale factor for plotting phase or freq data        */
/*                                                                           */
/*		Parameters:    F_TYPE min          = minimum freq value              */
/*      	           F_TYPE max          = maximum freq value              */
/*			           F_TYPE *p_scale_min = pointer to scale minimum        */
/*			           F_TYPE *p_scale_max = pointer to scale maximum        */
/*                                                                           */
/*      Return:        int = # tick intervals in scale,                      */
/*                           or -1 if error                                  */
/*                                                                           */
/*		Revision record:                                                     */
/*			03/10/02   Created from revised FindPlotScale()                  */
/*                                                                           */
/* (c) Copyright 1991-2002 Hamilton Technical Services All Rights Reserved   */
/*                                                                           */
/*****************************************************************************/

int PlotScale(F_TYPE min, F_TYPE max, F_TYPE *p_scale_min,
	F_TYPE *p_scale_max)
{
	double range, log_range, coarse, ticks;
	// double fine; // Not used
	double full_scale, digits, e;
	int k;                                                  /* # ticks added */

	e=1e-6;                                               /* error tolerance */
	range = max - min;

	if(range<=0)                                       /* check for NG range */
	{
		*p_scale_min=0;
		*p_scale_max=0;
		return(-1);                                            /* error code */
	}

	log_range = log10(range) + 100;

	coarse = floor(log_range)- log10(5);
	ticks=6;

	full_scale = pow(10, coarse - 100) * ticks;
	if(log10(full_scale) >= 0)
	{
		digits = floor(log10(full_scale));
	}                                                              /* end if */
	else
	{
		digits = ceil(log10(full_scale)) - 1;
	}                                                            /* end else */

	if((min / pow(10, digits)) >= 0)
	{
		*p_scale_min = floor(min / pow(10, digits)) * pow(10, digits);
	}                                                              /* end if */
	else
	{
		*p_scale_min = (floor(min / pow(10, digits))-1) * pow(10, digits);
	}                                                            /* end else */
	*p_scale_max = *p_scale_min + full_scale;

	k=0;

	if(*p_scale_max < max)                  /* check for scale max too small */
	{
		while(*p_scale_max < max)                           /* move scale up */
		{
			*p_scale_max += full_scale/ticks;
			*p_scale_min += full_scale/ticks;
		}
		if(*p_scale_min > min)
		{
			while(*p_scale_min > min)             /* add tic interval to max */
			{
				*p_scale_min -= full_scale/ticks;
				k++;
			}
		}
	}

	else if(*p_scale_min > min)             /* check for scale min too large */
	{
		while(*p_scale_min > min)                         /* move scale down */
		{
			*p_scale_max -= full_scale/ticks;
			*p_scale_min -= full_scale/ticks;
		}
		if(*p_scale_max < max)                    /* add tic interval to min */
		{
			while( *p_scale_max < max)
			{
				*p_scale_max += full_scale/ticks;
				k++;
			}
		}
	}

	ticks+=k;                                      /* adjust # tic intervals */

	return(int)ticks;
}                                                     /* end FindPlotScale() */

/*****************************************************************************/
/*                                                                           */
/*  							BasScale()                                   */
/*                                                                           */
/* 	    Function to find neat plot scale                                     */
/*      Ref: A. Bas, "Finding Neat Scales for Plotting", CUJ, March 2000     */
/*                                                                           */
/*		Parameters:    double XMin         = minimum data value              */
/*      	           double XMax         = maximum data value              */
/*                     int N               = max # divs in plot scale        */
/*			           double *SMin        = pointer to scale minimum        */
/*			           double *Step        = pointer to step size            */
/*                                                                           */
/*      Return:        int                 = 1 if OK or 0 if error           */
/*                                                                           */
/*		Revision record:                                                     */
/*			03/09/02   Downloaded and adapted                                */
/*					   Runs AOK and seems to produce nice tight scales       */
/*                                                                           */
/*****************************************************************************/
     
#include <math.h>
     
     
int BasScale (double XMin, double XMax, int N, double* SMin, double* Step)
//  Written by Antonio Gmiz Bas, 1999 based on subroutine SCALE
//  by W. Douglas Stirling
{
  int    iNegScl;       //  Negative scale flag
  int    iNm1;          //  Number of scale subintervals 
  double lfIniStep;     //  Initial step
  double lfSclStep;     //  Scaled step 
  double lfTmp;         //  Temporary value
  double lfSclFct;      //  Scaling back factor 
  double lfSMax;        //  Scale maximum value 
  int    it;            //  Iteration counter 
  int    i;             //  Neat step counter
     
  //  Neat steps

  //  Original version had these steps
  int Steps [] = {10, 12, 15, 16, 20, 25, 30, 40, 50, 60, 75, 80, 100, 120, 150};

  //  Try reduced choices for use in case of narrow range 
  //  int Steps [] = {10, 20, 50, 100};

  int iNS = sizeof (Steps) / sizeof (Steps [0]);
     
  //  Some checks
  if (XMin >  XMax)
    {
      lfTmp = XMin;
      XMin  = XMax;
      XMax  = lfTmp;
    }
  if (XMin == XMax) XMax = XMin == 0.0 ? 1.0 : XMin + fabs (XMin) / 10.0;
     
  //  Reduce to positive scale case if possible 
  if (XMax <= 0)
    {
      iNegScl = 1;
      lfTmp   = XMin;
      XMin    = -XMax;
      XMax    = -lfTmp;
    }
  else
    iNegScl = 0;
     
  if (N < 2) N = 2;
  iNm1 = N - 1;
     
  for (it = 0; it < 3; it++)
    {
      //  Compute initial and scaled steps 
      lfIniStep = (XMax - XMin) / iNm1;
      lfSclStep = lfIniStep;
     
      for (; lfSclStep <  10.0; lfSclStep *= 10.0); 
      for (; lfSclStep > 100.0; lfSclStep /= 10.0);
     
      //  Find a suitable neat step
      for (i = 0; i < iNS && lfSclStep > Steps [i]; i++); 
      lfSclFct = lfIniStep / lfSclStep;
     
      //  Compute step and scale minimum value 
      do
        {
          *Step  = lfSclFct * Steps [i];
          *SMin  = floor (XMin / *Step) * *Step; 
          lfSMax = *SMin + iNm1 * *Step;
          if (XMax <= lfSMax)           //  Function maximum is in the
										//  range: the work is done.
			{
              if (iNegScl) *SMin = -lfSMax;
              *Step *= iNm1 / (N - 1);
              return 1;
            }
          i++;
        }
      while (i < iNS);
     
      //  Double number of intervals
      iNm1 *= 2;
    }
     
  //  Could not solve the problem
  return 0;
}

/***************************************************************************/
/*                                                                         */
/*  							RoundAxes()                                */
/*                                                                         */
/* 	    Still another function to find plot scale                          */
/*      Ref: Quinn-Curtis S&E Tools IPC-TC-006 SEGRAPH.C                   */
/*                                                                         */
/*		Parameters:  realtype *a1   =   pointer to data min input          */
/*										becomes scale min output           */
/*      	         realtype *a2   =   pointer to data max input          */
/*                                      becomes scale max output           */
/*			         realtype *tics =   pointer to tics size output        */
/*                                                                         */
/*      Return:        None (void)                                         */
/*                                                                         */
/*		Revision record:                                                   */
/*			03/10/02	Copied and adapted                                 */
/*			03/12/02	Added log 0 trap                                   */
/*                                                                         */
/***************************************************************************/

// Subroutines adapted from SELIB.C
F_TYPE log10x(F_TYPE realnum)
{  F_TYPE fret;
	// Avoid log 0
	if(realnum==0.0)
	{
		realnum=1e-99;
	}
   fret = log10(realnum);
   return(fret);
}

int NumExp(F_TYPE realnum)
{
   int fret;
   fret = (int)floor(log10x(fabs(realnum)));
   return(fret);
}

F_TYPE PowerCalc(F_TYPE realnum, F_TYPE power)
{
   F_TYPE fret;
   fret = pow(realnum,power);
   return(fret);
}

// RoundAxes function
void RoundAxes(F_TYPE *a1, F_TYPE *a2, F_TYPE *tics)
{
   F_TYPE dr1, dr2, px2, pc2;
   int di1, di2, digits, a2neg, a1neg;

   a1neg = (*a1 < 0);
   a2neg = (*a2 > 0);
   if (NumExp(*a2) > NumExp(*a1))
     px2 = NumExp(*a2) - 1;
   else
     px2 = NumExp(*a1) - 1;
   pc2 = PowerCalc(10.0,px2);
   dr2 = (*a2) / pc2;
   dr1 = (*a1) / pc2;

   di2 = (int)ceil(dr2 + 0.000001+(dr2-dr1)*0.05);
   if ((di2 > 0) && (dr2 <= 0.0)) di2 = 0;

   di1 = (int)floor(dr1 - 0.000001- (dr2-dr1)*0.05);
   if ((di1 < 0) && (dr1 >= 0.0)) di1 = 0;

   if ( abs(di2) < 10 )
      di2 = di2 + a2neg;
   else
     if ( abs(di2) < 60 )
	di2 = ((di2 / 5) + a2neg ) * 5;
     else
       di2 = ((di2 / 10) + a2neg) * 10;

   if ( abs(di1) < 60 )
	di1 = ((di1 / 5) - a1neg) * 5;
   else
	di1 = ((di1 / 10) - a1neg) * 10;

   *a1 = di1 * pc2;
   *a2 = di2 * pc2;
   digits = abs(di2 - di1);
   if ( digits < 10 )
      *tics =  1.0;
   else
     if ( digits < 20 )
       *tics = 2.0;
     else
       if ( digits < 40 )
	 *tics = 5.0;
       else
	 if ( digits < 80 )
	   *tics = 10.0;
	 else
	   if ( digits < 100 )
	     *tics = 20.0;
	   else
	     if ( digits < 151 )
	       *tics = 20.0;
	     else
	       if ( digits < 200 )
		 *tics = 50.0;
	       else
		 *tics = 100.0;

   *tics = *tics * pc2 / 10.0;
}

/***************************************************************************/
#endif