/*****************************************************************************/
/*                                                                           */
/*                            STABLE_V.C                                     */
/*                                                                           */
/*		Plot Initialization Functions for Stable Windows 95/NT               */
/*                                                                           */
/*		Functions:		                                                     */
/*                      InitPhasePlotParams()    							 */
/*                      InitFreqPlotParams()    							 */
/*                      InitSigmaPlotParams()    							 */
/*						InitReadPlotParams(}                                 */
/*						CalcSigmaPlotScale()                                 */
/*                      InitAutocorrelationPlotParams()    	                 */
/*						UpdatePlotSubtitles()                                */
/*                                                                           */
/*	Revision record:                                                         */
/*                                                                           */
/*			12/24/97    Moved functions from STABLE_X.C                      */
/*						Made changes for error-free BC 32 bit compilation    */
/*			04/04/98	Added full drive/path to default *.TKF filenames     */
/*			08/05/03	Added autocorrelation plot initialization            */
/*			11/24/04	Added trap for too big timetags                      */
/*			11/22/06	Added UpdatePlotSubtitles()                          */
/*			07/26/07    Changed to safe string functions                     */
/*                                                                           */
/* (c) 1996-2007 W. Riley Hamilton Technical Services All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#define STRICT 					// For Win32 compatibility

// Include the stable_w header file
#include "stable_w.h"           // STABLE_W header file
#include <limits.h>				// For INT_MAX definition

/*****************************************************************************/
/*                                                                           */
/*                      InitPhasePlotParams()                                */
/*                                                                           */
/*		Initialize phase data plot parameters                                */
/*                                                                           */
/*		Parameters:		int bAll  0=y-axis scale only, 1=all params          */
/*                                                                           */
/*      Return:			None (void)                                          */
/*                                                                           */
/* 		Revision record:                                                     */
/*			10/17/96	Drafted                                              */
/*			12/31/96	Added additional INI file items                      */
/*			12/21/97    Added bFirstTime usage & line & note initializations */
/*			08/30/98	Added do_fit usage                                   */
/*			02/17/00	Added code for "nice" phase plot scale               */
/*			02/19/00	Added bAll param to control amount of initialization */
/*			02/20/00	Added rounding of GraphiC y-axis plot scales         */
/*			03/26/00	Fixed hangup problem re undefined roundq() arg       */
/*			06/01/00	Added code for PlotX config usage                    */
/*			03/11/01	Added plot line width parameter                      */
/*			03/10/02	Added code to try different plot scale algorithms    */
/*			03/11/02	Fixed femtosecond scale                              */
/*			03/23/02	Made more changes in plot scale code                 */
/*			05/10/02	Added usage of bPXDefault flag                       */
/*			10/01/02	Changed to DetermineXScale() usage                   */
/*			10/02/02	Changed to DetermineXMult() usage                    */
/*			11/24/04	Added test to avoid O/F for large timetags           */
/*                                                                           */
/*****************************************************************************/

void WINAPI InitPhasePlotParams(BOOL bAll)
{
	// Local variables
	// Chars
	char szBuffer[BUFFER_SIZE+1];   // Text buffer
	// BOOLs
	// This flag allows reading INI file 1st time only
	// After that, if phase plot params get reinitialized
	// they are set to defaults
	static BOOL bFirstTime=TRUE;	// Flag for first time initialized
	// Floats
	float scale_min;				// Plot scale minimum
	float scale_max;				// Plot scale maximum
	float fTau;						// Tau value
	// F_TYPEs
	F_TYPE min;						// Data min
	F_TYPE max;						// Data max
	F_TYPE smin;					// Scale min
	F_TYPE smax;					// Scale max
	F_TYPE sdiv;					// Scale div
	// Ints
	int nExp;						// Scale exponent
	int i;							// General index
	int nDOY;						// Day of year
	int nDay;						// Day of month
	int nMonth;						// Month of year
	int nYear;						// Year (4-digit)
	int nCoarseMJD;					// Leading digits of MJD
	int nDecade;					// Decade to remove from MJD scale

	// Variable initializations
	if(bUsePhasePlotIni && bFirstTime)
	{
		// Read info from ini file
		ReadIni(INI_PHASE);
		ReadIni(INI_PLINE);
		ReadIni(INI_PNOTE);

		// Reset 1st time flag
		bFirstTime=FALSE;
	}
	else
	{
		if(bAll)
		{
			if(bFirstTime)
			{
				// The idea here is to do a full read ini
				// and then set those items
				// to defaults that want to be.  In particular,
				// this will restore the previous x-axis scale.
				// Only read first time so that the Reset All button works.
				ReadIni(INI_PHASE);
				bFirstTime=FALSE;
			}

			if(bPXDefault)
			{
				strlcpy(sPPlot.x_label, "Data Point", XLABELSIZE);
				sPPlot.x_offset=0.0;
				sPPlot.x_mult=1.0;
				nPPlotX=0;
				sPPlot.x_div=5;
				sPPlot.do_line=FALSE; // Moved to default 9/27/02
			}

			strlcpy(sPPlot.title, "PHASE DATA", TITLESIZE);
			strlcpy(sPPlot.subtitle, szPhaseFileTitle, SUBTITLESIZE);
			strlcpy(sPPlot.message_str, "", MESSAGESIZE);
			strlcpy(sPPlot.left_y_label, "Phase", YLABELSIZE);
			sPPlot.y_offset=0.0;
			sPPlot.y_mult=1.0;
			sPPlot.position=MSG_TOP_LEFT;
			// sPPlot.do_line=FALSE; // Moved to default 9/27/02
			sPPlot.y_div=5;
			sPPlot.y_tic=5;
			sPPlot.do_mouse=FALSE;
			sPPlot.do_date=TRUE;
			sPPlot.do_box=TRUE;
			sPPlot.do_fit=TRUE;
			sPPlot.order=3;
		}
		else
		{
			strlcpy(sPPlot.left_y_label, "Phase", YLABELSIZE);
			sPPlot.y_mult=1.0;
			sPPlot.y_div=5;
		}

		// Get min & max of phase data
		FindMinMax(fPhaseData, &min, &max, PHASE_DATA);

		// Use "nice" phase scale
		if(bSmartScale)
		{
			// Find scale exponent
			nExp=(int)(floor(log10(max(fabs(min),fabs(max)))));

			// If exponent is multiplier is near 0 force it to 0
			if(nExp>-2 && nExp<3)
			{
				nExp=0; // nExp beween -1 and +2 becomes 0
			}

			// Set scale multiplier
			sPPlot.y_mult=(float)(pow(10, -nExp));

			// Make adjustments for standard units
			if(nExp==-2) // Milliseconds
			{
				nExp=-3;
				sPPlot.y_mult=(float)1e3;
			}

			if(nExp==-4 || nExp==-5) // Microseconds
			{
				nExp=-6;
				sPPlot.y_mult=(float)1e6;
			}

			if(nExp==-7 || nExp==-8) // Nanoseconds
			{
				nExp=-9;
				sPPlot.y_mult=(float)1e9;
			}

			if(nExp==-10 || nExp==-11) // Picoseconds
			{
				nExp=-12;
				sPPlot.y_mult=(float)1e12;
			}
			
			if(nExp==-13 || nExp==-14 || nExp<-15) // Femtoseconds
			{
				nExp=-15;
				sPPlot.y_mult=(float)1e15;
			}

			// Label y-axis
			if(nExp<-1) // nExp -2 and smaller
			{
				_snprintf(szBuffer, sizeof(szBuffer), ", Seconds x10[%d]",
					nExp);
				strlcat(sPPlot.left_y_label, szBuffer, YLABELSIZE);
			}

			if(nExp==0) // nExp between -1 to +2
			{
				strlcat(sPPlot.left_y_label, ", Seconds", YLABELSIZE);
			}

			if(nExp>2) // nExp +3 and larger 
			{
				_snprintf(szBuffer, sizeof(szBuffer), ", Seconds x10[%d]",
					nExp);
				strlcat(sPPlot.left_y_label, szBuffer, YLABELSIZE);
			}

			if(nExp==-3)
			{
				strlcpy(sPPlot.left_y_label, "Phase, Milliseconds",
					YLABELSIZE);
			}

			if(nExp==-6)
			{
				strlcpy(sPPlot.left_y_label, "Phase, Microseconds",
					YLABELSIZE);
			}

			if(nExp==-9)
			{
				strlcpy(sPPlot.left_y_label, "Phase, Nanoseconds",
					YLABELSIZE);
			}

			if(nExp==-12)
			{
				strlcpy(sPPlot.left_y_label, "Phase, Picoseconds",
					YLABELSIZE);
			}

			if(nExp<=-15)
			{
				strlcpy(sPPlot.left_y_label, "Phase, Femtoseconds",
					YLABELSIZE);
			}
		}
		
		// Determine y-axis plot scale
		if(((max-min)/((fabs(max)+fabs(min)*2)))>0.05)
		{
			// Display scale function for testing
			// strcpy(sPPlot.subtitle, "Y-Axis Init:RoundAxes()");

			// Determine plot scale using RoundAxes() function
			// This function gives nice scale when data range is large
			smin=min;
			smax=max;
			RoundAxes(&smin, &smax, &sdiv);
			sdiv*=5; // Note: This mult by 5 assures that 5 minor ticks are OK
			sPPlot.y_div=(int)ceil((smax-smin)/sdiv);
			scale_min=(float)smin;
			scale_max=(float)(smin+sPPlot.y_div*sdiv);
			
			// Reset # divs
			// sPPlot.x_div=(int)ceil((smax-smin)/sdiv); // Changed 03/20/08
			sPPlot.y_div=(int)ceil((smax-smin)/sdiv);

			// Tighten scale as much as possible
			while(min>=(scale_min+sdiv))
			{
				scale_min+=(float)sdiv;
				sPPlot.y_div-=1;
			}
			while(max<=(scale_max-sdiv))
			{
				scale_max-=(float)sdiv;
				sPPlot.y_div-=1;
			}

			// Round scales to 3 decimal places
			// Plot will require normalization before more digits are needed
			// This avoids silly small increments
			_snprintf(szBuffer, sizeof(szBuffer), "%1.3e", scale_min);
			scale_min=(float)atof(szBuffer);
			_snprintf(szBuffer, sizeof(szBuffer), "%1.3e", scale_max);
			scale_max=(float)atof(szBuffer);
		}
		else
		{
			// Display scale function for testing
			// strcpy(sPPlot.subtitle, "Y-Axis Init:BasScale()");

			// Determine plot scale using BasScale() function
			// This function is better with small relative data range
			// Even though its scale choice is often sub-optimum
			BasScale(min, max, 12, &smin, &sdiv);
			// sPPlot.x_div=(int)ceil((max-smin)/sdiv); // Changed 03/20/08
			sPPlot.y_div=(int)ceil((max-smin)/sdiv);
			// smax=smin+(sPPlot.x_div*sdiv); // Changed 03/20/08
			smax=smin+(sPPlot.y_div*sdiv);

			// Reset # divs
			// sPPlot.x_div=(int)ceil((smax-smin)/sdiv); // Changed 03/20/08
			sPPlot.y_div=(int)ceil((smax-smin)/sdiv);

			// Set scale limits
			scale_min=(float)smin;
			scale_max=(float)smax;

			// Tighten scale as much as possible
			while(min>=(scale_min+sdiv))
			{
				scale_min+=(float)sdiv;
				sPPlot.y_div-=1;
			}
			while(max<=(scale_max-sdiv))
			{
				scale_max-=(float)sdiv;
				sPPlot.y_div-=1;
			}
			
			// A potential problem with the BasScale() method is that
			// the # of minor tics is not always 5
			// And GraphiC doesn't alway honor the BasScale() values
		}
		
		// Set y-axis scale limits
		sPPlot.y_min=scale_min;
		sPPlot.y_max=scale_max;
	}

	// Set x-axis offset to zero
	sPPlot.x_offset=0.0;
	
	// Get special MJD-XXXXX or DOY XXXX scale
	// first time w/o opening plot options dialog box
	// Get phase tau
	GetWindowText(GetDlgItem(hwndMain, IDC_PHASETAU),
		szBuffer, BUFFER_SIZE);

	// Use its absolute value
	fTau=(float)fabs(atof(szBuffer));

	// Set the x-axis multiplier factor
	DetermineXMult(&sPPlot, nPPlotX, fTau);

	
	// Add special info to MJD & DOY labels if have timetags
	if(bPhaseTimetags)
	{
		// Find start point
		i=(int)fPhaseData[1]; // Start point #

		// Find # days in data
		nDay=(int)ceil((fPhaseData[2]-fPhaseData[1])*sPPlot.x_mult);

		// Determine # decades to remove from MJD scale
		nDecade=1000;
		if(nDay<10)
		{
			nDecade=100;
		}

		// Find # 100s or 1000s in MJD
		nCoarseMJD=0;

		// Set offset to start point timetag
		// Note:  This while loop hangs up if timetag too large
		// Must test for timetag larger than max int

		if(fPhaseTimetags[i+2]<INT_MAX)
		{
			while((float)fPhaseTimetags[i+2]-nCoarseMJD>=(float)nDecade)
			{
				nCoarseMJD+=nDecade;
			}
		}

		if(nPPlotX==X_MJD) // Adjust MJD scale
		{
			// Set offset to start point timetag
			sPPlot.x_offset=(float)fPhaseTimetags[i+2];

			#if(0) // Original code
			// Subtract leading part of MJD
			sPPlot.x_offset-=nCoarseMJD;
			#endif

			#if(1) // New code 03/20/08
			// Subtract leading part of MJD
			sPPlot.x_offset-=(nCoarseMJD+1);
			#endif

			// Scale to data point units
			sPPlot.x_offset*=(float)(86400.0/fTau);

			// Add leading part of MJD to x-scale label
			_snprintf(sPPlot.x_label, XLABELSIZE, "MJD - %d", nCoarseMJD);
		}

		 // Start point MJD
		lnMJD=(long)floor(fPhaseTimetags[i+2]);
		nDOY=MJDtoDOY(lnMJD); // Start point DOY

		// Convert integer MJD to date
		MJDToDate(lnMJD, &nDay, &nMonth, &nYear);

		if(nPPlotX==X_DOY)
		{
			#if(0) // Original code
			// Set x offset
			sPPlot.x_offset=(float)nDOY; // DOY
			#endif

			#if(1) // New code 03/20/08
			// Set x offset
			sPPlot.x_offset=(float)(nDOY-1); // DOY-1
			#endif

			// Add fractional day
			sPPlot.x_offset+=(float)fPhaseTimetags[i+2]-(float)lnMJD;

			 // Scale to data point units
			sPPlot.x_offset*=(float)(86400.0/fTau);

			// Add year to x-scale label
			_snprintf(sPPlot.x_label, XLABELSIZE, "DOY %d", nYear);
		}
	}

	// X-axis scale factor determination
	sPPlot.x_max=(float)(sPPlot.x_mult*(fPhaseData[2]+sPPlot.x_offset));
	sPPlot.x_min=(float)(sPPlot.x_mult*(fPhaseData[1]+sPPlot.x_offset));

	// Use fewer x divs if label is Data Point and # data points is large
	// and bPXDefault is TRUE
	if( (nPPlotX==X_POINT) && ((sPPlot.x_max-sPPlot.x_min)>10000) &&
		(bPXDefault) )
	{
		nPPlotX=X_KPOINT;
		sPPlot.x_mult=(float)0.001;
		sPPlot.x_max=(float)(sPPlot.x_mult*(fPhaseData[2]+sPPlot.x_offset));
		sPPlot.x_min=(float)(sPPlot.x_mult*(fPhaseData[1]+sPPlot.x_offset));
	}

	// Determine x-axis plot scale
	DetermineXScale(&sPPlot);

	if(bFirstTime || bAll)
	{
		// Set phase plot x-axis label
		LoadString(hInst, IDS_DPLOT_XLABEL_0+nPPlotX, szBuffer, BUFFER_SIZE);
		strlcpy(sPPlot.x_label, szBuffer, XLABELSIZE);

		// Get phase tau
		GetWindowText(GetDlgItem(hwndMain, IDC_PHASETAU),
			szBuffer, BUFFER_SIZE);

		// Use its absolute value
		fTau=(float)fabs(atof(szBuffer));

		// Set the x-axis multiplier factor
		DetermineXMult(&sPPlot, nPPlotX, fTau);

		// Adjust settings for modern plot style
		if(nStyle==MODERN_STYLE)
		{
			sPPlot.do_date=FALSE;
			sPPlot.do_wide=TRUE;
			strlcpy(sPPlot.subtitle, "", sizeof(sPPlot.subtitle));
		}

		// Set line thickness
		if(sPPlot.do_wide)
		{
			sPPlot.thick=WIDELINESIZE;
		}
		else
		{
			sPPlot.thick=0;
		}

		// Reset 1st time flag
		bFirstTime=FALSE;
	}
}

/*****************************************************************************/
/*                                                                           */
/*                      InitFreqPlotParams()                                 */
/*                                                                           */
/*		Initialize frequency data plot parameters                            */
/*                                                                           */
/*		Parameters:		int bAll  0=y-axis scale only, 1=all params          */
/*                                                                           */
/*      Return:			None (void)                                          */
/*                                                                           */
/* 		Revision record:                                                     */
/*			10/17/96	Drafted                                              */
/*			12/31/96	Added additional INI file items                      */
/*			12/21/97    Added bFirstTime usage & line & note initializations */
/*			02/17/00	Added code for "nice" freq plot scale                */
/*			02/19/00	Added bAll param to control amount of initialization */
/*			02/20/00	Added rounding of GraphiC y-axis plot scales         */
/*			03/26/00	Fixed hangup problem re undefined roundq() arg       */
/*			06/02/00	Added code for PlotX config usage                    */
/*			03/11/01	Added plot line width parameter                      */
/*			03/11/02	Added code for better plot scale algorithm           */
/*			03/23/02	Made more changes in plot scale code                 */
/*			05/10/02	Added usage of bFXDefault flag                       */
/*			10/01/02	Changed to DetermineXScale() usage                   */
/*			10/02/02	Changed to DetermineXMult() usage                    */
/*			11/24/04	Added test to avoid O/F for large timetags           */
/*                                                                           */
/*****************************************************************************/

void WINAPI InitFreqPlotParams(BOOL bAll)
{
	// Local variables
	// Chars
	// char scale_format[8];		// Dummy variable for roundq() call
	char szBuffer[BUFFER_SIZE+1];   // Text buffer
	// BOOLs
	// This flag allows reading INI file 1st time only
	// After that, if freq plot params get reinitialized
	// they are set to defaults
	static BOOL bFirstTime=TRUE;
	// Floats
	float scale_min;
	float scale_max;
	// float step;					// Dummy variable for roundq() call
	float fTau;						// Tau value
	// F_TYPEs
	F_TYPE min;
	F_TYPE max;
	F_TYPE smin;					// Scale min
	F_TYPE smax;					// Scale max
	F_TYPE sdiv;					// Scale div
	F_TYPE sigma;					// ADEV
	F_TYPE *fWorkData;              // Working data array
	// Ints
	int nExp;						// Scale exponent
	int i;							// General index
	int nDOY;						// Day of year
	int nDay;						// Day of month
	int nMonth;						// Month of year
	int nYear;						// Year (4-digit)
	int nCoarseMJD;					// Leading digits of MJD
	int nDecade;					// Decade to remove from MJD scale
	int nNum;						// # data points

	// TRACE_STR("InitFreqPlotParams()");
	
	// Variable initializations
	if(bUseFreqPlotIni && bFirstTime)
	{
		ReadIni(INI_FREQ);
		ReadIni(INI_FLINE);
		ReadIni(INI_FNOTE);

		bFirstTime=FALSE;
	}
	else
	{
		if(bAll)
		{
			if(bFirstTime)
			{
				// The idea here is to do a full read ini
				// and then set those items
				// to defaults that want to be.  In particular,
				// this will restore the previous x-axis scale.
				// Only read first time so that the Reset All button works.
				ReadIni(INI_FREQ);
			}

			if(bFXDefault)
			{
				strlcpy(sFPlot.x_label, "Data Point", XLABELSIZE);
				sFPlot.x_offset=0.0;
				sFPlot.x_mult=1.0;
				nFPlotX=0;
				sFPlot.x_div=5;
				sFPlot.do_line=FALSE; // Moved to default 9/27/02
			}

			strlcpy(sFPlot.title, "FREQUENCY DATA", TITLESIZE);
			strlcpy(sFPlot.subtitle, szFreqFileTitle, SUBTITLESIZE);
			strlcpy(sFPlot.message_str, "", MESSAGESIZE);
			strlcpy(sFPlot.left_y_label, "Frequency", YLABELSIZE);
			sFPlot.y_offset=0.0;
			sFPlot.y_mult=1.0;
			sFPlot.position=MSG_TOP_LEFT;
			// sFPlot.do_line=FALSE; // Moved to default 9/27/02
			sFPlot.y_div=5;
			sFPlot.y_tic=5;
			sFPlot.do_mouse=FALSE;
			sFPlot.do_date=TRUE;
			sFPlot.do_box=TRUE;
			sFPlot.do_fit=TRUE;
			sFPlot.order=3;
		}
		else
		{
			strlcpy(sFPlot.left_y_label, "Frequency", YLABELSIZE);
			sFPlot.y_mult=1.0;
			sFPlot.y_div=5; 
		}

		// Get min & max of freq data
		FindMinMax(fFreqData, &min, &max, FREQ_DATA);

		// Use "nice" freq scale
		if(bSmartScale)
		{
			// Find scale exponent
			nExp=(int)(ceil(log10(max(fabs(min),fabs(max)))));

			// Avoid fractional scales
			nExp--;
			
			// If exponent is multiplier is near 0 force it to 0
			if(nExp>-2 && nExp<2)
			{
				nExp=0; // nExp beween -1 and +1 becomes 0
			}

			// Set scale multiplier
			sFPlot.y_mult=(float)(pow(10, -nExp));

			// Set scale label
			if(nExp<-1) // nExp -2 and smaller
			{
				_snprintf(szBuffer, sizeof(szBuffer), ", pp10[%d]", -nExp);
				strlcat(sFPlot.left_y_label, szBuffer, YLABELSIZE);
			}

			if(nExp>1) // nExp +2 and larger
			{
				_snprintf(szBuffer, sizeof(szBuffer), ", x10[%d]", nExp);
				strlcat(sFPlot.left_y_label, szBuffer, YLABELSIZE);
			}
		}

		// Determine y-axis plot scale
		if(((max-min)/((fabs(max)+fabs(min)*2)))>0.05)
		{
			// Determine plot scale using RoundAxes() function
			// This function gives nice scale when data range is large
			smin=min;
			smax=max;
			RoundAxes(&smin, &smax, &sdiv);
			sdiv*=5; // Note: This mult by 5 assures that 5 minor ticks are OK
			sFPlot.y_div=(int)ceil((smax-smin)/sdiv);
			scale_min=(float)smin;
			scale_max=(float)(smin+sFPlot.y_div*sdiv);
			
			// Reset # divs
			//sFPlot.x_div=(int)ceil((smax-smin)/sdiv); // Changed 03/20/08
			sFPlot.y_div=(int)ceil((smax-smin)/sdiv);

			// Tighten scale as much as possible
			while(min>=(scale_min+sdiv))
			{
				scale_min+=(float)sdiv;
				sFPlot.y_div-=1;
			}
			while(max<=(scale_max-sdiv))
			{
				scale_max-=(float)sdiv;
				sFPlot.y_div-=1;
			}

			// Round scales to 3 decimal places
			// Plot will require normalization before more digits are needed
			// This avoids silly small increments
			_snprintf(szBuffer, sizeof(szBuffer), "%1.3e", scale_min);
			scale_min=(float)atof(szBuffer);
			_snprintf(szBuffer, sizeof(szBuffer), "%1.3e", scale_max);
			scale_max=(float)atof(szBuffer);
		}
		else
		{
			// Determine plot scale using BasScale() function
			// This function is better with small relative data range
			// Even though its scale choice is often sub-optimum
			// A potential problem with the BasScale() method is that
			// the # of minor tics is not always 5
			// And GraphiC doesn't alway honor the BasScale() values
			BasScale(min, max, 12, &smin, &sdiv);
			// sFPlot.x_div=(int)ceil((max-smin)/sdiv); // Changed 03/20/08
			sFPlot.y_div=(int)ceil((max-smin)/sdiv);
			// smax=smin+(sFPlot.x_div*sdiv); // Changed 03/20/08
			smax=smin+(sFPlot.y_div*sdiv);

			// Reset # divs
			// sFPlot.x_div=(int)ceil((smax-smin)/sdiv); // Changed 03/20/08
			sFPlot.y_div=(int)ceil((smax-smin)/sdiv);

			// Set scale limits
			scale_min=(float)smin;
			scale_max=(float)smax;

			// Tighten scale as much as possible
			while(min>=(scale_min+sdiv))
			{
				scale_min+=(float)sdiv;
				sFPlot.y_div-=1;
			}
			while(max<=(scale_max-sdiv))
			{
				scale_max-=(float)sdiv;
				sFPlot.y_div-=1;
			}
		}
		
		// Set y-axis scale limits
		sFPlot.y_min=scale_min;
		sFPlot.y_max=scale_max;
	}

	// Set x-axis offset to zero
	sFPlot.x_offset=0.0;
	
	// Get special MJD-XXXXX or DOY XXXX scale
	// first time w/o opening plot options dialog box
	// Get freq tau
	GetWindowText(GetDlgItem(hwndMain, IDC_FREQTAU),
		szBuffer, BUFFER_SIZE);

	// Use its absolute value
	fTau=(float)fabs(atof(szBuffer));

	// Set the x-axis multiplier factor
	DetermineXMult(&sFPlot, nFPlotX, fTau);

	// Add special info to MJD & DOY labels if have timetags
	if(bFreqTimetags)
	{
		// Find start point
		i=(int)fFreqData[1]; // Start point #

		// Find # days in data
		nDay=(int)ceil((fFreqData[2]-fFreqData[1])*sFPlot.x_mult);

		// Determine # decades to remove from MJD scale
		nDecade=1000;
		if(nDay<10)
		{
			nDecade=100;
		}

		// Find # 100s or 1000s in MJD
		nCoarseMJD=0;

		if(fFreqTimetags[i+2]<INT_MAX)
		{
			// Set offset to start point timetag
			while((float)fFreqTimetags[i+2]-
				nCoarseMJD>=(float)nDecade)
			{
				nCoarseMJD+=nDecade;
			}
		}
							
		if(nFPlotX==X_MJD) // Adjust MJD scale
		{
			// Set offset to start point timetag
			sFPlot.x_offset=(float)fFreqTimetags[i+2];

			// Subtract leading part of MJD
			sFPlot.x_offset-=nCoarseMJD;

			// Scale to data point units
			sFPlot.x_offset*=(float)(86400.0/fTau);

			// Add leading part of MJD to x-scale label
			_snprintf(sFPlot.x_label, XLABELSIZE, "MJD - %d", nCoarseMJD);
		}

		 // Start point MJD
		lnMJD=(long)floor(fFreqTimetags[i+2]);
		nDOY=MJDtoDOY(lnMJD); // Start point DOY

		// Convert integer MJD to date
		MJDToDate(lnMJD, &nDay, &nMonth, &nYear);

		if(nFPlotX==X_DOY)
		{
			// Set x offset
			sFPlot.x_offset=(float)nDOY; // DOY

			 // Add fractional day
			sFPlot.x_offset+=(float)fFreqTimetags[i+2]-(float)lnMJD;

			 // Scale to data point units
			sFPlot.x_offset*=(float)(86400.0/fTau);

			// Add year to x-scale label
			_snprintf(sFPlot.x_label, XLABELSIZE, "DOY %d", nYear);
		}
	}

	// X-axis scale factor determination
	sFPlot.x_max=(float)(sFPlot.x_mult*(fFreqData[2]+sFPlot.x_offset));
	sFPlot.x_min=(float)(sFPlot.x_mult*(fFreqData[1]+sFPlot.x_offset));

		// Use fewer x divs if label is Data Point and # data points is large
	// and bFXdefault is TRUE
	if( (nFPlotX==X_POINT) && ((sFPlot.x_max-sFPlot.x_min)>10000) &&
		(bFXDefault) )
	{
		nFPlotX=X_KPOINT;
		sFPlot.x_mult=(float)0.001;
		sFPlot.x_max=(float)(sFPlot.x_mult*(fFreqData[2]+sFPlot.x_offset));
		sFPlot.x_min=(float)(sFPlot.x_mult*(fFreqData[1]+sFPlot.x_offset));
	}

	#if(1) // New code 03/20/08
	// Adjust x_min so first point is plotted
	// This is certainly better for points scales
	// and is probably OK for all
	{
		sFPlot.x_min-=(float)1.0*sFPlot.x_mult;
	}
	#endif

	// Determine x-axis plot scale
	DetermineXScale(&sFPlot);

	// New code 02/02/08 for freq jump detection
	// Set # freq data points
	nNum=(int)(fFreqData[2]-fFreqData[1]+1.0);

	// Set default sFPlot.length as max(5, # freq data points/10)
	// Set sFPlot.offset=0
	// Wrong: Set sFPlot.limit as 3*ADEV at AF=num/length for all freq data
	//        e.g., get overlapping ADEV at AF=10 for entire large freq data set
	//        and AF=2 for 10-point data set
	// Right: Set sFPlot.limit as 3*ADEV at AF=length for all freq data
	sFPlot.length=max(5, nNum/10);
	sFPlot.offset=0;
	// sFPlot.method=SEQAVG; // Read from ini file
	
	#if(0) // Wrong
	CalcFreqOverlapSigma(fFreqData, &sigma,	nNum/sFPlot.length));
	#endif

	#if(0) // Right, but slow for large data set calculated from freq data
	CalcFreqOverlapSigma(fFreqData, &sigma, sFPlot.length);
	#endif

	#if(1) // Faster code 02/10/08
	// This code converts freq data to phase data
	// for faster calculation with large data sets
	// Note: If already have equivalent phase data, don't
	// need to alloc mem and do conversion.  Can just
	// use the existing phase data if bSynch flag is TRUE. 
	// In that case, need to assign fWorkData ptr to it

	if(!bSynch) // Don't have phase data
	{
		// Need to convert freq data to phase data
		// Allocate memory for working phase data
		// converted from freq data for ADEV calc
		if((fWorkData=(F_TYPE*)malloc((nNum+ARRAY_OFFSET+1+EXTRA_MEM)*
			sizeof(F_TYPE)))==NULL)
		{
			// Display error message
			#ifdef MEM_DEBUG
			AllocErrorMessageEx(__FILE__, __LINE__,
				(nNum+ARRAY_OFFSET+1+EXTRA_MEM)*
				sizeof(F_TYPE));
			#else
			AllocErrorMessage();
			#endif

			return;
		}

		// Convert freq data to phase data for sigma calc
		ConvFreqToPhase(fFreqData, fWorkData, fTau, FALSE);
	}
	else // Assign fWorkData pointer to phase data
	{
		fWorkData=fPhaseData;
	}

	// Calc overlapping sigma from fWorkData phase data
	// Note: The 3rd arg is the basic tau_0
	nDummy=CalcPhaseOverlapSigma(fWorkData, &sigma,	fTau, sFPlot.length);

	// Error trap for bad calc
	if(nDummy<1)
	{
		// Display error message?

		// Free working array if didn't have phase data
		if(!bSynch)
		{
			// Free working data array
			free(fWorkData);
		}
		return;
	}

	// Free working array if didn't have phase data
	if(!bSynch)
	{
		// Free working data array
		free(fWorkData);
	}
	#endif

	sFPlot.sigma=(float)sigma;
	sFPlot.limit=(float)(3.0*sigma);

	if(bFirstTime || bAll)
	{
		// Set freq plot x-axis label
		LoadString(hInst, IDS_DPLOT_XLABEL_0+nFPlotX, szBuffer, BUFFER_SIZE);
		strlcpy(sFPlot.x_label, szBuffer, XLABELSIZE);

		// Get freq tau
		GetWindowText(GetDlgItem(hwndMain, IDC_FREQTAU),
			szBuffer, BUFFER_SIZE);

		// Use its absolute value
		fTau=(float)fabs(atof(szBuffer));

		// Set the x-axis multiplier factor
		DetermineXMult(&sFPlot, nFPlotX, fTau);

		// Adjust settings for modern plot style
		if(nStyle==MODERN_STYLE)
		{
			sFPlot.do_date=FALSE;
			sFPlot.do_wide=TRUE;
			strlcpy(sFPlot.subtitle, "", sizeof(sFPlot.subtitle));
		}

		// Set line thickness
		if(sFPlot.do_wide)
		{
			sFPlot.thick=WIDELINESIZE;
		}
		else
		{
			sFPlot.thick=0;
		}
		
		// Reset 1st time flag
		bFirstTime=FALSE;
	}
}

/*****************************************************************************/
/*                                                                           */
/*                      InitSigmaPlotParams()                                */
/*                                                                           */
/*		Initialize sigma plot parameters                                     */
/*                                                                           */
/* 		Revision record:                                                     */
/*			10/17/96	Drafted                                              */
/*			10/26/96	Added do_errbar & do_table initializations           */
/*			01/23/97	Added ReadIni() logic                                */
/*			01/29/97	Made changes for all tau sigma                       */
/*			03/15/97	Added sSPlot.table_position initialization           */
/*			12/21/97    Added bFirstTime usage & line & note initializations */
/*			08/30/98	Added do_fit usage                                   */
/*			05/08/00	Added TTOT                                           */
/*			03/11/01	Added plot line width parameter                      */
/*			11/23/01	Added bNoGreek test                                  */
/*			02/24/12	Changed TIE rms and MTIE default plot titles         */
/*                                                                           */
/*****************************************************************************/

void WINAPI InitSigmaPlotParams(void)
{
	// Local variables
	// This flag allows reading INI file 1st time only
	// After that, if sigma plot params get reinitialized
	// they are set to defaults
	static BOOL bFirstTime=TRUE;
	
	// Variable initializations
	if(bUseSigmaPlotIni && bFirstTime)
	{
		ReadIni(INI_SIGMA);
		ReadIni(INI_SLINE);
		ReadIni(INI_SNOTE);
	}
	else
	{
		strlcpy(sSPlot.title, "FREQUENCY STABILITY", TITLESIZE);

		#if(0) // Original code
		if( (nSigmaPlotType==TIME_RUN) || (nSigmaPlotType==TIMETOTAL_RUN) )
		{
			strlcpy(sSPlot.title, "TIME STABILITY", TITLESIZE);
		}
		#endif

		#if(1) // New code 02/24/12 Add TIE rms and MTIE
		if( (nSigmaPlotType==TIME_RUN) || (nSigmaPlotType==TIMETOTAL_RUN)||
			(nSigmaPlotType==TIERMS_RUN)|| (nSigmaPlotType==MTIE_RUN) )
		{
			strlcpy(sSPlot.title, "TIME STABILITY", TITLESIZE);
		}
		#endif

		strlcpy(sSPlot.subtitle, szFileTitle, SUBTITLESIZE);

		strlcpy(sSPlot.message_str, "", MESSAGESIZE);
		sSPlot.position=MSG_TOP_LEFT;
		sSPlot.table_position=TAB_TOP_RIGHT;

		// Adjust default table position depending on slope of sigma-tau curve
		if(nSigmaPlotType==MODIFIED_RUN || nSigmaPlotType==TIME_RUN     ||
		   nSigmaPlotType==MODTOTAL_RUN || nSigmaPlotType==TIMETOTAL_RUN )
		{
			if(sModSigData.sigma[0]<sModSigData.sigma[sModSigData.points-1])
			{
				sSPlot.table_position=TAB_BOT_RIGHT;
			}

		}
		else // Norm sigma data array
		{
			if(sNormSigData.sigma[0]<sNormSigData.sigma[sNormSigData.points-1])
			{
				sSPlot.table_position=TAB_BOT_RIGHT;
			}
		}
	}

	// Reset 1st time flag
	bFirstTime=FALSE;

	LoadString(hInst, IDS_SPLOT_XLABEL_0, sSPlot.x_label, LABELSIZE);

	LoadString(hInst, IDS_SPLOT_YLABEL_0+nSigmaPlotType, sSPlot.left_y_label,
		LABELSIZE);

	#if(1) // New code 11/29/11 for modern plot style support
	if(nStyle==MODERN_STYLE)
	{
		LoadString(hInst,
			IDS_SPLOT_YLABEL_0+nSigmaPlotType+((NUM_SPLOT_YNAMES-1)*NUM_SPLOT_YLABELS),
			sSPlot.left_y_label, LABELSIZE);
	}
	#endif

	// Avoid Greek characters
	if(bNoGreek)
	{
		// Load the 2nd x-axis label
		LoadString(hInst, IDS_SPLOT_XLABEL_0+1, sSPlot.x_label, BUFFER_SIZE);

		// Eliminate comma and all later characters from y-axis label
		if(strchr(sSPlot.left_y_label, ','))
		{
			*strchr(sSPlot.left_y_label, ',')=(char)'\0';
		}
	}

	sSPlot.do_line=FALSE;
	sSPlot.do_mouse=FALSE;
	sSPlot.do_date=TRUE;
	sSPlot.do_box=TRUE;
	sSPlot.do_fit=TRUE;

	if(bAllTau)
	{
		if(!bUseSigmaPlotIni)
		{
			strlcpy(sSPlot.message_str, "Sigma For All Tau", MESSAGESIZE);
		}
		sSPlot.do_errbar=FALSE;
		// Allow new all tau table
		sSPlot.do_table=TRUE;
	}
	else
	{
		sSPlot.do_errbar=TRUE;
		sSPlot.do_table=TRUE;
	}

	// Handle MTIE case w/o error bars where bAllTau denotes fast method
	// and we want table in lower right
	if(nSigmaPlotType==MTIE_RUN)
	{
		if(!bUseSigmaPlotIni)
		{
			strlcpy(sSPlot.message_str, "", MESSAGESIZE);
		}
		sSPlot.do_errbar=FALSE;
		sSPlot.do_table=TRUE;
		sSPlot.table_position=TAB_BOT_RIGHT;
	}


	#if(1) // New code 02/16/10 for cross sigma
	if(nSigmaPlotType==CROSS_RUN)
	{
		strcpy(sSPlot.left_y_label, "Crosscorrelation Allan Deviation");
	}
	#endif

	// Adjust settings for modern plot style
	if(nStyle==MODERN_STYLE)
	{
		sSPlot.do_date=FALSE;
		sSPlot.do_wide=TRUE;
		strlcpy(sSPlot.subtitle, "", sizeof(sSPlot.subtitle));
		strcpy(sSPlot.x_label, "Averaging Time, s");
	}

	// Set line thickness
	if(sSPlot.do_wide)
	{
		sSPlot.thick=WIDELINESIZE;
	}
	else
	{
		sSPlot.thick=0;
	}
}

/*****************************************************************************/
/*                                                                           */
/*                      InitReadPlotParams()                                 */
/*                                                                           */
/*		Initialize read plot parameters                                      */
/*                                                                           */
/* 		Revision record:                                                     */
/*			11/17/96	Cloned from InitSigmaPlotParams()                    */
/*			01/29/97	Made changes for all tau sigma                       */
/*			03/15/97	Added sSPlot.table_position initialization           */
/*			08/31/98	Added do_fit usage                                   */
/*			03/11/01	Added plot line width parameter                      */
/*                                                                           */
/*****************************************************************************/

void WINAPI InitReadPlotParams(void)
{
	// Variable initializations
	if(!bUseSigmaPlotIni)
	{
		strlcpy(sSPlot.title, "FREQUENCY STABILITY", TITLESIZE);
		strlcpy(sSPlot.subtitle, szStabFileTitle, SUBTITLESIZE);
		strlcpy(sSPlot.message_str, "", MESSAGESIZE);

		// Full filename including drive & directory where program started
		strlcpy(sSPlot.plot_filename, szDrive, FILENAMESIZE);
		strlcat(sSPlot.plot_filename, szDirectory, FILENAMESIZE);
		strlcat(sSPlot.plot_filename, "SIGMA.TKF", FILENAMESIZE);
	}

	LoadString(hInst, IDS_SPLOT_XLABEL_0, sSPlot.x_label, LABELSIZE);
	LoadString(hInst, IDS_SPLOT_YLABEL_0+nSigmaPlotType, sSPlot.left_y_label,
		LABELSIZE);
	
	sSPlot.do_line=FALSE;
	sSPlot.position=MSG_TOP_LEFT;

	sSPlot.table_position=TAB_TOP_RIGHT;
	sSPlot.do_mouse=FALSE;
	sSPlot.do_date=TRUE;
	sSPlot.do_box=TRUE;
	sSPlot.do_fit=TRUE;

	#if(1) // New code 11/29/11 for modern plot style support
	if(nStyle==MODERN_STYLE)
	{
		LoadString(hInst,
			IDS_SPLOT_YLABEL_0+nSigmaPlotType+((NUM_SPLOT_YNAMES-1)*NUM_SPLOT_YLABELS),
			sSPlot.left_y_label, LABELSIZE);
	}
	#endif

	if(bAllTau)
	{
		if(!bUseSigmaPlotIni)
		{
			strlcpy(sSPlot.message_str, "Sigma For All Tau", MESSAGESIZE);
		}
		sSPlot.do_errbar=FALSE;
		// Allow new all tau table
		sSPlot.do_table=TRUE;
	}
	else
	{
		sSPlot.do_errbar=TRUE;
		sSPlot.do_table=TRUE;
	}

	// Adjust settings for modern plot style
	if(nStyle==MODERN_STYLE)
	{
		sSPlot.do_date=FALSE;
		sSPlot.do_wide=TRUE;
		strlcpy(sSPlot.subtitle, "", sizeof(sSPlot.subtitle));
		strcpy(sSPlot.x_label, "Averaging Time, s");
	}

	// Set line thickness
	if(sSPlot.do_wide)
	{
		sSPlot.thick=WIDELINESIZE;
	}
	else
	{
		sSPlot.thick=0;
	}
}

/*****************************************************************************/
/*                                                                           */
/*                      CalcSigmaPlotScale()                                 */
/*                                                                           */
/*		Parameters:		None(void)                                           */
/*                                                                           */
/*      Return:			int nPoints		# sigma plot points                  */
/*                                                                           */
/* 		Revision record:                                                     */
/*			10/22/96	Created from inline code                             */
/*			12/26/99	Added Mod Totvar                                     */
/*			05/08/00	Added TTOT                                           */
/*			02/15/03	Changes to support overlapping and Tho1 plot        */
/*						Combine scale info for BOTH_RUN and FULL_RUN         */
/*                                                                           */
/*****************************************************************************/

int WINAPI CalcSigmaPlotScale(void)
{
	// Local variables
	// Ints
	int nPoints;			 // # sigma plot data points
	int nMinTauScale;        // Min tau scale value
	int nMaxTauScale;        // Max tau scale value
	int nMinSigmaScale;      // Min sigma scale value
	int nMaxSigmaScale;      // Max sigma scale value
	int nOffset;			 // Array offset
	// Floats
	float fMinTau;           // Min tau value
	float fMaxTau;           // Max tau value
	float fMinSigma;         // Min sigma value
	float fMaxSigma;         // Max sigma value
	float fMin;				 // Temporary min value
	float fMax;				 // Temporary max value

	// Calc tau and sigma log scales
	// Get the sigma and tau max and min from sModSigData or sNormSigData
	// struct (depending on value of nSigmaPlotType)
	// and calc the corresponding log scale limits
	// Note: BOTH_RUN and FULL_RUN are accepted by both of these if statements
	// All other sigma types are processed by only one or the other of them
	if( (nSigmaPlotType==MODIFIED_RUN)  || (nSigmaPlotType==TIME_RUN)      ||
		(nSigmaPlotType==MODTOTAL_RUN)  || (nSigmaPlotType==TIMETOTAL_RUN) ||
		(nSigmaPlotType==BOTH_RUN)      || (nSigmaPlotType== FULL_RUN)     )
	{
		nPoints=sModSigData.points;
		minmax(&fMinTau, &fMaxTau, sModSigData.tau,
			nPoints);
		if( (fMinTau) && (fMaxTau) )
		{
			nMinTauScale=(int)floor(log10(fMinTau)+0.000001);
			nMaxTauScale=(int)ceil(log10(fMaxTau)-0.000001);
		}
		
		// Note: minmax() does not reject zero values
		// The overlapping and Tho1 sigma plot data can have zeros
		// The Tho1 missing points in the mod sigma data are at the beginning
		// So use this to ignore them - nOffset is # leading zero points
		nOffset=0;
		while(sModSigData.sigma[nOffset]==0.0)
		{
			nOffset++;
		}
		
		// Find minimum & maximum
		minmax(&fMinSigma, &fMaxSigma, &sModSigData.sigma[nOffset],
			nPoints-nOffset);

		// Copy temporary values
		fMin=fMinSigma;
		fMax=fMaxSigma;

		// Set scale limits
		if( (fMinSigma) && (fMaxSigma) )
		{
			nMinSigmaScale=(int)floor(log10(fMinSigma)+0.000001);
			nMaxSigmaScale=(int)ceil(log10(fMaxSigma)-0.000001);
		}
	}
	if( (nSigmaPlotType!=MODIFIED_RUN)  && (nSigmaPlotType!=TIME_RUN) &&
		(nSigmaPlotType!=MODTOTAL_RUN)  && (nSigmaPlotType!=TIMETOTAL_RUN) )
	{
		nPoints=sNormSigData.points;
		minmax(&fMinTau, &fMaxTau, sNormSigData.tau,
			nPoints);
		if( (fMinTau) && (fMaxTau) )
		{
			nMinTauScale=(int)floor(log10(fMinTau)+0.000001);
			nMaxTauScale=(int)ceil(log10(fMaxTau)-0.000001);
		}

		// Note: minmax() does not reject zero values
		// The overlapping and Tho1 sigma plot data can have zeros
		// The overlapping missing points in the mod sigma data are at the end
		// So use this to ignore them - nOffset is # non-zero points
		nOffset=nPoints;
		while(sNormSigData.sigma[nOffset-1]==0.0)
		{
			nOffset--;
		}

		// Find minimum & maximum
		minmax(&fMinSigma, &fMaxSigma, sNormSigData.sigma,
			nOffset);

		// Combine min & max for BOTH_RUN or FULL_RUN
		if( (nSigmaPlotType==BOTH_RUN) || (nSigmaPlotType==FULL_RUN) )
		{
			fMinSigma=min(fMinSigma, fMin);
			fMaxSigma=max(fMaxSigma, fMax);
		}

		// Set scale limits
		if( (fMinSigma) && (fMaxSigma) )
		{
			nMinSigmaScale=(int)floor(log10(fMinSigma)+0.000001);
			nMaxSigmaScale=(int)ceil(log10(fMaxSigma)-0.000001);
		}
	}

	// Calc the numerical plot scales and store them in sSPlot struct
	sSPlot.x_min=(float)pow(10.0, (float)nMinTauScale);
	sSPlot.x_max=(float)pow(10.0, (float)nMaxTauScale);
	sSPlot.y_min=(float)pow(10.0, (float)nMinSigmaScale);
	sSPlot.y_max=(float)pow(10.0, (float)nMaxSigmaScale);

	return nPoints;
}

/*****************************************************************************/
/*                                                                           */
/*                      InitAutocorrelationPlotParams()                      */
/*                                                                           */
/*		Initialize autocorrelation plot parameters                           */
/*                                                                           */
/*		Parameters:		int bAll  0=y-axis scale only, 1=all params          */
/*                                                                           */
/*      Return:			None (void)                                          */
/*                                                                           */
/* 		Revision record:                                                     */
/*			08/05/03	Created from InitPhasePlotParams()                   */
/*                                                                           */
/*****************************************************************************/

void WINAPI InitAutocorrelationPlotParams(BOOL bAll)
{
	// Local variables
	// BOOLs
	// This flag allows reading INI file 1st time only
	// After that, if phase plot params get reinitialized
	// they are set to defaults
	static BOOL bFirstTime=TRUE;	// Flag for first time initialized

	// Note: # autocorreleation points is in sAPlot.num
	
	// Variable initializations
	if(bUseAutoPlotIni && bFirstTime)
	{
		// Read info from ini file
		ReadIni(INI_AUTO);

		// Reset 1st time flag
		bFirstTime=FALSE;
	}
	else
	{
		if(bAll)
		{
			if(bFirstTime)
			{
				// The idea here is to do a full read ini
				// and then set those items to defaults that want to be.
				// In particular, this will restore the previous x-axis scale.
				// Only read first time so that the Reset All button works.
				// ReadIni(INI_PHASE);
				ReadIni(INI_AUTO);
				bFirstTime=FALSE;
			}

			if(bAXDefault)
			{
				strlcpy(sAPlot.x_label, "Lag", XLABELSIZE);
				sAPlot.do_line=FALSE;
			}

			strlcpy(sAPlot.title, "AUTOCORRELATION SEQUENCE", TITLESIZE);
			strlcpy(sAPlot.subtitle, szPhaseFileTitle, SUBTITLESIZE);
			strlcpy(sAPlot.message_str, "", MESSAGESIZE);
			strlcpy(sAPlot.left_y_label, "Autocorrelation", YLABELSIZE);
			sAPlot.position=MSG_TOP_LEFT;
			sAPlot.do_mouse=FALSE;
			sAPlot.do_date=TRUE;
			sAPlot.do_box=TRUE;
			sAPlot.do_fit=TRUE;
		}
		else
		{
			strlcpy(sAPlot.left_y_label, "Autocorrelation", YLABELSIZE);
		}
	}

	if(bFirstTime || bAll)
	{
		// Adjust settings for modern plot style
		if(nStyle==MODERN_STYLE)
		{
			sAPlot.do_date=FALSE;
			sAPlot.do_wide=TRUE;
			strlcpy(sAPlot.subtitle, "", sizeof(sAPlot.subtitle));
		}

		// Set line thickness
		if(sAPlot.do_wide)
		{
			sAPlot.thick=WIDELINESIZE;
		}
		else
		{
			sAPlot.thick=0;
		}

		// Reset 1st time flag
		bFirstTime=FALSE;
	}
}

/*****************************************************************************/
/*                                                                           */
/*                      UpdatePlotSubtitles()                                */
/*                                                                           */
/* 		Revision record:                                                     */
/*			11/22/06	Written                                              */
/*                                                                           */
/*****************************************************************************/

void WINAPI UpdatePlotSubtitles(void)
{

	if(bDataType==PHASE_DATA)
	{
		// Phase plot subtitle
		strlcpy(sPPlot.subtitle, szFileTitle, TITLESIZE);
	}
	else // Freq data
	{
		// Freq plot subtitle
		strlcpy(sFPlot.subtitle, szFileTitle, TITLESIZE);  	
	}

	// Sigma plot subtitle
	strlcpy(sSPlot.subtitle, szFileTitle, SUBTITLESIZE); 
	// Histo plot subtitle
	strlcpy(sHPlot.subtitle, szFileTitle, SUBTITLESIZE); 
	// Spectrum plot subtitle
	strlcpy(sPSPlot.subtitle, szFileTitle, SUBTITLESIZE);
	// ACF plot subtitle
	strlcpy(sAPlot.subtitle, szFileTitle, SUBTITLESIZE); 
}

/*****************************************************************************/
