/*****************************************************************************/
/*                                                                           */
/*                                  Theo1()                                  */
/*                                                                           */
/*      Function to calculate Theoretical Variance #1 from phase data        */
/*                                                                           */
/*      Parameters:     F_TYPE x[]    = phase data                           */
/*                                      x[0] = # data phase points           */
/*                                      x[1] = analysis start                */
/*                                      x[2] = analysis end                  */
/*                      F_TYPE *p_sig = pointer to Theo1 deviation           */
/*                      F_TYPE tau    = data averaging time = tau_0          */
/*                      int m         = averaging factor = tau/tau_0         */
/*                                                                           */
/*      Return:         int           = # analysis points,                   */
/*                                      or -1 if error                       */
/*                                                                           */
/*      Notes:          Embedded zero phase values treated as gaps in data.  */
/*                      The data consists of n phase samples at interval     */
/*                      tau_0.  Denominator term (n-m) must be >0.  This     */
/*                      sets max m=n-1.  m must be >0 and tau must be >0.    */
/*                      m must be even, and Theo1 is intended for m>=10.     */
/*                                                                           */
/*      Ref:            D.A. Howe and T.K. Peppler, "Estimation of Very      */
/*                      Long Term Frequency Stability Using a Special        */
/*                      Purpose Statistic", IEEE Int'l FCS 2003.             */
/*                                                                           */
/*      Revision record:                                                     */
/*          02/06/03    Created                                              */
/*                                                                           */
/* (c) Copyright 2003  Hamilton Technical Services  All Rights Reserved      */
/*                                                                           */
/*****************************************************************************/

int WINAPI Theo1(F_TYPE x[], F_TYPE *p_sig, F_TYPE tau, int m)
{
    double sum_i;                                  /* outer summing variable */
    double sum_j;                                  /* inner summing variable */
    float fm;                                          /* float version of m */
    int i;                                   /* index for outer summing loop */
    int j;                             /* array index for inner summing loop */
    int d;                                      /* loop index for inner loop */
    int count;                             /* # points in outer summing loop */
    int n;                               /* # phase data points for analysis */

    if( (m<1) || (n<m) || (tau<=0) || (m%2) )
    {
        return(-1);                                                 /* error */
    }

    // Initializations
    fm=(float)m;
    sum_i=0;
    n=(int)(x[2]-x[1]+1);
    count=0;

    // NOTE:  NO GAP HANDLING YET
    
    // Outer Loop
    // i goes from 1 to n-m where n=# phase points & m=avg factor
    for(i=1; i<=n-m; i++)  
    {
        // Initial inner sum
        sum_j=0;

        // Inner Loop
        // d goes from -(m/2)-1 to (m/2)-1
        // Index j includes offset for analysis start and array offset
        for(d=-(m/2)+1; d<=(m/2)-1; d++) 
        {
            // Set array index
            j=(int)(i+x[1]+1);

            // Inner summation
            sum_j+= (1/((m/2)-j))*(SQR(x[i]-x[i-j+(m/2)]+x[i+m]-x[i+j+(m/2)]));

            // Increment count
            count++;
        }

        // Outer summation
        sum_i+=sum_j;
    }

    // Final scaling
    if(sum_i>0)                                     /* make sure no div by 0 */
    {
        // Divide by (n-m) and (m*tau)^2 and take sqrt
        *p_sig=sqrt(sum_i/((n-m)*(tau*tau*fm*fm)));
        return(count);                                  /* # analysis points */
    }
    else
    {
        return(-1);                                                 /* error */
    }
}                                                             /* end Theo1() */

/*****************************************************************************/

